/*
//  iTMGridMenu.m
//  2DMenu
//
//  Created by jerome.laurens@u-bourgogne.fr on Wed Feb 20 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date: contribution(contributor) ") 
//  To Do List: (format "- proposition(percentage actually done) ") 
*/

#import <Foundation/NSGeometry.h>
#import <Foundation/Foundation.h>
#import <AppKit/NSMenu.h>
#import <AppKit/NSMenuView.h>
#import <AppKit/AppKit.h>
#import "iTMGridMenu.h"
#import "iTMGridMenuView.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMGridMenu
/*"The grid menu is meant to be used with the iTMGridMenuView as representation.
The menu items are stored in the inherited item array, in rows.
No facility is given to deal with rows and columns: inserting a menu item will probably break some alignment.
Only row/column based accessors are provided. Row and column numbering are 0 based"*/
@implementation iTMGridMenu
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initWithTitle: 
- (id) initWithTitle: (NSString *) aTitle;
/*"Designated initializer. The menu representation is created here (forced) as an iTMGridMenuView instance.
The number of columns is set to 1.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    self = [super initWithTitle: aTitle];
    [self setMenuRepresentation: [[[iTMGridMenuView alloc]initWithFrame: NSZeroRect] autorelease]];
    _NOC = 1;
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  numberOfRows
- (int) numberOfRows;
/*"Computed from the total number of items and the number of columns. The total number of items need not be a multiple of the declared number of columns.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    return MAX(1,[self numberOfItems]/[self numberOfColumns]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  numberOfColumns
- (int) numberOfColumns;
/*"At least 1 is returned.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    return MAX(_NOC, 1);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setNumberOfColumns: 
- (void) setNumberOfColumns: (int) NOC;
/*"Only positive NOC are acceptable. Nothing happens for a nonpositive argument.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    if((NOC>=1) && (NOC<=[self numberOfItems]) && (NOC != _NOC)) 
    {
        _NOC = NOC;
        // the geometry has changed: inform the associated menu view
        [[self menuRepresentation]setNeedsSizing: YES];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  itemAtIndex:
- (id <NSMenuItem>) itemAtIndex: (int) index;
/*"Returns nil to fill the (eventually) missing items to terminate the grid rectangle.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Raise an exception.
"*/
{
    return ((index>=0) && (index<[self numberOfItems]))? [super itemAtIndex: index]: nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  itemAtRow:column:
- (id <NSMenuItem>) itemAtRow: (int) row column: (int) column;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Raise an exception.
"*/
{
    return ((row>=0) && (column>=0) && (column<[self numberOfColumns]))?
        [self itemAtIndex: row + [self numberOfColumns] * column]: nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  itemIndexForRow:column:
- (int) itemIndexForRow: (int) row column: (int) column;
/*"Returns -1 if row or column is not consistent.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    return ((row>=0) && (column>=0) && (column<[self numberOfColumns]))?
        row + [self numberOfColumns] * column: -1;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  rowForItemIndex:
- (int) rowForItemIndex: (int) index;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    return index / [self numberOfColumns];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  columnForItemIndex:
- (int) columnForItemIndex: (int) index;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    return index % [self numberOfColumns];
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMGridMenu
