/*
//  iTMGridMenuView.m
//  2DMenu
//
//  Created by jerome.laurens@u-bourgogne.fr on Wed Feb 20 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "iTMGridMenuView.h"

@interface NSMenu(iTMGridMenuViewPrivate)
- (int) rowForItemIndex: (int) index;
- (int) columnForItemIndex: (int) index;
- (int) itemIndexForRow: (int) row column: (int) column;
- (int) numberOfColumns;
- (int) numberOfRows;
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMGridMenuView
/*"Description forthcoming."*/
@implementation iTMGridMenuView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initWithFrame:
- (id) initWithFrame: (NSRect) frame;
/*"Designated initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    self = [super initWithFrame: frame];
    [_CachedRowsHeights release];
    _CachedRowsHeights = nil;
//    NSLog(@"<%@ initWithFrame:...>", self);
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Designated initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [_CachedRowsHeights release];
    _CachedRowsHeights = nil;
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sizeToFit
- (void) sizeToFit;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMenu * M = [self menu];
    int numberOfColumns = [M numberOfColumns];
    int numberOfRows = [M numberOfRows];
    int numberOfItems = [M numberOfItems];
    int colIndex, rowIndex, itemIndex;
    float totalWidth = 0, totalHeight = 0;
    
//    NSLog(@"<%@ sizeToFit: %d>", self, [[self menu] numberOfColumns]);

    [self setNeedsSizing: NO];

    _CachedTitleWidth = 0;
    [_CachedRowsHeights release];
    _CachedRowsHeights = [[NSMutableArray alloc] initWithCapacity: numberOfRows];
    
//    NSLog(@"1");

    for(rowIndex = 0, itemIndex = 0; ((itemIndex<numberOfItems) && (rowIndex<numberOfRows)); ++rowIndex)
    {
        float rowHeight = 0;
//        NSLog(@"rowIndex = %d", rowIndex);
        for (colIndex = 0; ((itemIndex<numberOfItems) && (colIndex<numberOfColumns)); ++colIndex, ++itemIndex)
        {
            NSMenuItemCell * MIC = [self menuItemCellForItemAtIndex: itemIndex];
            NSSize S;
//            NSLog(@"itemIndex = %d, %@", itemIndex, [MIC title]);
            [MIC setImagePosition: NSImageLeft];
            [MIC calcSize];
            S = [MIC cellSize];
//            NSLog(@"cellSize vs title size: %f ?= %f", S.width, [MIC titleWidth]);
            _CachedTitleWidth = MAX(_CachedTitleWidth, [MIC titleWidth]);
            rowHeight = MAX(rowHeight, S.height);
        }
        [_CachedRowsHeights addObject: [NSNumber numberWithFloat: rowHeight]];
        totalHeight += rowHeight;
    }
//    NSLog(@"2");
    _CachedTitleWidth += 2 * [self horizontalEdgePadding];
    totalWidth = _CachedTitleWidth * [[self menu] numberOfColumns];
    [self setFrame: NSMakeRect(0, 0, totalWidth, totalHeight + 10)];
//    NSLog(@"</iTMGridMenuView sizeToFit: totalWidth = %f>", totalWidth);
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  rectOfItemAtIndex:
- (NSRect) rectOfItemAtIndex: (int) index;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSRect R = NSZeroRect;
    if([self needsSizing])
        [self sizeToFit];
    // fixing the height
    {
        NSMenuItemCell * MIC = [self menuItemCellForItemAtIndex: index];
        R.size.height = [MIC cellSize].height;
    }
    // fixing the x origin and the width
    R.origin.x += [[self menu] columnForItemIndex: index] * _CachedTitleWidth;
    R.size.width = _CachedTitleWidth;
    // fixing the y origin
    {
        int rowIndex = [[self menu] rowForItemIndex: index];
        int row;
        if([self isFlipped])
        {
            for(row = 0; row<rowIndex; ++row)
                R.origin.y += [[_CachedRowsHeights objectAtIndex: row] floatValue];
        }
        else
        {
            for(row = [[self menu] numberOfRows]-1; row>rowIndex; --row)
                R.origin.y += [[_CachedRowsHeights objectAtIndex: row] floatValue];
        }
        R.origin.y += 5;
    }
//    NSLog(@"rectOfItemAtIndex: %d\n%@\n%@", index, NSStringFromRect(R), NSStringFromRect([self frame]));
    return R;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  stateImageOffset
- (float) stateImageOffset;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return 0;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  stateImageWidth
- (float) stateImageWidth;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return 0;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  imageAndTitleOffset
- (float) imageAndTitleOffset;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [self horizontalEdgePadding];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  imageAndTitleWidth
- (float) imageAndTitleWidth;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([self needsSizing])
        [self sizeToFit];
    return _CachedTitleWidth;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  keyEquivalentOffset
- (float) keyEquivalentOffset;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return 0;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  keyEquivalentWidth
- (float) keyEquivalentWidth;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return 0;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMGridMenuView
#if 0
- (id)initWithFrame:(NSRect)frame;  // DI

- (void)setMenu:(NSMenu *)menu;
- (NSMenu *)menu;

- (void)itemChanged:(NSNotification *)notification;
- (void)itemAdded:(NSNotification *)notification;
- (void)itemRemoved:(NSNotification *)notification;

- (void)update;

- (void)setHorizontal:(BOOL)flag;
- (BOOL)isHorizontal;

- (void)setFont:(NSFont *)font;
- (NSFont *)font;

- (NSRect)innerRect; // inside the bezel (if any)
- (NSRect)rectOfItemAtIndex:(int)index;
- (int)indexOfItemAtPoint:(NSPoint)point;
- (void)setNeedsDisplayForItemAtIndex:(int)index;

- (void)setHighlightedItemIndex:(int)index;
- (int)highlightedItemIndex;

- (float) stateImageOffset;
- (float) stateImageWidth;
- (float) imageAndTitleOffset;
- (float) imageAndTitleWidth;
- (float) keyEquivalentOffset;
- (float) keyEquivalentWidth;

- (void)setMenuItemCell:(NSMenuItemCell *)cell forItemAtIndex:(int)index;
- (NSMenuItemCell *)menuItemCellForItemAtIndex:(int)index;

- (void)setNeedsSizing:(BOOL)flag;
- (BOOL)needsSizing;

- (void)setWindowFrameForAttachingToRect:(NSRect)screenRect onScreen:(NSScreen *)screen preferredEdge:(NSRectEdge)edge popUpSelectedItem:(int)selectedItemIndex;
- (float) horizontalEdgePadding;
- (void)setHorizontalEdgePadding:(float) pad;
    // How much empty pad should be left on either edge of the item.

#endif
