/*
//  NSMenu(iTeXMac).m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sat Sep 22 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSZone.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSEnumerator.h>
#import <Foundation/NSException.h>
#import <Foundation/NSString.h>
#import <Foundation/NSString.h>
#import <Foundation/Foundation.h>
#import <AppKit/NSCell.h>
#import <AppKit/NSFont.h>
#import <AppKit/NSMenuItem.h>
#import <AppKit/NSMenuView.h>
#import <AppKit/NSFileWrapper.h>
#import "NSMenu(iTeXMac).h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= NSMenu(iTeXMac)
/*"Description forthcoming."*/
@implementation NSMenu(iTeXMac)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= deepItemWithTitle:
- (id <NSMenuItem>) deepItemWithTitle: (NSString *) aTitle;
/*"Extends the #{itemWithTitle:} to look down the menu hierarchy beginning from the receiver.
Only items with no submenus are searched for.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    static int depth=0;
    id result = nil;
    ++depth;
    if(depth<8)
    {
        NSEnumerator * enumerator = [[self itemArray] objectEnumerator];
        id item;
        while(item=[enumerator nextObject])
        {
            if([item submenu]!=nil)
            {
                result = [[item submenu] deepItemWithTitle: aTitle];
                if(result!=nil)
                    break;
            }
            else if([[item title] isEqualToString: aTitle])
            {
                result = item;
                break;
            }
        }
    }
    --depth;
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= itemWithAction:
- (id <NSMenuItem>) itemWithAction: (SEL) aSelector;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSEnumerator * enumerator = [[self itemArray] objectEnumerator];
    id item;
    while(item=[enumerator nextObject])
        if([item action] == aSelector)
            return item;
    return nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= indexOfItemWithAction:
- (int) indexOfItemWithAction: (SEL) aSelector;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    int index = [self numberOfItems];
    while(--index >=0) if([[self itemAtIndex: index] action] == aSelector) break;
    return index;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= indexOfItemWithTarget:
- (int) indexOfItemWithTarget: (id) aTarget;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    int index = [self numberOfItems];
    while(--index >=0) if([[[self itemAtIndex: index] target] isEqual: aTarget]) break;
    return index;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= deepItemWithAction:
- (id <NSMenuItem>) deepItemWithAction: (SEL) aSelector;
/*"Extends the #{itemWithTitle:} to look down the menu hierarchy beginning from the receiver.
Only items with no submenus are searched for.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    static int depth = 0;
    id result = nil;
    ++depth;
    if(depth<32)
    {
        NSEnumerator * enumerator = [[self itemArray] objectEnumerator];
        id item;
        while(item=[enumerator nextObject])
        {
//            NSLog(@"%@, %@", [item title], NSStringFromSelector([item action]));
            if([item hasSubmenu])
            {
                result = [[item submenu] deepItemWithAction: aSelector];
                if(result!=nil)
                    break;
            }
            else if([item action] == aSelector)
            {
                result = item;
                break;
            }
        }
    }
    --depth;
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= deepItemWithRepresentedObject:
- (id <NSMenuItem>) deepItemWithRepresentedObject: (id) anObject;
/*"Seeks the menu hierarchy beginning from the receiver, the first positive occurrence is returned.
Only items with no submenus are searched for.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Better managtement of the nil arg
"*/
{
    static int depth = 0;
    id result = nil;
    ++depth;
    if(!anObject)
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] no nil argument.",
            [self class], NSStringFromSelector(_cmd)];
    if((anObject) && (depth<32))
    {
        NSEnumerator * enumerator = [[self itemArray] objectEnumerator];
        id item;
        while(item = [enumerator nextObject])
        {
            if([item hasSubmenu])
            {
                result = [[item submenu] deepItemWithRepresentedObject: anObject];
                if(result!=nil)
                    break;
            }
            else if([[item representedObject] isEqual: anObject])
            {
                result = item;
                break;
            }
        }
    }
    --depth;
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= itemWithRepresentedObject:
- (id <NSMenuItem>) itemWithRepresentedObject: (id) anObject;
/*"Seeks the menu hierarchy beginning from the receiver, the first positive occurrence is returned.
Only items with no submenus are searched for.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSEnumerator * enumerator = [[self itemArray] objectEnumerator];
    id item = nil;
    while(item = [enumerator nextObject])
        if([[item representedObject] isEqual: anObject])
            break;
    return item;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= isRootMenu
- (BOOL) isRootMenu;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return ([self supermenu] == nil);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= deselectItemsWithAction:
- (void) deselectItemsWithAction: (SEL) anAction;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSEnumerator * enumerator = [[self itemArray] objectEnumerator];
    id item;
    while(item=[enumerator nextObject])
        if([item action] == anAction)
            [item setState: NSOffState];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= removeItemsWithAction:
- (void) removeItemsWithAction: (SEL) anAction;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    int index;
    while((index = [self indexOfItemWithAction: anAction]) != -1)
            [self removeItemAtIndex: index];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= removeItemsWithRepresentedObject:
- (void) removeItemsWithRepresentedObject: (id) anObject;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    int index;
    while((index = [self indexOfItemWithRepresentedObject: anObject]) != -1)
            [self removeItemAtIndex: index];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= deepCopy
- (NSMenu *) deepCopy;
/*"Simply forwards a #{deepCopyWithZone:} to the receiver, with the receiver's zone as argument.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [self deepCopyWithZone: [self zone]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= deepCopyWithZone:
- (NSMenu *) deepCopyWithZone: (NSZone *) aZone;
/*"The menu items are also copied, not only retained.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: We should write a NSMenuItem category to implement the #{copy} method but it is worth doing it.
The cell is not copied. Better should be done.
"*/
{
    NSMenu * result = [[[self class] allocWithZone: aZone] initWithTitle: [self title]];
    int index = 0;
    while(index < [[self itemArray] count])
    {
        NSMenuItem * MI5 = [self itemAtIndex: index];
        NSMenuItem * MI6 = [[[MI5 class] allocWithZone: aZone] initWithTitle: [MI5 title]
                                            action: [MI5 action] keyEquivalent: [MI5 keyEquivalent]];
        [MI6 setTarget: [MI5 target]];
        [MI6 setRepresentedObject: [MI5 representedObject]];
        [result addItem: MI6];
        ++index;
        [MI6 release];
    }
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= makeCellSizeSmall
- (void) makeCellSizeSmall;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: We should write a NSMenuItem category to implement the #{copy} method but it is worth doing it.
The cell is not copied. Better should be done.
"*/
{
    if (![self menuRepresentation])
        [self setMenuRepresentation: [[[NSMenuView alloc] initWithFrame: NSZeroRect] autorelease]];
    [[self menuRepresentation] setFont: [NSFont menuFontOfSize: [NSFont smallSystemFontSize]]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= deepMakeCellSizeSmall
- (void) deepMakeCellSizeSmall;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: We should write a NSMenuItem category to implement the #{copy} method but it is worth doing it.
The cell is not copied. Better should be done.
"*/
{
    NSEnumerator * E = [[self itemArray] objectEnumerator];
    NSMenuItem * MI;
    while(MI = [E nextObject])
        if([MI hasSubmenu])
            [[MI submenu] deepMakeCellSizeSmall];
    [self makeCellSizeSmall];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= deepEnableItems
- (void) deepEnableItems;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: We should write a NSMenuItem category to implement the #{copy} method but it is worth doing it.
The cell is not copied. Better should be done.
"*/
{
    NSEnumerator * E = [[self itemArray] objectEnumerator];
    NSMenuItem * MI;
    while(MI = [E nextObject])
        if([MI hasSubmenu])
            [[MI submenu] deepEnableItems];
        else if([MI action] != NULL)
        {
            [MI setEnabled: YES];
            if(![MI isEnabled]) {[MI setEnabled: YES];NSLog(@"%@ %@", [MI title], @"NO");}
        }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= deepSetTarget:
- (void) deepSetTarget: (id) aTarget;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: We should write a NSMenuItem category to implement the #{copy} method but it is worth doing it.
The cell is not copied. Better should be done.
"*/
{
    NSEnumerator * E = [[self itemArray] objectEnumerator];
    NSMenuItem * MI;
    while(MI = [E nextObject])
        if([MI hasSubmenu])
            [[MI submenu] deepSetTarget: aTarget];
        else if([MI action] != NULL)
            [MI setTarget: aTarget];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= deepSetAutoenablesItems:
- (void) deepSetAutoenablesItems: (BOOL) aFlag;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSEnumerator * E = [[self itemArray] objectEnumerator];
    NSMenuItem * MI;
    while(MI = [E nextObject])
        if([MI hasSubmenu])
            [[MI submenu] deepSetAutoenablesItems: aFlag];
    [self setAutoenablesItems: aFlag];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= hierarchicalMenuAtPath:action:target:
+ (NSMenu *) hierarchicalMenuAtPath: (NSString *) aFullPath action: (SEL) aSelector target: (id) aTarget;
/*"Description Forthcoming. ".hidden" files compatible.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: retain?
"*/
{
    NSDictionary * D = [aFullPath length]?
        [[[[NSFileWrapper alloc] initWithPath: aFullPath] autorelease] fileWrappers]: nil;
    if(D)
    {
        NSMenu * result = [[NSMenu allocWithZone: [NSMenu menuZone]] initWithTitle: [NSString string]];
        NSEnumerator * enumerator = [[[D allKeys] sortedArrayUsingSelector: @selector(compare:)] objectEnumerator];
        NSString * key;
        NSMutableArray * hiddenFiles = [NSMutableArray array];
        {
            NSString * dotHidden = [NSString stringWithContentsOfFile:
                [[aFullPath stringByDeletingLastPathComponent] stringByAppendingPathComponent: @".hidden"]];
            NSEnumerator * E = [[dotHidden componentsSeparatedByString: @"\n"] objectEnumerator];
            id object;
            while(object = [E nextObject])
                if([object isKindOfClass: [NSString class]] && [object length])
                    [hiddenFiles addObjectsFromArray: [object componentsSeparatedByString: @"\r"]];
        }
        while(key = [enumerator nextObject])
        {
            NSFileWrapper * FW = [D objectForKey: key];
            NSString * menuItemTitle = [FW preferredFilename];
            if((![menuItemTitle hasPrefix: @"."]) && ([hiddenFiles indexOfObject: menuItemTitle] == NSNotFound))
            {
                int recursionFireWall = 0;
                while([FW isSymbolicLink] && (recursionFireWall++<32))
                    FW = [[[NSFileWrapper alloc] initWithPath: [FW symbolicLinkDestination]] autorelease];
                if([FW isRegularFile])
                {
                    NSMenuItem * MI = [result addItemWithTitle: menuItemTitle action: aSelector
                                                    keyEquivalent: [NSString string]];
                    [MI setTarget: aTarget];
                    [MI setRepresentedObject: [aFullPath stringByAppendingPathComponent: [FW preferredFilename]]];
                }
                else if([FW isDirectory])
                {
                    NSMenuItem * MI = [result addItemWithTitle: menuItemTitle action: NULL
                                                    keyEquivalent: [NSString string]];
                    [MI setSubmenu: [self hierarchicalMenuAtPath:
                                    [aFullPath stringByAppendingPathComponent: [FW preferredFilename]]
                                                    action: aSelector target: aTarget]];
                }
            }
        }
        return [result autorelease];
    }
    else
        return nil;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= NSMenu(iTeXMac)

