/*
//  NSUserDefaults(iTeXMac).m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Thu Oct 25 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSException.h>
#import <Foundation/NSString.h>
#import <Foundation/NSValue.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSNotification.h>
#import <AppKit/NSApplication.h>
#import "NSUserDefaults(iTeXMac).h"

NSString * const iTMUserDefaultsDidChangeFontOrColorNotification = @"iTMUserDefaultsDidChangeFontOrColorNotification";
NSString * const iTMUserDefaultsDidChangeNotification = @"iTMUserDefaultsDidChangeNotification";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= NSUserDefaults(iTeXMac)
/*"Description forthcoming."*/
@implementation NSUserDefaults(iTeXMac)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= notifyChangeNow
- (void) notifyChangeNow;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] postNotificationName: iTMUserDefaultsDidChangeNotification object: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= notifyFontOrColorChangeNow
- (void) notifyFontOrColorChangeNow;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] postNotificationName: iTMUserDefaultsDidChangeFontOrColorNotification object: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= fontForKey:
- (NSFont *) fontForKey: (NSString *) aKey;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [NSFont fontWithNameSizeDictionary: [self objectForKey: aKey]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setFont:forKey:
- (void) setFont: (NSFont *) aFont forKey: (NSString *) aKey;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setObject: [aFont nameSizeDictionary] forKey: aKey];
    [self notifyFontOrColorChangeNow];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  registerFont:forKey:
- (void) registerFont: (NSFont *) aFont forKey: (NSString *) aKey;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self registerDefaults: [NSDictionary dictionaryWithObject: [aFont nameSizeDictionary] forKey: aKey]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= colorForKey:
- (NSColor *) colorForKey: (NSString *) aKey;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [NSColor colorWithRGBADictionary: [self objectForKey: aKey]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setColor:forKey:
- (void) setColor: (NSColor *) aColor forKey: (NSString *) aKey;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setObject: [aColor RGBADictionary] forKey: aKey];
    [self notifyFontOrColorChangeNow];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  registerColor:forKey:
- (void) registerColor: (NSColor *) aColor forKey: (NSString *) aKey;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self registerDefaults: [NSDictionary dictionaryWithObject: [aColor RGBADictionary] forKey: aKey]];
    return;
}
@end

@implementation NSFont(iTMUserDefaults)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= fontWithNameSizeDictionary:
+ (NSFont *) fontWithNameSizeDictionary: (NSDictionary *) aDictionary;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(@"aDictionary: %@", aDictionary);
    if([aDictionary isKindOfClass: [NSDictionary class]])
    {
        NSString * fontName = [aDictionary objectForKey: @"Name"];
        NSNumber * fontSize = [aDictionary objectForKey: @"Size"];
//        NSLog(@"fontName; %@, [fontSize floatValue]: %f", fontName, [fontSize floatValue]);
        if([fontName isKindOfClass: [NSString class]] && [fontSize respondsToSelector: @selector(floatValue)])
        {
            NSFont * result = [NSFont fontWithName: fontName size: [fontSize floatValue]];
            if(result) return result;
        }
        else
            NSLog(@"NO 1");
    }
    return [NSFont userFontOfSize: 12];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  nameSizeDictionary
- (NSDictionary *) nameSizeDictionary;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [NSDictionary dictionaryWithObjectsAndKeys:
                        [self fontName], @"Name", [NSNumber numberWithFloat: [self pointSize]], @"Size", nil];
}
@end

@implementation NSColor(iTMUserDefaults)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= colorWithRGBADictionary:
+ (NSColor *) colorWithRGBADictionary: (NSDictionary *) aDictionary;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(@"aDictionary: %@", aDictionary);
    NSColor * result = [NSColor blackColor];
    NS_DURING
    if([aDictionary isKindOfClass: [NSDictionary class]])
        result = [NSColor colorWithCalibratedRed: [[aDictionary objectForKey: @"R"] floatValue]
                            green: [[aDictionary objectForKey: @"G"] floatValue]
                                blue:[[aDictionary objectForKey: @"B"] floatValue]
                                    alpha: [[aDictionary objectForKey: @"A"] floatValue]];
    NS_HANDLER
    [NSApp reportException: localException];
    NS_ENDHANDLER
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  RGBADictionary
- (NSDictionary *) RGBADictionary;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSColor * color = [self colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
    NSDictionary * result = [NSDictionary dictionaryWithObjectsAndKeys:
                        [NSNumber numberWithFloat: [color redComponent]], @"R",
                        [NSNumber numberWithFloat: [color greenComponent]], @"G",
                        [NSNumber numberWithFloat: [color blueComponent]], @"B",
                        [NSNumber numberWithFloat: [color alphaComponent]], @"A",
                            nil];
    return result;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= NSUserDefaults(iTeXMac)

