/*
//  iTMTextFinder.h
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Mon Sep 03 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <AppKit/NSWindowController.h>

#define Forward YES
#define Backward NO

@class NSNotification, NSWindow, NSString;

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTextFinder

@interface iTMTextFinder : NSWindowController
{
    id findTextField;
    id replaceTextField;
    id ignoreCaseButton;
    id replaceAllScopeMatrix;
    NSString * _FindString;
    NSString * _ReplaceString;
    BOOL findStringChangedSinceLastPasteboardUpdate;
    BOOL lastFindWasSuccessful;		/* A bit of a kludge */
    BOOL _EntireFileFlag;
    BOOL _CaseInsensitiveFlag;
}

/* Common way to get a text finder. One instance of TextFinder per app is good enough. But likely...*/
+ (id) sharedTextFinder;
- (void) dealloc;
- (void) windowDidLoad;

/* Main method for external users; does a find in the first responder. Selects found range or beeps. */
- (BOOL) find: (BOOL) direction;

/* Gets the first responder and returns it if it's an NSTextView */
- (NSTextView *) textViewToSearchIn;

- (void) installFindResponderForWindow: (NSWindow *) aWindow;

/* Get/set the current find string. Will update UI if UI is loaded */
- (NSString *) findString;
- (void) setFindString: (NSString *) aString;
- (NSString *) replaceString;
- (void) setReplaceString: (NSString *) aString;

/* Misc internal methods */
- (void) applicationWillResignActive: (NSNotification *) aNotification;

- (void) showFindPanel: (id) sender;
- (void) enterSelection: (id) sender;
- (void) enterFindPboardSelection: (id) sender;

/* Methods sent from the find panel UI */
- (void) findNextAndOrderFindPanelOut: (id) sender;
- (void) findNext: (id) sender;
- (void) findPrevious: (id) sender;
- (void) replace: (id) sender;
- (void) replaceAndFind:(id) sender;
- (void) replaceAll: (id) sender;

- (void) validateUserInterfaceItems;

- (id) initWithCoder: (NSCoder *) aDecoder;

@end


@interface NSString (NSStringTextFinding)
- (NSRange)rangeOfString:(NSString *)string selectedRange:(NSRange)selectedRange options:(unsigned)mask wrap:(BOOL)wrapFlag;
@end


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMFindResponder

@interface iTMFindResponder : NSResponder
/*"Class methods"*/
/*"Setters and Getters"*/
/*"Main methods"*/
- (void) centerSelectionInVisibleArea: (id) sender;
- (void) showFindPanel: (id) sender;
- (void) enterSelection: (id) sender;
- (void) enterFindPboardSelection: (id) sender;
- (void) findNext: (id) sender;
- (void) findPrevious: (id) sender;
- (void) replace: (id) sender;
- (void) replaceAndFind:(id) sender;
- (void) replaceAll: (id) sender;
/*"Overriden methods"*/
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTextFinder

