/*
//  iTMHelpManager.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Mon Sep 24 2001.
//  Copyright  2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import "iTMHelpManager.h"
#import "iTMToolbarItem.h"
#import "iTMHelpButton.h"
#import "NSString(iTeXMac).h"

NSString * const iTMToolbarShowHelpItemIdentifier = @"showHelp:";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMHelpManager
/*"Description forthcoming."*/
@implementation iTMHelpManager
static id _iTMHelpManager;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= sharedManager
+ (iTMHelpManager *) sharedManager;
/*"Lazy initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _iTMHelpManager? _iTMHelpManager: (_iTMHelpManager = [[self allocWithZone: [NSApp zone]] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= init
- (id) init;
/*"The first #{init} message sent defines the shared instance. Only one instance ever created.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(_iTMHelpManager)
    {
        if(![self isEqual: _iTMHelpManager])
            [super dealloc];
        return [_iTMHelpManager retain];
    }
    else
    {
        self = [super init];
        return _iTMHelpManager = self;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= helpButton
- (iTMHelpButton *) helpButton;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMHelpButton * helpButton = [[iTMHelpButton alloc] initWithFrame: NSZeroRect];
    [helpButton setAction: @selector(showHelp:)];
    [helpButton setTarget: self];
    return [helpButton autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= helpToolbarItem
- (iTMToolbarItem *) helpToolbarItem;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMToolbarItem * result = [[iTMToolbarItem alloc] initWithItemIdentifier: iTMToolbarShowHelpItemIdentifier];
    [result setView: [self helpButton]];
    [result setMinSize: [[result view] frame].size];
    [result setMaxSize: [[result view] frame].size];
    [result setToolTip: NSLocalizedStringFromTableInBundle(iTMToolbarShowHelpItemIdentifier, @"Basic",
        [NSBundle bundleForClass: [self class]], nil)];
    [result setPaletteLabel: NSLocalizedStringFromTableInBundle(iTMToolbarShowHelpItemIdentifier, @"Basic",
        [NSBundle bundleForClass: [self class]], nil)];
    [result setLabel: NSLocalizedStringFromTableInBundle(iTMToolbarShowHelpItemIdentifier, @"Basic",
        [NSBundle bundleForClass: [self class]], nil)];
    return [result autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showHelp:
- (void) showHelp: (id) sender;
/*"When the receiver is in a palette, it should always be valid. Its target should be itself. Forwards the message to NSApp.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: implement a sender specific Help
"*/
{
    [NSApp showHelp: self];
    return;
}
#if 1
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showHelpFor:key:
- (void) showHelpFor: (id) object key: (NSString *) aKey;
/*"Public method: you give a key and the help controller manages all the stuff in order to show the help
related to the key. It manages localization...
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * question = [NSLocalizedStringFromTableInBundle(aKey, @"HelpKeys",
        (object? [NSBundle bundleForClass: [object class]]: [NSBundle mainBundle]),
        "the question to be asked for...") stringWithSubstring: @" " replacedByString: @"%20"];
    if([question length])
    {
        NSString * bookName = [[[[NSBundle mainBundle] infoDictionary] objectForKey: @"CFBundleHelpBookName"]
                                            stringWithSubstring: @" " replacedByString: @"%20"];
        if([bookName length])
        {
            NSURL * url = [NSURL URLWithString:
                [[[@"help:search=\'" stringByAppendingString: question]
                    stringByAppendingString: @"\'%20bookID="]
                        stringByAppendingString: bookName]];
            if(![[NSWorkspace sharedWorkspace] openURL: url])
                NSBeep();
        }
    }
    return;
}
#endif
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMHelpManager

