/*
//  iTMStatusNotification.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Mon Dec 03 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "iTMStatusNotification.h"

NSString * const iTMStatusTimeOutKey = @"iTMStatusTimeOut";
NSString * const iTMStatusNotificationName = @"iTMStatusNotificationName";
NSString * const iTMSNStatusKey = @"status";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMStatusNotificationCenter
/*"Description forthcoming."*/
@implementation iTMStatusNotificationCenter
static id _iTMStatusNotificationCenter;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  defaultCenter
+ (void) initialize;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSDictionary * factoryDefaults = [NSDictionary dictionaryWithObjectsAndKeys: 
        [NSNumber numberWithFloat: 180],  iTMStatusTimeOutKey,
            nil];
    [[NSUserDefaults standardUserDefaults] registerDefaults: factoryDefaults];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  defaultCenter
+ (id) defaultCenter;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _iTMStatusNotificationCenter? _iTMStatusNotificationCenter:
                            (_iTMStatusNotificationCenter = [[self allocWithZone: [NSApp zone]] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"The first object inited is the shared one.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(_iTMStatusNotificationCenter)
    {
        if(![self isEqual: _iTMStatusNotificationCenter])
            [super dealloc];
        return [_iTMStatusNotificationCenter retain];
    }
    else
    {
//        NSLog(@"status notification center has been inited");
        return _iTMStatusNotificationCenter = [super init];
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
#if 0
/*"Never dealloc.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) reallyDealloc;
#endif
/*"Description Forhcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setCurrentStatus: nil];
    [self setTimer: nil];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  timer
- (NSTimer *) timer;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _Timer;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setTimer:
- (void) setTimer: (NSTimer *) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSTimer class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSTimer argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else
    {
        [_Timer invalidate];
        [_Timer autorelease];
        _Timer = [argument retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  currentStatus
- (NSString *) currentStatus;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [_CurrentStatus length]? [[_CurrentStatus copy] autorelease]: [NSString string];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setCurrentStatus:
- (void) setCurrentStatus: (NSString *) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSString class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSString argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else
    {
        [_CurrentStatus autorelease];
        _CurrentStatus = [argument copy];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  addObserver:
- (void) addObserver: (id) observer;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    SEL statusNotified = @selector(statusNotified:);
    NSAssert1([observer respondsToSelector: statusNotified], @"%@ must respond to statusNotified: message.", observer);
    [self addObserver: observer selector: statusNotified name: iTMStatusNotificationName object: nil];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  postNotificationWithStatus:
- (void) postNotificationWithStatus: (NSString *) aStatus;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    int length = [aStatus length];
    [self setCurrentStatus: (length? aStatus: nil)];
    [self postNotificationName: iTMStatusNotificationName
        object: nil
            userInfo: [NSDictionary dictionaryWithObject: [self currentStatus] forKey: iTMSNStatusKey]];
    if(length)
        [self setTimer: [NSTimer scheduledTimerWithTimeInterval:
                            [[NSUserDefaults standardUserDefaults] floatForKey: iTMStatusTimeOutKey]
            target: self
                selector: @selector(cleanStatusNotified:)
                    userInfo: nil
                        repeats: NO]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  cleanStatusNotified:
- (void) cleanStatusNotified: (NSNotification *) aNotification;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self cleanStatus];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  cleanStatus:
- (void) cleanStatus;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setTimer: nil];
    [self postNotificationWithStatus: nil];
    return;
}
@end

@implementation iTMStatusTextField
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initWithFrame:
- (id) initWithFrame: (NSRect) aFrame;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    self = [super initWithFrame: aFrame];
    [[iTMStatusNotificationCenter defaultCenter] addObserver: self];
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initWithCoder:
- (id) initWithCoder: (NSCoder *) decoder;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    self = [super initWithCoder: decoder];
    [[iTMStatusNotificationCenter defaultCenter] addObserver: self];
    [self setStringValue: [[iTMStatusNotificationCenter defaultCenter] currentStatus]];
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[iTMStatusNotificationCenter defaultCenter] removeObserver: self];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  statusNotified:
- (void) statusNotified: (NSNotification *) aNotification;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NS_DURING
//    NSLog(@"statusNotified: %@", [[aNotification userInfo] objectForKey: iTMSNStatusKey]);
    [self setStringValue: [[aNotification userInfo] objectForKey: iTMSNStatusKey]];
    NS_HANDLER
    NSLog(@"-[iTMStatusTextField statusNotified:]: Bad user info.");
    NS_ENDHANDLER
    [self display];
    return;
}
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMStatusNotificationCenter

@implementation NSNotificationCenter(iTMStatus)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  postNotificationWithStatus:
- (void) postNotificationWithStatus: (NSString *) aStatus;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: aStatus];
    return;
}
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  NSNotificationCenter(iTMStatus)

