/*
//  iTMWindowsObserver.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Wed Sep 12 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSDictionary.h>
#import "NSWindow(iTeXMac).h"
#import "iTMWindowsObserver.h"

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMWindowsObserver
/*"See the NSWindow(iTeXMac) for useful details."*/
@implementation iTMWindowsObserver
static id _iTMWindowsObserver;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Lazy initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [[iTMWindowsObserver sharedWindowsObserver] observeWindows];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sharedWindowsObserver
+ (id) sharedWindowsObserver;
/*"Lazy initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _iTMWindowsObserver? _iTMWindowsObserver: (_iTMWindowsObserver = [[self alloc] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"Only one instance.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(_iTMWindowsObserver)
    {
        if(![_iTMWindowsObserver isEqual: self])
            [self dealloc];
        return [_iTMWindowsObserver retain];
    }
    else
        return _iTMWindowsObserver = [super init];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  observeWindows
- (void) observeWindows;
/*"Registers the class object as an observer of any notification that indicates a change in the window frame.
THIS IS NOT MEANT TO BE OVERRIDEN. This MUST be called early.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    #define OBSERVE(SELECTOR, NOTIFICATION)\
        [[NSNotificationCenter defaultCenter]\
            addObserver: self\
                selector:@selector(SELECTOR)\
                    name: NOTIFICATION\
                        object: nil];
    OBSERVE(rememberCurrentWindowPosition:, NSWindowDidBecomeKeyNotification);
    OBSERVE(rememberCurrentWindowPosition:, NSWindowDidBecomeMainNotification);
    OBSERVE(rememberCurrentWindowPosition:, NSWindowDidDeminiaturizeNotification);
    OBSERVE(rememberCurrentWindowPosition:, NSWindowDidMoveNotification);
    OBSERVE(rememberCurrentWindowPosition:, NSWindowDidResizeNotification);
    OBSERVE(forgetCurrentWindowPosition:, NSWindowDidMiniaturizeNotification);
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  rememberCurrentWindowPosition:
- (void) rememberCurrentWindowPosition: (NSNotification *) aNotification;
/*"Subclasses should not need to override this method. Registers in the registration domain of the user defaults database the frame of aNotification object. The key used is the class frameIdentifier of this object.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    NSWindow * W = [aNotification object];
    Class windowClass = [W class];
    if([windowClass shouldBeObserved])
    {
        [W saveFrameUsingName: [windowClass frameAutosaveIdentifierForMode: iTMWindowFrameSavedMode]];
        [W saveFrameUsingName: [windowClass frameAutosaveIdentifierForMode: iTMWindowFrameCurrentMode]];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  forgetCurrentWindowPosition:
- (void) forgetCurrentWindowPosition: (NSNotification *) aNotification;
/*"Subclasses should not need to override this method. The key used is the class frameIdentifier of this object but the registered frame is a zero rect.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * windowIdentifier = [[[aNotification object] class] frameIdentifier];
    if([windowIdentifier length]>0)
    {
        [[NSUserDefaults standardUserDefaults] registerDefaults:
            [NSDictionary dictionaryWithObject: NSStringFromRect(NSZeroRect)
                forKey: windowIdentifier]];
    }
    return;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMWindowsObserver

