/*
//  iTMApplication.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Mon Dec 03 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSDebug.h>
#import <Foundation/NSFileManager.h>
#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "iTMApplication.h"

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMApplication
/*"Description forthcoming."*/
@implementation iTMApplication
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sendEvent:
- (void) sendEvent: (NSEvent *) anEvent;
/*"I am ashamed. This might be done elsewhere, but i do not know where or how...
This patch deals with command+arrow keystroke: different meanings while in text view or in pdf view.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    switch([anEvent type])
    {
        case NSKeyDown:
        {
            int mask = NSAlphaShiftKeyMask|NSShiftKeyMask|NSControlKeyMask|NSAlternateKeyMask|NSCommandKeyMask;
//            NSLog(@"[anEvent modifierFlags]: %u, %u, %u, %u", mask, [anEvent modifierFlags], mask & [anEvent modifierFlags], NSCommandKeyMask | NSAlternateKeyMask);
            if(([anEvent modifierFlags] & mask) == NSCommandKeyMask)
            {
                switch([[anEvent charactersIgnoringModifiers] characterAtIndex: 0])
                {
                    case NSUpArrowFunctionKey:
                    {
                        NSResponder * TV = [self targetForAction: @selector(moveToBeginningOfDocument:)];
                        if(TV)
                        {
                            [TV moveToBeginningOfDocument: self];
                            return;
                        }
                    }
                    case NSDownArrowFunctionKey:
                    {
                        NSResponder * TV = [self targetForAction: @selector(moveToEndOfDocument:)];
                        if(TV)
                        {
                            [TV moveToEndOfDocument: self];
                            return;
                        }
                    }
                    case NSLeftArrowFunctionKey:
                    {
                        NSResponder * TV = [self targetForAction: @selector(moveToBeginningOfLine:)];
                        if(TV)
                        {
                            [TV moveToBeginningOfLine: self];
                            return;
                        }
                    }
                    case NSRightArrowFunctionKey:
                    {
                        NSResponder * TV = [self targetForAction: @selector(moveToEndOfLine:)];
                        if(TV)
                        {
                            [TV moveToEndOfLine: self];
                            return;
                        }
                    }
                }
            }
            else if(([anEvent modifierFlags] & mask) == (NSCommandKeyMask | NSAlternateKeyMask))
            {
                switch([[anEvent charactersIgnoringModifiers] characterAtIndex: 0])
                {
                    case NSHomeFunctionKey:
                    {
                        NSResponder * TV = [self targetForAction: @selector(displayLastPage:)];
                        if(TV)
                        {
                            [TV displayLastPage: self];
                            return;
                        }
                    }
                    case NSEndFunctionKey:
                    {
                        NSResponder * TV = [self targetForAction: @selector(displayFirstPage:)];
                        if(TV)
                        {
                            [TV displayFirstPage: self];
                            return;
                        }
                    }
                }
                NSLog(@"%u, %u, %u", NSHomeFunctionKey, NSEndFunctionKey, 
                    [[anEvent charactersIgnoringModifiers] characterAtIndex: 0]);
            }
        }
        default:
            break;
    }
    [super sendEvent: anEvent];
    return;
}
@end

#import "iTMProjectEngineer.h"

NSString * const iTMApplFileNameKey = @"FileName";// the name of AppleScript argument

@implementation iTMApplication(iTMScripting)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  handleTypesetScriptCommand:
- (id) handleTypesetScriptCommand: (NSScriptCommand *) SC;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (02/06/2002)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * path = [[[SC evaluatedArguments] objectForKey: iTMApplFileNameKey] stringByStandardizingPath];
    if([[NSFileManager defaultManager] isReadableFileAtPath: path])
        [[iTMProjectEngineer sharedProjectEngineer] typeset: path];
    else
        NSLog(@"No readable file at path: %@", path);
    return nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  handleCompileScriptCommand:
- (id) handleCompileScriptCommand: (NSScriptCommand *) SC;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (02/06/2002)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * path = [[[SC evaluatedArguments] objectForKey: iTMApplFileNameKey] stringByStandardizingPath];
    if([[NSFileManager defaultManager] isReadableFileAtPath: path])
        [[iTMProjectEngineer sharedProjectEngineer] compile: path];
    else
        NSLog(@"No readable file at path: %@", path);
    return nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  handlebibTeXScriptCommand:
- (id) handlebibTeXScriptCommand: (NSScriptCommand *) SC;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (02/06/2002)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * path = [[[SC evaluatedArguments] objectForKey: iTMApplFileNameKey] stringByStandardizingPath];
    if([[NSFileManager defaultManager] isReadableFileAtPath: path])
        [[iTMProjectEngineer sharedProjectEngineer] bibTeX: path];
    else
        NSLog(@"No readable file at path: %@", path);
    return nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  handleMakeIndexScriptCommand:
- (id) handleMakeIndexScriptCommand: (NSScriptCommand *) SC;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (02/06/2002)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * path = [[[SC evaluatedArguments] objectForKey: iTMApplFileNameKey] stringByStandardizingPath];
    if([[NSFileManager defaultManager] isReadableFileAtPath: path])
        [[iTMProjectEngineer sharedProjectEngineer] makeIndex: path];
    else
        NSLog(@"No readable file at path: %@", path);
    return nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  handleRenderScriptCommand:
- (id) handleRenderScriptCommand: (NSScriptCommand *) SC;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (02/06/2002)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * path = [[[SC evaluatedArguments] objectForKey: iTMApplFileNameKey] stringByStandardizingPath];
    if([[NSFileManager defaultManager] isReadableFileAtPath: path])
        [[iTMProjectEngineer sharedProjectEngineer] render: path];
    else
        NSLog(@"No readable file at path: %@", path);
    return nil;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMApplication

