/*
//  iTMApplicationDelegate.m
//  iTeXMac
//
//  Created by dirk on Tue Jan 23 2001.
//  Modified by jerome.laurens@u-bourgogne.fr on Tue Jun 26 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  original method by dirk Holmes
//  jerome.laurens@u-bourgogne.fr (07/12/2001):
//    -applicationWillFinishLaunching: modified
//    -applicationOpenUntitledFile: added
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSValue.h>
#import <Foundation/NSString.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSUserDefaults.h>
#import "NSUserDefaults(iTeXMac).h"
#import <AppKit/NSApplication.h>
#import <AppKit/NSWindow.h>
#import <AppKit/NSDocumentController.h>
#import <AppKit/NSDocument.h>
#import "NSMenu(iTeXMac).h"
#import "iTMApplicationDelegate.h"
#import "iTMDefaultsController.h"
#import "iTMTextFinder.h"
#import "iTMDocumentController.h"
#import "iTMProjectsManager.h"
#import "iTMEncodingController.h"
#import "iTMTextView.h"
#import "iTMTeXDocument.h"
#import "iTMRepDocument.h"
#import "iTMRepWindowController.h"
#import "iTMTeXWindowController.h"
#import "iTMSessionManager.h"
#import "iTMGenericProjectsManager.h"
#import "iTMTeTeXController.h"
#import "iTMStatusNotification.h"
#import "iTMTextWatcher.h"
#import "iTMPDFView.h"

NSString * const iTMMakeEmptyDocumentKey = @"iTMMakeEmptyDocument";

/*" This class is registered as the delegate of the iTeXMac NSApplication object. We do various stuff here, e.g. registering factory defaults, etc.
"*/

@implementation iTMApplicationDelegate
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialize
+ (void) initialize;
/*"Registers some defaults: initialize iTMDefaultsController.
Version History: jerome.laurens@u-bourgogne.fr (07/12/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    {
        [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys: 
                        [NSNumber numberWithBool: NO], iTMMakeEmptyDocumentKey,
                            nil]];
    }
    [iTMSessionManager initialize];// to register defaults defaults
    [iTMEncodingController initialize];// to register defaults defaults
    [iTMTeXDocument initialize];// to register defaults defaults
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= dealloc
- (void) dealloc;
/*"Registers some defaults: initialize iTMDefaultsController.
Version History: jerome.laurens@u-bourgogne.fr (07/12/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setApplicationDockMenu: nil];
    [self setFirstPanel: nil];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showPreferences:
- (void) showFindPanel: (id) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[iTMTextFinder sharedTextFinder] showFindPanel: self];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= applicationOpenUntitledFile:
- (BOOL) applicationOpenUntitledFile: (NSApplication *) theApplication;
/*"Description forthcoming.
Proposed by jerome.laurens@u-bourgogne.fr (07/12/2001)
I really don't know why but this simple method could replace the previous one. Nevertheless, I prefer what I can understand...
"*/
{
    return ![[NSUserDefaults standardUserDefaults] boolForKey: iTMMakeEmptyDocumentKey];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= applicationDockMenu:
- (NSMenu *) applicationDockMenu: (id) sender;
/*"Lazy initializer and message.
Proposed by jerome.laurens@u-bourgogne.fr (07/12/2001)
- < 1.1: 03/10/2002
To Do List: Nothing at first glance
"*/
{
    if(!_ADM)
        [self setApplicationDockMenu: [[[iTMProjectsManager defaultManager] activeProjectsMenu] supermenu]];
    return _ADM;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setApplicationDockMenu:
- (void) setApplicationDockMenu: (NSMenu *) argument;
/*"Description forthcoming.
Proposed by jerome.laurens@u-bourgogne.fr (07/12/2001)
- < 1.1: 03/10/2002
To Do List: Nothing at first glance
"*/
{
    if(![_ADM isEqual: argument])
    {
        if(argument && ![argument isKindOfClass: [NSMenu class]])
            [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSMenu argument expected: %@.",
                [self class], NSStringFromSelector(_cmd), argument];
        else
        {
            [_ADM release];
            _ADM = [argument retain];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= firstPanel
- (id) firstPanel;
/*"Lazy initializer.
Proposed by jerome.laurens@u-bourgogne.fr (07/12/2001)
- < 1.1: 03/10/2002
To Do List: Nothing at first glance
"*/
{
//    if(!_FP)
//        _FP = [[[[[NSWindowController alloc] initWithWindowNibName: @"iTMFirstPanel"] autorelease] window] retain];
    return _FP;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setFirstPanel:
- (void) setFirstPanel: (NSPanel *) argument;
/*"Description forthcoming.
Proposed by jerome.laurens@u-bourgogne.fr (07/12/2001)
- < 1.1: 03/10/2002
To Do List: Nothing at first glance
"*/
{
    if(![_ADM isEqual: argument])
    {
        if(argument && ![argument isKindOfClass: [NSPanel class]])
            [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSPanel argument expected: %@.",
                [self class], NSStringFromSelector(_cmd), argument];
        else
        {
            [_FP autorelease];
            _FP = [argument retain];
        }
    }
    return;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  showPreferences:
- (IBAction) showPreferences: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[[[[iTMPreferencesController allocWithZone: [self zone]] initWithWindowNibName: @"iTMPreferences"] autorelease]
            window] makeKeyAndOrderFront];
    return;
}
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  customizeGenerics:
- (IBAction) customizeGenerics: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [iTMGenericProjectsManager customizeGenericProjects: sender];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  showPreferences:
- (IBAction) showPreferences: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[iTMDefaultsController sharedDefaultsController] showWindow: sender];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMenuItem:
- (BOOL) validateMenuItem: (id <NSMenuItem>) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    SEL action = [sender action];
    if(action == @selector(customizeGenerics:))
        return YES;
    else if(action == @selector(showPreferences:))
        return YES;
    else
        return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= applicationWillFinishLaunching:
- (void) applicationWillFinishLaunching: (NSNotification *) aNotification;
/*"Description forthcoming.
Proposed by jerome.laurens@u-bourgogne.fr (07/12/2001)
To Do: problem when there is no UI.
"*/
{
    [[self firstPanel] orderFront: self];
    [iTMTextWatcher initialize];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= applicationDidFinishLaunching:
- (void) applicationDidFinishLaunching: (NSNotification *) aNotification;
/*"Updates the templates and macros menus.
Proposed by jerome.laurens@u-bourgogne.fr (07/12/2001)
To Do: problem when there is no UI.
"*/
{
    [iTMTeTeXController sharedTeTeXController];// installs the tetex help menus as side effect
    [self setFirstPanel: nil];
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: [NSString string]];
    return;
}
@end