/*
//  iTMDocumentController.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sat Sep 22 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSObject.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSUserDefaults.h>
#import <AppKit/NSEvent.h>
#import <AppKit/NSView.h>
#import <AppKit/NSPanel.h>
#import <AppKit/NSDocument.h>
#import <AppKit/NSPopUpButton.h>
#import <AppKit/NSWorkspace.h>
#import <AppKit/NSMenu.h>
#import <AppKit/AppKit.h>
#import <Carbon/Carbon.h>
#import "NSMenu(iTeXMac).h"
#import "NSString(iTeXMac).h"
#import "iTMDocumentController.h"
#import "iTMProjectsManager.h"
#import "iTMProjectDocument.h"
#import "iTMTeXDocument.h"
#import "iTMRepDocument.h"
#import "iTMRenderDocument.h"
#import "iTMPDFDocument.h"
#import "iTMMacroDocument.h"
#import "iTMEncodingController.h"
#import "iTMSessionManager.h"

NSString * const iTMProjectDocumentsKey = @"Projects";
NSString * const iTMTeXDocumentsKey = @"TeX Documents";
NSString * const iTMMetaPostDocumentsKey = @"MetaPost Documents";
NSString * const iTMRepresentationDocumentsKey = @"Image, PDF Documents";
NSString * const iTMAuxiliaryDocumentsKey = @"Auxiliary Files";
NSString * const iTMLogDocumentsKey = @"Log Files";
NSString * const iTMTextDocumentsKey = @"Text Documents";
NSString * const iTMMacrosDocumentsKey = @"iTeXMac Macros";

NSString * const iTMUDUsesPDFHelperKey = @"iTMUsesExternalPDFViewer";
NSString * const iTMUDPDFHelperKey = @"iTMExternalPDFViewer";
NSString * const iTMUDUsesTeXHelperKey = @"iTMUsesExternalTextEditor";
NSString * const iTMUDTeXHelperKey = @"iTMExternalTextEditor";

NSString * const iTMDocumentControllerWillCloseAllNotification = @"iTMDocumentControllerWillCloseAll";
NSString * const iTMDocumentControllerDidCloseAllNotification = @"iTMDocumentControllerDidCloseAll";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMDocumentController
/*"Adds management for project documents.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: the popUp choice should be in the user defaults...
"*/
@implementation iTMDocumentController
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialize
+ (void) initialize;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    {
        NSDictionary * factoryDefaults = [NSDictionary dictionaryWithObjectsAndKeys: 
            [NSNumber numberWithBool: NO], iTMUDUsesPDFHelperKey,
            @"Acrobat Reader 5.0", iTMUDPDFHelperKey,
            [NSNumber numberWithBool: NO], iTMUDUsesTeXHelperKey,
            @"TextEdit", iTMUDTeXHelperKey,
                nil];
        [[NSUserDefaults standardUserDefaults] registerDefaults: factoryDefaults];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= dealloc
- (void) dealloc;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSDistributedNotificationCenter defaultCenter] removeObserver: self];
    [self setOpenStringEncodingPopUp: nil];
    [self setOpenFileTypePopUp: nil];
    [self setNewFileTypePopUp: nil];
    [self setAutoStringEncodingCheck: nil];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= awakeFromNib
- (void) awakeFromNib;
/*"The first instance MUST be created in the main nib file.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([[self superclass] instancesRespondToSelector: _cmd])
        [super performSelector: _cmd];
    [[NSDistributedNotificationCenter defaultCenter] addObserver: self
        selector: @selector(shouldOpenTextFile:)
            name: @"iTMTextEditorShouldOpenTextFile"
                object: nil
                    suspensionBehavior: NSNotificationSuspensionBehaviorDeliverImmediately];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateUserInterface
- (void) validateUserInterface;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSUserDefaults * SUD = [NSUserDefaults standardUserDefaults];
    [SUD setObject: [SUD objectForKey: iTMPreferredStringEncodingKey] forKey: iTMOpenStringEncodingKey];
    {
        [_ASEC setEnabled: YES];
        [_ASEC setState: [SUD boolForKey: iTMIsAutoStringEncodingKey]? NSOnState: NSOffState];
    }
    if(_OSEP)
    {
        int tag = [[NSUserDefaults standardUserDefaults] integerForKey: iTMOpenStringEncodingKey];
        [_OSEP removeAllItems];
        [_OSEP setMenu: [iTMEncodingController stringEncodingsMenuWithAction: NULL target: nil]];
        NS_DURING
        tag = [_OSEP indexOfItemWithTag: tag];
        [_OSEP selectItemAtIndex: tag];
//        NSLog(@"index %u", tag);
        NS_HANDLER
        if([_OSEP numberOfItems]) [_OSEP selectItemAtIndex: 0];
        NS_ENDHANDLER
    }
    {
        [_OFTP removeAllItems];
        [_OFTP addItemWithTitle: NSLocalizedStringFromTableInBundle(iTMTeXDocumentsKey, @"General",
                    [NSBundle bundleForClass: [self class]], "Human readable")];
        [[_OFTP lastItem] setRepresentedObject: [NSArray arrayWithObject: iTMTeXDocumentType]];
        [_OFTP addItemWithTitle: NSLocalizedStringFromTableInBundle(iTMProjectDocumentsKey, @"General",
                    [NSBundle bundleForClass: [self class]], "Human readable")];
        [[_OFTP lastItem] setRepresentedObject: [NSArray arrayWithObject: iTMProjectDocumentType]];
        [_OFTP addItemWithTitle: NSLocalizedStringFromTableInBundle(iTMRepresentationDocumentsKey, @"General",
                    [NSBundle bundleForClass: [self class]], "Human readable")];
        [[_OFTP lastItem] setRepresentedObject: [NSArray arrayWithObjects: iTMRepDocumentType,
                            iTMPDFDocumentType, iTMDVIDocumentType, iTMPSDocumentType, iTMEPSDocumentType, nil]];
        [_OFTP addItemWithTitle: NSLocalizedStringFromTableInBundle(iTMTextDocumentsKey, @"General",
                    [NSBundle bundleForClass: [self class]], "Human readable")];
        [[_OFTP lastItem] setRepresentedObject: [NSArray arrayWithObject: iTMTextDocumentType]];
        [_OFTP addItemWithTitle: NSLocalizedStringFromTableInBundle(iTMMacrosDocumentsKey, @"General",
                    [NSBundle bundleForClass: [self class]], "Human readable")];
        [[_OFTP lastItem] setRepresentedObject: [NSArray arrayWithObject: iTMMacroDocumentType]];
        #warning add defaults here ?
        [_OFTP selectItemAtIndex: 0];
    }
    {
        [_NFTP removeAllItems];
        [_NFTP addItemWithTitle: NSLocalizedStringFromTableInBundle(iTMProjectDocumentsKey, @"General",
                    [NSBundle bundleForClass: [self class]], "Human readable")];
        [[_NFTP lastItem] setRepresentedObject: iTMProjectDocumentType];
        [_NFTP addItemWithTitle: NSLocalizedStringFromTableInBundle(iTMTeXDocumentsKey, @"General",
                    [NSBundle bundleForClass: [self class]], "Human readable")];
        [[_NFTP lastItem] setRepresentedObject: iTMTeXDocumentType];
        [_NFTP addItemWithTitle: NSLocalizedStringFromTableInBundle(iTMMacrosDocumentsKey, @"General",
                    [NSBundle bundleForClass: [self class]], "Human readable")];
        [[_NFTP lastItem] setRepresentedObject: iTMMacroDocumentType];
        [_NFTP selectItemAtIndex: 1];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= shouldOpenTextFile:
- (void) shouldOpenTextFile: (NSNotification *) aNotification;
/*"This is the answer to the notification sent by the "e Helper" tool.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: see the warning below
"*/
{
    NSDictionary * dico = [aNotification userInfo];
    NSString * fileName = [dico objectForKey: @"fileName"];
    NSString * lineNumber = [dico objectForKey: @"lineNumber"];
    if([fileName isKindOfClass: [NSString class]])
    {
        iTMTeXDocument * doc = [self openDocumentWithContentsOfFile: [fileName stringByStandardizingPath] display: YES];
        if([lineNumber respondsToSelector: @selector(intValue)])
        {
            NSEnumerator * enumerator = [[doc windowControllers] objectEnumerator];
            id WC;
            while(WC = [enumerator nextObject])
                if([WC respondsToSelector: @selector(highlightAndScrollLineToVisible:)])
                    [WC highlightAndScrollLineToVisible: [lineNumber intValue]];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMGetter()
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= openFileTypePopUp
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= newFileTypePopUp
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= autoStringEncodingCheck
#define iTMGetter(source, getter) - (id) getter;{return source;}
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
iTMGetter(_OFTP, openFileTypePopUp);
iTMGetter(_OSEP, openStringEncodingPopUp);
iTMGetter(_NFTP, newFileTypePopUp);
iTMGetter(_ASEC, autoStringEncodingCheck);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMStrongSetter()
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setAutoStringEncodingCheck:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setOpenFileTypePopUp:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setNewFileTypePopUp:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setOpenStringEncodingPopUp:
#define iTMStrongSetter(source, className, setter)\
- (void) setter: (id) argument;\
{\
    if(![source isEqual: argument])\
    {\
        if(argument && ![argument isKindOfClass: [className class]])\
            [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] bad argument class: %@.",\
                [self class], NSStringFromSelector(_cmd), argument];\
        else\
        {\
            [source release];\
            source = [argument retain];\
        }\
    }\
    return;\
}
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
iTMStrongSetter(_ASEC, NSButton, setAutoStringEncodingCheck);
iTMStrongSetter(_OFTP, NSPopUpButton, setOpenFileTypePopUp);
iTMStrongSetter(_NFTP, NSPopUpButton, setNewFileTypePopUp);
iTMStrongSetter(_OSEP, NSPopUpButton, setOpenStringEncodingPopUp);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= openFileTypeChosen:
- (void) openFileTypeChosen: (id) sender;
/*"Sent by the file type pull down button. Validates the panel visible columns.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: see the warning below
"*/
{
    id panel = [sender window];
#warning next does not work, ask cocoa-dev, cocoa-dev asked, no answer yet, still no (4+ weeks)
    [panel validateVisibleColumns];
    {
        NSBrowser * browser = [[panel contentView] viewWithTag: NSFileHandlingPanelBrowser];
        [browser reloadColumn: [browser lastColumn]];
    }
    {
        id types = [[sender selectedItem] representedObject];
        BOOL flag = (([types indexOfObject: iTMTeXDocumentType] != NSNotFound) ||
            ([types indexOfObject: iTMTextDocumentType] != NSNotFound));
        [[self autoStringEncodingCheck] setEnabled: flag];
        [[self openStringEncodingPopUp] setEnabled: flag];
    }
//    NSLog(@"%@", [[[self openFileTypePopUp] selectedItem] representedObject]);
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  openStringEncodingChosen:
- (void) openStringEncodingChosen: (id) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
//    NSLog(@"openStringEncodingChosen %d", [[sender selectedItem] tag]);
    [[NSUserDefaults standardUserDefaults] setInteger: [[sender selectedItem] tag] forKey: iTMOpenStringEncodingKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  autoStringEncodingChecked:
- (void) autoStringEncodingChecked: (id) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    [[NSUserDefaults standardUserDefaults] setBool: ([sender state] != NSOffState) forKey: iTMIsAutoStringEncodingKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= panel:shouldShowFilename:
- (BOOL) panel: (id) sender shouldShowFilename: (NSString *) filename;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: deal with aliases and links
"*/
{
//    NSLog(@"fileName: %@\nfileType: %@", filename, [self typeFromFileExtension: [filename pathExtension]]);
    filename = [[filename stringByStandardizingPath] stringByResolvingSymlinksAndFinderAliasesInPath];
    {
        NSString * fileType = [[[NSFileManager defaultManager] fileAttributesAtPath: filename traverseLink: YES] fileType];
        if([[NSWorkspace sharedWorkspace] isFilePackageAtPath: filename] || [fileType isEqualToString: NSFileTypeRegular])
//            return [[filename pathExtension] isEqualToString: @"mTeXMac"] ||
                return [[[[self openFileTypePopUp] selectedItem] representedObject] indexOfObject:
                                [self typeFromFileExtension: [filename pathExtension]]] != NSNotFound;
        else
            return [fileType isEqualToString: NSFileTypeDirectory];
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= newDocument:
- (void) newDocument: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id newDoc = [NSString string];
    [self validateUserInterface];
    if([NSApp runModalForWindow: newDocumentAssistantPanel] == NSOKButton)
    {
    	newDoc = [self openUntitledDocumentOfType: [[[self newFileTypePopUp] selectedItem] representedObject] display: YES];
        [newDoc saveDocument: self];
    }
//    NSLog(@"openUntitledDocumentOfType: %@", [[[self newFileTypePopUp] selectedItem] representedObject]);
//    NSLog(@"newDoc: %@", newDoc);
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= newFileTypeChosen:
- (void) newFileTypeChosen: (id) sender;
/*"Sent by the file type pull down button. Validates the panel visible columns.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: see the warning below
"*/
{
    _DocumentType = [[sender selectedCell] tag];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= newDocumentOK:
- (void) newDocumentOK: (id) sender;
/*"Sent by the file type pull down button. Validates the panel visible columns.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: see the warning below
"*/
{
    [NSApp stopModalWithCode: NSOKButton];
    [[sender window] orderOut: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= newDocumentCancel:
- (void) newDocumentCancel: (id) sender;
/*"Sent by the file type pull down button. Validates the panel visible columns.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: see the warning below
"*/
{
    [NSApp stopModalWithCode: NSCancelButton];
    [[sender window] orderOut: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= openDocument:
- (void) openDocument: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSOpenPanel * OP = [NSOpenPanel openPanel];
    [OP setCanChooseFiles: YES];
    [OP setCanChooseDirectories: NO];
    [OP setTreatsFilePackagesAsDirectories: NO];
    [OP setAllowsMultipleSelection: YES];
    [OP setAccessoryView: openAccessoryView];
    [OP setDelegate: self];
    [OP setResolvesAliases: YES];
    [self validateUserInterface];
    _OptionFlag = ([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask) > 0;
    if(NSOKButton == [self runModalOpenPanel: OP forTypes: [self fileExtensionsFromType: iTMTeXDocumentType]])
    {
        NSArray * filenames = [OP filenames];
        NSEnumerator * enumerator = [filenames objectEnumerator];
        NSString * fileName;
        while(fileName = [enumerator nextObject])
        {
            fileName = [fileName stringByResolvingSymlinksAndFinderAliasesInPath];
            NS_DURING
            [self openDocumentWithContentsOfFile: fileName display: YES];
            NS_HANDLER
            // catch the "NSWindow: -_newFirstResponderAfterResigining is not a valid message outside of a responder's implementation of -resignFirstResponder. exception"
            NSLog(@"Problem opening %@ due to %@", fileName, [localException reason]);
            NS_ENDHANDLER
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= openUntitledDocumentOfType:display:
- (id) openUntitledDocumentOfType: (NSString *) aFileType display: (BOOL) aFlag;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([aFileType isEqualToString: iTMTeXDocumentType])
    {
        unsigned modifierFlags = [[NSApp currentEvent] modifierFlags];
        BOOL option = (modifierFlags & NSAlternateKeyMask) && !(modifierFlags & NSCommandKeyMask);
        BOOL usesTeXHelper = [self usesTeXHelper];
        if(option)
            usesTeXHelper = !usesTeXHelper;
        if(usesTeXHelper)
        {
            NSString * teXHelper = [[NSUserDefaults standardUserDefaults] stringForKey: iTMUDTeXHelperKey];
            if(([teXHelper length] > 0) &&
                            [[NSWorkspace sharedWorkspace] launchApplication: teXHelper])
                return nil;
        }
        else
            return [super openUntitledDocumentOfType: aFileType display: aFlag];
    }
    {
        iTMProjectDocument * newDocument;// just for the compiler
        newDocument = [super openUntitledDocumentOfType: aFileType display: aFlag];
        if(aFlag && [newDocument isKindOfClass: [iTMProjectDocument class]])
        {
            [newDocument reallyMakeWindowControllers];
            [newDocument showWindows];
        }
        return newDocument;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= openUntitledDocumentOfTypeTeXDisplay:
- (id) openUntitledDocumentOfTypeTeXDisplay: (BOOL) aFlag;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [self openUntitledDocumentOfType: iTMTeXDocumentType display: aFlag];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= openDocumentWithContentsOfFile:display;
- (id) openDocumentWithContentsOfFile: (NSString *) fileName display: (BOOL) display;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [self openDocumentWithContentsOfFile: fileName display: display useHelperIfNeeded: YES];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= openDocumentWithContentsOfFile:display:useHelperIfNeeded:
- (id) openDocumentWithContentsOfFile: (NSString *) fileName display: (BOOL) display useHelperIfNeeded: (BOOL) aFlag;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned modifierFlags = [[NSApp currentEvent] modifierFlags];
    BOOL option = (modifierFlags & NSAlternateKeyMask) && !(modifierFlags & NSCommandKeyMask) ?
                            YES: _OptionFlag;
    NSDocument * document = nil;
    NSString * type = [self typeFromFileExtension: [fileName pathExtension]];
    _OptionFlag = NO;// Critical (I do not remember why... hummm)
    if([type isEqualToString: iTMPDFDocumentType])
    {
        BOOL usesPDFHelper = aFlag && (option? ![self usesPDFHelper]: [self usesPDFHelper]);
        if(usesPDFHelper)
        {
            NSString * pdfHelper = [[NSUserDefaults standardUserDefaults] stringForKey: iTMUDPDFHelperKey];
            if(!(([pdfHelper length] > 0) &&
                            [[NSWorkspace sharedWorkspace] openFile: fileName withApplication: pdfHelper]))
                document = [super openDocumentWithContentsOfFile: fileName display: display];
            else
                // Critical: iTMGhostDocument is usefull when an external pdf viewer is used. see addDocument below.
                document = [self ghostDocument];
        }
        else
            document = [super openDocumentWithContentsOfFile: fileName display: display];
    }
    else if([type isEqualToString: iTMTeXDocumentType])
    {
        BOOL usesTeXHelper = aFlag && (option? ![self usesTeXHelper]: [self usesTeXHelper]);
        if(usesTeXHelper)
        {
            NSString * texHelper = [[NSUserDefaults standardUserDefaults] stringForKey: iTMUDTeXHelperKey];
            if(!(([texHelper length] > 0) &&
                            [[NSWorkspace sharedWorkspace] openFile: fileName withApplication: texHelper andDeactivate: YES]))
                document = [super openDocumentWithContentsOfFile: fileName display: display];
            else
                // critical: iTMGhostDocument is usefull when an external text editor is used. see addDocument below.
                document = [self ghostDocument];
        }
        else
            document = [super openDocumentWithContentsOfFile: fileName display: display];
    }
    else
    {
        document = [super openDocumentWithContentsOfFile: fileName display: display];
    }
    if([document isKindOfClass: [iTMProjectDocument class]])
    {
        if(![(iTMProjectDocument *)document isStrongGeneric] && display)
            [(iTMProjectDocument *)document showProject: self];
    }
//    [[iTMProjectsManager defaultManager] setCurrentProject:
//            [[iTMProjectsManager defaultManager] projectForDocument: document]];
    return document;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= addDocument:
- (void) addDocument: (NSDocument *) document;
/*"Description Forthcoming. Ghost documents are not added.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[iTMProjectsManager defaultManager] addDocument: document];
    [super addDocument: document];
    if([document isKindOfClass: [iTMGhostDocument class]] || [document isKindOfClass: [iTMRenderDocument class]])
        [self removeDocument: document];
    else if([document isKindOfClass: [iTMTeXDocument class]])
        ++_NumberOfTeXDocuments;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= removeDocument:
- (void) removeDocument: (NSDocument *) document;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[iTMProjectsManager defaultManager] removeDocument: document];
    [super removeDocument: document];
    if([document isKindOfClass: [iTMTeXDocument class]])
        --_NumberOfTeXDocuments;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= hasTeXDocuments
- (BOOL) hasTeXDocuments;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _NumberOfTeXDocuments>0;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= noteNewRecentDocument:
- (void) noteNewRecentDocument: (NSDocument *) document;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([document isKindOfClass: [iTMGhostDocument class]])
        return;
    if([document isKindOfClass: [iTMProjectDocument class]] && [(iTMProjectDocument *)document isStrongGeneric])
        return;
    [super noteNewRecentDocument: document];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMenuItem:
- (BOOL) validateMenuItem: (id) anItem;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    SEL action = [anItem action];
    if(action == @selector(openFileTypeChosen:))
        return YES;
    else if(action == @selector(newFileTypeChosen:))
        return YES;
    else if(action == @selector(openStringEncodingChosen:))
        return YES;
    else if(action == @selector(builtInPDFViewer:))
    {
        [anItem setState: ([self usesPDFHelper]? NSOffState: NSOnState)];
        return YES;
    }
    else if(action == @selector(preferredPDFViewer:))
    {
        NSString * title = [[[[NSUserDefaults standardUserDefaults] stringForKey: iTMUDPDFHelperKey]
                                    lastPathComponent] stringByDeletingPathExtension];
        [anItem setTitle: ([title length]? title: NSLocalizedStringFromTableInBundle(@"None", @"General",
                    [NSBundle bundleForClass: [self class]], "None"))];
        [anItem setState: ([self usesPDFHelper]? NSOnState: NSOffState)];
        return [title length] > 0;
    }
    else if(action == @selector(choosePreferredPDFViewer:))
        return YES;
    else if(action == @selector(builtInTeXEditor:))
    {
        [anItem setState: ([self usesTeXHelper]? NSOffState: NSOnState)];
        return YES;
    }
    else if(action == @selector(preferredTeXEditor:))
    {
        NSString * title = [[[[NSUserDefaults standardUserDefaults] stringForKey: iTMUDTeXHelperKey]
                                    lastPathComponent] stringByDeletingPathExtension];
        [anItem setTitle: ([title length]? title: NSLocalizedStringFromTableInBundle(@"None", @"General",
                    [NSBundle bundleForClass: [self class]], "None"))];
        [anItem setState: ([self usesTeXHelper]? NSOnState: NSOffState)];
        return [title length] > 0;
    }
    else if(action == @selector(choosePreferredTeXEditor:))
        return YES;
    else
        return [super validateMenuItem: anItem];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  closeAllDocumentsWithDelegate:didCloseAllSelector:contextInfo:
- (void) closeAllDocumentsWithDelegate: (id) delegate didCloseAllSelector: (SEL) didAllCloseSelector
contextInfo: (void *) contextInfo;
/*"Call back must have the following signature:
- (void) documentController: (id) DC didSaveAll: (BOOL) flag contextInfo: (void *) contextInfo;
Version History: jerome.laurens@u-bourgogne.fr (12/07/2001)
- < 1.1: 03/10/2002
To Do List: to be improved...
"*/
{
    [[NSNotificationCenter defaultCenter] postNotificationName: iTMDocumentControllerWillCloseAllNotification
        object: self];
    [super closeAllDocumentsWithDelegate: self
        didCloseAllSelector: @selector(_iTMPrivateDocumentController:didCloseAll:contextInfo:)
            contextInfo: [NSDictionary dictionaryWithObjectsAndKeys:
                                delegate, @"delegate",
                                NSStringFromSelector(didAllCloseSelector), @"selector",
                                (contextInfo? contextInfo: [NSNull null]), @"contextInfo",
                                                        nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  _iTMPrivateDocumentController:didCloseAll:contextInfo:
- (void) _iTMPrivateDocumentController: (NSDocumentController *) docController didCloseAll:(BOOL) didCloseAll
contextInfo: (void *) contextInfo;
/*"Call back must have the following signature:
- (void) documentController: (if) DC didSaveAll: (BOOL) flag contextInfo: (void *) contextInfo;
Version History: jerome.laurens@u-bourgogne.fr (01/02/2002)
- < 1.1: 03/10/2002
To Do List: to be improved..., post the notification in the end?
"*/
{
    if(didCloseAll)
        [[NSNotificationCenter defaultCenter] postNotificationName: iTMDocumentControllerDidCloseAllNotification
            object: self];
    {
        id delegate = [(id)contextInfo objectForKey: @"delegate"];
        SEL action = NSSelectorFromString([(id)contextInfo objectForKey: @"selector"]);
        NSMethodSignature * itsMS = [delegate methodSignatureForSelector: action];
        id contInfo = [(id)contextInfo objectForKey: @"contextInfo"];
        if(itsMS)
        {
            NSInvocation * I = [NSInvocation invocationWithMethodSignature: itsMS];
            [I setSelector: action];
            [I setTarget: delegate];
            if([itsMS numberOfArguments]>2)
                [I setArgument: &self atIndex: 2];
            if([itsMS numberOfArguments]>3)
                [I setArgument: &didCloseAll atIndex: 3];
            if([itsMS numberOfArguments]>4)
                if([[NSNull null] isEqual: contInfo])
                    [I setArgument: nil atIndex: 4];
                else
                    [I setArgument: &contInfo atIndex: 4];
            [I invoke];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= fakeDocumentController:didSaveAll:contextInfo:
- (void) fakeDocumentController: (id) DC didSaveAll: (BOOL) flag contextInfo: (void *) contextInfo;
/*"Call back must have the following signature:
- (void) documentController: (if) DC didSaveAll: (BOOL) flag contextInfo: (void *) contextInfo;
Version History: jerome.laurens@u-bourgogne.fr (12/07/2001)
- < 1.1: 03/10/2002
To Do List: to be improved...
"*/
{
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= saveAllDocumentsWithDelegate:didSaveSelector:contextInfo:
- (void) saveAllDocumentsWithDelegate: (id) delegate didSaveAllSelector: (SEL) action contextInfo: (void *) contextInfo;
/*"Call back must have the following signature:
- (void) documentController: (if) DC didSaveAll: (BOOL) flag contextInfo: (void *) contextInfo;
Version History: jerome.laurens@u-bourgogne.fr (12/07/2001)
- < 1.1: 03/10/2002
To Do List: to be improved...
"*/
{
    BOOL resultFlag;
    [self saveAllDocuments: self];
    resultFlag = YES;
    {
        NSMethodSignature * myMS = [self methodSignatureForSelector:
            @selector(fakeDocumentController:didSaveAll:contextInfo:)];
        NSMethodSignature * itsMS = [delegate methodSignatureForSelector: action];
        if([myMS isEqual: itsMS])
        {
            NSInvocation * I = [NSInvocation invocationWithMethodSignature: myMS];
            [I setSelector: action];
            [I setTarget: delegate];
            [I setArgument: &self atIndex: 2];
            [I setArgument: &resultFlag atIndex: 3];
            [I setArgument: &contextInfo atIndex: 4];
            [I invoke];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= ghostDocument
- (id) ghostDocument;
/*"Description forthcoming
Version History: jerome.laurens@u-bourgogne.fr (12/07/2001)
- < 1.1: 03/10/2002
To Do List: to be improved...
"*/
{
    return [[[iTMGhostDocument alloc] init] autorelease];
}
@end

@implementation iTMDocumentController(Helpers)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  builtInPDFViewer:
- (IBAction) builtInPDFViewer: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSUserDefaults standardUserDefaults] setObject: [NSNumber numberWithBool: NO] forKey: iTMUDUsesPDFHelperKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  preferredPDFViewer:
- (IBAction) preferredPDFViewer: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSUserDefaults standardUserDefaults] setObject: [NSNumber numberWithBool: YES] forKey: iTMUDUsesPDFHelperKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  choosePreferredPDFViewer:
- (IBAction) choosePreferredPDFViewer: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSArray * ApplicationDirectories = NSSearchPathForDirectoriesInDomains(NSApplicationDirectory, NSLocalDomainMask, YES);
    if([ApplicationDirectories count])
    {
        NSOpenPanel * OP = [NSOpenPanel openPanel];
        [OP setAllowsMultipleSelection: NO];
        [OP setCanChooseFiles: NO];
        [OP setCanChooseDirectories: YES];
        [OP setResolvesAliases: YES];
        [OP setTreatsFilePackagesAsDirectories: NO];
        if(NSOKButton == [OP runModalForDirectory: [ApplicationDirectories objectAtIndex: 0]
                                    file: nil types: [NSArray arrayWithObject: @"app"]] && [[OP filenames] count])
        {
            [[NSUserDefaults standardUserDefaults] setObject: [[OP filenames] objectAtIndex: 0] forKey: iTMUDPDFHelperKey];
            [[NSUserDefaults standardUserDefaults]
            setObject: [NSNumber numberWithBool: YES] forKey: iTMUDUsesPDFHelperKey];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  usesPDFHelper
- (BOOL) usesPDFHelper;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[NSUserDefaults standardUserDefaults] boolForKey: iTMUDUsesPDFHelperKey];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  builtInTeXEditor:
- (IBAction) builtInTeXEditor: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSUserDefaults standardUserDefaults] setObject: [NSNumber numberWithBool: NO] forKey: iTMUDUsesTeXHelperKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  preferredTeXEditor:
- (IBAction) preferredTeXEditor: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSUserDefaults standardUserDefaults] setObject: [NSNumber numberWithBool: YES] forKey: iTMUDUsesTeXHelperKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  choosePreferredTeXEditor:
- (IBAction) choosePreferredTeXEditor: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSArray * ApplicationDirectories = NSSearchPathForDirectoriesInDomains(NSApplicationDirectory, NSLocalDomainMask, YES);
    if([ApplicationDirectories count])
    {
        NSOpenPanel * OP = [NSOpenPanel openPanel];
        [OP setAllowsMultipleSelection: NO];
        [OP setCanChooseFiles: NO];
        [OP setCanChooseDirectories: YES];
        [OP setResolvesAliases: YES];
        [OP setTreatsFilePackagesAsDirectories: NO];
        if(NSOKButton == [OP runModalForDirectory: [ApplicationDirectories objectAtIndex: 0]
                                    file: nil types: [NSArray arrayWithObject: @"app"]] && [[OP filenames] count])
        {
            [[NSUserDefaults standardUserDefaults] setObject: [[OP filenames] objectAtIndex: 0] forKey: iTMUDTeXHelperKey];
            [[NSUserDefaults standardUserDefaults]
            setObject: [NSNumber numberWithBool: YES] forKey: iTMUDUsesTeXHelperKey];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  usesTeXHelper
- (BOOL) usesTeXHelper;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[NSUserDefaults standardUserDefaults] boolForKey: iTMUDUsesTeXHelperKey];
}
@end

@implementation iTMGhostDocument
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMDocumentController

