/*
//  iTMLaTeXToolsController.m
//  iTeXMac
//
//  Created by lenglin on Sun Aug 26 2001 under the name Autrecontroller.m
//  Copyright part (c) 2001 __NorignalSoft__. All rights reserved.
//
//  Improved by jerome.laurens@u-bourgogne.fr on Tue Dec 18 2001.
//  Copyright part  2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import "iTMLaTeXToolsController.h"

NSString * const iTMLaTeXToolsPanelName = @"iTMLaTeXTools";
NSString * const iTMUDLaTeXToolsPanelOriginKey = @"iTMLaTeXToolsPanelOrigin";
NSString * const iTMUDLaTeXToolsPanelOutlinesKey = @"iTMLaTeXToolsPanelOutlines";
NSString * const iTMUDUseLaTeXToolsPanelKey = @"iTMShowLaTeXToolsPanel";

@interface iTMLaTeXToolsController(iTMPrivate)
- (NSTextView *) textViewTarget;
- (void) hideWindow:(id)sender;
@end

@implementation iTMLaTeXToolsController
static id _sharedInstance = nil;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Defaults defaults settings.
Version History: jerome.laurens@u-bourgogne.fr (02/01/2002).
To do list:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
            NSStringFromPoint(NSMakePoint(0, 0)), iTMUDLaTeXToolsPanelOriginKey,
            [NSNumber numberWithBool: YES], iTMUDLaTeXToolsPanelOutlinesKey,
            [NSNumber numberWithBool: NO], iTMUDUseLaTeXToolsPanelKey,
                nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sharedInstance;
+ (id) sharedInstance;
/*"Lazy initialier.
Version History: jerome.laurens@u-bourgogne.fr (02/01/2002).
To do list:
"*/
{
    return _sharedInstance? _sharedInstance:
        (_sharedInstance = [[iTMLaTeXToolsController alloc] initWithWindowNibName: iTMLaTeXToolsPanelName]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initWithWindow:
- (id) initWithWindow: (NSWindow *) window;
/*"One shot initializer. Manages the shared instance and initialize all the stuff.
Version History: jerome.laurens@u-bourgogne.fr (02/01/2002).
To do list:
"*/
{
    if(_sharedInstance)
    {
        if(![self isEqual: _sharedInstance])
            [super dealloc];
        return [_sharedInstance retain];
    }
    else
    {
        _sharedInstance = self;// avoid recursive call
        if(self = [self initWithWindowNibName: iTMLaTeXToolsPanelName])
        {
            [self setShouldAskPermissionToShow: YES];
            {
                NSDictionary * D = [NSDictionary dictionaryWithContentsOfFile:
                    [[NSBundle bundleForClass: [self class]] pathForResource: iTMLaTeXToolsPanelName ofType: @"plist"]];
                arrayMath = [[D objectForKey: @"Math"] retain];
                arraySymbols = [[D objectForKey: @"Symbols"] retain];
            }
        }
        [[NSNotificationCenter defaultCenter]
            addObserver: self
                selector: @selector(windowDidBecomeMain:)
                    name: NSWindowDidBecomeMainNotification
                        object: nil];
        return _sharedInstance = self;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initWithCoder:
- (id) initWithCoder: (NSCoder *) decoder;
/*"Forwards #{initWithWindow}.
Version History: jerome.laurens@u-bourgogne.fr (02/01/2002).
To do list:
"*/
{
//    NSLog(@"initWithCoder");
    return [self initWithWindow: nil];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowDidLoad
- (void) windowDidLoad;
/*"Inialize the window content and registers for notifications.
Version History: Originally created by lenglin
big improvements by jerome.laurens@u-bourgogne.fr (02/01/2002).
To do list:
"*/
{
    [super windowDidLoad];

    [[self window] setFrameOrigin:
            NSPointFromString([[NSUserDefaults standardUserDefaults] stringForKey: iTMUDLaTeXToolsPanelOriginKey])];
    [[self window] setHidesOnDeactivate: YES];
    [[self window] setBecomesKeyOnlyIfNeeded: YES];

    [[NSNotificationCenter defaultCenter]
        addObserver: self
            selector: @selector(windowDidMove:)
                name: NSWindowDidMoveNotification
                    object: [self window]];
    [[NSNotificationCenter defaultCenter]
        addObserver: self
            selector: @selector(windowWillClose:)
                name: NSWindowWillCloseNotification
                    object: [self window]];

    if ([[NSUserDefaults standardUserDefaults] boolForKey: iTMUDLaTeXToolsPanelOutlinesKey])
    {
        int i;
        #define FIXSHOWSBORDER(matrix, min, max)\
        for (i=min;i<max;i++) [[matrix cellWithTag: i] setShowsBorderOnlyWhileMouseInside: YES];
        FIXSHOWSBORDER(mathMatrix, 0, 54);
        FIXSHOWSBORDER(symbolMatrix, 0, 55);
    }
    return;
// this adjusts the look of buttons at startup
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setShouldAskPermissionToShow:
- (void) setShouldAskPermissionToShow: (BOOL) flag;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (02/01/2002).
To do list:
"*/
{
    shouldAskPermissionToShow = flag;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textViewTarget
- (NSTextView *) textViewTarget;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (02/01/2002).
To do list:
"*/
{
    NSTextView * TV = [[NSApp mainWindow] firstResponder];
    return [TV isKindOfClass: [NSTextView class]]? TV: nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  insertMath:
- (IBAction) insertMath: (id) sender;
/*"Description Forthcoming.
Version History: lenglin
To do list:
"*/
{
    [[self textViewTarget] insertText: [arrayMath objectAtIndex: [[sender selectedCell] tag]]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  insertSymbol:
- (IBAction) insertSymbol: (id) sender;
/*"Description Forthcoming.
Version History: lenglin
To do list:
"*/
{
    [[self textViewTarget] insertText: [arraySymbols objectAtIndex: [[sender selectedCell] tag]]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  showWindow
- (IBAction) showWindow: (id) sender;
/*"Fix shouldShowPanel and forwards to super.
Version History: lenglin
To do list:
"*/
{
    [[NSUserDefaults standardUserDefaults] setObject: [NSNumber numberWithBool: YES] forKey: iTMUDUseLaTeXToolsPanelKey];
    [super showWindow: sender];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  hideWindow
- (void) hideWindow: (id) sender;
/*"Fix shouldShowPanel and closes the window.
Version History: lenglin
To do list:
"*/
{
    [[NSUserDefaults standardUserDefaults] setObject: [NSNumber numberWithBool: NO] forKey: iTMUDUseLaTeXToolsPanelKey];
    [[self window] close];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMenuItem:
- (BOOL) validateMenuItem: (id <NSMenuItem>) sender;
/*"Sets the state of the sender.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing at first glance.
"*/
{
    SEL action = [sender action];
    if(action == @selector(toggleShowLaTeXTools:))
    {
        [sender setState: ([[NSUserDefaults standardUserDefaults] boolForKey: iTMUDUseLaTeXToolsPanelKey]?
                                    NSOnState: NSOffState)];
        return YES;
    }
    else
        return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleShowLaTeXTools:
- (void) toggleShowLaTeXTools: (id) sender;
/*"Sets the state of the sender.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing at first glance.
"*/
{
    BOOL old = [[NSUserDefaults standardUserDefaults] boolForKey: iTMUDUseLaTeXToolsPanelKey];
//    [[NSUserDefaults standardUserDefaults] setObject: [NSNumber numberWithBool: !old] forKey: iTMUDUseLaTeXToolsPanelKey];
    if(!old && (!shouldAskPermissionToShow || [[NSApp mainWindow] allowsLaTeXToolsPanel]))
        [self showWindow: self];
    else
        [self hideWindow: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowDidBecomeMain:
- (void) windowDidBecomeMain: (NSNotification *) note;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (02/01/2002).
To do list:
"*/
{
    
    if ([[NSUserDefaults standardUserDefaults] boolForKey: iTMUDUseLaTeXToolsPanelKey] &&
        (!shouldAskPermissionToShow || [[note object] allowsLaTeXToolsPanel]))
        [self showWindow: self];
    else
        [[self window] orderOut: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowDidMove:
- (void) windowDidMove: (NSNotification *) notification;
/*"Cocoa bug fixed (frame saved...).
Version History: jerome.laurens@u-bourgogne.fr (02/01/2002).
To do list:
"*/
{
    [[NSUserDefaults standardUserDefaults]
        setObject: NSStringFromPoint([[self window] frame].origin)
            forKey: iTMUDLaTeXToolsPanelOriginKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowWillClose:
- (void) windowWillClose: (NSNotification *) notification;
/*"Cocoa bug fixed (frame saved...).
Version History: jerome.laurens@u-bourgogne.fr (02/01/2002).
To do list:
"*/
{
    [[NSUserDefaults standardUserDefaults]
        setObject: NSStringFromPoint([[self window] frame].origin)
            forKey: iTMUDLaTeXToolsPanelOriginKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (02/01/2002).
To do list:
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [arrayMath release];
    [arraySymbols release];
    [super dealloc];
    return;
}
@end

@implementation NSWindow(iTMLaTeXToolsController)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  allowsLaTeXToolsPanel
- (BOOL) allowsLaTeXToolsPanel;
/*"Never accepts the panel. Subclasses will return YES if needed.
Version History: jerome.laurens@u-bourgogne.fr (18/02/2002).
To do list:
"*/
{
    return NO;
}
@end

