/*
//  iTMPDFToolbarDelegate.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sun Jan 06 2002.
//  Copyright © 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import "iTMPDFToolbarDelegate.h"
#import "iTMToolbarItem.h"
#import "iTMNavigationFormatter.h"
#import "iTMNavigationButton.h"
#import "iTMHistoryButton.h"
#import "iTMHelpManager.h"
#import "iTMFlagsChangedView.h"

NSString * const iTMToolbarNavigationFieldItemIdentifier = @"AtNavigation";
NSString * const iTMToolbarNavigationSetItemIdentifier = @"Navigation";
NSString * const iTMToolbarPreviousSetItemIdentifier = @"Previous";
NSString * const iTMToolbarNextSetItemIdentifier = @"Next";
NSString * const iTMToolbarPreviousMixedSetItemIdentifier = @"Previous Mixed";
NSString * const iTMToolbarNextMixedSetItemIdentifier = @"Next Mixed";
NSString * const iTMToolbarFirstButtonItemIdentifier = @"displayFirstPage:";
NSString * const iTMToolbarPreviousButtonItemIdentifier = @"displayPreviousPage:";
NSString * const iTMToolbarPreviousPreviousButtonItemIdentifier = @"displayPreviousPreviousPage:";
NSString * const iTMToolbarLastButtonItemIdentifier = @"displayLastPage:";
NSString * const iTMToolbarNextButtonItemIdentifier = @"displayNextPage:";
NSString * const iTMToolbarNextNextButtonItemIdentifier = @"displayNextNextPage:";
NSString * const iTMToolbarBackButtonItemIdentifier = @"displayBackPage:";
NSString * const iTMToolbarForwardButtonItemIdentifier = @"displayForwardPage:";
NSString * const iTMToolbarHistorySetItemIdentifier = @"History";

@interface NSObject(iTMPDFToolbarDelegate)
- (int) currentPage;
- (NSView *) album;
@end

@implementation iTMPDFToolbarDelegate
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-  init
- (id) init;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(self = [super init])
    {
        [self->__LastNavigationITII release];
        self->__LastNavigationITII = nil;
        [self setNavigationField: nil];
        [[NSNotificationCenter defaultCenter] removeObserver: self
                    name: @"iTMPageNumberDidChange"
                                object: nil];
        [[NSNotificationCenter defaultCenter] addObserver: self
            selector: @selector(pageNumberDidChange:)
                name: @"iTMPageNumberDidChange"
                    object: nil];
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-  dealloc
- (void) dealloc;
/*"Description forthcoming."*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [__LastNavigationITII release];
    __LastNavigationITII = nil;
    [self setNavigationField: nil];
    [self setBackMenu: nil];
    [self setForwardMenu: nil];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  pageNumberDidChange
- (void) pageNumberDidChange: (NSNotification *) aNotification;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if([[[aNotification object] window] isEqual: [[self navigationField] window]])
    {
        NSNumber * oldPage = [[aNotification userInfo] objectForKey: @"OldPage"];
        NSNumber * newPage = [[aNotification userInfo] objectForKey: @"NewPage"];
        NSNumber * pageCount = [[aNotification userInfo] objectForKey: @"PageCount"];
        [[self navigationField] setIntValue: [newPage intValue]+1];
        [[[self navigationField] formatter] setMaximum: [NSDecimalNumber numberWithInt: [pageCount intValue]]];
        [[self navigationField] setEnabled: ([[[[self navigationField] formatter] maximum] intValue] > 1)];
        {
            int OP = [oldPage intValue];
            int NP = [newPage intValue];
            NP = MIN(NP, [pageCount intValue]);
            if(oldPage && (OP>=0) && (OP != NP))
            {
                int OPIndex;
                NSMenu * backMenu = [self backMenu];
                NSMenu * forwardMenu = [self forwardMenu];
                [backMenu insertItemWithTitle: [[NSNumber numberWithInt: OP+1] stringValue]
                        action: @selector(displayBackPageFromItem:) keyEquivalent: @"" atIndex: 0];
                [[backMenu itemAtIndex: 0] setTarget: nil];
                [[backMenu itemAtIndex: 0] setTag: OP];
                if((OPIndex = [forwardMenu indexOfItemWithTag: NP]) > -1)
                {
                // jump to the correct item in the forward menu
                    while(OPIndex>0)
                    {
                        NSMenuItem * menuItem = [[forwardMenu itemAtIndex: 0] retain];
                        [forwardMenu removeItemAtIndex: 0];
                        // Do not insert successive menu items with the same tag
                        if([[backMenu itemAtIndex: 0] tag]!=[menuItem tag])
                        {
                            [backMenu insertItem: menuItem atIndex: 0];
                            [[backMenu itemAtIndex: 0] setAction: @selector(displayBackPageFromItem:)];
                        }
                        [menuItem release];
                        --OPIndex;
                    }
                    [forwardMenu removeItemAtIndex: 0];
                }
                else if((OPIndex = [backMenu indexOfItemWithTag: NP]) > -1)
                {
        // jump to the correct item in the back menu
                    while(OPIndex>0)
                    {
                        NSMenuItem * menuItem = [[backMenu itemAtIndex: 0] retain];
                        [backMenu removeItemAtIndex: 0];
                        // Do not insert successive menu items with the same tag
                        if([forwardMenu numberOfItems] == 0 ||
                                    [[forwardMenu itemAtIndex: 0] tag]!=[menuItem tag])
                        {
                            [forwardMenu insertItem: menuItem atIndex: 0];
                            [[forwardMenu itemAtIndex: 0] setAction: @selector(displayForwardPageFromItem:)];
                        }
                        [menuItem release];
                        --OPIndex;
                    }
                    [backMenu removeItemAtIndex: 0];
                }
                else
                // remove all forward items
                    while([forwardMenu numberOfItems]>0)
                        [forwardMenu removeItemAtIndex: 0];
            }
        }
        {
            int MP = [pageCount intValue];
            int index = [[self backMenu] numberOfItems];
            while(index-- > 0)
                if([[[self backMenu] itemAtIndex: index] tag] > MP)
                    [[self backMenu] removeItemAtIndex: index];
            index = [[self forwardMenu] numberOfItems];
            while(index-- > 0)
                if([[[self forwardMenu] itemAtIndex: index] tag] > MP)
                    [[self forwardMenu] removeItemAtIndex: index];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- toolbarDefaultItemIdentifiers:
- (NSArray *) toolbarDefaultItemIdentifiers: (NSToolbar*) aToolbar;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return [NSArray arrayWithObjects:
                NSToolbarPrintItemIdentifier,
                NSToolbarSeparatorItemIdentifier,
                NSToolbarFlexibleSpaceItemIdentifier,
                iTMToolbarNavigationSetItemIdentifier,
                NSToolbarFlexibleSpaceItemIdentifier,
                iTMToolbarMagnificationSetItemIdentifier,
                NSToolbarFlexibleSpaceItemIdentifier,
                iTMToolbarHistorySetItemIdentifier,
                NSToolbarFlexibleSpaceItemIdentifier,
                NSToolbarSeparatorItemIdentifier,
                iTMToolbarShowHelpItemIdentifier,
                    nil];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- toolbarAllowedItemIdentifiers:
- (NSArray *) toolbarAllowedItemIdentifiers: (NSToolbar*) aToolbar;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return [NSArray arrayWithObjects:
                NSToolbarPrintItemIdentifier,
                NSToolbarFlexibleSpaceItemIdentifier,
                NSToolbarSpaceItemIdentifier,
                NSToolbarSeparatorItemIdentifier,
                iTMToolbarShowHelpItemIdentifier,
                iTMToolbarPreviousSetItemIdentifier,
                iTMToolbarNextSetItemIdentifier,
                iTMToolbarPreviousMixedSetItemIdentifier,
                iTMToolbarNextMixedSetItemIdentifier,
                iTMToolbarMagnificationFieldItemIdentifier,
                iTMToolbarMagnificationSetItemIdentifier,
                iTMToolbarNavigationFieldItemIdentifier,
                iTMToolbarNavigationSetItemIdentifier,
                iTMToolbarBackButtonItemIdentifier,
                iTMToolbarForwardButtonItemIdentifier,
                iTMToolbarHistorySetItemIdentifier,
                iTMToolbarFirstButtonItemIdentifier,
                iTMToolbarLastButtonItemIdentifier,
                iTMToolbarPreviousButtonItemIdentifier,
                iTMToolbarNextButtonItemIdentifier,
                iTMToolbarPreviousPreviousButtonItemIdentifier,
                iTMToolbarNextNextButtonItemIdentifier,
                    nil];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- toolbar:itemForItemIdentifier:willBeInsertedIntoToolbar:
- (NSToolbarItem *) toolbar: (NSToolbar *) aToolbar itemForItemIdentifier: (NSString *) anItemIdentifier willBeInsertedIntoToolbar: (BOOL) aFlag;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    iTMToolbarItem * result;
    BOOL hasField = NO;
#define IFTHENFIELD(iTMItemIdentifier, forToolbar, forPalette)\
    if([anItemIdentifier isEqualToString: iTMItemIdentifier])\
    {\
        NSView * view;\
        if(aFlag)\
        {\
            view = [self forToolbar];\
            result = [[iTMToolbarItem alloc] initWithItemIdentifier: iTMItemIdentifier];\
        }\
        else\
        {\
            view = [self forPalette];\
            result = [[NSToolbarItem alloc] initWithItemIdentifier: iTMItemIdentifier];\
        }\
        [result setView: view];\
        hasField = YES;\
    }
    IFTHENFIELD(iTMToolbarNavigationFieldItemIdentifier, navigationFieldForToolbar, navigationFieldForPalette)
    else IFTHENFIELD(iTMToolbarNavigationSetItemIdentifier, navigationSetForToolbar, navigationSetForPalette)
    #define IFTHEN(iTMItemIdentifier, set)\
    if([anItemIdentifier isEqualToString: iTMItemIdentifier])\
    {\
        NSView * view = [self set];\
        result = aFlag? [iTMToolbarItem alloc]: [NSToolbarItem alloc];\
        result = [result initWithItemIdentifier: iTMItemIdentifier];\
        [result setView: view];\
    }
    else IFTHEN(iTMToolbarPreviousSetItemIdentifier, previousSet)
    else IFTHEN(iTMToolbarNextSetItemIdentifier, nextSet)
    else IFTHEN(iTMToolbarPreviousMixedSetItemIdentifier, previousMixedSet)
    else IFTHEN(iTMToolbarNextMixedSetItemIdentifier, nextMixedSet)
    else IFTHEN(iTMToolbarHistorySetItemIdentifier, historySet)
    else IFTHEN(iTMToolbarFirstButtonItemIdentifier, firstButton)
    else IFTHEN(iTMToolbarPreviousButtonItemIdentifier, previousButton)
    else IFTHEN(iTMToolbarPreviousPreviousButtonItemIdentifier, previousPreviousButton)
    else IFTHEN(iTMToolbarLastButtonItemIdentifier, lastButton)
    else IFTHEN(iTMToolbarNextButtonItemIdentifier, nextButton)
    else IFTHEN(iTMToolbarNextNextButtonItemIdentifier, nextNextButton)
    else IFTHEN(iTMToolbarBackButtonItemIdentifier, backButton)
    else IFTHEN(iTMToolbarForwardButtonItemIdentifier, forwardButton)
    #undef IFTHEN
    #define IFTHEN(iTMItemIdentifier, image)\
    if([anItemIdentifier isEqualToString: iTMItemIdentifier])\
    {\
        result = [[NSToolbarItem alloc] initWithItemIdentifier: anItemIdentifier];\
        [result setImage: [NSImage imageNamed: image]];\
        [result setAction: NSSelectorFromString(anItemIdentifier)];\
        {\
            NSString * identifier = [result itemIdentifier];\
            [result setToolTip: NSLocalizedStringFromTableInBundle(identifier, @"ViewToolbarToolTip",\
                [NSBundle bundleForClass: [self class]], "")];\
            [result setPaletteLabel: NSLocalizedStringFromTableInBundle(identifier, @"ViewToolbarPaletteLabel",\
                [NSBundle bundleForClass: [self class]], "")];\
            [result setLabel: NSLocalizedStringFromTableInBundle(identifier, @"ViewToolbarLabel",\
                [NSBundle bundleForClass: [self class]], "")];\
        }\
        return [result autorelease];\
    }
    else if([anItemIdentifier isEqualToString: iTMToolbarShowHelpItemIdentifier])
        return [[iTMHelpManager sharedManager] helpToolbarItem];
    else
        return [super toolbar: aToolbar itemForItemIdentifier: anItemIdentifier willBeInsertedIntoToolbar: aFlag];
    [result setMinSize: [[result view] frame].size];
    [result setMaxSize: [[result view] frame].size];
    {
        NSString * identifier = [result itemIdentifier];
        [result setToolTip: NSLocalizedStringFromTableInBundle(identifier, @"BasicToolbarToolTip",
                                                        [NSBundle bundleForClass: [self class]], "")];
        [result setPaletteLabel: NSLocalizedStringFromTableInBundle(identifier, @"BasicToolbarPaletteLabel",
                                                        [NSBundle bundleForClass: [self class]], "")];
        [result setLabel: NSLocalizedStringFromTableInBundle(identifier, @"BasicToolbarLabel",
                                                        [NSBundle bundleForClass: [self class]], "")];
    }
    if(aFlag && hasField)
    {
        if((__LastNavigationITII!=nil)&&![__LastNavigationITII isEqualToString: anItemIdentifier])
        {
            BOOL removeFlag=NO;
            int index;
            for(index=0; index<[[aToolbar items] count]; ++index)
                if([[[[aToolbar items] objectAtIndex: index] itemIdentifier] isEqualToString: __LastNavigationITII])
                {
                    removeFlag=YES;
                    break;
                }
            if(removeFlag)
                [aToolbar removeItemAtIndex: index];
        }
        [__LastNavigationITII release];
        __LastNavigationITII = [anItemIdentifier copy];
    }
    return [result autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= historySet
- (NSView *) historySet;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    NSView * B = [self backButton];
    NSView * F = [self forwardButton];
    NSRect frame = NSMakeRect(0,0, [B frame].size.width+[F frame].size.width, 32);
    NSView * result = [[NSView alloc] initWithFrame: frame];
    [result addSubview: B];
    [F setFrameOrigin: NSMakePoint([B frame].origin.x+[B frame].size.width, 0)];
    [result addSubview: F];
    return [result autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  navigationFieldForPalette
- (NSTextField *) navigationFieldForPalette;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    NSTextField * textField = [[NSTextField alloc] initWithFrame: [[self navigationField] frame]];
    [textField setStringValue: @"p..."];
    return [textField autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  navigationFieldForToolbar
- (NSTextField *) navigationFieldForToolbar;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    NSTextField * textField = [[self navigationField] retain];
    [self setNavigationField: nil];
    [[self navigationField] setIntValue: [textField intValue]];
    [textField release];
    textField = [[self navigationField] retain];
    if ([textField retainCount]<3)
    {
        return [textField autorelease];
    }
    else
    {
        NSLog(@"No item navigationView, retainCount: %d", [textField retainCount]);
        [textField release];
        return nil;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  NAVIGATION_SET
#define NAVIGATION_SET(navigationSet, navigationField)\
- (NSView *) navigationSet;\
{\
    NSView * F = [self navigationField];\
    if(F != nil)\
    {\
        NSView * P = [self previousMixedSet];\
        NSView * N = [self nextMixedSet];\
        NSView * result = [[NSView alloc] initWithFrame: [P frame]];\
        [result addSubview: P];\
        [F setFrameOrigin: NSMakePoint([result frame].size.width+4, [F frame].origin.y)];\
        [result addSubview: F];\
        [result setFrame: NSUnionRect([result frame], NSInsetRect([F frame], -4, 0))];\
        [N setFrameOrigin: NSMakePoint([N frame].origin.x+[result frame].size.width, [N frame].origin.y)];\
        [result addSubview: N];\
        [result setFrame: NSUnionRect([result frame], [N frame])];\
        return [result autorelease];\
    }\
    else\
        return nil;\
}
NAVIGATION_SET(navigationSetForToolbar, navigationFieldForToolbar);
NAVIGATION_SET(navigationSetForPalette, navigationFieldForPalette);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  SET
#define SET(getter, one, two, three)\
- (NSView *) getter;\
{\
    NSView * result = [[NSView alloc] initWithFrame: NSMakeRect(0,0,96,32)];\
    iTMNavigationButton * button = [self one];\
    [result addSubview: button];\
    button = [self two];\
    [button setFrameOrigin: NSMakePoint(32,0)];\
    [result addSubview: button];\
    button = [self three];\
    [button setFrameOrigin: NSMakePoint(64,0)];\
    [result addSubview: button];\
    return [result autorelease];\
}
SET(nextSet, nextButton, nextNextButton, lastButton);
SET(previousSet, firstButton, previousPreviousButton, previousButton);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  MIXEDSET
#define MIXEDSET(getter, one, two, three)\
- (NSView *) getter;\
{\
    iTMNavigationButton * button = [self one];\
    id result = [[iTMFlagsChangedView alloc] initWithFrame: [button frame]];\
    [result addSubview: button];\
    [button setTag: 0];\
    button = [self two];\
    [result addSubview: button];\
    [button setTag: [result tagFromMask: NSShiftKeyMask]];\
    button = [self three];\
    [result addSubview: button];\
    [button setTag: [result tagFromMask: NSAlternateKeyMask]];\
    [result computeIndexFromTag];\
    return [result autorelease];\
}
MIXEDSET(previousMixedSet, previousButton, previousPreviousButton, firstButton);
MIXEDSET(nextMixedSet, nextButton, nextNextButton, lastButton);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  BUTTON
#define BUTTON(getter, action, class);\
- (class *) getter;\
{\
    class * button = [class button];\
    [button setAction: @selector(action)];\
    [button setTarget: nil];\
    [button setContinuous: YES];\
    return button;\
}
BUTTON(firstButton, displayFirstPage:, iTMFirstButton);
BUTTON(previousButton, displayPreviousPage:, iTMPreviousButton);
BUTTON(previousPreviousButton, displayPreviousPreviousPage:, iTMPreviousPreviousButton);
BUTTON(lastButton, displayLastPage:, iTMLastButton);
BUTTON(nextButton, displayNextPage:, iTMNextButton);
BUTTON(nextNextButton, displayNextNextPage:, iTMNextNextButton);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  NAVBUTTON
#define NAVBUTTON(getter, action, class, menu);\
- (class *) getter;\
{\
    class * button = [class button];\
    [button setAction: @selector(action)];\
    [button setTarget: nil];\
    [button setMenu: [self menu]];\
    return button;\
}
NAVBUTTON(backButton, displayBackPage:, iTMBackButton, backMenu);
NAVBUTTON(forwardButton, displayForwardPage:, iTMForwardButton, forwardMenu);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- navigationField
- (NSTextField *) navigationField;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(!_NavigationField)
    {
        iTMNavigationFormatter * NF = [[iTMNavigationFormatter alloc] init];
        _NavigationField = [[NSTextField alloc] initWithFrame: NSZeroRect];
        [_NavigationField setAction: @selector(displayPageFromField:)];
        [_NavigationField setTarget: nil];
        [_NavigationField setFormatter: NF];
        [_NavigationField setDelegate: NF];
        [_NavigationField setFrameOrigin: NSMakePoint(4,6)];
        [[_NavigationField cell] setSendsActionOnEndEditing: NO];
        {
            int maximum = 1000;
            [_NavigationField setFrameSize: NSMakeSize([[NF stringForObjectValue: [NSNumber numberWithInt: maximum]]
                            sizeWithAttributes: [NSDictionary dictionaryWithObjectsAndKeys:
                                    [[_NavigationField cell] font], NSFontAttributeName, nil]].width+8, 22)];
        }
        #if 0
        {
            int maximum = [[NF maximum] intValue];
            [_NavigationField setFrameSize: NSMakeSize([[[self navigationFormatter]
                    stringForObjectValue: [NSNumber numberWithInt: maximum]]
                            sizeWithAttributes: [NSDictionary dictionaryWithObjectsAndKeys:
                                    [[_NavigationField cell] font], NSFontAttributeName, nil]].width+8, 22)];
            [_NavigationField setEnabled: ([self representation] != nil)&&(maximum>1)];
        }
        #endif
        [NF release];
    }
    return _NavigationField;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- setNavigationField:
- (void) setNavigationField: (NSTextField *) aTextField;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(![aTextField isEqual: _NavigationField])
    {
        [_NavigationField release];
        _NavigationField = [aTextField retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- MENUGETTERSETTER
#define MENUGETTERSETTER(source, getter, setter)\
- (id) getter;\
{\
    if(!source)\
    {\
        [self setter: [[[NSMenu allocWithZone: [NSMenu menuZone]] initWithTitle: @""] autorelease]];\
        [source setMenuRepresentation: [[[NSMenuView alloc] initWithFrame: NSZeroRect] autorelease]];\
        [[source menuRepresentation] setFont: [NSFont menuFontOfSize: [NSFont smallSystemFontSize]]];\
    }\
    return source;\
}\
- (void) setter: (id) argument;\
{\
    if(![source isEqual: argument])\
    {\
        if(argument && ![argument isKindOfClass: [NSMenu class]])\
            [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSMenu argument expected: %@.",\
                [self class], NSStringFromSelector(_cmd), argument];\
        else\
        {\
            [source release];\
            source = [argument retain];\
        }\
    }\
    return;\
}
MENUGETTERSETTER(_BackMenu, backMenu, setBackMenu);
MENUGETTERSETTER(_ForwardMenu, forwardMenu, setForwardMenu);
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-  iTMPDFToolbarDelegate

