/*
//  iTMPDFView.m
//  TBR Tutorial
//
//  Created by jerome.laurens@u-bourgogne.fr on Wed Jun 27 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
*/

#import <Foundation/Foundation.h>
#import <AppKit/NSImage.h>
#import <AppKit/AppKit.h>
#import "iTMStatusNotification.h"
#import "iTMPDFView.h"
#import "iTMPDFDocument.h"
#include <pthread.h>

NSString * const iTMUDThreadedPDFRenderingKey = @"iTMUDThreadedPDFRendering";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMPDFView
/*"Description forthcoming."*/
@implementation iTMPDFView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults:
        [NSDictionary dictionaryWithObjectsAndKeys:
            [NSNumber numberWithBool: NO], iTMUDThreadedPDFRenderingKey,
                nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
- (void) dealloc
/*"Description forthcoming."*/
{
    [self setImage: nil];
    [super dealloc];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  drawRect:
- (void) drawRect: (NSRect) aRect;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if([[NSUserDefaults standardUserDefaults] boolForKey: iTMUDThreadedPDFRenderingKey])
        [[self image] compositeToPoint: [self bounds].origin operation: NSCompositeCopy];
    else
    {
        [[self representation] setCurrentPage: [self currentPage]];
        [[self representation] drawInRect: NSOffsetRect([self bounds], -[self bounds].origin.x,-[self bounds].origin.y)];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  updateGeometry
- (void) updateGeometry;
/*"Once the representation or the magnification has been changed, the geometry must be adapted consequently.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [super updateGeometry];
    [self setImage: nil];
    ++_ChangeCount;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= currentPage
- (int) currentPage;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return _CurrentPage;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setCurrentPage:
- (void) setCurrentPage: (int) aCurrentPage;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(_CurrentPage != aCurrentPage)
    {
        _CurrentPage = aCurrentPage;
        [self setImage: nil];
        ++_ChangeCount;
    }
    return;
}
#if 1
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= createImageInThread:
- (void) createImageInThread: (id) unused;
/*"Lazy initializer.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    // this part of code about thread priority is due to andrei@ids.bg
    pthread_t tid;
    int ret;
    struct sched_param param;
    int lowPriority;
    int policy;
    
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
    id representation = [self representation];
    lowPriority = 10;
    tid = pthread_self();
    ret = pthread_getschedparam(tid, &policy, &param);
    policy = SCHED_OTHER;
    param.sched_priority = lowPriority;
    ret = pthread_setschedparam(tid, policy, &param);
    
    if(representation)
        while(!_Image)
        {
            NSSize S = [self convertSize: [self bounds].size toView: nil];
            NSImage * image = [[NSImage alloc] initWithSize: S];
            int changeCount = _ChangeCount;
            [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
                    [NSString stringWithFormat: NSLocalizedStringFromTableInBundle
                        (@"PDF rendering for page %d.", @"View", [NSBundle bundleForClass: [self class]], ""),
                            [self currentPage]+1]];
            [image addRepresentation: representation];
            [image setScalesWhenResized: YES];
            [image setBackgroundColor: [NSColor whiteColor]];
            [representation setCurrentPage: [self currentPage]];
            [image lockFocus];
            #if 0
            {
                int i = 100000000;
                while(i>0)
                {
                    int j = 10000;
                    while(j>0) --j;
                    --i;
                }
            }
            #endif
            [image unlockFocus];
            {
                NSLock * lock = [[NSLock alloc] init];
                [lock lock];
                if(_ChangeCount == changeCount)
                {
                    [self setImage: image];
                    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
                        [NSString stringWithFormat: NSLocalizedStringFromTableInBundle
                            (@"PDF rendering done for page %d.", @"View", [NSBundle bundleForClass: [self class]], ""),
                                [self currentPage]+1]];
                    [self setNeedsDisplay: YES];
                }
                else
                    [[iTMStatusNotificationCenter defaultCenter] cleanStatus];
                [lock unlock];
                [image release];
                [lock release];
            }
        }
    [pool release];
    [NSThread exit];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= image
- (NSImage *) image;
/*"Lazy initializer.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(!_Image)
        [NSThread detachNewThreadSelector: @selector(createImageInThread:) toTarget: self withObject: nil];
    return _Image;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setImage:
- (void) setImage: (NSImage *) anImage;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(![anImage isEqual: _Image])
    {
        [_Image release];
        _Image = [anImage retain];
    }
    return;
}
#endif
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMPDFView

