/*
//  iTMRepAlbumView.m
//  TBR Tutorial
//
//  Created by jerome.laurens@u-bourgogne.fr on Wed Jun 27 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
*/

#import "NSView(iTeXMac).h"
#import "iTMRepAlbumView.h"
#import "iTMRepView.h"

NSString * const iTMMagnificationDidChangeNotification = @"iTMMagnificationDidChange";

NSString * const iTMMagnificationKey = @"iTMRepMagnification";
NSString * const iTMUDRepAlbumMarginKey = @"iTMRepAlbumMargin";
NSString * const iTMUDCurrentRepMagnificationKey = @"iTMCurrentRepMagnification";
NSString * const iTMUDPreferredRepMagnificationKey = @"iTMPreferredRepMagnification";
NSString * const iTMUDStickToWindowKey = @"iTMStickToWindow";
NSString * const iTMUDStickToWindowModeKey = @"iTMStickToWindowMode";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMRepAlbumView  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

@interface NSView(iTMRepAlbumViewPrivate)
- (void) setRepresentation: (id) argument;
- (void) setMagnification: (id) argument;
@end

@interface iTMRepAlbumView(iTMPrivate)
- (void) _SetMagnification: (NSDecimalNumber *) argument;
@end

@implementation iTMRepAlbumView
/*"This class draws a photo frame around each subviews. We definitely can not use NSImage
ability to draw this kind of frame as it is because this AppKit class scales the image.
So, the proportions are not respected at all. This could be annoying for precise measurements.
Only one usefull method is implemented #{drawrect:}, the others are for private use."*/
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults:
        [NSDictionary dictionaryWithObjectsAndKeys:
            [[NSNumber numberWithFloat: 10.0] stringValue], iTMUDRepAlbumMarginKey,
            [NSDecimalNumber one], iTMUDCurrentRepMagnificationKey,
            [NSDecimalNumber one], iTMUDPreferredRepMagnificationKey,
            [NSNumber numberWithBool: NO], iTMUDStickToWindowKey,
            [NSNumber numberWithInt: iTMStickToActualMode], iTMUDStickToWindowModeKey,
                nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initWithFrame:
- (id) initWithFrame: (NSRect) aRect;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    self = [super initWithFrame: aRect];
    [self setMagnification: [self preferredMagnification]];
    self->_CanStick = YES;
    [self setParametersHaveChanged: YES];
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= dealloc;
- (void) dealloc;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [[NSNotificationCenter defaultCenter] removeObserver: nil name: nil object: self];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  awakeFromNib
- (void) awakeFromNib;
/*"Registers for NSViewFrameDidChangeNotification sent by the enclosing scroll view."*/
{
    [[[self enclosingScrollView] contentView] setPostsFrameChangedNotifications: YES];
    [[NSNotificationCenter defaultCenter]
                removeObserver: self
                            name: NSViewFrameDidChangeNotification
                                        object: nil];
    [[NSNotificationCenter defaultCenter]
            addObserver: self
                    selector: @selector(enclosingScrollViewFrameDidChange:)
                            name: NSViewFrameDidChangeNotification
                                    object: [self enclosingScrollView]];
    [self registerCurrentMagnification];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setNeedsDisplay:
- (void) setNeedsDisplay: (BOOL) aFlag;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(aFlag)
        [self update];
    [super setNeedsDisplay: aFlag];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  prepareToUpdate
- (void) prepareToUpdate;
/*"Must be called at the beginning of the update sequence. Manages the stick to window options.
Subclasses won't forget to send the message to super...
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(_CanStick && [[NSUserDefaults standardUserDefaults] boolForKey: iTMUDStickToWindowKey])
        [self setMagnificationWithMode:
            [[NSUserDefaults standardUserDefaults] integerForKey: iTMUDStickToWindowModeKey]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  newSubview
- (NSView *) newSubview;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return [[[iTMRepView alloc] initWithFrame: NSZeroRect] autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  updateSubview:
- (void) updateSubview: (NSView *) subview;
/*"The representation and the magnification are set. Subclasses will append their own update stuff.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [subview setRepresentation: [self representation]];
    [subview setMagnification: [self magnification]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  update
- (void) update;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [self prepareToUpdate];
    if([self parametersHaveChanged])
    {
        NSPoint absoluteFocus = NSZeroPoint;
        [self setParametersHaveChanged: NO];
        if([self representation]!=nil)
        {
            iTMRepView * subview;
            if([[self subviews] count] == 0)
                [self addSubview: [self newSubview]];
            else
                while([[self subviews] count]>1)
                    [[[self subviews] objectAtIndex: 1] removeFromSuperviewWithoutNeedingDisplay];
            subview = [[self subviews] objectAtIndex: 0];
            absoluteFocus = [subview absoluteFocus];// remembers the last focused point before any change
            [self updateSubview: subview];
            {
                NSRect rect = NSInsetRect([subview frame], -10, -10);
                NSRect bounds = [[[self enclosingScrollView] contentView] bounds];//the contentView is tied to the window
                [self setFrame: NSUnionRect(NSOffsetRect(rect, -rect.origin.x, -rect.origin.y),
                            NSOffsetRect(bounds, -bounds.origin.x, -bounds.origin.y))];
            }
            [subview setFrameCenter: [self boundsCenter]];
            {
                NSSize size = [subview bounds].size;
                NSPoint origin = [subview bounds].origin;
                absoluteFocus.x*=size.width;
                absoluteFocus.y*=size.height;
                absoluteFocus.x+=origin.x;
                absoluteFocus.y+=origin.y;
            }
            {
                NSSize visibleSize = [self visibleRect].size;
                NSPoint visibleFocus = [self convertPoint: absoluteFocus fromView: subview];
                [self scrollRectToVisible: NSOffsetRect(NSMakeRect(visibleFocus.x, visibleFocus.y,
                        visibleSize.width, visibleSize.height),
                                -visibleSize.width/2, -visibleSize.height/2)];
            }
            if(![self window] || ![[self window] makeFirstResponder: subview])
                NSLog(@"[[self window] makeFirstResponder: subview]: Refused");
            else
                NSLog(@"%@ = %@", [[self window] firstResponder], subview);
        }
        else
            while([[self subviews] count]>0)
                [[[self subviews] objectAtIndex: 0] removeFromSuperviewWithoutNeedingDisplay];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  drawRect:
- (void) drawRect: (NSRect) aRect;
/*"Straightforward implementation: a loop on all the subviews, for each one we draw a series of rects
with different gray colors. No alpha implemented. No change in the graphics context.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [self setNeedsDisplay: NO];
    [[NSColor windowBackgroundColor] set];
    [NSBezierPath fillRect: aRect];
#warning DEBUG
#if 1
    if([[self subviews] count]>0)
    {
        NSEnumerator * enumerator = [[self subviews] objectEnumerator];
        NSView * subview;
        while(subview = [enumerator nextObject])
            NSDrawWhiteBezel(NSInsetRect([subview frame],-2,-2),aRect);
    }
#endif
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  clientView
- (void) enclosingScrollViewFrameDidChange:  (NSNotification*) notification;
/*"This is the response to NSViewFrameDidChangeNotification.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [self setNeedsDisplay: YES];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  preferredMagnificationKey
- (NSString *) preferredMagnificationKey;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return iTMUDPreferredRepMagnificationKey;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  preferredMagnification
- (NSDecimalNumber *) preferredMagnification;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return [[NSUserDefaults standardUserDefaults] objectForKey: [self preferredMagnificationKey]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  magnification
- (NSDecimalNumber *) magnification;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return _Magnification;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  currentMagnificationKey
- (NSString *) currentMagnificationKey;
/*"See #{registerCurrentMagnification}...
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return iTMUDCurrentRepMagnificationKey;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  registerCurrentMagnification
- (void) registerCurrentMagnification;
/*"Register here the current magnification in the user default registration domain for key #{currentMagnificationKey}...
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    id magnification = [self magnification];
    if(magnification)
    {
        [[NSUserDefaults standardUserDefaults] registerDefaults:
            [NSDictionary dictionaryWithObjectsAndKeys: magnification, [self currentMagnificationKey], nil]];
        [[NSNotificationCenter defaultCenter]
            postNotificationName: iTMMagnificationDidChangeNotification
                object: self
                    userInfo: [NSDictionary dictionaryWithObjectsAndKeys: _Magnification, iTMMagnificationKey, nil]];
    }
    else
        [[NSUserDefaults standardUserDefaults] removeObjectForKey: [self currentMagnificationKey]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  parametersHaveChanged
- (BOOL) parametersHaveChanged;
/*"Parameters are attributes that may change the aspect of the view: magnification, orientation...
When parameters have changed, the view is inited.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return _ParametersHaveChanged;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setParametersHaveChanged:
- (void) setParametersHaveChanged: (BOOL) aFlag;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(_ParametersHaveChanged != aFlag)
    {
        _ParametersHaveChanged = aFlag;
        if(_ParametersHaveChanged)
            [self setNeedsDisplay: YES];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMagnification:
- (void) setMagnification: (NSDecimalNumber *) aMagnification;
/*A wrapper of _SetMagnification: _CanStick added.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    _CanStick = NO;// critical: before truely setting the magnification
    [self _SetMagnification: aMagnification];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  _SetMagnification:
- (void) _SetMagnification: (NSDecimalNumber *) aMagnification;
/*"If there is something to set, this will be registered in the user default domain under key iTMCurrentRepMagnificationKey,
and a iTMMagnificationDidChangeNotification is posted with the receiver as object and as userInfo the new magnification
under key iTMMagnificationKey in a dictionary.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if([aMagnification compare: _Magnification] != NSOrderedSame)
    {
        [_Magnification release];
        _Magnification = [aMagnification retain];
        [self setParametersHaveChanged: YES];
        [self registerCurrentMagnification];
    }
    [[self window] makeFirstResponder: self];// Critical: see the responder added after the contentView (a bit tricky!!!)
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMagnificationWithMode:
- (void) setMagnificationWithMode: (iTMStickToWindowMode) mode;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
//    NSLog(@"setMagnificationWithMode: %i", mode);
    switch(mode)
    {
        case iTMStickToWidthMode:
            [self setMagnification: [self ratioContentVersusDocumentWidth]];
            _CanStick = YES;
            break;
        case iTMStickToHeightMode:
            [self setMagnification: [self ratioContentVersusDocumentHeight]];
            _CanStick = YES;
            break;
        case iTMStickToViewMode:
        {
            float widthRatio = [[self ratioContentVersusDocumentWidth] floatValue];
            float heightRatio = [[self ratioContentVersusDocumentHeight] floatValue];
            [self _SetMagnification: (NSDecimalNumber *)[NSDecimalNumber numberWithFloat: MIN(widthRatio, heightRatio)]];
            _CanStick = YES;
            break;
        }
        case iTMStickToActualMode:
        default:
            _CanStick = NO;
            break;
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  innerMagnification
- (NSDecimalNumber *) innerMagnification;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return (NSDecimalNumber *) [NSDecimalNumber numberWithFloat: MIN([[self magnification] floatValue] * sqrt(2), 6400)];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  outerMagnification
- (NSDecimalNumber *) outerMagnification;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return (NSDecimalNumber *) [NSDecimalNumber numberWithFloat: MAX([[self magnification] floatValue]/sqrt(2), 0.1)];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=  ratioContentVersusDocumentWidth
- (NSDecimalNumber *) ratioContentVersusDocumentWidth;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    float width=[[self representation] size].width;
    if(width>0)
    {
        NSView * contentView=[[self enclosingScrollView] contentView];
        if(contentView != nil)
            return (NSDecimalNumber *) [NSDecimalNumber numberWithFloat: ([contentView frame].size.width
            -2*[[NSUserDefaults standardUserDefaults] floatForKey: iTMUDRepAlbumMarginKey])/width];
    }
    return [NSDecimalNumber one];
}
//=-=-=-=-=-=-=-=-=-=-=-=  ratioContentVersusDocumentHeight
- (NSDecimalNumber *) ratioContentVersusDocumentHeight;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    float height=[[self representation] size].height;
    if(height>0)
    {
        NSView * contentView=[[self enclosingScrollView] contentView];
        if(contentView != nil)
            return (NSDecimalNumber *) [NSDecimalNumber numberWithFloat: ([contentView frame].size.height
            -2*[[NSUserDefaults standardUserDefaults] floatForKey: iTMUDRepAlbumMarginKey])/height];
    }
    return [NSDecimalNumber one];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  representation
- (id) representation;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return [[[[self window] windowController] document] representation];
}
@end

@implementation NSView(iTMRepAlbumView)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  absoluteFocus
- (NSPoint) absoluteFocus;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    NSPoint absoluteFocus;
    NSRect visibleRect = [self visibleRect];
    absoluteFocus = NSMakePoint(NSMidX(visibleRect), NSMidY(visibleRect));
    absoluteFocus.x-=[self bounds].origin.x;
    absoluteFocus.y-=[self bounds].origin.y;
    {
        NSSize size = [self bounds].size;
        float width = size.width > 0? size.width: 0.000001;
        float height = size.height > 0? size.height: 0.000001;
        absoluteFocus.x/=width;
        absoluteFocus.y/=height;
    }
    return absoluteFocus;
}
@end


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMRepAlbumView  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
