/*
//  iTMRepDocument.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sun Jun 24 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
*/

#import <Foundation/Foundation.h>
//#import <AppKit/NSStepper.h>
#import <AppKit/AppKit.h>
#import "NSMenu(iTeXMac).h"
#import "iTMRepDocument.h"
#import "iTMRepWindowController.h"
#import "iTMListMenu.h"
#import "iTMMagnificationFormatter.h"
#import "iTMPrintView.h"

NSString * const iTMRepDocumentType = @"Representation Document";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMRepDocument  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
@implementation iTMRepDocument
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  makeRepresentationWC
- (void) makeRepresentationWC;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [self removeWindowController: [self representationWC]];
    [self setRepresentationWC: nil];
    if([self data])
        [self addWindowController: [self representationWC]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  representationWindow
- (id) representationWindow;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return [[self representationWC] window];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= fileModificationDate
- (NSDate *) fileModificationDate;
/*"Description forthcoming."*/
{
    return _FMD;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setFileModificationDate:
- (void) setFileModificationDate: (NSDate *) argument;
/*"Description forthcoming."*/
{
    if(![argument isEqual: _FMD])
    {
        [_FMD release];
        _FMD = [argument retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  update
- (void) update;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List: save the file, if it has disappeared from the HD.
"*/
{
    if(([[[[NSFileManager defaultManager] fileAttributesAtPath: [self fileName] traverseLink: NO]
                    fileModificationDate] compare: [self fileModificationDate]] == NSOrderedDescending))
        if(![self readFromFile: [self fileName] ofType: [self fileType]])
        {
            NSRunCriticalAlertPanel(
                NSLocalizedStringFromTableInBundle(@"Updating representation.", @"View",
                                [NSBundle bundleForClass: [self class]], "Informational Panel Title"),
                NSLocalizedStringFromTableInBundle(@"Unable to update.", @"View",
                                [NSBundle bundleForClass: [self class]], "Problem"),
                nil, nil, nil);
        }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  representationWCClass
- (Class) representationWCClass;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return [iTMRepWindowController class];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  representationWC
- (id) representationWC;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(!_RepresentationWC)
        _RepresentationWC = [[[self representationWCClass] alloc] initWithWindowNibName: NSStringFromClass([self class])];
    return _RepresentationWC;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setRepresentationWC:
- (void) setRepresentationWC: (id) aWC;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(![_RepresentationWC isEqual: aWC])
    {
        [_RepresentationWC release];
        _RepresentationWC = [aWC retain];
        [_RepresentationWC setDocument: self];
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= data
- (NSData *) data;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return _Data;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setData:
- (void) setData: (NSData *) data;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(![data isEqual: _Data])
    {
        [_Data release];
        _Data = [data retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= representation
- (id) representation;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return _Representation;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setRepresentation:
- (void) setRepresentation: (id) aRepresentation;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(![aRepresentation isEqual: _Representation])
    {
        [_Representation release];
        _Representation = [aRepresentation retain];
        [[NSNotificationCenter defaultCenter] postNotificationName: @"iTMRepresentationDidChange"
            object: self
                userInfo: nil];
    }
    return;
}
@end

@implementation iTMRepDocument(Cocoa)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [self setData: nil];
    [self setRepresentation: nil];
    [self setRepresentationWC: nil];
    [self setFileModificationDate: nil];
    [super dealloc];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  printShowingPrintPanel:
- (void) printShowingPrintPanel: (BOOL) flag;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    NSImageRep * representation = [[NSImageRep imageRepClassForData: [self data]] imageRepWithData: [self data]];
    iTMPrintView * view = [[iTMPrintView allocWithZone: [self zone]] initWithRepresentation: representation];
    NSPrintOperation * printOperation = [NSPrintOperation printOperationWithView: view
        printInfo: [self printInfo]];
    NSWindow * window = [[NSWindow allocWithZone:[self zone]]
            initWithContentRect: NSMakeRect(0.0, 0.0, [view frame].size.width, [view frame].size.height)
                    styleMask: NSBorderlessWindowMask backing: NSBackingStoreNonretained defer: NO];
    [[window contentView] addSubview: view];
    [printOperation setShowPanels: flag];
    [printOperation setCanSpawnSeparateThread: YES];
    [printOperation runOperation];
    [view release];
    [window release];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  loadFileWrapperRepresentation:ofType:
- (BOOL) loadFileWrapperRepresentation: (NSFileWrapper *) wrapper ofType: (NSString *) docType;
/*"Beware, this is not meant to be called directly.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    BOOL result = [super loadFileWrapperRepresentation: wrapper ofType: docType];
    if(result)
    	[self setFileModificationDate: [[wrapper fileAttributes] fileModificationDate]];
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  loadDataRepresentation:ofType:
- (BOOL) loadDataRepresentation: (NSData *) data ofType: (NSString *) aType;
/*" Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    Class C = [NSImageRep imageRepClassForData: data];
    if(C)
    {
        [self setData: data];
        [self setRepresentation: [[[C alloc] initWithData: data] autorelease]];
    }
    return [self representation] != nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-  dataRepresentationOfType:
- (NSData *) dataRepresentationOfType: (NSString *) aType;
/*" Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return [self data];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  makeWindowControllers
- (void) makeWindowControllers;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [self makeRepresentationWC];
    return;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMRepDocument  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
