/*
//  iTMRepView.m
//  TBR Tutorial
//
//  Created by jerome.laurens@u-bourgogne.fr on Wed Jun 27 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
*/

#import <Foundation/Foundation.h>
#import <AppKit/NSView.h>
#import <AppKit/AppKit.h>
#import "iTMRepView.h"
#import "iTMRepDocument.h"

#define dPDFMaximumMagnification 8.0

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMRepView
/*"Description forthcoming."*/
@implementation iTMRepView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= representation
- (id) representation;
/*"Lazy initializer.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return _Representation;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setRepresentation:
- (void) setRepresentation: (id) aRepresentation;
/*"Designated initializer.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(![aRepresentation isEqual: _Representation])
    {
        [_Representation release];
        _Representation = [aRepresentation retain];
        [self updateGeometry];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= magnification
- (id) magnification;
/*"Lazy initializer.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return _Magnification;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMagnification:
- (void) setMagnification: (NSDecimalNumber *) aMagnification;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(![_Magnification isEqual: aMagnification])
    {
        [_Magnification release];
        if([aMagnification floatValue]>0)
            _Magnification = [aMagnification retain];
        else
        {
            _Magnification = [[NSDecimalNumber one] retain];
            NSLog(@"Bad magnification given (<= 0) : 1 used instead.");
        }
        [self updateGeometry];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  updateGeometry
- (void) updateGeometry;
/*"Once the representation or the magnification has been changed, the geometry must be adapted consequently.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    NSSize size = [[self representation] size];
    float F = [[self magnification] floatValue];
    [self setFrameSize: NSMakeSize(size.width*F, size.height*F)];
    [self setBoundsSize: size];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  drawRect:
- (void) drawRect: (NSRect) aRect;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
//    [[self image] compositeToPoint: [self bounds].origin operation: NSCompositeCopy];
//    [[[self image] bestRepresentationForDevice: nil] drawInRect: [self bounds]];
    [[self cachedRepresentation] drawInRect: [self bounds]];
//    NSLog(@"frame: %@, bounds: %@", NSStringFromRect([self frame]), NSStringFromRect([self bounds]));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  pageFrameSizeWithMagnification:
- (NSSize) pageFrameSizeWithMagnification: (NSDecimalNumber *) aMagnification;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(aMagnification<=0)
        return NSZeroSize;
    else
    {
        NSSize result = [self bounds].size;
        float M = [aMagnification floatValue];
        result.width *= M;
        result.height *= M;
        return result;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isFlipped  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
- (BOOL) isFlipped
/*"YES."*/
{
    return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  acceptsFirstResponder  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
- (BOOL) acceptsFirstResponder
/*"YES."*/
{
    return YES;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  acceptsFirstMouse  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
- (BOOL) acceptsFirstMouse: (NSEvent *) theEvent
/*"YES."*/
{
    return YES;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  recache
- (void) recache;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [_CachedRepresentation release];
    _CachedRepresentation = nil;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= cachedRepresentation
- (id) cachedRepresentation;
/*"Lazy initializer.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(!_CachedRepresentation)
    {
        if(_Representation)
        {
//            NSSize S = [self convertSize: [self bounds].size toView: nil];
//            _Image = [[NSImage alloc] initWithSize: S];
            NSImage * image = [[NSImage alloc] initWithSize: [self bounds].size];
            [image addRepresentation: _Representation];
            [image setBackgroundColor: [NSColor whiteColor]];
            [image lockFocus];
            [image unlockFocus];
            _CachedRepresentation = [[image bestRepresentationForDevice: nil] retain];
            [image release];
        }
    }
    return _CachedRepresentation;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
- (void) dealloc
/*"Description forthcoming."*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [self setRepresentation: nil];
    [self setMagnification: nil];
    [self recache];
    [super dealloc];
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMRepView

