/*
//  iTMRepWindowController.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sun Jan 06 2002.
//  Copyright © 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSNotification.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "NSMenu(iTeXMac).h"
#import "NSWindow(iTeXMac).h"
#import "iTMRepWindowController.h"
#import "iTMMixedButton.h"
#import "iTMHelpManager.h"
#import "iTMMagnificationFormatter.h"
#import "iTMRepToolbarDelegate.h"
#import "iTMRepAlbumView.h"
#import <math.h>

NSString * const iTMUDAutoUpdatesViewKey = @"iTMAutoUpdatesView";
NSString * const iTMUDSmartUpdatesViewKey = @"iTMSmartUpdatesView";

@interface iTMRepWindowController(iTMPrivate)
- (Class) _ToolbarDelegateClass;
@end

@interface NSDocument(iTMRepWindowController)
- (void) updateFromFile;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMRepWindowController
/*"Description forthcoming."*/
@implementation iTMRepWindowController
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults:
        [NSDictionary dictionaryWithObjectsAndKeys:
            [NSNumber numberWithBool: YES], iTMUDAutoUpdatesViewKey,
            [NSNumber numberWithBool: YES], iTMUDSmartUpdatesViewKey,
                nil]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [self setToolbarDelegate: nil];
    [super dealloc];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowWillLoad
- (void) windowWillLoad;
/*"Cocoa design.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{    
    [super windowWillLoad];
    [self setShouldCascadeWindows: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowDidLoad
- (void) windowDidLoad;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (11/27/2001)
To Do List:.
"*/
{
    [super windowDidLoad];
    if([self respondsToSelector: @selector(installToolbarWithIdentifier:)])
        [self installToolbarWithIdentifier: [self toolbarIdentifier]];
    {
        NSView * album = [self album];
        NSPoint point = [album frame].origin;
        point.y+=[album frame].size.height;
        [album scrollPoint: point];
        [album setNeedsDisplay: YES];
    }
    {
        NSWindow * W = [self window];
        [self setWindowFrameAutosaveName: [[W class] frameIdentifier]];
        [W setFrameFromStandardUserDefaults];
        #warning JL: this MUST live elsewhere
        {
            NSMenuItem * menuItem = [[NSApp mainMenu] deepItemWithAction: @selector(displayAtMagnification:)];
            NSEnumerator * menuEnumerator = [[[menuItem menu] itemArray] objectEnumerator];
            iTMMagnificationFormatter * MF = [[iTMMagnificationFormatter alloc] init];
            while(menuItem=[menuEnumerator nextObject])
            {
                NSNumber * number;
                if([MF getObjectValue: &number forString: [menuItem title] errorDescription: nil])
                    [menuItem setRepresentedObject: number];
            }
            [MF release];
        }
    }
    _UpdateIsLocked = NO;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowDidUpdate:
- (void) windowDidUpdate: (NSNotification *) aNotification;
/*"Part of the autoupdate mechanism. Auto only works for documents with a window.
History: jerome.laurens@u-bourgogne.fr
To Do List: save the file if needed...
"*/
{
    if([[self window] isEqual: [NSApp mainWindow]] &&
            ![[NSUserDefaults standardUserDefaults] boolForKey: @"iTMIsCompiling"] &&
            [[NSUserDefaults standardUserDefaults] boolForKey: iTMUDAutoUpdatesViewKey] &&
                !_UpdateIsLocked)
        #warning _UpdateIsLocked == iTMIsCompiling ?
    {
        NSDate * refDate = _QFMD? _QFMD: [[self document] fileModificationDate];
        [_NQFMD autorelease];
        #if 0
        _NQFMD = [[[[NSFileManager defaultManager] fileAttributesAtPath: [[self document] fileName] traverseLink: NO]
                                fileModificationDate] retain];
        #else
        _NQFMD = [[[[[[NSFileWrapper alloc] initWithPath: [[self document] fileName]] autorelease] fileAttributes]
                                fileModificationDate] retain];
        #endif
        if(refDate && ([_NQFMD compare: refDate] == NSOrderedDescending))
            if([[NSUserDefaults standardUserDefaults] boolForKey: iTMUDSmartUpdatesViewKey])
            {
                NSBundle * B = [NSBundle bundleForClass: [self class]];
                _UpdateIsLocked = YES;
                    NSBeginAlertSheet(
                        NSLocalizedStringFromTableInBundle(@"Update View", @"View", B,"Panel title"),
                    NSLocalizedStringFromTableInBundle( @"Update", @"View", B, "Button title"),
                NSLocalizedStringFromTableInBundle( @"Save a Copy Before...", @"View", B, "Button title"),//alt
            NSLocalizedStringFromTableInBundle( @"Ignore", @"View", B, "Button title"),//other
                    [self window],
                    self,
                    NULL,
                    @selector(updateSheetDidDismiss:returnCode:contextInfo:),
                    nil,
                    NSLocalizedStringFromTableInBundle( @"%@\nhas been edited externally.",
                        @"View", B, "Panel Information"),
                    [[self document] fileName]);
            }
            else
            {
                if([[self document] respondsToSelector: @selector(updateFromFile)])
                    [[self document] updateFromFile];
                [_QFMD autorelease];
                _QFMD = _NQFMD;
                _NQFMD = nil;
            }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  updateSheetDidDismiss:returnCode:contextInfo:
- (void) updateSheetDidDismiss: (NSWindow *) sheet returnCode: (int) returnCode contextInfo: (void *) irrelevant;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    _UpdateIsLocked = NO;
    switch(returnCode)
    {
        case NSAlertAlternateReturn:
            [[self document] saveDocumentTo: self];//will try update once more
            break;
        case NSAlertOtherReturn://ignore the modification date
            [_QFMD autorelease];
            _QFMD = _NQFMD;
            _NQFMD = nil;
            break;
        case NSAlertDefaultReturn:
        default:
            if([[self document] respondsToSelector: @selector(updateFromFile)])
                [[self document] updateFromFile];
            [_QFMD autorelease];
            _QFMD = _NQFMD;
            _NQFMD = nil;
            break;
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  _ToolbarDelegateClass
- (Class) _ToolbarDelegateClass;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (11/27/2001)
To Do List:
"*/
{
    if([self respondsToSelector: @selector(repToolbarDelegateClass)])
        return [self repToolbarDelegateClass];
    else
        return [iTMRepToolbarDelegate class];        
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toolbarDelegate
- (id) toolbarDelegate;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (11/27/2001)
To Do List:
"*/
{
    if(!_TD)
        [self setToolbarDelegate: [[[[self _ToolbarDelegateClass] alloc] init] autorelease]];
    return _TD;        
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setToolbarDelegate:
- (void) setToolbarDelegate: (id) argument;
/*"Registers the toolbar delegate as an observer of iTMMagnificationDidChangeNotification sent by the album.
Version history: jerome.laurens@u-bourgogne.fr (11/27/2001)
To Do List:
"*/
{
    if(![_TD isEqual: argument])
    {
        [_TD autorelease];
        _TD = [argument retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toolbarIdentifier
- (NSString *) toolbarIdentifier;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return @"View";
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  _InstallToolbarWithIdentifier:
- (void) installToolbarWithIdentifier: (NSString *) anIdentifier;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if([self toolbarDelegate])
    {
        id toolbar = [[NSToolbar alloc] initWithIdentifier: anIdentifier];
        [toolbar setDelegate: [self toolbarDelegate]];
        [toolbar setAllowsUserCustomization: YES];
        [toolbar setAutosavesConfiguration: YES];
        [toolbar setDisplayMode: NSToolbarDisplayModeIconOnly];
        [[self window] setToolbar: toolbar];
        [toolbar release];
        NSLog(@"The toolbar has been installed for identifier: %@", anIdentifier);
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  representationDidChange:
- (void) representationDidChange: (NSNotification *) aNotification;
/*"Answer of the receiver to a Notification. Marks its album as needing to be displayed.
Version history: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [[self album] setNeedsDisplay: YES];
//    [[[self toolbarDelegate] magnificationField] setFloatValue: [[self magnification] floatValue]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setWindow
- (void) setWindow: (NSWindow *) aWindow;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter]
            removeObserver: self
                    name: NSWindowDidUpdateNotification
                            object: nil];
    [super setWindow: aWindow];
    if(aWindow && [self respondsToSelector: @selector(windowDidUpdate:)])
    {
        [[NSNotificationCenter defaultCenter]
            addObserver: self
                selector: @selector(windowDidUpdate:)
                    name: NSWindowDidUpdateNotification
                        object: aWindow];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setDocument
- (void) setDocument: (NSDocument *) aDocument;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    [super setDocument: aDocument];
    [[NSNotificationCenter defaultCenter]
            removeObserver: self
                    name: @"iTMRepresentationDidChange"
                            object: nil];
    if(aDocument && [self respondsToSelector: @selector(representationDidChange:)])
    {
        [[NSNotificationCenter defaultCenter]
            addObserver: self
                selector: @selector(representationDidChange:)
                    name: @"iTMRepresentationDidChange"
                        object: aDocument];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  album
- (id) album;
/*"Returns the album."*/
{
    return self->_Album;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setAlbum:
- (void) setAlbum: (NSView *) aView;
/*"Registers the toolbar delegate as an observer of iTMMagnificationDidChangeNotification sent by the album.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    if(![aView isEqual: _Album])
    {
        if(_Album)
            [[NSNotificationCenter defaultCenter] removeObserver: self
                    name: iTMMagnificationDidChangeNotification
                            object: _Album];
    	[_Album autorelease];
    	_Album = [aView retain];
        if(_Album)
        {
            SEL MDC = @selector(magnificationDidChange:);
            if([self respondsToSelector: MDC])
                [[NSNotificationCenter defaultCenter]
                    addObserver: self
                        selector: MDC
                            name: iTMMagnificationDidChangeNotification
                                object: _Album];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= magnification
- (NSDecimalNumber *) magnification;
/*"Description forthcoming.
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    return [[self album] magnification];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  magnificationDidChange:
- (void) magnificationDidChange: (NSNotification *) aNotification;
/*"Answer to a Notification post. The userInfo is meant to be a dictionary with the new magnification for key "iTMRepMagnification". 
History: jerome.laurens@u-bourgogne.fr
To Do List:
"*/
{
    NSDictionary * D = (NSDictionary *)[aNotification userInfo];
    if([D isKindOfClass: [NSDictionary class]])
    {
        NSNumber * N = [D objectForKey: iTMMagnificationKey];
        if([N respondsToSelector: @selector(floatValue)])
        {
            [[[self toolbarDelegate] magnificationField] setFloatValue: [N floatValue]];
            return;
        }
        NSLog(@"ERROR: iTMRepWindowController <%@>", N);
    }
    NSLog(@"iTMRepWindowController expected a dictionary userInfo or a number for key iTMRepMagnification");
    return;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMRepWindowController

