/*
//  iTMEditorPrefPane.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Fri Mar 15 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/Foundation.h>
#import <AppKit/NSTabViewItem.h>
#import <AppKit/NSWindow.h>
#import <AppKit/AppKit.h>
#import "iTMEditorPrefPane.h"
#import "iTMAutoSaveController.h"
#import "iTMDocumentController.h"
#import "iTMTeXDocument.h"
#import "iTMUndoManager.h"
#import "iTMSettings.h"
#import "iTMTextView.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMEditorPrefPane
/*"Description forthcoming."*/
@implementation iTMEditorPrefPane
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [iTMAutoSaveController initialize];
    [iTMDocumentController initialize];
    [iTMTeXDocument initialize];
    [iTMUndoManager initialize];    
    [iTMTextView initialize];    
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"Designated Initializer.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    if(self = [super init])
    {
        [_ExternalUsed autorelease];
        _ExternalUsed = nil;
        [_BuiltInUsed autorelease];
        _BuiltInUsed = nil;
        _EditorHasChanged = YES;
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Default implementation does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [_ExternalUsed autorelease];
    _ExternalUsed = nil;
    [_BuiltInUsed autorelease];
    _BuiltInUsed = nil;
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  autoSaveFieldEdited:
- (IBAction) autoSaveFieldEdited: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setFloat: [sender floatValue] forKey: iTMUDAutoSaveIntervalKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateAutoSaveFieldEdited:
- (void) validateAutoSaveFieldEdited: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender setFloatValue: [[self preferences] floatForKey: iTMUDAutoSaveIntervalKey]];
    [sender setEnabled: NO];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  chooseHelper:
- (IBAction) chooseHelper: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSArray * ApplicationDirectories = NSSearchPathForDirectoriesInDomains(NSApplicationDirectory, NSLocalDomainMask, YES);
    if([ApplicationDirectories count])
    {
        NSOpenPanel * OP = [NSOpenPanel openPanel];
        [OP setAllowsMultipleSelection: NO];
        [OP setCanChooseFiles: NO];
        [OP setCanChooseDirectories: YES];
        [OP setResolvesAliases: YES];
        [OP setTreatsFilePackagesAsDirectories: NO];
        if(NSOKButton == [OP runModalForDirectory: [ApplicationDirectories objectAtIndex: 0]
                                    file: nil types: [NSArray arrayWithObject: @"app"]] && [[OP filenames] count])
        {
            [[self preferences] setObject: [[OP filenames] objectAtIndex: 0] forKey: iTMUDTeXHelperKey];
        }
    }
    [self validateUserInterface];
    return;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateChooseHelper:
- (void) validateChooseHelper: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return;
}
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  helperFieldEdited:
- (IBAction) helperFieldEdited: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setObject: [sender stringValue] forKey: iTMUDTeXHelperKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateHelperFieldEdited:
- (void) validateHelperFieldEdited: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender setStringValue: [[self preferences] stringForKey: iTMUDTeXHelperKey]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleAutoSave:
- (IBAction) toggleAutoSave: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setBool: ([sender state] != NSOffState) forKey: iTMUDAutoSaveOnKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToggleAutoSave:
- (void) validateToggleAutoSave: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender setState: NO && ([[self preferences] boolForKey: iTMUDAutoSaveOnKey]? NSOnState: NSOffState)];
    [sender setEnabled: NO];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleKeepBackupFile:
- (IBAction) toggleKeepBackupFile: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setBool: ([sender state] != NSOffState) forKey: iTMUDKeepBackupFileKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToggleKeepBackupFile:
- (void) validateToggleKeepBackupFile: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender setState: ([[self preferences] boolForKey: iTMUDKeepBackupFileKey]? NSOnState: NSOffState)];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleUsesTeXHelper:
- (IBAction) toggleUsesTeXHelper: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setBool: ([sender state] != NSOffState) forKey: iTMUDUsesTeXHelperKey];
    _EditorHasChanged = YES;
    [self validateUserInterface];
    _EditorHasChanged = NO;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToggleUsesTeXHelper:
- (void) validateToggleUsesTeXHelper: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender setState: ([[self preferences] boolForKey: iTMUDUsesTeXHelperKey]? NSOnState: NSOffState)];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleSmartUndo:
- (IBAction) toggleSmartUndo: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setBool: ([sender state] != NSOffState) forKey: iTMUDSmartUndoKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToggleSmartUndo:
- (void) validateToggleSmartUndo: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender setState: ([[self preferences] boolForKey: iTMUDSmartUndoKey]? NSOnState: NSOffState)];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleShiftDelete:
- (IBAction) toggleShiftDelete: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setBool: ([sender state] != NSOffState) forKey: iTMUDShiftDeleteKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToggleShiftDelete:
- (void) validateToggleShiftDelete: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender setState: ([[self preferences] boolForKey: iTMUDShiftDeleteKey]? NSOnState: NSOffState)];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  popUpLevelsOfUndo:
- (IBAction) popUpLevelsOfUndo: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setInteger: [[sender selectedItem] tag] forKey: iTMUDLevelsOfUndoKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validatePopUpLevelsOfUndo:
- (void) validatePopUpLevelsOfUndo: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender selectItemAtIndex: [sender indexOfItemWithTag: [[self preferences] integerForKey: iTMUDLevelsOfUndoKey]]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  externalCommentField:
- (IBAction) externalCommentField: (id) sender;
/*"Fake action used only for validation.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateExternalCommentField:
- (IBAction) validateExternalCommentField: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    if(_EditorHasChanged)
    {
        if(![_ExternalUsed length])
        {
            NSAttributedString * AS = [sender attributedStringValue];
            NSDictionary * attributes = [AS length]? [AS attributesAtIndex: 0 effectiveRange: nil]: nil;
            _ExternalUsed = [[NSAttributedString alloc]
                initWithString: NSLocalizedStringFromTableInBundle(
                    @"Option key to use the built in editor",
                        @"Preferences", [NSBundle bundleForClass: [self class]],
                            "title of the info field in the external tab view of the editor pref pane")
            	attributes: attributes];
        }
        if(![_BuiltInUsed length])
        {
            NSAttributedString * AS = [sender attributedStringValue];
            NSDictionary * attributes = [AS length]? [AS attributesAtIndex: 0 effectiveRange: nil]: nil;
            _BuiltInUsed = [[NSAttributedString alloc]
                initWithString: NSLocalizedStringFromTableInBundle(
                    @"Option key to use the external editor",
                        @"Preferences", [NSBundle bundleForClass: [self class]],
                            "title of the info field in the external tab view of the editor pref pane")
            	attributes: attributes];
        }
        [sender setAttributedStringValue:
            ([[self preferences] boolForKey: iTMUDUsesTeXHelperKey]? _ExternalUsed: _BuiltInUsed)];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= tabView:didSelectTabViewItem:
- (void) tabView: (NSTabView *) tabView didSelectTabViewItem: (NSTabViewItem *) tabViewItem;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self validateUserInterface];
    [[self window]makeFirstResponder: [tabViewItem initialFirstResponder]];
    return;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  message
- (void) message;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return;
}
#endif
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMEditorPrefPane

