/*
//  iTMLogFontColorPrefPane.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Mar 05 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/Foundation.h>
#import <AppKit/NSPopUpButton.h>
#import <AppKit/NSColorWell.h>
#import <AppKit/AppKit.h>
#import "iTMLogFontColorPrefPane.h"
#import "NSUserDefaults(iTeXMac).h"
#import "NSControl(iTeXMac).h"
#import "iTMProjectWindowController.h"
//#import "iTMProjectDocument.h"
#import "iTMTextStorage.h"
#import "iTMSyntaxParser.h"
#import "iTMLogSyntaxParser.h"
#import "iTMResourcesController.h"
#import "iTMSettings.h"


@interface iTMLogFontColorPrefPane(Private)
- (IBAction) _ToggleUseSyntaxColoring: (id) sender;
- (IBAction) _ToggleUseTextViewColors: (id) sender;
- (void) _ChangeFontTo: (NSFont *) font;
- (NSDictionary *) representedDictionaryWithFontKey: (NSString *) fKey colorKey: (NSString *) cKey;
- (void) _ChangeColorTo: (NSColor *) color;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMLogFontColorPrefPane
/*"Description forthcoming."*/
@implementation iTMLogFontColorPrefPane
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [iTMLogSyntaxParser initialize];// critical
    [iTMProjectWindowController initialize];// critical
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setFontKey: nil];
    [self setColorKey: nil];
    [super dealloc];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  loadWindow
- (void) loadWindow;
/*"Some initialization.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super loadWindow];
    [syntaxPopUp setAutoenablesItems: NO];
    [syntaxPopUp setTarget: self];
    [syntaxPopUp setAction: @selector(popUpSyntaxAttributes:)];
    NS_DURING
    [[syntaxPopUp itemAtIndex: 0]  setRepresentedObject:
        [self representedDictionaryWithFontKey: iTMUDLogRegularFontKey colorKey: iTMUDLogRegularColorKey]];
    [[syntaxPopUp itemAtIndex: 1]  setRepresentedObject:
        [self representedDictionaryWithFontKey: iTMUDLogWarningFontKey colorKey: iTMUDLogWarningColorKey]];
    [[syntaxPopUp itemAtIndex: 2]  setRepresentedObject:
        [self representedDictionaryWithFontKey: iTMUDLogErrorFontKey colorKey: iTMUDLogErrorColorKey]];
    [[syntaxPopUp itemAtIndex: 3]  setRepresentedObject:
        [self representedDictionaryWithFontKey: iTMUDLogInfoFontKey colorKey: iTMUDLogInfoColorKey]];
    NS_HANDLER
    [NSApp reportException: localException];
    [NSApp reportException: [NSException exceptionWithName: NSInternalInconsistencyException
        reason: @"Bad syntaxPopUp in nib file." userInfo: nil]];
    NSLog(@"RATE");
    NS_ENDHANDLER
    [textViewPopUp setAutoenablesItems: NO];
    [textViewPopUp setTarget: self];
    [textViewPopUp setAction: @selector(popUpTextViewAttributes:)];
    NS_DURING
    [[textViewPopUp itemAtIndex: 0] setRepresentedObject: 
        [self representedDictionaryWithFontKey: iTMUDLogStandardFontKey colorKey: iTMUDLogStandardColorKey]];
    [[textViewPopUp itemAtIndex: 1] setRepresentedObject: 
        [self representedDictionaryWithFontKey: nil colorKey: iTMUDLogTextBackgroundColorKey]];
    [[textViewPopUp itemAtIndex: 2] setRepresentedObject: 
        [self representedDictionaryWithFontKey: nil colorKey: iTMUDLogSelectedTextColorKey]];
    [[textViewPopUp itemAtIndex: 3] setRepresentedObject: 
        [self representedDictionaryWithFontKey: nil colorKey: iTMUDLogSelectedTextBackgroundColorKey]];
    [[textViewPopUp itemAtIndex: 4] setRepresentedObject: 
        [self representedDictionaryWithFontKey: nil colorKey: iTMUDLogInsertionPointColorKey]];
    NS_HANDLER
    [NSApp reportException: localException];
    [NSApp reportException: [NSException exceptionWithName: NSInternalInconsistencyException
        reason: @"Bad textViewPopUp in nib file." userInfo: nil]];
    NSLog(@"RATE");
    NS_ENDHANDLER

    {
        iTMTextStorage * TS = [[[iTMTextStorage alloc] initWithString: [self initialTextViewContents]
                            attributes: nil] autorelease];
        iTMLogSyntaxParser * SP = [[self preferences] boolForKey: iTMUDLogSyntaxColoringKey]?
            [[[iTMLogSyntaxParser alloc] init] autorelease]: [[[iTMSyntaxParser alloc] init] autorelease];
        [[textView layoutManager] replaceTextStorage: TS];
        [TS replaceSyntaxParser: SP];
//        [SP fontOrColorDidChange: [NSNotification notificationWithName: iTMFontOrColorDidChangeNotification
//                object: [self preferences]]];
        // the end of the initialization is made in the validateUserInterface
    }
    [textView setEditable: YES];
    [textView setSelectable: YES];
    [textView setDelegate: self];
    [textView setUsesFontPanel: NO];
    
    [tabView selectFirstTabViewItem: self];
    
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialTextViewContentsFileName
- (NSString *) initialTextViewContentsFileName;
/*"Subclassers will return the correct value.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return @"LogPrefPaneTextViewContents.tex";
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialTextViewContents
- (NSString *) initialTextViewContents;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * path;
    path = [[[iTMResourcesController sharedResourcesController]generalDirectoryPath]
        stringByAppendingString: [self initialTextViewContentsFileName]];
    if(![[NSFileManager defaultManager] isReadableFileAtPath: path])
        path = [[NSBundle bundleForClass: [self class]] pathForResource: [self initialTextViewContentsFileName]
                ofType: nil inDirectory: @"Preferences"];
    if(!path)
        path = [[NSBundle bundleForClass: [self class]] pathForResource: [self initialTextViewContentsFileName]
                ofType: nil inDirectory: nil];
    if([path length])
        return [NSString stringWithContentsOfFile: path];
    else
        return @"COUCOU\\ROU\%COUCOU";
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= representedDictionaryWithFontKey:colorKey:
- (NSDictionary *) representedDictionaryWithFontKey: (NSString *) fKey colorKey: (NSString *) cKey;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMutableDictionary * MD = [NSMutableDictionary dictionary];
    id object;
    if([fKey length] && (object = [[self preferences] objectForKey: fKey]))
    {
        [MD setObject: fKey forKey: @"fontKey"];
        [MD setObject: object forKey: @"fontDictionary"];
    }
    if([cKey length] && (object = [[self preferences] objectForKey: cKey]))
    {
        [MD setObject: cKey forKey: @"colorKey"];
        [MD setObject: object forKey: @"colorDictionary"];
    }
    return [[MD copy] autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= popUpSyntaxAttributes:
- (void) popUpSyntaxAttributes: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setFontKey: [[[sender selectedItem] representedObject] objectForKey: @"fontKey"]];
    [self setColorKey: [[[sender selectedItem] representedObject] objectForKey: @"colorKey"]];
    [[sender selectedItem] setRepresentedObject: 
        [self representedDictionaryWithFontKey: [self fontKey] colorKey: [self colorKey]]];
    [self validateUserInterface];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validatePopUpSyntaxAttributes:
- (void) validatePopUpSyntaxAttributes: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSEnumerator * E = [[syntaxPopUp itemArray]objectEnumerator];
    id item;
    while(item = [E nextObject])
        if([[[item representedObject]objectForKey: @"fontKey"] isEqualToString: [self fontKey]])
        {
            [syntaxPopUp selectItem: item];
            break;
        }
    [self setColorKey: [[[syntaxPopUp selectedItem]representedObject] objectForKey: @"colorKey"]];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= popUpTextViewAttributes:
- (void) popUpTextViewAttributes: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setColorKey: [[[sender selectedItem] representedObject] objectForKey: @"colorKey"]];
    [self setFontKey: [[[sender selectedItem] representedObject] objectForKey: @"fontKey"]];
    [[sender selectedItem] setRepresentedObject: 
        [self representedDictionaryWithFontKey: [self fontKey] colorKey: [self colorKey]]];
    [self validateUserInterface];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validatePopUpTextViewAttributes:
#warning only one method
- (void) validatePopUpTextViewAttributes: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSEnumerator * E = [[textViewPopUp itemArray]objectEnumerator];
    id item;
    while(item = [E nextObject])
        if([[[item representedObject]objectForKey: @"colorKey"] isEqualToString: [self colorKey]])
        {
            [textViewPopUp selectItem: item];
            break;
        }
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  didSelect
- (void) didSelect;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self tabView: tabView didSelectTabViewItem: [tabView selectedTabViewItem]];
    [[NSColorPanel sharedColorPanel] setAction: @selector(changeColor:)];
    [[NSColorPanel sharedColorPanel] setTarget: self];
    [[NSFontManager sharedFontManager] setAction: @selector(changeFont:)];
    [super didSelect];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  willSelect
- (void) willSelect;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super willSelect];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  willUnselect
- (void) willUnselect;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super willUnselect];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  didUnselect
- (void) didUnselect;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super didUnselect];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  showFontPanel:
- (IBAction) showFontPanel: (id) sender;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSFontPanel sharedFontPanel] orderFront: self];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  showColorPanel:
- (IBAction) showColorPanel: (id) sender;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSColorPanel sharedColorPanel] orderFront: self];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  _ToggleUseSyntaxColoring:
- (IBAction) _ToggleUseSyntaxColoring: (id) sender;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender setNextState];
    [self toggleUseSyntaxColoring: sender];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleUseSyntaxColoring:
- (IBAction) toggleUseSyntaxColoring: (id) sender;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    [[self undoManager] registerUndoWithTarget: (id)target selector:(SEL)selector object:(id)anObject];
    // beware of the order
    [[self undoManager] registerUndoWithTarget: self
        selector: @selector(_ToggleUseSyntaxColoring:)
            object: sender];
    [[self preferences] setBool: ([sender state] != NSOffState) forKey: iTMUDLogSyntaxColoringKey];
    {
        iTMTextStorage * TS = [textView textStorage];
        NSAssert1([TS isKindOfClass: [iTMTextStorage class]],
            @"-[iTMLogFontColorPrefPane toggleUseSyntaxColoring:] iTMTextStorage expected, found %@", TS);
        {
            // no more UD when the option is project dependant
            BOOL wasSyntaxColoring = [[TS syntaxParser] isColoring];
            if([[[self preferences] objectForKey: iTMUDLogSyntaxColoringKey] boolValue])
            {
                if(!wasSyntaxColoring)
                {
                    [TS replaceSyntaxParser: [iTMLogSyntaxParser syntaxParser]];
                }
            }
            else
            {
                if(wasSyntaxColoring)
                {
                    [TS replaceSyntaxParser: [iTMSyntaxParser syntaxParser]];
                }
            }
            [[TS syntaxParser]fontOrColorDidChange: nil];
            {
                NSRange R = NSMakeRange(0, [TS length]);
                [TS invalidateAttributesInRange: R];
                [[textView layoutManager] invalidateLayoutForCharacterRange: R isSoft: NO actualCharacterRange: nil];
            }
        }
    }
    [textView setNeedsDisplay: YES];
    [tabView selectTabViewItemWithIdentifier: @"Syntax"];// Critical: Here! if not: inconsistent state of sender when undoing
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToggleUseSyntaxColoring:
- (void) validateToggleUseSyntaxColoring: (id) sender;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender setState: ([[self preferences] boolForKey: iTMUDLogSyntaxColoringKey]? NSOnState: NSOffState)];
    [sender setEnabled: YES];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  _ToggleUseTextViewColors:
- (IBAction) _ToggleUseTextViewColors: (id) sender;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender setNextState];
    [self toggleUseTextViewColors: sender];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleUseTextViewColors:
- (IBAction) toggleUseTextViewColors: (id) sender;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self undoManager] registerUndoWithTarget: self
        selector: @selector(_ToggleUseTextViewColors:)
            object: sender];
    [[self preferences] setBool: ([sender state] != NSOffState) forKey: iTMUDLogDrawsTextBackgroundKey];
    [textView setFont: [NSFont fontWithNameSizeDictionary: [[self preferences] objectForKey: iTMUDLogRegularFontKey]]];
    [textView setDrawsBackground: [[self preferences] boolForKey: iTMUDLogDrawsTextBackgroundKey]];
    if([textView drawsBackground])
    {
        [textView setBackgroundColor: [NSColor colorWithRGBADictionary: [[self preferences] objectForKey: iTMUDLogTextBackgroundColorKey]]];
        [textView setInsertionPointColor:
                [NSColor colorWithRGBADictionary: [[self preferences] objectForKey: iTMUDLogInsertionPointColorKey]]];
        [textView setSelectedTextAttributes: [NSDictionary dictionaryWithObjectsAndKeys:
                [NSColor colorWithRGBADictionary: [[self preferences] objectForKey: iTMUDLogSelectedTextColorKey]],
                        NSForegroundColorAttributeName,
                [NSColor colorWithRGBADictionary: [[self preferences] objectForKey: iTMUDLogSelectedTextBackgroundColorKey]], 				NSBackgroundColorAttributeName,
                    nil]];
    }
    else
    {
        [textView setBackgroundColor: [NSColor textBackgroundColor]];
        [textView setInsertionPointColor: [NSColor blackColor]];
        [textView setSelectedTextAttributes: [NSDictionary dictionaryWithObjectsAndKeys:
                [NSColor selectedTextColor], NSForegroundColorAttributeName,
                [NSColor selectedTextBackgroundColor], NSBackgroundColorAttributeName,
                        nil]];
    }
    {
        NSScrollView * SV = [textView enclosingScrollView];
        [SV setDrawsBackground: YES];
        [SV setBackgroundColor: ([textView drawsBackground]? [textView backgroundColor]: [NSColor whiteColor])];
        [SV setNeedsDisplay: YES];
    }
    [tabView selectTabViewItemWithIdentifier: @"Text View"];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToggleUseTextViewColors:
- (void) validateToggleUseTextViewColors: (id) sender;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender setState: ([[self preferences] boolForKey: iTMUDLogDrawsTextBackgroundKey]? NSOnState: NSOffState)];
    [sender setEnabled: YES];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= fontKey
- (NSString *) fontKey;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(NSStringFromSelector(_cmd));
    return _FontKey;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setFontKey:
- (void) setFontKey: (NSString *) argument;
/*"Description forthcoming. Also updates the font manager and the font text field.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSString class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSString expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else if(![_FontKey isEqualToString: argument])
    {
        [_FontKey release];
        _FontKey = [argument retain];
        {
            NSFont * F = [NSFont fontWithNameSizeDictionary: [[self preferences] objectForKey: _FontKey]];
            [[self fontNameTextField] setEditable: YES];
            [[self fontSizeTextField] setEditable: YES];
            if([_FontKey length])
            {
                [[self fontNameTextField] setStringValue: [F displayName]];
                [[self fontSizeTextField] setStringValue: [NSString stringWithFormat: @"%.1f", [F pointSize]]];
            }
            else
            {
                [[self fontNameTextField] setStringValue: [NSString string]];
                [[self fontSizeTextField] setStringValue: [NSString string]];
            }
            [[self fontNameTextField] setEditable: NO];
            [[self fontSizeTextField] setEditable: NO];
            [[NSFontManager sharedFontManager] setSelectedFont: F isMultiple: NO];
        }
    }
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= colorKey
- (NSString *) colorKey;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(NSStringFromSelector(_cmd));
    return _ColorKey;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setColorKey:
- (void) setColorKey: (NSString *) argument;
/*"Description forthcoming. Also updates the color panel and current well colors.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![_ColorKey isEqualToString: argument])
    {
        if(argument && ![argument isKindOfClass: [NSString class]])
            [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSString expected: %@.",
                [self class], NSStringFromSelector(_cmd), argument];
        else
        {
            [_ColorKey release];
            _ColorKey = [argument retain];
            {
                NSColor * C = [NSColor colorWithRGBADictionary: [[self preferences] objectForKey: _ColorKey]];
                [[self undoManager]disableUndoRegistration];
                [[NSColorPanel sharedColorPanel] setColor: C];
                [[self undoManager]enableUndoRegistration];
                [[self currentWell] setColor: C];
            }
        }
    }
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= changeFont:
- (void) changeFont: (id) fontManager;
/*"Final receiver of the message sent by the font panel.
Posts an iTMUserDefaultsDidChangeFontOrColorNotification notification with object its own user default.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSFont * font = [fontManager convertFont: [NSFont fontWithNameSizeDictionary: [[self preferences] objectForKey: [self fontKey]]]];
    [self _ChangeFontTo: font];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= _ChangeFont:
- (void) _ChangeFontTo: (NSFont *) font;
/*"Undo compliant change font.
Posts an iTMUserDefaultsDidChangeFontOrColorNotification notification with object its own user default.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self undoManager] registerUndoWithTarget: self
        selector: @selector(_ChangeFontTo:)
            object: [NSFont fontWithNameSizeDictionary: [[self preferences] objectForKey: [self fontKey]]]];
    [[[self undoManager] prepareWithInvocationTarget: [self currentPopUp]]
                                    selectItemAtIndex: [[self currentPopUp] indexOfSelectedItem]];
    [[self undoManager] registerUndoWithTarget: tabView
        selector: @selector(selectTabViewItem)
            object: [tabView selectedTabViewItem]];
    [[self preferences] setObject: [font nameSizeDictionary] forKey: [self fontKey]];
    if([[self preferences] boolForKey: iTMUDLogSyntaxColoringKey])
    {
        iTMTextStorage * TS = [textView textStorage];
        if([TS isKindOfClass: [iTMTextStorage class]])
        {
            [[TS syntaxParser] fontOrColorDidChange:
                [NSNotification notificationWithName: iTMFontOrColorDidChangeNotification
                    object: self]];
        }
        {
            NSRange range = NSMakeRange(0, [TS length]);
            [TS invalidateAttributesInRange: range];
            [[textView layoutManager] invalidateLayoutForCharacterRange: range isSoft: NO actualCharacterRange: nil];
        }
    }
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= changeColor:
- (void) changeColor: (id) colorPanel;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self _ChangeColorTo: [colorPanel color]];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= _ChangeColorTo:
- (void) _ChangeColorTo: (NSColor *) color;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
#if 0
    [[self undoManager] registerUndoWithTarget: self
        selector: @selector(_ChangeColorTo:)
            object: [NSColor colorWithRGBADictionary: [[self preferences] objectForKey: [self colorKey]]]];
    [[[self undoManager] prepareWithInvocationTarget: [self currentPopUp]]
                                    selectItemAtIndex: [[self currentPopUp] indexOfSelectedItem]];
    [[self undoManager] registerUndoWithTarget: tabView
        selector: @selector(selectTabViewItem)
            object: [tabView selectedTabViewItem]];
#endif
    [[self preferences] setObject: [color RGBADictionary] forKey: [self colorKey]];
    [self validateUserInterface];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateUserInterface
- (void) validateUserInterface;
/*"The only validation concerns the text view for syntax coloring. All other stuff is maintained elsewhere.
Then this message can be sent to the receiver quite safely.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    
    [textView setFont: [NSFont fontWithNameSizeDictionary: [[self preferences] objectForKey: iTMUDLogRegularFontKey]]];
    [textView setDrawsBackground: [[self preferences] boolForKey: iTMUDLogDrawsTextBackgroundKey]];
    [textView setBackgroundColor: [NSColor colorWithRGBADictionary: [[self preferences] objectForKey: iTMUDLogTextBackgroundColorKey]]];
    if([textView drawsBackground])
    {
        [textView setInsertionPointColor:
                [NSColor colorWithRGBADictionary: [[self preferences] objectForKey: iTMUDLogInsertionPointColorKey]]];
        [textView setSelectedTextAttributes: [NSDictionary dictionaryWithObjectsAndKeys:
                [NSColor colorWithRGBADictionary: [[self preferences] objectForKey: iTMUDLogSelectedTextColorKey]],
                        NSForegroundColorAttributeName,
                [NSColor colorWithRGBADictionary: [[self preferences] objectForKey: iTMUDLogSelectedTextBackgroundColorKey]], 				NSBackgroundColorAttributeName,
                    nil]];
    }
    {
        iTMTextStorage * TS = [textView textStorage];
        NSAssert1(!TS || [TS isKindOfClass: [iTMTextStorage class]],
            @"-[iTMLogFontColorPrefPane validateUserInterface:]: iTMTextStorage expected, found: %@", TS);
        {
            // no more UD when the option is project dependant
            BOOL wasSyntaxColoring = [[TS syntaxParser] isColoring];
            if([[self preferences] boolForKey: iTMUDLogSyntaxColoringKey])
            {
                if(!wasSyntaxColoring)
                    [TS replaceSyntaxParser: [iTMLogSyntaxParser syntaxParser]];
            }
            else
            {
                if(wasSyntaxColoring)
                    [TS replaceSyntaxParser: [iTMSyntaxParser syntaxParser]];
            }
        }
        [[TS syntaxParser] fontOrColorDidChange: 
            [NSNotification notificationWithName: iTMFontOrColorDidChangeNotification object: [self preferences]]];
        {
            NSRange R = NSMakeRange(0, [TS length]);
            [TS invalidateAttributesInRange: R];
            [[textView layoutManager] invalidateLayoutForCharacterRange: R isSoft: NO actualCharacterRange: nil];
        }
    }
    [textView setNeedsDisplay: YES];// flush window ?
    if(![[self currentWell]isActive])
        [[self currentWell] setColor: [NSColor colorWithRGBADictionary: [[self preferences] objectForKey: _ColorKey]]];
    [super validateUserInterface];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= tabView:willSelectTabViewItem:
- (void) tabView: (NSTabView *) tabView willSelectTabViewItem: (NSTabViewItem *) tabViewItem;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= tabView:didSelectTabViewItem:
- (void) tabView: (NSTabView *) tabView didSelectTabViewItem: (NSTabViewItem *) tabViewItem;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(NSStringFromSelector(_cmd));
    NS_DURING
    {
        NSDictionary * D = [[[self currentPopUp] selectedItem] representedObject];
        NSAssert([D isKindOfClass: [NSDictionary class]],
            @"-[iTMLogFontColorPrefPane tabView:didSelectTabViewItem:] internal error: dictionary expected");
        [self setFontKey: [D objectForKey: @"fontKey"]];
        [self setColorKey: [D objectForKey: @"colorKey"]];
    }
    NS_HANDLER
    [NSApp reportException: localException];
    [NSApp reportException: [NSException exceptionWithName: NSInternalInconsistencyException
        reason: @"Bad represented object." userInfo: nil]];
    NS_ENDHANDLER
    [[self currentWell] setColor: [NSColor colorWithRGBADictionary: [[self preferences] objectForKey: [self colorKey]]]];
    [self validateUserInterface];
    [[self window]setInitialFirstResponder: [tabViewItem initialFirstResponder]];
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textViewDidChangeSelection:
- (void) textViewDidChangeSelection: (NSNotification *) notification;
/*"The Syntax pop Up button selected item is synchronized with the selected text.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing at first glance.
"*/
{
    if([[[tabView selectedTabViewItem] identifier] isEqualToString: @"Syntax"])
    {
        NSTextView * TV = [notification object];
        iTMTextStorage * TS = [TVtextStorage];
        if([TS isKindOfClass: [iTMTextStorage class]])
        {
            NSRange R;
            int index = [[TS attribute: iTMLogInputAttributeName
                atIndex: [TV selectedRange].location longestEffectiveRange: &R inRange: [TV selectedRange]] intValue];
            if((index<0) ||(index>=[syntaxPopUp numberOfItems]))
                NSLog(@"-[iTMLogFontColorPrefPane textViewDidChangeSelection:] bad index: 0 < %d < %d",
                    index, [syntaxPopUp numberOfItems]);
            else if(R.length == [TV selectedRange].length)
                [syntaxPopUp selectItemAtIndex: index];
            else
                [syntaxPopUp selectItem: nil];
            [self popUpSyntaxAttributes: syntaxPopUp];
        }
    }
//    NSLog(NSStringFromSelector(_cmd));
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  userDefaultsWrittenAndSynchronized
- (void) userDefaultsWrittenAndSynchronized;
/*"Default implementation does nothing. Subclassers will perform additional action to synchronize with objects in the rest of the application.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter]
        postNotificationName: iTMFontOrColorDidChangeNotification
            object: [NSUserDefaults standardUserDefaults]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fontNameTextField
- (NSTextField *) fontNameTextField;
/*"Default implementation does nothing. Subclassers will perform additional action to synchronize with objects in the rest of the application.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[tabView selectedTabViewItem] identifier]isEqualToString: @"Syntax"]?
                    syntaxFontNameTextField: generalFontNameTextField;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fontSizeTextField
- (NSTextField *) fontSizeTextField;
/*"Default implementation does nothing. Subclassers will perform additional action to synchronize with objects in the rest of the application.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[tabView selectedTabViewItem] identifier]isEqualToString: @"Syntax"]?
                    syntaxFontSizeTextField: generalFontSizeTextField;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= currentWell
- (NSColorWell *) currentWell;
/*"The current well is used to see what tab is actually selected.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[tabView selectedTabViewItem] identifier]isEqualToString: @"Syntax"]?
                    syntaxColorWell: textViewColorWell;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= currentPopUp
- (NSPopUpButton *) currentPopUp;
/*"The current well is used to see what tab is actually selected.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[tabView selectedTabViewItem] identifier]isEqualToString: @"Syntax"]?
                    syntaxPopUp: textViewPopUp;
}
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMLogFontColorPrefPane

