/*
//  iTMPreferencePane.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Mar 05 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSDictionary.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/Foundation.h>
#import <AppKit/NSWindowController.h>
#import <AppKit/NSWindow.h>
#import <AppKit/AppKit.h>
#import "NSControl(iTeXMac).h"
#import "iTMPreferencePane.h"
#import "iTMSettings.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMPreferencePane
/*"Description forthcoming."*/
@implementation iTMPreferencePane
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"Designated Initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(self = [super init])
    {
        [self setInitialFirstResponder: nil];
        [self setMainView: nil];
        [self setWindow: nil];
//        [self readUserDefaults];
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Default implementation does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setInitialFirstResponder: nil];
    [self setMainView: nil];
    [self setWindow: nil];
    [self setPreferences: nil];
    [self setUndoManager: nil];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  mainNibName
- (NSString *) mainNibName;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return NSStringFromClass([self class]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  willSelect
- (void) willSelect;
/*"Cleans the preferences...
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setPreferences: nil];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  didSelect
- (void) didSelect;
/*"Calls #{validateUserInterface} and sets the mainView's window delegate to the receiver.
Subclassers will call the inherited method. This is actually for undo operations but subclassers will
find their own benefit.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[[self mainView]window]setDelegate: self];// before: loads the window as side effect
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  willUnselect
- (void) willUnselect;
/*"Default implementation does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  didUnselect
- (void) didUnselect;
/*"Default implementation does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  loadWindow
- (void) loadWindow;
/*"The nib file is searched in the Preferences folder inside the receiver's bundle, then in the resource folder.
If it fails, it tries to find such a nib file in the main bundle.
Subclassers will override this method to perform initialization, before or after the window is loaded.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * path;
    path = [[NSBundle bundleForClass: [self class]] pathForResource: [self mainNibName]
                    ofType: @"nib" inDirectory: @"Preferences"];
    if(![path length])
        path = [[NSBundle bundleForClass: [self class]] pathForResource: [self mainNibName] ofType: @"nib"];
    if(![path length])
        path = [[NSBundle mainBundle] pathForResource: [self mainNibName] ofType: @"nib" inDirectory: @"Preferences"];
    if(![path length])
        path = [[NSBundle mainBundle] pathForResource: [self mainNibName] ofType: @"nib"];
    if(![path length])
        NSLog(@"-[iTMPreferencePane loadWindow]: Unable to find a nib file named %@", [self mainNibName]);
    else
    {
        NSWindowController * WC = [[[NSWindowController alloc]initWithWindowNibPath: path owner: self] autorelease];
        [WC window];
        [self setMainView: [[self window]contentView]];
        [self setWindow: nil];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  window
- (NSWindow *) window;
/*"Default implementation does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _Window;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setWindow:
- (void) setWindow: (NSWindow *) argument;
/*"Default implementation does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSWindow class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSWindow argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    [_Window autorelease];
    _Window = [argument retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  mainView
- (NSView *) mainView;
/*"Lazy initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!_MainView)
        [self loadWindow];
    return _MainView;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMainView:
- (void) setMainView: (NSView *) argument;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSView class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSView argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    [_MainView autorelease];
    _MainView = [argument retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  preferences
- (iTMSettings *) preferences;
/*"Lazy initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _PPUD? _PPUD: (_PPUD = [[iTMSettings alloc] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setPreferences:
- (void) setPreferences: (iTMSettings *) argument;
/*"Default implementation does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [iTMSettings class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] iTMSettings argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    [_PPUD autorelease];
    _PPUD = [argument retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialFirstResponder
- (NSView *) initialFirstResponder;
/*"Default implementation does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _InitialFirstResponder;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setInitialFirstResponder:
- (void) setInitialFirstResponder: (NSView *) argument;
/*"Default implementation does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSView class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSView argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    [_InitialFirstResponder autorelease];
    _InitialFirstResponder = [argument retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  undoManager
- (NSUndoManager *) undoManager;
/*"Lazy initializer and nil if the receiver's #{hasUndoManager} flah is NO.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return (_PPUM? _PPUM: (_PPUM = [[NSUndoManager alloc]init]));
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setUndoManager:
- (void) setUndoManager: (NSUndoManager *) argument;
/*"The undo manager group by events.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSUndoManager class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSUndoManager argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    [_PPUM autorelease];
    _PPUM = [argument retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  hasUndoManager
- (BOOL) hasUndoManager;
/*"Lazy initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _HUM;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setHasUndoManager:
- (void) setHasUndoManager: (BOOL) argument;
/*"Default implementation does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _HUM = argument;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowWillReturnUndoManager:
- (NSUndoManager *) windowWillReturnUndoManager: (NSWindow *) window;
/*"When selected, the receiver is the delegate of the preference window,
and is the owner of the undo manager.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [self undoManager];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateUserInterface
- (void) validateUserInterface;
/*"Sends a #{validateUserInterfaceItems} message to the window contents view
of the window actually containing the receiver's main view. The defaults controller counts on this to update the contents!!!
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[[[self mainView]window]contentView] validateUserInterfaceItems];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  writeAndSynchronizeUserDefaults
- (void) writeAndSynchronizeUserDefaults;
/*"All the objects in the receiver's user default persistent domain are stored in the standard user default.
No clever managements using the different domains. No verification of the class of the objects.
Synchronizes the standard user defaults.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSDictionary * D = [[self preferences] dictionaryRepresentation];
    NSEnumerator * E = [D keyEnumerator];
    NSString * key;
    while(key = [E nextObject])
        if([key isKindOfClass: [NSString class]] && [key length])
        {
            id object = [DobjectForKey: key];
            if(object)
                [[NSUserDefaults standardUserDefaults] setObject: object forKey: key];
            else
                NSLog(@"-[iTMPreferencePane writeAndSynchronizeUserDefaults:] No preferences object for key: %@", key);
        }
        else
            NSLog(@"-[iTMPreferencePane writeAndSynchronizeUserDefaults:] Bad key object: %@", key);
    [[NSUserDefaults standardUserDefaults]synchronize];
    [self userDefaultsWrittenAndSynchronized];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  userDefaultsWrittenAndSynchronized
- (void) userDefaultsWrittenAndSynchronized;
/*"Default implementation just discards the locally stored preferences and validates the user interface,
causing the apply and revert buttons to be dismmed.
Subclassers will perform additional action to synchronize with objects in the rest of the application.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setPreferences: nil];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  discardAnyChange
- (void) discardAnyChange;
/*"Cleans the locally stored user preferences of the receiver and validates the user interface.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setPreferences: nil];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  canApplyChangeNow
- (BOOL) canApplyChangeNow;  
/*"Subclassers will return YES or NO to enable disable the Apply button of the defaults controller window.
Default implementation returns YES according to the number of keys really used in the preferences.
No such keys means that no standard user default has been overriden.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[[self preferences] dictionaryRepresentation] allKeys] count]!= 0;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  canRevertChangeNow
- (BOOL) canRevertChangeNow;  
/*"Subclassers will return YES or NO to enable disable the Apply button of the defaults controller window.
Default implementation just return #{canApplyChangeNow}.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [self canApplyChangeNow];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMenuItem:
- (BOOL) validateMenuItem: (id <NSMenuItem>) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    return NO;
}
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMPreferencePane
