/*
//  iTMViewerPrefPane.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Fri Mar 15 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSAttributedString.h>
#import <Foundation/Foundation.h>
#import <AppKit/NSTabViewItem.h>
#import <AppKit/NSWindow.h>
#import <AppKit/AppKit.h>
#import "iTMViewerPrefPane.h"
#import "iTMRepWindowController.h"
#import "iTMDocumentController.h"
#import "iTMRepAlbumView.h"
#import "iTMPDFAlbumView.h"
#import "iTMPDFView.h"
#import "iTMRepDocument.h"
#import "iTMMagnificationFormatter.h"
#import "iTMSettings.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMViewerPrefPane
/*"Description forthcoming."*/
@implementation iTMViewerPrefPane
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize:
+ (void) initialize;
/*"Default implementation does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [iTMRepWindowController initialize];
    [iTMDocumentController initialize];
    [iTMRepAlbumView initialize];
    [iTMPDFView initialize];
    [iTMPDFAlbumView initialize];
    [iTMRepDocument initialize];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"Designated Initializer.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    if(self = [super init])
    {
        [_ExternalUsed autorelease];
        _ExternalUsed = nil;
        [_BuiltInUsed autorelease];
        _BuiltInUsed = nil;
        [_CurrentFormat autorelease];
        _CurrentFormat = nil;
        _ViewerHasChanged = YES;
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Default implementation does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [_ExternalUsed autorelease];
    _ExternalUsed = nil;
    [_BuiltInUsed autorelease];
    _BuiltInUsed = nil;
    [_CurrentFormat autorelease];
    _CurrentFormat = nil;
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  chooseHelper:
- (IBAction) chooseHelper: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    NSArray * ApplicationDirectories = NSSearchPathForDirectoriesInDomains(NSApplicationDirectory, NSLocalDomainMask, YES);
    if([ApplicationDirectories count])
    {
        NSOpenPanel * OP = [NSOpenPanel openPanel];
        [OP setAllowsMultipleSelection: NO];
        [OP setCanChooseFiles: NO];
        [OP setCanChooseDirectories: YES];
        [OP setResolvesAliases: YES];
        [OP setTreatsFilePackagesAsDirectories: NO];
        if(NSOKButton == [OP runModalForDirectory: [ApplicationDirectories objectAtIndex: 0]
                                    file: nil types: [NSArray arrayWithObject: @"app"]] && [[OP filenames] count])
        {
            [[self preferences] setObject: [[OP filenames] objectAtIndex: 0] forKey: iTMUDPDFHelperKey];
        }
    }
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  helperFieldEdited:
- (IBAction) helperFieldEdited: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setObject: [sender stringValue] forKey: iTMUDPDFHelperKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateHelperFieldEdited:
- (void) validateHelperFieldEdited: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [sender setStringValue: [[self preferences] stringForKey: iTMUDPDFHelperKey]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  magnificationCurrent:
- (IBAction) magnificationCurrent: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setObject:
                [[NSUserDefaults standardUserDefaults] objectForKey: [iTMPDFAlbumView currentMagnificationKey]]
                            forKey: iTMUDPDFFixedMagnificationKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  magnificationFieldEdited:
- (IBAction) magnificationFieldEdited: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setObject: [sender objectValue] forKey: iTMUDPDFFixedMagnificationKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  popUpMagnification:
- (IBAction) popUpMagnification: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setInteger: [[sender selectedItem] tag] forKey: iTMUDPDFAlbumDisplayModeKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  popUpStickMagnification:
- (IBAction) popUpStickMagnification: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setInteger: [[sender selectedItem] tag] forKey: iTMUDPDFAlbumStickModeKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  popUpNewPDFPageMode:
- (IBAction) popUpNewPDFPageMode: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setInteger: [[sender selectedItem] tag] forKey: iTMUDNewPDFPageModeKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleAutoUpdate:
- (IBAction) toggleAutoUpdate: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setBool: ([sender state] != NSOffState) forKey: iTMUDAutoUpdatesViewKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleBackgroundPDFRendering:
- (IBAction) toggleBackgroundPDFRendering: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setBool: ([sender state] != NSOffState) forKey: iTMUDThreadedPDFRenderingKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleCompareSize:
- (IBAction) toggleCompareSize: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setBool: ([sender state] != NSOffState) forKey: iTMUDSSetUpPageWhenBadPaperSizeKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleHelper:
- (IBAction) toggleHelper: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setBool: ([sender state] != NSOffState) forKey: iTMUDUsesPDFHelperKey];
    _ViewerHasChanged = YES;
    [self validateUserInterface];
    _ViewerHasChanged = NO;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  togglePreferA4:
- (IBAction) togglePreferA4: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setBool: ([sender state] != NSOffState) forKey: iTMUDPreferA4PaperKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleSmartUpdate:
- (IBAction) toggleSmartUpdate: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [[self preferences] setBool: ([sender state] != NSOffState) forKey: iTMUDSmartUpdatesViewKey];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  externalCommentField:
- (IBAction) externalCommentField: (id) sender;
/*"Fake action used only for validation.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateExternalCommentField:
- (IBAction) validateExternalCommentField: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    if(_ViewerHasChanged)
    {
        if(![_ExternalUsed length])
        {
            NSAttributedString * AS = [sender attributedStringValue];
            NSDictionary * attributes = [AS length]? [AS attributesAtIndex: 0 effectiveRange: nil]: nil;
            _ExternalUsed = [[NSAttributedString alloc]
                initWithString: NSLocalizedStringFromTableInBundle(
                    @"Option key to use the built in viewer",
                        @"Preferences", [NSBundle bundleForClass: [self class]],
                            "title of the info field in the external tab view of the viewer pref pane")
            	attributes: attributes];
        }
        if(![_BuiltInUsed length])
        {
            NSAttributedString * AS = [sender attributedStringValue];
            NSDictionary * attributes = [AS length]? [AS attributesAtIndex: 0 effectiveRange: nil]: nil;
            _BuiltInUsed = [[NSAttributedString alloc]
                initWithString: NSLocalizedStringFromTableInBundle(
                    @"Option key to use the external viewer",
                        @"Preferences", [NSBundle bundleForClass: [self class]],
                            "title of the info field in the external tab view of the viewer pref pane")
            	attributes: attributes];
        }
        [sender setAttributedStringValue:
            ([[self preferences] boolForKey: iTMUDUsesPDFHelperKey]? _ExternalUsed: _BuiltInUsed)];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMagnificationCurrent:
- (void) validateMagnificationCurrent: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    if(![_CurrentFormat length])
        _CurrentFormat = [NSLocalizedStringFromTableInBundle(@"Current %.0f %%", @"Preferences",
            [NSBundle bundleForClass: [self class]],
                "format of the current magnification button title in the Viewer preferences pane") retain];
    [sender setTitle: [NSString stringWithFormat: _CurrentFormat,
            [[NSUserDefaults standardUserDefaults] floatForKey: [iTMPDFAlbumView currentMagnificationKey]]* 100]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMagnificationFieldEdited:
- (void) validateMagnificationFieldEdited: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    if(![sender formatter])
        [sender setFormatter: [[[iTMMagnificationFormatter alloc]init]autorelease]];
    [sender setFloatValue: [[self preferences] floatForKey: iTMUDPDFFixedMagnificationKey]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validatePopUpMagnification:
- (void) validatePopUpMagnification: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [sender selectItemAtIndex: [sender indexOfItemWithTag: [[self preferences] integerForKey: iTMUDPDFAlbumDisplayModeKey]]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validatePopUpStickMagnification:
- (void) validatePopUpStickMagnification: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [sender selectItemAtIndex: [sender indexOfItemWithTag: [[self preferences] integerForKey: iTMUDPDFAlbumStickModeKey]]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validatePopUpNewPDFPageMode:
- (void) validatePopUpNewPDFPageMode: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [sender selectItemAtIndex: [sender indexOfItemWithTag: [[self preferences] integerForKey: iTMUDNewPDFPageModeKey]]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToggleAutoUpdate:
- (void) validateToggleAutoUpdate: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [sender setState: ([[self preferences] boolForKey: iTMUDAutoUpdatesViewKey]? NSOnState: NSOffState)];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToggleBackgroundPDFRendering:
- (void) validateToggleBackgroundPDFRendering: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [sender setState: ([[self preferences] boolForKey: iTMUDThreadedPDFRenderingKey]? NSOnState: NSOffState)];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToggleCompareSize:
- (void) validateToggleCompareSize: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [sender setState: ([[self preferences] boolForKey: iTMUDSSetUpPageWhenBadPaperSizeKey]? NSOnState: NSOffState)];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToggleHelper:
- (void) validateToggleHelper: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [sender setState: ([[self preferences] boolForKey: iTMUDUsesPDFHelperKey]? NSOnState: NSOffState)];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateTogglePreferA4:
- (void) validateTogglePreferA4: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [sender setState: ([[self preferences] boolForKey: iTMUDPreferA4PaperKey]? NSOnState: NSOffState)];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToggleSmartUpdate:
- (void) validateToggleSmartUpdate: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [sender setState: ([[self preferences] boolForKey: iTMUDSmartUpdatesViewKey]? NSOnState: NSOffState)];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= tabView:didSelectTabViewItem:
- (void) tabView: (NSTabView *) tabView didSelectTabViewItem: (NSTabViewItem *) tabViewItem;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 10/22/2001
To Do List:
"*/
{
    [self validateUserInterface];
    [[self window]makeFirstResponder: [tabViewItem initialFirstResponder]];
    return;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMViewerPrefPane

