/*
//  iTMWindowsPrefPane.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Fri Mar 15 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/Foundation.h>
#import <AppKit/NSTabViewItem.h>
#import <AppKit/NSWindow.h>
#import <AppKit/AppKit.h>
#import "NSWindow(iTeXMac).h"
#import "iTMWindowsPrefPane.h"
#import "iTMTeXWindow.h"
#import "iTMProjectWindow.h"
#import "iTMRepWindow.h"
#import "iTMPDFWindow.h"
#import "iTMTeXWindowController.h"
#import "iTMProjectWindowController.h"
#import "iTMRepWindowController.h"
#import "iTMPDFWindowController.h"
#import "iTMRepAlbumView.h"
#import "iTMPDFAlbumView.h"
#import "iTMSettings.h"
#if 0
#import "iTMDocumentController.h"
#import "iTMRepDocument.h"
#endif
#import "iTMMagnificationFormatter.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMWindowsPrefPane
/*"Description forthcoming."*/
@implementation iTMWindowsPrefPane
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize:
+ (void) initialize;
/*"Default implementation does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/22/2002
To Do List:
"*/
{
    [super initialize];
    [iTMRepWindow initialize];
    [iTMPDFWindow initialize];
    [iTMProjectWindow initialize];
    [iTMTeXWindow initialize];
    [iTMRepAlbumView initialize];
    [iTMPDFAlbumView initialize];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"Designated Initializer.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    if(self = [super init])
    {
        [_LoadFormat autorelease];
        _LoadFormat = nil;
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Default implementation does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/10/2002
To Do List:
"*/
{
    [_LoadFormat autorelease];
    _LoadFormat = nil;
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  TeXLoad:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  projectLoad:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  PDFLoad:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  repLoad:
#define LOAD(selector, validateSelector, key, currentKey)\
- (IBAction) selector: (id) sender;\
{\
    NSString * S = [[NSUserDefaults standardUserDefaults] stringForKey:\
        [NSString stringWithFormat: @"NSWindow Frame %@", currentKey]];\
    [[self preferences] setObject: S forKey: [NSString stringWithFormat: @"NSWindow Frame %@", key]];\
    [self validateUserInterface];\
    return;\
}\
- (void) validateSelector: (id) sender;\
{\
    NSString * S = [[NSUserDefaults standardUserDefaults] stringForKey:\
        [NSString stringWithFormat: @"NSWindow Frame %@", currentKey]];\
    [sender setEnabled: ([S length] > 0)];\
    return;\
}
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/22/2002
To Do List:
"*/
LOAD(TeXLoad, validateTeXLoad, [iTMTeXWindow frameAutosaveIdentifierForMode: iTMWindowFrameFixedMode], [iTMTeXWindow frameAutosaveIdentifierForMode: iTMWindowFrameCurrentMode]);
LOAD(projectLoad, validateProjectLoad, [iTMProjectWindow frameAutosaveIdentifierForMode: iTMWindowFrameFixedMode], [iTMProjectWindow frameAutosaveIdentifierForMode: iTMWindowFrameCurrentMode]);
LOAD(PDFLoad, validatePDFLoad, [iTMPDFWindow frameAutosaveIdentifierForMode: iTMWindowFrameFixedMode], [iTMPDFWindow frameAutosaveIdentifierForMode: iTMWindowFrameCurrentMode]);
LOAD(repLoad, validateRepLoad, [iTMRepWindow frameAutosaveIdentifierForMode: iTMWindowFrameFixedMode], [iTMRepWindow frameAutosaveIdentifierForMode: iTMWindowFrameCurrentMode]);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  PDFMagLoad:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  repMagLoad:
#define MAGLOAD(selector, validateSelector, windowClass)\
- (IBAction) selector: (id) sender;\
{\
    id O = [[NSUserDefaults standardUserDefaults] objectForKey: [windowClass currentMagnificationKey]];\
    [[self preferences] setObject: O forKey: [windowClass currentMagnificationKey]];\
    [self validateUserInterface];\
    return;\
}\
- (void) validateSelector: (id) sender;\
{\
    if(![_LoadFormat length])\
	_LoadFormat = [NSLocalizedStringFromTableInBundle(@"Current %.0f %%", @"Preferences",\
            [NSBundle bundleForClass: [self class]],\
                "format of the current magnification button title in the Viewer preferences pane") retain];\
    [sender setTitle: [NSString stringWithFormat: _LoadFormat,\
            [[NSUserDefaults standardUserDefaults] floatForKey: [windowClass currentMagnificationKey]]* 100]];\
    return;\
}
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/22/2002
To Do List:
"*/
MAGLOAD(PDFMagLoad, validatePDFMagLoad, iTMPDFAlbumView);
MAGLOAD(repMagLoad, validateRepMagLoad, iTMRepAlbumView);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  TeXRadio:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  projectRadio:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  PDFRadio:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  repRadio:
#define RADIO(selector, validateSelector, key)\
- (IBAction) selector: (id) sender;\
{\
    [[self preferences] setInteger: [[sender selectedCell] tag] forKey: key];\
    [self validateUserInterface];\
    return;\
}\
- (void) validateSelector: (id) sender;\
{\
    [sender selectCellWithTag: [[self preferences] integerForKey: key]];\
    return;\
}
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/22/2002
To Do List:
"*/
RADIO(TeXRadio, validateTeXRadio, [iTMTeXWindow frameAutosaveModeKey]);
RADIO(projectRadio, validateProjectRadio, [iTMProjectWindow frameAutosaveModeKey]);
RADIO(PDFRadio, validatePDFRadio, [iTMPDFWindow frameAutosaveModeKey]);
RADIO(repRadio, validateRepRadio, [iTMRepWindow frameAutosaveModeKey]);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  PDFMagPopUp:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  repMagPopUp:
#define POPUP(selector, validateSelector, key)\
- (IBAction) selector: (id) sender;\
{\
    [[self preferences] setInteger: [[sender selectedItem] tag] forKey: key];\
    [self validateUserInterface];\
    return;\
}\
- (void) validateSelector: (id) sender;\
{\
    [sender selectItemAtIndex: [sender indexOfItemWithTag: [[self preferences] integerForKey: key]]];\
    return;\
}
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/22/2002
To Do List:
"*/
POPUP(PDFMagPopUp, validatePDFMagPopUp, iTMUDPDFAlbumDisplayModeKey);
POPUP(repMagPopUp, validateRepMagPopUp, iTMUDRepAlbumDisplayModeKey);
POPUP(PDFStickPopUp, validatePDFStickPopUp, iTMUDPDFAlbumStickModeKey);
POPUP(repStickPopUp, validateRepStickPopUp, iTMUDRepAlbumStickModeKey);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  TeXShow:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  projectShow:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  PDFShow:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  repShow:
#define SHOW(selector, validateSelector, currentKey)\
- (IBAction) selector: (id) sender;\
{\
    return;\
}\
- (void) validateSelector: (id) sender;\
{\
    NSString * S = [[NSUserDefaults standardUserDefaults] stringForKey:\
        [NSString stringWithFormat: @"NSWindow Frame %@", currentKey]];\
    [sender setEnabled: ([S length] > 0)];\
    [sender setEnabled: NO];\
    return;\
}
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/22/2002
To Do List: Complete the implementation
"*/
SHOW(TeXShow, validateTeXShow, [iTMTeXWindow frameAutosaveIdentifierForMode: iTMWindowFrameCurrentMode]);
SHOW(projectShow, validateProjectShow, [iTMProjectWindow frameAutosaveIdentifierForMode: iTMWindowFrameCurrentMode]);
SHOW(PDFShow, validatePDFShow, [iTMPDFWindow frameAutosaveIdentifierForMode: iTMWindowFrameCurrentMode]);
SHOW(repShow, validateRepShow, [iTMRepWindow frameAutosaveIdentifierForMode: iTMWindowFrameCurrentMode]);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  TeXToggle:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  projectToggle:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  PDFToggle:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  repToggle:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  PDFMagToggle:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  repMagToggle:
#define TOGGLE(selector, validateSelector, key)\
- (IBAction) selector: (id) sender;\
{\
    [[self preferences] setBool: ([sender state] != NSOffState) forKey: key];\
    [self validateUserInterface];\
    return;\
}\
- (void) validateSelector: (id) sender;\
{\
    [sender setState: ([[self preferences] boolForKey: key]? NSOnState: NSOffState)];\
    return;\
}
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/22/2002
To Do List:
"*/
TOGGLE(TeXToggle, validateTeXToggle, iTMUDTeXCascadeWindowsKey);
TOGGLE(projectToggle, validateProjectToggle, iTMUDProjectCascadeWindowsKey);
TOGGLE(PDFToggle, validatePDFToggle, iTMUDPDFCascadeWindowsKey);
TOGGLE(repToggle, validateRepToggle, iTMUDRepCascadeWindowsKey);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  magnificationField:
#define FIELD(selector, validateSelector, key)\
- (IBAction) selector: (id) sender;\
{\
    [[self preferences] setFloat: [sender floatValue]/100 forKey: key];\
    [self validateUserInterface];\
    return;\
}\
- (void) validateSelector: (id) sender;\
{\
    if(![sender formatter])\
        [sender setFormatter: [[[iTMMagnificationFormatter alloc]init]autorelease]];\
    [sender setFloatValue: [[self preferences] floatForKey: key]];\
    return;\
}
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a6: 03/22/2002
To Do List:
"*/
FIELD(PDFMagField, validatePDFMagField, iTMUDPDFFixedMagnificationKey);
FIELD(repMagField, validateRepMagField, iTMUDRepFixedMagnificationKey);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= tabView:didSelectTabViewItem:
- (void) tabView: (NSTabView *) tabView didSelectTabViewItem: (NSTabViewItem *) tabViewItem;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr 
- 1.1.a6: 03/22/2002
To Do List:
"*/
{
    [[self window]makeFirstResponder: [tabViewItem initialFirstResponder]];
    [self validateUserInterface];
    return;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMWindowsPrefPane

