/*
//  iTMProjectDocument.h
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Sep 11 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>

extern NSString * const iTMProjectDocumentType;
extern NSString * const iTMPDProjectKey;

@class iTMProjectsManager, iTMProjectWindowController, iTMTerminalController;

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMProjectDocument

@interface iTMProjectDocument : NSDocument
{
@private
    iTMTerminalController * _Terminal;
    NSMutableDictionary * _ProjectDictionary;
    iTMProjectWindowController * _WC;
    NSString * _AbsoluteInput;
    NSString * _RelativeInput;
    NSString * _ProjectName;
    BOOL _IsAbsolute;
//    BOOL _IsGeneric;
    BOOL _IsVoid;
    NSString * _CEP;
}
/*"Class methods"*/
/*"Setters and Getters"*/
- (BOOL) isVoid;
- (BOOL) isGeneric;
- (BOOL) isStrongGeneric;
- (BOOL) canShowFiles;
- (BOOL) canShowErrors;
- (BOOL) canBibTeX;
- (BOOL) canMakeIndex;
//- (BOOL) canShowTerminal: (id) sender;
- (void) setCompileScript: (NSString *) argument;
- (void) setTypesetScript: (NSString *) argument;
- (id) windowController;
- (void) setWindowController: (iTMProjectWindowController *) argument;
- (id) currentErrorPath;
- (void) setCurrentErrorPath: (id) argument;
/*"Main methods"*/
- (void) becomeGeneric;
- (void) becomeNormal;
//- (void) createVoidToolbarViewItem;
//- (void) createTabViewItemWithSource: (id) aSource identifier: (id) identifier;
//- (void) fixWindowResize;
- (void) terminalStop: (id) irrelevant;
- (void) terminalEscape: (id) irrelevant;
- (void) terminalExit: (id) irrelevant;
- (void) terminalQuiet: (id) irrelevant;
- (void) terminalIgnore: (id) irrelevant;
- (void) showPreviousError: (id) sender;
- (void) showNextError: (id) sender;
- (void) showSourceErrorForLogRange: (NSRange) aRange;
- (void) scrollToPreviousError: (id) sender;
- (void) scrollToNextError: (id) sender;
- (void) terminalRestart: (id) sender;
- (BOOL) canStopTerminal;
- (void) showProject: (id) sender;
- (IBAction) showProjectWindow: (id) sender;
- (void) showTerminalInBackGround;
- (void) showErrors: (id) sender;
- (void) errorFileChosen: (NSPopUpButton *) sender;
- (void) validateErrorFileChosen: (NSPopUpButton *) sender;
//- (void) tabView: (NSTabView *) tabView willSelectTabViewItem: (NSTabViewItem *) tabViewItem;
//- (void) tabView: (NSTabView *) tabView didSelectTabViewItem: (NSTabViewItem *) tabViewItem;
- (void) validateUserInterface;
//- (void) validateTypesetTextView;
//- (void) validateCompileTextView;
- (void) reallyMakeWindowControllers;
- (void) makeWindows;
- (unsigned int) comboBox: (NSComboBox *) aCB indexOfItemWithStringValue: (NSString *) aString;
//- (NSString *) comboBox: (NSComboBox *) aCB completedString: (NSString *) aString;
- (id) comboBox: (NSComboBox *) aCB objectValueForItemAtIndex: (int) index;
- (int) numberOfItemsInComboBox: (NSComboBox *) aCB;
/*"Overriden methods"*/
@end

@interface iTMProjectDocument(Helper)
- (void) setOutputIdentifier: (NSString *) argument;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMProjectDocument
