/*
//  iTMProjectWindowController.h
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Nov 27 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

//#import <Foundation/NSString.h>
#import <AppKit/NSWindowController.h>
#import "NSResponder(iTeXMac).h"

@class NSString;
extern NSString * const iTMPWCProjectIdentifier;
extern NSString * const iTMPWCTerminalIdentifier;
extern NSString * const iTMPWCFilesIdentifier;
extern NSString * const iTMPWCErrorsIdentifier;
extern NSString * const iTMPWCInputIdentifier;
extern NSString * const iTMPWCCompileIdentifier;
extern NSString * const iTMPWCCustomCompileIdentifier;
extern NSString * const iTMPWCBuiltInCompileIdentifier;
extern NSString * const iTMPWCTypesetIdentifier;
extern NSString * const iTMPWCCustomTypesetIdentifier;
extern NSString * const iTMPWCBuiltInTypesetIdentifier;
extern NSString * const iTMPWCHelpersIdentifier;
extern NSString * const iTMPWCMakeIndexIdentifier;
extern NSString * const iTMPWCBibTeXIdentifier;
extern NSString * const iTMPWCRenderIdentifier;
extern NSString * const iTMPWCMacrosIdentifier;

extern NSString * const iTMUDLogDrawsTextBackgroundKey;
extern NSString * const iTMUDLogSyntaxColoringKey;

extern NSString * const iTMUDLogTextBackgroundColorKey;
extern NSString * const iTMUDLogSelectedTextBackgroundColorKey;
extern NSString * const iTMUDLogSelectedTextColorKey;
extern NSString * const iTMUDLogInsertionPointColorKey;

extern NSString * const iTMUDLogStandardFontKey;
extern NSString * const iTMUDLogStandardColorKey;

extern NSString * const iTMUDProjectCascadeWindowsKey;

@class NSView, NSTabView, NSTextView, NSTextField, NSPopUpButton;
 
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMProjectWindowController

@interface iTMProjectWindowController : NSWindowController
{
@private
    //outlets
    NSView * _FileView;
    NSView * _CustomCompileView;
    NSView * _BuiltInCompileView;
    NSView * _CustomTypesetView;
    NSView * _BuiltInTypesetView;
    NSView * _HelpersView;
    NSView * _TerminalView;
    NSView * _ProjectView;
    NSView * _FilesView;
    NSView * _ErrorsView;
    NSView * _MacrosView;
    NSView * _RenderView;
    NSTabView * _ProjectTabView;
    NSTabView * _MainTabView;
    NSTabView * _HelpersTabView;
    NSTextView * _TerminalOutput;
    NSTextField * _TerminalInput;
    NSTextView * _TypesetTextView;
    NSTextView * _ErrorsTextView;
    NSPopUpButton * _ErrorsPopUp;
    //not outlets
    id _TD;
    NSString * _CurrentCompileIdentifier;
    NSString * _CurrentTypesetIdentifier;
    NSString * _CurrentHelperIdentifier;
}
/*"Class methods"*/
/*"Setters and Getters"*/
- (id) toolbarDelegate;
- (void) setToolbarDelegate: (id) argument;
- (NSView *) fileView;
- (NSView *) customCompileView;
- (NSView *) builtInCompileView;
- (NSView *) customTypesetView;
- (NSView *) builtInTypesetView;
- (NSView *) helpersView;
- (NSView *) terminalView;
- (NSView *) projectView;
- (NSView *) filesView;
- (NSView *) errorsView;
- (NSView *) macrosView;
- (NSView *) renderView;
- (NSTabView *) mainTabView;
- (NSTabView *) helpersTabView;
- (NSTabView *) projectTabView;
- (NSPopUpButton *) errorsPopUp;
- (NSTextView *) typesetTextView;
- (NSTextView *) errorsTextView;
- (NSTextView *) terminalOutput;
- (NSTextField *) terminalInput;
- (void) setErrorsPopUp: (NSPopUpButton *) argument;
- (void) setTypesetTextView: (NSTextView *) argument;
- (void) setErrorsTextView: (NSTextView *) argument;
- (void) setTerminalOutput: (NSTextView *) argument;
- (void) setTerminalInput: (NSTextField *) argument;
- (void) setFileView: (NSView *) argument;
- (void) setCustomCompileView: (NSView *) argument;
- (void) setBuiltInCompileView: (NSView *) argument;
- (void) setCustomTypesetView: (NSView *) argument;
- (void) setBuiltInTypesetView: (NSView *) argument;
- (void) setHelpersView: (NSView *) argument;
- (void) setTerminalView: (NSView *) argument;
- (void) setProjectView: (NSView *) argument;
- (void) setFilesView: (NSView *) argument;
- (void) setErrorsView: (NSView *) argument;
- (void) setMainTabView: (NSTabView *) argument;
- (void) setHelpersTabView: (NSTabView *) argument;
- (void) setProjectTabView: (NSTabView *) argument;
- (void) setMacrosView: (NSTabView *) argument;
- (void) setRenderView: (NSTabView *) argument;
/*"Main methods"*/
- (void) cleanOutlets;
- (IBAction) switchToPaneWithIdentifier: (NSString *) anIdentifier;
- (BOOL) alreadySwitchedToPaneWithIdentifier: (NSString *) anIdentifier;
- (void) fontOrColorDidChangeNotified: (NSNotification *) aNotification;
/*"Overriden methods"*/
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMProjectWindowController
