/*
//  iTMTeXEngineManager.h
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Oct 02 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSObject.h>

@class NSString;

extern NSString * const iTMEnginesKey;
extern NSString * const iTMFormatsKey;
extern NSString * const iTMInputKey;
extern NSString * const iTMOutputsKey;
extern NSString * const iTMMLTeXKey;
extern NSString * const iTMTranslateKey;
extern NSString * const iTMInteractionKey;

@class NSNotification, NSComboBox;

typedef enum _iTMEngineIOFileFormat 
{
    kiTMEngineIOTeXFormat = 0,
    kiTMEngineIODVIFormat = 1,
    kiTMEngineIOPSFormat = 2,
    kiTMEngineIOPDFFormat = 3
} iTMTypesetEnvironmentIndex;

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTeXEngineManager

@interface iTMTeXEngineManager: NSObject
{
@private
    id _Engines;
}
/*"Class methods"*/
+ (id) sharedTeXEngineManager;
/*"Setters and Getters"*/
- (NSArray *) engines;
- (NSDictionary *) engineForKey: (NSString *) anEngine;
- (NSArray *) formatsForEngine: (NSString *) anEngine;
- (NSArray *) outputsForEngine: (NSString *) anEngine;
- (int) inputForEngine: (NSString *) anEngine;
- (NSString *) pathForEngine: (NSString *) anEngine;
- (BOOL) engineHasTeXOutput: (NSString *) anEngine;
- (BOOL) engineHasDVIOutput: (NSString *) anEngine;
- (BOOL) engineHasPSOutput: (NSString *) anEngine;
- (BOOL) engineHasPDFOutput: (NSString *) anEngine;
- (BOOL) engineUsesInteraction: (NSString *) anEngine;
- (BOOL) engineUsesTranslate: (NSString *) anEngine;
- (BOOL) engineUsesMLTeX: (NSString *) anEngine;
/*"Main methods"*/
/*"Overriden methods"*/
- (id) init;
- (void) dealloc;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTeXEngineManager

