 /*
//  iTMProjectDocument(Accessors).m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Oct 02 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSUndoManager.h>
#import <Foundation/NSScanner.h>
#import <AppKit/NSView.h>
#import <AppKit/NSSavePanel.h>
#import "NSString(iTeXMac).h"
#import "iTMProjectDocument(Accessors).h"
#import "iTMProjectDocument.h"
#import "iTMProjectsManager.h"
#import "iTMProjectWindowController.h"
#import "iTMTerminalController.h"
#import "iTMTerminalController(UI).h"

/*"Keys prefixed with "iTMPD" are used as environment keys. Only some ef them are used to store information in the project dictionary: some infos may be redundant."*/
NSString * const iTMPDVersionKey = @"version";

NSString * const iTMPDRootKey = @"root";
NSString * const iTMPDUseCustomIdentifierKey = @"Use Custom Identifier";
NSString * const iTMPDIsGenericKey = @"Is Generic Project";

NSString * const iTMPDDirectoryKey = @"iTMDirectory";
NSString * const iTMPDRootInputKey = @"iTMInput";
NSString * const iTMPDRootOutputKey = @"iTMOutput";
NSString * const iTMPDInputExtensionKey = @"iTMInputExtension";
NSString * const iTMPDOutputExtensionKey = @"iTMOutputExtension";
NSString * const iTMPDCustomIdentifierKey = @"iTMCustomIdentifier";

NSString * const iTMPDKeepsDVIKey = @"iTMCompileKeepsDVI";
NSString * const iTMPDKeepsPSKey = @"iTMCompileKeepsPS";
NSString * const iTMPDMLTeXKey = @"iTMCompileMLTeX";
NSString * const iTMPDInteractionKey = @"iTMCompileInteraction";

NSString * const iTMPDCompileEngineKey = @"iTMCompileEngine";
NSString * const iTMPDCompileFormatKey = @"iTMCompileFormat";
NSString * const iTMPDCompileBaseKey = @"iTMCompileBase";

NSString * const iTMPDCompileSilentlyKey = @"iTMCompileSilently";
NSString * const iTMPDUseCustomCompileKey = @"iTMUseCustomCompile";
NSString * const iTMPDCompileScriptKey = @"iTMCompile";

NSString * const iTMPDTypesetSilentlyKey = @"iTMTypesetSilently";
NSString * const iTMPDUseCustomTypesetKey = @"iTMUseCustomTypeset";
NSString * const iTMPDTypesetScriptKey = @"iTMTypeset";

NSString * const iTMPDBibTeXScriptKey = @"iTMBibTeX";
NSString * const iTMPDBibTeXSilentlyKey = @"iTMBibTeXSilently";
NSString * const iTMPDBMinXReferencesKey = @"iTMBibTeXMinXReferences";

NSString * const iTMPDIndexScriptKey = @"iTMMakeIndex";
NSString * const iTMPDIndexSilentlyKey = @"iTMIndexSilently";//default NO, YES = -q
NSString * const iTMPDICompressBlankKey = @"iTMIndexCompressBlank";//default NO, YES = -c
NSString * const iTMPDIGermanOrderingKey = @"iTMIndexGermanOrdering";//default NO, YES = -g
NSString * const iTMPDILetterOrderingKey = @"iTMIndexLetterOrdering";//default NO, YES = -l
NSString * const iTMPDINoImplicitRangeKey = @"iTMIndexNoImplicitRange";//default NO, YES = -r
NSString * const iTMPDIndexOutputKey = @"iTMIndexOutput";//default @"" for ${root}, -o value
NSString * const iTMPDIndexStartingKey = @"iTMIndexStarting";//default @"any", -p [any|odd|even], or page number
NSString * const iTMPDIndexStyleKey = @"iTMIndexStyle";//default @"", content: -s ${indexStyle}
NSString * const iTMPDUseIndexStyleKey = @"iTMUseIndexStyle";
NSString * const iTMPDUseIndexOutputKey = @"iTMUseIndexOutput";

NSString * const iTMPDRenderDVIKey = @"iTMRenderDVI";
NSString * const iTMPDRenderPSKey = @"iTMRenderPS";
NSString * const iTMPDRenderEPSKey = @"iTMRenderEPS";

NSString * const iTMPDMacrosPathDocKey = @"macrosPathDoc";
NSString * const iTMPDMacrosPathGfxKey = @"macrosPathGfx";
NSString * const iTMPDMacrosPathMathKey = @"macrosPathMath";
NSString * const iTMPDMacrosPathMiscKey = @"macrosPathMisc";
NSString * const iTMPDMacrosPathTextKey = @"macrosPathText";
NSString * const iTMPDMacrosPathTmplKey = @"macrosPathTmpl";

@interface NSDocument(iTMProjectDocumentAccessors)
- (NSTextStorage *) textStorage;
@end

@interface NSDocumentController(iTMProjectDocumentAccessors)
- (NSDocument *) openUntitledDocumentOfTypeTeXDisplay: (BOOL) aFlag;
@end


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMProjectDocument(Accessors)
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
@interface NSString(iTMProjectDocument_Accessors_Private)
- (NSString *) stringBySubstitutingVariablesFromDictionary: (NSDictionary *) dictionary;
@end
@implementation NSString(iTMProjectDocument_Accessors_Private)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  stringBySubstitutingVariablesFromDictionary:
- (NSString *) stringBySubstitutingVariablesFromDictionary: (NSDictionary *) dictionary;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([self length])
    {
        NSScanner * S = [NSScanner scannerWithString: self];
        NSMutableString * result = [NSMutableString string];
        NSString * temp = [NSString string];
        NSString * bgroup = @"{";
        NSString * egroup = @"}";
        [S setCharactersToBeSkipped: nil];
        while(![S isAtEnd])
            if([S scanUpToString: @"$" intoString: &temp])
            {
                [result appendString: temp];
                [S scanString: @"$" intoString: nil];
                if([S scanString: bgroup intoString: nil] && [S scanUpToString: egroup intoString: &temp])
                {
                    NSString * suffix = [dictionary objectForKey: temp];
                    if([suffix isKindOfClass: [NSString class]])
                        [result appendString: suffix];
                    else
                    {
                        [result appendString: @"$"];
                        [result appendString: temp];
                    }
                    [S scanString: egroup intoString: nil];
                }
                else if([S scanUpToCharactersFromSet: [NSCharacterSet whitespaceAndNewlineCharacterSet] intoString: &temp])
                {
                    NSString * suffix = [dictionary objectForKey: temp];
                    if([suffix isKindOfClass: [NSString class]])
                        [result appendString: suffix];
                    else
                    {
                        [result appendString: @"$"];
                        [result appendString: temp];
                    }
                }
            }
            else
            {
                [S scanUpToString: nil intoString: &temp];
                [result appendString: temp];
            }
        return result;
    }
    else
        return [NSString string];
}
@end

@implementation iTMProjectDocument(Accessors)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=  cleanOutlets
- (void) cleanOutlets;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setTerminal: nil];
    [self _SetAbsoluteInput: nil];
    [self _SetRelativeInput: nil];
    [self setProjectDictionary: nil];
    [self setProjectName: nil];
    return;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= displayName
- (NSString *) displayName;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[iTMProjectsManager defaultManager] titleForProject: self];
}
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= coreFileName
- (NSString *) coreFileName;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[self fileName] lastPathComponent] stringByDeletingPathExtension];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= projectName
- (NSString *) projectName;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [_ProjectName length]? [[_ProjectName copy] autorelease]: [[super displayName] stringByDeletingPathExtension];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setProjectName:
- (void) setProjectName: (NSString *) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSString class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSString argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else if(![argument isEqualToString: _ProjectName])
    {
        [_ProjectName release];
        _ProjectName = [argument copy];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= projectDictionary
- (NSMutableDictionary *) projectDictionary;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!_ProjectDictionary)
    {
        [self setProjectDictionary: [NSDictionary dictionaryWithObjectsAndKeys:
                [NSString string], iTMPDRootKey,
                [NSString string], iTMPDDirectoryKey,
                [NSString string], iTMPDRootInputKey,
                [NSString string], iTMPDRootOutputKey,
                @"tex", iTMPDInputExtensionKey,
                @"pdf", iTMPDOutputExtensionKey,
                [NSString string], iTMPDCustomIdentifierKey,
                @"NO", iTMPDUseCustomIdentifierKey,
                @"NO", iTMPDKeepsDVIKey,
                @"NO", iTMPDKeepsPSKey,
                @"NO", iTMPDMLTeXKey,
                [NSString string], iTMPDInteractionKey,
                [NSString string], iTMPDCompileEngineKey,
                [NSString string], iTMPDCompileFormatKey,
                [NSString string], iTMPDCompileBaseKey,
                @"NO", iTMPDCompileSilentlyKey,
                @"YES", iTMPDUseCustomCompileKey,
                [NSString string], iTMPDCompileScriptKey,
                @"NO", iTMPDTypesetSilentlyKey,
                @"YES", iTMPDUseCustomTypesetKey,
                [NSString string], iTMPDTypesetScriptKey,
                @"NO", iTMPDBibTeXSilentlyKey,
                @"2", iTMPDBMinXReferencesKey,
                @"NO", iTMPDIndexSilentlyKey,
                @"NO", iTMPDICompressBlankKey,
                @"NO", iTMPDILetterOrderingKey,
                @"NO", iTMPDILetterOrderingKey,
                @"NO", iTMPDINoImplicitRangeKey,
                @"NO", iTMPDUseIndexStyleKey,
                @"NO", iTMPDUseIndexOutputKey,
                [NSString string], iTMPDIndexOutputKey,
                @"any", iTMPDIndexStartingKey,
                @"", iTMPDIndexStyleKey,
                [NSNumber numberWithInt: [[self class] version]], iTMPDVersionKey,
                @"Document/LaTeX", iTMPDMacrosPathDocKey,
                @"Graphics/LaTeX", iTMPDMacrosPathGfxKey,
                @"Math/LaTeX", iTMPDMacrosPathMathKey,
                @"Misc/LaTeX", iTMPDMacrosPathMiscKey,
                @"Text/LaTeX", iTMPDMacrosPathTextKey,
                @"LaTeX", iTMPDMacrosPathTmplKey,
                @"altpdftex ${iTMInput}.dvi", iTMPDRenderDVIKey,
                @"altpdftex ${iTMInput}.ps", iTMPDRenderPSKey,
                @"pstopdf ${iTMInput}.eps ${iTMInput}.pdf", iTMPDRenderEPSKey,
                        nil]];
    }
    return _ProjectDictionary;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setProjectDictionary:
- (void) setProjectDictionary: (NSDictionary *) argument;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSDictionary class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSDictionary argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    if(![_ProjectDictionary isEqualToDictionary: argument])
    {
        [_ProjectDictionary release];
        _ProjectDictionary = [argument mutableCopy];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= prepareProjectDictionaryForSaving
- (void) prepareProjectDictionaryForSaving;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([self isGeneric])
        [[self projectDictionary] setObject: [NSString string] forKey: iTMPDRootKey];
    else if([self isAbsolute])
        [[self projectDictionary] setObject: [self absoluteInput] forKey: iTMPDRootKey];
    else
        [[self projectDictionary] setObject: [@"." stringByAppendingPathComponent: [self relativeInput]] forKey: iTMPDRootKey];
    {
        NSString * S;
        if(!(S = [[[self windowController] typesetTextView] string])) S = [NSString string];
        [[self projectDictionary] setObject: S forKey: iTMPDTypesetScriptKey];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= prepareProjectManagement
- (void) prepareProjectManagement;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self readInputHeader];
    [self prepareRoot];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= readInputHeader
- (void) readInputHeader;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(@"readInputHeader");
    if([self isVoid])
    {
        NSDocument * doc = [[NSDocumentController sharedDocumentController] documentForFileName: [self absoluteInput]];
        NSString * string = [doc respondsToSelector: @selector(textStorage)]?
                                [[doc textStorage] mutableString]:
                                    (doc? [NSString string]: [NSString stringWithContentsOfFile: [self absoluteInput]]);
        if([string hasPrefix: @"%&"])
        {
            NSScanner * S = [NSScanner scannerWithString: string];
            NSCharacterSet * WSNL = [[S charactersToBeSkipped] retain];
            NSString * format = nil;
            [S setScanLocation: 2];
            [S setCharactersToBeSkipped: nil];
            if([S scanUpToCharactersFromSet: WSNL intoString: &format])
            {
                if([format hasPrefix: @"pdf"] || [format hasPrefix: @"alt"])
                    [[self projectDictionary] setObject: [NSString stringWithFormat: @"%@ %@.%@", format,
                                @"${iTMInput}", @"${iTMInputExtension}"]
                        forKey: iTMPDCompileScriptKey];
                else
                    [[self projectDictionary] setObject: [NSString stringWithFormat: @"altpdf%@ %@.%@", format,
                                @"${iTMInput}", @"${iTMInputExtension}"]
                        forKey: iTMPDCompileScriptKey];
                [[self projectDictionary] setObject: @"${iTMCompile}" forKey: iTMPDTypesetScriptKey];
            }
            else
                NSLog(@"Don't know how to compile.");
            [WSNL release];
        }
        else if([string length])
        {
            NSRange r = NSMakeRange(0, 0);
            NSString * value;
            if((value = [string stringForCommentedKey: @"compile" forRange: r effectiveRange: nil inHeader: YES]) &&
                            [value length])
                [[self projectDictionary] setObject: value forKey: iTMPDCompileScriptKey];
        }
        if([string length])
        {
            NSRange r = NSMakeRange(0, 0);
            NSString * value;
            if((value = [string stringForCommentedKey: @"typeset" forRange: r effectiveRange: nil inHeader: YES]) &&
                            [value length])
                [[self projectDictionary] setObject: value forKey: iTMPDTypesetScriptKey];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= prepareRoot
- (void) prepareRoot;
/*"The purpose is to synchronize the project dictionary data and the instance variables. However, it is a blank method for generic projects because the root info is not stored into the project dictionary data.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![self isGeneric])
    {
        NSString * root = [[self stringForKey: iTMPDRootKey] stringByStandardizingPath];
        if(![root length] || [root hasPrefix: @"/"])
        {
            NSString * relative = [root stringByAbbreviatingWithDotRelativeToDirectory:
                                [[self fileName] stringByDeletingLastPathComponent]];
            [self _SetRelativeInput: relative];
            [self _SetAbsoluteInput: root];
            [self _SetAbsolute: YES];
        }
        else
        {
            NSString * absoluteNew = [[[[self fileName] stringByDeletingLastPathComponent]
                                                stringByAppendingPathComponent: root] stringByStandardizingPath];
            [self _SetRelativeInput: root];
            [self _SetAbsoluteInput: absoluteNew];
            [self _SetAbsolute: NO];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= absoluteDirectory
- (NSString *) absoluteDirectory;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self absoluteInput] stringByDeletingLastPathComponent];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= absoluteRoot
- (NSString *) absoluteRoot;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self absoluteInput] stringByDeletingPathExtension];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= relativeRoot
- (NSString *) relativeRoot;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self absoluteRoot] lastPathComponent];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= absoluteOutput
- (NSString *) absoluteOutput;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * result = [self absoluteRoot];
    if(![result length]) return [NSString string];
    if([self isCustomIdent])
    {
        NSString * ident = [self stringForKey: iTMPDCustomIdentifierKey];
        if([ident length])
            result = [result stringByAppendingPathExtension: ident];
    }
    return [result stringByAppendingPathExtension: [self stringForKey: iTMPDOutputExtensionKey]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= absoluteInput
- (NSString *) absoluteInput;
/*"Description Forthcoming. Returns an autoreleased copy.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!_AbsoluteInput) _AbsoluteInput = [NSString string];
    return [[_AbsoluteInput copy] autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setAbsoluteInput:smart:
- (void) setAbsoluteInput: (NSString *) argument smart: (BOOL) aFlag;
/*"Tries to create the file before setting the value, calls the lower level setter #{setAbsoluteInput:}.
argument is a full path, tests are made to see if the path points to some readable location.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSString class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSString argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else
    {
        NSString * new = argument? [self rootFileNameByConverting: argument]: [NSString string];
        NSDocumentController * SDC = [NSDocumentController sharedDocumentController];
        NSDocument * doc;
        if([SDC respondsToSelector: @selector(openUntitledDocumentOfTypeTeXDisplay:)] &&
            [new length] && ![[NSFileManager defaultManager] isReadableFileAtPath: new] &&
                (doc = [SDC openUntitledDocumentOfTypeTeXDisplay: NO]))
        {
            NSSavePanel * SP = [NSSavePanel savePanel];
            NSButton * OKButton = [[SP contentView] viewWithTag: NSFileHandlingPanelOKButton];
            if([OKButton respondsToSelector: @selector(setTitle:)])
                [OKButton setTitle: NSLocalizedStringFromTableInBundle(@"Create", @"Project",
                    [NSBundle bundleForClass: [self class]], "Save panel button title")];
            if([SP runModalForDirectory: [new stringByDeletingLastPathComponent]
                            file: [new lastPathComponent]] == NSFileHandlingPanelOKButton)
            {
                [doc writeToFile: new ofType: [doc fileType]];
                [doc setFileName: new];
                [doc showWindows];
            }
        }
        [self setAbsoluteInput: argument];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setAbsoluteInput:
- (void) setAbsoluteInput: (NSString *) argument;
/*"Undo manager compliant setter, calls the lower level setter.
argument is a full path, tests are made to see if the path points to some readable location.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSString class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSString argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else
    {
        NSString * old = [self absoluteInput];
        NSString * new = argument? [self rootFileNameByConverting: argument]: [NSString string];
        if(![old isEqualToString: new])
        {
            NSString * relativeNew = [new stringByAbbreviatingWithDotRelativeToDirectory:
                                [[self fileName] stringByDeletingLastPathComponent]];
            [self _SetRelativeInput: relativeNew];
            if(![self isGeneric])
                [[[self undoManager] prepareWithInvocationTarget: self] setAbsoluteInput: old];
            [self _SetAbsoluteInput: new];
            [self fixErrorsPopUp];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= _SetAbsoluteInput:
- (void) _SetAbsoluteInput: (NSString *) argument;
/*"Lower level setter.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSString class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSString argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else
    {
        [_AbsoluteInput release];
        _AbsoluteInput = [argument retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= rootFileNameByConverting: 
- (NSString *) rootFileNameByConverting: (NSString *) argument;
/*"The entry argument is a relative or absolute file path whichever file extension. The result is the standardized path.
If argument's file extension is "pdf" it is replaced by a "tex" file extension. For example, the method returns "foo.tex" for entry "foo.whateveristhere.pdf". The returned path can point to an invalid location, the sender is responsible for such tests.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List: Nothing
"*/
{
    if(![[[argument pathExtension] lowercaseString] isEqualToString: @"pdf"])
        return [argument stringByStandardizingPath];
    else
    {
        NSString * result = argument;
        while([[result pathExtension] length])
            result = [result stringByDeletingPathExtension];
        result = [result stringByAppendingPathExtension: @"tex"];
        return [result stringByStandardizingPath];    
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= relativeInput
- (NSString *) relativeInput;
/*"Description Forthcoming. Returns an autoreleased copy.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!_RelativeInput) _RelativeInput = [NSString string];
    return [[_RelativeInput copy] autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setRelativeInput:smart:
- (void) setRelativeInput: (NSString *) argument smart: (BOOL) aFlag;
/*"Creates the file before calling the lower level setter #{setRelativeInput:}.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSString class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSString argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else
    {
        NSString * absoluteNew = [[[[[self fileName] stringByDeletingLastPathComponent]
                                            stringByAppendingPathComponent: @"."]
                                            stringByAppendingPathComponent: argument] stringByStandardizingPath];
        NSDocumentController * SDC = [NSDocumentController sharedDocumentController];
        NSDocument * doc;
        NSLog(@"setRelativeInput new: %@", absoluteNew);
        if([SDC respondsToSelector: @selector(openUntitledDocumentOfTypeTeXDisplay:)] &&
            [absoluteNew length] && ![[NSFileManager defaultManager] isReadableFileAtPath: absoluteNew] &&
                (doc = [SDC openUntitledDocumentOfTypeTeXDisplay: NO]))
        {
            NSSavePanel * SP = [NSSavePanel savePanel];
            NSButton * OKButton = [[SP contentView] viewWithTag: NSFileHandlingPanelOKButton];
            if([OKButton respondsToSelector: @selector(setTitle:)])
                [OKButton setTitle: NSLocalizedStringFromTableInBundle(@"Create", @"Project",
                    [NSBundle bundleForClass: [self class]], "Save panel button title")];
            if([SP runModalForDirectory: [absoluteNew stringByDeletingLastPathComponent]
                            file: [absoluteNew lastPathComponent]] == NSFileHandlingPanelOKButton)
            {
                [doc writeToFile: absoluteNew ofType: [doc fileType]];
                [doc setFileName: absoluteNew];
                [doc showWindows];
            }
        }
        [self setRelativeInput: argument];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setRelativeInput:
- (void) setRelativeInput: (NSString *) argument;
/*"Undo manager compliant setter, calls the lower level setter.
argument is a full path, tests are made to see if the path points to some readable location.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSString class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSString argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else
    {
        NSString * new = argument? [self rootFileNameByConverting: argument]: [NSString string];
        NSString * absoluteNew = [[[[[self fileName] stringByDeletingLastPathComponent]
                                            stringByAppendingPathComponent: @"."]
                                            stringByAppendingPathComponent: argument] stringByStandardizingPath];
        NSString * old = [self relativeInput];
        if(![old isEqualToString: new])
        {
            if(![self isGeneric])
                [[[self undoManager] prepareWithInvocationTarget: self] setRelativeInput: old];
            [self _SetRelativeInput: new];
            [self _SetAbsoluteInput: absoluteNew];
            [self fixErrorsPopUp];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= _SetRelativeInput:
- (void) _SetRelativeInput: (NSString *) argument;
/*"Lower level setter.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSString class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSString argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else
    {
        [_RelativeInput release];
        _RelativeInput = [argument retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= isAbsolute:
- (BOOL) isAbsolute;
/*"Bools are strings only (remember it is an environment).
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _IsAbsolute;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setAbsolute:
- (void) setAbsolute: (BOOL) aFlag;
/*"High level undo compliant.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL oldFlag = [self isAbsolute];
    if(aFlag != oldFlag)
    {
        [[[self undoManager] prepareWithInvocationTarget: self] setAbsolute: oldFlag];
        [self _SetAbsolute: aFlag];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= _SetAbsolute:
- (void) _SetAbsolute: (BOOL) aFlag;
/*"Low level.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _IsAbsolute = aFlag;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= isCustomIdent:
- (BOOL) isCustomIdent;
/*"Bools are strings only (remember it is an environment).
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [self boolForKey: iTMPDUseCustomIdentifierKey];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setCustomIdent:
- (void) setCustomIdent: (BOOL) aFlag;
/*"High level undo compliant.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL oldFlag = [self isCustomIdent];
    if(aFlag != oldFlag)
    {
        [[[self undoManager] prepareWithInvocationTarget: self] setCustomIdent: oldFlag];
        [[self projectDictionary] setObject: (aFlag? @"YES": @"NO") forKey: iTMPDUseCustomIdentifierKey];
    }
    return;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= _SetCustomIdent:
- (void) _SetCustomIdent: (BOOL) aFlag;
/*"Low level.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self projectDictionary] setObject: (aFlag? @"YES": @"NO") forKey: iTMPDUseCustomIdentifierKey];
//    [[self projectDictionary] setObject: [NSNumber numberWithBool: aFlag] forKey: iTMPDUseCustomIdentifierKey];
    return;
}
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= isCustomTypeset:
- (BOOL) isCustomTypeset;
/*"Bools are strings only (remember it is an environment).
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [self boolForKey: iTMPDUseCustomTypesetKey];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setCustomTypeset:
- (void) setCustomTypeset: (BOOL) aFlag;
/*"High level undo compliant.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL oldFlag = [self isCustomTypeset];
    if(aFlag != oldFlag)
    {
        NSString * object = aFlag? @"YES": @"NO";
        [[self projectDictionary] setObject: object forKey: iTMPDUseCustomTypesetKey];
        // register undo AFTER the project dictionary has been changed
        [[[self undoManager] prepareWithInvocationTarget: self] setCustomTypeset: oldFlag];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= isCustomCompile:
- (BOOL) isCustomCompile;
/*"Bools are strings only (remember it is an environment).
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [self boolForKey: iTMPDUseCustomCompileKey];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setCustomCompile:
- (void) setCustomCompile: (BOOL) aFlag;
/*"High level undo compliant.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL oldFlag = [self isCustomCompile];
    if(aFlag != oldFlag)
    {
        NSString * object = aFlag? @"YES": @"NO";
        [[self projectDictionary] setObject: object forKey: iTMPDUseCustomCompileKey];
        // register undo AFTER the project dictionary has been changed
        [[[self undoManager] prepareWithInvocationTarget: self] setCustomCompile: oldFlag];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= terminal
- (iTMTerminalController *) terminal;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!_Terminal)
        [self setTerminal: [[[iTMTerminalController alloc] init] autorelease]];
    return _Terminal;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setTerminal:
- (void) setTerminal: (iTMTerminalController *) aTerminal;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(aTerminal && ![aTerminal isKindOfClass: [iTMTerminalController class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] iTMTerminalController argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), aTerminal];
    if(![_Terminal isEqual: aTerminal])
    {
        [_Terminal release];
        _Terminal = [aTerminal retain];
        [[_Terminal output] setDelegate: self];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= environmentDictionary
- (NSDictionary *) environmentDictionary;
/*"Sets up the riht file objects. The extension should be set by the one who will fill up a task environemnt.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMutableDictionary * ED = [NSMutableDictionary dictionary];
    id object;
    #define addEntry(key)\
    if(object = [[self projectDictionary] objectForKey: key])\
        [ED setObject: object forKey: key];\
    else\
        [ED setObject: [NSString string] forKey: key]
    addEntry(iTMPDCustomIdentifierKey);
    addEntry(iTMPDKeepsDVIKey);
    addEntry(iTMPDKeepsPSKey);
    addEntry(iTMPDMLTeXKey);
    addEntry(iTMPDInteractionKey);
    addEntry(iTMPDCompileEngineKey);
    addEntry(iTMPDCompileFormatKey);
    addEntry(iTMPDCompileBaseKey);
    addEntry(iTMPDBibTeXSilentlyKey);
    addEntry(iTMPDBMinXReferencesKey);
    addEntry(iTMPDIndexSilentlyKey);
    addEntry(iTMPDICompressBlankKey);
    addEntry(iTMPDIGermanOrderingKey);
    addEntry(iTMPDILetterOrderingKey);
    addEntry(iTMPDINoImplicitRangeKey);
    addEntry(iTMPDIndexStartingKey);
    addEntry(iTMPDInputExtensionKey);
    addEntry(iTMPDRootInputKey);
    addEntry(iTMPDOutputExtensionKey);
    addEntry(iTMPDRootOutputKey);
    if([self boolForKey: iTMPDUseIndexOutputKey])
        addEntry(iTMPDIndexOutputKey);
    else
        [ED setObject: [NSString string] forKey: iTMPDIndexOutputKey];
    if([self boolForKey: iTMPDUseIndexStyleKey])
        addEntry(iTMPDIndexStyleKey);
    else
        [ED setObject: [NSString string] forKey: iTMPDIndexStyleKey];
    {
        NSString * absoluteInput = [self absoluteInput];
        [ED addEntriesFromDictionary: [NSDictionary dictionaryWithObjectsAndKeys:
                [absoluteInput pathExtension], iTMPDInputExtensionKey,
                [[absoluteInput lastPathComponent] stringByDeletingPathExtension], iTMPDRootInputKey,
                [[absoluteInput lastPathComponent] stringByDeletingPathExtension], iTMPDRootOutputKey,
                [self absoluteDirectory], iTMPDDirectoryKey,
                        nil]];
    }
    [ED setObject: [[NSUserDefaults standardUserDefaults] stringForKey: @"TEXEDIT"] forKey: @"TEXEDIT"];
    {
        NSString * script = [[self projectDictionary] objectForKey: iTMPDCompileScriptKey];
        script = [script isKindOfClass: [NSString class]]?
                        [script stringBySubstitutingVariablesFromDictionary: ED]: [NSString string];
        [ED setObject: script forKey: iTMPDCompileScriptKey];
    }
    {
        NSString * script = [[self projectDictionary] objectForKey: iTMPDTypesetScriptKey];
        script = [script isKindOfClass: [NSString class]]?
                        [script stringBySubstitutingVariablesFromDictionary: ED]: [NSString string];
        [ED setObject: script forKey: iTMPDTypesetScriptKey];
    }
    return ED;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= boolForKey:
- (BOOL) boolForKey: (NSString *) aKey;
/*"Bools are strings only (remember it is an environment).
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMutableDictionary * PD = [self projectDictionary];
    NSString * result = [PD objectForKey: aKey];
    if([result isKindOfClass: [NSString class]])
    {
        NSString * upper = [result uppercaseString];
        if([upper isEqualToString: @"YES"])
            return YES;
        else if([upper isEqualToString: @"NO"])
            return NO;
    }
    [PD setObject: @"NO" forKey: aKey];
    return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setBool:forKey:
- (void) setBool: (BOOL) aFlag forKey: (id) aKey;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL oldFlag = [self boolForKey: aKey];
    if(aFlag != oldFlag)
    {
        id new = aFlag? @"YES": @"NO";
        [[[self undoManager] prepareWithInvocationTarget: self] setBool: oldFlag forKey: aKey];
        [[self projectDictionary] setObject: new forKey: aKey];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= stringForKey:
- (NSString *) stringForKey: (id) aKey;
/*"If the object read for key aKey is a number, everything is fine.
If there is nothing or a wrong kind of object, a default false bool number is associated to key aKey.
The result is an autoreleased copy.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMutableDictionary * PD = [self projectDictionary];
    NSString * result = [PD objectForKey: aKey];
    #warning DANGER
    if(![result isKindOfClass: [NSString class]] || ![result length])
    {
        result = [NSString string];
        [PD setObject: result forKey: aKey];
    }
    return [[result copy] autorelease];        
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setString:forKey:
- (void) setString: (NSString *) aString forKey: (id) aKey;
/*"A nil argument is replaced by [NSString string].
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(aString && ![aString isKindOfClass: [NSString class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSString argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), aString];
    {
        NSString * old = [self stringForKey: aKey];
        NSString * new = aString ? aString: [NSString string];
        if(![new isEqualToString: old])
        {
            [[[self undoManager] prepareWithInvocationTarget: self] setString: old forKey: aKey];
            [[self projectDictionary] setObject: new forKey: aKey];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= fixErrorsPopUp:
- (void) fixErrorsPopUp;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSPopUpButton * popUp = [[self windowController] errorsPopUp];
    [popUp setAction: @selector(errorFileChosen:)];
    [popUp setTarget: self];
    [popUp removeAllItems];
    {
        NSString * relativeRoot = [self relativeRoot];
        if([relativeRoot length])
        {
            [popUp addItemWithTitle: [relativeRoot stringByAppendingPathExtension: @"log"]];
            [popUp addItemWithTitle: [relativeRoot stringByAppendingPathExtension: @"blg"]];
            [popUp addItemWithTitle: [relativeRoot stringByAppendingPathExtension: @"ilg"]];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setOutputIdentifier:
- (void) setOutputIdentifier: (NSString *) argument;
/*"Lower level setter.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(@"-");
    if(argument && ![argument isKindOfClass: [NSString class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSString argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else if([argument length])
    {
        [[self projectDictionary] setObject: @"YES" forKey: iTMPDUseCustomIdentifierKey];
        [self setString: argument forKey: iTMPDCustomIdentifierKey];
    }
    else
        [[self projectDictionary] setObject: @"NO" forKey: iTMPDUseCustomIdentifierKey];
//    NSLog(@"+");
    return;
}
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMProjectDocument(Accessors)

