/*
//  iTMProjectDocument(Cocoa).m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Dec 11 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSArray.h>
#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "iTMProjectDocument.h"
#import "iTMProjectDocument(Accessors).h"
#import "iTMProjectDocument(Cocoa).h"
#import "iTMProjectWindowController.h"
#import "iTMTextStorage(TeXLogFile).h"
#import "iTMStatusNotification.h"

NSString * const iTMUDTerminalBehindKey = @"iTMUDTerminalBehind";

@interface NSWindowController(iTMProjectDocument_Cocoa_Private)
- (void) highlightAndScrollLineRangeToVisible: (NSRange) aRange;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMProjectDocument(Cocoa)
/*"Description forthcoming."*/
@implementation iTMProjectDocument(Cocoa)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [self setVersion: '0001'];
    {
        NSString * string = [NSString stringWithFormat: @"\"%@\" %@",
            [[NSBundle mainBundle] pathForAuxiliaryExecutable: @"e Helper"], @"-line %d -file %s"];
        [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
                        string,  @"TEXEDIT",
                        [NSNumber numberWithBool: YES], iTMUDTerminalBehindKey,
                        nil]];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    self = [super init];
    self->_IsAbsolute = YES;
    self->_IsVoid = NO;
    [self cleanOutlets];
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self cleanOutlets];
    [self setCurrentErrorPath: nil];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  makeWindowControllers
- (void) makeWindowControllers;
/*"Critical code: We do not want the window nib to be loaded at all. No critical portion of code must reside inside the nib.
When opening generic projects, it is speedier to not load the nib files, especially when they will never be needed.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowShouldClose:
- (BOOL) windowShouldClose: (id) sender;
/*"This is a window delegate method to prevent closing project documents.
If option key is down, the closing behavior is normal.
Otherwise, the window is closed but not the project document.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask) > 0)
    {
        NSLog(@"NSAlternateKeyMask: closing...");
        return YES;
    }
    else
    {
        [[[self windowController] window] orderOut: self];
        return NO;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  showWindows
- (void) showWindows;
/*"Patch to call #{prepareRoot}.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: adapt to 10.1
"*/
{
    [super showWindows];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMenuItem:
- (BOOL) validateMenuItem: (id) sender;
/*" Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: FULL implementation missing
"*/
#warning Implementation missing
{
    SEL action = [sender action];
    if((action == @selector(makeIndex:)) || (action == @selector(bibTeX:)) ||
            (action == @selector(typeset:)) || (action == @selector(compile:)))
        return ([[self absoluteInput] length] != 0);
    else if(action == @selector(performClose:))
        return NO;
    else if(action == @selector(performCloseProject:))
        return YES;
    else if(action == @selector(showTerminal:))
        return ![[self windowController] alreadySwitchedToPaneWithIdentifier: iTMPWCTerminalIdentifier];
    else if(action == @selector(showProject:))
        return ![[self windowController] alreadySwitchedToPaneWithIdentifier: iTMPWCProjectIdentifier];
    else if(action == @selector(showErrors:))
        return [self canShowErrors];
    else if(action == @selector(showFiles:))
        return [self canShowFiles];
    else if(action == @selector(saveDocument:))
        return ![self isStrongGeneric] && ![self isVoid];
    else if((action == @selector(saveDocumentAs:)) ||(action == @selector(saveDocumentTo:)))
        return ![self isVoid];
    else if ((action == @selector(scrollToNextError:)) || (action == @selector(scrollToPreviousError:)))
        return YES;
    else
        return [super validateUserInterfaceItem: sender];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToolbarItem:
- (BOOL) validateToolbarItem: (id) sender;
/*" Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    SEL action = [sender action];
    if(action == @selector(showProject:))
        return ![[[self windowController] window] isEqual: [NSApp keyWindow]] ||
            ![[self windowController] alreadySwitchedToPaneWithIdentifier: iTMPWCProjectIdentifier];
    else if(action == @selector(showProjectWindow:))
        return ![[[self windowController] window] isEqual: [NSApp keyWindow]];
    else if(action == @selector(showFiles:))
        return ![[[self windowController] window] isEqual: [NSApp keyWindow]] ||
            ![[self windowController] alreadySwitchedToPaneWithIdentifier: iTMPWCFilesIdentifier];
    else if(action == @selector(showErrors:))
        return ![[[self windowController] window] isEqual: [NSApp keyWindow]] ||
            ![[self windowController] alreadySwitchedToPaneWithIdentifier: iTMPWCErrorsIdentifier];
    else if(action == @selector(showErrors:))
        return [self canShowErrors];
    else if(action == @selector(showFiles:))
        return [self canShowFiles];
    else if((action == @selector(typeset:)) || (action == @selector(compile:)))
        return YES;//[self canShowInput];
    else if(action == @selector(bibTeX:))
        return [self canBibTeX];
    else if(action == @selector(makeIndex:))
        return [self canMakeIndex];
    else
        return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  writeToFile:ofType:
- (BOOL) writeToFile: (NSString *) fileName ofType: (NSString *) type;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL result = [super writeToFile: fileName ofType: type];
    if(result) [self validateUserInterface];
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setFileName:
- (void) setFileName: (NSString *) fileName;
/*"If the receiver was a strong generic project document, it tries to become a normal document after the change.
Sends a #{becomeNormal} message.
Patch to call #{prepareRoot} also.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL wasStrongGeneric = [self isStrongGeneric];
    [super setFileName: fileName];
    [self prepareRoot];
    if(wasStrongGeneric) [self becomeNormal];
    if([self isStrongGeneric]) [self becomeGeneric];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dataRepresentationOfType:ofType:
- (NSData *) dataRepresentationOfType: (NSString *) aType;
/*" NSMacOSRomanStringEncoding allowing Lossy Conversion.
Version History: koch@math.uoregon.edu
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self prepareProjectDictionaryForSaving];
    return (NSData *)CFPropertyListCreateXMLData(NULL, [self projectDictionary]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dataRepresentationOfType:ofType:
- (BOOL) loadDataRepresentation: (NSData *) data ofType: (NSString *) aType;
/*" Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL result = NO;
    NSString * string;
    NSDictionary * dictionary;
    NS_DURING
    string = [[NSString alloc] initWithData: data encoding: NSMacOSRomanStringEncoding];
    dictionary = [string propertyList];
    if(dictionary)
    {
        id PD = [self projectDictionary];
        /*"Critical: the new key/value pairs parsed above erase the default ones of the lazy initializer."*/
        [PD addEntriesFromDictionary: dictionary];
        [self prepareProjectManagement];
        result = YES;
    }
    NS_HANDLER
        if([[NSDocumentController sharedDocumentController] shouldCreateUI])
            NSRunCriticalAlertPanel(
                NSLocalizedStringFromTableInBundle(@"Project Document.", @"Project",
                                [NSBundle bundleForClass: [self class]], "Critical Panel Title"),
                    NSLocalizedStringFromTableInBundle(@"Exception caught:\n%@\nopening %@.", @"Project",
                                    [NSBundle bundleForClass: [self class]], "Exception, 2x%@ format."),
                        nil, nil, nil, [localException reason], [self fileName]);
        [localException raise];
    NS_ENDHANDLER;
    [string release];
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fileWrapperRepresentationOfType:ofType:
- (NSFileWrapper *) fileWrapperRepresentationOfType: (NSString *) aType;
/*" NSMacOSRomanStringEncoding allowing Lossy Conversion.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSFileWrapper * wrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers: nil];
    [wrapper addRegularFileWithContents: [self dataRepresentationOfType: aType]
                    preferredFilename: [iTMPDProjectKey stringByAppendingPathExtension: @"plist"]];
    return [wrapper autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  loadFileWrapperRepresentation:ofType:
- (BOOL) loadFileWrapperRepresentation: (NSFileWrapper *) wrapper ofType: (NSString *) docType;
/*" Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL result = YES;
    if([wrapper isDirectory])
    {
        NSDictionary * fileWrappers = [wrapper fileWrappers];
        NSEnumerator * keyEnumerator = [fileWrappers keyEnumerator];
        NSString * key;
        while(key = [keyEnumerator nextObject])
        {
            id W = [fileWrappers objectForKey: key];
            if([W isRegularFile])
            {
                if([[W filename] isEqualToString: [iTMPDProjectKey stringByAppendingPathExtension: @"plist"]])
                    result = [self loadDataRepresentation: [W regularFileContents] ofType: @"plist"];
            }
            else if([W isDirectory])
            {
                ;
            }
            else
                [NSException raise: NSInternalInconsistencyException format: @"old .texshop versions."];
            if(!result) break;
        }
    }
    else
        [NSException raise: NSInternalInconsistencyException format: @"old .texshop versions."];
    [self validateUserInterface];
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isDocumentEdited
- (BOOL) isDocumentEdited;
/*"If the document is generic, the inherited method is not called and the changes are not reflected in the window close button. As a consequence, the user will not be prompted to save generic projects while terminating the application.
Version History: jerome.laurens@u-bourgogne.fr (10/08/2001)
- < 1.1: 03/10/2002
To Do List: Does not work?
"*/
{
    if([self isStrongGeneric])
        return NO;
    else
        return [super isDocumentEdited];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  updateChangeCount:
- (void) updateChangeCount: (NSDocumentChangeType) change;
/*"A patch to synchronize the UI with #{validateUserInterface} each time a change is made.
Version History: jerome.laurens@u-bourgogne.fr (10/08/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self validateUserInterface];
    [super updateChangeCount: change];
    if([self isStrongGeneric])
        [[[self windowController] window] setDocumentEdited: NO];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textView:clickedOnLink:atIndex:
- (BOOL) textView: (NSTextView *) textView clickedOnLink: (id) link atIndex: (unsigned) charIndex;
/*"Opens the document corresponding to the link.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    iTMTextStorage * TS = [textView textStorage];
    if([TS isKindOfClass: [iTMTextStorage class]])
    {
        [TS markVisitedLinkAtIndex: charIndex];
        if([link respondsToSelector: @selector(objCType)])
        {
            NSString * type = [NSString stringWithCString: [link objCType]];
            if([type isEqualToString: @"s"] || [type isEqualToString: @"i"])
            {
                int linkInt = [link intValue];
                if(linkInt<0)
                {
                    #if 0
                    NSString * file = [[TS texRootDocumentPath] stringByDeletingPathExtension];
                    NSDocument * document = [[NSDocumentController sharedDocumentController]
                        openDocumentWithContentsOfFile: ([file hasPrefix: @"/"]?
                            [file stringByAppendingPathExtension: @"pdf"]:
                            [[[self currentErrorPath] stringByAppendingPathComponent: [TS texRootDocumentPath]]
                                stringByAppendingPathExtension: @"pdf"])
                            display: YES];
                    if(!document)
                    #endif
                    NSDocument * document = [[NSDocumentController sharedDocumentController]
                                        openDocumentWithContentsOfFile: [self absoluteOutput] display: YES];
                    if([document respondsToSelector: @selector(setCurrentPage:)])
                        [document setCurrentPage: -linkInt-1];
                    if(document)
                        return YES;
                    else
                        NSRunCriticalAlertPanel(
                    NSLocalizedStringFromTableInBundle(@"Project Document.", @"Project",
                            [NSBundle bundleForClass: [self class]], "Informational Panel Title"),
                NSLocalizedStringFromTableInBundle(@"Unable to switch to\n%@.", @"Project",
                        [NSBundle bundleForClass: [self class]], "Problem opening the file."),
            nil, nil, nil, [self absoluteOutput]);
                }
                else
                {
                    NSString * file = [TS texDocumentPathAtIndex: charIndex];
                    #if 0
                    NSDocument * document = [[NSDocumentController sharedDocumentController]
                        openDocumentWithContentsOfFile: ([file hasPrefix: @"/"]?
                [file stringByStandardizingPath]:
        [[[self currentErrorPath] stringByAppendingPathComponent: [TS texRootDocumentPath]] stringByStandardizingPath])
                                display: YES];
                    if(!document)
                    #endif
                        NSDocument * document = [[NSDocumentController sharedDocumentController]
                                    openDocumentWithContentsOfFile: [[[self absoluteDirectory]
                                        stringByAppendingPathComponent: file] stringByStandardizingPath] display: YES];
                    if(document)
                    {
                        NSEnumerator * enumerator = [[document windowControllers] objectEnumerator];
                        NSWindowController * WC;
                        while(WC = [enumerator nextObject])
                            if([WC respondsToSelector: @selector(highlightAndScrollLineRangeToVisible:)])
                                [WC highlightAndScrollLineToVisible: linkInt];
                        return YES;
                    }
                    else
                    {
                        [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
                            [NSString stringWithFormat: NSLocalizedStringFromTableInBundle
                                (@"Unable to open file %@", @"Project",
                                    [NSBundle bundleForClass: [self class]], "status info format, 1%@"), file]];
                    }
                }
            }
            else if([type isEqualToString: @"NSRange"])
            {
                NSString * file = [TS texDocumentPathAtIndex: charIndex];
                NSDocument * document = [[NSDocumentController sharedDocumentController]
                    openDocumentWithContentsOfFile: ([file hasPrefix: @"/"]?
            [file stringByStandardizingPath]:
    [[[self currentErrorPath] stringByAppendingPathComponent: [TS texRootDocumentPath]] stringByStandardizingPath])
                            display: YES];
                if(!document)
                    document = [[NSDocumentController sharedDocumentController]
                                openDocumentWithContentsOfFile: [[[self absoluteDirectory]
                                    stringByAppendingPathComponent: file] stringByStandardizingPath] display: YES];
                if(document)
                {
                    NSEnumerator * enumerator = [[document windowControllers] objectEnumerator];
                    NSWindowController * WC;
                    while(WC = [enumerator nextObject])
                        if([WC respondsToSelector: @selector(highlightAndScrollLineRangeToVisible:)])
                            [WC highlightAndScrollLineRangeToVisible: [link rangeValue]];
                    return YES;
                }
                else
                {
                    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
                        [NSString stringWithFormat: NSLocalizedStringFromTableInBundle
                            (@"Unable to open file %@", @"Project",
                                [NSBundle bundleForClass: [self class]], "status info format, 1%@"), file]];
                }
            }
        }
    }
    return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textDidChange:
- (void) textDidChange: (NSNotification *) aNotification;
/*"To catch the change of the script when the user is editing.
Version History: jerome.laurens@u-bourgogne.fr (11/18/01)
- < 1.1: 03/10/2002
To Do List: no undo here!!!
"*/
{
    NSText * T = [aNotification object];
    if([T isEqual: [[self windowController] typesetTextView]])
        [self setTypesetScript: [T string]];
    return;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMProjectDocument(Cocoa)

