/*
//  iTMProjectDocument.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Sep 11 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSFileManager.h>
#import <Foundation/Foundation.h>
#import <AppKit/NSTabView.h>
#import <AppKit/NSApplication.h>
#import <AppKit/NSWindowController.h>
#import <AppKit/NSDocument.h>
#import <AppKit/NSLayoutManager.h>
#import <AppKit/NSAttributedString.h>
#import <AppKit/AppKit.h>
#import "NSString(iTeXMac).h"
#import "NSControl(iTeXMac).h"
#import "NSWindow(iTeXMac).h"
#import "iTMProjectDocument.h"
#import "iTMProjectDocument(Cocoa).h"
#import "iTMProjectDocument(Accessors).h"
#import "iTMResourcesController(GenericProjects).h"
#import "iTMTerminalController.h"
#import "iTMTerminalController(UI).h"
#import "iTMTextStorage(TeXLogFile).h"
#import "iTMLogSyntaxParser.h"
#import "iTMXTextView.h"
#import "iTMProjectWindowController.h"
#import "iTMTeXEngineManager.h"
#import "iTMTextFinder.h"
#import "iTMStatusNotification.h"
#import "iTMSettings.h"

NSString * const iTMProjectVoidIdentifier = @"iTM Void";
NSString * const iTMProjectTerminalIdentifier = @"iTM Terminal";
NSString * const iTMProjectCompileDescriptorIdentifier = @"iTM Typeset Descriptor";
NSString * const iTMProjectDocumentType = @"iTeXMac Project";
NSString * const iTMPDProjectKey = @"Project";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMProjectDocument
/*"Description forthcoming."*/
@implementation iTMProjectDocument
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  reallyMakeWindowControllers
- (void) reallyMakeWindowControllers;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![self windowController])
        [self setWindowController: [[[iTMProjectWindowController alloc]
                            initWithWindowNibName: @"iTMProjectDocument"] autorelease]];
    if([self windowController])
    	[self addWindowController: [self windowController]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowController
- (id) windowController;
/*"No lazy initialization.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _WC;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setWindowController:
- (void) setWindowController: (iTMProjectWindowController *) argument;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![_WC isEqual: argument])
    {
        if(argument && ![argument isKindOfClass: [iTMProjectWindowController class]])
            [NSException raise: NSInvalidArgumentException format:
                @"-[%@ %@] iTMProjectWindowController argument expected: %@.",
                    [self class], NSStringFromSelector(_cmd), argument];
        else
        {
            [_WC release];
            _WC = [argument retain];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  makeWindows
- (void) makeWindows;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self reallyMakeWindowControllers];
    {
        [[[self windowController] window] setDelegate: self];// critical: to prevent the document closing behaviour
        [[self terminal] setInput: [[self windowController] terminalInput]];
        [[self terminal] setOutput: [[self windowController] terminalOutput]];
        [[[self windowController] terminalOutput] setDelegate: self];// used for clickedAtIndex:
//        [[[self windowController] compileTextField] setDelegate: self];
//        [[[self windowController] typesetTextView] setDelegate: self];
        [[[self windowController] errorsTextView] setDelegate: self];
        [[[self windowController] errorsTextView] setEditable: NO];
        // Critical: next 2 lines is supposed to be done!!!! if there is a UI, it must be up to date!!!
        [[[self windowController] typesetTextView] setString: [self stringForKey: iTMPDTypesetScriptKey]];
        [[[self windowController] typesetTextView] setEditable: [self isCustomTypeset]];
        [[iTMTextFinder sharedTextFinder] installFindResponderForWindow: [[self windowController] window]];
#if 0
        {
            NSWindow * W = [aController window];
            [W setDocumentEdited: [self isDocumentEdited]];// minor problem when close window and unsaved
            [[W contentView] validateUserInterfaceItems];// cannot use validateContenInterfaceItem yet
//            [W makeKeyAndOrderFront: self];
            [iTMFontColorResponder installResponderForWindow: W];
            [W setDelegate: self];
        }
#endif
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= isVoid
- (BOOL) isVoid;
/*"Description forthcoming. See #{becomeVoid}.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _IsVoid;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= isGeneric
- (BOOL) isGeneric;
/*"Description forthcoming. See #{becomeGeneric}.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([self isStrongGeneric])
        return YES;
    else
    {
        id result = [[self projectDictionary] objectForKey: iTMPDIsGenericKey];
        if([result respondsToSelector: @selector(boolValue)])
            return [result boolValue];
        else
        {
            [[self projectDictionary] setObject: [NSNumber numberWithBool: NO] forKey: iTMPDIsGenericKey];
            return NO;
        }
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= isStrongGeneric
- (BOOL) isStrongGeneric;
/*"If the receiver comes from ~/Library/Application Support/iTeXMac/Generic Projects, it is a strong generic project.
It cannot become a normal project.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[[self fileName] stringByResolvingSymlinksInPath] stringByStandardizingPath] hasPrefix: 
                    [[[[iTMResourcesController sharedResourcesController] genericProjectsPath]
                            stringByResolvingSymlinksInPath] stringByStandardizingPath]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= becomeGeneric
- (void) becomeGeneric;
/*"Generic projects are those stored in Resources/Generic or ~/Library/Application Support/iTeXMac/Generic Projects. They are used as templates and helpers to allow users to typeset a file without having a project attached to it. The change count for these documents is squeezed, no save is available. Only save as can work. If saved as generic documents don't belong to a generic subfolder, they #{becomeNormal} otherwise, they stay generic. #{becomeNormal} is some kind of opposit to #{becomeGeneric}.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: implement what is announced.
"*/
{
    if(![self isGeneric])
    {
        [[[self undoManager] prepareWithInvocationTarget: self] becomeNormal];
        [[self projectDictionary] setObject: [NSNumber numberWithBool: YES] forKey: iTMPDIsGenericKey];
        _IsVoid = NO;
        [self setProjectName: [[[self fileName] lastPathComponent] stringByDeletingPathExtension]];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= becomeNormal
- (void) becomeNormal;
/*"The opposit of #{becomeGeneric}. A generic project saved as ... should become normal.
If the receiver belongs to ~/Library/Application Support/iTeXMac/Generic Projects/ it cannot become normal,
it stays generic.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![self isStrongGeneric] && [self isGeneric])
    {
        [[[self undoManager] prepareWithInvocationTarget: self] becomeGeneric];
        [[self projectDictionary] setObject: [NSNumber numberWithBool: NO] forKey: iTMPDIsGenericKey];
        _IsVoid = NO;
        [self setProjectName: nil];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=  performCloseProject:
- (void) performCloseProject: (id) sender;
/*"This is the only way to close a project document. Calls the inherited #{close}.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[[self windowController] window] close];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateUserInterface
- (void) validateUserInterface;
/*"Sends a #{validateUserInterfaceItems} to the window content view.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[[[self windowController] window] contentView] validateUserInterfaceItems];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= terminalRestart:
- (void) terminalRestart: (id) irrelevant;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self terminal] addShellTaskWithEnvironment: nil
        standardInput: [NSString string]
            directory: [self absoluteDirectory]
                delegate: nil
                    didExecuteSelector: NULL
                        userInfo: nil
                            restart: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= canStopTerminal
- (BOOL) canStopTerminal;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
     return [[[self terminal] currentTask] isRunning];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= terminalStop:
- (void) terminalStop: (id) irrelevant;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self terminal] closeTerminal: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= terminalEscape:
- (void) terminalEscape: (id) irrelevant;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self terminal] standardInputString: @"e\nexit" verbose: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= terminalExit:
- (void) terminalExit: (id) irrelevant;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self terminal] standardInputString: @"x\nexit" verbose: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= terminalQuiet:
- (void) terminalQuiet: (id) irrelevant;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self terminal] standardInputString: @"q\nexit" verbose: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= terminalIgnore:
- (void) terminalIgnore: (id) irrelevant;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self terminal] standardInputString: @"\n" verbose: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateXEQIS:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateExitTerminal:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateEscapeTerminal:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateQuietTerminal:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateIgnoreTerminal:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateStopTerminal:
#define validateXEQIS(validate) - (void) validate: (id) sender;\
{\
    [sender setEnabled: [[[self terminal] currentTask] isRunning]];\
    if(![[sender toolTip] length])\
        [sender setToolTip: NSLocalizedStringFromTableInBundle(\
            NSStringFromSelector([sender action]), @"Terminal",\
                [NSBundle bundleForClass: [self class]], "")];\
    return;\
}
/*"Validating process (see NSControl(iTeXMac)). The terminal validates its window each time it receives an output.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
validateXEQIS(validateTerminalExit);
validateXEQIS(validateTerminalEscape);
validateXEQIS(validateTerminalQuiet);
validateXEQIS(validateTerminalIgnore);
validateXEQIS(validateTerminalStop);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showPreviousError:
- (void) showPreviousError: (id) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (03/03/2002)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self scrollToPreviousError: sender];
    [self showSourceErrorForLogRange: [[[self windowController] errorsTextView] selectedRange]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showNextError:
- (void) showNextError: (id) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (03/13/2002)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self scrollToNextError: sender];
    [self showSourceErrorForLogRange: [[[self windowController] errorsTextView] selectedRange]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showSourceErrorForLogRange:
- (void) showSourceErrorForLogRange: (NSRange) aRange;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSTextView * TV = [[self windowController] errorsTextView];
    NSLog([[TV string] substringWithRange: aRange]);
    if(aRange.length)
    {
        NSRange R = [[TV string] rangeOfString: @"\nl." options: 0 range: aRange];
        if(R.length)
        {
            id link = [[TV textStorage] attribute: NSLinkAttributeName atIndex: R.location+1 effectiveRange: nil];
            if(link)
            {
                id delegate = [TV delegate];
                if([delegate respondsToSelector: @selector(textView:clickedOnLink:atIndex:)])
                    [delegate textView: TV clickedOnLink: link atIndex: R.location];
                else
                    NSLog(@"-[iTMProjectDocument showSourceErrorForLogRange:] no suitable delegate: %@", delegate);
            }
            else
                NSLog(@"-[iTMProjectDocument showSourceErrorForLogRange:] no link.");
        }
        else
            NSLog(@"-[iTMProjectDocument showSourceErrorForLogRange:] no \"l.\" found.");
    }
    else
        NSLog(@"-[iTMProjectDocument showSourceErrorForLogRange:] no TeX error found, bizarre.");
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= scrollToPreviousError:
- (void) scrollToPreviousError: (id) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSTextView * TV = [[self windowController] errorsTextView];
    iTMTextStorage * TS = [TV textStorage];
    if(![TS respondsToSelector: @selector(rangeOfErrorFromIndex:cycle:backwards:)])
    {
        [self fixErrorsPopUp];
        [[[self windowController] errorsPopUp] selectItemWithTitle:
                [[self relativeRoot] stringByAppendingPathExtension: @"log"]];
        [self errorFileChosen: [[self windowController] errorsPopUp]];
    }
    if([TS respondsToSelector: @selector(rangeOfErrorFromIndex:cycle:backwards:)])
    {
        NSRange range = [TS rangeOfErrorFromIndex: [TV selectedRange].location-1 cycle: YES backwards: YES];
        [TV setSelectedRange: range];
        [TV scrollRangeToVisible: [TV selectedRange]];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= scrollToNextError:
- (void) scrollToNextError: (id) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSTextView * TV = [[self windowController] errorsTextView];
    iTMTextStorage * TS = [TV textStorage];
    if(![TS respondsToSelector: @selector(rangeOfErrorFromIndex:cycle:backwards:)])
    {
        [self fixErrorsPopUp];
        [[[self windowController] errorsPopUp] selectItemWithTitle:
                [[self relativeRoot] stringByAppendingPathExtension: @"log"]];
        [self errorFileChosen: [[self windowController] errorsPopUp]];
    }
    if([TS respondsToSelector: @selector(rangeOfErrorFromIndex:cycle:backwards:)])
    {
        NSRange range = [TS rangeOfErrorFromIndex: NSMaxRange([TV selectedRange]) cycle: YES backwards: NO];
        [TV setSelectedRange: range];
        [TV scrollRangeToVisible: [TV selectedRange]];
        [TV display];
    }
    return;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateTypesetTextView
- (void) validateTypesetTextView;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([self isCustomTypeset])
    {
        [[self typesetTextView] setEditable: YES];
        [[self typesetTextView] setSelectedRange: NSMakeRange(0, [[[self typesetTextView] string] length])];
        [[self typesetTextView] insertText: [[self projectDictionary] stringForKey: iTMPDTypesetScriptKey]];
        [[self typesetTextView] setSelectedRange: NSMakeRange([[[self typesetTextView] string] length],0)];
        [[self typesetTextView] scrollRangeToVisible: [[self typesetTextView] selectedRange]];
        [[[self typesetTextView] window] makeFirstResponder: typesetTextView];
    }
    else
    {
        [[self typesetTextView] setEditable: NO];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateCompileTextView
- (void) validateCompileTextView;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([self isCustomCompile])
    {
        [[self compileTextView] setEditable: YES];
        [[self compileTextView] setSelectedRange: NSMakeRange(0, [[[self compileTextView] string] length])];
        [[self compileTextView] insertText: [[self projectDictionary] stringForKey: iTMPDCompileScriptKey]];
        [[self compileTextView] setSelectedRange: NSMakeRange([[[self compileTextView] string] length],0)];
        [[self compileTextView] scrollRangeToVisible: [[self compileTextView] selectedRange]];
        [[[self compileTextView] window] makeFirstResponder: compileTextView];
    }
    else
    {
        [[self compileTextView] setEditable: NO];
    }
    return;
}
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= canShowFiles
- (BOOL) canShowFiles;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return YES;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= canShowTerminal
- (BOOL) canShowTerminal;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return ![[NSApp keyWindow] isEqual: [[self windowController] window]] ||
                ![[self windowController] alreadySwitchedToPaneWithIdentifier: iTMPWCTerminalIdentifier];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= canShowErrors
- (BOOL) canShowErrors;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self validateErrorFileChosen: [[self windowController] errorsPopUp]];
    return [[[self windowController] errorsPopUp] isEnabled] ||
        ![[NSApp keyWindow] isEqual: [[self windowController] window]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= canBibTeX
- (BOOL) canBibTeX;
/*"aux file needed.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * core = [[self absoluteInput] stringByDeletingPathExtension];
    NSString * aux = [core length]? [core stringByAppendingPathExtension: @"aux"]: nil;
    return [aux length]? [[NSFileManager defaultManager] fileExistsAtPath: aux]: NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= canMakeIndex
- (BOOL) canMakeIndex;
/*"ilg file needed.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * core = [[self absoluteInput] stringByDeletingPathExtension];
    NSString * idx = [core length]? [core stringByAppendingPathExtension: @"idx"]: nil;
    return [idx length]? [[NSFileManager defaultManager] fileExistsAtPath: idx]: NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= selectMainTabView()
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showTerminal:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showFiles:
#define selectMainTabView(selectAction, itemIdentifier)\
- (IBAction) selectAction: (id) sender;\
{\
    [self makeWindows];\
    [[self windowController] switchToPaneWithIdentifier: itemIdentifier];\
    [self showWindows];\
    return;\
}
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (10/04/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
//selectMainTabView(showTerminal, iTMPWCTerminalIdentifier);
selectMainTabView(showFiles, iTMPWCFilesIdentifier);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showTerminalInBackGround
- (void) showTerminalInBackGround;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (10/04/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    [self reallyMakeWindowControllers];
    [self makeWindows];
    {
        NSWindow * mainWindow = [NSApp mainWindow];
        NSWindow * window = [[self windowController] window];
        if(mainWindow && ![window isEqual: mainWindow] &&
                                [[self settings] boolForKey: iTMUDTerminalBehindKey])
            [window orderWindow: NSWindowBelow relativeTo: [mainWindow windowNumber]];
        else
            [self showWindows];
    }
    [[self windowController] switchToPaneWithIdentifier: iTMPWCTerminalIdentifier];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showProjectWindow:
- (IBAction) showProjectWindow: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (10/04/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self makeWindows];
    [self showWindows];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showProject:
- (IBAction) showProject: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (10/04/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    [self reallyMakeWindowControllers];
    [self makeWindows];
    if([[[self windowController] window] isEqual: [NSApp mainWindow]])
        [[self windowController] switchToPaneWithIdentifier: iTMPWCProjectIdentifier];
    [self showWindows];
    [self validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showErrors:
- (IBAction) showErrors: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (10/04/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self reallyMakeWindowControllers];
    [self makeWindows];
    [self fixErrorsPopUp];
    if(![[self windowController] alreadySwitchedToPaneWithIdentifier: iTMPWCErrorsIdentifier])
    {
        [[self windowController] switchToPaneWithIdentifier: iTMPWCErrorsIdentifier];
        [self errorFileChosen: [[self windowController] errorsPopUp]];
    }
    [self showWindows];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  currentErrorPath
- (id) currentErrorPath
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _CEP;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setCurrentErrorPath:
- (void) setCurrentErrorPath: (id) argument
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSString class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSString argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else if(![_CEP isEqual: argument])
    {
        [_CEP autorelease];
        _CEP = [argument retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= errorFileChosen:
- (void) errorFileChosen: (NSPopUpButton *) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * path = [[self absoluteDirectory] stringByAppendingPathComponent: [sender titleOfSelectedItem]];
    if([[NSFileManager defaultManager] isReadableFileAtPath: path])
    {
        NSString * string = [NSString stringWithContentsOfFile: path];
        iTMTextStorage * TS = [[[iTMTextStorage alloc] initWithString: string attributes: nil] autorelease];
        [self setCurrentErrorPath: path];
        [[[[self windowController] errorsTextView] layoutManager] replaceTextStorage: TS];
        [TS replaceSyntaxParser: ([[self settings] boolForKey: iTMUDLogSyntaxColoringKey]?
            [iTMLogSyntaxParser syntaxParser]: [iTMSyntaxParser syntaxParser])];
        [TS invalidateAttributesInRange: NSMakeRange(0, [TS length])];
     }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateErrorFileChosen:
- (void) validateErrorFileChosen: (NSPopUpButton *) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    BOOL senderFlag = NO;
    NSEnumerator * enumerator = [[sender itemArray] objectEnumerator];
    NSMenuItem * MI;
    while(MI = [enumerator nextObject])
    {
        NSString * path = [[self absoluteDirectory] stringByAppendingPathComponent: [MI title]];
        NSFileManager * FM = [NSFileManager defaultManager];
        BOOL isDirectory;
        BOOL flag = ([path length] > 0) &&
                                [FM fileExistsAtPath: path isDirectory: &isDirectory] &&
        						[FM isReadableFileAtPath: path] && !isDirectory;
        [MI setEnabled: flag];
        senderFlag |= flag;
    }
    [sender setEnabled: senderFlag];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setCompileScript:
- (void) setCompileScript: (NSString *) argument;
/*"For the '&format' header, see projectForDocument somewhere.
Version History: jerome.laurens@u-bourgogne.fr (11/18/01)
- < 1.1: 03/10/2002
To Do List: no undo here!!!
"*/
{
    [[self projectDictionary] setObject: argument forKey: iTMPDCompileScriptKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setTypesetScript:
- (void) setTypesetScript: (NSString *) argument;
/*"For the '&format' header, see projectForDocument somewhere.
Version History: jerome.laurens@u-bourgogne.fr (11/18/01)
- < 1.1: 03/10/2002
To Do List: no undo here!!!
"*/
{
    [[self projectDictionary] setObject: argument forKey: iTMPDTypesetScriptKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= comboBox:indexOfItemWithStringValue:
- (unsigned int) comboBox: (NSComboBox *) aCB indexOfItemWithStringValue: (NSString *) aString;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([aCB action] == @selector(engineEdited:))
        return [[[iTMTeXEngineManager sharedTeXEngineManager] engines] indexOfObject: aString];
    else
        return [[[iTMTeXEngineManager sharedTeXEngineManager] formatsForEngine: [self stringForKey: iTMPDCompileEngineKey]]
            indexOfObject: aString];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= comboBox:objectValueForItemAtIndex:
- (id) comboBox: (NSComboBox *) aCB objectValueForItemAtIndex: (int) index;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([aCB action] == @selector(engineEdited:))
    {
        NSArray * engines = [[iTMTeXEngineManager sharedTeXEngineManager] engines];
        if((index>=0) && (index<[engines count]))
            return [engines objectAtIndex: index];
        else
            NSLog(@"out of engines bounds: %d, %d", index, [engines count]);
    }
    else
    {
        NSArray * formats = [[iTMTeXEngineManager sharedTeXEngineManager]
                                    formatsForEngine: [self stringForKey: iTMPDCompileEngineKey]];
        if((index>=0) && (index<[formats count]))
            return [formats objectAtIndex: index];
        else
            NSLog(@"out of formats bounds: %d, %d", index, [formats count]);
    }
    return nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= numberOfItemsInComboBox:
- (int) numberOfItemsInComboBox: (NSComboBox *) aCB;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([aCB action] == @selector(engineEdited:))
        return [[[iTMTeXEngineManager sharedTeXEngineManager] engines] count];
    else
        return [[[iTMTeXEngineManager sharedTeXEngineManager]
            formatsForEngine: [self stringForKey: iTMPDCompileEngineKey]] count];
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  comboBox:completedString:
- (NSString *) comboBox: (NSComboBox *) aCB completedString: (NSString *) aString;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSEnumerator * enumerator;
    if([aCB isEqual: [self engineCombo]])
        enumerator = [[[iTMTeXEngineManager sharedTeXEngineManager] engines] objectEnumerator];
    else if(([aCB isEqual: [self formatCombo]]) || ([aCB isEqual: [self baseCombo]]))
        enumerator = [[[iTMTeXEngineManager sharedTeXEngineManager]
            formatsForEngine: [[self compileDescriptor] engine]] objectEnumerator];
    else
        return [NSString string];
    {
        NSString * object;
        NSString * result = [NSString string];
        while(object = [enumerator nextObject])
            if(([object hasPrefix: aString]))
                if([result length]>0)
                    result = [object commonPrefixWithString: result options: NSLiteralSearch];
                else
                    result = object;
        return [[result copy] autorelease];
    }
}
#endif
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMProjectDocument

