/*
//  iTMProjectEngineer.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sun Dec 02 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSBundle.h>
#import <Foundation/Foundation.h>
#import <AppKit/NSDocumentController.h>
#import <AppKit/NSApplication.h>
#import <AppKit/AppKit.h>
#import "NSDocumentController(iTeXMac).h"
#import "iTMProjectEngineer.h"
#import "iTMStatusNotification.h"
#import "iTMProjectDocument.h"
#import "iTMProjectWindowController.h"
#import "iTMProjectDocument(Accessors).h"
#import "iTMProjectDocument(Navigation).h"
#import "iTMProjectsManager.h"
#import "iTMTerminalController.h"
#import "iTMHelpersController.h"
#import "iTMTeTeXController.h"

NSString * const iTMTTProjectKey = @"project";
NSString * const iTMUDIsCompilingKey = @"iTMIsCompiling";

@interface NSDocumentController(iTMProjectEngineer)
- (BOOL) usesPDFHelper;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMProjectEngineer
/*"Description forthcoming."*/
@implementation iTMProjectEngineer
static id _iTMProjectEngineer;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults:
        [NSDictionary dictionaryWithObjectsAndKeys:
            [NSNumber numberWithBool: NO], iTMUDIsCompilingKey,
                nil]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sharedProjectEngineer
+ (id) sharedProjectEngineer;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _iTMProjectEngineer? _iTMProjectEngineer:
                        (_iTMProjectEngineer = [[self allocWithZone: [NSApp zone]] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"The first object inited is the shared one.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(_iTMProjectEngineer)
    {
        if(![_iTMProjectEngineer isEqual: self])
            [super dealloc];
        return [_iTMProjectEngineer retain];
    }
    else
    {
        return _iTMProjectEngineer = [super init];
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  pathForAuxiliaryExecutable
- (NSString *) pathForAuxiliaryExecutable: (NSString *) argument;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    NSString * result = [[NSBundle bundleForClass: [self class]] pathForAuxiliaryExecutable: argument];
    if(![result length])
	result = [[NSBundle bundleForClass: [self class]] pathForResource: argument ofType: nil];
    if(![result length])
        NSLog(@"No auxiliary executable for: %@");
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  envDictionaryByCompletingEnvDictionary:
- (NSDictionary *) envDictionaryByCompletingEnvDictionary: (NSDictionary *) aDictionary;
/*"The sender is the document, project to index.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    NSMutableDictionary * result = [NSMutableDictionary dictionaryWithCapacity: 3];
    [result setObject: [iTMHelpersController auxiliaryExecutablesPath] forKey: @"iTMAuxiliaryExecutablesPATH"];
    [result setObject: [[iTMTeTeXController sharedTeTeXController] GSBinariesPath] forKey: @"iTMGSBinariesPATH"];
    [result setObject: [[iTMTeTeXController sharedTeTeXController] TeTeXBinariesPath] forKey: @"iTMTeTeXBinariesPATH"];
    [result addEntriesFromDictionary: aDictionary];
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  makeIndex:
- (void) makeIndex: (id) sender;
/*"The sender is the document, project to index.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    [[NSDocumentController sharedDocumentController] saveAllDocumentsWithDelegate: self
        didSaveAllSelector: @selector(documentController:receiverCanMakeIndex:contextInfo:) contextInfo: sender];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  bibTeX:
- (void) bibTeX: (id) sender;
/*"The sender is the document, project to bibtex.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    [[NSDocumentController sharedDocumentController] saveAllDocumentsWithDelegate: self
        didSaveAllSelector: @selector(documentController:receiverCanBibTeX:contextInfo:) contextInfo: sender];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  compile:
- (void) compile: (id) sender;
/*"The sender is the document, project to compile.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSDocumentController sharedDocumentController] saveAllDocumentsWithDelegate: self
        didSaveAllSelector: @selector(documentController:receiverCanCompile:contextInfo:) contextInfo: sender];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  typeset:
- (void) typeset: (id) sender;
/*"The sender is the document, project to typeset.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSDocumentController sharedDocumentController] saveAllDocumentsWithDelegate: self
        didSaveAllSelector: @selector(documentController:receiverCanTypeset:contextInfo:) contextInfo: sender];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  documentController:receiverCanMakeIndex:contextInfo:
- (void) documentController: (id) DC receiverCanMakeIndex: (BOOL) flag contextInfo: (void *) contextInfo;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    iTMProjectDocument * project = [[iTMProjectsManager defaultManager] projectForSource: contextInfo];
    NSString * status = [NSString stringWithFormat:
        NSLocalizedStringFromTableInBundle(@"Indexing %@ with %@...", @"Project",
                    [NSBundle bundleForClass: [self class]], "format with 2 %@"),
            [[project absoluteInput] lastPathComponent], [project coreFileName]];
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: status];
    [project makeWindows];
    [project showTerminalInBackGround];
    [project prepareProjectDictionaryForSaving];
    [[project terminal] addTaskWithLaunchPath: [iTMHelpersController indexMakerPath]
        environment: [self envDictionaryByCompletingEnvDictionary: [project environmentDictionary]]
            arguments: nil
                directory: [project absoluteDirectory]
                    delegate: self
                        didExecuteSelector: @selector(terminalDidMakeIndex:userInfo:)
                            userInfo: [NSDictionary dictionaryWithObject: project forKey: iTMTTProjectKey]
                                restart: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  terminalDidMakeIndex:userInfo
- (void) terminalDidMakeIndex: (id) aTerminal userInfo: (NSDictionary *) userInfo;
/*"Here come the actions to be performed when the makeIndex task has completed.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    NSDictionary * environment = [[userInfo objectForKey: iTMTCTaskKey] environment];
    iTMProjectDocument * project = [[userInfo objectForKey: iTMTCUserInfoKey] objectForKey: iTMTTProjectKey];
    NSString * input = [[environment objectForKey: iTMPDRootInputKey] lastPathComponent];
    NSString * status = [NSString stringWithFormat:
        NSLocalizedStringFromTableInBundle(@"Indexing %@ with %@ complete.", @"Project",
                    [NSBundle bundleForClass: [self class]], "format with 2 %@"),
            input, [project coreFileName]];
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: status];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  documentController:receiverCanBibTeX:contextInfo:
- (void) documentController: (id) DC receiverCanBibTeX: (BOOL) flag contextInfo: (void *) contextInfo;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    iTMProjectDocument * project = [[iTMProjectsManager defaultManager] projectForSource: contextInfo];
    NSString * status = [NSString stringWithFormat:
        NSLocalizedStringFromTableInBundle(@"Bibliography of %@ with %@...", @"Project",
                    [NSBundle bundleForClass: [self class]], "format with 2 %@"),
            [[project absoluteInput] lastPathComponent], [project coreFileName]];
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: status];
    [project makeWindows];
    [project showTerminalInBackGround];
    [project prepareProjectDictionaryForSaving];
//    NSLog([iTMHelpersController bibliographyMakerPath]);
//    NSLog(@"terminalWillBibTeX: %@", [project environmentDictionary]);
    [[project terminal] addTaskWithLaunchPath: [iTMHelpersController bibliographyMakerPath]
        environment: [self envDictionaryByCompletingEnvDictionary: [project environmentDictionary]]
            arguments: nil
                directory: [project absoluteDirectory]
                    delegate: self
                        didExecuteSelector: @selector(terminalDidBibTeX:userInfo:)
                            userInfo: [NSDictionary dictionaryWithObject: project forKey: iTMTTProjectKey]
                                restart: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  terminalDidBibTeX:userInfo
- (void) terminalDidBibTeX: (id) aTerminal userInfo: (NSDictionary *) userInfo;
/*"Here come the actions to be performed when the BibTeX task has completed.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    NSDictionary * environment = [[userInfo objectForKey: iTMTCTaskKey] environment];
    iTMProjectDocument * project = [[userInfo objectForKey: iTMTCUserInfoKey] objectForKey: iTMTTProjectKey];
    NSString * extension = [environment objectForKey: iTMPDInputExtensionKey];
    NSString * input = [[environment objectForKey: iTMPDRootInputKey] lastPathComponent];
    NSString * blgPath = [[environment objectForKey: iTMPDDirectoryKey] stringByAppendingPathComponent:
                                [[environment objectForKey: iTMPDRootInputKey] stringByAppendingPathExtension: @"blg"]];
    int NOR = [[self class] numberOfErrorsInBlgAtPath: blgPath];
    NSString * status = [NSString stringWithFormat:
        NSLocalizedStringFromTableInBundle(@"Bibliography of %@ with %@ complete: %d error(s).", @"Project",
                    [NSBundle bundleForClass: [self class]], "format with 2 %@ and %d"),
            ([extension length]? [input stringByAppendingPathExtension: extension]: input), [project coreFileName], NOR];
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: status];
//    NSLog(@"terminalDidBibTeX: %@", environment);
    if(NOR)
    {
        [[project windowController] switchToPaneWithIdentifier: iTMPWCErrorsIdentifier];
        [[[project windowController] errorsPopUp] selectItemWithTitle: [blgPath lastPathComponent]];
        [project errorFileChosen: [[project windowController] errorsPopUp]];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  documentController:receiverCanCompile:contextInfo:
- (void) documentController: (id) DC receiverCanCompile: (BOOL) flag contextInfo: (void *) contextInfo;
/*"Never dealloc.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMProjectDocument * project = [[iTMProjectsManager defaultManager] projectForSource: contextInfo];
    NSString * status = [NSString stringWithFormat:
        NSLocalizedStringFromTableInBundle(@"Compiling %@ with %@...", @"Project",
                    [NSBundle bundleForClass: [self class]], "format with 2 %@"),
            [[project absoluteInput] lastPathComponent], [project coreFileName]];
    [[NSUserDefaults standardUserDefaults] registerDefaults:
        [NSDictionary dictionaryWithObject: [NSNumber numberWithBool: YES] forKey: iTMUDIsCompilingKey]];
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: status];
    [project makeWindows];
    [project showTerminalInBackGround];
    [project prepareProjectDictionaryForSaving];
    [project readInputHeader];
    if([project isCustomCompile])
        [[project terminal] addShellTaskWithEnvironment:
            [self envDictionaryByCompletingEnvDictionary: [project environmentDictionary]]
                standardInput: [NSString stringWithFormat: @"source \"%@\"\n%@",
                        [iTMHelpersController pathMakerPath],
                        [[project environmentDictionary] objectForKey: iTMPDCompileScriptKey]]
                    directory: [project absoluteDirectory]
                        delegate: self
                            didExecuteSelector: @selector(terminalDidCompile:userInfo:)
                                userInfo: [NSDictionary dictionaryWithObject: project forKey: iTMTTProjectKey]
                                    restart: YES];
    else
        [[project terminal] addTaskWithLaunchPath: [iTMHelpersController compileMakerPath]
            environment: [self envDictionaryByCompletingEnvDictionary: [project environmentDictionary]]
                arguments: nil
                    directory: [project absoluteDirectory]
                        delegate: self
                            didExecuteSelector: @selector(terminalDidCompile:userInfo:)
                                userInfo: [NSDictionary dictionaryWithObject: project forKey: iTMTTProjectKey]
                                    restart: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  terminalDidCompile:userInfo
- (void) terminalDidCompile: (id) aTerminal userInfo: (NSDictionary *) userInfo;
/*"Here come the actions to be performed when the compile task has completed.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    NSDictionary * environment = [[userInfo objectForKey: iTMTCTaskKey] environment];
    iTMProjectDocument * project = [[userInfo objectForKey: iTMTCUserInfoKey] objectForKey: iTMTTProjectKey];
    NSString * extension = [environment objectForKey: iTMPDInputExtensionKey];
    NSString * directory = [environment objectForKey: iTMPDDirectoryKey];
    NSString * input = [[environment objectForKey: iTMPDRootInputKey] lastPathComponent];
    NSString * logPath = [directory stringByAppendingPathComponent: [input stringByAppendingPathExtension: @"log"]];
    int NOR = [[self class] numberOfErrorsInLogAtPath: logPath];
    {
        NSString * status = [NSString stringWithFormat:
            NSLocalizedStringFromTableInBundle(@"Compiling %@ with %@ complete: %d error(s).", @"Project",
                    [NSBundle bundleForClass: [self class]], "format with 2 %@ and %d"),
                ([extension length]? [input stringByAppendingPathExtension: extension]: input), [project coreFileName], NOR];
        [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: status];
    }
    if(NOR)
    {
        [project fixErrorsPopUp];
        [[[project windowController] errorsPopUp] selectItemWithTitle: [logPath lastPathComponent]];
        [project errorFileChosen: [[project windowController] errorsPopUp]];
        [[project windowController] switchToPaneWithIdentifier: iTMPWCErrorsIdentifier];
    }
    else
    // the same as below
    {
        NSString * extension = [environment objectForKey: iTMPDOutputExtensionKey];
        NSString * output = [environment objectForKey: iTMPDRootOutputKey];
        NSString * identifier = [environment objectForKey: iTMPDCustomIdentifierKey];
        NSString * outputFile = [identifier length]?
            [directory stringByAppendingPathComponent:
                [[output stringByAppendingPathExtension: identifier] stringByAppendingPathExtension: extension]]:
            [directory stringByAppendingPathComponent:
                [output stringByAppendingPathExtension: extension]];
        if([[NSDocumentController sharedDocumentController] respondsToSelector: @selector(usesPDFHelper)] &&
                [[NSDocumentController sharedDocumentController] usesPDFHelper])
            [[NSDocumentController sharedDocumentController] openDocumentWithContentsOfFile: outputFile display: YES];
        else
        {
            NSWindow * keyWindow = [NSApp keyWindow];
            id keyDoc = [[keyWindow windowController] document];
            id doc = [[NSDocumentController sharedDocumentController]
                                    openDocumentWithContentsOfFile: outputFile display: NO];
            if([doc respondsToSelector: @selector(update)])
                [doc performSelector: @selector(update)];
            if(!NOR && [keyDoc isKindOfClass: [iTMProjectDocument class]])
                [doc showWindows];
            else
            {
                NSEnumerator * enumerator;
                id WC;
                if(![[doc windowControllers] count]) [doc makeWindowControllers];
                enumerator = [[doc windowControllers] objectEnumerator];
                while(WC = [enumerator nextObject])
                    [[WC window] orderWindow: NSWindowBelow relativeTo: [keyWindow windowNumber]];
            }
        }
    }
    [[NSUserDefaults standardUserDefaults] registerDefaults:
        [NSDictionary dictionaryWithObject: [NSNumber numberWithBool: NO] forKey: iTMUDIsCompilingKey]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  documentController:receiverCanTypeset:contextInfo:
- (void) documentController: (id) DC receiverCanTypeset: (BOOL) flag contextInfo: (void *) contextInfo;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    iTMProjectDocument * project = [[iTMProjectsManager defaultManager] projectForSource: contextInfo];
    {
        NSString * status = [NSString stringWithFormat:
            NSLocalizedStringFromTableInBundle(@"Typesetting %@ with %@...", @"Project",
                    [NSBundle bundleForClass: [self class]], "format with 2 %@"),
                [[project absoluteInput] lastPathComponent], [project coreFileName]];
        [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: status];
    }
    // next disables the auto updates of the PDF window
    [[NSUserDefaults standardUserDefaults] registerDefaults:
        [NSDictionary dictionaryWithObject: [NSNumber numberWithBool: YES] forKey: iTMUDIsCompilingKey]];
    [project makeWindows];
    [project readInputHeader];
    [project showTerminalInBackGround];
    [project prepareProjectDictionaryForSaving];
    if([project isCustomTypeset])
        [[project terminal] addShellTaskWithEnvironment:
                                [self envDictionaryByCompletingEnvDictionary: [project environmentDictionary]]
            standardInput: [NSString stringWithFormat: @"source \"%@\"\n%@",
                        [iTMHelpersController pathMakerPath],
                        [[project environmentDictionary] objectForKey: iTMPDTypesetScriptKey]]
                    directory: [project absoluteDirectory]
                        delegate: self
                            didExecuteSelector: @selector(terminalDidTypeset:userInfo:)
                                userInfo: [NSDictionary dictionaryWithObject: project forKey: iTMTTProjectKey]
                                    restart: YES];
    else
        [[project terminal] addTaskWithLaunchPath: [iTMHelpersController typesetMakerPath]
            environment: [self envDictionaryByCompletingEnvDictionary: [project environmentDictionary]]
                arguments: nil
                    directory: [project absoluteDirectory]
                        delegate: self
                            didExecuteSelector: @selector(terminalDidTypeset:userInfo:)
                                userInfo:
                [NSDictionary dictionaryWithObject: project forKey: iTMTTProjectKey]
                                    restart: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  terminalDidTypeset:userInfo
- (void) terminalDidTypeset: (id) aTerminal userInfo: (NSDictionary *) userInfo;
/*"Here come the actions to be performed when the typeset task has completed.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    NSDictionary * environment = [[userInfo objectForKey: iTMTCTaskKey] environment];
    iTMProjectDocument * project = [[userInfo objectForKey: iTMTCUserInfoKey] objectForKey: iTMTTProjectKey];
    NSString * extension = [environment objectForKey: iTMPDInputExtensionKey];
    NSString * directory = [environment objectForKey: iTMPDDirectoryKey];
    NSString * input = [[environment objectForKey: iTMPDRootInputKey] lastPathComponent];
    NSString * logPath = [directory stringByAppendingPathComponent: [input stringByAppendingPathExtension: @"log"]];
    int NOR = [[self class] numberOfErrorsInLogAtPath: logPath];
    NSString * status = [NSString stringWithFormat:
        NSLocalizedStringFromTableInBundle(@"Typesetting %@ with %@ complete: %d error(s).", @"Project",
                    [NSBundle bundleForClass: [self class]], "format with 2 %@ and %d"),
            ([extension length]? [input stringByAppendingPathExtension: extension]: input), [project coreFileName], NOR];
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: status];
    if(NOR)
    {
        [project fixErrorsPopUp];
        [[[project windowController] errorsPopUp] selectItemWithTitle: [logPath lastPathComponent]];
        [project errorFileChosen: [[project windowController] errorsPopUp]];
        [[project windowController] switchToPaneWithIdentifier: iTMPWCErrorsIdentifier];
    }
    {
        NSString * extension = [environment objectForKey: iTMPDOutputExtensionKey];
        NSString * output = [environment objectForKey: iTMPDRootOutputKey];
        NSString * identifier = [environment objectForKey: iTMPDCustomIdentifierKey];
        NSString * outputFile = [identifier length]?
            [directory stringByAppendingPathComponent:
                [[output stringByAppendingPathExtension: identifier] stringByAppendingPathExtension: extension]]:
            [directory stringByAppendingPathComponent:
                [output stringByAppendingPathExtension: extension]];
        if([[NSDocumentController sharedDocumentController] respondsToSelector: @selector(usesPDFHelper)] &&
                [[NSDocumentController sharedDocumentController] usesPDFHelper])
            [[NSDocumentController sharedDocumentController] openDocumentWithContentsOfFile: outputFile display: YES];
        else
        {
            NSWindow * keyWindow = [NSApp keyWindow];
            id keyDoc = [[keyWindow windowController] document];
            id doc = [[NSDocumentController sharedDocumentController]
                                    openDocumentWithContentsOfFile: outputFile display: NO];
            if([doc respondsToSelector: @selector(update)])
                [doc performSelector: @selector(update)];
            if(!NOR && [keyDoc isKindOfClass: [iTMProjectDocument class]])
                [doc showWindows];
            else
            {
                NSEnumerator * enumerator;
                id WC;
                if(![[doc windowControllers] count]) [doc makeWindowControllers];
                enumerator = [[doc windowControllers] objectEnumerator];
                while(WC = [enumerator nextObject])
                    [[WC window] orderWindow: NSWindowBelow relativeTo: [keyWindow windowNumber]];
            }
        }
    }
    [[NSUserDefaults standardUserDefaults] registerDefaults:
        [NSDictionary dictionaryWithObject: [NSNumber numberWithBool: NO] forKey: iTMUDIsCompilingKey]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  numberOfErrorsInLogAtPath:
+ (int) numberOfErrorsInLogAtPath: (NSString *) aPath;
/*"Reads the log, then returns the number of errors and warnings.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * log = [NSString stringWithContentsOfFile: aPath];
    if([log length])
    {
        int numberOfErrors = 0;
        NSScanner * S = [NSScanner scannerWithString: log];
        [S setCharactersToBeSkipped: [[NSCharacterSet characterSetWithCharactersInString: @"! \r\n"] invertedSet]];
        while([S scanUpToString: @"! " intoString: nil] && ![S isAtEnd])
        {
            unsigned scanLocation = [S scanLocation];
            unichar theChar = [log characterAtIndex: (scanLocation>0? scanLocation - 1: 0)];
            if((theChar == '\r') || (theChar == '\n') || (scanLocation == 0))
            {
                ++numberOfErrors;
                [S scanString: @"! " intoString: nil];
            }
            else
                break;
        }
        return numberOfErrors;
    }
    else
        return 0;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  numberOfErrorsInBlgAtPath:
+ (int) numberOfErrorsInBlgAtPath: (NSString *) aPath;
/*"Reads the log, then returns the number of errors and warnings.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * blg = [NSString stringWithContentsOfFile: aPath];
    if([blg length])
    {
        NSScanner * S = [NSScanner scannerWithString: blg];
        [S setCharactersToBeSkipped: [[NSCharacterSet characterSetWithCharactersInString: @"(There were"] invertedSet]];
        if([S scanUpToString: @"(There were " intoString: nil] && ![S isAtEnd])
        {
            int numberOfErrors;
            [S scanString: @"(There were " intoString: nil];
            [S setCharactersToBeSkipped: nil];
            if([S scanInt: &numberOfErrors])
                return numberOfErrors;
        }
    }
    return 0;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMenuItem:
- (BOOL) validateMenuItem: (id) anItem;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    SEL action = [anItem action];
    if(action == @selector(bibTeX:))
        return [[[iTMProjectsManager defaultManager] currentProject] canBibTeX];
    else if(action == @selector(makeIndex:))
        return [[[iTMProjectsManager defaultManager] currentProject] canMakeIndex];
    else if((action == @selector(typeset:)) || (action == @selector(compile:)))
        return [[NSFileManager defaultManager]
            isReadableFileAtPath: [[[iTMProjectsManager defaultManager] currentProject] absoluteInput]];
    else
        return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToolbarItem:
- (BOOL) validateToolbarItem: (id) sender;
/*" Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    SEL action = [sender action];
    if(action == @selector(typeset:))
        return ![[[[iTMProjectsManager defaultManager] currentProject] absoluteInput] hasSuffix: @".mp"];
    else if(action == @selector(compile:))
        return YES;//[[[iTMProjectsManager defaultManager] currentProject] canShowInput];
    else if(action == @selector(bibTeX:))
        return [[[iTMProjectsManager defaultManager] currentProject] canBibTeX];
    else if(action == @selector(makeIndex:))
        return [[[iTMProjectsManager defaultManager] currentProject] canMakeIndex];
    else
        return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  render:
- (void) render: (id) sender;
/*"The sender is the document, project to render.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMProjectDocument * project = [[iTMProjectsManager defaultManager] projectForSource: sender];
    NSString * status = [NSString stringWithFormat:
        NSLocalizedStringFromTableInBundle(@"Rendering %@ with %@...", @"Project",
                    [NSBundle bundleForClass: [self class]], "format with 2 %@"),
            [[project absoluteInput] lastPathComponent], [project coreFileName]];
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: status];
    [project makeWindows];
    [project showTerminalInBackGround];
    [project prepareProjectDictionaryForSaving];
    [project readInputHeader];
    {
        NSString * renderScriptKey;
        NSString * extension = [[[project absoluteInput] pathExtension] lowercaseString];
        if([extension isEqualToString: @"dvi"])
            renderScriptKey = iTMPDRenderDVIKey;
        else if([extension isEqualToString: @"ps"])
            renderScriptKey = iTMPDRenderPSKey;
        else if([extension isEqualToString: @"eps"])
            renderScriptKey = iTMPDRenderEPSKey;
        else
            return;
        [[project terminal] addShellTaskWithEnvironment: [project environmentDictionary]
            standardInput: [project stringForKey: renderScriptKey]
                directory: [project absoluteDirectory]
                    delegate: self
                        didExecuteSelector: @selector(terminalDidRender:userInfo:)
                            userInfo: [NSDictionary dictionaryWithObject: project forKey: iTMTTProjectKey]
                                restart: NO];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  terminalDidRender:userInfo
- (void) terminalDidRender: (id) aTerminal userInfo: (NSDictionary *) userInfo;
/*"Here come the actions to be performed when the typeset task has completed.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    NSString * input = [[[userInfo objectForKey: iTMTCUserInfoKey] objectForKey: iTMTTProjectKey] absoluteInput];
//    [[[NSDocumentController sharedDocumentController] documentForFileName: input] close];
    [[NSDocumentController sharedDocumentController] openDocumentWithContentsOfFile:
        [[input stringByDeletingPathExtension] stringByAppendingPathExtension: @"pdf"] display: YES];
    return;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMProjectEngineer

