/*
//  iTMProjectToolbarDelegate.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Thu Nov 27 2001.
//  Copyright  2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSArray.h>
#import <AppKit/NSImage.h>
#import <AppKit/NSToolbar.h>
#import <AppKit/NSToolbarItem.h>
#import "iTMProjectToolbarDelegate.h"
#import "iTMProjectsManager.h"
#import "iTMProjectEngineer.h"
#import "iTMHelpManager.h"

NSString * const iTMToolbarTerminalItemIdentifier = @"showTerminal:";
NSString * const iTMToolbarProjectItemIdentifier = @"showProject:";
NSString * const iTMToolbarProjectWindowItemIdentifier = @"showProjectWindow:";
NSString * const iTMToolbarSourceItemIdentifier = @"showInput:";
NSString * const iTMToolbarOutputItemIdentifier = @"showOutput:";
NSString * const iTMToolbarFilesItemIdentifier = @"showFiles:";
NSString * const iTMToolbarErrorsItemIdentifier = @"showErrors:";
NSString * const iTMToolbarTypesetItemIdentifier = @"typeset:";
NSString * const iTMToolbarCompileItemIdentifier = @"compile:";
NSString * const iTMToolbarBibTeXItemIdentifier = @"bibTeX:";
NSString * const iTMToolbarMakeIndexItemIdentifier = @"makeIndex:";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMProjectToolbarDelegate
/*"Description forthcoming."*/
@implementation iTMProjectToolbarDelegate;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sharedInstance
static id _SharedInstance;
+ (id) sharedInstance;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr 
- 1.1: 09/02/2001
To Do List:
"*/
{
    return _SharedInstance? _SharedInstance: (_SharedInstance = [[self alloc] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toolbar:itemForItemIdentifier:willBeInsertedIntoToolbar:
- (NSToolbarItem *) toolbar: (NSToolbar *) aToolbar itemForItemIdentifier: (NSString *) anItemIdentifier willBeInsertedIntoToolbar: (BOOL) aFlag;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr 
- 1.1: 09/02/2001
To Do List:
"*/
{
    NSBundle * bundle = [NSBundle bundleForClass: [self class]];
    #define IMAGENAMED(name) [[[NSImage alloc] initByReferencingFile: [bundle pathForImageResource: name]] autorelease]
    NSToolbarItem * result = [[[NSToolbarItem alloc] initWithItemIdentifier: anItemIdentifier] autorelease];
    if([anItemIdentifier isEqualToString: iTMToolbarTerminalItemIdentifier])
    {
        [result setImage: IMAGENAMED(@"Terminal")];
        [result setTarget: [iTMProjectsManager defaultManager]];
    }
    else if([anItemIdentifier isEqualToString: iTMToolbarProjectItemIdentifier])
    {
        [result setImage: IMAGENAMED(@"Project")];
        [result setTarget: [iTMProjectsManager defaultManager]];
    }
    else if([anItemIdentifier isEqualToString: iTMToolbarProjectWindowItemIdentifier])
    {
        [result setImage: IMAGENAMED(@"ProjectWindow")];
        [result setTarget: [iTMProjectsManager defaultManager]];
    }
    else if([anItemIdentifier isEqualToString: iTMToolbarSourceItemIdentifier])
    {
        [result setImage: IMAGENAMED(@"TeX")];
        [result setTarget: [iTMProjectsManager defaultManager]];
    }
    else if([anItemIdentifier isEqualToString: iTMToolbarOutputItemIdentifier])
    {
        [result setImage: IMAGENAMED(@"PDF")];
        [result setTarget: [iTMProjectsManager defaultManager]];
    }
    else if([anItemIdentifier isEqualToString: iTMToolbarFilesItemIdentifier])
    {
        [result setImage: IMAGENAMED(@"Files")];
        [result setTarget: [iTMProjectsManager defaultManager]];
    }
    else if([anItemIdentifier isEqualToString: iTMToolbarErrorsItemIdentifier])
    {
        [result setImage: IMAGENAMED(@"Errors")];
        [result setTarget: [iTMProjectsManager defaultManager]];
    }
    else if([anItemIdentifier isEqualToString: iTMToolbarTypesetItemIdentifier])
    {
        [result setImage: IMAGENAMED(@"T")];
        [result setTarget: [iTMProjectEngineer sharedProjectEngineer]];
    }
    else if([anItemIdentifier isEqualToString: iTMToolbarCompileItemIdentifier])
    {
        [result setImage: IMAGENAMED(@"C")];
        [result setTarget: [iTMProjectEngineer sharedProjectEngineer]];
    }
    else if([anItemIdentifier isEqualToString: iTMToolbarBibTeXItemIdentifier])
    {
        [result setImage: IMAGENAMED(@"B")];
        [result setTarget: [iTMProjectEngineer sharedProjectEngineer]];
    }
    else if([anItemIdentifier isEqualToString: iTMToolbarMakeIndexItemIdentifier])
    {
        [result setImage: IMAGENAMED(@"I")];
        [result setTarget: [iTMProjectEngineer sharedProjectEngineer]];
    }
    else if([anItemIdentifier isEqualToString: iTMToolbarShowHelpItemIdentifier])
    {
        return (NSToolbarItem *) [[iTMHelpManager sharedManager] helpToolbarItem];
    }
    else
        return nil;
    [result setAction: NSSelectorFromString(anItemIdentifier)];
    [result setToolTip: NSLocalizedStringFromTableInBundle(anItemIdentifier, @"ProjectToolbarToolTip", bundle, "")];
    [result setPaletteLabel: NSLocalizedStringFromTableInBundle(anItemIdentifier, @"ProjectToolbarPaletteLabel", bundle, "")];
    [result setLabel: NSLocalizedStringFromTableInBundle(anItemIdentifier, @"ProjectToolbarLabel", bundle, "")];
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toolbarDefaultItemIdentifiers:
- (NSArray *) toolbarDefaultItemIdentifiers: (NSToolbar*) aToolbar;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr 
- 1.1: 09/02/2001
To Do List:
"*/
{
    return [NSArray arrayWithObjects:
                NSToolbarPrintItemIdentifier,
                NSToolbarSeparatorItemIdentifier,
                iTMToolbarTerminalItemIdentifier,
                iTMToolbarErrorsItemIdentifier,
                iTMToolbarFilesItemIdentifier,
                iTMToolbarProjectItemIdentifier,
                iTMToolbarSourceItemIdentifier,
                iTMToolbarOutputItemIdentifier,
                NSToolbarSeparatorItemIdentifier,
                NSToolbarFlexibleSpaceItemIdentifier,
                iTMToolbarTypesetItemIdentifier,
                iTMToolbarCompileItemIdentifier,
                iTMToolbarBibTeXItemIdentifier,
                iTMToolbarMakeIndexItemIdentifier,
                NSToolbarFlexibleSpaceItemIdentifier,
                NSToolbarSeparatorItemIdentifier,
                iTMToolbarShowHelpItemIdentifier,
                    nil];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toolbarAllowedItemIdentifiers:
- (NSArray *) toolbarAllowedItemIdentifiers: (NSToolbar*) aToolbar;
/*" Apple comment of the NSToolbar header: Required method.  Returns the list of all allowed items by identifier.  By default, the toolbar does not assume any items are allowed, even the separator.  So, every allowed item must be explicitly listed.  The set of allowed items is used to construct the customization palette.  The order of items does not necessarily guarantee the order of appearance in the palette.  At minimum, you should return the default item list. End of comment.*/
{
    return [NSArray arrayWithObjects:
                NSToolbarPrintItemIdentifier,
                NSToolbarFlexibleSpaceItemIdentifier,
                NSToolbarSpaceItemIdentifier,
                NSToolbarSeparatorItemIdentifier,
                iTMToolbarTerminalItemIdentifier,
                iTMToolbarErrorsItemIdentifier,
                iTMToolbarFilesItemIdentifier,
                iTMToolbarProjectItemIdentifier,
                iTMToolbarSourceItemIdentifier,
                iTMToolbarOutputItemIdentifier,
                iTMToolbarTypesetItemIdentifier,
                iTMToolbarCompileItemIdentifier,
                iTMToolbarBibTeXItemIdentifier,
                iTMToolbarMakeIndexItemIdentifier,
                iTMToolbarShowHelpItemIdentifier,
                    nil];
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMProjectToolbarDelegate

