/*
//  iTMProjectWindowController.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Nov 27 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSException.h>
#import <AppKit/NSWindowController.h>
#import <AppKit/NSToolbar.h>
#import <AppKit/NSView.h>
#import <AppKit/NSTabView.h>
#import <AppKit/NSTextView.h>
#import <AppKit/NSTextField.h>
#import <AppKit/NSPopUpButton.h>
#import "NSControl(iTeXMac).h"
#import "NSWindow(iTeXMac).h"
#import "NSUserDefaults(iTeXMac).h"
#import "iTMProjectWindowController.h"
#import "iTMProjectToolbarDelegate.h"
#import "iTMTextFinder.h"
#import "iTMTextStorage.h"
#import "iTMSyntaxParser.h"
#import "iTMLogSyntaxParser.h"
#import "iTMSettings.h"

// main tab view
NSString * const iTMPWCProjectIdentifier = @"Project";
NSString * const iTMPWCTerminalIdentifier = @"Terminal";
NSString * const iTMPWCFilesIdentifier = @"Files";
NSString * const iTMPWCErrorsIdentifier = @"Errors";
// project tab view
NSString * const iTMPWCInputIdentifier = @"Input";
NSString * const iTMPWCCompileIdentifier = @"Compile";
NSString * const iTMPWCCustomCompileIdentifier = @"CustomCompile";
NSString * const iTMPWCBuiltInCompileIdentifier = @"BuiltInCompile";
NSString * const iTMPWCTypesetIdentifier = @"Typeset";
NSString * const iTMPWCCustomTypesetIdentifier = @"CustomTypeset";
NSString * const iTMPWCBuiltInTypesetIdentifier = @"BuiltInTypeset";
NSString * const iTMPWCMakeIndexIdentifier = @"MakeIndex";
NSString * const iTMPWCBibTeXIdentifier = @"BibTeX";
NSString * const iTMPWCRenderIdentifier = @"Render";
NSString * const iTMPWCHelpersIdentifier = @"Helpers";
NSString * const iTMPWCMacrosIdentifier = @"Macros";

NSString * const iTMUDLogDrawsTextBackgroundKey = @"iTMUDLogDrawsTextBackground";
NSString * const iTMUDLogSyntaxColoringKey = @"iTMLogSyntaxColoring";

NSString * const iTMUDLogTextBackgroundColorKey = @"iTMUDLogTextBackgroundColor";
NSString * const iTMUDLogSelectedTextBackgroundColorKey = @"iTMUDLogSelectedTextBackgroundColor";
NSString * const iTMUDLogSelectedTextColorKey = @"iTMUDLogSelectedTextColor";
NSString * const iTMUDLogInsertionPointColorKey = @"iTMUDLogInsertionPointColor";

NSString * const iTMUDLogStandardFontKey = @"iTMUDLogStandardFont";
NSString * const iTMUDLogStandardColorKey = @"iTMUDLogStandardColor";

NSString * const iTMUDProjectCascadeWindowsKey = @"iTMUDProlectCascadeWindows";

@interface iTMProjectWindowController(Toolbar)
- (Class) projectToolbarDelegateClass;
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMProjectWindowController
/*"Description forthcoming."*/
@implementation iTMProjectWindowController
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialize
+ (void) initialize;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
        [[NSFont fontWithName: @"Monaco" size: 12.0] nameSizeDictionary], iTMUDLogStandardFontKey,
        [[NSColor textColor] RGBADictionary], iTMUDLogStandardColorKey,
        [[NSColor textBackgroundColor] RGBADictionary], iTMUDLogTextBackgroundColorKey,
        [[NSColor selectedTextBackgroundColor] RGBADictionary], iTMUDLogSelectedTextBackgroundColorKey,
        [[NSColor selectedTextColor] RGBADictionary], iTMUDLogSelectedTextColorKey,
        [[NSColor blackColor] RGBADictionary], iTMUDLogInsertionPointColorKey,
        [NSNumber numberWithBool: NO], iTMUDLogDrawsTextBackgroundKey,
        [NSNumber numberWithBool: YES], iTMUDLogSyntaxColoringKey,
        [NSNumber numberWithBool: NO], iTMUDProjectCascadeWindowsKey,
            nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initWithWindow:
- (id) initWithWindow: (NSWindow *) window;
/*"Designated initializer.
Version history: jerome.laurens@u-bourgogne.fr 
- 1.1.a6 : 03/27/2002
To Do List:
"*/
{
    if(self = [super initWithWindow: window])
    {
        #warning default HERE?
        [_CurrentCompileIdentifier release];
        _CurrentCompileIdentifier = [iTMPWCCustomCompileIdentifier retain];
        [_CurrentTypesetIdentifier release];
        _CurrentTypesetIdentifier = [iTMPWCCustomTypesetIdentifier retain];
        [_CurrentHelperIdentifier release];
        _CurrentHelperIdentifier = [iTMPWCBibTeXIdentifier retain];
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr 
- 1: 11/27/2001
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [self setToolbarDelegate: nil];
    [_CurrentCompileIdentifier release];
    [_CurrentTypesetIdentifier release];
    [_CurrentHelperIdentifier release];
    [self cleanOutlets];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  cleanOutlets
- (void) cleanOutlets;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setTypesetTextView: nil];
    [self setFileView: nil];
    [self setCustomCompileView: nil];
    [self setBuiltInCompileView: nil];
    [self setCustomTypesetView: nil];
    [self setBuiltInTypesetView: nil];
    [self setHelpersView: nil];
    [self setTerminalView: nil];
    [self setProjectView: nil];
    [self setFilesView: nil];
    [self setErrorsView: nil];
    [self setErrorsTextView: nil];
    [self setErrorsPopUp: nil];
    [self setMacrosView: nil];
    [self setRenderView: nil];
    [self setMainTabView: nil];
    [self setProjectTabView: nil];
    [self setHelpersTabView: nil];
    [self setTerminalInput: nil];
    [self setTerminalOutput: nil];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  _ToolbarDelegateClass
- (Class) _ToolbarDelegateClass;
/*"If the receiver responds to #{projectToolbarDelegateClass}, it returns the result of this message.
Otherwise, it returns the default value: iTMProjectToolbarDelegate
Version history: jerome.laurens@u-bourgogne.fr 
- 1: 11/27/2001
To Do List:
"*/
{
    if([self respondsToSelector: @selector(projectToolbarDelegateClass)])
        return [self projectToolbarDelegateClass];
    else
        return [iTMProjectToolbarDelegate class];        
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toolbarDelegate
- (id) toolbarDelegate;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!_TD)
        [self setToolbarDelegate: [[[[self _ToolbarDelegateClass] alloc] init] autorelease]];
    return _TD;        
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setToolbarDelegate:
- (void) setToolbarDelegate: (id) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr 
- 1: 11/27/2001
To Do List:
"*/
{
    [_TD autorelease];
    _TD = [argument retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toolbarIdentifier
- (NSString *) toolbarIdentifier;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return @"Project";
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  _InstallToolbarWithIdentifier:
- (void) installToolbarWithIdentifier: (NSString *) anIdentifier;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([self toolbarDelegate])
    {
        id toolbar = [[NSToolbar alloc] initWithIdentifier: anIdentifier];
        [toolbar setDelegate: [self toolbarDelegate]];
        [toolbar setAllowsUserCustomization: YES];
        [toolbar setAutosavesConfiguration: YES];
        [toolbar setDisplayMode: NSToolbarDisplayModeIconOnly];
        [[self window] setToolbar: toolbar];
        [toolbar release];
//        NSLog(@"The toolbar has been installed for identifier: %@", anIdentifier);
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowDidLoad
- (void) windowDidLoad;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr 
- 1: 11/27/2001
To Do List:.
"*/
{
    [super windowDidLoad];
    {
        NSWindow * W = [self window];
        NSString * identifier = [[W class] frameIdentifier];
        identifier = [identifier length]? identifier: NSStringFromClass([self class]);
        [W setFrameFromStandardUserDefaults];
        [[iTMTextFinder sharedTextFinder] installFindResponderForWindow: W];
        if([self respondsToSelector: @selector(installToolbarWithIdentifier:)])
            [self installToolbarWithIdentifier: [self toolbarIdentifier]];
    }
    {
        int PTVIC = [[self projectTabView] numberOfTabViewItems];
        int MTVIC = [[self mainTabView] numberOfTabViewItems];
        {
            NSTabViewItem * TVI;
            //beware the next key view is used to point to the first responder candidate. It does not work for NSComboBox.
            #define installItem(identifier, view, tabView)\
            TVI = [[NSTabViewItem alloc] initWithIdentifier: identifier];\
            [TVI setView: [self view]];\
            {\
                NSView * firstResponder = [[self view] nextKeyView];\
                if(firstResponder)\
                {\
                    [TVI setInitialFirstResponder: firstResponder];\
                    [[self view] setNextKeyView: nil];\
                }\
            }\
            [[self tabView] addTabViewItem: TVI];\
            [TVI release];
            installItem(iTMPWCInputIdentifier, fileView, projectTabView);
            installItem(iTMPWCCustomTypesetIdentifier, customTypesetView, projectTabView);
            installItem(iTMPWCBuiltInTypesetIdentifier, builtInTypesetView, projectTabView);
            installItem(iTMPWCCustomCompileIdentifier, customCompileView, projectTabView);
            installItem(iTMPWCBuiltInCompileIdentifier, builtInCompileView, projectTabView);
            installItem(iTMPWCHelpersIdentifier, helpersView, projectTabView);
            installItem(iTMPWCMacrosIdentifier, macrosView, projectTabView);
            installItem(iTMPWCRenderIdentifier, renderView, projectTabView);
            installItem(iTMPWCTerminalIdentifier, terminalView, mainTabView);
            installItem(iTMPWCProjectIdentifier, projectView, mainTabView);
            installItem(iTMPWCFilesIdentifier, filesView, mainTabView);
            installItem(iTMPWCErrorsIdentifier, errorsView, mainTabView);
        }
        [[self projectTabView] selectTabViewItemWithIdentifier: iTMPWCInputIdentifier];
        [[self mainTabView] selectTabViewItemWithIdentifier: iTMPWCProjectIdentifier];
        // remove the tab view items previously added in IB (could not have 0 tab view item)
        while(MTVIC-->0)
            [[self mainTabView] removeTabViewItem: [[[self mainTabView] tabViewItems] objectAtIndex: 0]];
        while(PTVIC-->0)
            [[self projectTabView] removeTabViewItem: [[[self projectTabView] tabViewItems] objectAtIndex: 0]];
    }
    [self setShouldCascadeWindows: [[self settings] boolForKey: iTMUDProjectCascadeWindowsKey]];
    [self setShouldCloseDocument: YES];
    {
        iTMTextStorage * TS = [[[iTMTextStorage alloc] initWithString: [NSString string] attributes: nil] autorelease];
        [TS replaceSyntaxParser: ([[self settings] boolForKey: iTMUDLogSyntaxColoringKey]?
            [iTMLogSyntaxParser syntaxParser]: [iTMSyntaxParser syntaxParser])];
        [[[self errorsTextView] layoutManager] replaceTextStorage: TS];
    }
    {
        iTMTextStorage * TS = [[[iTMTextStorage alloc] initWithString: [NSString string] attributes: nil] autorelease];
        [TS replaceSyntaxParser: ([[self settings] boolForKey: iTMUDLogSyntaxColoringKey]?
            [iTMLogSyntaxParser syntaxParser]: [iTMSyntaxParser syntaxParser])];
        [[[self terminalOutput] layoutManager] replaceTextStorage: TS];
    }
    [self fontOrColorDidChangeNotified: nil];
//    [[self errorsTextView] setDelegate: [self document]];
//    [[self terminalOutput] setDelegate: [self document]];
//    NSLog(@"[[self errorsTextView] delegate]: %@", [[self errorsTextView] delegate]);
//    NSLog(@"[[self terminalOutput] delegate]: %@", [[self terminalOutput] delegate]);
    [[NSNotificationCenter defaultCenter] addObserver: self
        selector: @selector(installMenusInButtons:)
            name: @"iTMLoadMacrosAndTemplates"
                object: nil];
    [[NSNotificationCenter defaultCenter] addObserver: self
        selector: @selector(fontOrColorDidChangeNotified:)
            name: iTMFontOrColorDidChangeNotification
                object: nil];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMGetter()
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  typesetTextView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fileView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  customCompileView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  builtInCompileView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  customTypesetView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  builtInTypesetView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  bibTeXView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  makeIndexView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  terminalView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  projectView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  macrosView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  renderView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  mainTabView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  projectTabView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  helpersTabView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  terminalInput
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  terminalOutput
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  errorsTextView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  errorsPopUp
#define iTMGetter(source, getter) - (id) getter;{return source;}
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr 
- 1: 11/27/2001
To Do List:
"*/
iTMGetter(_TypesetTextView, typesetTextView);
iTMGetter(_ErrorsTextView, errorsTextView);
iTMGetter(_ErrorsPopUp, errorsPopUp);
iTMGetter(_FileView, fileView);
iTMGetter(_CustomCompileView, customCompileView);
iTMGetter(_BuiltInCompileView, builtInCompileView);
iTMGetter(_CustomTypesetView, customTypesetView);
iTMGetter(_BuiltInTypesetView, builtInTypesetView);
iTMGetter(_HelpersView, helpersView);
iTMGetter(_TerminalView, terminalView);
iTMGetter(_ProjectView, projectView);
iTMGetter(_FilesView, filesView);
iTMGetter(_ErrorsView, errorsView);
iTMGetter(_MacrosView, macrosView);
iTMGetter(_RenderView, renderView);
iTMGetter(_MainTabView, mainTabView);
iTMGetter(_ProjectTabView, projectTabView);
iTMGetter(_HelpersTabView, helpersTabView);
iTMGetter(_TerminalInput, terminalInput);
iTMGetter(_TerminalOutput, terminalOutput);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMStrongSetter()
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setFileView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setCustomCompileView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setBuiltInCompileView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setCustomTypesetView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setBuiltInTypesetView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setBibTeXView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMakeIndexView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setTerminalView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setProjectView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMainTabView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMacrosView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setRenderView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setProjectTabView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setHelpersTabView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setTerminalInput:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setTerminalOutput:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setErrorsTextView:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setTypesetTextView:
#define iTMStrongSetter(source, className, setter)\
- (void) setter: (id) argument;\
{\
    if(argument && ![argument isKindOfClass: [className class]])\
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] bad argument class: %@.",\
            [self class], NSStringFromSelector(_cmd), argument];\
    [source autorelease];\
    source = [argument retain];\
    return;\
}
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr 
- 1: 11/27/2001
To Do List:
"*/
iTMStrongSetter(_ErrorsTextView, NSTextView, setErrorsTextView);
iTMStrongSetter(_FileView, NSView, setFileView);
iTMStrongSetter(_CustomCompileView, NSView, setCustomCompileView);
iTMStrongSetter(_BuiltInCompileView, NSView, setBuiltInCompileView);
iTMStrongSetter(_CustomTypesetView, NSView, setCustomTypesetView);
iTMStrongSetter(_BuiltInTypesetView, NSView, setBuiltInTypesetView);
iTMStrongSetter(_HelpersView, NSView, setHelpersView);
iTMStrongSetter(_TerminalView, NSView, setTerminalView);
iTMStrongSetter(_ProjectView, NSView, setProjectView);
iTMStrongSetter(_FilesView, NSView, setFilesView);
iTMStrongSetter(_ErrorsView, NSView, setErrorsView);
iTMStrongSetter(_MacrosView, NSView, setMacrosView);
iTMStrongSetter(_RenderView, NSView, setRenderView);
iTMStrongSetter(_MainTabView, NSTabView, setMainTabView);
iTMStrongSetter(_ProjectTabView, NSTabView, setProjectTabView);
iTMStrongSetter(_HelpersTabView, NSTabView, setHelpersTabView);
iTMStrongSetter(_TerminalInput, NSTextField, setTerminalInput);
iTMStrongSetter(_TerminalOutput, NSTextView, setTerminalOutput);
iTMStrongSetter(_TypesetTextView, NSTextView, setTypesetTextView);
iTMStrongSetter(_ErrorsPopUp, NSPopUpButton, setErrorsPopUp);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  switchToPaneWithIdentifier:
- (IBAction) switchToPaneWithIdentifier: (NSString *) anIdentifier;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr 
- 1: 11/27/2001
To Do List:
"*/
{
    // main panes: Project, Terminal, Files, Errors
    if([anIdentifier isEqualToString: iTMPWCProjectIdentifier] ||
        [anIdentifier isEqualToString: iTMPWCTerminalIdentifier] ||
            [anIdentifier isEqualToString: iTMPWCFilesIdentifier] ||
                [anIdentifier isEqualToString: iTMPWCErrorsIdentifier])
        [[self mainTabView] selectTabViewItemWithIdentifier: anIdentifier];
    else if([anIdentifier isEqualToString: iTMPWCInputIdentifier])
    {
        [[self mainTabView] selectTabViewItemWithIdentifier: iTMPWCProjectIdentifier];
        [[self projectTabView] selectTabViewItemWithIdentifier: anIdentifier];
    }
    else if([anIdentifier isEqualToString: iTMPWCMacrosIdentifier])
    {
        [[self mainTabView] selectTabViewItemWithIdentifier: iTMPWCProjectIdentifier];
        [[self projectTabView] selectTabViewItemWithIdentifier: iTMPWCMacrosIdentifier];
    }
    else if([anIdentifier isEqualToString: iTMPWCRenderIdentifier])
    {
        [[self mainTabView] selectTabViewItemWithIdentifier: iTMPWCProjectIdentifier];
        [[self projectTabView] selectTabViewItemWithIdentifier: iTMPWCRenderIdentifier];
    }
    else if([anIdentifier isEqualToString: iTMPWCCustomCompileIdentifier] ||
                [anIdentifier isEqualToString: iTMPWCBuiltInCompileIdentifier])
    {
        [[self mainTabView] selectTabViewItemWithIdentifier: iTMPWCProjectIdentifier];
        [[self projectTabView] selectTabViewItemWithIdentifier: anIdentifier];
        [_CurrentCompileIdentifier autorelease];
        _CurrentCompileIdentifier = [anIdentifier retain];
    }
    else if([anIdentifier isEqualToString: iTMPWCCustomTypesetIdentifier] ||
                [anIdentifier isEqualToString: iTMPWCBuiltInTypesetIdentifier])
    {
        [[self mainTabView] selectTabViewItemWithIdentifier: iTMPWCProjectIdentifier];
        [[self projectTabView] selectTabViewItemWithIdentifier: anIdentifier];
        [_CurrentTypesetIdentifier autorelease];
        _CurrentTypesetIdentifier = [anIdentifier retain];
    }
    else if([anIdentifier isEqualToString: iTMPWCMakeIndexIdentifier] ||
                [anIdentifier isEqualToString: iTMPWCBibTeXIdentifier])
    {
        [[self mainTabView] selectTabViewItemWithIdentifier: iTMPWCProjectIdentifier];
        [[self projectTabView] selectTabViewItemWithIdentifier: iTMPWCHelpersIdentifier];
        [[self helpersTabView] selectTabViewItemWithIdentifier: anIdentifier];
        [_CurrentHelperIdentifier autorelease];
        _CurrentHelperIdentifier = [anIdentifier retain];
    }
    else if([anIdentifier isEqualToString: iTMPWCCompileIdentifier])
    {
        [[self mainTabView] selectTabViewItemWithIdentifier: iTMPWCProjectIdentifier];
        [[self projectTabView] selectTabViewItemWithIdentifier: _CurrentCompileIdentifier];
    }
    else if([anIdentifier isEqualToString: iTMPWCTypesetIdentifier])
    {
        [[self mainTabView] selectTabViewItemWithIdentifier: iTMPWCProjectIdentifier];
        [[self projectTabView] selectTabViewItemWithIdentifier: _CurrentTypesetIdentifier];
    }
    else if([anIdentifier isEqualToString: iTMPWCHelpersIdentifier])
    {
        [[self mainTabView] selectTabViewItemWithIdentifier: iTMPWCProjectIdentifier];
        [[self projectTabView] selectTabViewItemWithIdentifier: _CurrentHelperIdentifier];
    }
    else
        NSLog(@"What the hell is this identifier? %@", anIdentifier);
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  alreadySwitchedToPaneWithIdentifier:
- (BOOL) alreadySwitchedToPaneWithIdentifier: (NSString *) anIdentifier;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr 
- 1: 11/27/2001
To Do List: Some more to implement because all the cases are not managed.
"*/
{
    // main panes: Project, Terminal, Files, Errors
    if([anIdentifier isEqualToString: iTMPWCProjectIdentifier] ||
        [anIdentifier isEqualToString: iTMPWCTerminalIdentifier] ||
            [anIdentifier isEqualToString: iTMPWCFilesIdentifier] ||
                [anIdentifier isEqualToString: iTMPWCErrorsIdentifier])
        return [[[[self mainTabView] selectedTabViewItem] identifier] isEqualToString: anIdentifier];
    else if([anIdentifier isEqualToString: iTMPWCCompileIdentifier])
    {
        NSString * identifier = [[[self projectTabView] selectedTabViewItem] identifier];
        return ([identifier isEqualToString: iTMPWCCustomCompileIdentifier]||
                        [identifier isEqualToString: iTMPWCBuiltInCompileIdentifier]);
    }
    else if([anIdentifier isEqualToString: iTMPWCTypesetIdentifier])
    {
        NSString * identifier = [[[self projectTabView] selectedTabViewItem] identifier];
        return ([identifier isEqualToString: iTMPWCCustomTypesetIdentifier]||
                        [identifier isEqualToString: iTMPWCBuiltInTypesetIdentifier]);
    }
    else if([anIdentifier isEqualToString: iTMPWCHelpersIdentifier] ||
                [anIdentifier isEqualToString: iTMPWCMacrosIdentifier] ||
                    [anIdentifier isEqualToString: iTMPWCInputIdentifier] ||
                        [anIdentifier isEqualToString: iTMPWCRenderIdentifier])
        return ([[[[self projectTabView] selectedTabViewItem] identifier] isEqualToString: anIdentifier]);
    else
        return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fontOrColorDidChangeNotified:
- (void) fontOrColorDidChangeNotified: (NSNotification *) aNotification;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id settings = [aNotification object];
    if(!settings) settings = [self settings];
    {
        NSTextView * TV = [self errorsTextView];
        [TV setFont: [NSFont fontWithNameSizeDictionary: [settings objectForKey: iTMUDStandardFontKey]]];
        [TV setDrawsBackground: [settings boolForKey: iTMUDLogDrawsTextBackgroundKey]];
        if([TV drawsBackground])
        {
            [TV setBackgroundColor: [NSColor colorWithRGBADictionary: [settings objectForKey: iTMUDLogTextBackgroundColorKey]]];
            [TV setInsertionPointColor:
                    [NSColor colorWithRGBADictionary: [settings objectForKey: iTMUDLogInsertionPointColorKey]]];
            [TV setSelectedTextAttributes: [NSDictionary dictionaryWithObjectsAndKeys:
                    [NSColor colorWithRGBADictionary: [settings objectForKey: iTMUDLogSelectedTextColorKey]],
                            NSForegroundColorAttributeName,
                    [NSColor colorWithRGBADictionary: [settings objectForKey: iTMUDLogSelectedTextBackgroundColorKey]], 				NSBackgroundColorAttributeName,
                        nil]];
        }
        else
        {
            [TV setBackgroundColor: [NSColor textBackgroundColor]];
            [TV setInsertionPointColor: [NSColor blackColor]];
            [TV setSelectedTextAttributes: [NSDictionary dictionaryWithObjectsAndKeys:
                    [NSColor selectedTextColor], NSForegroundColorAttributeName,
                    [NSColor selectedTextBackgroundColor], NSBackgroundColorAttributeName,
                            nil]];
        }
        {
            NSScrollView * SV = [TV enclosingScrollView];
            [SV setDrawsBackground: YES];
            [SV setBackgroundColor: ([TV drawsBackground]? [TV backgroundColor]: [NSColor textBackgroundColor])];
            [SV setNeedsDisplay: YES];
        }
        {
            iTMTextStorage * TS = [TV textStorage];
            NSAssert1([TS isKindOfClass: [iTMTextStorage class]],
                @"-[iTMProjectWindowController fontOrColorDidChangeNotified:]: iTMTextStorage expected, found: %@", TS);
            {
                // no more UD when the option is project dependant
                BOOL wasSyntaxColoring = [[TS syntaxParser] isColoring];
                if([settings boolForKey: iTMUDLogSyntaxColoringKey])
                {
                    if(!wasSyntaxColoring)
                    {
                        [TS replaceSyntaxParser: [iTMLogSyntaxParser syntaxParser]];
                    }
                }
                else
                {
                    if(wasSyntaxColoring)
                    {
                        [TS replaceSyntaxParser: [iTMSyntaxParser syntaxParser]];
                    }
                }
                [[TS syntaxParser] fontOrColorDidChange: nil];
                {
                    NSRange R = NSMakeRange(0, [TS length]);
                    [TS invalidateAttributesInRange: R];
                    [[TV layoutManager] invalidateLayoutForCharacterRange: R isSoft: NO actualCharacterRange: nil];
                }
            }
        }
    }
    {
        NSTextView * TV = [self terminalOutput];
        [TV setFont: [NSFont fontWithNameSizeDictionary: [settings objectForKey: iTMUDStandardFontKey]]];
        [TV setDrawsBackground: [settings boolForKey: iTMUDLogDrawsTextBackgroundKey]];
        if([TV drawsBackground])
        {
            [TV setBackgroundColor: [NSColor colorWithRGBADictionary: [settings objectForKey: iTMUDLogTextBackgroundColorKey]]];
            [TV setInsertionPointColor:
                    [NSColor colorWithRGBADictionary: [settings objectForKey: iTMUDLogInsertionPointColorKey]]];
            [TV setSelectedTextAttributes: [NSDictionary dictionaryWithObjectsAndKeys:
                    [NSColor colorWithRGBADictionary: [settings objectForKey: iTMUDLogSelectedTextColorKey]],
                            NSForegroundColorAttributeName,
                    [NSColor colorWithRGBADictionary: [settings objectForKey: iTMUDLogSelectedTextBackgroundColorKey]], 				NSBackgroundColorAttributeName,
                        nil]];
        }
        else
        {
            [TV setBackgroundColor: [NSColor textBackgroundColor]];
            [TV setInsertionPointColor: [NSColor blackColor]];
            [TV setSelectedTextAttributes: [NSDictionary dictionaryWithObjectsAndKeys:
                    [NSColor selectedTextColor], NSForegroundColorAttributeName,
                    [NSColor selectedTextBackgroundColor], NSBackgroundColorAttributeName,
                            nil]];
        }
        {
            NSScrollView * SV = [TV enclosingScrollView];
            [SV setDrawsBackground: YES];
            [SV setBackgroundColor: ([TV drawsBackground]? [TV backgroundColor]: [NSColor textBackgroundColor])];
            [SV setNeedsDisplay: YES];
        }
        {
            iTMTextStorage * TS = [TV textStorage];
            NSAssert1([TS isKindOfClass: [iTMTextStorage class]],
                @"-[iTMProjectWindowController fontOrColorDidChangeNotified:]: iTMTextStorage expected, found: %@", TS);
            {
                // no more UD when the option is project dependant
                BOOL wasSyntaxColoring = [[TS syntaxParser] isColoring];
                if([settings boolForKey: iTMUDLogSyntaxColoringKey])
                {
                    if(!wasSyntaxColoring)
                    {
                        [TS replaceSyntaxParser: [iTMLogSyntaxParser syntaxParser]];
                    }
                }
                else
                {
                    if(wasSyntaxColoring)
                    {
                        [TS replaceSyntaxParser: [iTMSyntaxParser syntaxParser]];
                    }
                }
                [[TS syntaxParser] fontOrColorDidChange: nil];
                {
                    NSRange R = NSMakeRange(0, [TS length]);
                    [TS invalidateAttributesInRange: R];
                    [[TV layoutManager] invalidateLayoutForCharacterRange: R isSoft: NO actualCharacterRange: nil];
                }
            }
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= tabView:didSelectTabViewItem:
- (void) tabView: (NSTabView *) tabView didSelectTabViewItem: (NSTabViewItem *) tabViewItem;
/*"Validates the user interface.
Version History: jerome.laurens@u-bourgogne.fr (10/22/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[[self window] contentView] validateUserInterfaceItems];
    return;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMProjectWindowController

