/*
//  iTMProjectsManager.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sun Oct 07 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSTask.h>
#import <Foundation/NSFileManager.h>
#import <Foundation/NSBundle.h>
#import <Foundation/Foundation.h>
#import <AppKit/NSDocumentController.h>
#import <AppKit/NSApplication.h>
#import <AppKit/AppKit.h>
#import "NSMenu(iTeXMac).h"
#import "NSString(iTeXMac).h"
#import "NSToolbar(iTeXMac).h"
#import "iTMProjectsManager.h"
#import "iTMResourcesController(GenericProjects).h"
#import "iTMProjectWindowController.h"
#import "iTMProjectDocument.h"
#import "iTMProjectDocument(Accessors).h"
#import "iTMProjectDocument(Navigation).h"
//#import "iTMTeXDocument.h"
//#import "iTMPDFDocument.h"
//#import "iTMRenderDocument.h"
#import "iTMTerminalController.h"
#import "iTMLiteScanner.h"
#import "iTMStatusNotification.h"
#import "iTMGeneralProject.h"

NSString * const iTMPCDefaultGenericProjectKey = @"iTMDefaultGenericProject";

NSString * const iTMBadInputFormat = @"iTM<%@><bad input><%@>";
NSString * const iTMBadProjectFormat = @"iTM<%@><bad project><%@>";

@interface NSDocument(iTMProjectManager)
- (NSString *) coreFileName;// see iTMPDFDocument
- (NSTextStorage *) textStorage;// see iTMTeXDocument
- (iTMProjectDocument *) cachedProject;
- (void) setCachedProject: (iTMProjectDocument *) argument;
- (BOOL) isPDFDocument;
- (BOOL) isTeXDocument;
- (BOOL) isRenderDocument;
@end

@interface NSDocumentController(iTMProjectManager)
- (id) openDocumentWithContentsOfFile: (NSString *) fileName display: (BOOL) display useHelperIfNeeded: (BOOL) aFlag;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMProjectsManager
/*"Description forthcoming."*/
@implementation iTMProjectsManager
static id _iTMProjectsManager = nil;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialize
+ (void) initialize;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    {
        NSDictionary * factoryDefaults = [NSDictionary dictionaryWithObjectsAndKeys: 
                        @"pdflatex", iTMPCDefaultGenericProjectKey,
                        nil];
        [[NSUserDefaults standardUserDefaults] registerDefaults: factoryDefaults];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= defaultManager
+ (id) defaultManager;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _iTMProjectsManager? _iTMProjectsManager:
                        (_iTMProjectsManager = [[self allocWithZone: [NSApp zone]] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= init
- (id) init;
/*"The first object inited is the shared one.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(_iTMProjectsManager)
    {
        if(![self isEqual: _iTMProjectsManager])
            [super dealloc];
        return [_iTMProjectsManager retain];
    }
    else
    {
        if(self = [super init])
        {
            [[NSNotificationCenter defaultCenter] addObserver: self
                selector: @selector(windowDidBecomeKey:)
                    name: NSWindowDidBecomeMainNotification
                        object: nil];
            [[NSNotificationCenter defaultCenter] addObserver: self
                selector: @selector(windowWillClose:)
                    name: NSWindowWillCloseNotification
                        object: nil];
            self->_NumberOfOpenProjects = 0;
            [self setCurrentActiveProject: nil];
            [self->_TerminalImage release];
            self->_TerminalImage = [[NSImage alloc] initByReferencingFile:
                            [[NSBundle bundleForClass: [self class]] pathForImageResource: @"Terminal"]];
            [self->_RunningTerminalImage release];
            self->_RunningTerminalImage = [[NSImage alloc] initByReferencingFile:
                            [[NSBundle bundleForClass: [self class]] pathForImageResource: @"RunningTerminal"]];
        }
        return _iTMProjectsManager = self;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= dealloc
- (void) dealloc;
/*"Never dealloc.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self->_TerminalImage release];
    self->_TerminalImage = nil;
    [self->_RunningTerminalImage release];
    self->_RunningTerminalImage = nil;
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= loadGenericProjects
- (void) loadGenericProjects: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing, at first glance.
"*/
{
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
                NSLocalizedStringFromTableInBundle(@"Loading generic projects.", @"Project",
                                    [NSBundle bundleForClass: [self class]], "")];
    {
        NSEnumerator * enumerator = [[[iTMResourcesController sharedResourcesController] genericProjects] objectEnumerator];
        NSString * path;
        while(path = [enumerator nextObject])
        {
            iTMProjectDocument * PD = [[NSDocumentController sharedDocumentController]
                openDocumentWithContentsOfFile: [[path stringByResolvingSymlinksInPath] stringByStandardizingPath]
                    display: NO];
            [PD becomeGeneric];
        }
    }
    return;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= cleanOutlets
- (void) cleanOutlets;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [self setShowProjectsMenuItem: nil];
    [self setActiveProjectsMenu: nil];
    [self setProjectMenuItem: nil];
    [self setNoProjectMenuItem: nil];
    [self setCurrentActiveProject: nil];
    [self setCurrentProject: nil];
    [self setProjectMenuItemTitleSeed: nil];
    [super dealloc];
    return;
}
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= windowDidBecomeKey:
- (void) windowDidBecomeKey: (NSNotification *) aNotification;
/*"Fixes the new current project.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id document = [[[aNotification object] windowController] document];
    if([document isKindOfClass: [iTMProjectDocument class]])
        [self setCurrentProject: document];
    else
    {
        [self setCurrentProject: [self projectForDocument: document]];
        if([document respondsToSelector: @selector(setCachedProject:)])
            [document setCachedProject: [self currentProject]];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= windowWillClose:
- (void) windowWillClose: (NSNotification *) aNotification;
/*"Fixes the new main project.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id document = [[[aNotification object] windowController] document];
    if([document respondsToSelector: @selector(isTeXDocument)] && [document isTeXDocument])
    {
        NSEnumerator * enumerator = [[[self showProjectsMenu] itemArray] objectEnumerator];
        NSMenuItem * MI;
        while(MI = [enumerator nextObject])
        {
            iTMProjectDocument * PD = [MI representedObject];
            if([PD isGeneric] && [[PD absoluteInput] isEqualToString: [document fileName]])
                [PD _SetAbsoluteInput: nil];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMGetter()
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showProjectsMenuItem
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= activeProjectsMenu
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= projectMenuItem
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= noProjectMenuItem
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= currentActiveProject
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= currentProject
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= projectMenuItemTitleSeed
#define iTMGetter(source, getter) - (id) getter;{return source;}
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
iTMGetter(_SPMI, showProjectsMenuItem);
iTMGetter(_APM, activeProjectsMenu);
iTMGetter(_PMI, projectMenuItem);
iTMGetter(_NPMI, noProjectMenuItem);
iTMGetter(_CAP, currentActiveProject);
iTMGetter(_CP, currentProject);
iTMGetter(_PMIiTM, projectMenuItemTitleSeed);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMStrongSetter()
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setActiveProjectsMenu
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setShowProjectMenuItem
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setProjectMenuItem
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setCurrentProject
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setActiveProject
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setNoProjectMenuItem
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setProjectMenuItemTitleSeed
#define iTMStrongSetter(source, className, setter)\
- (void) setter: (id) argument;\
{\
    if(![source isEqual: argument])\
    {\
        if(argument && ![argument isKindOfClass: [className class]])\
            [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] bad argument class: %@.",\
                [self class], NSStringFromSelector(_cmd), argument];\
        else\
        {\
            [source release];\
            source = [argument retain];\
        }\
    }\
    return;\
}
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
iTMStrongSetter(_APM, NSMenu, setActiveProjectsMenu);
iTMStrongSetter(_SPMI, NSMenuItem, setShowProjectsMenuItem);
iTMStrongSetter(_PMI, NSMenuItem, setProjectMenuItem);
iTMStrongSetter(_NPMI, NSMenuItem, setNoProjectMenuItem);
iTMStrongSetter(_CAP, iTMProjectDocument, setCurrentActiveProject);
iTMStrongSetter(_CP, iTMProjectDocument, setCurrentProject);
iTMStrongSetter(_PMIiTM, NSString, setProjectMenuItemTitleSeed);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  showProjectsMenu
- (NSMenu *) showProjectsMenu;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self showProjectsMenuItem] submenu];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= projectForName:
- (iTMProjectDocument *) projectForName: (NSString *) aName;
/*"Looks for a project with name aName among the current open projects, regardless the extensions. Does not open files.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: look also in the projects stored.
"*/
{
    NSEnumerator * enumerator = [[[self activeProjectsMenu] itemArray] objectEnumerator];
    NSMenuItem * MI;
    iTMProjectDocument * PD = [self currentActiveProject];
//    NSLog(@"projectForName: %@", aName);
    aName = [[aName stringByStandardizingPath] stringByDeletingPathExtension];
    if([aName hasPrefix: @"/"])
    {
        if([[[[PD fileName] stringByStandardizingPath] stringByDeletingPathExtension] isEqualToString: aName])
            return PD;
        else// the first comes
            while( MI = [enumerator nextObject])
            {
                PD = [MI representedObject];
                if([[[PD fileName] stringByDeletingPathExtension] isEqualToString: aName])
                    return PD;
            }
    }
    else
    {
        if([[[[PD fileName] lastPathComponent] stringByDeletingPathExtension] isEqualToString: aName])
            return PD;
        else// the first comes
            while( MI = [enumerator nextObject])
            {
                PD = [MI representedObject];
                NSLog(@"> %@ = %@?", [[[PD fileName] lastPathComponent] stringByDeletingPathExtension], aName);
                if([[[[PD fileName] lastPathComponent] stringByDeletingPathExtension] isEqualToString: aName])
                    return PD;
            }
    }
    {// case insensitive
        NSString * lowercaseName = [aName lowercaseString];
        if(![lowercaseName isEqualToString: aName])
        {
            if([aName hasPrefix: @"/"])
            {
                if([[[[[PD fileName] stringByStandardizingPath] stringByDeletingPathExtension]
                                                lowercaseString] isEqualToString: lowercaseName])
                    return PD;
                else// the first comes
                    while( MI = [enumerator nextObject])
                    {
                        PD = [MI representedObject];
                        if([[[[PD fileName] stringByDeletingPathExtension]
                                                    lowercaseString] isEqualToString: lowercaseName])
                            return PD;
                    }
            }
            else
            {
                if([[[[[PD fileName] lastPathComponent] stringByDeletingPathExtension]
                                                lowercaseString] isEqualToString: lowercaseName])
                    return PD;
                else// the first comes
                    while( MI = [enumerator nextObject])
                    {
                        PD = [MI representedObject];
                        if([[[[[PD fileName] lastPathComponent] stringByDeletingPathExtension]
                                                    lowercaseString] isEqualToString: lowercaseName])
                            return PD;
                    }
            }
        }
    }
//    NSLog(@"Could not find a project for name:\n%@", aName);
    return nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= projectForInput:
- (iTMProjectDocument *) projectForInput: (NSString *) fileName;
/*"Given an absolute input path, this returns a project from the open project list that has this input.
Non generic projects are first checked. If no project fits, tries to parse the file content.
This is a designated project document provider.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(@"projectForInput: %@", fileName);
    if([[[fileName pathExtension] lowercaseString] isEqualToString: @"tex"])
    {
        iTMProjectDocument * SPD = [self currentProject];
        if([fileName isEqualToString: [SPD absoluteInput]])
            return SPD;
        else
        {
            SPD = [self currentActiveProject];
            if(![fileName isEqualToString: [SPD absoluteInput]])
            {
                NSEnumerator * enumerator = [[[self showProjectsMenu] itemArray] objectEnumerator];
                NSMenuItem * MI;
                // looking for non generic projects
                while(MI = [enumerator nextObject])
                {
                    SPD = [MI representedObject];
//                    NSLog(@"projectForInput?\n%@",
//                        ([SPD isKindOfClass: [iTMProjectDocument class]]? [SPD fileName]: @"rien"));
                    if([SPD isKindOfClass: [iTMProjectDocument class]] &&
                            ![SPD isGeneric] &&
                                [fileName isEqualToString: [SPD absoluteInput]])
                        return SPD;
                }
                // looking for generic projects
                enumerator = [[[self showProjectsMenu] itemArray] objectEnumerator];
                while(MI = [enumerator nextObject])
                {
                    SPD = [MI representedObject];
                    if([SPD isKindOfClass: [iTMProjectDocument class]] &&
                            [SPD isGeneric] &&
                                [fileName isEqualToString: [SPD absoluteInput]])
                        return SPD;
                }
                //nothing found: trying a project
                if(SPD = [self projectForName: [fileName stringByDeletingPathExtension]])
                    return SPD;
                if(SPD = [[NSDocumentController sharedDocumentController]
                                openDocumentWithContentsOfFile: [[fileName stringByDeletingPathExtension]
                                        stringByAppendingPathExtension: @"pTeXMac"] display: NO])
                    return SPD;
                //nothing found: parsing the content
                {
                    NSString * contents = nil;
                    {// looking for the contents
                        id D = [[NSDocumentController sharedDocumentController] documentForFileName: fileName];
                        if([D respondsToSelector: @selector(textStorage)])
                            contents = [[D textStorage] string];
                        else
                            contents = [NSString stringWithContentsOfFile: fileName];
                    }
                    {
                        // looking for an input directive
                        NSString * value = nil;
                        NSRange r = NSMakeRange(0, [contents length]);
                        if((value = [contents stringForCommentedKey: @"input" forRange: r
                                                                effectiveRange: nil inHeader: YES]) && [value length])
                        {
                            if(![value hasPrefix: @"/"])
                                value = [[fileName stringByDeletingLastPathComponent]
                                                                stringByAppendingPathComponent: value];
                            value = [value stringByStandardizingPath];
                            if(![[value pathExtension] length])
                                value = [value stringByAppendingPathExtension: [fileName pathExtension]];
                            if(![fileName isEqualToString: value])
                                return [self projectForInput: value];
                            else
                            {
//                                NSLog(@"Could not find a project for input:\n%@", value);
                                return nil;
                            }
                        }
                        // looking for a project directive
                        else if((value = [contents stringForCommentedKey: @"project" forRange: r
                                                    effectiveRange: nil inHeader: YES]) && [value length])
                        {
                            // project named value already open ?
                             if([value hasPrefix: @"./"])
                                value = [[fileName stringByDeletingLastPathComponent]
                                                                stringByAppendingPathComponent: value];
                            value = [value stringByStandardizingPath];
                            NSLog(@"Header directive: %@", value);
                            if(SPD = [self projectForName: value])
                            {
                                if([SPD isGeneric]) [SPD setAbsoluteInput: fileName];
                                NSLog(@"result should be %@", SPD);
                                return SPD;
                            }
                            else
                            {
                                NSLog(@"ERROR");
                                // try to open one in the HD with the same core file name
                                if(![[value pathExtension] length])
                                    value = [value stringByAppendingPathExtension: @"pTeXMac"];
                                if([value hasPrefix: @"./"])
                                    value = [[[fileName stringByDeletingLastPathComponent]
                                                    stringByAppendingPathComponent: value] stringByStandardizingPath];
                                if(SPD = [[NSDocumentController sharedDocumentController]
                                                        openDocumentWithContentsOfFile: value display: NO])
                                    return SPD;
                                #warning manage header error?
                                #if 0
                                {
                                    NSString * key =
                                        [NSString stringWithFormat: iTMBadProjectFormat, fileName, value];
                                    NSUserDefaults * SUD = [NSUserDefaults standardUserDefaults];
                                    if(![SUD boolForKey: key])
                                    {
                                        // BUG here: change the SUD BEFORE calling the alert!!!
                                        [SUD registerDefaults: [NSDictionary dictionaryWithObject:
                                                                        [NSNumber numberWithBool: YES] forKey: key]];
                                        NSRunCriticalAlertPanel(
                                    NSLocalizedStringFromTableInBundle(@"Source Project.", @"TeX",
                                        [NSBundle bundleForClass: [self class]], "Informational Panel Title"),
                                NSLocalizedStringFromTableInBundle(@"Bad project\n%@\nfor document\n%@.", @"TeX",
                                        [NSBundle bundleForClass: [self class]], "Bad project in file header."),
                            nil, nil, nil, value,
                        [fileName shortestStringByAbbreviatingWithTildeInPath]);
                                    }
                                }
                                #endif
                                NSLog(@"Bad project directive in header: %@ not found:", value);
                                return nil;
                            }
                        }
                        // looking for compile directive
                        else if(((value = [contents stringForCommentedKey: @"compile" forRange: r
                                effectiveRange: nil inHeader: YES]) && [value length]) ||
                                    ((value = [contents stringForCommentedKey: @"typeset" forRange: r
                                        effectiveRange: nil inHeader: YES]) && [value length]) ||
                                            ((value = [contents stringForCommentedKey: @"bibtex" forRange: r
                                                effectiveRange: nil inHeader: YES]) && [value length]) ||
                                                    ((value = [contents stringForCommentedKey: @"makeindex" forRange: r
                                                        effectiveRange: nil inHeader: YES]) && [value length]))
                        {
                            SPD = [iTMVirtualProject virtualProject];
                            [SPD setAbsoluteInput: fileName];
                            return SPD;
                        }
                    }
                    // looking for a %&format directive
                    {
                        iTMLiteScanner * S = [iTMLiteScanner scannerWithString: contents];
                        [S setCharactersToBeSkipped: nil];
                        if([S scanString: @"%&" intoString: nil] &&
                            [S scanUpToCharactersFromSet: [NSCharacterSet whitespaceAndNewlineCharacterSet] intoString: nil])
                        {
                            SPD = [iTMVirtualProject virtualProject];
                            [SPD setAbsoluteInput: fileName];
                            return SPD;
                        }
                    }
                }
            }
        }
        SPD = [self currentActiveProject];
        if([SPD isGeneric])
            [SPD setAbsoluteInput: fileName];
        return SPD;
    }
    else if([[[fileName pathExtension] lowercaseString] isEqualToString: @"mp"])
    {
        iTMProjectDocument * SPD = [self projectForName: @"metapost"];
        if([SPD isGeneric])
            [SPD setAbsoluteInput: fileName];
        return SPD;
    }
    else
    {
//        if([fileName length])
//            NSLog(@"Could not find a project for input: %@", fileName);
        return nil;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= projectForPDFDocument:
- (iTMProjectDocument *) projectForPDFDocument: (NSDocument *) argument;
/*"Given an absolute input path, this returns a project from the open project list that has this input.
Non generic projects are first checked. No change is made to any project.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * fileName = [argument fileName];
    iTMProjectDocument * SPD = [self currentProject];
    if(![fileName isEqualToString: [SPD absoluteOutput]])
    {
        SPD = [self currentActiveProject];
        if(![fileName isEqualToString: [SPD absoluteOutput]])
        {
            NSEnumerator * enumerator = [[[self showProjectsMenu] itemArray] objectEnumerator];
            NSMenuItem * MI;
            // looking for non generic projects
            while(MI = [enumerator nextObject])
            {
                SPD = [MI representedObject];
                if([SPD isKindOfClass: [iTMProjectDocument class]] &&
                        ![SPD isGeneric] &&
                            [fileName isEqualToString: [SPD absoluteOutput]])
                    return SPD;
            }
            // looking for generic projects
            enumerator = [[[self showProjectsMenu] itemArray] objectEnumerator];
            while(MI = [enumerator nextObject])
            {
                SPD = [MI representedObject];
                if([SPD isKindOfClass: [iTMProjectDocument class]] &&
                        [SPD isGeneric] &&
                            [fileName isEqualToString: [SPD absoluteOutput]])
                    return SPD;
            }
            //nothing found
            {
                NSString * coreFileName = [argument respondsToSelector: @selector(coreFileName)]?
                            [argument coreFileName]: [[argument fileName] stringByDeletingPathExtension];
                SPD = [[NSDocumentController sharedDocumentController]
                            openDocumentWithContentsOfFile: [coreFileName
                                        stringByAppendingPathExtension: @"pTeXMac"] display: NO];
            }
        }
    }
    return SPD;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= projectForTeXDocument:
- (iTMProjectDocument *) projectForTeXDocument: (NSDocument *) argument;
/*"Given an absolute input path, this returns a project from the open project list that has this input.
Non generic projects are first checked. No change is made to any project.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: beware of recursivity, look into classic locations for projects
"*/
{
//    NSString * fileName = [argument fileName];
//    NSLog(@"projectForTeXDocument: %@", [argument fileName]);
    return [self projectForInput: [argument fileName]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= projectForDocument
- (iTMProjectDocument *) projectForDocument: (id) argument;
/*"Returns the main window document if it is a iTMProjectDocument instance, or the project that fits best the document.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id realArgument = argument;// critical: beware of recursivity
    if(!realArgument) realArgument = [[NSDocumentController sharedDocumentController] currentDocument];
//    NSLog(@"%@ = nil => Current document = %@", argument, realArgument);
    if(!realArgument) realArgument =  [self currentActiveProject];
    if([realArgument isKindOfClass: [iTMProjectDocument class]])
        return realArgument;
    else if([realArgument respondsToSelector: @selector(isPDFDocument)] && [realArgument isPDFDocument])
        return [self projectForPDFDocument: realArgument];
    else if([realArgument respondsToSelector: @selector(isTeXDocument)] && [realArgument isTeXDocument])
        return [self projectForTeXDocument: realArgument];
    else if([realArgument respondsToSelector: @selector(isRenderDocument)] && [realArgument isRenderDocument])
    {
        iTMProjectDocument * RP = [iTMRenderProject renderProject];
        [RP setAbsoluteInput: [realArgument fileName]];
        return RP;
    }
    else if (argument)
        return [self projectForDocument: nil];
    NSLog(@"Could not find a project for document (named):\n%@", [argument fileName]);
    return nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= projectForSource
- (iTMProjectDocument *) projectForSource: (id) source;
/*"Source is the either a NSDocument subclass instance (nil) included or a path to a file. If it is a string,
we create a document at that path, then we call #{projectForDocument:} if path points to a regular location.
It is used in AppleScripting when iTeXMac is not meant to edit the source document
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(@"projectForSource %@", source);
    if([source isKindOfClass: [NSString class]])
        return [self projectForInput: source];
    else
        return [self projectForDocument: source];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= cachedProjectForDocument
- (iTMProjectDocument *) cachedProjectForDocument: (id) argument;
/*"Returns the main window document if it is a iTMProjectDocument instance, or the project that fits best the document.
This is where the TeX source document header is parsed to know what is the project...
Given by the "%!iTeXMac(project): myCleverProject" line in the very beginning of the file: before any source line.
Given by the "%!iTeXMac(input): myCleverRoot" line in the very beginning of the file.
currentProject should not be used or set here, to avoid recursive calls.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id realArgument = argument;// critical: beware of recursivity
    if(!realArgument) realArgument = [[NSDocumentController sharedDocumentController] currentDocument];
    if(!realArgument) realArgument =  [self currentActiveProject];
    if([realArgument isKindOfClass: [iTMProjectDocument class]])
        return realArgument;
    else if([realArgument respondsToSelector: @selector(isPDFDocument)] && [realArgument isPDFDocument])
        return [self projectForPDFDocument: realArgument];
    else if([realArgument respondsToSelector: @selector(isTeXDocument)] && [realArgument isTeXDocument])
    {
        //Critical: iTMTeXDocument MUST respond to cachedProject and setCachedProject
        if(![realArgument cachedProject]) [realArgument setCachedProject: [self projectForDocument: realArgument]];
        return [realArgument cachedProject];
    }
    else if (argument)
        return [self projectForDocument: nil];
    else
        return nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= awakeFromNib
- (void) awakeFromNib;
/*"Initialize #{noShowProjectSubmenuItem} and opens the generic projects.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: better use of lazy initialization for project submenu?
"*/
{
    [self synchronizeActiveAndShowMenus];
    [self loadGenericProjects: self];
    [self setProjectMenuItemTitleSeed:
        [[[NSApp mainMenu] deepItemWithAction: @selector(projectMenuItem:)] title]];
    {// activating the default
        NSEnumerator * enumerator = [[[self activeProjectsMenu] itemArray] objectEnumerator];
        NSMenuItem * item;
        while(item = [enumerator nextObject])
            if([[[[[item representedObject] fileName] lastPathComponent] stringByDeletingPathExtension]
                isEqualToString: [[NSUserDefaults standardUserDefaults] stringForKey: iTMPCDefaultGenericProjectKey]])
            {
                [self activateProject: item];
                break;
            }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= synchronizeActiveAndShowMenus;
- (void) synchronizeActiveAndShowMenus;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
#warning Eventual problems with targets actions and represented objects
    NSMenu * menu = [[self activeProjectsMenu] deepCopy];
    [[self showProjectsMenuItem] setSubmenu: menu];
    [menu release];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= addDocument:
- (void) addDocument: (NSDocument *) document;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([document isKindOfClass: [iTMProjectDocument class]])
    {
        if(_NumberOfOpenProjects)
            [self addInTypesetSubmenus: (iTMProjectDocument *) document];
        else
            [self createTypesetSubmenusWith: (iTMProjectDocument *) document];
        ++_NumberOfOpenProjects;
        // we clean all the cached projects, no subtlety, ness...
        {
            NSEnumerator * enumerator = [[[NSDocumentController sharedDocumentController] documents] objectEnumerator];
            NSDocument * doc;
            while(doc = [enumerator nextObject])
                if([doc respondsToSelector: @selector(isTeXDocument)] && [doc isTeXDocument])
                    [doc setCachedProject: nil];
        }
    }
    else
    {
        NSString * fileName = [document fileName];
        NSString * coreFileName = [fileName stringByDeletingPathExtension];
        // automatically opens an associated project if any file
        if([document respondsToSelector: @selector(isPDFDocument)] && [document isPDFDocument])
        {// removing all the extensions and identifiers
            unsigned oldLength = [fileName length];
            unsigned newLength = [coreFileName length];
            while(newLength < oldLength)
            {
                oldLength = newLength;
                coreFileName = [coreFileName stringByDeletingPathExtension];
                newLength = [coreFileName length];
            }
        }
        {
            NSString * NFN = [coreFileName stringByAppendingPathExtension: @"pTeXMac"];
            NS_DURING
            if(![fileName isEqualToString: NFN])
                [[NSDocumentController sharedDocumentController] openDocumentWithContentsOfFile: NFN display: NO];
            NS_HANDLER
            // catch the "NSWindow: -_newFirstResponderAfterResigining is not a valid message outside of a responder's implementation of -resignFirstResponder. exception"
            NSLog(@"Problem opening %@ due to %@", fileName, [localException reason]);
            NS_ENDHANDLER
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= addInTypesetSubmenus:
- (void) addInTypesetSubmenus: (iTMProjectDocument *) document;
/*"The active menu is the list of open projects visible in the typeset menu.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * title = [document projectName];
    // in the active project menu part
    NSMenu * menu = [self activeProjectsMenu];
    SEL action = @selector(activateProject:);
    NSMenuItem * menuItem = [menu itemWithAction: action];
    int index = [menu indexOfItem: menuItem];
    while(++index<[menu numberOfItems])
    {
        NSMenuItem * item = [menu itemAtIndex: index];
        if([item action] != action)
            break;
    }
    menuItem = [[NSMenuItem alloc] initWithTitle: title action: action keyEquivalent: @""];
    [menuItem setRepresentedObject: document];
    [menuItem setTarget: self];
    [menu insertItem: menuItem atIndex: index];
    [self setCurrentProject: document];
    [menuItem release];
    menuItem = [[NSMenuItem alloc] initWithTitle: title action: @selector(showProject:) keyEquivalent: @""];
    [menuItem setRepresentedObject: document];
    [menuItem setTarget: self];
    [[self showProjectsMenu] insertItem: menuItem atIndex: index];
    [menuItem release];
    #if 0
    if([document isStrongGeneric])
        [[[[self showProjectsMenu] menuRepresentation] menuItemCellForItemAtIndex: index] setAttributedTitle:
            [[[NSAttributedString alloc] initWithString: title
                attributes: [NSDictionary dictionaryWithObject:
                [[NSFontManager sharedFontManager] convertFont: [NSFont menuFontOfSize: [NSFont systemFontSize]]
                    toHaveTrait: NSItalicFontMask]
                forKey: NSFontAttributeName]] autorelease]];
    #endif
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= createTypesetSubmenusWith:
- (void) createTypesetSubmenusWith: (iTMProjectDocument *) document;
/*"Replaces the "No project" menu item (supposed to be at index 0) by a project item, both in the show and activate menus.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * title = [document projectName];
    NSMenu * menu = [self activeProjectsMenu];
    NSMenuItem * menuItem = [[NSMenuItem alloc] initWithTitle: title
                                    action: @selector(activateProject:) keyEquivalent: @""];
    [menuItem setRepresentedObject: document];
    [menuItem setTarget: self];
    [menuItem setState: NSOnState];
    [menu removeItemAtIndex: 0];// this is meant to be a No Project menu item removed
    [menu insertItem: menuItem atIndex: 0];
    [menuItem release];
    menu = [self showProjectsMenu];
    menuItem = [[NSMenuItem alloc] initWithTitle: title
                                    action: @selector(showProject:) keyEquivalent: @""];
    [menuItem setRepresentedObject: document];
    [menuItem setTarget: self];
    [menuItem setState: NSOffState];
    [menu removeItemAtIndex: 0];// this is meant to be a No Project menu item removed
    [menu insertItem: menuItem atIndex: 0];
    [menuItem release];
    [self setCurrentActiveProject: document];
    if(![self currentProject])
        [self setCurrentProject: document];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= removeDocument:
- (void) removeDocument: (NSDocument *) aDocument;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([aDocument isKindOfClass: [iTMProjectDocument class]])
    {
        --_NumberOfOpenProjects;
        if(_NumberOfOpenProjects)
            [self removeInTypesetSubmenus: (iTMProjectDocument *)aDocument];
        else
            [self deleteTypesetSubmenus];
        if([[self currentProject] isEqual: aDocument])
            [self setCurrentProject: [self currentActiveProject]];
        // we clean all the cached projects, no subtlety, ness...
        {
            NSEnumerator * enumerator = [[[NSDocumentController sharedDocumentController] documents] objectEnumerator];
            NSDocument * doc;
            while(doc = [enumerator nextObject])
                if([doc respondsToSelector: @selector(isTeXDocument)] && [doc isTeXDocument])
                    [doc setCachedProject: nil];
        }
    }
    else if([aDocument respondsToSelector: @selector(isTeXDocument)] && [aDocument isTeXDocument])
    {
        // this is the key used when the input file in the source header is wrong, see projectForDocument
        NSString * string = [[aDocument textStorage] string];
        NSString * value = [string stringForCommentedKey: @"input"
            forRange: NSMakeRange(0, [string length]) effectiveRange: nil inHeader: YES];
        NSString * key = [NSString stringWithFormat: iTMBadInputFormat, [aDocument fileName], value];
        // we register NO to recover the state of the first use.
        [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObject:
                                        [NSNumber numberWithBool: NO] forKey: key]];
        value = [string stringForCommentedKey: @"project"
            forRange: NSMakeRange(0, [string length]) effectiveRange: nil inHeader: YES];
        key = [NSString stringWithFormat: iTMBadProjectFormat, [aDocument fileName], value];
        // we register NO to recover the state of the first use.
        [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObject:
                                        [NSNumber numberWithBool: NO] forKey: key]];

    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= deleteTypesetSubmenusWith:
- (void) deleteTypesetSubmenus;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMenu * menu = [self activeProjectsMenu];
    [menu removeItemsWithAction: @selector(activateProject:)];
    [menu insertItem: [[[self noProjectMenuItem] copyWithZone: [NSMenu menuZone]] autorelease] atIndex: 0];
    menu = [self showProjectsMenu];
    [menu removeItemsWithAction: @selector(showProject:)];
    [menu insertItem: [[[self noProjectMenuItem] copyWithZone: [NSMenu menuZone]] autorelease] atIndex: 0];
    [self setCurrentActiveProject: nil];
    [self setCurrentProject: nil];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= removeInTypesetSubmenus:
- (void) removeInTypesetSubmenus: (iTMProjectDocument *) document;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self activeProjectsMenu] removeItemsWithRepresentedObject: document];
    [[self showProjectsMenu] removeItemsWithRepresentedObject: document];
    if([[self currentActiveProject] isEqual: document])
        [self setCurrentActiveProject: nil];
    if([[self currentProject] isEqual: document])
        [self setCurrentProject: [self projectForDocument: nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= activateProject:
- (void) activateProject: (id <NSMenuItem>) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSEnumerator * enumerator = [[[sender menu] itemArray] objectEnumerator];
    NSMenuItem * item;
    NSString * absoluteInput = [[self currentProject] isGeneric]? [[self currentProject] absoluteInput]: [NSString string];
    while(item = [enumerator nextObject])
        if([item action] == _cmd)
        {
            iTMProjectDocument * PD = [item representedObject];
            [item setState: NSOffState];
            #warning Strong Generic only ???
            if([PD isGeneric])
                [PD setAbsoluteInput: [NSString string]];
        }
    [sender setState: NSOnState];
    [self setCurrentActiveProject: [sender representedObject]];
    [self setCurrentProject: [self currentActiveProject]];
    if([[self currentActiveProject] isGeneric])
        [[self currentActiveProject] setAbsoluteInput: absoluteInput];
    // we clean all the cached projects, no subtlety, ness...
    {
        NSEnumerator * enumerator = [[[NSDocumentController sharedDocumentController] documents] objectEnumerator];
        NSDocument * doc;
        while(doc = [enumerator nextObject])
            if([doc respondsToSelector: @selector(isTeXDocument)] && [doc isTeXDocument])
                [doc setCachedProject: nil];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToolbarItem:
- (BOOL) validateToolbarItem: (id) anItem;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([[[anItem toolbar] window] isEqual: [NSApp mainWindow]])
    {
        SEL action = [anItem action];
        id document = [[[[anItem toolbar] window] windowController] document];
        iTMProjectDocument * validator = [self cachedProjectForDocument: document];
        if([validator respondsToSelector: action])
            return [validator validateToolbarItem: anItem];
        else if(action == @selector(showTerminal:))
        {
            [anItem setImage: ([[[validator terminal] currentTask] isRunning]? _RunningTerminalImage: _TerminalImage)];
            return ![[[validator windowController] window] isEqual: [NSApp keyWindow]] ||
                ![[validator windowController] alreadySwitchedToPaneWithIdentifier: iTMPWCTerminalIdentifier];
        }
        #if 1
        else if(action == @selector(showInput:))
        {
            NSString * input = validator?
                [validator absoluteInput]:
                ([document respondsToSelector: @selector(coreFileName)]?
                    [[document coreFileName] stringByAppendingPathExtension: @"tex"]:
                    [[[document fileName] stringByDeletingPathExtension] stringByAppendingPathExtension: @"tex"]);
            return [[NSDocumentController sharedDocumentController] documentForFileName: input] ||
                        [[NSFileManager defaultManager] isReadableFileAtPath: input];
        }
        else if(action == @selector(showOutput:))
        {
            NSString * output = validator?
                [validator absoluteOutput]:
                    [[[document fileName] stringByDeletingPathExtension] stringByAppendingPathExtension: @"pdf"];
            return [[NSDocumentController sharedDocumentController] documentForFileName: output] ||
                        [[NSFileManager defaultManager] isReadableFileAtPath: output];
        }
        #endif
        else
            return NO;
    }
    else
        return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMenuItem:
- (BOOL) validateMenuItem: (id) anItem;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    SEL action = [anItem action];
    //critical: the first menu item validated updates and caches the current project!!
    if(action == @selector(projectMenuItem:))
    {
        iTMProjectDocument * PD = [self cachedProjectForDocument: nil];
        if (PD)
            [self setCurrentProject: PD];
        {
            NSString * title = [self titleForProject: [self currentProject]];
            if(!title) title = [NSString string];
            [anItem setTitle: title];
        }
        return NO;
    }
    else if((action == @selector(showProject:)) || (action == @selector(showProjectWindow:)))
    {
        if(![NSApp isActive])
            return ([self currentProject] != nil);//dock menu
        else
        {
            iTMProjectDocument * doc = [anItem representedObject];
            if(doc)
            {
                NSString * title = [doc projectName];
                if(!title) title = [NSString string];
                [anItem setTitle: title];
    //            int index = [[anItem menu] indexOfItem: anItem];
                #if 0
                if([doc isStrongGeneric])
                    [[[[anItem menu] menuRepresentation] menuItemCellForItemAtIndex: index] setAttributedTitle:
                        [[[NSAttributedString alloc] initWithString: title
                            attributes: [NSDictionary dictionaryWithObject:
                            [[NSFontManager sharedFontManager] convertFont: [NSFont menuFontOfSize: 17.5]
                                toHaveTrait: NSItalicFontMask]
                            forKey: NSFontAttributeName]] autorelease]];
                #endif
                return YES;
            }
            else
            {
//                doc = [self projectForDocument: [[NSDocumentController sharedDocumentController] currentDocument]];
                return ([self currentProject] != nil);
            }
        }
    }
    else if(action == @selector(activateProject:))
    {
        iTMProjectDocument * doc = [anItem representedObject];
        if(!doc || ![NSApp isActive])
            return NO;//dock menu
        else
        {
            NSString * title = [doc projectName];
            if(!title) title = [NSString string];
            [anItem setTitle: title];
            return YES;
        }
    }
    #warning to the iTMProjectEngine
    else if(action == @selector(terminalStop:))
        return [[self currentProject] canStopTerminal];
    else if(action == @selector(terminalRestart:))
        return YES;
    else if(action == @selector(showInput:))
    {
        id document = [[NSDocumentController sharedDocumentController] currentDocument];
        iTMProjectDocument * validator = [self currentProject];
        NSString * input = validator?
            [validator absoluteInput]:
            ([document respondsToSelector: @selector(coreFileName)]?
                [[document coreFileName] stringByAppendingPathExtension: @"tex"]:
                [[[document fileName] stringByDeletingPathExtension] stringByAppendingPathExtension: @"tex"]);
        return [[NSDocumentController sharedDocumentController] documentForFileName: input] ||
                    [[NSFileManager defaultManager] isReadableFileAtPath: input];
    }
    else if(action == @selector(showOutput:))
        return [[NSFileManager defaultManager] isReadableFileAtPath: [[self currentProject] absoluteOutput]];
    else if(action == @selector(showErrors:))
        return [[self currentProject] canShowErrors];
    else if(action == @selector(showFiles:))
        return [[self currentProject] canShowFiles];
    else
        return ([self currentProject] != nil);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  titleForProject:
- (NSString *) titleForProject: (iTMProjectDocument *) aDocument;
/*"This is the title of the firts menu item of the project menu. It is a disabled item that gives the actual project: the one who responds to the compile, typeset, bibtex, makeindex commands.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: nothing
"*/
{
    return [[self projectMenuItemTitleSeed] stringByAppendingString: [aDocument projectName]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  showAction()
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  showFile:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  showTypeset:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  showCompile:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  showBibTeX:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  showMakeIndex:
#define showAction(act, forw)\
- (IBAction) act: (id) sender;\
{\
    iTMProjectDocument * document = [self projectForDocument: nil];\
    [document forw: sender];\
    [NSApp activateIgnoringOtherApps: YES];\
    return;\
}
/*"Forwards the action to the current project.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
showAction(showFile, fileView);
showAction(showTypeset, typesetView);
showAction(showCompile, compileView);
showAction(showBibTeX, bibTeXView);
showAction(showMakeIndex, makeIndexView);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showProjectWindow:
- (void) showProjectWindow: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMProjectDocument * PD = ([sender respondsToSelector: @selector(representedObject)] && [sender representedObject])?
            [sender representedObject]: [self projectForDocument: nil];
    [self setCurrentProject: PD];
    [PD showProjectWindow: sender];
    [NSApp activateIgnoringOtherApps: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showProject:
- (void) showProject: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMProjectDocument * PD = ([sender respondsToSelector: @selector(representedObject)] && [sender representedObject])?
            [sender representedObject]: [self projectForDocument: nil];
    [PD showProject: sender];
    [self setCurrentProject: PD];
    [NSApp activateIgnoringOtherApps: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showFiles:
- (void) showFiles: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self projectForDocument: nil] showFiles: sender];
    [NSApp activateIgnoringOtherApps: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showErrors:
- (void) showErrors: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self projectForDocument: nil] showErrors: sender];
    [NSApp activateIgnoringOtherApps: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showOutput:
- (void) showOutput: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMProjectDocument * PD = [self projectForDocument: nil];
    if(PD)
    {
        NSString * output = [PD absoluteOutput];
        if([output length])
        {
            NSDocument * document = [[NSDocumentController sharedDocumentController]
                                                openDocumentWithContentsOfFile: output display: YES];
            if([document respondsToSelector: @selector(update)])
                [document performSelector: @selector(update)];
            [PD validateUserInterface];
        }
        else if(output)
            [NSException raise: NSGenericException format: @"-[%@ %@] absolute output expected.",
                [self class], NSStringFromSelector(_cmd)];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showInput:
- (void) showInput: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id document = [[NSDocumentController sharedDocumentController] currentDocument];
    iTMProjectDocument * PD = [self projectForDocument: document];
    NSString * input = PD?
        [[[PD absoluteInput] stringByDeletingPathExtension] stringByAppendingPathExtension: @"tex"]:
        ([document respondsToSelector: @selector(coreFileName)]?
            [[document coreFileName] stringByAppendingPathExtension: @"tex"]:
            [[[document fileName] stringByDeletingPathExtension] stringByAppendingPathExtension: @"tex"]);
    if([input length])
        [[NSDocumentController sharedDocumentController] openDocumentWithContentsOfFile: input display: YES];
    else
        [NSException raise: NSGenericException format: @"-[%@ %@] absolute input expected.",
            [self class], NSStringFromSelector(_cmd)];
    [PD validateUserInterface];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= showTerminal:
- (IBAction) showTerminal: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (10/04/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMProjectDocument * PD = [self projectForDocument: nil];
    [PD makeWindows];
    [[PD windowController] switchToPaneWithIdentifier: iTMPWCTerminalIdentifier];
    [PD showWindows];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= projectMenuItem:
- (void) projectMenuItem: (id) sender;
/*"Fake action to update a menu item title like "Project: projectName". See the validateMenuItem.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= terminalRestart:
- (void) terminalRestart: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self currentProject] terminalRestart: sender];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= terminalStop:
- (void) terminalStop: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self currentProject] terminalStop: sender];
    return;
}

@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMProjectsManager

