/*
//  iTMResourcesController(GenericProjects).m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Wed Jan 09 2002.
//  Copyright © 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import "iTMResourcesController(GenericProjects).h"

NSString * const iTMRCGenericProjectsKey = @"Generic Projects";
NSString * const iTMRCHiddenGenericProjectsKey = @"Hidden Generic Projects";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMResourcesController(GenericProjects)
/*"Description forthcoming."*/
@implementation iTMResourcesController(GenericProjects)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= genericProjects
- (NSArray *) genericProjects;
/*"Returns the directories at "~/Library/Application Support/iTeXMaC/Generic Projects/".
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Do not use the info.plist facilities.
"*/
{
    NSFileWrapper * wrapper = [[[self userResourcesDirectoryWrapper] fileWrappers] objectForKey: iTMRCGenericProjectsKey];
    NSString * prefix = [[self libraryPath] stringByAppendingPathComponent: [wrapper filename]];
    NSMutableArray * MA = [NSMutableArray array];
    NSDictionary * fileWrappers = [wrapper fileWrappers];
    NSEnumerator * enumerator = [fileWrappers objectEnumerator];
    NSMutableArray * hiddenFiles = [NSMutableArray array];
    {
        NSString * dotHidden = [NSString stringWithContentsOfFile: [prefix stringByAppendingPathComponent: @".hidden"]];
        NSEnumerator * E = [[dotHidden componentsSeparatedByString: @"\n"] objectEnumerator];
        NSString * object;
        while(object = [E nextObject])
            if([object isKindOfClass: [NSString class]] && [object length])
                [hiddenFiles addObjectsFromArray: [object componentsSeparatedByString: @"\r"]];
    }
    while(wrapper = [enumerator nextObject])
        if([wrapper isDirectory])
        {
            NSString * name = [wrapper filename];
            if((![name hasPrefix: @"."]) && ([hiddenFiles indexOfObject: name] == NSNotFound))
                [MA addObject: [prefix stringByAppendingPathComponent: name]];
        }
    return [MA sortedArrayUsingSelector: @selector(compare:)];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= hiddenGenericProjects
- (NSArray *) hiddenGenericProjects;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Do not use the info.plist facilities.
"*/
{
    NSFileWrapper * wrapper = [[[self userResourcesDirectoryWrapper] fileWrappers]
        objectForKey: iTMRCHiddenGenericProjectsKey];
    NSString * prefix = [[self libraryPath] stringByAppendingPathComponent: [wrapper filename]];
    NSMutableArray * MA = [NSMutableArray array];
    NSDictionary * fileWrappers = [wrapper fileWrappers];
    NSEnumerator * enumerator = [fileWrappers objectEnumerator];
    NSMutableArray * hiddenFiles = [NSMutableArray array];
    {
        NSString * dotHidden = [NSString stringWithContentsOfFile: [prefix stringByAppendingPathComponent: @".hidden"]];
        NSEnumerator * E = [[dotHidden componentsSeparatedByString: @"\n"] objectEnumerator];
        NSString * object;
        while(object = [E nextObject])
            if([object isKindOfClass: [NSString class]] && [object length])
                [hiddenFiles addObjectsFromArray: [object componentsSeparatedByString: @"\r"]];
    }
    while(wrapper = [enumerator nextObject])
        if([wrapper isDirectory])
        {
            NSString * name = [wrapper filename];
            if((![name hasPrefix: @"."]) && ([hiddenFiles indexOfObject: name] == NSNotFound))
                [MA addObject: [prefix stringByAppendingPathComponent: name]];
        }
    return [MA sortedArrayUsingSelector: @selector(compare:)];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= allGenericProjects
- (NSArray *) allGenericProjects;
/*"Description forthcoming. Subclassing iTMProjectDocument will cause problems here. Ask cocoa-dev.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMutableArray * MA = [[[self genericProjects] mutableCopy] autorelease];
    [MA addObjectsFromArray: [self hiddenGenericProjects]];
    return [MA sortedArrayUsingSelector: @selector(compareLastPathComponents:)];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= genericProjectsPath
- (NSString *) genericProjectsPath;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self libraryPath] stringByAppendingPathComponent: iTMRCGenericProjectsKey];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= hiddenGenericProjectsPath
- (NSString *) hiddenGenericProjectsPath;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self libraryPath] stringByAppendingPathComponent: iTMRCHiddenGenericProjectsKey];
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMResourcesController(GenericProjects)

