/*
//  iTMTeTeXController.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Dec 18 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSBundle.h>
#import <Foundation/NSRunLoop.h>
#import <Foundation/Foundation.h>
#import <AppKit/NSWorkspace.h>
#import <AppKit/NSDocumentController.h>
#import <AppKit/AppKit.h>
#import "NSMenu(iTeXMac).h"
#import "iTMTeTeXController.h"
#import "iTMResourcesController(iTeXMac).h"
#import "iTMStatusNotification.h"
#include <pthread.h>

NSString * const iTMUDTeXMFPATHKey = @"iTMUDTeXMFPATH";
NSString * const iTMUDTeTeXBinariesPATHKey = @"iTMUDTeXMFBinariesPATH";
NSString * const iTMUDGSBinariesPATHKey = @"iTMUDGSBinariesPATH";
NSString * const iTMUDTeXMFDocPATHKey = @"iTMUDTeXMFDocPATH";
NSString * const iTMUDTeXMFOtherPATHKey = @"iTMUDTeXMFOtherPATH";
NSString * const iTMUDTeTeXBinariesOtherPATHKey = @"iTMUDTeXMFBinariesOtherPATH";
NSString * const iTMUDGSBinariesOtherPATHKey = @"iTMUDGSBinariesOtherPATH";
NSString * const iTMUDTeXMFDocOtherPATHKey = @"iTMUDTeXMFDocOtherPATH";

NSString * const iTMStandardPATHS = @"StandardPaths";
NSString * const iTMFinkPATHS = @"FinkPaths";

NSString * const iTMThreadLauncherKey = @"iTMThreadLauncher";
NSString * const iTMThreadFAQsMenuKey = @"iTMThreadFAQs";
NSString * const iTMThreadDocsMenuKey = @"iTMThreadDocs";
NSString * const iTMThreadLoopKey = @"iTMThreadLoop";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTeTeXController
/*"Description forthcoming."*/
@implementation iTMTeTeXController
static id _iTMTeTeXController = nil;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-==  initialize;
+ (void) initialize;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (12/18/2001).
To do list:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
                    @"/usr/local/teTeX/bin/powerpc-apple-darwin-current", iTMUDTeTeXBinariesPATHKey,
                    @"/usr/local/teTeX/share/texmf", iTMUDTeXMFPATHKey,
                    @"/usr/local/teTeX/share/texmf/doc", iTMUDTeXMFDocPATHKey,
                    @"/usr/local/bin", iTMUDGSBinariesPATHKey,
                    [NSString string], iTMUDTeTeXBinariesOtherPATHKey,
                    [NSString string], iTMUDTeXMFOtherPATHKey,
                    [NSString string], iTMUDTeXMFDocOtherPATHKey,
                    [NSString string], iTMUDGSBinariesOtherPATHKey,
                                    nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sharedResourcesController
+ (id) sharedTeTeXController;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _iTMTeTeXController? _iTMTeTeXController:
        (_iTMTeTeXController = [[self allocWithZone: [NSApp zone]] initWithWindowNibName: NSStringFromClass([self class])]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initWithWindow:
- (id) initWithWindow: (NSWindow *) window;
/*"The first ressources controller created is the shared one.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(_iTMTeTeXController)
    {
        if(![self isEqual: _iTMTeTeXController])
            [super dealloc];
        return [_iTMTeTeXController retain];
    }
    else if(self = [super initWithWindow: window])
    {
        [_FAQs autorelease];
        _FAQs = nil;
        [_Docs autorelease];
        _Docs = nil;
        [_SPD autorelease];
        _SPD = nil;
        [_FPD autorelease];
        _FPD = nil;
    }
    return _iTMTeTeXController = self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initWithCoder:
- (id) initWithCoder: (NSCoder *) coder;
/*"The first ressources controller created is the shared one.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(_iTMTeTeXController)
    {
        if(![self isEqual: _iTMTeTeXController])
            [super dealloc];
        return [_iTMTeTeXController retain];
    }
    else if(self = [super initWithCoder: coder])
    {
        [_FAQs autorelease];
        _FAQs = nil;
        [_Docs autorelease];
        _Docs = nil;
        [_SPD autorelease];
        _SPD = nil;
        [_FPD autorelease];
        _FPD = nil;
    }
    return _iTMTeTeXController = self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  createMenus
- (void) createMenus;
/*"The first ressources controller created is the shared one.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a29: 04/16/2002
To Do List:
"*/
{
    NSString * path = [[iTMResourcesController sharedResourcesController] TeXMFDocumentationMenuPath];
    NSMenu * menu1, * menu2;

    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
        NSLocalizedStringFromTableInBundle(@"Loading teTeX documentation menu.", @"General",
                                    [NSBundle bundleForClass: [self class]], "")];
    NS_DURING
    menu1 = [NSUnarchiver unarchiveObjectWithFile: path];
    NS_HANDLER
    menu1 = nil;
    NS_ENDHANDLER
    if(![menu1 isKindOfClass: [NSMenu class]])
    {
        [[NSWorkspace sharedWorkspace]
            performFileOperation: NSWorkspaceRecycleOperation
                source: [path stringByDeletingPathExtension]
                    destination: @""
                        files: [NSArray arrayWithObject: [path lastPathComponent]]
                            tag: nil];
        menu1 = [[NSMenu hierarchicalMenuAtPath: [self TeXMFDocumentationPath]
            action: @selector(openTeXMFDocumentation:) target: nil] retain];
        if(menu1) [NSArchiver archiveRootObject: menu1 toFile: path];
    }
    [menu1 deepSetTarget: self];
    [menu1 deepEnableItems];
    [menu1 deepSetAutoenablesItems: NO];
    
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
        NSLocalizedStringFromTableInBundle(@"Loading teTeX FAQs menu1.", @"General",
                                    [NSBundle bundleForClass: [self class]], "")];
    
    path = [[iTMResourcesController sharedResourcesController] TeXMFFAQsMenuPath];
    NS_DURING
    menu2 = [NSUnarchiver unarchiveObjectWithFile: path];
    NS_HANDLER
    menu2 = nil;
    NS_ENDHANDLER
    if(![menu2 isKindOfClass: [NSMenu class]])
    {
        [[NSWorkspace sharedWorkspace]
            performFileOperation: NSWorkspaceRecycleOperation
                source: [path stringByDeletingPathExtension]
                    destination: @""
                        files: [NSArray arrayWithObject: [path lastPathComponent]]
                            tag: nil];
        menu2 = [[NSMenu hierarchicalMenuAtPath: [self TeXMFFAQsPath]
            action: @selector(openTeXMFDocumentation:) target: nil] retain];
        if(menu2)
            [NSArchiver archiveRootObject: menu2 toFile: path];
        else
            NSLog(@"Sorry, no FAQs menu2 created\n(path: %@)", [self TeXMFFAQsPath]);
    }
    [menu2 deepSetTarget: self];
    [menu2 deepEnableItems];
    [menu2 deepSetAutoenablesItems: NO];

    _Docs = [menu1 retain];
    _FAQs = [menu2 retain];

    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"The first ressources controller created is the shared one.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [_FAQs autorelease];
    _FAQs = nil;
    [_Docs autorelease];
    _Docs = nil;
    [_SPD autorelease];
    _SPD = nil;
    [_FPD autorelease];
    _FPD = nil;
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  GSBinariesPath
- (NSString *) GSBinariesPath;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[NSUserDefaults standardUserDefaults] stringForKey: iTMUDGSBinariesPATHKey] stringByResolvingSymlinksInPath];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  TeTeXBinariesPath
- (NSString *) TeTeXBinariesPath;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[NSUserDefaults standardUserDefaults] stringForKey: iTMUDTeTeXBinariesPATHKey] stringByResolvingSymlinksInPath];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  TeXMFDocumentationPath
- (NSString *) TeXMFDocumentationPath;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[[NSUserDefaults standardUserDefaults] stringForKey: iTMUDTeXMFPATHKey]
                        stringByAppendingPathComponent: @"doc"] stringByResolvingSymlinksInPath];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  TeXMFFAQsPath
- (NSString *) TeXMFFAQsPath;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[[self TeXMFDocumentationPath] stringByAppendingPathComponent: @"help"]
                        stringByAppendingPathComponent: @"faq"] stringByResolvingSymlinksInPath];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  openTeXMFDocumentation:
- (void) openTeXMFDocumentation: (id <NSMenuItem>) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * path = [sender representedObject];
    if([path isKindOfClass: [NSString class]])
    {
        if(([[[path pathExtension] lowercaseString] isEqualToString: @"pdf"] &&
        	[[NSDocumentController sharedDocumentController] openDocumentWithContentsOfFile: path display: YES]) ||
                    [[NSWorkspace sharedWorkspace] openFile: path])
            return;
        else if(![[NSDocumentController sharedDocumentController] openDocumentWithContentsOfFile: path display: YES])
            [[NSWorkspace sharedWorkspace] openFile: path];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  orderTeXMFPanelFront:
- (void) orderTeXMFPanelFront: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self window] makeKeyAndOrderFront: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowDidLoad
- (void) windowDidLoad;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self window] makeKeyAndOrderFront: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  standardPathsDictionary
- (NSDictionary *) standardPathsDictionary;
/*"The dictionary where are stored the standard paths...
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!_SPD)
    {
        NSString * path;
        path = [[[[iTMResourcesController sharedResourcesController] generalDirectoryPath]
            stringByAppendingString: iTMStandardPATHS] stringByAppendingPathExtension: @"plist"];
        if(![[NSFileManager defaultManager] isReadableFileAtPath: path])
            path = [[NSBundle bundleForClass: [self class]] pathForResource: iTMStandardPATHS
                ofType: @"plist" inDirectory: nil];
        if([path length])
            _SPD = [[NSDictionary dictionaryWithContentsOfFile: path] retain];
    }
    return _SPD;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  finkPathsDictionary
- (NSDictionary *) finkPathsDictionary;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!_FPD)
    {
        NSString * path;
        path = [[[[iTMResourcesController sharedResourcesController] generalDirectoryPath]
            stringByAppendingString: iTMFinkPATHS] stringByAppendingPathExtension: @"plist"];
        if(![[NSFileManager defaultManager] isReadableFileAtPath: path])
            path = [[NSBundle bundleForClass: [self class]] pathForResource: iTMFinkPATHS
                ofType: @"plist" inDirectory: nil];
        if([path length])
            _FPD = [[NSDictionary dictionaryWithContentsOfFile: path] retain];
    }
    return _FPD;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  TeXMFFAQsMenu
- (IBAction) TeXMFFAQsMenu;
/*"Does nothing: just catches the message in the #{validateMenuItem} process.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a30: 04/16/2002
To Do List:
"*/
{
    return _FAQs;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  TeXMFDocumentationMenu
- (IBAction) TeXMFDocumentationMenu;
/*"Does nothing: just catches the message in the #{validateMenuItem} process.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a30: 04/16/2002
To Do List:
"*/
{
    return _Docs;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMenuItem:
- (BOOL) validateMenuItem: (id) aMenuItem;
/*"Installs the menus if they are available.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a30: 04/16/2002
To Do List:
"*/
{
    SEL action = [aMenuItem action];
    NSLog(@"validating: %@", NSStringFromSelector(action));
    if(action == @selector(TeXMFFAQsMenuItem:))
    {
        NSMenu * menu = [[self FAQsMenu] retain];
        if(menu)
        {
            NSMenu * M = [aMenuItem menu];
            int index = [M indexOfItem: aMenuItem];
            NSMenuItem * MI = [[NSMenuItem alloc] init];
            [MI setTitle: [aMenuItem title]];
            [MI setSubmenu: menu];
            [M removeItemAtIndex: index];
            [M insertItem: MI atIndex: index];
            [menu release];
            return YES;
        }
        else
            return NO;
    }
    else if(action == @selector(TeXMFDocumentationMenuItem:))
    {
        NSMenu * menu = [[self documentationMenu] retain];
        if(menu)
        {
            NSMenu * M = [aMenuItem menu];
            int index = [M indexOfItem: aMenuItem];
            NSMenuItem * MI = [[NSMenuItem alloc] init];
            [MI setTitle: [aMenuItem title]];
            [MI setSubmenu: menu];
            [M removeItemAtIndex: index];
            [M insertItem: MI atIndex: index];
            [menu release];
            return YES;
        }
        else
            return NO;
    }
    else if(action == @selector(openTeXMFDocumentation:))
    {
        return YES;
    }
    else
        return NO;
}
#endif
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTeTeXController
#if 0
@implementation NSMenuItem(PPP)
- (NSString *) description;
{
    return [NSString stringWithFormat: @"%@\n%@\n%@\n%@",
        [super description], NSStringFromSelector([self action]), [self target], ([self isEnabled]? @"YES": @"NO")];
}
@end
#endif