/*
//  iTMTeXWindowController.h
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Mon Dec 03 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>

extern NSString * const iTMUDContinuousSpellCheckingKey;
extern NSString * const iTMUDTeXSyntaxColoringKey;

extern NSString * const iTMUDTeXDrawsTextBackgroundKey;
extern NSString * const iTMUDTeXTextBackgroundColorKey;
extern NSString * const iTMUDTeXSelectedTextBackgroundColorKey;
extern NSString * const iTMUDTeXSelectedTextColorKey;
extern NSString * const iTMUDTeXInsertionPointColorKey;

extern NSString * const iTMSourceToolbarIdentifier;
extern NSString * const iTMUDTeXCascadeWindowsKey;

@class iTMToolbarDelegate;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTeXWindowController

@interface iTMTeXWindowController : NSWindowController
{
@private
    IBOutlet NSTextView * _TextView;	/*" textView displaying the current TeX source "*/
    IBOutlet NSTextView * _HiddenTextView;	/*" other textView displaying the current TeX source "*/
    IBOutlet id _LineField;
    IBOutlet id _StatusField;	/*"used for size adjustments only"*/
    IBOutlet id _MarkButton;
    IBOutlet id _SectionButton;
    IBOutlet id _LabelButton;
    IBOutlet id _DocButton;
    IBOutlet id _GfxButton;
    IBOutlet id _MathButton;
    IBOutlet id _MiscButton;
    IBOutlet id _TextButton;
    IBOutlet id _TmplButton;
    IBOutlet id _GreekButton;
    struct __iTMTWCFlags
    {
        unsigned int isiTeXMacCommentEdited: 1;
        unsigned int reserved: 31;
    } _iTMTWCFlags;
    id _TD;
    id _HiddenScrollView;
}
/*"Class methods"*/
/*"Setters and Getters"*/
- (NSString *) toolbarIdentifier;
- (void) installToolbarWithIdentifier: (NSString *) anIdentifier;
- (id) toolbarDelegate;
- (void) setToolbarDelegate: (id) argument;
- (id) textView;
- (void) setTextView: (id) aTextView;
- (id) hiddenTextView;
- (void) setHiddenTextView: (id) aTextView;
- (id) mainTextView;
- (id) lineField;
- (void) setLineField: (id) aLineField;
- (id) statusField;
- (void) setStatusField: (id) aStatusField;
- (id) markButton;
- (id) sectionButton;
- (id) labelButton;
- (id) docButton;
- (id) gfxButton;
- (id) mathButton;
- (id) miscButton;
- (id) textButton;
- (id) tmplButton;
- (id) greekButton;
- (void) setMarkButton: (id) argument;
- (void) setSectionButton: (id) argument;
- (void) setLabelButton: (id) argument;
- (void) setDocButton: (id) argument;
- (void) setGfxButton: (id) argument;
- (void) setMathButton: (id) argument;
- (void) setMiscButton: (id) argument;
- (void) setTextButton: (id) argument;
- (void) setTmplButton: (id) argument;
- (void) setGreekButton: (id) argument;
- (int) currentLineNumber;
/*"Main methods"*/
- (void) installToolbarWithIdentifier: (NSString *) anIdentifier;
- (void) installMenusInButtons: (NSNotification *) aNotification;
/*"Overriden methods"*/
@end

@interface iTMTeXWindowController(Toolbar)
- (Class) TeXToolbarDelegateClass;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTeXWindowController
