/*
//  iTMTeXDocument(Cocoa).m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sat Dec 08 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <CoreFoundation/CFString.h>
#import <Foundation/NSString.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSFileManager.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSURL.h>
#import <Foundation/NSUndoManager.h>
#import <AppKit/NSTextView.h>
#import <AppKit/NSLayoutManager.h>
#import "NSString(iTeXMac).h"
#import "NSUserDefaults(iTeXMac).h"
#import "NSWindow(iTeXMac).h"

#import "iTMTeXDocument.h"
#import "iTMTeXDocument(Navigation).h"
#import "iTMUndoManager.h"
#import "iTMXTextView.h"
#import "iTMTeXTextStorage.h"
#import "iTMStatusNotification.h"
#import "iTMNavigationFormatter.h"
#import "iTMTeXWindowController.h"
#import "iTMTeXWindowController(Action).h"
#import "iTMTeXWindowController(Cocoa).h"
#import "iTMSettings.h"

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTeXDocument(Cocoa)
/*"Here is gathered all the pure cocoa stuff."*/
@implementation iTMTeXDocument(Cocoa)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize;
+ (void) initialize;
/*"Description Forthcoming.
Version History: Originally created by RK, huge corrections by JL.
To do list:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
            [NSNumber numberWithInt: 0], iTMUDLevelsOfUndoKey,
            [NSNumber numberWithBool: NO], iTMUDKeepBackupFileKey,
                nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"Description Forthcoming.
Version History: Originally created by RK, huge corrections by JL.
To do list:
"*/
{
    self = [super init];
    [self setUndoManager: [[[iTMUndoManager alloc] init] autorelease]];
    [[self undoManager] setLevelsOfUndo: [[NSUserDefaults standardUserDefaults] integerForKey: iTMUDLevelsOfUndoKey]];
    [[NSNotificationCenter defaultCenter] addObserver: self
        selector: @selector(userDefaultsDidChange:)
            name: iTMUserDefaultsDidChangeNotification
                object: [NSUserDefaults standardUserDefaults]];
    [self userDefaultsDidChange: [NSNotification notificationWithName: iTMUserDefaultsDidChangeNotification
                object: [NSUserDefaults standardUserDefaults]]];
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description Forthcoming.
Version History: Originally created by RK, huge corrections by JL.
To do list: there remains a lot to do, hu, hu, hu
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [[NSNotificationCenter defaultCenter] removeObserver: nil name: nil object: self];
    [self setSourceWC: nil];
    [self setTextStorage: nil];
    [self setFileAttributes: nil];
    [self setCachedProject: nil];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  makeWindowControllers
- (void) makeWindowControllers;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![[self windowControllers] count])
    {
        [self setSourceWC: [[[iTMTeXWindowController alloc]
            initWithWindowNibName: NSStringFromClass([self class])] autorelease]];
        if([self sourceWC])
            [self addWindowController: [self sourceWC]];
        else
            NSLog(@"Could not create the source WC");
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  saveToFile:saveOperation:delegate:didSaveSelector:contextInfo:
- (void) saveToFile: (NSString *) fileName saveOperation: (NSSaveOperationType) saveOperation delegate: (id) delegate didSaveSelector: (SEL) didSaveSelector contextInfo: (void *) contextInfo;
/*"Spaces are replaced by underscores.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * path = [fileName stringByDeletingLastPathComponent];
    NSString * last = [fileName lastPathComponent];
    NSString * newName = [path stringByAppendingPathComponent: [last stringWithSubstring: @" " replacedByString: @"_"]];
    [super setFileName: newName];
    [super saveToFile: newName saveOperation: saveOperation delegate: delegate
                                        didSaveSelector: didSaveSelector contextInfo: contextInfo];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= fileAttributesToWriteToFile:ofType:saveOperation:
- (NSDictionary *) fileAttributesToWriteToFile: (NSString *) fullDocumentPath ofType: (NSString *) documentTypeName
saveOperation: (NSSaveOperationType) saveOperationType;
/*" The routine sets the document type to 'TEXT' and creator to '????'. Cocoa feature.
Version History: jerome.laurens@u-bourgogne.fr (11/11/2001),
- < 1.1: 03/10/2002
To Do List: TEST
"*/
{
    NSMutableDictionary * result = [[super fileAttributesToWriteToFile: fullDocumentPath
                                        ofType: documentTypeName saveOperation: saveOperationType] mutableCopy];
    if(![result fileHFSTypeCode])
    {
        char newType[] = "TEXT";
        [result setObject: [NSNumber numberWithInt: *(int*)newType] forKey: NSFileHFSTypeCode];
    }
    if(![result fileHFSCreatorCode])
    {
        char newCreator[] = "????";
        [result setObject: [NSNumber numberWithInt: *(int*)newCreator] forKey: NSFileHFSCreatorCode];
    }
    [result setObject: iTMTeXDocumentType forKey: @"Document Type"];
    return [result autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= writeToFile:ofType:
- (BOOL) writeToFile: (NSString *) aFileName ofType: (NSString *) aType;
/*"An awful patch to force the text view to change the undo grouping. Cocoa feature.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: TEST
"*/
{
    if([aFileName length])
    {
        BOOL result = NO;
        aFileName = [aFileName stringByStandardizingPath];
        if(result = [super writeToFile: aFileName ofType: aType])
        {
            // critical: forces the text view to have different groups of undo, before and after the save.
            // IT IS AN AWFUL but efficient PATCH.
            NSTextView * TV = [self textView];
            NSRange old = [TV selectedRange];
            if(old.location)
            {
                [[TV undoManager] disableUndoRegistration];
                [TV setSelectedRange: NSMakeRange(old.location - 1, 0)];
                [TV insertText: @"x"];
                [TV setSelectedRange: NSMakeRange(old.location - 1, 1)];
                [TV insertText: @""];
                [TV setSelectedRange: old];
                [[TV undoManager] enableUndoRegistration];
            }
        }
        return result;
    }
    else
        return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= keepBackupFile
- (BOOL) keepBackupFile;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _iTMTDFlags.keepBackupFile != 0;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= dataRepresentationOfType:
- (NSData *) dataRepresentationOfType: (NSString *) aType;
/*"Just deals with the lossy conversion.
The text storage provides the data with message #{encodedStringDataAllowLossyConversion:}.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSData * result = [[self textStorage] encodedStringDataAllowLossyConversion: NO];
    if(!result)
    {
        if(NSAlertAlternateReturn == NSRunCriticalAlertPanel(
            NSLocalizedStringFromTableInBundle(@"Saving.", @"TeX",
                                                [NSBundle bundleForClass: [self class]], "Informational Panel Title"),
            NSLocalizedStringFromTableInBundle(@"Information may be lost while saving with encoding %@.", @"TeX",
                                                [NSBundle bundleForClass: [self class]], "unsaved"),
            NSLocalizedStringFromTableInBundle(@"Abort", @"TeX", [NSBundle bundleForClass: [self class]], "Abort"),
            NSLocalizedStringFromTableInBundle(@"Force", @"TeX", [NSBundle bundleForClass: [self class]], "Force"),
            nil,
            [NSString localizedNameOfStringEncoding: [[self textStorage] dataStringEncoding]]))
        {
            result = [[self textStorage] encodedStringDataAllowLossyConversion: YES];
        }
    }
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= loadDataRepresentation:ofType:
- (BOOL) loadDataRepresentation: (NSData *) docData ofType: (NSString *) docType;
/*" NSMacOSRomanStringEncoding allowing Lossy Conversion.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: implement the stringEncoding, is it usefull ? stringEncoding
"*/
{
    iTMTextStorage * TS = [[self settings] boolForKey: iTMUDTeXSyntaxColoringKey]?
            [[[iTMTeXTextStorage allocWithZone: [self zone]]
                                initWithEncodedStringData: docData] autorelease]:
            [[[iTMTextStorage allocWithZone: [self zone]]
                                initWithEncodedStringData: docData] autorelease];
//    NSLog(@"docData length, TS, [TS class]: %d,\n<%@>(%@)", [docData length], TS, [TS class]);
    if(TS)
    {
        [self setTextStorage: TS];
        [self cleanCachedData];
        if([docData length])
            [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: [NSString stringWithFormat:
                NSLocalizedStringFromTableInBundle(@"Data loaded with string encoding: %@.", @"TeX",
                                                                [NSBundle bundleForClass: [self class]], ""),
                    [NSString localizedNameOfStringEncoding: [TS stringEncoding]]]];
        return YES;
    }
    else
        return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= readFromFile:ofType:
- (BOOL) readFromFile: (NSString *) aFileName ofType: (NSString *) type;
/*" Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL result = [super readFromFile: aFileName ofType: type];
    if(result)
         [self setFileAttributes: [[NSFileManager defaultManager] fileAttributesAtPath: aFileName traverseLink: YES]];
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= printDocument:
- (void) printDocument: (id) aSender;
/*"Prints the associate document if the sourceWindow is the main window, otherwise calls super.
Version History: jerome.laurens@u-bourgogne.fr (07/17/2001)
- < 1.1: 03/10/2002
To Do List: ask to typeset before."*/
#warning to be improved, ask to typeset before (JL)
{
    if([[NSApp mainWindow] isEqual: [self sourceWindow]])
    {
        NSPrintInfo* PI = [self printInfo];
        NSPrintOperation * PO = [NSPrintOperation
                printOperationWithView: [self textView] printInfo: PI];
        [PI setHorizontalPagination: NSFitPagination];
        [PI setVerticallyCentered: NO];
        [PO setShowPanels: YES];
        [PO runOperation];
    }
    else// if([[NSApp mainWindow] isEqual: [self representationWindow]])
    {
    #warning sheet for typesetting!!!
        [super printDocument: aSender];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateMenuItem:
- (BOOL) validateMenuItem: (NSMenuItem *) anItem;
/*"Replaces old stuff which had absolutely no cocoa flavour.
Version history: jerome.laurens@u-bourgogne.fr (08/29/2001):
- < 1.1: 03/10/2002
To Do List: implement other actions
"*/
{
    SEL selector = [anItem action];
    // File menu
    if (selector == @selector(saveDocument:))
        return [self isDocumentEdited] || ![[NSFileManager defaultManager] isReadableFileAtPath: [self fileName]];
    else if (selector == @selector(revertDocumentToSaved:))
        return [self isDocumentEdited] && [[NSFileManager defaultManager] isReadableFileAtPath: [self fileName]];
    else if(selector == @selector(revertDocumentToSavedWithStringEncoding:))
        return YES;
    else if (selector == @selector(printDocument:))
        return [[[self textView] string] length]>0;
    else if ((selector == @selector(scrollToNextError:)) ||
                (selector == @selector(scrollToPreviousError:)))
        return YES;
    else if(selector == @selector(toggleSyntaxColoring:))
    {
        [anItem setState: [[self settings] boolForKey: iTMUDTeXSyntaxColoringKey]? NSOnState: NSOffState];
        return YES;
    }
    else if(selector == @selector(toggleDrawsTextBackground:))
    {
        [anItem setState: [[self settings] boolForKey: iTMUDTeXDrawsTextBackgroundKey]? NSOnState: NSOffState];
        return YES;
    }
    // Source menu
    else
        return [super validateMenuItem: anItem];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleDrawsTextBackground:
- (IBAction) toggleDrawsTextBackground: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL oldOnState = ([sender state] != NSOffState);
    [sender setState: (oldOnState? NSOffState: NSOnState)];
    [[self settings] setBool: !oldOnState forKey: iTMUDTeXDrawsTextBackgroundKey];
    [[self sourceWC] fontOrColorDidChangeNotified: nil];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleSyntaxColoring:
- (IBAction) toggleSyntaxColoring: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL oldOnState = ([sender state] != NSOffState);
    [sender setState: (oldOnState? NSOffState: NSOnState)];
    [[self settings] setBool: !oldOnState forKey: iTMUDTeXSyntaxColoringKey];
    [[self sourceWC] fontOrColorDidChangeNotified: nil];
    return;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTeXDocument(Cocoa)
