/*
//  iTMTeXDocument.h
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Dec 18 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <CoreFoundation/CFString.h>
#import <Foundation/NSString.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSFileManager.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSURL.h>
#import <Foundation/NSUndoManager.h>
#import <AppKit/NSTextView.h>
#import <AppKit/NSLayoutManager.h>
#import "NSString(iTeXMac).h"
#import "NSUserDefaults(iTeXMac).h"
#import "NSWindow(iTeXMac).h"

#import "iTMTeXDocument.h"
#import "iTMTeXDocument(Navigation).h"
#import "iTMXTextView.h"
#import "iTMTeXTextStorage.h"
#import "iTMNavigationFormatter.h"
#import "iTMEncodingController.h"
#import "iTMTeXWindowController.h"
#import "iTMSettings.h"

// I am not sure this is absolutely necessary, don't understand the document type, i am afraid
// I should read the custom info.plist myself...
NSString * const iTMTeXDocumentType = @"TeX Document";// beware, this MUST appear in the target file...
NSString * const iTMTextDocumentType = @"Text Document";// beware, this MUST appear in the target file...

NSString * const iTMUDKeepBackupFileKey = @"iTMKeepBackupFile";
NSString * const iTMUDLevelsOfUndoKey = @"iTMUDLevelsOfUndo";

@interface NSObject(iTMTeXDocument_ForTheCompiler)
- (id) syntaxParser;
- (void) replaceSyntaxParser: (id) argument;
- (void) invalidateLayout;
@end

@implementation iTMTeXDocument
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isTeXDocument
- (BOOL) isTeXDocument;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return YES;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  userDefaultsDidChange:
- (void) userDefaultsDidChange: (NSNotification *) aNotification;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _iTMTDFlags.keepBackupFile = [[aNotification object] boolForKey: iTMUDKeepBackupFileKey]? 1: 0;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sourceWindow
- (id) sourceWindow;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self sourceWC] window];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sourceWC
- (id) sourceWC;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return self->_SourceWC;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setSourceWC:
- (void) setSourceWC: (id) aWC;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![_SourceWC isEqual: aWC])
    {
        [_SourceWC release];
        _SourceWC = [aWC retain];
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textView
- (id) textView;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self sourceWC] textView];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setLineEndingsUsed:
- (void) setLineEndingsUsed: (unsigned int) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    //critical: if old is not set, we did not know which kind of endings used
    unsigned int old = [self lineEndingsUsed];
    argument = argument % 8;
    if(old != argument)
    {
        [[[self undoManager] prepareWithInvocationTarget: self] setLineEndingsUsed: old];
        [[self textStorage] setLineEndingsUsed: argument];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= lineEndingsUsed
- (unsigned int) lineEndingsUsed;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self textStorage] lineEndingsUsed];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= stringEncoding
- (NSStringEncoding) stringEncoding;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self textStorage] stringEncoding];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setStringEncoding:
- (void) setStringEncoding: (NSStringEncoding) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSStringEncoding old = [self stringEncoding];
    if(old != argument)
    {
        [[[self undoManager] prepareWithInvocationTarget: self] setStringEncoding: old];
        [[self textStorage] setStringEncoding: argument];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= canSetStringEncoding
- (BOOL) canSetStringEncoding;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self textStorage] canSetStringEncoding];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= fileAttributes
- (id) fileAttributes;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _FileAttributes;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setFileAttributes:
- (void) setFileAttributes: (id) argument;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![_FileAttributes isEqual: argument])
    {
        [_FileAttributes release];
        _FileAttributes = [argument retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= cachedProject
- (id) cachedProject;
/*"We must compute the project associate to a document every time the toolbar button are validated. This is extremely expensive so we cache here the information. The project manager is the one who sets this storage with a fully meaning value. However, the project manager and the document itself can both clean this storage when the associate project might have changed for editing or open/close reasons.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _Private1;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setCachedProject:
- (void) setCachedProject: (id) argument;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [_Private1 autorelease];
    _Private1 = [argument retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= settings
- (id) settings;
/*"Lazy initializer.
Version History: jerome.laurens@u-bourgogne.fr
- 1.1.b: 04/16/2002
To Do List:
"*/
{
    return _Settings? _Settings: (_Settings = [[iTMSettings alloc] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setSettings:
- (void) setSettings: (id) argument;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [_Settings autorelease];
    _Settings = [argument retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= textStorage
- (id) textStorage;
/*"Lazy initialization.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!_TextStorage)
    {
        [self setTextStorage: [[[iTMTextStorage allocWithZone: [self zone]] init] autorelease]];
//        NSLog(@"[_TextStorage class]: %@", NSStringFromClass([_TextStorage class]));
    }
    return _TextStorage;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setTextStorage:
- (void) setTextStorage: (id) argument;
/*"Critical: invalidates attributes of the new text storage. Tis might be undesirable if the attributes have been already fixed.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
#warning invalidates should not occur: see above.
{
    if(![argument isEqual: _TextStorage])
    {
        NSArray * LM = [_TextStorage layoutManagers];
        id SP = [_TextStorage respondsToSelector: @selector(syntaxParser)]? [_TextStorage syntaxParser]: nil;
        if(argument && [LM count]) [[LM objectAtIndex: 0] replaceTextStorage: argument];
        [_TextStorage autorelease];
        _TextStorage = [argument retain];
        if(SP && [_TextStorage respondsToSelector: @selector(replaceSyntaxParser:)])
            [_TextStorage replaceSyntaxParser: SP];
        [_TextStorage invalidateAttributesInRange: NSMakeRange(0, [_TextStorage length])];
        if([_TextStorage respondsToSelector: @selector(invalidateLayout)])
            [_TextStorage performSelector: @selector(invalidateLayout)];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= cleanCachedData
- (void) cleanCachedData;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setCachedProject: nil];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  revertDocumentToSavedWithStringEncoding
- (void) revertDocumentToSavedWithStringEncoding: (id) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr.
To do list: ASK!!!
"*/
{
    int old = [iTMEncodingController preferredStringEncoding];
    [iTMEncodingController preferStringEncoding: sender];
    if(![self isDocumentEdited] ||
        (NSAlertDefaultReturn == NSRunCriticalAlertPanel(
                    NSLocalizedStringFromTableInBundle(@"Revert", @"TeX",
                                        [NSBundle bundleForClass: [self class]], "Executable resource problem title"),
                        NSLocalizedStringFromTableInBundle( @"Discard changes?", @"TeX",
                                        [NSBundle bundleForClass: [self class]], ""),
                            nil,  
                                NSLocalizedStringFromTableInBundle( @"Cancel", @"TeX",
                                        [NSBundle bundleForClass: [self class]], ""), nil)))
        [self revertToSavedFromFile: [self fileName] ofType: [self fileType]];
    [iTMEncodingController setPreferredStringEncoding: old];
    return;
}
@end
