/*
//  iTMTeXWindowController(Cocoa).m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sat Dec 08 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <AppKit/NSDocument.h>
#import <AppKit/NSDocumentController.h>
#import <AppKit/NSWindowController.h>
#import "NSUserDefaults(iTeXMac).h"
#import "NSTextView(iTeXMac).h"
#import "NSWindow(iTeXMac).h"
#import "iTMTextFinder.h"
#import "iTMTeXDocument.h"
#import "iTMTeXWindowController.h"
#import "iTMTeXWindowController(Cocoa).h"
#import "iTMTeXWindowController(Action).h"
#import "iTMTextStorage.h"
#import "iTMTextView.h"
#import "iTMSyntaxParser.h"
#import "iTMTeXSyntaxParser.h"
#import "iTMMixedButton.h"
#import "iTMSettings.h"

@interface NSTextStorage(iTMTeXWindowController)
- (BOOL) isITMCommentAtIndex: (unsigned) aLocation;//Critical: the text storage is meant to be a iTMTextStorage...
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTeXWindowController(Cocoa)
/*"Description forthcoming.
For the responder chain: the window delegate is the document."*/
@implementation iTMTeXWindowController(Cocoa)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize;
+ (void) initialize;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (12/18/2001).
To do list:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
            [NSNumber numberWithBool: NO], iTMUDContinuousSpellCheckingKey,
            [NSNumber numberWithBool: YES], iTMUDSmartInsertDeleteEnabledKey,
            [NSNumber numberWithBool: NO], iTMUDTeXSyntaxColoringKey,
            [NSNumber numberWithBool: NO], iTMUDTeXDrawsTextBackgroundKey,
            [NSNumber numberWithBool: NO], iTMUDTeXCascadeWindowsKey,
            [[NSColor textBackgroundColor] RGBADictionary], iTMUDTeXTextBackgroundColorKey,
            [[NSColor selectedTextColor] RGBADictionary], iTMUDTeXSelectedTextColorKey,
            [[NSColor selectedTextBackgroundColor] RGBADictionary], iTMUDTeXSelectedTextBackgroundColorKey,
            [[NSColor blackColor] RGBADictionary], iTMUDTeXInsertionPointColorKey,
                nil]];
    // compatibility with previous versions: some keys have changed
    {
        id object;
        #define COMP(oldKey, newKey)\
        if(object = [[NSUserDefaults standardUserDefaults] objectForKey: oldKey])\
        {\
            [[NSUserDefaults standardUserDefaults] setObject: object forKey: newKey];\
            [[NSUserDefaults standardUserDefaults] removeObjectForKey: oldKey];\
        }
        COMP(@"iTMUDTextBackgroundColor", iTMUDTeXTextBackgroundColorKey);
        COMP(@"iTMUDSelectedTextBackgroundColor", iTMUDTeXSelectedTextBackgroundColorKey);
        COMP(@"iTMUDSelectedTextColor", iTMUDTeXSelectedTextColorKey);
        COMP(@"iTMUDInsertionPointColor", iTMUDTeXInsertionPointColorKey);
        [[NSUserDefaults standardUserDefaults] synchronize];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description Forthcoming.
Version History: Originally created by RK, huge corrections by JL.
To do list: there remains a lot to do, hu, hu, hu
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [self setMarkButton: nil];
    [self setSectionButton: nil];
    [self setLabelButton: nil];
    [[self docButton] setAction: NULL];
    [[self docButton] removeFromSuperviewWithoutNeedingDisplay];
    [self setDocButton: nil];
    [self setGfxButton: nil];
    [self setMathButton: nil];
    [self setMiscButton: nil];
    [self setTextButton: nil];
    [self setTmplButton: nil];
    [self setGreekButton: nil];
    [self setTextView: nil];
    [self setLineField: nil];
    [self setStatusField: nil];
    [_HiddenScrollView autorelease];
    _HiddenScrollView = nil;
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowDidLoad
- (void) windowDidLoad;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([self respondsToSelector: @selector(installToolbarWithIdentifier:)])
        [self installToolbarWithIdentifier: [self toolbarIdentifier]];
    {
        NSWindow * W = [self window];
        [W setDelegate: [self document]];// very early!!!
        [W setFrameFromStandardUserDefaults];
        [[iTMTextFinder sharedTextFinder] installFindResponderForWindow: W];
        {
            NSTextView * TV = [self textView];
            [TV setContinuousSpellCheckingEnabled: 
                    [[self settings] boolForKey: iTMUDContinuousSpellCheckingKey]];
            [TV setSmartInsertDeleteEnabled: 
                    [[self settings] boolForKey: iTMUDSmartInsertDeleteEnabledKey]];
            [W setInitialFirstResponder: TV];
            [W makeFirstResponder: TV];
            [[TV layoutManager] replaceTextStorage: [[self document] textStorage]];
            [[TV layoutManager] setDelegate: self];
        }
        {
            NSTextView * TV = [self hiddenTextView];
            [TV setContinuousSpellCheckingEnabled: 
                    [[self settings] boolForKey: iTMUDContinuousSpellCheckingKey]];
            [TV setSmartInsertDeleteEnabled: 
                    [[self settings] boolForKey: iTMUDSmartInsertDeleteEnabledKey]];
            [[TV layoutManager] replaceTextStorage: [[self document] textStorage]];
            [[TV layoutManager] setDelegate: self];
        }
        [self fontOrColorDidChangeNotified: nil];// after the text storage has been replaced
        [self updateNumberOfLines];
        [self updateLineField];
        [[NSNotificationCenter defaultCenter] addObserver: self
            selector: @selector(installMenusInButtons:)
                name: @"iTMLoadMacrosAndTemplates"
                    object: nil];
        [[NSNotificationCenter defaultCenter] addObserver: self
            selector: @selector(fontOrColorDidChangeNotified:)
                name: iTMFontOrColorDidChangeNotification
                    object: nil];
        [self installMenusInButtons: nil];
    }
    [self setShouldCascadeWindows: [[self settings] boolForKey: iTMUDTeXCascadeWindowsKey]];
    [self setShouldCloseDocument: YES];
    [super windowDidLoad];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fontOrColorDidChangeNotified:
- (void) fontOrColorDidChangeNotified: (NSNotification *) aNotification;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSTextView * TV = [self textView];
    id settings = [aNotification object];
    if(!settings) settings = [self settings];
    [TV setFont: [NSFont fontWithNameSizeDictionary: [settings objectForKey: iTMUDTeXStandardFontKey]]];
    [TV setDrawsBackground: [settings boolForKey: iTMUDTeXDrawsTextBackgroundKey]];
    if([TV drawsBackground])
    {
        [TV setBackgroundColor: [NSColor colorWithRGBADictionary: [settings objectForKey: iTMUDTeXTextBackgroundColorKey]]];
        [TV setInsertionPointColor:
                [NSColor colorWithRGBADictionary: [settings objectForKey: iTMUDTeXInsertionPointColorKey]]];
        [TV setSelectedTextAttributes: [NSDictionary dictionaryWithObjectsAndKeys:
                [NSColor colorWithRGBADictionary: [settings objectForKey: iTMUDTeXSelectedTextColorKey]],
                        NSForegroundColorAttributeName,
                [NSColor colorWithRGBADictionary: [settings objectForKey: iTMUDTeXSelectedTextBackgroundColorKey]], 				NSBackgroundColorAttributeName,
                    nil]];
    }
    else
    {
        [TV setBackgroundColor: [NSColor textBackgroundColor]];
        [TV setInsertionPointColor: [NSColor blackColor]];
        [TV setSelectedTextAttributes: [NSDictionary dictionaryWithObjectsAndKeys:
                [NSColor selectedTextColor], NSForegroundColorAttributeName,
                [NSColor selectedTextBackgroundColor], NSBackgroundColorAttributeName,
                        nil]];
    }
    {
        NSScrollView * SV = [TV enclosingScrollView];
        [SV setDrawsBackground: YES];
        [SV setBackgroundColor: ([TV drawsBackground]? [TV backgroundColor]: [NSColor textBackgroundColor])];
        [SV setNeedsDisplay: YES];
    }
    {
        iTMTextStorage * TS = [TV textStorage];
        NSAssert1([TS isKindOfClass: [iTMTextStorage class]],
            @"-[iTMTeXWindowController(cocoa) fontOrColorDidChangeNotified:]: iTMTextStorage expected, found: %@", TS);
        {
            // no more UD when the option is project dependant
            BOOL wasSyntaxColoring = [[TS syntaxParser] isColoring];
            if([settings boolForKey: iTMUDTeXSyntaxColoringKey])
            {
                if(!wasSyntaxColoring)
                {
                    [TS replaceSyntaxParser: [iTMTeXSyntaxParser syntaxParser]];
                }
            }
            else
            {
                if(wasSyntaxColoring)
                {
                    [TS replaceSyntaxParser: [iTMSyntaxParser syntaxParser]];
                }
            }
            [[TS syntaxParser] fontOrColorDidChange: nil];
            {
                NSRange R = NSMakeRange(0, [TS length]);
                [TS invalidateAttributesInRange: R];
                [[TV layoutManager] invalidateLayoutForCharacterRange: R isSoft: NO actualCharacterRange: nil];
            }
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textViewDidChangeSelection:
- (void) textViewDidChangeSelection: (NSNotification *) aNotification;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self updateLineField];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textDidChange:
- (void) textDidChange: (NSNotification *) aNotification;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self updateNumberOfLines];
    [self updateLineField];
    // critical: it does not work when edition is complicated, but should in most cases.
    if((_iTMTWCFlags.isiTeXMacCommentEdited>0) || [[[self document] textStorage]
                        isITMCommentAtIndex: [[self textView] selectedRange].location])
        [[self document] cleanCachedData];
    _iTMTWCFlags.isiTeXMacCommentEdited = 0;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  splitViewDidResizeSubviews:
- (void) splitViewDidResizeSubviews: (NSNotification *) aNotification;
/*"Only 2 subviews are supported. If one of them is collapsed, it is replaced with a basic view that will catch the size changes. The scroll view will come back in place if neither view is collapsed. Only one subview is supposed to collapse at a time.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSSplitView * SV = [aNotification object];
    NSEnumerator * E = [[SV subviews] objectEnumerator];
    NSView * V;
    while(V = [E nextObject])
    {
        if([SV isSubviewCollapsed: V] && [V isKindOfClass: [NSScrollView class]])
        {
            NSView * view = [[NSView alloc] initWithFrame: [V frame]];
            [_HiddenScrollView release];
            _HiddenScrollView = [V retain];
            [SV replaceSubview: V with: view];
            [view release];
        }
        else if(![SV isSubviewCollapsed: V] && ![V isKindOfClass: [NSScrollView class]])
        {
            [_HiddenScrollView setFrame: [V frame]];
            [SV replaceSubview: V with: _HiddenScrollView];
            [_HiddenScrollView release];
            _HiddenScrollView = nil;
        }
    }
    return;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  layoutManager:didCompleteLayoutForTextContainer:atEnd:
- (void) layoutManager: (NSLayoutManager *) layoutManager didCompleteLayoutForTextContainer: (NSTextContainer *) textContainer atEnd: (BOOL) layoutFinishedFlag;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(@"Terminato");
    return;
}
#endif
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTeXWindowController(Cocoa)

