/*
//  iTMTeXWindowController.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Mon Dec 03 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import "iTMTeXWindowController.h"
#import "NSWindow(iTeXMac).h"
#import "iTMNavigationFormatter.h"
#import "iTMTextFinder.h"
#import "iTMTextView.h"
#import "iTMMixedButton.h"
#import "iTMTeXToolbarDelegate.h"
#import "iTMGreekMenu.h"

NSString * const iTMUDContinuousSpellCheckingKey = @"iTMCheckSpellingAsYouType";
NSString * const iTMUDTeXSyntaxColoringKey = @"iTMTeXSyntaxColoring";

NSString * const iTMUDTeXDrawsTextBackgroundKey = @"iTMUDTeXDrawsTextBackground";
NSString * const iTMUDTeXTextBackgroundColorKey = @"iTMUDTeXTextBackgroundColor";
NSString * const iTMUDTeXSelectedTextBackgroundColorKey = @"iTMUDTeXSelectedTextBackgroundColor";
NSString * const iTMUDTeXSelectedTextColorKey = @"iTMUDTeXSelectedTextColor";
NSString * const iTMUDTeXInsertionPointColorKey = @"iTMUDTeXInsertionPointColor";

NSString * const iTMSourceToolbarIdentifier = @"TeX";
NSString * const iTMUDTeXCascadeWindowsKey = @"iTMUDTeXCascadeWindows";

@interface iTMTeXWindowController(Private)
- (Class) _ToolbarDelegateClass;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTeXWindowController
/*"Description forthcoming."*/
@implementation iTMTeXWindowController
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  _ToolbarDelegateClass
- (Class) _ToolbarDelegateClass;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (11/27/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([self respondsToSelector: @selector(TeXToolbarDelegateClass)])
        return [self TeXToolbarDelegateClass];
    else
        return [iTMTeXToolbarDelegate class];        
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toolbarDelegate
- (id) toolbarDelegate;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!_TD)
        [self setToolbarDelegate: [[[[self _ToolbarDelegateClass] alloc] init] autorelease]];
    return _TD;        
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setToolbarDelegate:
- (void) setToolbarDelegate: (id) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (11/27/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [_TD autorelease];
    _TD = [argument retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toolbarIdentifier
- (NSString *) toolbarIdentifier;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return @"TeX";
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  _InstallToolbarWithIdentifier:
- (void) installToolbarWithIdentifier: (NSString *) anIdentifier;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([self toolbarDelegate])
    {
        id toolbar = [[NSToolbar alloc] initWithIdentifier: anIdentifier];
        [toolbar setDelegate: [self toolbarDelegate]];
        [toolbar setAllowsUserCustomization: YES];
        [toolbar setAutosavesConfiguration: YES];
        [toolbar setDisplayMode: NSToolbarDisplayModeIconOnly];
        [[self window] setToolbar: toolbar];
        [toolbar release];
        NSLog(@"The toolbar has been installed for identifier: %@", anIdentifier);
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textView
- (id) textView;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _TextView;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setTextView:
- (void) setTextView: (id) argument;
/*"Initializer of the text view (no nib conection).
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSTextView class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSTextView argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    [_TextView autorelease];
    _TextView = [argument retain];
    if(_TextView)
    {
        [_TextView setEditable: YES];
        [_TextView setDelegate: self];
        [_TextView setRichText: NO];
        [_TextView setImportsGraphics: NO];
        [_TextView setUsesFontPanel: NO];
        [_TextView setAllowsUndo: YES];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  hiddenTextView
- (id) hiddenTextView;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _HiddenTextView;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setHiddenTextView:
- (void) setHiddenTextView: (id) argument;
/*"Initializer of the text view (no nib conection).
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSTextView class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSTextView argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    [_HiddenTextView autorelease];
    _HiddenTextView = [argument retain];
    if(_HiddenTextView)
    {
        [_HiddenTextView setEditable: YES];
        [_HiddenTextView setDelegate: self];
        [_HiddenTextView setRichText: NO];
        [_HiddenTextView setImportsGraphics: NO];
        [_HiddenTextView setUsesFontPanel: NO];
        [_HiddenTextView setAllowsUndo: YES];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  mainTextView
- (id) mainTextView;
/*"Returns the hiddenTextView if it the first responder and visible otherwise it returns the textView.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([[[[self hiddenTextView] window] firstResponder] isEqual: [self hiddenTextView]])
    {
        NSSize S = [[self hiddenTextView] visibleRect].size;
        if((S.width>0) && (S.height>0))
            return [self hiddenTextView];
    }
    return [self textView];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  lineField
- (id) lineField;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return self->_LineField;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setLineField:
- (void) setLineField: (id) argument;
/*"DF
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSTextField class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSTextField argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    [_LineField autorelease];
    _LineField = [argument retain];
    [_LineField setTarget: nil];
    [_LineField setAction: @selector(selectedLineChosen:)];
    if(![_LineField formatter])
    {
        [_LineField setFormatter: [[[iTMNavigationFormatter alloc] init] autorelease]];
        [[_LineField formatter] setNavigationFormat: NSLocalizedStringFromTableInBundle( @"Line %1$@ of %2$@", @"TeX",
                    [NSBundle bundleForClass: [self class]], @"DF")];
        [[_LineField formatter] setMinimum: [NSDecimalNumber one]];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  statusField
- (id) statusField;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return self->_StatusField;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setStatusField:
- (void) setStatusField: (id) argument;
/*"DF
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSTextField class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSTextField argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    [_StatusField autorelease];
    _StatusField = [argument retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  BUTTON(, )
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  markButton
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sectionButton
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  labelButton
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  docButton
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  gfxButton
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  mathButton
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  miscButton
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textButton
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  tmplButton
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  greekButton
#define BUTTON(selector, source) - (id) selector;{return self->source;}
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
BUTTON(markButton, _MarkButton);
BUTTON(sectionButton, _SectionButton);
BUTTON(labelButton, _LabelButton);
BUTTON(docButton, _DocButton);
BUTTON(gfxButton, _GfxButton);
BUTTON(mathButton, _MathButton);
BUTTON(miscButton, _MiscButton);
BUTTON(textButton, _TextButton);
BUTTON(tmplButton, _TmplButton);
BUTTON(greekButton, _GreekButton);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  SETBUTTON(, )
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMarkButton:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setSectionButton:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setLabelButton:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setDocButton:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setGfxButton:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMathButton:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMiscButton:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setTextButton:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setGreekButton:
#define SETBUTTON(setter, source) - (void) setter: (id) argument;\
{\
    if(argument && ![argument isKindOfClass: [iTMMixedButton class]])\
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] iTMMixedButton argument expected: %@.",\
            [self class], NSStringFromSelector(_cmd), argument];\
    [self->source autorelease];\
    self->source = [argument retain];\
    return;\
}
/*"DF
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
SETBUTTON(setMarkButton, _MarkButton);
SETBUTTON(setSectionButton, _SectionButton);
SETBUTTON(setLabelButton, _LabelButton);
SETBUTTON(setDocButton, _DocButton);
SETBUTTON(setGfxButton, _GfxButton);
SETBUTTON(setMathButton, _MathButton);
SETBUTTON(setMiscButton, _MiscButton);
SETBUTTON(setTextButton, _TextButton);
SETBUTTON(setTmplButton, _TmplButton);
SETBUTTON(setGreekButton, _GreekButton);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  installMenusInButtons:
- (void) installMenusInButtons: (NSNotification *) aNotification;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMMixedButton * button;
    #define prepareButton(selectorButton)\
    button = [self selectorButton];\
    [button setTarget: [self document]];\
    [button setAction: @selector(selectorButton:)];\
    [button setMixedAction: @selector(notifyAllDelayedInvocations:)]
    prepareButton(markButton);
    prepareButton(sectionButton);
    prepareButton(labelButton);
    #undef prepareButton
    #define prepareButton(selectorButton)\
    button = [self selectorButton];\
    [button setTarget: [self document]];\
    [button setAction: @selector(selectorButton:)];\
    [button setMenu: nil]
    prepareButton(docButton);
    prepareButton(gfxButton);
    prepareButton(mathButton);
    prepareButton(miscButton);
    prepareButton(textButton);
    prepareButton(tmplButton);
//    [[self docButton] validateWindowContent];
    [[self greekButton] setMenu: [[[iTMGreekMenu alloc] initWithTitle: [NSString string]] autorelease]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  currentLineNumber
- (int) currentLineNumber;
/*"Displays aNumber in the line text field.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self lineField] intValue];
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTeXWindowController

