/*
//  iTMTerminalController(UI).m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Wed Jul 18 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
*/

#import <AppKit/NSWindowController.h>
#import <AppKit/NSTextView.h>
#import <AppKit/AppKit.h>
#import "iTMTerminalController(UI).h"
#import "iTMTerminalController.h"

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTerminalController(UI)
/*"This object manages the UI of the iTeXMac terminal."*/
@implementation iTMTerminalController(UI)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initUI
- (void) initUI;
/*"Registers some defaults.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self deallocUI];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= deallocUI
- (void) deallocUI;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setInput: nil];
    [self setOutput: nil];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMGetter()
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= windowController
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= output
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= input
#define iTMGetter(source, getter) - (id) getter;{return source;}
iTMGetter(_Output, output);
iTMGetter(_Input, input);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setOutput:
- (void) setOutput: (NSTextView *) argument;
/*"The delegate of this text view is the receiver.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![_Output isEqual: argument])
    {
        if(argument && ![argument isKindOfClass: [NSTextView class]])
            [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSTextView argument expected: %@.",
                [self class], NSStringFromSelector(_cmd), argument];
        else
        {
            [_Output release];
            _Output = [argument retain];
            [self eraseOutput];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setInput:
- (void) setInput: (NSTextField *) argument;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![_Input isEqual: argument])
    {
        if(argument && ![argument isKindOfClass: [NSTextField class]])
            [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSTextField argument expected: %@.",
                [self class], NSStringFromSelector(_cmd), argument];
        else
        {
            [_Input release];
            _Input = [argument retain];
            [_Input setTarget: self];
            [_Input setAction: @selector(performInput:)];
            [self dontListenToInput];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  logMessage:
- (void) logMessage: (NSString *) aMessage;
/*"Logs the message in black. Not yet customizable.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSTextView * output = [self output];
    [output setEditable: YES];
    [output setSelectedRange: NSMakeRange([[output string] length], 0)];
    [output insertText: aMessage];
    [output setEditable: NO];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  logError:
- (void) logError: (NSString *) aMessage;
/*"Logs the message in blue. Not yet customizable.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSTextView * output = [self output];
    [output setEditable: YES];
    [output setSelectedRange: NSMakeRange([[output string] length], 0)];
    [output insertText: aMessage];
    [output setEditable: NO];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  logCommand:
- (void) logCommand: (NSString *) aMessage;
/*"Logs the message in blue. Not yet customizable.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSTextView * output = [self output];
    [output setEditable: YES];
    [output setSelectedRange: NSMakeRange([[output string] length], 0)];
    [output insertText: aMessage];
    [output setEditable: NO];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= performInput:
- (IBAction) performInput: (id) sender;
/*"Action of the input text field. Sends #{standardInputString:} to the terminal.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self standardInputString: [sender stringValue] verbose: YES];
    [sender setStringValue: @""];
    [[sender window] makeFirstResponder: sender];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= eraseOutput
- (void) eraseOutput;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self output] setString: [NSString string]];
    [[self output] setTypingAttributes: nil];
    return;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTerminalController(UI)

