/*
//  iTMTerminalController.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Wed Jul 18 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
*/

#import <Foundation/NSTask.h>
#import <Foundation/NSBundle.h>
#import <Foundation/NSProcessInfo.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/Foundation.h>
#import <AppKit/NSTextView.h>
#import <AppKit/NSTextField.h>
#import "NSControl(iTeXMac).h"
#import "iTMTerminalController.h"
#import "iTMTerminalController(UI).h"
#import "iTMStatusNotification.h"

NSString * const iTMTCTaskKey = @"task";
NSString * const iTMTCInvocationKey = @"invocation";
NSString * const iTMTCUserInfoKey = @"userInfo";
NSString * const iTMTCPrefixKey = @"prefix";
NSString * const iTMTCScriptKey = @"script";
NSString * const iTMTCOutputKey = @"output";

NSString * const iTMTaskTerminationStatus = @"iTM Task Termination Status";
NSString * const iTMTerminalCurrentTaskKey = @"iTM Terminal Current Task";
NSString * const iTMTerminalBehaviorKey = @"iTM Terminal Behavior";
NSString * const iTMiTeXMacTerminalKey = @"iTeXMac Terminal";
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTerminalController
/*"Task part of the iTeXMac terminal. Somehow dirty class. Needs to be redesigned (not the public meant API)
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List: implement listenOutput, post a notification when input is needed. start with a callback
"*/
@implementation iTMTerminalController
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialize
+ (void) initialize;
/*"Registers some defaults.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    {
        NSDictionary * factoryDefaults = [NSDictionary dictionaryWithObjectsAndKeys: 
                        @"0", iTMTerminalBehaviorKey,
                        nil];
        [[NSUserDefaults standardUserDefaults] registerDefaults: factoryDefaults];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= addShellTaskWithEnvironment:standardInput:directory:arguments:delegate:didExecuteSelector:userInfo:restart:
- (void) addShellTaskWithEnvironment: (NSDictionary *) aDictionary
standardInput: (NSString *) aScript
directory: (NSString *) aPath
delegate: (id) aDelegate
didExecuteSelector: (SEL) aSelector
userInfo: (NSDictionary *) aUI
restart: (BOOL) restart;
/*"Creates a /bin/tcsh task.
Each time a command is executed, the call back is called. When there is no call back given, we create a fake invocation.
There is no call back when either no delegate or no selector is given. When the delegate does not respond to the selector,
there is no call back too.
aSelector should have the following signature:
- (void) terminalDidStop: (id) aTerminal userInfo: (NSDictionary *) aUI;
The returned user info given in the call back is a dictionary with two entries: for key iTMTCTaskKey we have the task that has terminate, and for key iTMTCOriginalUserInfoKey, the sender of the present message  will retrieve the user information he provided.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List: Little.
"*/
{
    if(restart) [self closeTerminal: self];
    {
        NSMutableDictionary * MD = [NSMutableDictionary dictionary];
        {
            NSTask * task = [[NSTask alloc] init];
            [task setLaunchPath: @"/bin/tcsh"];
            if([[NSFileManager defaultManager] isReadableFileAtPath: aPath])
                [task setCurrentDirectoryPath: aPath];
            else
                [task setCurrentDirectoryPath: NSHomeDirectory()];
            {
                NSMutableDictionary * ED = [NSMutableDictionary dictionary];
                NSMutableArray * prefix = [NSMutableArray arrayWithObject: @"source /usr/share/init/tcsh/login"];
                #if 0
                {
                    NSString * texmfPath = [[NSUserDefaults standardUserDefaults] stringForKey: iTMUDTeXMFPATHKey];
                    if([texmfPath length])
                    {
                        [ED setObject: texmfPath forKey: iTMUDTeXMFPATHKey];
                        [prefix addObject: [NSString stringWithFormat: @"set texmfPath = \"${%@}\"", iTMUDTeXMFPATHKey]];
                        [prefix addObject: @"setenv PATH \"${texmfPath}:${PATH}\""];
                    }
                }
                {
                    NSString * gsPath = [[NSUserDefaults standardUserDefaults] stringForKey: iTMGhostScriptPATHKey];
                    if([gsPath length])
                    {
                        [ED setObject: gsPath forKey: iTMGhostScriptPATHKey];
                        [prefix addObject: [NSString stringWithFormat: @"set gsPath = \"${%@}\"", iTMGhostScriptPATHKey]];
                        [prefix addObject: @"setenv PATH \"${gsPath}:${PATH}\""];
                    }
                }
                {
                    NSString * executablePath = [[[NSBundle mainBundle]
                                pathForAuxiliaryExecutable: @"e Helper"] stringByDeletingLastPathComponent];
                    if([executablePath length])
                    {
                        [ED setObject: executablePath forKey: executablePath];
                        [prefix addObject: [NSString stringWithFormat:
                                    @"set auxiliaryPath = \"%@\"", executablePath]];
                        [prefix addObject: @"setenv PATH \"${auxiliaryPath}:${PATH}\""];
                    }
                    else if(![[NSUserDefaults standardUserDefaults] boolForKey: @"iTMAuxiliaryExecutableMissing"])
                    {
                        NSRunCriticalAlertPanel(
                    NSLocalizedStringFromTableInBundle(@"iTeXMac Helpers", @"Basic",
                    [NSBundle bundleForClass: [self class]], "Executable resource problem title"), NSLocalizedStringFromTableInBundle(
                @"Missing built in helper scripts. Please reinstall soon. You can still use command lines.",
            @"Basic",
                    [NSBundle bundleForClass: [self class]], "Resource problem"), nil, nil, nil);
                [[NSUserDefaults standardUserDefaults] registerDefaults:
                    [NSDictionary dictionaryWithObject: [NSNumber numberWithBool: YES]
                        forKey: @"iTMAuxiliaryExecutableMissing"]];
                    }
                }
                #endif
                [MD setObject: [prefix componentsJoinedByString: @"\n"] forKey: iTMTCPrefixKey];
                if(aDictionary) [ED addEntriesFromDictionary: aDictionary];
                [task setEnvironment: ED];
            }
            [MD setObject: task forKey: iTMTCTaskKey];
            [task release];
        }
        {
            NSInvocation * invocation = nil;
            if((aSelector != NULL) && [aDelegate respondsToSelector: aSelector])
            {
                NSMethodSignature * MS = [aDelegate methodSignatureForSelector: aSelector];
                invocation = [NSInvocation invocationWithMethodSignature: MS];
                [invocation setSelector: aSelector];
                [invocation setTarget: aDelegate];
                if(aUI)[MD setObject: aUI forKey: iTMTCUserInfoKey];
                [invocation setArgument: &MD atIndex: 3];
                [invocation retainArguments];
            }
            else
                NSLog(@"No invocation call back: %@, %@", aDelegate, NSStringFromSelector(aSelector));
            if(invocation) [MD setObject: invocation forKey: iTMTCInvocationKey];
        }
        if([aScript length])
        {
//            NSLog(@">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>    aScript: %@", aScript);
            [MD setObject: [aScript stringByAppendingString: @"\nexit\n"] forKey: iTMTCScriptKey];// why not ; ?????
        }
        else
            [MD setObject: @"alias precmd \'echo \"iTeXMac>\"\'\n" forKey: iTMTCScriptKey];
        [[self taskDictionaries] addObject: MD];
        [self startFirstTask];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= addTaskWithLaunchPath:environment:directory:arguments:delegate:didExecuteSelector:userInfo:restart:
- (void) addTaskWithLaunchPath: (NSString *) aLaunchPath
environment: (NSDictionary *) aDictionary
arguments: (NSArray *) arguments
directory: (NSString *) aPath
delegate: (id) aDelegate
didExecuteSelector: (SEL) aSelector
userInfo: (NSDictionary *) aUI
restart: (BOOL) restart;
/*"Creates a task assuming that there is an executable file at aPath (a consistency test is performed).
Each time a command is executed, the call back is called. When there is no call back given, we create a fake invocation.
There is no call back when either no delegate or no selector is given. When the delegate does not respond to the selector,
there is no call back too.
aSelector should have the following signature:
- (void) terminalDidStop: (id) aTerminal userInfo: (NSDictionary *) aUI;
The returned user info given in the call back is a dictionary with two entries: for key iTMTCTaskKey we have the task that has terminate, and for key iTMTCOriginalUserInfoKey, the sender of the present message  will retrieve the user information he 
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List: Little.
"*/
{
    if(restart) [self closeTerminal: self];
    aLaunchPath = [aLaunchPath stringByStandardizingPath];
    if([[NSFileManager defaultManager] isExecutableFileAtPath: aLaunchPath])
    {
        NSMutableDictionary * MD = [[NSMutableDictionary alloc] initWithCapacity: 3];
        {
            NSTask * task = [[NSTask alloc] init];
            [task setLaunchPath: aLaunchPath];
            if([[NSFileManager defaultManager] isReadableFileAtPath: aPath])
                [task setCurrentDirectoryPath: aPath];
            else
                [task setCurrentDirectoryPath: NSHomeDirectory()];
            if(arguments) [task setArguments: arguments];
            if(aDictionary) [task setEnvironment: aDictionary];
            [MD setObject: task forKey: iTMTCTaskKey];
            [task release];
        }
        {
            NSInvocation * invocation = nil;
            if((aSelector != NULL) && [aDelegate respondsToSelector: aSelector])
            {
                NSMethodSignature * MS = [aDelegate methodSignatureForSelector: aSelector];
//                NSLog(@"%@: %@", NSStringFromSelector(aSelector), MS);
                invocation = [NSInvocation invocationWithMethodSignature: MS];
                [invocation setSelector: aSelector];
                [invocation setTarget: aDelegate];
                if(aUI)[MD setObject: aUI forKey: iTMTCUserInfoKey];
                [invocation setArgument: &MD atIndex: 3];
                [invocation retainArguments];
            }
            else
                NSLog(@"No invocation call back: %@, %@", aDelegate, NSStringFromSelector(aSelector));
            if(invocation) [MD setObject: invocation forKey: iTMTCInvocationKey];
        }
        [[self taskDictionaries] addObject: MD];
        [MD release];
        [self startFirstTask];
    }
    else
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] no executable file at path %@.",
            [self class], NSStringFromSelector(_cmd), aLaunchPath];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= startFirstTask
- (void) startFirstTask;
/*"Launching the firts available task in the task stack. Connets the I/O, registers the receiver to the default notification center.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([[self taskDictionaries] count])
    {
        NSDictionary * D = [[self taskDictionaries] objectAtIndex: 0];
        NSTask * task = [D objectForKey: iTMTCTaskKey];
        if(task &&![task isRunning])
        {
            [task setStandardError: [NSPipe pipe]];
            [task setStandardOutput: [NSPipe pipe]];
            [task setStandardInput: [NSPipe pipe]];
            [[NSNotificationCenter defaultCenter] removeObserver: self name: NSFileHandleReadCompletionNotification
                object: [[task standardOutput] fileHandleForReading]];
            [[NSNotificationCenter defaultCenter] removeObserver: self name: NSFileHandleReadCompletionNotification
                object: [[task standardError] fileHandleForReading]];
            [[NSNotificationCenter defaultCenter] removeObserver: self name: NSTaskDidTerminateNotification object: task];
            [[NSNotificationCenter defaultCenter] addObserver: self
                selector: @selector(outputReadCompletion:)
                    name: NSFileHandleReadCompletionNotification
                        object: [[task standardOutput] fileHandleForReading]];
            [[NSNotificationCenter defaultCenter] addObserver: self
                selector: @selector(errorReadCompletion:)
                    name: NSFileHandleReadCompletionNotification
                        object: [[task standardError] fileHandleForReading]];
            [[NSNotificationCenter defaultCenter] addObserver: self
                selector: @selector(_taskDidTerminate:) 
                    name: NSTaskDidTerminateNotification
                        object: task];
            _NormalTermination = YES;
    //        [self dontListenToInput];
            [self listenToInput];
            // lazy: never retained, not orthogonal
            [self listenToTask: task];
            [self eraseOutput];
            [self logCommand: [task launchPath]];
            [self logCommand: @"...\n"];
            {
                NSMutableDictionary * ED = [[[NSProcessInfo processInfo] environment] mutableCopy];
                [ED addEntriesFromDictionary: [task environment]];
                [task setEnvironment: ED];
                [ED release];
            }
            [task launch];
//            NSLog(@"%@ launched", task);
            {
                NSString * prefix = [D objectForKey: iTMTCPrefixKey];
                if([prefix length]) [self standardInputString: prefix verbose: YES];
            }
            {
                NSString * script = [D objectForKey: iTMTCScriptKey];
                if([script length]) [self standardInputString: script verbose: YES];
            }
            return;
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= _taskDidTerminate:
- (void) _taskDidTerminate: (NSNotification *) aNotification;
/*"Description Forthcoming. Not orthogonal.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSTask * task = [aNotification object];
    NSDictionary * D = [[self taskDictionaries] count]? [[self taskDictionaries] objectAtIndex: 0]: nil;
    [[NSNotificationCenter defaultCenter] removeObserver: nil name: nil object: task];
    NSLog(@"_taskDidTerminate: %@", task);
    if(![task isEqual: [D objectForKey: iTMTCTaskKey]])
        [NSException raise: NSInternalInconsistencyException format: @"-[%@ %@] tasks not corresponding: %@ != %@.",
            [self class], NSStringFromSelector(_cmd), task, [D objectForKey: iTMTCTaskKey]];
    else
    {
        [self dontListenToInput];
        {
            NSData* data = [[[task standardOutput] fileHandleForReading] availableData];
            if([data length]>0)
            {
                NSString * string = [[[NSString alloc] initWithData: data encoding: NSMacOSRomanStringEncoding] autorelease];
                [self logMessage: string];
            }
            data = [[[task standardError] fileHandleForReading] availableData];
            if([data length]>0)
            {
                NSString * string = [[[NSString alloc] initWithData: data encoding: NSMacOSRomanStringEncoding] autorelease];
                [self logError: string];
            }
        }
        if(_NormalTermination)
        {
            NSInvocation * invocation = [D objectForKey: iTMTCInvocationKey];
            if(invocation)
            {
                id userInfo = [D objectForKey: iTMTCUserInfoKey];
                NSMutableDictionary * MD = [[NSMutableDictionary alloc] initWithCapacity: 3];
                [MD setObject: task forKey: iTMTCTaskKey];
                if(userInfo)[MD setObject: userInfo forKey: iTMTCUserInfoKey];
                {
                    NSString * output = [[self output] string];
                    if([output length] && ([output length]<1000))
                        [MD setObject: [[output copy] autorelease] forKey: iTMTCOutputKey];
                }
                [invocation setArgument: &MD atIndex: 3];
                [invocation setArgument: &self atIndex: 2];
                [invocation invoke];
                [MD release];
            }
            else
                NSLog(@"Nothing to invoke.");
        }
        else
            NSLog(@"Abnormal termination: %d.", _NormalTermination);
        if([[self taskDictionaries] count])
            [[self taskDictionaries] removeObjectAtIndex: 0];
        else
            NSLog(@"An object has disappeared in [self taskDictionaries].");
        [self startFirstTask];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= standardInputString:verbose:
- (void) standardInputString: (NSString *) aCommand verbose: (BOOL) verbose;
/*"Description Forthcoming. Rough input, no verification.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List: add verification. Split BIG strings into small ones...
"*/
{
    NSTask * CT = [self currentTask];
    if(CT)
    {
        if(verbose)
            [self logCommand: aCommand];
//        NSLog(@"standardInputString %@", [[CT standardInput] fileHandleForWriting]);
        [[[CT standardInput] fileHandleForWriting] writeData:
            [aCommand dataUsingEncoding: NSMacOSRomanStringEncoding allowLossyConversion: YES]];
        if(![aCommand hasSuffix: @"\n"])
        {
            [[[CT standardInput] fileHandleForWriting] writeData:
                [@"\n" dataUsingEncoding: NSMacOSRomanStringEncoding allowLossyConversion: YES]];
            if(verbose)
                [self logCommand: @"\n"];
        }
        [self listenToTask: CT];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= currentTask
- (NSTask *) currentTask;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List: add verification. Split BIG strings into small ones...
"*/
{
    return [[self taskDictionaries] count]? [[[self taskDictionaries] objectAtIndex: 0] objectForKey: iTMTCTaskKey]: nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= outputReadCompletion:
- (void) outputReadCompletion: (NSNotification *) aNotification;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * string = [[NSString alloc] initWithData:
            [[aNotification userInfo] objectForKey: @"NSFileHandleNotificationDataItem"]
                    encoding: NSMacOSRomanStringEncoding];
    if([string length]>0)
    {
        [self logMessage: string];
        {
            NSTextView * output = [self output];
            NSString * S = [output string];
            unsigned startIndex, contentsEndIndex, endIndex;
            NSRange range = NSMakeRange([S length]-[string length], [string length]);
            [S getLineStart: &startIndex end: &endIndex contentsEnd: &contentsEndIndex forRange: range];
            range = NSMakeRange(startIndex, contentsEndIndex - startIndex);
            {
                NSString * SS = [S substringWithRange: range];
                if([SS hasSuffix: @"? "] || [SS hasSuffix: @"Enter file name: "])
                {
                    [self listenToInput];
                    NSBeep();
                    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
                        NSLocalizedStringFromTableInBundle(@"Terminal may need your help...",
                            @"General", [NSBundle bundleForClass: [self class]],
                                @"Status displayed when an error occurred.")];
                }
                else if([[self currentTask] isRunning])
                    [self listenToTask: [self currentTask]];
            }
        }
    }
    [string release];
    [[self output] validateWindowContent];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= errorReadCompletion:
- (void) errorReadCompletion: (NSNotification *) aNotification;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * string = [[[NSString alloc] initWithData:
            [[aNotification userInfo] objectForKey: @"NSFileHandleNotificationDataItem"]
                    encoding: NSMacOSRomanStringEncoding] autorelease];
//    NSLog(@"errorReadCompletion <%@>", string);
    if([string length]>0)
    {
        [self logError: string];
        if([[self currentTask] isRunning])
            [self listenToTask: [self currentTask]];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= listenToInput
- (void) listenToInput;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _Listens = YES;
    [[self input] setEnabled: YES];
    [[[self input] window] makeFirstResponder: [self input]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= listenIfNeeded
- (void) listenIfNeeded;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List: THIS CODE IS NOT STRONG!!! crash when the contentView and the window are separated
"*/
{
    if(_Listens)
    {
        [[self input] setEnabled: YES];
        [[[self input] window] makeFirstResponder: [self input]];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= dontListenToInput
- (void) dontListenToInput;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _Listens = NO;
    [[self input] setEnabled: NO];
    [[[self input] window] makeFirstResponder: [self output]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= listenToTask
- (void) listenToTask: (NSTask *) task;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[[task standardOutput] fileHandleForReading] readInBackgroundAndNotify];
    [[[task standardError] fileHandleForReading] readInBackgroundAndNotify];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= taskDictionaries
- (NSMutableArray *) taskDictionaries;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!_TDs)
        [self setTaskDictionaries: [NSMutableArray array]];
    return _TDs;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setTaskDictionaries:
- (void) setTaskDictionaries: (id) argument;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![_TDs isEqual: argument])
    {
        if(argument && ![argument isKindOfClass: [NSMutableArray class]])
            [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSMutableArray argument expected: %@.",
                [self class], NSStringFromSelector(_cmd), argument];
        else
        {
            [_TDs release];
            _TDs = [argument retain];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= closeTerminal:
- (void) closeTerminal: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: nil name: nil object: self];
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    {
        NSTask * currentTask = [[self currentTask] retain];
        [self setTaskDictionaries: nil];
        if([currentTask isRunning])
        {
            [currentTask terminate];
            #warning risky enterprise
            [currentTask waitUntilExit];
                [currentTask release];
        }
    }
    #warning bug if uncommented, to JL: see the problem further please...
//    [[self output] validateWindowContent];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= dealloc
- (void) dealloc;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/11/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self closeTerminal: self];
    [self deallocUI];
    [super dealloc];
    return;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTerminalController

