/*
//  iTMEncodingController.h
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sat Nov 10 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSObject.h>
#import <Foundation/NSString.h>

@class NSMenu, NSNotification;
@protocol NSMenuItem;

extern NSString * const iTMPreferedLineEndingsKey;
extern NSString * const iTMIsAutoStringEncodingKey;
extern NSString * const iTMPreferredStringEncodingKey;
extern NSString * const iTMOpenStringEncodingKey;

extern NSString * const iTMEncodingHeaderKey;

typedef enum
{
    iTMUNIXLineEndings = 1,
    iTMMacintoshLineEndings = 2,
    iTMWindowsLineEndings = 3,
    iTMUnicodeLineEndings = 4,
    iTMUnknownLineEndings = 5//for a further version...
} iTMLineEndings;

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMEncodingController
/*"Description forthcoming."*/
@interface iTMEncodingController: NSObject
/*"Class methods"*/
+ (void) initialize;
/*"Public methods"*/
+ (void) useLineEndings: (id <NSMenuItem>) sender;
+ (void) preferLineEndings: (id <NSMenuItem>) sender;
+ (void) useStringEncoding: (id <NSMenuItem>) sender;
+ (void) preferStringEncoding: (id <NSMenuItem>) sender;
+ (void) useStringEncoding: (id <NSMenuItem>) sender;
/*"Setters and Getters"*/
+ (int) preferredStringEncoding;
+ (void) setPreferredStringEncoding: (int) argument;
+ (NSMenu *) lineEndingsMenuWithAction: (SEL) anAction target: (id) aTarget;
+ (NSMenu *) stringEncodingsMenuWithAction: (SEL) anAction target: (id) aTarget;
+ (NSArray *) supportedEncodings;
+ (NSString *) terminationStringForLineEndings: (iTMLineEndings) lineEndings;
+ (iTMLineEndings) lineEndingsUsedInString: (NSString *) argument;
+ (iTMLineEndings) lineEndingsUsedInDocument: (id) argument;
+ (CFStringEncoding) coreFoundationStringEncodingFromString: (NSString *) argument;
+ (NSNumber *) stringEncodingFromString: (NSString *) argument;
/*"Main methods"*/
+ (void) applicationDidFinishLaunching: (NSNotification *) aNotification;
+ (BOOL) validateMenuItem: (id <NSMenuItem>) sender;
/*"Overriden methods"*/
@end


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= NSString(iTMEncodingController)
/*"Description forthcoming."*/
@interface NSString(iTMEncodingController)
- (NSString *) stringByUsingLineEndings: (iTMLineEndings) lineEndings;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= NSObject(iTMEncodingController)
/*"Description forthcoming."*/
@interface NSObject(iTMEncodingController)
- (int unsigned) lineEndingsUsed;
- (void) setLineEndingsUsed: (int unsigned) argument;
- (NSStringEncoding) stringEncoding;
- (void) setStringEncoding: (NSStringEncoding) argument;
- (BOOL) canSetStringEncoding;
- (id) textStorage;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMEncodingController

