/*
//  iTMTextStorage.h
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Oct 16 2001.
//  Copyright  2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>

#import <Foundation/NSString.h>
#import <AppKit/NSTextStorage.h>

@class NSMutableAttributedString, NSArray, NSDictionary, iTMSyntaxParser;

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTextStorage

@interface iTMTextStorage : NSTextStorage
{
@protected
    id _SP;    
@private
    NSMutableAttributedString * _MAS;
    NSStringEncoding _OpenStringEncoding;
    id _IAC; /*"in[put|clude] (La)TeX anchors"*/
    id _MAC; /*"mark (no in[put|clude] (La)TeX anchors)"*/
    id _SAC; /*"section (no in[put|clude] (La)TeX anchors)"*/
    id _LAC; /*"label/ref (no in[put|clude] (La)TeX anchors)"*/
    NSRange _InvalidAttributesRange;
    struct __iTMiTSFlags
    {
        unsigned int canSetStringEncoding: 1;
        unsigned int lineEndingsUsed: 3;
        unsigned int timer: 1;
        unsigned int reserved: 27;
    } _iTMiTSFlags;
}
/*"Class methods"*/
+ (void) initialize;
+ (Class) syntaxParserClass;
/*"Setters and Getters"*/
- (NSData *) encodedStringDataAllowLossyConversion: (BOOL) flag;
- (id) syntaxParser;
- (void) replaceSyntaxParser: (id) argument;
- (void) setSyntaxParser: (id) argument;
- (BOOL) isITMCommentAtIndex: (unsigned) aLocation;
- (NSRange) groupRangeAtIndex: (unsigned) index;
- (NSStringEncoding) stringEncoding;
- (NSStringEncoding) dataStringEncoding;
- (void) setStringEncoding: (NSStringEncoding) argument;
- (BOOL) canSetStringEncoding;
/*"Main methods"*/
- (void) initSyntaxParser;
- (void) invalidateLayout;
- (NSRange) doubleClickAtIndex: (unsigned) index;
- (NSRange) smartDoubleClickAtIndex: (unsigned) index;
- (id) initWithEncodedStringData: (NSData *) docData;
- (void) cleanAttributesInRange: (NSRange) aRange;
- (void) refixAttributesInRange: (NSRange) aRange;
/*"Overriden methods"*/
- (NSString *) string;
- (NSMutableAttributedString *) mutableAttributedString;
- (NSDictionary *) attributesAtIndex: (unsigned) location effectiveRange: (NSRangePointer) aRangePtr;
- (void) setAttributes: (NSDictionary *) attributes range: (NSRange) aRange;
- (void) replaceCharactersInRange: (NSRange) aRange withString: (NSString *) aString;
- (void) fixAttributesInRange: (NSRange) aRange;
- (id) init;
- (id) initWithString: (NSString *) aString attributes: (NSDictionary *) attributes;
- (void) dealloc;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTextStorage
