/*
//  NSString(iTeXMac).m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sat Jun 16 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
*/

#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSFileManager.h>
#import <Foundation/NSCharacterSet.h>
#import <AppKit/NSMenu.h>
#include <Carbon/Carbon.h>
#import "NSString(iTeXMac).h"
#import "iTMEncodingController.h"
#import "iTMLiteScanner.h"

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= NSString(iTeXMac)
/*"Description forthcoming."*/
@implementation NSString(iTeXMac)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= isFinderAliasTraverseLink:isDirectory:
- (BOOL) isFinderAliasTraverseLink: (BOOL) aFlag isDirectory: (BOOL *) isDirectory;
/*"Returns YES if the receiver is a Finder Alias.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    self = [self stringByStandardizingPath];
//    NSLog(@"introspecting %@", self);
    if([[[[NSFileManager defaultManager] fileAttributesAtPath: self traverseLink: aFlag] fileType]
        isEqualToString: NSFileTypeRegular])
    {
        FSRef fileSystemRef;
        int error = FSPathMakeRef( [[NSFileManager defaultManager] fileSystemRepresentationWithPath: self],
                        &fileSystemRef, isDirectory );
        if(error == noErr)
        {
            BOOL aliasFileFlag;
            BOOL safeIsDirectory;
            if(!isDirectory)
                isDirectory = &safeIsDirectory;
            error = FSIsAliasFile ( &fileSystemRef, &aliasFileFlag, isDirectory );
            if(error == noErr)
                return aliasFileFlag;
            else
                NSLog(@"-[%@ %@] FSIsAliasFile error: %d.", [self class], NSStringFromSelector(_cmd), error);
        }
        NSLog(@"-[%@ %@] FSPathMakeRef error: %d.", [self class], NSStringFromSelector(_cmd), error);
    }
    return NO;
}
/*
OSStatus FSPathMakeRef ( const UInt8 *path, FSRef *ref, Boolean *isDirectory );
OSStatus FSRefMakePath ( const FSRef *ref, UInt8 *path, UInt32 pathSize );
 OSErr FSIsAliasFile ( const FSRef *fileRef, Boolean *aliasFileFlag, Boolean *folderFlag ); 
*/
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= stringByResolvingFinderAliasesInPath
- (NSString *) stringByResolvingFinderAliasesInPath;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    FSRef ref;
    OSErr error;
    error = FSPathMakeRef([[NSFileManager defaultManager] fileSystemRepresentationWithPath: self], &ref, nil);
    if(error)
    {
        NSLog(@"-[%@ %@] FSPathMakeRef error: %d.", [self class], NSStringFromSelector(_cmd), error);
        return self;
    }
    else
    {
        BOOL isDirectory = NO;
        BOOL wasAlias = NO;
        error = FSResolveAliasFile(&ref, YES, &isDirectory, &wasAlias);
        if(error)
        {
            NSLog(@"-[%@ %@] FSResolveAliasFile error: %d.", [self class], NSStringFromSelector(_cmd), error);
            return self;
        }
        else if(wasAlias)
        {
            char path[2048];
            error = FSRefMakePath (&ref, path, 2048);
            if(error)
            {
                NSLog(@"-[%@ %@] FSResolveAliasFile error: %d.", [self class], NSStringFromSelector(_cmd), error);
                return self;
            }
            else
                return [NSString stringWithUTF8String: path];
        }
        else
            return self;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= stringByResolvingSymlinksAndFinderAliasesInPath
- (NSString *) stringByResolvingSymlinksAndFinderAliasesInPath;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    int firewall = 257;
    NSString * temp;
    do
    {
        temp = self;
        self = [temp stringByResolvingSymlinksInPath];
        self = [self stringByResolvingFinderAliasesInPath];
    }
    while((--firewall>0) && ![temp isEqualToString: self]);
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= stringByAbbreviatingWithDotRelativeToDirectory:
- (NSString *) stringByAbbreviatingWithDotRelativeToDirectory: (NSString *) aPath;
/*"It is not strong: the sender is responsible of the arguments, if they do not represent directory paths, the shame on it.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    self = [self stringByStandardizingPath];
    aPath = [aPath stringByStandardizingPath];
    if([self length] && [aPath length] && ([self characterAtIndex: 0]=='/') && ([aPath characterAtIndex: 0]=='/'))
    {
        NSArray * components = [self pathComponents];
        NSArray * pathComponents = [aPath pathComponents];
        // common part
        int commonIdx = 0, index = 0;
        int bound = MIN([components count], [pathComponents count]);
        while(commonIdx < bound)
            if([[components objectAtIndex: commonIdx] isEqualToString: [pathComponents objectAtIndex: commonIdx]])
                ++commonIdx;
            else
                break;
        index = commonIdx;
        self = [NSString string];
        while(index++ < [pathComponents count])
            self = [self stringByAppendingPathComponent: @".."];
        index = commonIdx;
        while(index < [components count])
            self = [self stringByAppendingPathComponent: [components objectAtIndex: index++]];
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= rangeForLine:nextLine:
- (NSRange) rangeForLine: (int) aLine nextLine: (int *) aNextLinePtr;
/*"Given a line number, it returns the line range including the ending characters.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned int start, end;
    NSRange range = NSMakeRange(0, 0);
    typedef void (*GetLineStartIMP) (id, SEL, unsigned *, unsigned *, unsigned *, NSRange);
    GetLineStartIMP GLS = (GetLineStartIMP)
        [self methodForSelector: @selector(getLineStart:end:contentsEnd:forRange:)];
    if(!aNextLinePtr)
        aNextLinePtr = &end;
    * aNextLinePtr = 0;
    while (aLine-->0)
    {
        GLS(self, @selector(getLineStart:end:contentsEnd:forRange:), &start, aNextLinePtr, nil, range);
        range.location = * aNextLinePtr;
    }
    return NSMakeRange(start, * aNextLinePtr - start);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= rangeForLineRange:nextLine:
- (NSRange) rangeForLineRange: (NSRange) aLineRange nextLine: (int *) aNextLinePtr;
/*"Given a line range number, it returns the range including the ending characters.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned int start, end;
    NSRange range = NSMakeRange(0, 0);
    typedef void (*GetLineStartIMP) (id, SEL, unsigned *, unsigned *, unsigned *, NSRange);
    GetLineStartIMP GLS = (GetLineStartIMP)
        [self methodForSelector: @selector(getLineStart:end:contentsEnd:forRange:)];
    if(!aNextLinePtr) aNextLinePtr = &end;
    * aNextLinePtr = 0;
    while (aLineRange.location-->0)
    {
        GLS(self, @selector(getLineStart:end:contentsEnd:forRange:), &start, aNextLinePtr, nil, range);
        range.location = * aNextLinePtr;
    }
    while (--aLineRange.length>0)
    {
        GLS(self, @selector(getLineStart:end:contentsEnd:forRange:), nil, aNextLinePtr, nil, range);
        range.location = * aNextLinePtr;
    }
    return NSMakeRange(start, * aNextLinePtr - start);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= rangeContentForLine:nextLine:
- (NSRange) rangeContentForLine: (int) aLine nextLine: (int *) aNextLinePtr;
/*"Given a line number, it returns the line range NOT including the ending characters.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned start, end, contentsEnd;
    NSRange range = NSMakeRange(0, 0);
    typedef void (*GetLineStartIMP) (id, SEL, unsigned *, unsigned *, unsigned *, NSRange);
    GetLineStartIMP GLS = (GetLineStartIMP)
        [self methodForSelector: @selector(getLineStart:end:contentsEnd:forRange:)];
    if(!aNextLinePtr)
        aNextLinePtr = &end;
    * aNextLinePtr = 0;
    while (aLine-->0)
    {
        GLS(self, @selector(getLineStart:end:contentsEnd:forRange:), &start, aNextLinePtr, &contentsEnd, range);
        range.location = end;
    }
    return NSMakeRange(start, contentsEnd - start);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= lineForRange:
- (unsigned) lineForRange: (NSRange) aRange;
/*"Given a range, it returns the line number of the first char of the range.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: improve the search avoiding the whole scan of the string, refer to the midle of the string or to the first visible character.
"*/
{
    int result = 0;
    unsigned end = 0;
    unsigned contentsEnd = 0;
    int ceiling = MIN(aRange.location + 1, [self length]);
    NSRange range = NSMakeRange(0, 0);
    typedef void (*GetLineStartIMP) (id, SEL, unsigned *, unsigned *, unsigned *, NSRange);
    GetLineStartIMP GLS = (GetLineStartIMP)
        [self methodForSelector: @selector(getLineStart:end:contentsEnd:forRange:)];
    while (end < ceiling)
    {
        ++result;
        GLS(self, @selector(getLineStart:end:contentsEnd:forRange:), nil, &end, &contentsEnd, range);
        range.location = end;
    }
    if((aRange.location > contentsEnd) || ([self length] == 0))
        ++result;
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= numberOfLines
- (unsigned) numberOfLines;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned result = [self lineForRange: NSMakeRange([self length], 0)];
//    NSLog(@"%@: %d lines", self, result);
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= boolValue
- (BOOL) boolValue;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMLiteScanner * scanner = [iTMLiteScanner scannerWithString: self];
    [scanner setCaseSensitive: NO];
    if([scanner scanString: @"YES" intoString: nil])
        return YES;
    else if([scanner scanString: @"NO" intoString: nil])
        return NO;
    else
        return ([self intValue] != 0);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= stringByRemovingTrailingWhites
- (NSString *) stringByRemovingTrailingWhites;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned index = [self length];
    if(!index)
        return [NSString string];
    else
    {
        while((index>0) && [[NSCharacterSet whitespaceCharacterSet] characterIsMember: [self characterAtIndex: --index]]);
        return [self substringWithRange: NSMakeRange(0, ++index)];
    }
}
//=-=-=-=-=-=  stringWithSubstring:replacedByString:
- (NSString*) stringWithSubstring: (NSString*) oldString replacedByString: (NSString*) newString;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMutableString * result = [self mutableCopy];
    NSRange searchRange = NSMakeRange(0,[self length]);
    if(!newString) newString = [NSString string];
    while(YES)
    {
        NSRange range = [result rangeOfString: oldString options: 0 range: searchRange];
        if (range.length)
        {
            [result replaceCharactersInRange: range withString: newString];
            range.location+=[newString length];//no recursive change
            range.length=searchRange.length=[result length];
            searchRange.location=0;
            searchRange=NSIntersectionRange(range, searchRange);
        }
        else
            break;
    }
    return [result autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= commentedStringForKey:value:
- (NSString *) commentedStringForKey: (NSString *) aKey value: (NSString *) aValue;
/*"Description forthcoming. aKey should absolutely not begin nor end with a blank character. Not consistency test is done.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * termination = [iTMEncodingController terminationStringForLineEndings:
                                                        [iTMEncodingController lineEndingsUsedInString: self]];
    return [NSString stringWithFormat: @"% !iTeXMac(%@): %@%@", aKey, aValue, (termination? termination: [NSString string])];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= stringForCommentedKey:forRange:effectiveRange:inHeader:
- (NSString *) stringForCommentedKey: (NSString *) aKey forRange: (NSRange) aRange effectiveRange: (NSRangePointer) aRangePtr inHeader: (BOOL) aFlag;
/*"Scans the string for "% !iTeXMac(key): value" pairs. If the search takes place only in the header, aRange is the whole string and the search will stop as soon as an uncommented line is scanned. In the other case, the search is performed on the smallest set of lines containing aRange. If aRangePtr is not nil, it will hold on return the range of the whole line corresponding to aKey, including the termination characters if any.
Scanning the key is not case sensitive while scanning the (on return) value is case sensitive.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSRange r =  NSMakeRange(0, [self length]);
    aRange = aFlag? r: NSIntersectionRange(aRange, r);
//    NSLog(@"<%@> stringForCommentedKey: <%@>, %@", self, aKey, NSStringFromRange(aRange));
    if([aKey length])
    {
        unsigned ceiling = NSMaxRange(aRange);
        unsigned start;
        iTMLiteScanner * S = [iTMLiteScanner scannerWithString: self];
        NSCharacterSet * CRLF = [NSCharacterSet characterSetWithCharactersInString: @"\n\r"];
        [self getLineStart: &start end: nil contentsEnd: nil forRange: aRange];
        [S setScanLocation: start];
        while(([S scanLocation] < ceiling) && ![S isAtEnd])
        {
            [S setCharactersToBeSkipped: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
//            NSLog(@"<%@>", [S string]);
            if([S scanString: @"%" intoString: nil])
            {
                unsigned location = [S scanLocation];
                [S setCaseSensitive: NO];
                [S setCharactersToBeSkipped: [NSCharacterSet whitespaceCharacterSet]];
//                NSLog(@"1");
                if([S scanString: @"!" intoString: nil] &&
                    ([S scanString: @"texshop" intoString: nil] || [S scanString: @"itexmac" intoString: nil]) &&
                        [S scanString: @"("//)
                                    intoString: nil] &&
                            [S scanString: aKey intoString: nil] &&
                                [S scanString://(
                                    @")" intoString : nil] &&
                                    [S scanString: @":" intoString : nil])
                {
                    NSString * result;
                    [S setCaseSensitive: YES];
                    if(![S scanUpToCharactersFromSet: CRLF intoString: &result])
                        result = [NSString string];
                    if(aRangePtr)
                    {
                        [S scanCharactersFromSet: CRLF intoString: nil];
                        * aRangePtr = NSMakeRange(location, [S scanLocation] - location);
                    }
                    return [result stringByRemovingTrailingWhites];
                }
            }
            else if(![S isAtEnd] && aFlag)
                break;
            [S setCharactersToBeSkipped: [NSCharacterSet whitespaceCharacterSet]];
            [S scanUpToCharactersFromSet: CRLF intoString: nil];
        }
    }
    if(aRangePtr) * aRangePtr = NSMakeRange(NSNotFound, 0);
    return [NSString string];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= shortestStringByAbbreviatingWithTildeInPath
- (NSString *) shortestStringByAbbreviatingWithTildeInPath;
/*"In between the receiver and the path abbreviated with tilde, return the shortest one.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * S = [self stringByAbbreviatingWithTildeInPath];
    return ([S length]<[self length]? S: self);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= escapedBackslashAtIndex:escaped:
- (BOOL) isBackslashAtIndex: (unsigned) index escaped: (BOOL *) aFlagPtr;
/*" Returns YES if there is a '\' at index index. For example "\\ " is a 3 length string.
For index = 0, 1 and 2, the aFlagPtr* is NO, YES, NO.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(NSLocationInRange(index, NSMakeRange(0, [self length])) &&
                [self characterAtIndex: index] == '\\')
    {
        if(aFlagPtr)
        {
            unsigned level = 0;
            while(index-->0)
                if([self characterAtIndex: index] == '\\')
                    ++level;
                else
                    break;
            * aFlagPtr = (level%2 > 0);
        }
        return YES;
    }
    return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= isTeXCommentAtIndex:
- (BOOL) isTeXCommentAtIndex: (unsigned) index;
/*" Description Forthcoming
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    int anchor;
    [self getLineStart: &anchor end: nil contentsEnd: nil forRange: NSMakeRange(index, 0)];
    while(anchor<index)
    {
        NSRange R = [self rangeOfString: @"%" options: 0 range: NSMakeRange(anchor, index-anchor)];
        if(R.length)
        {
            if(R.location>anchor)
            {
                BOOL escaped;
                if([self isBackslashAtIndex: R.location-1 escaped: &escaped] && !escaped)
                    anchor = NSMaxRange(R);
                else
                    return YES;
            }
            else
                return YES;
        }
        else
            break;
    }
    return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= groupRangeAtIndex:beginDelimiter:endDelimiter:
- (NSRange) groupRangeAtIndex: (unsigned) index beginDelimiter: (unichar) bgroup endDelimiter: (unichar) egroup;
/*"Returns the range of the smallest group in TeX sense, containing index. If index is out of the string range, the classical not found range is returned. If no group is found, returns a 1 length range at location index. Otherwise, the first character in the range is '{' and the last one is '}'. It is implemented TeX friendly.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    int scanLocation =index;
    int maxLocation = [self length];
    int groupLevel = 1;
    // speedy
    typedef unichar (* CharacterAtIndexIMP) (id, SEL, unsigned);
    CharacterAtIndexIMP CAI = (CharacterAtIndexIMP) [self methodForSelector: @selector(characterAtIndex:)];
    #define NextCharacter CAI(self, @selector(characterAtIndex:), scanLocation)
    BOOL escaped = YES;
    if(![self isBackslashAtIndex: scanLocation-1 escaped: &escaped] || escaped)
    {
        unichar uchar = NextCharacter;
        if(uchar == bgroup)
        {
            while (++scanLocation < maxLocation)
            {
                if(escaped)
                    escaped = NO;
                else
                {
                    uchar = NextCharacter;
                    if(uchar == '\\')
                        escaped = YES;
                    else if (uchar == bgroup)
                        ++groupLevel;
                    else if (uchar == egroup)
                    {
                        --groupLevel;
                        if (groupLevel == 0)
                            return NSMakeRange(index, scanLocation - index + 1);
                    }
                }
            }
        }
        else if(uchar == egroup)
        {
            while (scanLocation-- > 0)
            {
                uchar = NextCharacter;
                if (uchar == egroup)
                {
                    if([self isBackslashAtIndex: scanLocation-1 escaped: &escaped])
                    {
                        if(escaped)
                        {
                            scanLocation -= 2;
                            ++groupLevel;                        
                        }
                        else
                            --scanLocation;
                    }
                    else
                        ++groupLevel;                        
                }
                else if (uchar == bgroup)
                {
                    if([self isBackslashAtIndex: scanLocation-1 escaped: &escaped])
                    {
                        if(escaped)
                        {
                            --groupLevel;                        
                            if (groupLevel == 0)
                                return (index < scanLocation)?
                                    NSMakeRange(index, scanLocation - index + 1):
                                        NSMakeRange(NSNotFound, 0);
                            else
                                scanLocation -= 2;
                        }
                        else
                            --scanLocation;
                    }
                    else
                    {
                        --groupLevel;
                        if (groupLevel == 0)
                            return (scanLocation < index)?
                                NSMakeRange(scanLocation, index - scanLocation + 1):
                                    NSMakeRange(NSNotFound, 0);
                    }
                }
            }
        }
    }
    return  NSMakeRange(NSNotFound, 0);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= groupRangeAtIndex:
- (NSRange) groupRangeAtIndex: (unsigned) index;
/*"Returns the range of the smallest group in TeX sense, containing index. If index is out of the string range, the classical not found range is returned. If no group is found, returns a 1 length range at location index. Otherwise, the first character in the range is '{' and the last one is '}'. It is implemented TeX friendly.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [self groupRangeAtIndex: index beginDelimiter: '{' endDelimiter: '}'];
}
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= NSString(iTeXMac)

