/*
//  NSTextView(iTeXMac).m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sat Dec 08 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSException.h>
#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "NSTextView(iTeXMac).h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  NSTextView(iTeXMac)
/*"Description forthcoming."*/
@implementation NSTextView(iTeXMac)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  indentSelectedTextToPrevious:
- (void) indentSelectedTextToPrevious: (id) irrelevant;
/*"Adds at the beginning of the lines of the receiver the same indentation that was on the preceding line.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    NSRange R = [self selectedRange];
    NSString * S = [self string];
    // expanding the range to cover all line beginnings 
    unsigned begin;
    [S getLineStart: &begin end: nil contentsEnd: nil forRange: R];
    R = NSMakeRange(begin, NSMaxRange(R) - begin);
    // finding the indentation:
    if(!begin)
    {// there is no previous line
        return;
    }
    // the beginning of the previous line: (begin>0 !!!)
    [S getLineStart: &begin end: nil contentsEnd: nil forRange: NSMakeRange(begin-1, 0)];
    // finding the indentation:
    {
        NSCharacterSet * blackCharacterSet = [[NSCharacterSet whitespaceCharacterSet] invertedSet];
        unsigned indentLength = [S rangeOfCharacterFromSet: blackCharacterSet
            options: 0 range: NSMakeRange(begin, [S length] - begin)].location - begin;
        if(indentLength)
            [self insertLinePrefix: [S substringWithRange: NSMakeRange(begin, indentLength)] inRange: R];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  alignSelectedTextToPrevious:
- (void) alignSelectedTextToPrevious: (id) irrelevant;
/*"Adds at the beginning of the lines of the receiver the same indentation that was on the preceding line.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    NSRange R = [self selectedRange];
    NSString * S = [self string];
    // expanding the range to cover all line beginnings 
    unsigned begin;
    [S getLineStart: &begin end: nil contentsEnd: nil forRange: R];
    R = NSMakeRange(begin, NSMaxRange(R) - begin);
    // finding the indentation:
    if(!begin)
    {// there is no previous line
        return;
    }
    // the beginning of the previous line: (begin>0 !!!)
    [S getLineStart: &begin end: nil contentsEnd: nil forRange: NSMakeRange(begin-1, 0)];
    // finding the indentation:
    {
        NSCharacterSet * blackCharacterSet = [[NSCharacterSet whitespaceCharacterSet] invertedSet];
        unsigned prefixLength = [S rangeOfCharacterFromSet: blackCharacterSet
            options: NSAnchoredSearch range: NSMakeRange(begin, [S length] - begin)].location - begin;
        if(prefixLength)
        {
            unsigned last = NSMaxRange(R);
            unsigned begin = last;
            NSString * prefix = [[self string]substringWithRange: NSMakeRange(begin, prefixLength)];
            [[self undoManager]beginGrouping];
            while(begin > R.location)
            {
                [S getLineStart: &begin end: nil contentsEnd: nil forRange: NSMakeRange(begin - 1, 0)];
                {
                    unsigned indentLength = [S rangeOfCharacterFromSet: blackCharacterSet
                    options: NSAnchoredSearch range: NSMakeRange(begin, [S length] - begin)].location - begin;
                    if (indentLength)
                    {
                        [self setSelectedRange: R];
                        [self insertText: prefix];
                        last -= indentLength;
                        last += prefixLength;
                    }
                }
            }
            [[self undoManager]endGrouping];
            if(last)
                [S getLineStart: nil end: &last contentsEnd: nil forRange: NSMakeRange(last-1, 0)];
            else if([[self string]length])
                [S getLineStart: nil end: &last contentsEnd: nil forRange: NSMakeRange(last, 0)];    
            if(begin < last)
                [self setSelectedRange: NSMakeRange(begin, last - begin)];
            return;
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  deleteLeadingWhitesOfSelectedText:
- (void) deleteLeadingWhitesOfSelectedText: (id) irrelevant;
/*"Adds at the beginning of the lines of the receiver the same indentation that was on the preceding line.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    NSRange R = [self selectedRange];
    NSString * S = [self string];
    unsigned first, last = NSMaxRange(R), begin, end;
    NSMutableArray * MA = [NSMutableArray arrayWithCapacity: 1];
    BOOL hasChanged = NO;
    NSCharacterSet * blackCharacterSet = [[NSCharacterSet whitespaceCharacterSet] invertedSet];
    [S getLineStart: &first end: nil contentsEnd: nil forRange: NSMakeRange(R.location, 0)];
    begin = first;
    // next loop is processed at least once because first <= R.location < last = NSMaxRange(R)
    while(begin < last)
    {
        [S getLineStart: nil end: &end contentsEnd: nil forRange: NSMakeRange(begin, 0)];
        {
            unsigned location = [S rangeOfCharacterFromSet: blackCharacterSet
                                    options: 0 range: NSMakeRange(begin, [S length] - begin)].location;
            [MA addObject: [S substringWithRange: NSMakeRange(location , end - location)]];
            if(location > begin)
                hasChanged = YES;
        }
        begin = end;
    }
    // at the end of the loop, the last line have been parsed
    last = end;
    if(hasChanged)
    {
        NSString * string = [MA componentsJoinedByString: [NSString string]];
        NSRange replaceRange = NSMakeRange(first, last - first);
        if ([self shouldChangeTextInRange: replaceRange replacementString: string])
        {
            [[self textStorage] beginEditing];
            [[self textStorage] replaceCharactersInRange: replaceRange withString: string];
            [[self textStorage] endEditing];
            [self didChangeText];
            [self setSelectedRange: NSMakeRange(first, [string length])];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  indentSelectedText:
- (void) indentSelectedText: (id) irrelevant;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    [self insertLinePrefix: @"\t" inRange: [self selectedRange]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  commentSelectedText:
- (void) commentSelectedText: (id) irrelevant;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    [self insertLinePrefix: @"%" inRange: [self selectedRange]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  insertLinePrefix:inRange:
- (void) insertLinePrefix: (NSString *) prefix inRange: (NSRange) range;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    NSString * S = [self string];
    unsigned last;
    unsigned prefixLength = [prefix length];
    range = NSIntersectionRange(range, NSMakeRange(0, [[self string]length]));
    [S getLineStart: nil end: &last contentsEnd: nil forRange: range];
    [[self undoManager]beginUndoGrouping];
    {
        unsigned begin = last;
        while(begin>range.location)
        {
            [S getLineStart: &begin end: nil contentsEnd: nil forRange: NSMakeRange(begin-1, 0)];
            if ([self shouldChangeTextInRange: NSMakeRange(begin, 0) replacementString: prefix])
            {
                [[self textStorage] replaceCharactersInRange: NSMakeRange(begin, 0) withString: prefix];
                [self didChangeText];
                last += prefixLength;
            }
        }
        [self setSelectedRange: NSMakeRange(begin, last - begin)];
    }
    [[self undoManager]endUndoGrouping];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  unindentSelectedText:
- (void) unindentSelectedText: (id) irrelevant;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    [self deleteLinePrefix: @"\t" inRange: [self selectedRange]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  uncommentSelectedText:
- (void) uncommentSelectedText: (id) irrelevant;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    [self deleteLinePrefix: @"%" inRange: [self selectedRange]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  deleteLinePrefix:inRange:
- (void) deleteLinePrefix: (NSString *) prefix inRange: (NSRange) range;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    NSString * S = [self string];
    unsigned last;
    unsigned prefixLength = [prefix length];
    if(prefixLength)
    {
        [[self undoManager]beginUndoGrouping];
        range = NSIntersectionRange(range, NSMakeRange(0, [[self string]length]));
        [S getLineStart: nil end: &last contentsEnd: nil forRange: range];
        {
            unsigned begin = last;
            while(begin>range.location)
            {
                [S getLineStart: &begin end: nil contentsEnd: nil forRange: NSMakeRange(begin-1, 0)];
                if([prefix isEqualToString: [S substringWithRange: NSMakeRange(begin, prefixLength)]])
                {
                    NSRange R = NSMakeRange(begin, prefixLength);
                    [self setSelectedRange: R];
                    [self insertText: @""];
                    last -= prefixLength;
                }
            }
            [self setSelectedRange: NSMakeRange(begin, last - begin)];
        }
        [[self undoManager]endUndoGrouping];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= visibleRange
- (NSRange) visibleRange;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSRect rect = [self visibleRect];
    unsigned topLeftIndex = [self characterIndexForPoint:
        [[self window] convertBaseToScreen:
            [self convertPoint: NSMakePoint(NSMinX(rect), NSMinY(rect))
                toView: nil]]];
    unsigned botRightIndex = [self characterIndexForPoint:
        [[self window] convertBaseToScreen:
            [self convertPoint: NSMakePoint(NSMaxX(rect), NSMaxY(rect))
                toView: nil]]];
    if(topLeftIndex>botRightIndex)
        [NSException raise: NSGenericException format: @"-[%@ %@] view not flipped", [self class], NSStringFromSelector(_cmd)];
    return NSMakeRange(topLeftIndex, botRightIndex - topLeftIndex);
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  NSTextView(iTeXMac)

