/*
//  iTMEncodingController.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sat Nov 10 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <CoreFoundation/CFString.h>
#import <Foundation/NSData.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSString.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSEnumerator.h>
#import <Foundation/NSException.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSUndoManager.h>
#import <Foundation/NSBundle.h>
#import <Foundation/NSNull.h>
#import <Foundation/NSScanner.h>
#import <AppKit/NSMenu.h>
#import <AppKit/NSMenuItem.h>
#import <AppKit/NSMenuItemCell.h>
#import <AppKit/NSApplication.h>
#import <AppKit/NSWindow.h>
#import <AppKit/NSWindowController.h>
#import <AppKit/NSTextStorage.h>
#import "NSMenu(iTeXMac).h"
#import "iTMEncodingController.h"
#import "iTMResourcesController.h"
#import "iTMStatusNotification.h"

NSString * const iTMPreferredLineEndingsKey = @"iTMPreferredLineEndings";
NSString * const iTMIsAutoStringEncodingKey = @"iTMIsAutoStringEncoding";
NSString * const iTMPreferredStringEncodingKey = @"iTMPreferredStringEncoding";
NSString * const iTMOpenStringEncodingKey = @"iTMOpenStringEncoding";

NSString * const iTMEncodingHeaderKey = @"charset";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMEncodingController
/*"To control and manage the file encoding and the line endings."*/

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= NSString(iTMEncodingController)
/*"Description forthcoming."*/
@implementation NSString(iTMEncodingController)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  stringByUsingLineEndings:
- (NSString *) stringByUsingLineEndings: (iTMLineEndings) lineEndings;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    NSString * endOfLine = [iTMEncodingController terminationStringForLineEndings: lineEndings];
    NSMutableString * MS = [NSMutableString string];
    NSRange r = NSMakeRange(0, 0);
    unsigned int end = 0, contentsEnd = 0, ceiling = [self length], nextStart = 0;
    unsigned int corrected = 0, total = 0;
    while(end < ceiling)
    {
        [self getLineStart: nil end: &end contentsEnd: &contentsEnd forRange: r];
        if((end>contentsEnd) &&
            ![[self substringWithRange: NSMakeRange(contentsEnd, end - contentsEnd)] isEqualToString: endOfLine])
        {
            [MS appendString: [self substringWithRange: NSMakeRange(nextStart, contentsEnd - nextStart)]];
            [MS appendString: endOfLine];
            nextStart = end;
            ++corrected;
        }
        ++total;
        r.location = end;
    }
    NSLog(@"line endings corrected: %d of %d", corrected, total);
    if(!nextStart) // the original string used the good line endings
        return self;
    else if(nextStart < ceiling) // there is something left to be copied
        [MS appendString: [self substringWithRange: NSMakeRange(nextStart, ceiling - nextStart)]];
    return MS;
}
@end

@implementation iTMEncodingController
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
                [NSNumber numberWithUnsignedInt: iTMUNIXLineEndings], iTMPreferredLineEndingsKey,
                [NSNumber numberWithUnsignedInt: NSMacOSRomanStringEncoding], iTMPreferredStringEncodingKey,
                [NSNumber numberWithUnsignedInt: NSMacOSRomanStringEncoding], iTMOpenStringEncodingKey,
                [NSNumber numberWithBool: YES], iTMIsAutoStringEncodingKey,
                            nil]];
    [[NSNotificationCenter defaultCenter] addObserver: self
        selector: @selector(applicationDidFinishLaunching:)
            name: NSApplicationDidFinishLaunchingNotification
                object: NSApp];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  lineEndingsMenuWithAction:target:
+ (NSMenu *) lineEndingsMenuWithAction: (SEL) anAction target: (id) aTarget;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    NSMenu * menu = [[NSMenu allocWithZone: [NSMenu menuZone]] initWithTitle:
        NSLocalizedStringFromTableInBundle(@"Line Endings", @"Basic",
            [NSBundle bundleForClass: [self class]], "Title of line endings menu")];
    NSMenuItem * MI;
    MI = [menu addItemWithTitle:
            NSLocalizedStringFromTableInBundle(@"Use Mac Line Endings (CR)", @"Basic",
            [NSBundle bundleForClass: [self class]], "Title of line endings menu item")
                    action: anAction keyEquivalent: [NSString string]];
    [MI setTag: iTMMacintoshLineEndings];
    [MI setTarget: aTarget];
    MI = [menu addItemWithTitle:
            NSLocalizedStringFromTableInBundle(@"Use UNIX Line Endings (LF)", @"Basic",
            [NSBundle bundleForClass: [self class]], "Title of line endings menu item")
                    action: anAction keyEquivalent: [NSString string]];
    [MI setTag: iTMUNIXLineEndings];
    [MI setTarget: aTarget];
    MI = [menu addItemWithTitle:
            NSLocalizedStringFromTableInBundle(@"Use Windows Line Endings (CR+LF)", @"Basic",
            [NSBundle bundleForClass: [self class]], "Title of line endings menu item")
                action: anAction keyEquivalent: [NSString string]];
    [MI setTag: iTMWindowsLineEndings];
    [MI setTarget: aTarget];
    MI = [menu addItemWithTitle:
            NSLocalizedStringFromTableInBundle(@"Use Unicode Line Endings", @"Basic",
            [NSBundle bundleForClass: [self class]], "Title of line endings menu item")
                    action: anAction keyEquivalent: [NSString string]];
    [MI setTag: iTMUnicodeLineEndings];
    [MI setTarget: aTarget];
    MI = [menu addItemWithTitle:
            NSLocalizedStringFromTableInBundle(@"Unknown Line Endings Used", @"Basic",
            [NSBundle bundleForClass: [self class]], "Title of line endings menu item")
                    action: anAction keyEquivalent: [NSString string]];
    [MI setTag: iTMUnknownLineEndings];
    [MI setTarget: aTarget];
    return [menu autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  lineEndingsMenu
+ (NSMenu *) lineEndingsMenu;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    return [self lineEndingsMenuWithAction: @selector(useLineEndings:) target: self];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  preferredLineEndingsMenu
+ (NSMenu *) preferredLineEndingsMenu;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    return [self lineEndingsMenuWithAction: @selector(preferLineEndings:) target: self];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  stringEncodingsMenuWithAction:target:
+ (NSMenu *) stringEncodingsMenuWithAction: (SEL) anAction target: (id) aTarget;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    NSMenu * menu = [[NSMenu allocWithZone: [NSMenu menuZone]] initWithTitle:
        NSLocalizedStringFromTableInBundle(@"Encodings", @"Basic",
            [NSBundle bundleForClass: [self class]], "Title of stringEncodings menu, unused")];
    NSArray * stringEncodings = [self supportedEncodings];
    NSEnumerator * enumerator = [stringEncodings objectEnumerator];
    NSNumber * number;
    BOOL canInsertSeparatorItem = NO;
    while(number = [enumerator nextObject])
    {
        if([number respondsToSelector: @selector(intValue)])
        {
            NSStringEncoding encoding = [number intValue];
            NSString * title = [NSString localizedNameOfStringEncoding: encoding];
            if(![title length])
                title = [NSString stringWithFormat: @"Encoding: %d", encoding];
            {
                NSMenuItem * MI = [menu addItemWithTitle: title action: anAction keyEquivalent: [NSString string]];
                [MI setTag: encoding];
                [MI setTarget: aTarget];
                canInsertSeparatorItem = YES;
    //            NSLog(@"encoding %@ for %u", [MI title], [MI tag]);
            }
        }
        else if(canInsertSeparatorItem)
        {
            [menu addItem: [NSMenuItem separatorItem]];
            canInsertSeparatorItem = NO;
        }
    }
    return [menu autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMUse()
#define iTMUse(selecteur, action) + (void) selecteur: (id <NSMenuItem>) sender;\
{\
    [[NSApp targetForAction: @selector(action:)] action: [sender tag]];\
    return;\
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  preferOpenStringEncoding:
+ (void) preferStringEncoding: (id <NSMenuItem>) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    [self setPreferredStringEncoding: [sender tag]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  preferredStringEncoding:
+ (int) preferredStringEncoding;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    return [[[NSUserDefaults standardUserDefaults] objectForKey: iTMPreferredStringEncodingKey] intValue];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setPreferredStringEncoding:
+ (void) setPreferredStringEncoding: (int) argument;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    [[NSUserDefaults standardUserDefaults] setObject: [NSNumber numberWithUnsignedInt: argument]
            forKey: iTMPreferredStringEncodingKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  useStringEncoding:
+ (void) useStringEncoding: (id <NSMenuItem>) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    [[NSApp targetForAction: @selector(setStringEncoding:)] setStringEncoding: [sender tag]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= useLineEndings:
+ (void) useLineEndings: (id <NSMenuItem>) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    [[NSApp targetForAction: @selector(setLineEndingsUsed:)] setLineEndingsUsed: [sender tag]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= preferLineEndings:
+ (void) preferLineEndings: (id <NSMenuItem>) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    [[NSUserDefaults standardUserDefaults] setObject: [NSNumber numberWithUnsignedInt: [sender tag]]
            forKey: iTMPreferredLineEndingsKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  supportedEncodings
+ (NSArray *) supportedEncodings;
/*"Encodings.plist file contains an array of integers.
Encodings constants in the Encodings.plist are meant in core foundation sense, not in yellow box sense.
However, they are in yellow box sense in the returned array.
If the file ~/Library/Application Support/iTeXMac/General/Encoding.plist does not exist,
such a file is seeked for in the main bundle. If nothing has been found, a default array is returned.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001)
To do list: External stringEncodings are not supported now.
"*/
{
    #if 1
    NSString * path = [[[iTMResourcesController sharedResourcesController] generalDirectoryPath]
                                stringByAppendingPathComponent: @"Encodings.plist"];
    NSArray * encodingStringsFromFile = [NSArray arrayWithContentsOfFile: path];
    if (!encodingStringsFromFile)
    {
        path = [[NSBundle mainBundle] pathForResource: @"Encodings" ofType: @"plist"];
        encodingStringsFromFile = [NSArray arrayWithContentsOfFile: path];
    }
    else
        NSLog(@"User defined encodings list");
    if(encodingStringsFromFile)
    {
        NSMutableArray * result = [NSMutableArray array];
        NSEnumerator * enumerator = [encodingStringsFromFile objectEnumerator];
        id string;
        BOOL canInsertNull = NO;
        while(string = [enumerator nextObject])
            if([string isKindOfClass: [NSString class]])
            {
                if([string length])
                {
                    int encoding = [self coreFoundationStringEncodingFromString: string];
                    if(encoding == kCFStringEncodingInvalidId)
                        encoding = CFStringConvertEncodingToNSStringEncoding([string intValue]);
                    else
                        encoding = CFStringConvertEncodingToNSStringEncoding(encoding);
                    [result addObject: [NSNumber numberWithInt: encoding]];
                    canInsertNull = YES;
                }
                else if(canInsertNull)
                {
                    [result addObject: [NSNull null]];
                    canInsertNull = NO;
                }
            }
        if([result count])
            return result;
    }
    else
        NSLog(@"iTMEncodingController: Bad file format at\n<%@>", path);
    #endif
    return [NSArray arrayWithObjects:
                    [NSNumber numberWithUnsignedInt: NSMacOSRomanStringEncoding],
                    [NSNumber numberWithUnsignedInt: NSWindowsCP1252StringEncoding],
                    [NSNumber numberWithUnsignedInt: NSISOLatin1StringEncoding],
                    [NSNumber numberWithUnsignedInt:
                            CFStringConvertEncodingToNSStringEncoding(kCFStringEncodingMacCentralEurRoman)],
                    [NSNumber numberWithUnsignedInt:
                            CFStringConvertEncodingToNSStringEncoding(kCFStringEncodingWindowsLatin2)],
                    [NSNumber numberWithUnsignedInt:
                            CFStringConvertEncodingToNSStringEncoding(kCFStringEncodingISOLatin2)],
                    [NSNumber numberWithUnsignedInt: NSNEXTSTEPStringEncoding],
                    [NSNumber numberWithUnsignedInt: NSASCIIStringEncoding],
                    [NSNull null],//stands for separatorItem
                    [NSNumber numberWithUnsignedInt: NSUnicodeStringEncoding],
                    [NSNumber numberWithUnsignedInt: NSUTF8StringEncoding],
                    [NSNumber numberWithUnsignedInt: NSNonLossyASCIIStringEncoding],
                                nil];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= applicationDidFinishLaunching:
+ (void) applicationDidFinishLaunching: (NSNotification *) aNotification;
/*"Installs menus.
Version History: jerome.laurens@u-bourgogne.fr 
- 1: 09/02/2001
To Do List:
"*/
{
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
        NSLocalizedStringFromTableInBundle(@"Loading String Encoding Menus.", @"Basic",
            [NSBundle bundleForClass: [self class]], "")];
    [[NSNotificationCenter defaultCenter] removeObserver: self name: [aNotification name] object: nil];
    {
        NSMenu * MM = [NSApp mainMenu];
        [[MM deepItemWithAction: @selector(lineEndingsMenuItem:)]
                    setSubmenu: [self lineEndingsMenuWithAction: @selector(useLineEndings:) target: self]];
        [[MM deepItemWithAction: @selector(preferredLineEndingsMenuItem:)]
                    setSubmenu: [self lineEndingsMenuWithAction: @selector(preferLineEndings:) target: self]];
        [[MM deepItemWithAction: @selector(stringEncodingCurrentMenuItem:)]
                    setSubmenu: [self stringEncodingsMenuWithAction: @selector(useStringEncoding:) target: self]];
        [[MM deepItemWithAction: @selector(stringEncodingPreferredMenuItem:)]
                    setSubmenu: [self stringEncodingsMenuWithAction: @selector(preferStringEncoding:) target: self]];
        [[MM deepItemWithAction: @selector(stringEncodingRevertMenuItem:)] setSubmenu:
            [self stringEncodingsMenuWithAction: @selector(revertDocumentToSavedWithStringEncoding:) target: nil]];
        [[MM deepItemWithAction: @selector(toggleAutomaticStringEncoding:)] setTarget: self];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= toggleAutoStringEncoding:
+ (void) toggleAutomaticStringEncoding: (id <NSMenuItem>) sender;
/*"Description forthcoming.
Proposed by jerome.laurens@u-bourgogne.fr (07/12/2001)
- < 1.1: 03/10/2002
To Do List: Nothing at first glance
"*/
{
    NSUserDefaults * SUD = [NSUserDefaults standardUserDefaults];
    BOOL old = [SUD boolForKey: iTMIsAutoStringEncodingKey];
    [SUD setBool: !old forKey: iTMIsAutoStringEncodingKey];
    [sender setState: ([SUD boolForKey: iTMIsAutoStringEncodingKey]? NSOnState: NSOffState)];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMenuItem:
+ (BOOL) validateMenuItem: (id <NSMenuItem>) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    SEL action = [sender action];
    if(action == @selector(useLineEndings:))
    {
        int tag = [sender tag];
        id object = [NSApp targetForAction: @selector(lineEndingsUsed)];
        [sender setState: (tag == [object lineEndingsUsed]? NSOnState: NSOffState)];
        if(!object) return NO;
        switch(tag)
        {
            case iTMUNIXLineEndings: 
            case iTMMacintoshLineEndings: 
            case iTMWindowsLineEndings:
                return YES;
            default:
                return NO;
        }
    }
    else if(action == @selector(preferLineEndings:))
    {
        int tag = [sender tag];
        [sender setState: (tag == [[NSUserDefaults standardUserDefaults] integerForKey: iTMPreferredLineEndingsKey]?
                    NSOnState: NSOffState)];
        switch(tag)
        {
            case iTMUNIXLineEndings: 
            case iTMMacintoshLineEndings: 
            case iTMWindowsLineEndings:
                return YES;
            default:
                return NO;
        }
    }
    else if(action == @selector(useStringEncoding:))
    {
        id object = [NSApp targetForAction: @selector(stringEncoding)];
        [sender setState: ([sender tag] == [object stringEncoding]? NSOnState: NSOffState)];
        return [object respondsToSelector: @selector(canSetStringEncoding)] && [object canSetStringEncoding];
    }
    else if(action == @selector(preferStringEncoding:))
    {
        [sender setState: ([sender tag] == [[NSUserDefaults standardUserDefaults]
                    integerForKey: iTMPreferredStringEncodingKey]? NSOnState: NSOffState)];
        return YES;
    }
    else if(action == @selector(toggleAutomaticStringEncoding:))
    {
        NSUserDefaults * SUD = [NSUserDefaults standardUserDefaults];
        [sender setState: ([SUD boolForKey: iTMIsAutoStringEncodingKey]? NSOnState: NSOffState)];
        return YES;
    }
    else
        return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= lineEndingsUsedInString:
+ (iTMLineEndings) lineEndingsUsedInString: (NSString *) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([argument length])
    {
        unsigned contentsEnd, end;
        [argument getLineStart: nil end: &end contentsEnd: &contentsEnd forRange: NSMakeRange(0, 0)];
        if(end > contentsEnd)
        {
            NSString * endOfLine = [argument substringWithRange: NSMakeRange(contentsEnd, end - contentsEnd)];
            if([endOfLine isEqualToString: @"\r\n"])
                return iTMWindowsLineEndings;
            else if([endOfLine isEqualToString: @"\r"])
                return iTMMacintoshLineEndings;
            else if([endOfLine isEqualToString: @"\n"])
                return iTMUNIXLineEndings;
            else if([endOfLine length] == 1)
            {
                unichar theChar = [endOfLine characterAtIndex: 0];
                if((theChar == 0x2028) || (theChar == 0x2029))
                    return iTMUnicodeLineEndings;
            }
            else
                return iTMUnknownLineEndings;
        }
    }
    return [[NSUserDefaults standardUserDefaults] integerForKey: iTMPreferredLineEndingsKey];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= lineEndingsUsedInDocument:
+ (iTMLineEndings) lineEndingsUsedInDocument: (id) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([argument respondsToSelector: @selector(textStorage)])
        return [self lineEndingsUsedInString: [[argument textStorage] mutableString]];
    else
        return iTMUnknownLineEndings;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= terminationStringForLineEndings:
+ (NSString *) terminationStringForLineEndings: (iTMLineEndings) lineEndings;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!lineEndings)
        lineEndings = [[NSUserDefaults standardUserDefaults] integerForKey: iTMPreferredLineEndingsKey];
    switch(lineEndings)
    {
        case iTMMacintoshLineEndings:	return @"\r";
        case iTMWindowsLineEndings:	return @"\r\n";
        case iTMUNIXLineEndings:
        default: 			return @"\n";
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= coreFoundationStringEncodingFromString:
+ (CFStringEncoding) coreFoundationStringEncodingFromString: (NSString *) argument;
/*"Yellow box string encoding returned. Core Foundation string encodings constants used.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSScanner * s = [NSScanner scannerWithString: argument];
    CFStringEncoding result = kCFStringEncodingInvalidId;
    [s setCaseSensitive: NO];
    // a Core Foundation number
    if([s scanInt: (int*)&result])
    {
        if(!CFStringIsEncodingAvailable(result))
            result = kCFStringEncodingInvalidId;
    }
    else if([s scanString: @"non" intoString: nil])
    {
        if([s scanString: @"lossy" intoString: nil] && [s scanString: @"ascii" intoString: nil])
        {
            result = kCFStringEncodingNonLossyASCII;
        }
    }
    else if([s scanString: @"nextstep" intoString: nil] && [s scanString: @"latin" intoString: nil])
    {
        result = kCFStringEncodingNextStepLatin;
    }
    else if([s scanString: @"ascii" intoString: nil])
    {
        result = kCFStringEncodingASCII;
    }
    else if([s scanString: @"unicode" intoString: nil])
    {
        result = kCFStringEncodingUnicode;
    }
    else if([s scanString: @"utf" intoString: nil] &&
        ([s scanString: @"-" intoString: nil], [s scanString: @"8" intoString: nil]))
    {
        result = kCFStringEncodingUTF8;
    }
    else if([s scanString: @"mac" intoString: nil])
    {
        if([s scanString: @"roman" intoString: nil])
            result = kCFStringEncodingMacRoman;
        else if([s scanString: @"japanese" intoString: nil])
            result = kCFStringEncodingMacJapanese;
        else if([s scanString: @"korean" intoString: nil])
            result = kCFStringEncodingMacKorean;
        else if([s scanString: @"arabic" intoString: nil])
            result = kCFStringEncodingMacArabic;
        else if([s scanString: @"hebrew" intoString: nil])
            result = kCFStringEncodingMacHebrew;
        else if([s scanString: @"greek" intoString: nil])
            result = kCFStringEncodingMacGreek;
        else if([s scanString: @"cyrillic" intoString: nil])
            result = kCFStringEncodingMacCyrillic;
        else if([s scanString: @"devanagari" intoString: nil])
            result = kCFStringEncodingMacDevanagari;
        else if([s scanString: @"gurmukhi" intoString: nil])
            result = kCFStringEncodingMacGurmukhi;
        else if([s scanString: @"gujarati" intoString: nil])
            result = kCFStringEncodingMacGujarati;
        else if([s scanString: @"oriya" intoString: nil])
            result = kCFStringEncodingMacOriya;
        else if([s scanString: @"bengali" intoString: nil])
            result = kCFStringEncodingMacBengali;
        else if([s scanString: @"tamil" intoString: nil])
            result = kCFStringEncodingMacTamil;
        else if([s scanString: @"telugu" intoString: nil])
            result = kCFStringEncodingMacTelugu;
        else if([s scanString: @"kannada" intoString: nil])
            result = kCFStringEncodingMacKannada;
        else if([s scanString: @"malayalam" intoString: nil])
            result = kCFStringEncodingMacMalayalam;
        else if([s scanString: @"sinhalese" intoString: nil])
            result = kCFStringEncodingMacSinhalese;
        else if([s scanString: @"burmese" intoString: nil])
            result = kCFStringEncodingMacBurmese;
        else if([s scanString: @"khmer" intoString: nil])
            result = kCFStringEncodingMacKhmer;
        else if([s scanString: @"thai" intoString: nil])
            result = kCFStringEncodingMacThai;
        else if([s scanString: @"laotian" intoString: nil])
            result = kCFStringEncodingMacLaotian;
        else if([s scanString: @"georgian" intoString: nil])
            result = kCFStringEncodingMacGeorgian;
        else if([s scanString: @"armenian" intoString: nil])
            result = kCFStringEncodingMacArmenian;
        else if([s scanString: @"chinese" intoString: nil] && [s scanString: @"traditional" intoString: nil])
            result = kCFStringEncodingMacChineseTrad;
        else if([s scanString: @"chinese" intoString: nil] && [s scanString: @"simplified" intoString: nil])
            result = kCFStringEncodingMacChineseSimp;
        else if([s scanString: @"tibetan" intoString: nil])
            result = kCFStringEncodingMacTibetan;
        else if([s scanString: @"mongolian" intoString: nil])
            result = kCFStringEncodingMacMongolian;
        else if([s scanString: @"ethiopic" intoString: nil])
            result = kCFStringEncodingMacEthiopic;
        else if([s scanString: @"central" intoString: nil] &&	
                    [s scanString: @"european" intoString: nil] &&
                        [s scanString: @"roman" intoString: nil])
            result = kCFStringEncodingMacCentralEurRoman;
        else if([s scanString: @"vietnamese" intoString: nil])
            result = kCFStringEncodingMacVietnamese;
        else if([s scanString: @"extended" intoString: nil] && [s scanString: @"arabic" intoString: nil])
            result = kCFStringEncodingMacExtArabic;
        else if([s scanString: @"symbol" intoString: nil])
            result = kCFStringEncodingMacSymbol;
        else if([s scanString: @"dingbats" intoString: nil])
            result = kCFStringEncodingMacDingbats;
        else if([s scanString: @"turkish" intoString: nil])
            result = kCFStringEncodingMacTurkish;
        else if([s scanString: @"croatian" intoString: nil])
            result = kCFStringEncodingMacCroatian;
        else if([s scanString: @"icelandic" intoString: nil])
            result = kCFStringEncodingMacIcelandic;
        else if([s scanString: @"romanian" intoString: nil])
            result = kCFStringEncodingMacRomanian;
        else if([s scanString: @"celtic" intoString: nil])
            result = kCFStringEncodingMacCeltic;
        else if([s scanString: @"gaelic" intoString: nil])
            result = kCFStringEncodingMacGaelic;
        else if([s scanString: @"farsi" intoString: nil])
            result = kCFStringEncodingMacFarsi;
        else if([s scanString: @"ukrainian" intoString: nil])
            result = kCFStringEncodingMacUkrainian;
        else if([s scanString: @"inuit" intoString: nil])
            result = kCFStringEncodingMacInuit;
        else if([s scanString: @"vt100" intoString: nil])
            result = kCFStringEncodingMacVT100;
    }
    else if([s scanString: @"iso" intoString: nil])
    {
        [s setCharactersToBeSkipped: [NSCharacterSet characterSetWithCharactersInString: @" -\t"]];
        if([s scanString: @"8859" intoString: nil])
        {
            CFStringEncoding CFSE;
            [s scanInt: (int*)&CFSE];
            switch(CFSE)
            {
                case  1: result = kCFStringEncodingISOLatin1;		break;
                case  2: result = kCFStringEncodingISOLatin2;		break;
                case  3: result = kCFStringEncodingISOLatin3;		break;
                case  4: result = kCFStringEncodingISOLatin4;		break;
                case  5: result = kCFStringEncodingISOLatinCyrillic;	break;
                case  6: result = kCFStringEncodingISOLatinArabic;	break;
                case  7: result = kCFStringEncodingISOLatinGreek;	break;
                case  8: result = kCFStringEncodingISOLatinHebrew;	break;
                case  9: result = kCFStringEncodingISOLatin5;		break;
                case 10: result = kCFStringEncodingISOLatin6;		break;
                case 11: result = kCFStringEncodingISOLatinThai;	break;
//                case 12: result = ?;
                case 13: result = kCFStringEncodingISOLatin7;		break;
                case 14: result = kCFStringEncodingISOLatin8;		break;
                case 15: result = kCFStringEncodingISOLatin9;		break;
            }
        }
        else if([s scanString: @"latin" intoString: nil])
        {
            CFStringEncoding CFSE;
            [s setCharactersToBeSkipped: [NSCharacterSet characterSetWithCharactersInString: @" -\t"]];
            if([s scanInt: (int*)&CFSE])
                switch(CFSE)
                {
                    case 1: result = kCFStringEncodingISOLatin1;	break;
                    case 2: result = kCFStringEncodingISOLatin2;	break;
                    case 3: result = kCFStringEncodingISOLatin3;	break;
                    case 4: result = kCFStringEncodingISOLatin4;	break;
                    case 5: result = kCFStringEncodingISOLatin5;	break;
                    case 6: result = kCFStringEncodingISOLatin6;	break;
                    case 7: result = kCFStringEncodingISOLatin7;	break;
                    case 8: result = kCFStringEncodingISOLatin8;	break;
                    case 9: result = kCFStringEncodingISOLatin9;	break;
                }
            else if([s scanString: @"cyrillic" intoString: nil])
                result = kCFStringEncodingISOLatinCyrillic;
            else if([s scanString: @"arabic" intoString: nil])
                result = kCFStringEncodingISOLatinArabic;
            else if([s scanString: @"greek" intoString: nil])
                result = kCFStringEncodingISOLatinGreek;
            else if([s scanString: @"hebrew" intoString: nil])
                result = kCFStringEncodingISOLatinHebrew;
            else if([s scanString: @"thai" intoString: nil])
                result = kCFStringEncodingISOLatinThai;
        }
    }
    else if([s scanString: @"dos" intoString: nil])
    {
        [s setCharactersToBeSkipped: [NSCharacterSet characterSetWithCharactersInString: @" -\t"]];
        if([s scanString: @"latin" intoString: nil] && [s scanString: @"us" intoString: nil])
            result = kCFStringEncodingDOSLatinUS;
        else if([s scanString: @"greek" intoString: nil])
            result = kCFStringEncodingDOSGreek;
        else if([s scanString: @"baltic" intoString: nil] && [s scanString: @"rim" intoString: nil])
            result = kCFStringEncodingDOSBalticRim;
        else if([s scanString: @"cyrillic" intoString: nil])
            result = kCFStringEncodingDOSCyrillic;
        else if([s scanString: @"turkish" intoString: nil])
            result = kCFStringEncodingDOSTurkish;
        else if([s scanString: @"potuguese" intoString: nil])
            result = kCFStringEncodingDOSPortuguese;
        else if([s scanString: @"icelandic" intoString: nil])
            result = kCFStringEncodingDOSIcelandic;
        else if([s scanString: @"hebrew" intoString: nil])
            result = kCFStringEncodingDOSHebrew;
        else if([s scanString: @"canadian" intoString: nil] && [s scanString: @"french" intoString: nil])
            result = kCFStringEncodingDOSCanadianFrench;
        else if([s scanString: @"arabic" intoString: nil])
            result = kCFStringEncodingDOSArabic;
        else if([s scanString: @"nordic" intoString: nil])
            result = kCFStringEncodingDOSNordic;
        else if([s scanString: @"russian" intoString: nil])
            result = kCFStringEncodingDOSRussian;
        else if([s scanString: @"thai" intoString: nil])
            result = kCFStringEncodingDOSThai;
        else if([s scanString: @"japanese" intoString: nil])
            result = kCFStringEncodingDOSJapanese;
        else if([s scanString: @"chinese" intoString: nil] && [s scanString: @"simplified" intoString: nil])
            result = kCFStringEncodingDOSChineseSimplif;
        else if([s scanString: @"korean" intoString: nil])
            result = kCFStringEncodingDOSKorean;
        else if([s scanString: @"chinese" intoString: nil] && [s scanString: @"traditional" intoString: nil])
            result = kCFStringEncodingDOSChineseTrad;
        else if([s scanString: @"latin" intoString: nil])
        {
            CFStringEncoding CFSE;
            [s scanInt: (int*)&CFSE];
            switch(CFSE)
            {
                case 1: result = kCFStringEncodingDOSLatin1; break;
                case 2: result = kCFStringEncodingDOSLatin2; break;
            }
        }
        else if([s scanString: @"greek" intoString: nil])
        {
            CFStringEncoding CFSE;
            [s scanInt: (int*)&CFSE];
            switch(CFSE)
            {
                case 1: result = kCFStringEncodingDOSGreek1; break;
                case 2: result = kCFStringEncodingDOSGreek2; break;
            }
        }
    }
    else if([s scanString: @"windows" intoString: nil])
    {
        [s setCharactersToBeSkipped: [NSCharacterSet characterSetWithCharactersInString: @" -\t"]];
        if([s scanString: @"nt" intoString: nil])
        {
            if([s scanString: @"korean" intoString: nil] && [s scanString: @"johab" intoString: nil])
                result = kCFStringEncodingWindowsKoreanJohab;
        }
        else if([s scanString: @"baltic" intoString: nil] && [s scanString: @"rim" intoString: nil])
            result = kCFStringEncodingWindowsBalticRim;
        else if([s scanString: @"thai" intoString: nil])
            result = kCFStringEncodingDOSThai;
        else if([s scanString: @"japanese" intoString: nil])
            result = kCFStringEncodingDOSJapanese;
        else if([s scanString: @"chinese" intoString: nil] && [s scanString: @"simplified" intoString: nil])
            result = kCFStringEncodingDOSChineseSimplif;
        else if([s scanString: @"chinese" intoString: nil] && [s scanString: @"traditional" intoString: nil])
            result = kCFStringEncodingDOSChineseTrad;
        else if([s scanString: @"korean" intoString: nil])
            result = kCFStringEncodingDOSKorean;
        else if([s scanString: @"cyrillic" intoString: nil])
            result = kCFStringEncodingWindowsCyrillic;
        else if([s scanString: @"greek" intoString: nil])
            result = kCFStringEncodingWindowsGreek;
        else if([s scanString: @"hebrew" intoString: nil])
            result = kCFStringEncodingWindowsHebrew;
        else if([s scanString: @"arabic" intoString: nil])
            result = kCFStringEncodingWindowsArabic;
        else if([s scanString: @"vietnamese" intoString: nil])
            result = kCFStringEncodingWindowsVietnamese;
        else if([s scanString: @"latin" intoString: nil])
        {
            CFStringEncoding CFSE;
            [s scanInt: (int*)&CFSE];
            switch(CFSE)
            {
                case 1: result = kCFStringEncodingWindowsLatin1; break;
                case 2: result = kCFStringEncodingWindowsLatin2; break;
                case 5: result = kCFStringEncodingWindowsLatin5; break;
            }
        }
    }
    else if(([s scanString: @"code" intoString: nil] && [s scanString: @"page" intoString: nil]) ||
                        ([s scanString: @"cp" intoString: nil]))
    {
        CFStringEncoding CFSE;
        [s setCharactersToBeSkipped: [NSCharacterSet characterSetWithCharactersInString: @" -\t"]];
        [s scanInt: (int*)&CFSE];
        switch(CFSE)
        {
            case   37: result = kCFStringEncodingEBCDIC_CP037;		break;
            case  437: result = kCFStringEncodingDOSLatinUS;		break;
            case  737: result = kCFStringEncodingDOSGreek;		break;
            case  775: result = kCFStringEncodingDOSBalticRim;		break;
            case  850: result = kCFStringEncodingDOSLatin1;		break;
            case  851: result = kCFStringEncodingDOSGreek1;		break;
            case  852: result = kCFStringEncodingDOSLatin2;		break;
            case  855: result = kCFStringEncodingDOSCyrillic;		break;
            case  857: result = kCFStringEncodingDOSTurkish;		break;
            case  860: result = kCFStringEncodingDOSPortuguese;		break;
            case  861: result = kCFStringEncodingDOSIcelandic;		break;
            case  862: result = kCFStringEncodingDOSHebrew;		break;
            case  863: result = kCFStringEncodingDOSCanadianFrench;	break;
            case  864: result = kCFStringEncodingDOSArabic;		break;
            case  865: result = kCFStringEncodingDOSNordic;		break;
            case  866: result = kCFStringEncodingDOSRussian;		break;
            case  869: result = kCFStringEncodingDOSGreek2;		break;
            case  874: result = kCFStringEncodingDOSThai;		break;
            case  932: result = kCFStringEncodingDOSJapanese;		break;
            case  936: result = kCFStringEncodingDOSChineseSimplif;	break;
            case  949: result = kCFStringEncodingDOSKorean;		break;
            case  950: result = kCFStringEncodingDOSChineseTrad;	break;
            case 1250: result = kCFStringEncodingWindowsLatin2;		break;
            case 1251: result = kCFStringEncodingWindowsCyrillic;	break;
            case 1252: result = kCFStringEncodingWindowsLatin1;		break;
            case 1253: result = kCFStringEncodingWindowsGreek;		break;
            case 1254: result = kCFStringEncodingWindowsLatin5;		break;
            case 1255: result = kCFStringEncodingWindowsHebrew;		break;
            case 1256: result = kCFStringEncodingWindowsArabic;		break;
            case 1257: result = kCFStringEncodingWindowsBalticRim;	break;
            case 1361: result = kCFStringEncodingWindowsKoreanJohab;	break;
            case 1258: result = kCFStringEncodingWindowsVietnamese;	break;
        }
    }
    if(result == kCFStringEncodingInvalidId)
        NSLog(@"Unrecognized encoding string name: <%@>", argument);
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= stringEncodingFromString:
+ (NSNumber *) stringEncodingFromString: (NSString *) argument;
/*"Yellow box string encoding returned. Core Foundation string encodings constants used.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    CFStringEncoding result = [self coreFoundationStringEncodingFromString: argument];
    if(result != kCFStringEncodingInvalidId)
    {
        result = CFStringConvertEncodingToNSStringEncoding(result);
        return [NSNumber numberWithUnsignedInt: result];
    }
    else
        return nil;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMEncodingController

#if 0
typedef enum {
    kCFStringEncodingInvalidId = 0xffffffffU,
    kCFStringEncodingMacRoman = 0,
    kCFStringEncodingWindowsLatin1 = 0x0500, /* ANSI codepage 1252 */
    kCFStringEncodingISOLatin1 = 0x0201, /* ISO 8859-1 */
    kCFStringEncodingNextStepLatin = 0x0B01, /* NextStep encoding*/
    kCFStringEncodingASCII = 0x0600, /* 0..127 (in creating CFString, values greater than 0x7F are treated as corresponding Unicode value) */
    kCFStringEncodingUnicode = 0x0100, /* kTextEncodingUnicodeDefault  + kTextEncodingDefaultFormat (aka kUnicode16BitFormat) */
    kCFStringEncodingUTF8 = 0x08000100, /* kTextEncodingUnicodeDefault + kUnicodeUTF8Format */
    kCFStringEncodingNonLossyASCII = 0x0BFF /* 7bit Unicode variants used by YellowBox & Java */
} CFStringBuiltInEncodings;
typedef enum {
/*  kCFStringEncodingMacRoman = 0L, defined in CoreFoundation/CFString.h */
    kCFStringEncodingMacJapanese = 1,
    kCFStringEncodingMacChineseTrad = 2,
    kCFStringEncodingMacKorean = 3,
    kCFStringEncodingMacArabic = 4,
    kCFStringEncodingMacHebrew = 5,
    kCFStringEncodingMacGreek = 6,
    kCFStringEncodingMacCyrillic = 7,
    kCFStringEncodingMacDevanagari = 9,
    kCFStringEncodingMacGurmukhi = 10,
    kCFStringEncodingMacGujarati = 11,
    kCFStringEncodingMacOriya = 12,
    kCFStringEncodingMacBengali = 13,
    kCFStringEncodingMacTamil = 14,
    kCFStringEncodingMacTelugu = 15,
    kCFStringEncodingMacKannada = 16,
    kCFStringEncodingMacMalayalam = 17,
    kCFStringEncodingMacSinhalese = 18,
    kCFStringEncodingMacBurmese = 19,
    kCFStringEncodingMacKhmer = 20,
    kCFStringEncodingMacThai = 21,
    kCFStringEncodingMacLaotian = 22,
    kCFStringEncodingMacGeorgian = 23,
    kCFStringEncodingMacArmenian = 24,
    kCFStringEncodingMacChineseSimp = 25,
    kCFStringEncodingMacTibetan = 26,
    kCFStringEncodingMacMongolian = 27,
    kCFStringEncodingMacEthiopic = 28,
    kCFStringEncodingMacCentralEurRoman = 29,
    kCFStringEncodingMacVietnamese = 30,
    kCFStringEncodingMacExtArabic = 31,
    /* The following use script code 0, smRoman */
    kCFStringEncodingMacSymbol = 33,
    kCFStringEncodingMacDingbats = 34,
    kCFStringEncodingMacTurkish = 35,
    kCFStringEncodingMacCroatian = 36,
    kCFStringEncodingMacIcelandic = 37,
    kCFStringEncodingMacRomanian = 38,
    kCFStringEncodingMacCeltic = 39,
    kCFStringEncodingMacGaelic = 40,
    /* The following use script code 4, smArabic */
    kCFStringEncodingMacFarsi = 0x8C,	/* Like MacArabic but uses Farsi digits */
    /* The following use script code 7, smCyrillic */
    kCFStringEncodingMacUkrainian = 0x98,
    /* The following use script code 32, smUnimplemented */
    kCFStringEncodingMacInuit = 0xEC,
    kCFStringEncodingMacVT100 = 0xFC,	/* VT100/102 font from Comm Toolbox: Latin-1 repertoire + box drawing etc */
    /* Special Mac OS encodings*/
    kCFStringEncodingMacHFS = 0xFF,	/* Meta-value, should never appear in a table */

    /* Unicode & ISO UCS encodings begin at 0x100 */
    /* We don't use Unicode variations defined in TextEncoding; use the ones in CFString.h, instead. */

    /* ISO 8-bit and 7-bit encodings begin at 0x200 */
/*  kCFStringEncodingISOLatin1 = 0x0201, defined in CoreFoundation/CFString.h */
    kCFStringEncodingISOLatin2 = 0x0202,	/* ISO 8859-2 */
    kCFStringEncodingISOLatin3 = 0x0203,	/* ISO 8859-3 */
    kCFStringEncodingISOLatin4 = 0x0204,	/* ISO 8859-4 */
    kCFStringEncodingISOLatinCyrillic = 0x0205,	/* ISO 8859-5 */
    kCFStringEncodingISOLatinArabic = 0x0206,	/* ISO 8859-6, =ASMO 708, =DOS CP 708 */
    kCFStringEncodingISOLatinGreek = 0x0207,	/* ISO 8859-7 */
    kCFStringEncodingISOLatinHebrew = 0x0208,	/* ISO 8859-8 */
    kCFStringEncodingISOLatin5 = 0x0209,	/* ISO 8859-9 */
    kCFStringEncodingISOLatin6 = 0x020A,	/* ISO 8859-10 */
    kCFStringEncodingISOLatinThai = 0x020B,	/* ISO 8859-11 */
    kCFStringEncodingISOLatin7 = 0x020D,	/* ISO 8859-13 */
    kCFStringEncodingISOLatin8 = 0x020E,	/* ISO 8859-14 */
    kCFStringEncodingISOLatin9 = 0x020F,	/* ISO 8859-15 */

    /* MS-DOS & Windows encodings begin at 0x400 */
    kCFStringEncodingDOSLatinUS = 0x0400,	/* code page 437 */
    kCFStringEncodingDOSGreek = 0x0405,		/* code page 737 (formerly code page 437G) */
    kCFStringEncodingDOSBalticRim = 0x0406,	/* code page 775 */
    kCFStringEncodingDOSLatin1 = 0x0410,	/* code page 850, "Multilingual" */
    kCFStringEncodingDOSGreek1 = 0x0411,	/* code page 851 */
    kCFStringEncodingDOSLatin2 = 0x0412,	/* code page 852, Slavic */
    kCFStringEncodingDOSCyrillic = 0x0413,	/* code page 855, IBM Cyrillic */
    kCFStringEncodingDOSTurkish = 0x0414,	/* code page 857, IBM Turkish */
    kCFStringEncodingDOSPortuguese = 0x0415,	/* code page 860 */
    kCFStringEncodingDOSIcelandic = 0x0416,	/* code page 861 */
    kCFStringEncodingDOSHebrew = 0x0417,	/* code page 862 */
    kCFStringEncodingDOSCanadianFrench = 0x0418, /* code page 863 */
    kCFStringEncodingDOSArabic = 0x0419,	/* code page 864 */
    kCFStringEncodingDOSNordic = 0x041A,	/* code page 865 */
    kCFStringEncodingDOSRussian = 0x041B,	/* code page 866 */
    kCFStringEncodingDOSGreek2 = 0x041C,	/* code page 869, IBM Modern Greek */
    kCFStringEncodingDOSThai = 0x041D,		/* code page 874, also for Windows */
    kCFStringEncodingDOSJapanese = 0x0420,	/* code page 932, also for Windows */
    kCFStringEncodingDOSChineseSimplif = 0x0421, /* code page 936, also for Windows */
    kCFStringEncodingDOSKorean = 0x0422,	/* code page 949, also for Windows; Unified Hangul Code */
    kCFStringEncodingDOSChineseTrad = 0x0423,	/* code page 950, also for Windows */
/*  kCFStringEncodingWindowsLatin1 = 0x0500, defined in CoreFoundation/CFString.h */
    kCFStringEncodingWindowsLatin2 = 0x0501,	/* code page 1250, Central Europe */
    kCFStringEncodingWindowsCyrillic = 0x0502,	/* code page 1251, Slavic Cyrillic */
    kCFStringEncodingWindowsGreek = 0x0503,	/* code page 1253 */
    kCFStringEncodingWindowsLatin5 = 0x0504,	/* code page 1254, Turkish */
    kCFStringEncodingWindowsHebrew = 0x0505,	/* code page 1255 */
    kCFStringEncodingWindowsArabic = 0x0506,	/* code page 1256 */
    kCFStringEncodingWindowsBalticRim = 0x0507,	/* code page 1257 */
    kCFStringEncodingWindowsKoreanJohab = 0x0510, /* code page 1361, for Windows NT */
    kCFStringEncodingWindowsVietnamese = 0x0508, /* code page 1258 */

    /* Various national standards begin at 0x600 */
/*  kCFStringEncodingASCII = 0x0600, defined in CoreFoundation/CFString.h */
    kCFStringEncodingJIS_X0201_76 = 0x0620,
    kCFStringEncodingJIS_X0208_83 = 0x0621,
    kCFStringEncodingJIS_X0208_90 = 0x0622,
    kCFStringEncodingJIS_X0212_90 = 0x0623,
    kCFStringEncodingJIS_C6226_78 = 0x0624,
    kCFStringEncodingGB_2312_80 = 0x0630,
    kCFStringEncodingGBK_95 = 0x0631,		/* annex to GB 13000-93; for Windows 95 */
    kCFStringEncodingKSC_5601_87 = 0x0640,	/* same as KSC 5601-92 without Johab annex */
    kCFStringEncodingKSC_5601_92_Johab = 0x0641, /* KSC 5601-92 Johab annex */
    kCFStringEncodingCNS_11643_92_P1 = 0x0651,	/* CNS 11643-1992 plane 1 */
    kCFStringEncodingCNS_11643_92_P2 = 0x0652,	/* CNS 11643-1992 plane 2 */
    kCFStringEncodingCNS_11643_92_P3 = 0x0653,	/* CNS 11643-1992 plane 3 (was plane 14 in 1986 version) */

    /* ISO 2022 collections begin at 0x800 */
    kCFStringEncodingISO_2022_JP = 0x0820,
    kCFStringEncodingISO_2022_JP_2 = 0x0821,
    kCFStringEncodingISO_2022_CN = 0x0830,
    kCFStringEncodingISO_2022_CN_EXT = 0x0831,
    kCFStringEncodingISO_2022_KR = 0x0840,

    /* EUC collections begin at 0x900 */
    kCFStringEncodingEUC_JP = 0x0920,		/* ISO 646, 1-byte katakana, JIS 208, JIS 212 */
    kCFStringEncodingEUC_CN = 0x0930,		/* ISO 646, GB 2312-80 */
    kCFStringEncodingEUC_TW = 0x0931,		/* ISO 646, CNS 11643-1992 Planes 1-16 */
    kCFStringEncodingEUC_KR = 0x0940,		/* ISO 646, KS C 5601-1987 */

    /* Misc standards begin at 0xA00 */
    kCFStringEncodingShiftJIS = 0x0A01,		/* plain Shift-JIS */
    kCFStringEncodingKOI8_R = 0x0A02,		/* Russian internet standard */
    kCFStringEncodingBig5 = 0x0A03,		/* Big-5 (has variants) */
    kCFStringEncodingMacRomanLatin1 = 0x0A04,	/* Mac OS Roman permuted to align with ISO Latin-1 */
    kCFStringEncodingHZ_GB_2312 = 0x0A05,	/* HZ (RFC 1842, for Chinese mail & news) */

    /* Other platform encodings*/
/*  kCFStringEncodingNextStepLatin = 0x0B01, defined in CoreFoundation/CFString.h */

    /* EBCDIC & IBM host encodings begin at 0xC00 */
    kCFStringEncodingEBCDIC_US = 0x0C01,	/* basic EBCDIC-US */
    kCFStringEncodingEBCDIC_CP037 = 0x0C02	/* code page 037, extended EBCDIC (Latin-1 set) for US,Canada... */
} CFStringEncodings;
#endif
