/*
//  iTMLiteScanner.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Wed Dec 05 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSCharacterSet.h>
#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "iTMLiteScanner.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMLiteScanner
/*"This is quite a NSScanner, except that it retains its string while NSScanner makes a copy. Speed is acceptable.
It is not a subclass of NSScanner..."*/
@implementation iTMLiteScanner
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= scannerWithString:
+ (id) scannerWithString: (NSString *) aString;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[self alloc] initWithString: aString] autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= init
- (id) init;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    self = [super init];
    [self setString: nil];
    [self setCharactersToBeSkipped: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initWithString:
- (id) initWithString: (NSString *) aString;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    self = [self init];
    [self setString: aString];
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= dealloc
- (void) dealloc;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setString: nil];
    // only one is necessary...
    [self setCharactersToBeSkipped: nil];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= string
- (NSString *) string;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return self->string;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setString
- (void) setString: (NSString *) aString;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self->string autorelease];
    self->string = [aString retain];
//    CAI = (iTMCharacterAtIndexIMP) [aString methodForSelector: @selector(characterAtIndex:)];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= scanLocation
- (unsigned) scanLocation;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return self->scanLocation;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setScanLocation:
- (void) setScanLocation: (unsigned) pos;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    self->scanLocation = pos;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= charactersToBeSkipped
- (NSCharacterSet *) charactersToBeSkipped;
/*"Critical Description: if the set of charactersToBeSkipped is nil, it means that there is no character to be skipped.
In one word, no set is equivalent to a void one. It is the opposit for the charactersNotToBeSkipped!!!
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return self->charactersToBeSkipped;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setCharactersToBeSkipped:
- (void) setCharactersToBeSkipped: (NSCharacterSet *) set;
/*"Beware, both charactersNotToBeSkipped and charactersNotToBeSkipped are changed by this message.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self->charactersToBeSkipped autorelease];
    self->charactersToBeSkipped = [set retain];
    [self->charactersNotToBeSkipped autorelease];
    self->charactersNotToBeSkipped = [[set invertedSet] retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= charactersNotToBeSkipped
- (NSCharacterSet *) charactersNotToBeSkipped;
/*"Critical Description: if the set of charactersNotToBeSkipped is nil, it means that there is no character to be skipped.
In one word, no set is equivalent to a full one. It is the opposit for the charactersToBeSkipped!!!
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return self->charactersNotToBeSkipped;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setCharactersNotToBeSkipped:
- (void) setCharactersNotToBeSkipped: (NSCharacterSet *) set;
/*"Beware, both charactersNotToBeSkipped and charactersNotToBeSkipped are changed by this message.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self->charactersNotToBeSkipped autorelease];
    self->charactersNotToBeSkipped = [set retain];
    [self->charactersToBeSkipped autorelease];
    self->charactersToBeSkipped = [[set invertedSet] retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= caseSensitive
- (BOOL) caseSensitive;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return self->caseSensitive;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setCaseSensitive
- (void) setCaseSensitive: (BOOL) flag;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    self->caseSensitive = flag;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= scanString:intoString:
- (BOOL) scanString: (NSString *) aString intoString: (NSString **) value;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSRange searchRange = NSMakeRange(self->scanLocation, [self->string length] - self->scanLocation);
    NSRange r;
    if(self->charactersNotToBeSkipped)
    {
        r = [self->string rangeOfCharacterFromSet: self->charactersNotToBeSkipped
                    options: NSLiteralSearch
                        range: searchRange];
        if((r.location != NSNotFound) && (r.location<[self->string length]))
            searchRange = NSMakeRange(r.location, [self->string length] - r.location);
        else
            return NO;
    }
    r = [self->string rangeOfString: aString
            options: (self->caseSensitive? NSLiteralSearch: NSCaseInsensitiveSearch)
                range: searchRange];
    if(r.length && (r.location == searchRange.location))
    {
        [self setScanLocation: NSMaxRange(r)];
        if(value) *value = [self->string substringWithRange: r];
        return YES;
    }
    else
        return NO;
}
#if 0
*	NSCaseInsensitiveSearch
*	NSLiteralSearch
*	NSBackwardsSearch
*	NSAnchoredSearch
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= scanCharactersFromSet:intoString:
- (BOOL) scanCharactersFromSet: (NSCharacterSet *) set intoString: (NSString **) value;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSRange searchRange = NSMakeRange(self->scanLocation, [self->string length] - self->scanLocation);
    NSRange r;
    if(self->charactersNotToBeSkipped)
    {
        r = [self->string rangeOfCharacterFromSet: self->charactersNotToBeSkipped
                    options: NSLiteralSearch
                        range: searchRange];
        if((r.location != NSNotFound) && (r.location<[self->string length]))
            searchRange = NSMakeRange(r.location, [self->string length] - r.location);
        else
            return NO;
    }
    r = [self->string rangeOfCharacterFromSet: [set invertedSet]
            options: NSLiteralSearch
                range: searchRange];
    if(r.location > searchRange.location)
    {
        [self setScanLocation: r.location];
        if(value) *value = [self->string substringWithRange:
                            NSMakeRange(searchRange.location, r.location - searchRange.location)];
        return YES;
    }
    else
        return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= scanUpToString:intoString:
- (BOOL) scanUpToString: (NSString *) aString intoString: (NSString **) value;
/*"If aString is not found, the whole remaining string is scanned into *value.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSRange searchRange = NSMakeRange(self->scanLocation, [self->string length] - self->scanLocation);
    NSRange r;
    if(self->charactersNotToBeSkipped)
    {
        r = [self->string rangeOfCharacterFromSet: self->charactersNotToBeSkipped
                    options: NSLiteralSearch
                        range: searchRange];
        if((r.location != NSNotFound) && (r.location<[self->string length]))
            searchRange = NSMakeRange(r.location, [self->string length] - r.location);
        else
            return NO;
    }
    r = [self->string rangeOfString: aString
            options: (self->caseSensitive? NSLiteralSearch: NSCaseInsensitiveSearch)
                range: searchRange];
    if(r.length)
    {
        if(r.location > searchRange.location)
        {
            if(value) *value = [self->string substringWithRange:
                NSMakeRange(self->scanLocation, r.location - self->scanLocation)];
            [self setScanLocation: r.location];
            return YES;
        }
        else
            return NO;
    }
    else
    {
        if(value) *value = [self->string substringWithRange:
            NSMakeRange(self->scanLocation, [self->string length] - self->scanLocation)];
        [self setScanLocation: [self->string length]];
        return YES;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= scanUpToCharactersFromSet:intoString:
- (BOOL) scanUpToCharactersFromSet: (NSCharacterSet *) set intoString: (NSString **) value;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSRange searchRange = NSMakeRange(self->scanLocation, [self->string length] - self->scanLocation);
    NSRange r;
    if(self->charactersNotToBeSkipped)
    {
        r = [self->string rangeOfCharacterFromSet: self->charactersNotToBeSkipped
                    options: NSLiteralSearch
                        range: searchRange];
        if((r.location != NSNotFound) && (r.location<[self->string length]))
            searchRange = NSMakeRange(r.location, [self->string length] - r.location);
        else
            return NO;
    }
    r = [self->string rangeOfCharacterFromSet: set
            options: NSLiteralSearch
                range: searchRange];
    if(r.location == NSNotFound)
    {
        if(value) *value = [self->string substringWithRange: searchRange];
        [self setScanLocation: [self->string length]];
        return YES;
    }
    else if(r.location < [self->string length])
    {
        if(value && (r.location > searchRange.location)) *value = [self->string substringWithRange:
                NSMakeRange(searchRange.location, r.location - searchRange.location)];
        [self setScanLocation: r.location];
        return YES;
    }
    else
        return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= scanUpToEOLIntoString:
- (BOOL) scanUpToEOLIntoString: (NSString **) value;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Use characters to be skipped...
"*/
{
    return [self scanUpToCharactersFromSet: [NSCharacterSet characterSetWithCharactersInString: @"\n\r"]
        intoString: value];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= isAtEnd
- (BOOL) isAtEnd;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Use characters to be skipped...
"*/
{
    return self->scanLocation >= [self->string length];
}
@end

@implementation iTMLiteScanner (iTMExtendedScanner) 
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= scanInt
- (BOOL) scanInt: (int *) value;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned end;
    BOOL result;
    NSScanner * S;
    [self->string getLineStart: nil end: &end contentsEnd: nil forRange: NSMakeRange(self->scanLocation, 0)];
    S = [NSScanner scannerWithString: [self->string substringWithRange:
        NSIntersectionRange(NSMakeRange(self->scanLocation, end - self->scanLocation),
            NSMakeRange(0, [self->string length]))]];
    result = [S scanInt: value];
    self->scanLocation += [S scanLocation];
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= scanHexInt
- (BOOL) scanHexInt: (unsigned *) value;		/* Optionally prefixed with "0x" or "0X" */
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned end;
    BOOL result;
    NSScanner * S;
    [self->string getLineStart: nil end: &end contentsEnd: nil forRange: NSMakeRange(self->scanLocation, 0)];
    S = [NSScanner scannerWithString: [self->string substringWithRange:
        NSIntersectionRange(NSMakeRange(self->scanLocation, end - self->scanLocation),
            NSMakeRange(0, [self->string length]))]];
    result = [S scanHexInt: value];
    self->scanLocation += [S scanLocation];
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= scanLongLong
- (BOOL) scanLongLong: (long long *) value;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned end;
    BOOL result;
    NSScanner * S;
    [self->string getLineStart: nil end: &end contentsEnd: nil forRange: NSMakeRange(self->scanLocation, 0)];
    S = [NSScanner scannerWithString: [self->string substringWithRange:
        NSIntersectionRange(NSMakeRange(self->scanLocation, end - self->scanLocation),
            NSMakeRange(0, [self->string length]))]];
    result = [S scanLongLong: value];
    self->scanLocation += [S scanLocation];
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= scanFloat
- (BOOL) scanFloat: (float *) value;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned end;
    BOOL result;
    NSScanner * S;
    [self->string getLineStart: nil end: &end contentsEnd: nil forRange: NSMakeRange(self->scanLocation, 0)];
    S = [NSScanner scannerWithString: [self->string substringWithRange:
        NSIntersectionRange(NSMakeRange(self->scanLocation, end - self->scanLocation),
            NSMakeRange(0, [self->string length]))]];
    result = [S scanFloat: value];
    self->scanLocation += [S scanLocation];
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= scanDouble:
- (BOOL) scanDouble: (double *) value;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned end;
    BOOL result;
    NSScanner * S;
    [self->string getLineStart: nil end: &end contentsEnd: nil forRange: NSMakeRange(self->scanLocation, 0)];
    S = [NSScanner scannerWithString: [self->string substringWithRange:
        NSIntersectionRange(NSMakeRange(self->scanLocation, end - self->scanLocation),
            NSMakeRange(0, [self->string length]))]];
    result = [S scanDouble: value];
    self->scanLocation += [S scanLocation];
    return result;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= message
- (void) message;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return;
}
#endif
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMLiteScanner


#if 0
- (BOOL)hasPrefix:(NSString *)aString;
- (BOOL)hasSuffix:(NSString *)aString;
- (NSRange)rangeOfString:(NSString *)aString options:(unsigned)mask range:(NSRange)searchRange;
- (NSRange)rangeOfCharacterFromSet:(NSCharacterSet *)aSet options:(unsigned int)mask range:(NSRange)searchRange;
#endif

