/*
//  iTMLogSyntaxParser.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Fri Dec 07 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSNotification.h>
#import <Foundation/NSAttributedString.h>
#import <Foundation/NSException.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSCharacterSet.h>
#import <Foundation/NSUserDefaults.h>
#import <AppKit/NSApplication.h>
#import <AppKit/NSAttributedString.h>
#import "NSUserDefaults(iTeXMac).h"
#import "iTMLiteScanner.h"
#import "iTMLogSyntaxParser.h"

NSString * const iTMUDLogRegularFontKey = @"iTMUDLogRegularFont";
NSString * const iTMUDLogWarningFontKey = @"iTMUDLogWarningFont";
NSString * const iTMUDLogErrorFontKey = @"iTMUDLogErrorFont";
NSString * const iTMUDLogInfoFontKey = @"iTMUDLogInfoFont";

NSString * const iTMUDLogRegularColorKey = @"iTMUDLogRegularColor";
NSString * const iTMUDLogWarningColorKey = @"iTMUDLogWarningColor";
NSString * const iTMUDLogErrorColorKey = @"iTMUDLogErrorColor";
NSString * const iTMUDLogInfoColorKey = @"iTMUDLogInfoColor";

NSString * const iTMLogInputAttributeName = @"I";
#define diTMIAN iTMLogInputAttributeName

//NSString * const iTMRegularLink = @"1";
NSString * const iTMWarningInput = @"2";
//NSString * const iTMWarningLineInput = @"3";
NSString * const iTMErrorInput = @"4";
//NSString * const iTMErrorLineInput = @"5";
NSString * const iTMInfoInput = @"6";
//NSString * const iTMInfoLineInput = @"7";

NSString * const iTMBadInput = @"1";

NSString * const iTMLineAttributeName = @"L";
NSString * const iTMDiagnosticAttributeName = @"D";

#define diTMRegularInput nil
//#define kiTMRegularInput 0
//#define kiTMRegularLinkInput 1
#define kiTMWarningInput 2
//#define kiTMWarningLineInput 3
#define kiTMErrorInput 4
//#define kiTMErrorLineInput 5
#define kiTMInfoInput 6
//#define kiTMInfoLineInput 7
#define kiTMFileInput 8
//#define kiTMFileLinkInput 9

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMLogSyntaxParser
/*"Description forthcoming."*/
@implementation iTMLogSyntaxParser
static id _iTMLogSyntaxParser = nil;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialize
+ (void) initialize;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
        [[NSFont fontWithName: @"Monaco" size: 12.0] nameSizeDictionary], iTMUDLogRegularFontKey,
        [[NSFont fontWithName: @"Monaco" size: 12.0] nameSizeDictionary], iTMUDLogErrorFontKey,
        [[NSFont fontWithName: @"Monaco" size: 12.0] nameSizeDictionary], iTMUDLogInfoFontKey,
        [[NSFont fontWithName: @"Monaco" size: 12.0] nameSizeDictionary], iTMUDLogWarningFontKey,
        [[NSColor blackColor] RGBADictionary], iTMUDLogRegularColorKey,
        [[NSColor colorWithCalibratedRed: 0.0 green: 0.0 blue: 0.6 alpha: 1.0] RGBADictionary], iTMUDLogWarningColorKey,
        [[NSColor colorWithCalibratedRed: 0.5 green: 0.0 blue: 0.0 alpha: 1.0] RGBADictionary], iTMUDLogErrorColorKey,
        [[NSColor colorWithCalibratedRed: 0.5 green: 0.0 blue: 0.0 alpha: 1.0] RGBADictionary], iTMUDLogInfoColorKey,
            nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  syntaxParser
+ (id) syntaxParser;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[self alloc] init] autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fontOrColorDidChange:
- (void) fontOrColorDidChange: (NSNotification *) aNotification;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: NYI
"*/
{
    id D = [aNotification object];
    if(!D)
        D = [NSUserDefaults standardUserDefaults];
    {
        #define SETATTRIBUTES(mode, modeLink, fontKey, colorKey)\
        NSDictionary * mode = [NSDictionary dictionaryWithObjectsAndKeys:\
                [NSFont fontWithNameSizeDictionary: [D objectForKey: fontKey]], NSFontAttributeName,\
                [NSColor colorWithRGBADictionary: [D objectForKey: colorKey]], NSForegroundColorAttributeName, nil];\
        NSDictionary * modeLink = [NSDictionary dictionaryWithObjectsAndKeys:\
                [NSFont fontWithNameSizeDictionary: [D objectForKey: fontKey]], NSFontAttributeName,\
                [NSColor colorWithRGBADictionary: [D objectForKey: colorKey]], NSForegroundColorAttributeName,\
                [NSNumber numberWithInt: 1], NSUnderlineStyleAttributeName,\
                nil]
        SETATTRIBUTES(regular, regularLink, iTMUDLogRegularFontKey, iTMUDLogRegularColorKey);
        SETATTRIBUTES(error, errorLine, iTMUDLogErrorFontKey, iTMUDLogErrorColorKey);
        SETATTRIBUTES(warning, warningLine, iTMUDLogWarningFontKey, iTMUDLogWarningColorKey);
        SETATTRIBUTES(info, infoLine, iTMUDLogInfoFontKey, iTMUDLogInfoColorKey);

        [self setAttributesArray: [NSArray arrayWithObjects:
            regular, regularLink, warning, warningLine, error, errorLine, info, infoLine, info, infoLine,
                                    nil]];
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isColoring
- (BOOL) isColoring;
/*"Subclasses will answer YES or NO depending on their "coloring" status.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return YES;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:attributesAtIndex:effectiveRange:
- (NSDictionary *) attributedString: (NSAttributedString *) attributedString attributesAtIndex: (unsigned) aLocation
effectiveRange: (NSRangePointer) aRangePtr;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    aLocation = MIN(aLocation, [attributedString length]);
    {
        int mode = [[attributedString attribute: diTMIAN atIndex: aLocation effectiveRange: aRangePtr] intValue];
        NSRange r;
        NSRangePointer rPtr = aRangePtr? &r: nil;
        NSNumber * lineNumber = [attributedString attribute: iTMLineAttributeName
                        atIndex: aLocation effectiveRange: rPtr];
        int lineCorrection = [lineNumber intValue]? 1: 0;
        if(lineCorrection)
        {
            NSMutableDictionary * result = [NSMutableDictionary
                    dictionaryWithDictionary: [[self attributesArray] objectAtIndex: mode + lineCorrection]];
            [result setObject: lineNumber forKey: NSLinkAttributeName];
            if(aRangePtr) *aRangePtr = r;
            return result;
        }
        else
            return [[self attributesArray] objectAtIndex: mode];
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:invalidRangeForCharactersInOldRange:replacedWithString:
- (NSRange) attributedString: (NSAttributedString *) anAttributedString invalidRangeForCharactersInOldRange: (NSRange) aRange replacedWithString: (NSString *) aString;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned length = [aString length];
    if(length > 0)
    {
	unsigned begin, end;
	NSRange r = NSIntersectionRange(NSMakeRange(aRange.location, length), NSMakeRange(0, length));
	[[anAttributedString string] getLineStart: &begin end: &end contentsEnd: nil forRange: r];
	return NSMakeRange(begin, end - begin);
    }
    return NSMakeRange(0, 0);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:fixAttributesInRange:
- (void) attributedString: (NSAttributedString *) anAttributedString fixAttributesInRange: (NSRange) aRange;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * string = [anAttributedString string];
    iTMLiteScanner * scanner = [iTMLiteScanner scannerWithString: string];
    unsigned max;
    unsigned scanLocation = aRange.location;
    NSRange r;
    BOOL badInput = NO;
    int previousPageNumber = 0;
    aRange = NSIntersectionRange(aRange, NSMakeRange(0, [string length]));
    r = aRange;
    max = NSMaxRange(aRange);
    if((scanLocation >0) && ([[anAttributedString attribute: iTMDiagnosticAttributeName
                atIndex: scanLocation -1 longestEffectiveRange: &r inRange: NSMakeRange(0, max)] intValue] > 0))
        aRange = NSUnionRange(aRange, r);
    {// always parsing from the beginning of the line
        unsigned begin;
        [string getLineStart: &begin end: nil contentsEnd: nil forRange: NSMakeRange(aRange.location, 0)];
        aRange = NSUnionRange(aRange, NSMakeRange(begin, 0));
    }
    max = NSMaxRange(aRange);
    scanLocation = aRange.location;
    // looking for the last page number previously parsed (problem with non continuous page numbers...)
    while(scanLocation>0)
    {
        NSRange r = [string rangeOfString: @"[" options: NSBackwardsSearch range: NSMakeRange(0,scanLocation)];//]
        if(r.length)
        {
            [scanner setScanLocation: r.location + 1];
            if([scanner scanInt: nil])
            {
                id value = [anAttributedString attribute: iTMLineAttributeName
                    atIndex: [scanner scanLocation] -1 longestEffectiveRange: nil inRange: NSMakeRange(0, max)];
                if([value respondsToSelector: @selector(intValue)])
                {
                    previousPageNumber = [value intValue];
                    break;
                }
            }
        }
        else
            break;
        scanLocation = MIN(r.location, scanLocation - 1);
    }
    scanLocation = aRange.location;    
    while(scanLocation < max)
    {
        [scanner setScanLocation: scanLocation];
        r = NSMakeRange(scanLocation, 0);
        if([scanner scanUpToCharactersFromSet:
                        [[NSCharacterSet whitespaceAndNewlineCharacterSet] invertedSet] intoString: nil])
            r.length = [scanner scanLocation] - r.location;
        [anAttributedString addAttribute: diTMIAN value: diTMRegularInput range: r];
        [anAttributedString addAttribute: iTMDiagnosticAttributeName value: nil range: r];
        scanLocation = [scanner scanLocation];
//        NSLog(@"substring: <%@>", [string substringWithRange: NSMakeRange(scanLocation, 10)]);
        if([scanner scanString: @"!" intoString: nil])
        {
            unsigned end = 0;
            unsigned nextEnd = 0;
            unsigned nextStart = 0;
            unsigned lineNumber = 0;
            int counter;
            NSRange lineKeyRange;
            iTMLiteScanner * subScanner;
            NSString * line;
            [string getLineStart: nil end: &end contentsEnd: nil forRange: NSMakeRange(scanLocation, 0)];
            nextStart = end;
            for(counter = 0; counter < 10; ++counter)
            {
                [string getLineStart: nil end: &nextEnd contentsEnd: nil forRange: NSMakeRange(nextStart, 0)];
                line = [string substringWithRange: NSMakeRange(nextStart, nextEnd - nextStart)];
                subScanner = [iTMLiteScanner scannerWithString: line];
                [subScanner scanUpToString: @"l." intoString: nil];
                lineKeyRange.location = [subScanner scanLocation];
                if([subScanner scanString: @"l." intoString: nil] && [subScanner scanInt: &lineNumber])
                {
                    lineKeyRange.length = [subScanner scanLocation] - lineKeyRange.location;
                    lineKeyRange.location += nextStart;
                    [anAttributedString addAttribute: iTMLineAttributeName value: [NSNumber numberWithInt: lineNumber] range: lineKeyRange];
                    end = NSMaxRange(lineKeyRange);
                    break;
                }
                nextStart = nextEnd;
                if(nextStart > [string length])
                {
                    badInput = YES;
                    break;
                }
            }
            r = NSMakeRange(scanLocation, end - scanLocation);
            [anAttributedString addAttribute: diTMIAN value: iTMErrorInput range: r];
            if(badInput)
                [anAttributedString addAttribute: iTMDiagnosticAttributeName value: iTMBadInput range: r];
            else
                [anAttributedString addAttribute: iTMDiagnosticAttributeName value: nil range: r];
            scanLocation = end;
        }
        else if(([scanner scanString: @"LaTeX Warning:" intoString: nil]) ||
                    ([scanner scanString: @"LaTeX Font Warning:" intoString: nil]) ||
                            ([scanner scanString: @"pdfTeX Warning:" intoString: nil]))
        {// two lines parsing
            unsigned nextEnd = 0;
            unsigned nextStart = 0;
            unsigned contentsEnd = 0;
            unsigned nextContentsEnd = 0;
            unsigned line = 0;
            NSRange lineKeyRange;
            NSString * substring;
            iTMLiteScanner * subScanner;
            [string getLineStart: nil end: &nextStart contentsEnd: &contentsEnd
                                                forRange: NSMakeRange([scanner scanLocation], 0)];
            substring = [string substringWithRange: NSMakeRange(scanLocation, contentsEnd - scanLocation)];
            if(nextStart <= [string length])
            {
                [string getLineStart: nil end: &nextEnd contentsEnd: &nextContentsEnd forRange: NSMakeRange(nextStart, 0)];
                substring = [substring stringByAppendingString:
                    [string substringWithRange: NSMakeRange(nextStart, nextContentsEnd - nextStart)]];
                subScanner = [iTMLiteScanner scannerWithString: substring];
//                NSLog(@"substring: <%@>", substring);
                if(([subScanner scanUpToString: @"line" intoString: nil],
                        [subScanner scanString: @"line" intoString: nil]))
                {
                    lineKeyRange.location = [subScanner scanLocation] - 4;
                    if([subScanner scanInt: &line])
                    {
                        lineKeyRange.length = [subScanner scanLocation] - lineKeyRange.location;
                        lineKeyRange.location += scanLocation;
                        if(lineKeyRange.location >= contentsEnd)
                            lineKeyRange.location += nextStart - contentsEnd;
                        else if(NSMaxRange(lineKeyRange) >= contentsEnd)
                            lineKeyRange.length += nextStart - contentsEnd;
                        [anAttributedString addAttribute: iTMLineAttributeName
                                                    value: [NSNumber numberWithInt: line] range: lineKeyRange];
                        nextStart = nextEnd;
                    }
                }
            }
            else
                badInput = YES;
            r = NSMakeRange(scanLocation, nextStart - scanLocation);
            [anAttributedString addAttribute: diTMIAN value: iTMWarningInput range: r];
            if(badInput)
                [anAttributedString addAttribute: iTMDiagnosticAttributeName value: iTMBadInput range: r];
            else
                [anAttributedString addAttribute: iTMDiagnosticAttributeName value: nil range: r];
            scanLocation = nextStart;
        }
        else if([scanner scanString: @"LaTeX Font Info:" intoString: nil])
        {// two lines parsing
            unsigned end = 0;
            unsigned line = 0;
            NSRange lineKeyRange;
            NSString * substring;
            iTMLiteScanner * subScanner;
            
            [string getLineStart: nil end: &end contentsEnd: nil
                                                forRange: NSMakeRange([scanner scanLocation], 0)];
            r = NSMakeRange(scanLocation, end - scanLocation);
            substring = [string substringWithRange: r];
            subScanner = [iTMLiteScanner scannerWithString: substring];
//            NSLog(@"substring: <%@>", substring);
            while(![subScanner isAtEnd])
            {
                [subScanner scanUpToString: @"line" intoString: nil];
                if([subScanner scanString: @"line" intoString: nil])
                {
                    lineKeyRange.location = [subScanner scanLocation] - 4;
                    if([subScanner scanInt: &line])
                    {
                        lineKeyRange.length = [subScanner scanLocation] - lineKeyRange.location;
                        lineKeyRange.location += scanLocation;
                        [anAttributedString addAttribute: iTMLineAttributeName
                                        value: [NSNumber numberWithInt: line] range: lineKeyRange];
                        break;
                    }
                    else
                        badInput = YES;
                }
            }
            [anAttributedString addAttribute: diTMIAN value: iTMInfoInput range: r];
            if(badInput)
                [anAttributedString addAttribute: iTMDiagnosticAttributeName value: iTMBadInput range: r];
            else
                [anAttributedString addAttribute: iTMDiagnosticAttributeName value: nil range: r];
            scanLocation = end;
        }
        else
        {
            unsigned end = 0;
            [string getLineStart: nil end: &end contentsEnd: nil forRange: NSMakeRange(scanLocation, 0)];
            {
                iTMLiteScanner * subScanner = [iTMLiteScanner scannerWithString:
                                [string substringWithRange: NSMakeRange(scanLocation, end - scanLocation)]];
                BOOL page = NO;
                while(([subScanner scanUpToString: @"[" intoString: nil],
                    [subScanner scanString: @"[" intoString: nil]))
                {
                    unsigned begin = [subScanner scanLocation];
                    if(page = [subScanner scanInt: nil] || page)
                        [anAttributedString addAttribute: iTMLineAttributeName
                            value: [NSNumber numberWithInt: --previousPageNumber]
                                range: NSMakeRange(begin + scanLocation, [subScanner scanLocation] - begin)];
                    [anAttributedString setAttributes: nil
                        range: NSMakeRange(scanLocation + [subScanner scanLocation],
                                                end - [subScanner scanLocation] - scanLocation)];
                    if(!([subScanner scanUpToString: @"]" intoString: nil],
                            [subScanner scanString: @"]" intoString: nil]))
                        break;
                }
                if(page)
                {
                    r = NSMakeRange(scanLocation, end - scanLocation);
                    [anAttributedString addAttribute: diTMIAN value: diTMRegularInput range: r];
                }
                else if(([subScanner setScanLocation: 0],
                        [subScanner scanUpToString: @"on input line" intoString: nil],
                            [subScanner scanString: @"on input line" intoString: nil]))
                {
                    NSRange lineKeyRange;
                    int line;
                    lineKeyRange.location = [subScanner scanLocation] - 4;
                    if([subScanner scanInt: &line])
                    {
                        lineKeyRange.length = [subScanner scanLocation] - lineKeyRange.location;
                        lineKeyRange.location += scanLocation;
                        [anAttributedString addAttribute: iTMLineAttributeName
                                        value: [NSNumber numberWithInt: line] range: lineKeyRange];
                        r = NSMakeRange(scanLocation, end - scanLocation);
                        [anAttributedString addAttribute: diTMIAN value: iTMInfoInput range: r];
                    }
                    else
                        badInput = YES;
//                    NSLog(@"on input line: %@", [subScanner string]);
                }
//                else
//                    NSLog(@"rien dans\n<%@>", [subScanner string]);
                if(badInput)
                    [anAttributedString addAttribute: iTMDiagnosticAttributeName value: iTMBadInput range: r];
                else
                    [anAttributedString addAttribute: iTMDiagnosticAttributeName value: nil range: r];
                scanLocation = end;
            }
        }
    }
//    NSLog(@"+ anAttributedString: %@", anAttributedString);
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:rangeOfErrorFromIndex:cycle:backwards:
- (NSRange) attributedString: (NSAttributedString *) attributedString rangeOfErrorFromIndex: (unsigned) charIndex cycle: (BOOL) cycle backwards: (BOOL) backwards;
/*"Returns the range of the first error after index charIndex...
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: does not work well because the attributes are not stored well by fixAttributes...
"*/
{
    unsigned ceiling = [attributedString length];
    unsigned scanLocation = charIndex;
    NSRange range;
    if(backwards)
    {
        while (scanLocation > 0)
        {
            int mode = [[attributedString attribute: diTMIAN atIndex: scanLocation - 1
                            longestEffectiveRange: &range inRange: NSMakeRange(0, ceiling)] intValue];
            if((mode == kiTMWarningInput) || (mode == kiTMErrorInput))
                return range;
            scanLocation = range.location;
        }
        if(cycle)
            return [self attributedString: attributedString
                            rangeOfErrorFromIndex: [attributedString length] cycle: NO backwards: backwards];
    }
    else
    {
        while (scanLocation < ceiling)
        {
            int mode = [[attributedString attribute: diTMIAN atIndex: scanLocation
                            longestEffectiveRange: &range inRange: NSMakeRange(0, ceiling)] intValue];
            if((mode == kiTMWarningInput) || (mode == kiTMErrorInput))
                return range;
            scanLocation = MAX(NSMaxRange(range), charIndex + 1);
        }
        if(cycle)
            return [self attributedString: attributedString rangeOfErrorFromIndex: 0 cycle: NO backwards: backwards];
    }
    return NSMakeRange(charIndex, 0);
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMLogSyntaxParser

