/*
//  iTMSyntaxParser.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Wed Nov 07 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSNotification.h>
#import <Foundation/NSAttributedString.h>
#import <Foundation/NSException.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSCharacterSet.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSScanner.h>
#import <AppKit/NSApplication.h>
#import <AppKit/NSAttributedString.h>
#import "NSUserDefaults(iTeXMac).h"
#import "NSString(iTeXMac).h"
#import "iTMSyntaxParser.h"
#import "iTMTextWatcher.h"
#import "iTMTextStorage.h"
#import "iTMSettings.h"
//#import "iTMLiteScanner.h"

NSString * const iTMFontOrColorDidChangeNotification = @"iTMFontOrColorDidChange";

NSString * const iTMUDStandardFontKey = @"iTMUDStandardFont";
NSString * const iTMUDStandardColorKey = @"iTMUDStandardColor";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMSyntaxParser
/*"Semi abstract class inn the sense that it is very poor, concrete subclasses are iTMTeXSyntaxParser and iTMLogSyntaxParser.
All attributes messages refer to attributes used by the cocoa layout manager to properly display text. Deep attributes are stored in the attributed string. Logical attributes are filtered deep attributes. For example the TeX command \begin is stored with a "begin command" deep attribute for '\' (which is a control char) and a "command" deep attribute for the 5 remaining chars. The attributes for this word (all 6 chars) are the font and fore ground color in the user defaults corresponding to commands. The logical attributes for the whole sequence characters is a "command" attribute, then merging the "begin command" and "command" deep attributes."*/
@implementation iTMSyntaxParser
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialize
+ (void) initialize;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
        [[NSFont fontWithName: @"Monaco" size: 12.0] nameSizeDictionary], iTMUDStandardFontKey,
        [[NSColor blackColor] RGBADictionary], iTMUDStandardColorKey,
            nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  syntaxParser
+ (id) syntaxParser;
/*"One shared instance per class: subclassers will override this method.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[self alloc] init] autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"Designated initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(self = [super init])
    {
        #warning Should not always observe notifications (the text view in the defaults does not need to...)
        [[NSNotificationCenter defaultCenter]
            removeObserver: self
                name: iTMFontOrColorDidChangeNotification
                    object: nil];
        [[NSNotificationCenter defaultCenter]
            addObserver: self
                selector: @selector(defaultsFontOrColorDidChange:)
                    name: iTMFontOrColorDidChangeNotification
                        object: nil];
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [[NSNotificationCenter defaultCenter] removeObserver: nil name: nil object: self];
    [self setTextStorage: nil];
    [self setAttributesArray: nil];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textStorage
- (iTMTextStorage *) textStorage;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _TS;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  replaceTextStorage:
- (void) replaceTextStorage: (iTMTextStorage *) argument;
/*"Description forthcoming. Registers the receiver as the delegate of the layout managers of the text storage.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: NSLayoutManager
"*/
{
    [self setTextStorage: argument];
    if([self textStorage])
        [self fontOrColorDidChange: nil];
    [[[self textStorage] layoutManagers] makeObjectsPerformSelector: @selector(setDelegate:) withObject: self];
    _LayoutFinishedFlag = NO;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  layoutManager:didCompleteLayoutForTextContainer:atEnd:
- (void) layoutManager: (NSLayoutManager *) layoutManager didCompleteLayoutForTextContainer: (NSTextContainer *) textContainer atEnd: (BOOL) layoutFinishedFlag;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: NSLayoutManager
"*/
{
//    NSBeep();
//    NSLog(@"didCompleteLayoutForTextContainer %@", (layoutFinishedFlag? @"YES": @"NO"));
    if(layoutFinishedFlag)
        _LayoutFinishedFlag = YES;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setTextStorage:
- (void) setTextStorage: (iTMTextStorage *) argument;
/*"Description forthcoming. Use the replaceTextStorage above.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [iTMTextStorage class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] iTMTextStorage argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    [_TS autorelease];
    _TS = [argument retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  defaultsFontOrColorDidChange:
- (void) defaultsFontOrColorDidChange: (NSNotification *) aNotification;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Completely use of the notificationobject. 
"*/
{
    [self fontOrColorDidChange: aNotification];
    [_TSinvalidateLayout];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fontOrColorDidChange:
- (void) fontOrColorDidChange: (NSNotification *) aNotification;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Completely use of the notificationobject. 
"*/
{
    id settings = [aNotification object]? [aNotification object]: [self settings];
    NSDictionary * standard = [NSDictionary dictionaryWithObjectsAndKeys:
        [NSFont fontWithNameSizeDictionary: [settings objectForKey: iTMUDStandardFontKey]],
            NSFontAttributeName,
        [NSColor colorWithRGBADictionary: [settings objectForKey: iTMUDStandardColorKey]],
            NSForegroundColorAttributeName,
        nil];
    [self setAttributesArray: [NSArray arrayWithObjects: standard, nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributesArray
- (id) attributesArray;
/*"Lazy initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _AsA;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isColoring
- (BOOL) isColoring;
/*"Subclasses will answer YES or NO depending on their "coloring" status.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isITMCommentAtIndex:
- (BOOL) isITMCommentAtIndex: (unsigned) aLocation;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setAttributesArray:
- (void) setAttributesArray: (id) argument;
/*"The attributes really stored are those used by the mutable attributed string of the receiver.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSArray class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSArray argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    {
        NSMutableArray * AsA = [NSMutableArray arrayWithCapacity: [argument count]];
        NSEnumerator * E = [argument objectEnumerator];
        id O;
        int length = [_TS length];
        NSMutableAttributedString * MAS = [_TS mutableAttributedString];
        if(length)
        {
            id old = [MAS attributesAtIndex: 0 effectiveRange: nil];
            --length;
            while(O = [E nextObject])
                if([O isKindOfClass: [NSDictionary class]])
                {
                    [MAS setAttributes: O range: NSMakeRange(length, 1)];
                    [AsA addObject: [MAS attributesAtIndex: length effectiveRange: nil]];
                }
            [MAS setAttributes: old range: NSMakeRange(length, 1)];
        }
        else if(_TS)
        {
            [MAS replaceCharactersInRange: NSMakeRange(0, 0) withString: @"1"];
            while(O = [E nextObject])
                if([O isKindOfClass: [NSDictionary class]])
                {
                    [MAS setAttributes: O range: NSMakeRange(0, 1)];
                    [AsA addObject: [MAS attributesAtIndex: 0 effectiveRange: nil]];
                }
            [MAS replaceCharactersInRange: NSMakeRange(0, 1) withString: @""];
        }
        [_AsA autorelease];
        _AsA = [AsA count]? [AsA copy]: nil;
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= replaceCharactersInRange:withString:
- (void) replaceCharactersInRange: (NSRange) aRange withString: (NSString *) aString invalidRange: (NSRangePointer) rangePtr;
/*"Relies on the shared syntax parser.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[_TS mutableAttributedString] replaceCharactersInRange: aRange withString: aString];
//    if(rangePtr) * rangePtr = NSMakeRange(aRange.location, [aString length]);
    [_TS edited: NSTextStorageEditedCharacters range: aRange changeInLength: [aString length] - aRange.length];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= fixAttributesInRange:
- (void) fixAttributesInRange: (NSRange) aRange;
/*"Does nothing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    [[self syntaxParser] fixAttributesInRange: aRange];
    return;
}
@end

@implementation iTMSyntaxParser(iTMAttributes)
/*"All the following methods MUST return a non zero length range when asked for. Forever spinning wheel in iTM-1.0.32"*/
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributesAtIndex:effectiveRange:
- (NSDictionary *) attributesAtIndex: (unsigned) aLocation effectiveRange: (NSRangePointer) aRangePtr;
/*"This MUST be overriden by subclassers.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(aRangePtr) * aRangePtr = NSMakeRange(0, [_TS length]);
    return [[self attributesArray] objectAtIndex: 0];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributesAtIndex:longestEffectiveRange:inRange:
- (NSDictionary *) attributesAtIndex: (unsigned) aLocation longestEffectiveRange: (NSRangePointer) aRangePtr inRange: (NSRange) rangeLimit;
/*"Just calls the non longest version, cutting the range pointered to the rangle limit.
Subclassers will override this method for performance reasons.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSDictionary * result = [self attributesAtIndex: aLocation effectiveRange: aRangePtr];
    if(aRangePtr) * aRangePtr = NSIntersectionRange(rangeLimit, * aRangePtr);
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attribute:atIndex:effectiveRange:
- (id) attribute: (NSString *) name atIndex: (unsigned) aLocation effectiveRange: (NSRangePointer) aRangePtr;
/*"Subclassers will override this method for performance reasons.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self attributesAtIndex: aLocation effectiveRange: aRangePtr] objectForKey: name];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributesAtIndex:effectiveRange:
- (NSDictionary *) attribute: (NSString *) name atIndex: (unsigned) aLocation longestEffectiveRange: (NSRangePointer) aRangePtr inRange: (NSRange) rangeLimit;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self attributesAtIndex: aLocation longestEffectiveRange: aRangePtr inRange: rangeLimit] objectForKey: name];
}
@end

@implementation iTMSyntaxParser(iTMSettings)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  settings:
- (id) settings;
/*"Returns the settings of the first text view of its first layout manager.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.b0: 04/17/2002
To Do List:
"*/
{
    return [[self textStorage] settings];
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMSyntaxParser
