/*
//  iTMTeXSyntaxParser.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Fri Dec 07 2001.
//  Copyright ¬© 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSNotification.h>
#import <Foundation/NSAttributedString.h>
#import <Foundation/NSException.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSCharacterSet.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSScanner.h>
#import <AppKit/NSApplication.h>
#import <AppKit/NSAttributedString.h>
#import "NSUserDefaults(iTeXMac).h"
#import "NSString(iTeXMac).h"
#import "iTMLiteScanner.h"
#import "iTMTeXSyntaxParser.h"
#import "iTMSyntaxParser.h"

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMSyntaxParser

NSString * const iTMUDTeXStandardFontKey = @"iTMUDTeXStandardFont";
NSString * const iTMUDTeXStandardColorKey = @"iTMUDTeXStandardColor";

NSString * const iTMUDTeXRegularFontKey = @"iTMUDTeXRegularFont";
NSString * const iTMUDTeXCommandFontKey = @"iTMUDTeXCommandFont";
NSString * const iTMUDTeXCommentFontKey = @"iTMUDTeXCommentFont";
NSString * const iTMUDTeXMarkFontKey = @"iTMUDTeXMarkFont";
NSString * const iTMUDTeXDollarFontKey = @"iTMUDTeXDollarFont";
NSString * const iTMUDTeXDelimiterFontKey = @"iTMUDTeXDelimiterFont";

NSString * const iTMUDTeXRegularColorKey = @"iTMUDTeXRegularColor";
NSString * const iTMUDTeXCommandColorKey = @"iTMUDTeXCommandColor";
NSString * const iTMUDTeXCommentColorKey = @"iTMUDTeXCommentColor";
NSString * const iTMUDTeXMarkColorKey = @"iTMUDTeXMarkColor";
NSString * const iTMUDTeXDollarColorKey = @"iTMUDTeXDollarColor";
NSString * const iTMUDTeXDelimiterColorKey = @"iTMUDTeXDelimiterColor";

NSString * const iTMUDLigatureAttributeKey = @"iTMUDLigaturesMode";
NSString * const iTMUDKernAttributeKey = @"iTMUDKerningOffset";
NSString * const iTMUDRegularTextKerningOnlyKey = @"iTMUDRegularTextKerningOnly";

NSString * const iTMTeXInputAttributeName = @"I";
#define diTMIAN iTMTeXInputAttributeName


//NSString * const iTMRegularInput = @"0";
NSString * const iTMBeginCommandInput = @"-1";
NSString * const iTMCommandInput = @"1";
NSString * const iTMShortCommandInput = @"-2";
NSString * const iTMBeginCommentInput = @"-3";
NSString * const iTMBeginSuperscriptInput = @"-4";
NSString * const iTMBeginSubscriptInput = @"-5";
NSString * const iTMCommentInput = @"2";
NSString * const iTMMarkInput = @"3";
NSString * const iTMDollarInput = @"4";
NSString * const iTMDelimiterInput = @"5";

#define diTMRegularInput nil
#define kiTMRegularInput 0
#define kiTMBeginCommandInput -1
#define kiTMCommandInput 1
#define kiTMShortCommandInput -2
#define kiTMBeginCommentInput -3
#define kiTMCommentInput 2
#define kiTMMarkInput 3
#define kiTMDollarInput 4
#define kiTMDelimiterInput 5

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTeXSyntaxParser
/*"This object is the delegate of an iTMTextStorage. It manages the visual attributes corresponding to the TeX contents of attributed strings. Beware, the implementation is minimal for iTeXMac use and needs more work to be fully consistent."*/
@implementation iTMTeXSyntaxParser
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialize
+ (void) initialize;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
        [[NSFont fontWithName: @"Times" size: 14.0] nameSizeDictionary], iTMUDTeXRegularFontKey,
        [[NSFont fontWithName: @"Monaco" size: 12.0] nameSizeDictionary], iTMUDTeXCommandFontKey,
        [[NSFont fontWithName: @"Helvetica-Oblique" size: 12.0] nameSizeDictionary], iTMUDTeXCommentFontKey,
        [[NSFont fontWithName: @"Helvetica-Oblique" size: 12.0] nameSizeDictionary], iTMUDTeXMarkFontKey,
        [[NSFont fontWithName: @"Monaco" size: 9.0] nameSizeDictionary], iTMUDTeXDollarFontKey,
        [[NSFont fontWithName: @"Monaco" size: 12.0] nameSizeDictionary], iTMUDTeXDelimiterFontKey,
        [[NSColor blackColor] RGBADictionary], iTMUDTeXRegularColorKey,
        [[NSColor colorWithCalibratedRed: 0.0 green: 0.0 blue: 0.6 alpha: 1.0] RGBADictionary], iTMUDTeXCommandColorKey,
        [[NSColor colorWithCalibratedRed: 0.0 green: 0.4 blue: 0.0 alpha: 1.0] RGBADictionary], iTMUDTeXCommentColorKey,
        [[NSColor colorWithCalibratedRed: 0.5 green: 0.0 blue: 0.0 alpha: 1.0] RGBADictionary], iTMUDTeXMarkColorKey,
        [[NSColor colorWithCalibratedRed: 0.4 green: 0.1 blue: 0.4 alpha: 1.0] RGBADictionary], iTMUDTeXDollarColorKey,
        [[NSColor colorWithCalibratedRed: 0.4 green: 0.1 blue: 0.4 alpha: 1.0] RGBADictionary], iTMUDTeXDelimiterColorKey,
        [NSNumber numberWithInt: 0], iTMUDLigatureAttributeKey,
        [NSNumber numberWithFloat: 0.0], iTMUDKernAttributeKey,
        [NSNumber numberWithBool: YES], iTMUDRegularTextKerningOnlyKey,
            nil]];
    // compatibility with previous versions: some keys have changed
    {
        id object;
        #define COMP(oldKey, newKey)\
        if(object = [[NSUserDefaults standardUserDefaults] objectForKey: oldKey])\
        {\
            [[NSUserDefaults standardUserDefaults] setObject: object forKey: newKey];\
            [[NSUserDefaults standardUserDefaults] removeObjectForKey: oldKey];\
        }
        COMP(@"iTMUDRegularTeXColor", iTMUDTeXRegularColorKey);
        COMP(@"iTMUDCommandColor", iTMUDTeXCommandColorKey);
        COMP(@"iTMUDCommentColor", iTMUDTeXCommentColorKey);
        COMP(@"iTMUDMarkColor", iTMUDTeXMarkColorKey);
        COMP(@"iTMUDDollarColor", iTMUDTeXDollarColorKey);
        COMP(@"iTMUDDelimiterColor", iTMUDTeXDelimiterColorKey);
        [[NSUserDefaults standardUserDefaults] synchronize];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  syntaxParser
+ (id) syntaxParser;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSLog(@"TEX");
    return [[[self alloc]¬†init] autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fontOrColorDidChange:
- (void) fontOrColorDidChange: (NSNotification *) aNotification;
/*"The notification object is used to retrieve font and color info. If no object is given, the NSFontColorManager class object is used.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: NYI
"*/
{
    id D = [aNotification object];
    if(!D)
        D = [NSUserDefaults standardUserDefaults];
    {
        #define SETATTRIBUTES(mode, fontKey, colorKey)\
        NSDictionary * mode = [[D objectForKey: iTMUDRegularTextKerningOnlyKey] boolValue]?\
            [NSDictionary dictionaryWithObjectsAndKeys:\
                [NSFont fontWithNameSizeDictionary: [D objectForKey: fontKey]], NSFontAttributeName,\
                [NSColor colorWithRGBADictionary: [D objectForKey: colorKey]], NSForegroundColorAttributeName,\
                [D objectForKey: iTMUDLigatureAttributeKey], NSLigatureAttributeName,\
                    nil]:\
            [NSDictionary dictionaryWithObjectsAndKeys:\
                [NSFont fontWithNameSizeDictionary: [D objectForKey: fontKey]], NSFontAttributeName,\
                [NSColor colorWithRGBADictionary: [D objectForKey: colorKey]], NSForegroundColorAttributeName,\
                [D objectForKey: iTMUDLigatureAttributeKey], NSLigatureAttributeName,\
                [D objectForKey: iTMUDKernAttributeKey], NSKernAttributeName,\
                    nil]
        SETATTRIBUTES(command, iTMUDTeXCommandFontKey, iTMUDTeXCommandColorKey);
        SETATTRIBUTES(comment, iTMUDTeXCommentFontKey, iTMUDTeXCommentColorKey);
        SETATTRIBUTES(mark, iTMUDTeXMarkFontKey, iTMUDTeXMarkColorKey);
        SETATTRIBUTES(dollar, iTMUDTeXDollarFontKey, iTMUDTeXDollarColorKey);
        SETATTRIBUTES(delimiter, iTMUDTeXDelimiterFontKey, iTMUDTeXDelimiterColorKey);
        NSDictionary * regular = [NSDictionary dictionaryWithObjectsAndKeys:
                [NSFont fontWithNameSizeDictionary: [D objectForKey: iTMUDTeXRegularFontKey]], NSFontAttributeName,
                [NSColor colorWithRGBADictionary: [D objectForKey: iTMUDTeXRegularColorKey]], NSForegroundColorAttributeName,
                [D objectForKey: iTMUDLigatureAttributeKey], NSLigatureAttributeName,
                [D objectForKey: iTMUDKernAttributeKey], NSKernAttributeName,
                    nil];
        [self setAttributesArray: [NSArray arrayWithObjects:
                        regular, command, comment, mark, dollar, delimiter,
                                    nil]];
    }
//    NSLog(@"iTMTeXSyntaxParser-------------------------\n%@", [self attributesArray]);
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isRegularAtIndex:
- (BOOL) isRegularAtIndex: (unsigned) aLocation;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id MAS = [[self textStorage] mutableAttributedString];
    return (aLocation < [MAS¬†length])?
        [[MAS attribute: diTMIAN atIndex: aLocation effectiveRange: nil] intValue] == 0:
        NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isColoring
- (BOOL) isColoring;
/*"Subclasses will answer YES or NO depending on their "coloring" status.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return YES;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isITMCommentAtIndex:
- (BOOL) isITMCommentAtIndex: (unsigned) aLocation;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id MAS = [[self textStorage] mutableAttributedString];
    return (aLocation < [MAS¬†length])?
        [[MAS attribute: diTMIAN atIndex: aLocation effectiveRange: nil] intValue] == kiTMMarkInput:
        NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributesAtIndex:effectiveRange:
- (NSDictionary *) attributesAtIndex: (unsigned) aLocation
effectiveRange: (NSRangePointer) aRangePtr;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    static int level = 0;
    int mode = 0;
    id MAS = [[self textStorage] mutableAttributedString];
//    NSLog(@"BEGIN of attributesAtIndex:effectiveRange:");
    if(aLocation < [MAS¬†length])
        mode = [[MAS attribute: diTMIAN atIndex: aLocation effectiveRange: aRangePtr] intValue];
    switch(mode)
    {
        case kiTMBeginCommandInput:
        case kiTMShortCommandInput:
            mode = kiTMCommandInput;
            break;
        case kiTMBeginCommentInput:
        {
            unsigned int contentsEnd;
            [[MAS string] getLineStart: nil end: nil contentsEnd: &contentsEnd
                forRange: NSMakeRange(aLocation, 0)];
            if(contentsEnd>aLocation+1)
                mode = [[MAS attribute: diTMIAN atIndex: contentsEnd-1 effectiveRange: nil] intValue];
            else
                mode = kiTMCommentInput;
            if(aRangePtr)
                *aRangePtr = NSMakeRange(aLocation, contentsEnd - aLocation);
        }
    }
//    NSLog(@"END of attributesAtIndex:effectiveRange:");
    return [[self attributesArray] objectAtIndex: mode];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attribute:atIndex:longestEffectiveRange:inRange:
- (id) attribute: (NSString *) name atIndex: (unsigned) aLocation longestEffectiveRange: (NSRangePointer) aRangePtr inRange: (NSRange) rangeLimit;
/*"The others are not implemented. Use the primitive deep attribute getter. This method is used in the text attributes pref pane.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: No support for marks!!!
"*/
{
//    NSLog(NSStringFromSelector(_cmd));
    id MAS = [[self textStorage] mutableAttributedString];
    NSRange R;
    if(!aRangePtr)
        aRangePtr = &R;
    if([name isEqualToString: diTMIAN])
    {
        id result = [MAS attribute: name atIndex: aLocation
            longestEffectiveRange: aRangePtr inRange: rangeLimit];
        int mode = [result intValue];
        if((mode==kiTMBeginCommandInput) || (mode==kiTMCommandInput) || (mode==kiTMShortCommandInput))
        {
            unsigned int max = NSMaxRange(*aRangePtr);
            unsigned int ceiling = MIN(NSMaxRange(rangeLimit), [MAS length]);
            unsigned int floor = MAX(0, rangeLimit.location);
            unsigned int min = aRangePtr->location;
            while(max<ceiling)
            {
                int value = [[MAS attribute: name atIndex: max effectiveRange: nil] intValue];
                if((value == kiTMBeginCommandInput) || (value == kiTMCommandInput) || (mode==kiTMShortCommandInput))
                    ++max;
                else
                    break;
            }
            while(min>floor)
            {
                int value = [[MAS attribute: name atIndex: --min effectiveRange: nil] intValue];
                if((value != kiTMBeginCommandInput) && (value != kiTMCommandInput) && (mode!=kiTMShortCommandInput))
                {
                    ++min;
                    break;
                }
            }
            *aRangePtr = NSMakeRange(min, max - min);
            return iTMCommandInput;
        }
        else if(mode==kiTMBeginCommentInput)
        {
            unsigned int contentsEnd;
            [[MAS string]¬†getLineStart: nil end: nil contentsEnd: &contentsEnd
                forRange: NSMakeRange(aLocation, 0)];
            if(contentsEnd>aLocation+1)
                result = [MAS attribute: name atIndex: contentsEnd-1 effectiveRange: nil];
            else
                result = iTMCommentInput;
            if(aRangePtr)
                *aRangePtr = NSIntersectionRange(NSMakeRange(aLocation, contentsEnd - aLocation), rangeLimit);
            return result;
        }
        else
            return result;
    }
    else
        return [MAS attribute: name atIndex: aLocation longestEffectiveRange: aRangePtr inRange: rangeLimit];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fixAttributesInRange:
- (void) fixAttributesInRange: (NSRange) aRange;
/*"Description forthcoming.
Version history: 
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    static unsigned NNN = 0;
    id MAS = [[self textStorage] mutableAttributedString];
    NSString * string = [[self textStorage] string];
    aRange = NSIntersectionRange(aRange, NSMakeRange(0, [string length]));
//    NSLog(@"fixAttributesInRange: %u, %@", NNN++, NSStringFromRange(aRange));
    if(aRange.length>0)
    {
        unsigned scanLocation = aRange.location;
        unsigned top = NSMaxRange(aRange);
        id previousMode = diTMRegularInput;
        if(scanLocation > 0)
            switch([string characterAtIndex: scanLocation-1])
            {
                case '\r':
                case '\n':
                    previousMode = diTMRegularInput;
                    break;
                default:
                    previousMode = [MAS attribute: diTMIAN atIndex: scanLocation - 1 effectiveRange: nil];
                    break;
            }
        while(scanLocation < top)
        {
            unichar theChar = [string characterAtIndex: scanLocation];
            NSRange aRange = NSMakeRange(scanLocation, 1);
//            NSLog(@"-- character <%@> = %u", [NSString stringWithCharacters: &theChar length: 1], theChar);
            if(((theChar>64) && (theChar < 91)) || ((theChar>96) && (theChar < 123)))
            {
                // we are just inserting a letter
//                NSLog(@"lettre %u", scanLocation);
                switch([previousMode intValue])
                {
                    case kiTMRegularInput:
                    case kiTMShortCommandInput:
                    case kiTMDollarInput:
                    case kiTMDelimiterInput:
                        previousMode = diTMRegularInput;
                    break;
                    
                    case kiTMBeginCommandInput:
                        previousMode = iTMCommandInput;
                    break;
                    
                    case kiTMBeginCommentInput:
                        previousMode = iTMCommentInput;
                    break;
                }
            }
        #if 1
            else
            {
            // the character inserted is not a letter (unaccented!)
//                NSLog(@"pas lettre %u", scanLocation);
                switch(theChar)
                {
                    case '\\':
                        switch([previousMode intValue])
                        {
                            case kiTMRegularInput:
                            case kiTMShortCommandInput:
                            case kiTMCommandInput:
                            case kiTMDollarInput:
                            case kiTMDelimiterInput:
                                previousMode = iTMBeginCommandInput;
                            break;
                            
                            case kiTMBeginCommandInput:
                                previousMode = iTMShortCommandInput;
                            break;
                            
                            case kiTMBeginCommentInput:
                                previousMode = iTMCommentInput;
                            break;
                        }
                    break;
    
                    case '{':
                    case '}':
                        switch([previousMode intValue])
                        {
                            case kiTMRegularInput:
                            case kiTMShortCommandInput:
                            case kiTMCommandInput:
                            case kiTMDelimiterInput:
                            case kiTMDollarInput:
                                previousMode = iTMDelimiterInput;
                            break;
                            
                            case kiTMBeginCommandInput:
                                previousMode = iTMShortCommandInput;
                            break;
                            
                            case kiTMBeginCommentInput:
                                previousMode = iTMCommentInput;
                            break;
                        }
                    break;
    
                    case '$':
                        switch([previousMode intValue])
                        {
                            case kiTMRegularInput:
                            case kiTMShortCommandInput:
                            case kiTMCommandInput:
                            case kiTMDelimiterInput:
                            case kiTMDollarInput:
                                previousMode = iTMDollarInput;
                            break;
                            
                            case kiTMBeginCommandInput:
                                previousMode = iTMShortCommandInput;
                            break;
                            
                            case kiTMBeginCommentInput:
                                previousMode = iTMCommentInput;
                            break;
                        }
                    break;
    
                    case '\r':
                    case '\n':
                        previousMode = diTMRegularInput;
                    break;

                    case '%':
                        switch([previousMode intValue])
                        {
                            case kiTMRegularInput:
                            case kiTMCommandInput:
                            case kiTMShortCommandInput:
                            case kiTMDollarInput:
                            case kiTMDelimiterInput:
                            // the rest of the line should be commented out or marked
                                previousMode = iTMBeginCommentInput;
                            break;
                            
                            case kiTMBeginCommandInput:
                                previousMode = iTMShortCommandInput;
                            break;
                            
                            case kiTMBeginCommentInput:
                                previousMode = iTMCommentInput;
                            break;
                        }
                    break;
                    
                    case '!':
                        switch([previousMode intValue])
                        {
                            case kiTMRegularInput:
                            case kiTMShortCommandInput:
                            case kiTMCommandInput:
                            case kiTMDelimiterInput:
                            case kiTMDollarInput:
                                [MAS setAttributes: nil range: aRange];
                                return;
                            break;
                            
                            case kiTMBeginCommandInput:
                                previousMode = iTMShortCommandInput;
                            break;
                            
                            case kiTMBeginCommentInput:
                                previousMode = iTMMarkInput;
                            break;
                        }
                    break;

                    default:
                        switch([previousMode intValue])
                        {
                            case kiTMRegularInput:
                            case kiTMShortCommandInput:
                            case kiTMCommandInput:
                            case kiTMDelimiterInput:
                            case kiTMDollarInput:
                                previousMode = diTMRegularInput;
                            break;
                            
                            case kiTMBeginCommandInput:
                                previousMode = iTMShortCommandInput;
                            break;
                            
                            case kiTMBeginCommentInput:
                                previousMode = iTMCommentInput;
                            break;
                        }
                    break;
                }
            }
        #endif
            if(previousMode)
                [MAS addAttribute: diTMIAN value: previousMode range: aRange];
            else
                [MAS setAttributes: nil range: aRange];
//            NSLog(@"mode %u", [previousMode intValue]);
//            NSLog(@"scanLocation: %u", scanLocation);
            ++scanLocation;
        }
    }
//    NSLog(@"%d", ++NNN);
//    NSLog(@"fixAttributesInRange: END");
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  invalidRangeForCharactersInOldRange:replacedWithString:
- (NSRange) invalidRangeForCharactersInOldRange: (NSRange) aRange replacedWithString: (NSString *) aString;
/*"The changes have already been made.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned length;// = [aString length];
    NSRange result = NSMakeRange(aRange.location, 0);
    id MAS = [[self textStorage] mutableAttributedString];
//    NSLog(@"invalidRangeForCharactersInOldRange: %@", NSStringFromRange(aRange));
//    NS_DURING
    length = [aString length];
    if((aRange.length > 0) || (length > 1))
    {
	unsigned end;
	NSRange r = NSIntersectionRange(NSMakeRange(aRange.location, length),
            NSMakeRange(0, [[MAS string] length]));
	[[MAS string] getLineStart: nil end: &end contentsEnd: nil forRange: r];
	result = NSMakeRange(aRange.location, end - aRange.location);
    }
    else if(length > 0)
    {
	unsigned location = aRange.location;
	unsigned end = location + 1;
        NSString * string = [MAS string];
	NSRange r;
        return NSMakeRange(aRange.location, 1);
	switch([aString characterAtIndex: 0])
	{
	    case '\r':
	    case '\n':
            {
                unsigned start, nextEnd;
		[string getLineStart: &start end: &end contentsEnd: nil forRange: NSMakeRange(location, 0)];
//		if(location > 0) --location;
		[string getLineStart: nil end: &nextEnd contentsEnd: nil forRange: NSMakeRange(end, 0)];
		result = NSMakeRange(start, nextEnd - start);
		break;
            }
	    case '!':
            if((location>0) &&
    ([[MAS attribute: diTMIAN atIndex: --location effectiveRange: nil] intValue] == kiTMBeginCommentInput))
            {
		[string getLineStart: nil end: &end contentsEnd: nil forRange: NSMakeRange(location, 0)];
		result = NSMakeRange(location, end - location);
		break;
            }
	    case '%':
		r = NSMakeRange(location, 1);
		[string getLineStart: nil end: &end contentsEnd: nil forRange: r];
		if(location > 0) --location;
		result = NSMakeRange(location, end - location);
		break;
	    case '\\':
		r = NSMakeRange(location, 1);
		[string getLineStart: nil end: &end contentsEnd: nil forRange: r];
		if(location > 0) --location;
		result = NSMakeRange(location, end - location);
		break;
	    case '$':
		r = NSMakeRange(location, 1);
		[string getLineStart: nil end: &end contentsEnd: nil forRange: r];
		if(location > 0) --location;
		result = NSMakeRange(location, end - location);
		break;
//	    case '^':
//	    case '_':
	    default:
		result = NSMakeRange(location, 1);
		break;
	}
    }
//    NS_HANDLER
//    NSLog(@"BOUH CPABO");
//    NS_ENDHANDLER
//    NSLog(@"invalidRangeForCharactersInOldRange: %@", NSStringFromRange(result));
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= replaceCharactersInRange:withString:invalidRange:
- (void) replaceCharactersInRange: (NSRange) aRange withString: (NSString *) aString invalidRange: (NSRangePointer) rangePtr;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned length = [aString length];
    NSMutableAttributedString * MAS = [[self textStorage] mutableAttributedString];
    if((aRange.length != 0) || (length != 1))
    {
        [MAS replaceCharactersInRange: aRange withString: aString];
        [[self textStorage] edited: NSTextStorageEditedCharacters range: aRange changeInLength: length - aRange.length];
        if(rangePtr)
        {
            unsigned end;
            aRange.length = [aString length];
            // this is a poor man solution for the '\r\n' problem (windows problem)
            if(NSMaxRange(aRange) < [MAS length])
                ++aRange.length;
            [[[self textStorage] string] getLineStart: nil end: &end contentsEnd: nil forRange: aRange];
            *rangePtr = NSMakeRange(aRange.location, end - aRange.location);
        }
        return;
    }
    else
    {
        id previousMode = aRange.location? [MAS attribute: diTMIAN atIndex: aRange.location-1 effectiveRange: nil]: nil;
        unichar theChar = [aString characterAtIndex: 0];
        [MAS replaceCharactersInRange: aRange withString: aString];
        [[self textStorage] edited: NSTextStorageEditedCharacters range: aRange changeInLength: 1];
        aRange.length = 1;
        if(((theChar>64) && (theChar < 91)) || ((theChar>96) && (theChar < 123)))
        {
            // we are just inserting a letter
             switch([previousMode intValue])
            {
                case kiTMRegularInput:
                case kiTMShortCommandInput:
                case kiTMDollarInput:
                case kiTMDelimiterInput:
                    [MAS setAttributes: nil range: aRange];
                    return;
                break;
                
                case kiTMBeginCommandInput:
                    previousMode = iTMCommandInput;
                break;
                
                case kiTMBeginCommentInput:
                    if(rangePtr)
                    {
                        unsigned int end;
                        [[MAS string] getLineStart: nil end: &end contentsEnd: nil forRange: aRange];
                        rangePtr->location = aRange.location;
                        rangePtr->length = end - aRange.location;
                    }
                    previousMode = iTMCommentInput;
                break;
            }
            [MAS addAttribute: diTMIAN value: previousMode range: aRange];
//            [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
            return;
        }
    #if 1
        else
        {
        // the character inserted is not a letter (unaccented!)
            switch(theChar)
            {
                case '\\':
                    switch([previousMode intValue])
                    {
                        case kiTMRegularInput:
                        case kiTMShortCommandInput:
                        case kiTMCommandInput:
                        case kiTMDollarInput:
                        case kiTMDelimiterInput:
                            previousMode = iTMBeginCommandInput;
                        break;
                        
                        case kiTMBeginCommandInput:
                            previousMode = iTMShortCommandInput;
                        break;
                        
                        
                        case kiTMBeginCommentInput:
                            previousMode = iTMCommentInput;
                        break;
                        
                        case kiTMCommentInput:
                        case kiTMMarkInput:
                            [MAS addAttribute: diTMIAN value: previousMode range: aRange];
                            return;
                        break;
                    }
                    // except for marks and comments, the \ can really change the meaning of the rest of the line
                    if(rangePtr)
                    {
                        unsigned int end;
                        [[MAS string] getLineStart: nil end: &end contentsEnd: nil forRange: aRange];
                        rangePtr->location = aRange.location;
                        rangePtr->length = end - aRange.location;
                    }
                break;

                case '%':
                    switch([previousMode intValue])
                    {
                        case kiTMRegularInput:
                        case kiTMCommandInput:
                        case kiTMShortCommandInput:
                        case kiTMDollarInput:
                        case kiTMDelimiterInput:
                        // the rest of the line should be commented out or marked
                            {
                                NSString * S = [MAS string];
                                unsigned int end;
                                [MAS addAttribute: diTMIAN value: iTMBeginCommentInput range: aRange];
                                [S getLineStart: nil end: &end contentsEnd: nil forRange: aRange];
                                ++aRange.location;
                                if((aRange.location<end) && ([S characterAtIndex: aRange.location] == '!'))
                                    previousMode = iTMMarkInput;
                                else
                                    previousMode = iTMCommentInput;
                                aRange.length = end - aRange.location;
                                [MAS addAttribute: diTMIAN value: previousMode range: aRange];
                                [[self textStorage] invalidateAttributesInRange: aRange];
                                return;
                            }
                        break;
                        
                        case kiTMBeginCommandInput:
                            previousMode = iTMShortCommandInput;
                            if(rangePtr)
                            {
                                unsigned int end;
                                [[MAS string] getLineStart: nil end: &end contentsEnd: nil forRange: aRange];
                                rangePtr->location = aRange.location;
                                rangePtr->length = end - rangePtr->location;
                            }
                        break;
                        
                        case kiTMBeginCommentInput:
                            if(rangePtr)
                            {
                                unsigned int end;
                                [[MAS string] getLineStart: nil end: &end contentsEnd: nil forRange: aRange];
                                rangePtr->location = aRange.location;
                                rangePtr->length = end - aRange.location;
                            }
                            previousMode = iTMCommentInput;
                        break;
                    }
                break;
                
                case '{':
                case '}':
                    switch([previousMode intValue])
                    {
                        case kiTMRegularInput:
                        case kiTMShortCommandInput:
                        case kiTMCommandInput:
                        case kiTMDelimiterInput:
                        case kiTMDollarInput:
                            previousMode = iTMDelimiterInput;
                        break;
                        
                        case kiTMBeginCommandInput:
                            previousMode = iTMShortCommandInput;
                        break;
                        
                        case kiTMBeginCommentInput:
                            if(rangePtr)
                            {
                                unsigned int end;
                                [[MAS string] getLineStart: nil end: &end contentsEnd: nil forRange: aRange];
                                rangePtr->location = aRange.location;
                                rangePtr->length = end - aRange.location;
                            }
                            previousMode = iTMCommentInput;
                        break;
                    }
                break;

                case '$':
                    switch([previousMode intValue])
                    {
                        case kiTMRegularInput:
                        case kiTMShortCommandInput:
                        case kiTMCommandInput:
                        case kiTMDelimiterInput:
                        case kiTMDollarInput:
                            previousMode = iTMDollarInput;
                        break;
                        
                        case kiTMBeginCommandInput:
                            previousMode = iTMShortCommandInput;
                        break;
                        
                        case kiTMBeginCommentInput:
                            if(rangePtr)
                            {
                                unsigned int end;
                                [[MAS string] getLineStart: nil end: &end contentsEnd: nil forRange: aRange];
                                rangePtr->location = aRange.location;
                                rangePtr->length = end - aRange.location;
                            }
                            previousMode = iTMCommentInput;
                        break;
                    }
                break;

                case '!':
                    switch([previousMode intValue])
                    {
                        case kiTMRegularInput:
                        case kiTMShortCommandInput:
                        case kiTMCommandInput:
                        case kiTMDelimiterInput:
                        case kiTMDollarInput:
                            [MAS setAttributes: nil range: aRange];
                            return;
                        break;
                        
                        case kiTMBeginCommandInput:
                            previousMode = iTMShortCommandInput;
                        break;
                        
                        case kiTMBeginCommentInput:
                            if(rangePtr)
                            {
                                unsigned int end;
                                [[MAS string] getLineStart: nil end: &end contentsEnd: nil forRange: aRange];
                                rangePtr->location = aRange.location-1;// the location > 0
                                rangePtr->length = end - rangePtr->location;
                            }
                            previousMode = iTMMarkInput;
                        break;
                    }
                break;

                case '\r':
                case '\n':
                    if(rangePtr)
                    {
                        unsigned contentsEnd, end;
                        NSString * S = [MAS string];
                        [S getLineStart: &(rangePtr->location)
                            end: &end contentsEnd: &contentsEnd forRange: aRange];
                        [MAS setAttributes: nil range: NSMakeRange(contentsEnd, end - contentsEnd)];
                        [S getLineStart: nil end: &end contentsEnd: nil forRange: NSMakeRange(end, 0)];
                        rangePtr->length = end - rangePtr->location;
                        return;                        
                    }
                    else
                    {
                        unsigned contentsEnd, end;
                        [[MAS string] getLineStart: nil end: &end contentsEnd: &contentsEnd forRange: aRange];
                        [MAS setAttributes: nil range: NSMakeRange(contentsEnd, end - contentsEnd)];
                        return;                        
                    }
                break;

                default:
                    switch([previousMode intValue])
                    {
                        case kiTMRegularInput:
                        case kiTMShortCommandInput:
                        case kiTMCommandInput:
                        case kiTMDelimiterInput:
                        case kiTMDollarInput:
                            [MAS setAttributes: nil range: aRange];
                            return;
                        break;
                        
                        case kiTMBeginCommandInput:
                            previousMode = iTMShortCommandInput;
                        break;
                        
                        case kiTMBeginCommentInput:
                            if(rangePtr)
                            {
                                unsigned int end;
                                [[MAS string] getLineStart: nil end: &end contentsEnd: nil forRange: aRange];
                                rangePtr->location = aRange.location-1;// the location > 0
                                rangePtr->length = end - rangePtr->location;
                            }
                            previousMode = iTMCommentInput;
                        break;
                    }
                break;

#if 0
                case 0:
                    switch([previousMode intValue])
                    {
                        case kiTMRegularInput:
                        break;
                        
                        case kiTMBeginCommandInput:
                        break;
                        
                        case kiTMShortCommandInput:
                        break;
                        
                        case kiTMCommandInput:
                        break;
                        
                        case kiTMBeginCommentInput:
                            if(rangePtr)
                            {
                                unsigned int end;
                                [[MAS string] getLineStart: nil end: &end contentsEnd: nil forRange: aRange];
                                rangePtr->location = aRange.location;
                                rangePtr->length = end - aRange.location;
                            }
                            previousMode = iTMCommentInput;
                        break;
                        
                        case kiTMDollarInput:
                        break;
                        
                        case kiTMDelimiterInput:
                        break;                
                    }
                break;
#endif
            }
            if(previousMode)
                [MAS addAttribute: diTMIAN value: previousMode range: aRange];
            else
                [MAS setAttributes: nil range: aRange];
            return;
        }
    #endif
    }
    return;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTeXSyntaxParser

#if 0
    if(aRange.length>0)
    {
        unsigned scanLocation = aRange.location;
        unsigned top = NSMaxRange(aRange);
        id previousMode = diTMRegularInput;
        [MAS setAttributes: nil range: aRange];
        #if 1
        if(scanLocation > 0)
            switch([string characterAtIndex: scanLocation-1])
            {
                case '\r':
                case '\n':
                    previousMode = diTMRegularInput;
                    break;
                default:
                    previousMode = [MAS attribute: diTMIAN atIndex: scanLocation - 1 effectiveRange: nil];
                    break;
            }
        #endif
        while(scanLocation < top)
        {
            unichar theChar = [string characterAtIndex: scanLocation];
            NSRange range = NSMakeRange(scanLocation, 1);
            [MAS setAttributes: nil range: aRange];
            switch([previousMode intValue])
            {
                case kiTMRegularInput:
                case kiTMShortCommandInput:
                case kiTMDelimiterInput:
                case kiTMDollarInput:
                    switch(theChar)
                    {
                        case '%':
                            previousMode = iTMBeginCommentInput;
                            break;
                        case '\\':
                            previousMode = iTMBeginCommandInput;
                            break;
                        case '$':
                            previousMode = iTMDollarInput;
                            break;
                        case '{':
                        case '}':
                            previousMode = iTMDelimiterInput;
                            break;
                        default:
                            previousMode = diTMRegularInput;
                            break;
                    }
                    break;
                case kiTMCommentInput:
                case kiTMMarkInput:
                    switch(theChar)
                    {
                        case '\r':
                        case '\n':
                            previousMode = diTMRegularInput;
                            break;
                        default:
                            break;
                    }
                    break;
                case kiTMBeginCommentInput:
                    switch(theChar)
                    {
                        case '\r':
                        case '\n':
                            previousMode = diTMRegularInput;
                            break;
                        case '!':
                            previousMode = iTMMarkInput;
                            break;
                        default:
                            previousMode = iTMCommentInput;
                            break;
                    }
                    break;
                // previous mode is not regular, nor comment, nor mark, nor begin comment.
                // so, command, begin command, delimiter, dollar
                default:
                    switch(theChar)
                    {
                        case '\r':
                        case '\n':
                            previousMode = diTMRegularInput;
                            break;
                        case '%':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    previousMode = iTMShortCommandInput;
                                    break;
                                case kiTMBeginCommentInput:
                                    previousMode = iTMCommentInput;
                                    break;
                                default:
                                    previousMode = iTMBeginCommentInput;
                                    break;
                            }
                            break;
                        case '\\':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    previousMode = iTMShortCommandInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    break;
                                case kiTMBeginCommentInput:
                                     previousMode = iTMCommentInput;
                                     break;
                                default:
                                    previousMode = iTMBeginCommandInput;
                                    break;
                            }
                            break;
                        case '$':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    previousMode = iTMShortCommandInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    [MAS addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                                case kiTMBeginCommentInput:
                                    [MAS addAttribute: diTMIAN value: iTMCommentInput range: range];
                                    break;
                                default:
                                    [MAS addAttribute: diTMIAN value: iTMDollarInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                            }
                            break;
                    #if 0
                        case '^':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [MAS addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    [MAS addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                                case kiTMBeginCommentInput:
                                    [MAS addAttribute: diTMIAN value: iTMCommentInput range: range];
                                    break;
                                default:
                                    [MAS addAttribute: diTMIAN value: iTMBeginSuperscriptInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                            }
                            break;
                        case '_':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [MAS addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    [MAS addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                                case kiTMBeginCommentInput:
                                    [MAS addAttribute: diTMIAN value: iTMCommentInput range: range];
                                    break;
                                default:
                                    [MAS addAttribute: diTMIAN value: iTMBeginSubscriptInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                            }
                            break;
                    #endif
                        case '{':
                        case '}':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [MAS addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    [MAS addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                                case kiTMBeginCommentInput:
                                    [MAS addAttribute: diTMIAN value: iTMCommentInput range: range];
                                    break;
                                default:
                                    [MAS addAttribute: diTMIAN value: iTMDelimiterInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                            }
                            break;
                        default:
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    if([[NSCharacterSet letterCharacterSet] characterIsMember: theChar] || (theChar == '@'))
                                    {
                                        [MAS addAttribute: diTMIAN value: iTMCommandInput range: range];
                                        previousMode = iTMCommandInput;
                                    }
                                    else
                                    {
                                        [MAS addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                        previousMode = diTMRegularInput;
                                    }
                                    break;
                                default:
                                    if(![[NSCharacterSet letterCharacterSet] characterIsMember: theChar] &&
                                            (theChar != '@'))
                                        previousMode = diTMRegularInput;
                                    if(previousMode)
                                        [MAS addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                            }
                            break;
                        }
                    break;
            }
            if(previousMode)
                [MAS addAttribute: diTMIAN value: previousMode range: range];
            else
                [MAS sAttributes: nil range: range];
            ++scanLocation;
        }
    }
#endif
