/*
//  iTMTextStorage(TeXLogFile).m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Oct 16 2001.
//  Copyright  2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import "iTMTextStorage(TeXLogFile).h"
#import "iTMLogSyntaxParser.h"
#import "iTMLiteScanner.h"

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTextStorage(TeXLogFile)
/*"Description forthcoming."*/
@implementation iTMTextStorage(TeXLogFile)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= texDocumentPathAtIndex:
- (NSString *) texDocumentPathAtIndex: (unsigned) charIndex;
/*"When tex opens a file it displays a '(./foo.tex' in the log and when it closes the file, it displays a matching ')'.
The purpose of this method is to give the relative path ./foo.tex corresponding to the smallest (...) group containing charIndex. A backwards search is performed.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unichar right = ')';
    unichar left = '(';
    NSString * string = [self string];
    unsigned scanLocation = charIndex;
    int groupLevel = 1;
    unichar currentCharacter;
    typedef unichar (* CharacterAtIndexIMP) (id, SEL, unsigned);
    CharacterAtIndexIMP CAI = (CharacterAtIndexIMP) [string methodForSelector: @selector(characterAtIndex:)];
    #undef NextCharacterScanned
    #define NextCharacterScanned CAI(string, @selector(characterAtIndex:), --scanLocation)
    while (scanLocation > 0)
    {
        currentCharacter = NextCharacterScanned;
        {
            if (currentCharacter == right)
            {
                int start;
                [string getLineStart: &start end: nil contentsEnd: nil forRange: NSMakeRange(scanLocation, 0)];
                {
                    int end = MIN(start+2, scanLocation);
                    NSString * S = [string substringWithRange: NSMakeRange(start, end-start)];
                    if([S hasPrefix: @"l."] || [S hasPrefix: @" "])
                        scanLocation = start;
                    else
                    {
//                        NSLog(@"+");
                        ++groupLevel;
                    }
                }
            }
            else if (currentCharacter == left)
            {
                int start;
                [string getLineStart: &start end: nil contentsEnd: nil forRange: NSMakeRange(scanLocation, 0)];
                {
                    int end = MIN(start+2, scanLocation);
                    NSString * S = [string substringWithRange: NSMakeRange(start, end-start)];
                    if([S hasPrefix: @"l."] || [S hasPrefix: @" "])
                        scanLocation = start;
                    else
                    {
//                        NSLog(@"-");
                        --groupLevel;
                    }
                }
            }
            if (groupLevel == 0)
            {
                unsigned contentsEnd;
                [string getLineStart: nil end: nil contentsEnd: &contentsEnd forRange: NSMakeRange(scanLocation + 1, 0)];
                {
                    iTMLiteScanner * S = [iTMLiteScanner scannerWithString: string];
		    [S setScanLocation: scanLocation + 1];
		    [S setCaseSensitive: NO];
                    if([S scanUpToString: @".tex" intoString: nil] && [S scanString: @".tex" intoString: nil])
                    	return [string substringWithRange:
			    NSMakeRange(scanLocation + 1, [S scanLocation] - scanLocation - 1)];
		    [S setScanLocation: scanLocation + 1];
                    if([S scanUpToString: @".mp" intoString: nil] && [S scanString: @".mp" intoString: nil])
                    	return [string substringWithRange:
			    NSMakeRange(scanLocation + 1, [S scanLocation] - scanLocation - 1)];
//		    ++groupLevel;
                    return [NSString string];
                }
                break;
            }
        }
    }
    return [NSString string];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= texRootDocumentPath
- (NSString *) texRootDocumentPath;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: UNUSED AND UNIMPLEMENTED
"*/
{
    NSRange R = [[self string] rangeOfString: @".tex" options: NSCaseInsensitiveSearch];
    if(R.length)
    {
        unsigned start;
        [[self string] getLineStart: &start end: nil contentsEnd: nil forRange: R];
        if(start+1<R.location)
            return [[self string] substringWithRange: NSMakeRange(start, R.location-start)];
    }
    return [NSString string];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= rangeOfErrorFromIndex:cycle:backwards:
- (NSRange) rangeOfErrorFromIndex: (unsigned) charIndex cycle: (BOOL) cycle backwards: (BOOL) backwards;
/*"Returns the range of the first error after index charIndex...
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: does not work well because the attributes are not stored well by fixAttributes...
"*/
{
    return [[self syntaxParser] attributedString: [self mutableAttributedString] rangeOfErrorFromIndex: charIndex cycle: cycle backwards: backwards];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= markVisitedLinkAtIndex:
- (void) markVisitedLinkAtIndex: (unsigned) charIndex;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSRange linkRange;
    NSDictionary * attributes = [self attributesAtIndex: charIndex effectiveRange: &linkRange];
    NSColor * visitedLinkColor = [[attributes objectForKey: NSForegroundColorAttributeName] shadowWithLevel: 0.5];
    NSEnumerator * enumerator = [[self layoutManagers] objectEnumerator];
    NSLayoutManager * LM;
    while(LM = [enumerator nextObject])
    {
        [LM addTemporaryAttributes:
            [NSDictionary dictionaryWithObject: visitedLinkColor forKey: NSForegroundColorAttributeName]
                forCharacterRange: linkRange];
        [[LM firstTextView] display];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= errorStringAtIndex:
- (NSString *) errorStringAtIndex: (unsigned) charIndex;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSRange R;
    NSString * S = [self string];
    charIndex = MIN(charIndex, [S length]);
    R = [S rangeOfString: @"\n! " options: NSBackwardsSearch range: NSMakeRange(0, charIndex)];
    if(R.length)
    {
        int contentsEnd = 0;
        int end = 0;
        NSString * result;
        [S getLineStart: nil end: &end contentsEnd: &contentsEnd forRange: NSMakeRange(R.location + 3, 0)];
        result = [S substringWithRange: NSMakeRange(R.location + 3, contentsEnd - R.location - 3)];
        [S getLineStart: nil end: nil contentsEnd: &contentsEnd forRange: NSMakeRange(end, 0)];
        if(contentsEnd > end)
        {
            R = [S rangeOfString: @" " options: 0 range: NSMakeRange(end, contentsEnd - end)];
            if(R.length)
                return [NSString stringWithFormat: @"%@: %@", result,
                                [S substringWithRange: NSMakeRange(R.location, contentsEnd - R.location)]];
        }
        return result;
    }
    return [NSString string];
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTextStorage(TeXLogFile)

