/*
//  iTMTextStorage(iTMAnchors).m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Wed Mar 20 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSException.h>
#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "NSString(iTeXMac).h"
#import "iTMTextStorage(iTMAnchors).h"
#import "iTMLiteScanner.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTextStorage(iTMAnchors)
/*"Extension of the iTMTextStorage to parse the model for \section's, \label's...."*/

@interface iTMAnchorP: NSObject
{
@private
    NSString * _Title;
    NSRange _Range;
    int _Offset;
}
+ (id) anchorWithTitle: (NSString *) title range: (NSRange) range;
- (id) initWithTitle: (NSString *) title range: (NSRange) range;
- (void) dealloc;
- (NSString *) title;
- (void) setTitle: (NSString *) argument;
- (NSRange) range;
- (NSRange) offset;
- (void) translateTo: (unsigned int) location;
- (void) translateBy: (int) offset;
- (void) fixLocation;
@end

@interface iTMAnchorsController: NSObject
{
@private
    NSMutableArray * _Anchors;
}
- (void) textStorageDidProcessEditing: (NSNotification *) aNotification;
- (unsigned int) addAnchorsAtIndex: (unsigned int) idx parsingTextStorage: (iTMTextStorage *) TS inRange: (NSRange) aRange;
- (NSMutableArray *) anchors;
- (void) setAnchors: (NSMutableArray *) argument;
@end

@implementation iTMTextStorage(iTMAnchors)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initAnchorsControllers
- (void) initAnchorsControllers;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSLog(@"initAnchorsControllers");
    [self setInputAnchorsController: [[[iTMAnchorsController alloc] init] autorelease]];
    [self setMarkAnchorsController: [[[iTMAnchorsController alloc] init] autorelease]];
    [self setSectionAnchorsController: [[[iTMAnchorsController alloc] init] autorelease]];
    [self setLabelAnchorsController: [[[iTMAnchorsController alloc] init] autorelease]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  inputAnchorsController
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  markAnchorsController
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sectionAnchorsController
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  labelAnchorsController
#define GETTERSETTER(source, getter, setter)\
- (id) getter;\
{\
    return source;\
}\
- (void) setter: (id) argument;\
{\
    if(![source isEqual: argument])\
    {\
        if(source)\
        {\
            [[NSNotificationCenter defaultCenter]\
                removeObserver: source\
                    name: NSTextStorageDidProcessEditingNotification\
                        object: self];\
        }\
        [source autorelease];\
        source = [argument retain];\
        if([source respondsToSelector: @selector(textStorageDidProcessEditing:)])\
        {\
            [[NSNotificationCenter defaultCenter]\
                addObserver: source\
                    selector: @selector(textStorageDidProcessEditing:)\
                    name: NSTextStorageDidProcessEditingNotification\
                        object: self];\
        }\
    }\
    return;\
}
/*"All of them are mutable arrays of restricted use (iTMTextStorage only).
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
GETTERSETTER(_IAC, inputAnchorsController, setInputAnchorsController);
GETTERSETTER(_MAC, markAnchorsController, setMarkAnchorsController);
GETTERSETTER(_SAC, sectionAnchorsController, setSectionAnchorsController);
GETTERSETTER(_LAC, labelAnchorsController, setLabelAnchorsController);
@end

@implementation iTMAnchorP
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  anchorWithTitle:range:
+ (id) anchorWithTitle: (NSString *) string range: (NSRange) range;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[self alloc]initWithTitle: string range: range] autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initWithTitle:range:
- (id) initWithTitle: (NSString *) string range: (NSRange) range;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(self = [super init])
    {
        _Range = range;
        [self setTitle: string];
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setTitle: nil];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  title
- (NSString *) title;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _Title;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setTitle
- (void) setTitle: (NSString *) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [_Title autorelease];
    argument = [argument stringByRemovingTrailingWhites];
    _Title = ([argument length] > 48)?
                    [NSString stringWithFormat: @"%@[...]",
                            [argument substringWithRange: NSMakeRange(0,43)]]: argument;
    [_Title retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  range
- (NSRange) range;
/*"The range is the effective range that corresponds to the receiver. If the string from which it is derived changes, the range must also be changed accordingly.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _Range;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  translateTo:
- (void) translateTo: (unsigned int) location;
/*"Change the location to the good one.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _Range.location = location;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  translateBy:
- (void) translateBy: (int) offset;
/*"Offset the actual location. location is cached but the receiver's range will only be exact once the location has been fixed.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _Range.location += offset;
//    _Offset += offset;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fixLocation
- (void) fixLocation;
/*"The range is the effective range that corresponds to the receiver. If the string from which it is derived changes, the range must also be changed accordingly.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _Range.location += _Offset;
    _Offset = 0;
    return;
}
@end

@implementation iTMAnchorsController
/*"Attention: this class is private and controlled by the iTMTextStorage (problems with notifications).
The objects are instances of iTMAnchorP, they are supposed to be srored in ascending order with respect to their location."*/
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"Updating the stuff.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(self = [super init])
        [self setAnchors: [NSArray array]];// mutable copied
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Updating the stuff.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter]removeObserver: self];
    [self setAnchors: nil];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textStorageDidProcessEditing:
- (void) textStorageDidProcessEditing: (NSNotification *) aNotification;// will process?
/*"Updating the stuff.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMTextStorage * TS = [aNotification object];
    if([TS editedMask] & NSTextStorageEditedCharacters)
    {
        NSRange editedRange = [TS editedRange];
        int changeInLength = [TS changeInLength];
        unsigned idx = 0;
        NSMutableArray * anchors = [self anchors];
        // all items which whole range is before the edited range location are unchanged
        while((idx<[anchors count]) && (NSMaxRange([[anchors objectAtIndex: idx] range]) <= editedRange.location))
            ++idx;
        // all items which range intersects the edited range should be properly removed
        while(idx<[anchors count])
        {
            if(NSIntersectionRange([[anchors objectAtIndex: idx] range], editedRange).length)
                [anchors removeObjectAtIndex: idx];
            else
                break;
        }
        // parse for new items:
        {
            unsigned start = idx>0? NSMaxRange([[anchors objectAtIndex: idx-1] range]): 0;
            unsigned end = NSMaxRange(editedRange) + changeInLength;
            // the changes have been made...
            idx = [self addAnchorsAtIndex: idx parsingTextStorage: TS inRange: NSMakeRange(start, end - start)];
        }
        // all remaining items are after the edited range, they shoul be offset.
        while(idx<[anchors count])
            [[anchors objectAtIndex: idx] translateBy: changeInLength];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  addAnchorsAtIndex:parsingTextStorage:inRange:
- (unsigned int) addAnchorsAtIndex: (unsigned int) idx parsingTextStorage: (iTMTextStorage *) TS inRange: (NSRange) aRange;
/*"The returned value is the next index following all the items added.
Subclasses will add their own method: the default implementation just returns idx.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return idx;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  anchors
- (NSMutableArray *) anchors;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _Anchors;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setAnchors:
- (void) setAnchors: (NSMutableArray *) argument;
/*"Stores a mutable copy af the argument. Exception when argument is not an NSArray, except nil of course.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument &&![argument isKindOfClass: [NSArray class]])
        NSLog(@"-[%@ %@] NSMutableArray expected, got %@", [self class], NSStringFromSelector(_cmd), argument);
    else
    {
        [_Anchors autorelease];
        _Anchors = [argument mutableCopy];
    }
    return;
}
@end
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  addAnchorsAtIndex:parsingTextStorage:inRange:
- (unsigned int) addAnchorsAtIndex: (unsigned int) idx parsingTextStorage: (iTMTextStorage *) TS inRange: (NSRange) aRange;
/*"The returned value is the next index following all the items added.
Subclasses will add their own method: the default implementation just returns idx.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMutableArray * anchors = [self anchors];
    NSAssert2((idx<=[anchors count]),
        @"-[iTMInputAnchorsP addAnchorsAtIndex:parsingTextStorage:inRange:] invalide parameter not satisfying %d <= %d",
            idx, [anchors count]);
    {
        NSString * bgroup = @"{";
        NSString * egroup = @"}";
        NSString * title;
        iTMLiteScanner * S = [iTMLiteScanner scannerWithString: [TS string]];
        unsigned int top = NSMaxRange(aRange);
        [S setScanLocation: aRange.location];
        while(([S scanLocation]<top) && [S scanString: @"\\in" intoString: nil])
        {
            unsigned int lengthCorrection = 0;// 1 if a closing group char must be read (include case)
            unsigned int scanLocation = [S scanLocation] - 3;// is it strong?
            if(([S scanString: @"put " intoString: nil]&&// the space is important
                        [S scanCharactersFromSet: [NSCharacterSet letterCharacterSet] intoString: &title]) ||
                ([S scanString: @"clude" intoString: nil] &&
            (lengthCorrection=1, [S scanString: bgroup intoString: nil] && [S scanUpToString: egroup intoString: &title])))
            {
                unsigned int length = [S scanLocation] + lengthCorrection - scanLocation;
                [anchors insertObject:
                    [iTMAnchorP anchorWithTitle: title range: NSMakeRange(scanLocation, length)]
                        atIndex: idx];
                ++idx;
            }
        }
    }
    return idx;
}
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTextStorage(iTMAnchors)

