/*
//  iTMTextView.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Oct 16 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSArray.h>
#import <Foundation/Foundation.h>
#import <AppKit/NSTextStorage.h>
#import <AppKit/NSPasteboard.h>
#import <AppKit/AppKit.h>
#import "NSString(iTeXMac).h"
#import "NSTextView(iTeXMac).h"
#import "iTMTextWatcher.h"
#import "iTMTextView.h"
#import "iTMTextStorage.h"
#import "iTMSettings.h"
#import "iTMStatusNotification.h"

NSString * const iTMUDAutoIndentNewlineKey = @"iTMUDAutoIndentNewline";
NSString * const iTMUDSmartInsertKey = @"iTMUDSmartInsert";
NSString * const iTMUDUseSpacesAsTabsKey = @"iTMUDUseSpacesAsTabs";
NSString * const iTMUDNumberOfSpacesForTabKey = @"iTMUDNumberOfSpacesForTab";
NSString * const iTMUDShiftDeleteKey = @"iTMUDShiftDelete";
NSString * const iTMUDSmartInsertDeleteEnabledKey = @"iTMUDSmartInsertDeleteEnabled";

@interface NSTextStorage(iTMTextView_Private)
- (void) cleanAttributesInRange: (NSRange) R;
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTextView
/*"The only purpose of this posed as NSTextView class is to provide a different changeFont and changeColor behaviour.
The message is send to the receiver and forwarded to the next responder. iTeXMac does not use the text view straight in the way the class was originally designed."*/
@implementation iTMTextView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Default Defaults settings.
Version history: jerome.laurens@u-bourgogne.fr
- 1.1.a10: 04/04/2002
To Do List: Nothing at first glance.
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults:
        [NSDictionary dictionaryWithObjectsAndKeys:
            [NSNumber numberWithBool: YES], iTMUDAutoIndentNewlineKey,
            [NSNumber numberWithBool: YES], iTMUDSmartInsertKey,
            [NSNumber numberWithBool: YES], iTMUDShiftDeleteKey,
            [NSNumber numberWithBool: NO], iTMUDUseSpacesAsTabsKey,
            [NSNumber numberWithInt: 8], iTMUDNumberOfSpacesForTabKey,
                nil]];
    [iTMTextWatcher sharedTextWatcher];// initialize as side effect
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  awakeFromNib
- (id) initWithFrame: (NSRect) frameRect textContainer: (NSTextContainer *) container;
/*"Designated Initializer. container may be nil.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 04/04/2002
To Do List: ...
"*/
{
    if(self = [super initWithFrame: frameRect textContainer: container])
    {
        [self awakeFromNib];
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  awakeFromNib
- (void) awakeFromNib;
/*"Designated Initializer. container may be nil.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 04/04/2002
To Do List: ...
"*/
{
    _iTMTVFlags.smartInsert = [[self settings] boolForKey: iTMUDSmartInsertKey]? -1: 0;
    _iTMTVFlags.smartInsertIsGlobal = -1;
    _iTMTVFlags.autoIndent = [[self settings] boolForKey: iTMUDAutoIndentNewlineKey]? -1: 0;
    _iTMTVFlags.autoIndentIsGlobal = -1;
    _iTMTVFlags.shiftDelete = [[self settings] boolForKey: iTMUDShiftDeleteKey]? -1: 0;
    _iTMTVFlags.shiftDeleteIsGlobal = -1;
    [self setRichText: NO];
    [self setSmartInsertDeleteEnabled: YES];
    [[NSNotificationCenter defaultCenter]
        addObserver: self
            selector: @selector(userDefaultsDidChange:)
                name: NSUserDefaultsDidChangeNotification
                    object: nil];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Designated Initializer. container may be nil.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 04/04/2002
To Do List: ...
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  userDefaultsDidChange:
- (void) userDefaultsDidChange: (NSNotification *) aNotification;
/*"Designated Initializer. container may be nil.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 04/04/2002
To Do List: ...
"*/
{
    if(_iTMTVFlags.smartInsertIsGlobal)
        _iTMTVFlags.smartInsert = [[aNotification object] boolForKey: iTMUDSmartInsertKey]? 1: 0;
    if(_iTMTVFlags.autoIndentIsGlobal)
        _iTMTVFlags.autoIndent = [[aNotification object] boolForKey: iTMUDAutoIndentNewlineKey]? 1: 0;
    if(_iTMTVFlags.shiftDeleteIsGlobal)
        _iTMTVFlags.shiftDelete = [[aNotification object] boolForKey: iTMUDShiftDeleteKey]? 1: 0;
    return;
}
#if 1
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  insertText:
- (void) insertText: (id) argument;
/*"Only strings are accepted.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing at first glance.
"*/
{
//    NSLog(@"<%@>", argument);
    if(_iTMTVFlags.tempAttrArePending != 0)
        [[self layoutManager] removeTemporaryAttribute: NSBackgroundColorAttributeName
            forCharacterRange: NSMakeRange(0, [[self string] length])];
    _iTMTVFlags.tempAttrArePending = 0;
    
    if([argument isKindOfClass: [NSAttributedString class]])
        argument = [argument string];
//    NSLog(@"<%@>", argument);
    if(([self selectedRange].length == 0) && ([argument length] == 1))
    {
        if(!_iTMTVFlags.isEscaped)
        {
            NSRange R = [self selectedRange];
            if(_iTMTVFlags.smartInsert)
            {
                switch([argument characterAtIndex: 0])
                {
                    case '{'://}
                    {
                        BOOL escaped;
                        NSString * S = [self string];
                        int index = [self selectedRange].location;
                        if(!index || ![S isBackslashAtIndex: index-1 escaped: &escaped] || escaped)
                        {
                            NSRange R = [self selectedRange];
                            if(R.length)
                            {
                                [super insertText: [NSString stringWithFormat: @"{%@}", [[self string] substringWithRange: R]]];
                            }
                            else
                            {
                                [super insertText: @"{}"];
                                [self setSelectedRange: NSMakeRange(R.location+1, 0)];
                            }
                        }
                        else
                            [super insertText: argument];
                        _iTMTVFlags.isEscaped = 0;
                        [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: nil];
                        return;
                    }
                    case '['://]
                    {
                        BOOL escaped;
                        NSString * S = [self string];
                        int index = [self selectedRange].location;
                        if(!index || ![S isBackslashAtIndex: index-1 escaped: &escaped] || escaped)
                        {
                            NSRange R = [self selectedRange];
                            if(R.length)
                            {
                                [super insertText: [NSString stringWithFormat: @"[%@]", [[self string] substringWithRange: R]]];
                            }
                            else
                            {
                                [super insertText: @"[]"];
                                [self setSelectedRange: NSMakeRange(R.location+1, 0)];
                            }
                        }
                        else
                            [super insertText: argument];
                        _iTMTVFlags.isEscaped = 0;
                        [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: nil];
                        return;
                    }
                    case '('://)
                    {
                        BOOL escaped;
                        NSString * S = [self string];
                        int index = [self selectedRange].location;
                        if(!index || ![S isBackslashAtIndex: index-1 escaped: &escaped] || escaped)
                        {
                            NSRange R = [self selectedRange];
                            if(R.length)
                            {
                                [super insertText: [NSString stringWithFormat: @"(%@)", [[self string] substringWithRange: R]]];
                            }
                            else
                            {
                                [super insertText: @"()"];
                                [self setSelectedRange: NSMakeRange(R.location+1, 0)];
                            }
                        }
                        else
                            [super insertText: argument];
                        _iTMTVFlags.isEscaped = 0;
                        [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: nil];
                        return;
                    }
                    case '^':// beware of dead letters
                    case '_':
                    {
                        BOOL escaped;
                        NSString * S = [self string];
                        int index = [self selectedRange].location;
                        if(!index || ![S isBackslashAtIndex: index-1 escaped: &escaped] || !escaped)
                        {
                            [super insertText: argument];
                            [self insertText: @"{"//}
                                ];
                        }
                        else
                            [super insertText: argument];
                        _iTMTVFlags.isEscaped = 0;
                        [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: nil];
                        return;
                    }
                }
            }    
            if(_iTMTVFlags.autoIndent || (([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask) > 0))
            {
                 switch([argument characterAtIndex: 0])
                {
                    case '\n':
                    case '\r':
                    {
                        int begin = 0, contentsEnd = 0, index = 0;
                        NSString * S = [self string];
                        [super insertText: argument];
                        [S getLineStart: &begin end: nil contentsEnd: &contentsEnd forRange: NSMakeRange(R.location, 0)];
                        index = begin;
                        while(index <= contentsEnd)
                        {
                            unsigned u = [S characterAtIndex: index];
                            if(u==' ' || u=='\t')
                                ++index; 
                            else
                                break;
                        }
                        if(index>begin)
                            [self insertText: [S substringWithRange: NSMakeRange(begin, index - begin)]];
                    }
                    break;
                    
                    default:
                        [super insertText: argument];
                    break;
                }
            }
            else
            {
                [super insertText: argument];
            }
        }
        else
        {
            [super insertText: argument];
        }
    }
    else
        [super insertText: argument];
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: nil];
    _iTMTVFlags.isEscaped = 0;
    return;
}
#endif
#if 0
- (void)draggedImage:(NSImage *)anImage beganAt:(NSPoint)aPoint
{
    NSLog(@"isSmartInsertDeleteEnabled %@", ([self smartInsertDeleteEnabled]? @"YES": @"NO"));
}
#else
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  paste:
- (void) paste: (id) sender;
/*"Awfull text view. Bug catcher: cmd-A+cmd-C+cmd-V+cmd-V = forever spinning wheel
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing at first glance.
"*/
{
    NSString * type = [[NSPasteboard generalPasteboard] availableTypeFromArray:
                                                            [NSArray arrayWithObject: NSStringPboardType]];
    if([type length])
    {
        NSString * before;
        NSString * after;
        NSString * string = [[NSPasteboard generalPasteboard] stringForType: NSStringPboardType];
        if([string length])
        {
            [self smartInsertForString: string
                replacingRange: [self selectedRange]
                    beforeString: &before afterString: &after];
            if(before)
                [self insertText: before];
            [self insertText: string];
            if(after)
                [self insertText: after];
            return;
        }
    }
    else
        NSLog(@"NOTHING TO PASTE");
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  readSelectionFromPasteboard:type:
- (BOOL) readSelectionFromPasteboard: (NSPasteboard *) pboard type: (NSString *) type;
/*"Awfull text view. Bug catcher: use service from??? = forever spinning wheel
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing at first glance.
"*/
{
    if([type isEqualToString: NSFilenamesPboardType])
    {
        NSArray * names = [pboard propertyListForType: NSFilenamesPboardType];
        [self insertText: [names componentsJoinedByString: @"\n"]];
        return YES;
    }
    else if([type isEqualToString: NSStringPboardType])
    {
        NSString * before;
        NSString * after;
        NSString * string = [pboard stringForType: NSStringPboardType];
        if([string length])
        {
            [self smartInsertForString: string
                replacingRange: [self selectedRange]
                    beforeString: &before afterString: &after];
            if(before)
                [self insertText: before];
            [self insertText: string];
            if(after)
                [self insertText: after];
            return YES;
        }
    }
    else
        NSLog(@"NOTHING TO INSERT");
    return NO;
}
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleAutoIndentNewLine:
- (IBAction) toggleAutoIndentNewLine: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _iTMTVFlags.autoIndentIsGlobal = 0;
    _iTMTVFlags.autoIndent = (_iTMTVFlags.autoIndent != 0)? 0: -1;
    [[self settings] setBool: (_iTMTVFlags.autoIndent != 0) forKey: iTMUDAutoIndentNewlineKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleMatchDelimiter:
- (void) toggleMatchDelimiter: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    [[self settings]
        setObject: [NSNumber numberWithBool: ![[self settings] boolForKey: iTMUDSmartSelectionKey]]
            forKey: iTMUDMatchDelimiterKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleSmartSelection:
- (void) toggleSmartSelection: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    [[self settings]
        setObject: [NSNumber numberWithBool: ![[self settings] boolForKey: iTMUDSmartSelectionKey]]
            forKey: iTMUDSmartSelectionKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleSmartInsert:
- (IBAction) toggleSmartInsert: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _iTMTVFlags.smartInsertIsGlobal = 0;
    _iTMTVFlags.smartInsert = (_iTMTVFlags.smartInsert != 0)? 0: 1;
    [[self settings] setBool: (_iTMTVFlags.smartInsert != 0) forKey: iTMUDSmartInsertKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleShiftDelete:
- (IBAction) toggleShiftDelete: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _iTMTVFlags.shiftDeleteIsGlobal = 0;
    _iTMTVFlags.shiftDelete = (_iTMTVFlags.shiftDelete != 0)? 0: 1;
    [[self settings] setBool: (_iTMTVFlags.shiftDelete != 0) forKey: iTMUDShiftDeleteKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMenuItem:
- (BOOL) validateMenuItem: (id <NSMenuItem>) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    SEL action = [sender action];
    if(action == @selector(toggleAutoIndentNewLine:))
    {
        [sender setState: (_iTMTVFlags.autoIndent != 0)? NSOnState: NSOffState];
        return YES;
    }
    else if(action == @selector(toggleSmartInsert:))
    {
        [sender setState: (_iTMTVFlags.smartInsert != 0)? NSOnState: NSOffState];
        return YES;
    }
    else if(action == @selector(toggleShiftDelete:))
    {
        [sender setState: (_iTMTVFlags.shiftDelete != 0)? NSOnState: NSOffState];
        return YES;
    }
    else if(action == @selector(toggleMatchDelimiter:))
    {
        [sender setState: ([[self settings] boolForKey: iTMUDMatchDelimiterKey]? NSOnState: NSOffState)];
        return YES;
    }
    else if(action == @selector(toggleSmartSelection:))
    {
        [sender setState: ([[self settings] boolForKey: iTMUDSmartSelectionKey]? NSOnState: NSOffState)];
        return YES;
    }
    else if((action == @selector(commentSelectedText:)) ||
                (action == @selector(uncommentSelectedText:)) ||
                    (action == @selector(unindentSelectedText:)) ||
                        (action == @selector(indentSelectedText:)) ||
                            (action == @selector(parseVisibleText:)) ||
                                (action == @selector(parseAllText:)))
    {
        return YES;
    }
    else if(action == @selector(parseSelectedText:))
    {
        return [self selectedRange].length > 0;
    }
    else
        return [super validateMenuItem: sender];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  performKeyEquivalent:
- (BOOL) performKeyEquivalent: (NSEvent *) theEvent;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list: problem when more than one key is pressed.
"*/
{
//    NSLog(@"tvperformKeyEquivalent %@: <%@>", self, [theEvent charactersIgnoringModifiers]);
    if((_iTMTVFlags.smartInsert != 0) && ([[theEvent charactersIgnoringModifiers] characterAtIndex: 0] == 27))
    {
        if(_iTMTVFlags.isEscaped != 0)
        {
            [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus: nil];
            _iTMTVFlags.isEscaped = 0;
        }
        else
        {
            [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
                NSLocalizedStringFromTableInBundle(@"Escaped mode.", @"TeX",
                    [NSBundle bundleForClass: [self class]], "status displayed when the escaped key is pressed")];
            _iTMTVFlags.isEscaped = -1;
        }
        return YES;
    }
    else 
        return [super performKeyEquivalent: theEvent];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  deleteBackward:
- (void) deleteBackward: (id) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list: may be it is wrong...
"*/
{
    if((_iTMTVFlags.shiftDelete != 0) && ([[NSApp currentEvent] modifierFlags] & NSShiftKeyMask))
        [self deleteForward: self];
    else
        [super deleteBackward: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  parseSelectedText:
- (void) parseSelectedText: (id) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list: may be it is wrong...
"*/
{
    [[self textStorage] cleanAttributesInRange: [self selectedRange]];
    [self setNeedsDisplay: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  parseVisibleText:
- (void) parseVisibleText: (id) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list: may be it is wrong...
"*/
{
    [[self textStorage] cleanAttributesInRange: [self visibleRange]];
    [self setNeedsDisplay: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  parseAllText:
- (void) parseAllText: (id) sender;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list: may be it is wrong...
"*/
{
    [[self textStorage] cleanAttributesInRange: NSMakeRange(0, [[self textStorage] length])];
    [self setNeedsDisplay: YES];
    return;
}
#if 1
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setSelectedRange:affinity:stillSelecting:
- (void) setSelectedRange: (NSRange) charRange affinity: (NSSelectionAffinity) affinity stillSelecting: (BOOL) flag;
/*"Paren matching.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    if(!flag && [[self settings] boolForKey: iTMUDSmartSelectionKey] && ![[NSUserDefaults standardUserDefaults] boolForKey: iTMUDSmartSelectionKey])
    {
        NSRange R = [self selectedRange];
        [super setSelectedRange: charRange affinity: affinity stillSelecting: flag];
        [[iTMTextWatcher sharedTextWatcher] textView: self didChangeSelectionWithOldSelectedRange: R];
    }
    else
        [super setSelectedRange: charRange affinity: affinity stillSelecting: flag];
    if(_iTMTVFlags.tempAttrArePending != 0)
        [[self layoutManager] removeTemporaryAttribute: NSBackgroundColorAttributeName
            forCharacterRange: NSMakeRange(0, [[self string] length])];
    _iTMTVFlags.tempAttrArePending = 0;
    return;
}
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  highlightAndScrollLineToVisible:
- (void) highlightAndScrollLineToVisible: (int) aLine;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self highlightAndScrollRangeToVisible: [[self string] rangeForLine: aLine nextLine: nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  highlightAndScrollLineRangeToVisible:
- (void) highlightAndScrollLineRangeToVisible: (NSRange) aLineRange;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self highlightAndScrollRangeToVisible: [[self string] rangeForLineRange: aLineRange nextLine: nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  highlightAndScrollRangeToVisible:
- (void) highlightAndScrollRangeToVisible: (NSRange) aRange;
/*"Description forthcoming. The receiver is the first responder as side effect.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * S = [self string];
    // if one of them is the first responder
    aRange = NSIntersectionRange(aRange, NSMakeRange(0, [S length]));
    [self highlightRange: aRange cleanBefore: YES];
    [self scrollRangeToVisible: aRange];
    [[self window] makeFirstResponder: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  highlightRange:cleanBefore:
- (void) highlightRange: (NSRange) aRange cleanBefore: (BOOL) aFlag;
/*"Description forthcoming. Clean before to remove NSBackgroundColorAttributeName pending attribute.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSLayoutManager * LM = [self layoutManager];
    NSString * S = [self string];
    NSRange R = NSMakeRange(0, [S length]);
    aRange = NSIntersectionRange(aRange, R);
    if(aFlag && (_iTMTVFlags.tempAttrArePending != 0))
    {
        [LM removeTemporaryAttribute: NSBackgroundColorAttributeName forCharacterRange: R];
    }
    [LM addTemporaryAttributes:
        [NSDictionary dictionaryWithObject: [NSColor secondarySelectedControlColor]
            forKey: NSBackgroundColorAttributeName]
                forCharacterRange: aRange];
    _iTMTVFlags.tempAttrArePending = 1;
    return;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  keyDown:
- (void) keyDown: (NSEvent *) theEvent;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    NSString * S = [theEvent charactersIgnoringModifiers];
    NSLog(@"%u", [S characterAtIndex: 0]);
    [super keyDown: theEvent];
    return;
}
- (BOOL)performMnemonic:(NSString *)aString
{
    NSLog(@"tvperformMnemonic %@: <%@>", self, aString);
    return [super performMnemonic: aString];
}
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  readablePasteboardTypes
- (NSArray *) readablePasteboardTypes;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (11/10/2001).
To do list:
"*/
{
    NSMutableArray * MA = [NSMutableArray arrayWithObject: NSFilenamesPboardType];
    [MA addObjectsFromArray: [super readablePasteboardTypes]];
    return MA;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTextView

