/*
//  iTMTextWatcher.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Thu Dec 20 2001.
//  Copyright  2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date: contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSDictionary.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSNotification.h>
#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "NSString(iTeXMac).h"
#import "iTMTextWatcher.h"
#import "iTMSettings.h"
#import "iTMStatusNotification.h"

NSString * const iTMUDSmartSelectionKey = @"iTM-Text: Smart Selection";
NSString * const iTMUDMatchDelimiterKey = @"iTM-Text: Match Delimiter";

@interface NSMenu(iTMTextWatcher)
- (id <NSMenuItem>) iTMDeepItemWithAction: (SEL) aSelector;
@end

@interface iTMTextWatcher(Private)
+ (void) initialize;
+ (id) sharedTextWatcher;
+ (void) applicationDidFinishLaunching: (NSNotification *) notification;
- (void) observe;
- (void) textViewDidChangeSelection: (NSNotification *) notification;
- (void) applicationWillTerminate: (NSNotification *) notification;
- (id) init;
- (void) dealloc;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTextWatcher
/*"Description forthcoming."*/
@implementation iTMTextWatcher
static id _iTMTextWatcher;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults]registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
            [NSNumber numberWithBool: NO], iTMUDSmartSelectionKey,
            [NSNumber numberWithBool: NO], iTMUDMatchDelimiterKey,
                    nil]];
    [[NSNotificationCenter defaultCenter] addObserver: self
        selector: @selector(applicationDidFinishLaunching:)
            name: NSApplicationDidFinishLaunchingNotification
                object: nil];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  applicationDidFinishLaunching:
+ (void) applicationDidFinishLaunching: (NSNotification *) notification;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:          
            NSLocalizedStringFromTableInBundle(@"Installing Text Watcher.", @"Basic",
        [NSBundle bundleForClass: [self class]], "Just in case")];
    [[iTMTextWatcher sharedTextWatcher] observe];
    [[NSNotificationCenter defaultCenter] removeObserver: self name: [notification name] object: nil];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sharedTextWatcher
+ (id) sharedTextWatcher;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    return _iTMTextWatcher? _iTMTextWatcher: (_iTMTextWatcher = [[self alloc] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  observe
- (void) observe;
#warning cache some flags about should match, smart select...
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    static BOOL wasObserving = NO;
    NSUserDefaults * SUD = [NSUserDefaults standardUserDefaults];
    BOOL shouldObserve = ([SUD boolForKey: iTMUDSmartSelectionKey] ||
                                            [SUD boolForKey: iTMUDMatchDelimiterKey]);
    if(shouldObserve && !wasObserving)
    {
        [[NSNotificationCenter defaultCenter] addObserver: self
            selector: @selector(textViewDidChangeSelection:)
                name: NSTextViewDidChangeSelectionNotification
                    object: nil];
        wasObserving = YES;
    }
    else
        [[NSNotificationCenter defaultCenter] removeObserver: self
            name: NSTextViewDidChangeSelectionNotification
                object: nil];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textView:didChangeSelectionWithOldSelectedRange:
- (void) textView: (NSTextView *) TV didChangeSelectionWithOldSelectedRange: (NSRange) oldSelectedRange;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr, from Mike Ferris Text Extras
- < 1.1: 03/10/2002
To Do List: Change doubleClickAtIndex with a groupRangeAtIndex
"*/
{
    NSRange selectedRange = [TV selectedRange];
    if ((selectedRange.length == 0) && ([[NSApp currentEvent] type] == NSKeyDown))
    {
        if((oldSelectedRange.location == selectedRange.location - 1) ||
                (oldSelectedRange.location == selectedRange.location + 1))
        {
            unsigned index = MIN(selectedRange.location, oldSelectedRange.location);
            NSString * string = [TV string];
            NSRange range = NSMakeRange(0, [string length]);
            BOOL isInRange = NSLocationInRange(index, range);
            unichar delimiter = isInRange? [string characterAtIndex: index]: 0;
            unichar left = 0, right = 0;
            BOOL isDelimiter = NO;
            BOOL isLeft = NO;
            switch(delimiter)
            {
                case '{':
                    isLeft = YES;
                case '}':
                    left = '{';
                    right = '}';
                    isDelimiter = YES;
                    break;
                case '(':
                    isLeft = YES;
                case ')':
                    left = '(';
                    right = ')';
                    isDelimiter = YES;
                    break;
                case '[':
                    isLeft = YES;
                case ']':
                    left = '[';
                    right = ']';
                    isDelimiter = YES;
                    break;
            }
            if(isDelimiter && (index>0))
            {
                BOOL isEscaped = NO;
                isDelimiter = ![string isBackslashAtIndex: index-1 escaped: &isEscaped] || isEscaped; 
            }
            if(isDelimiter)
            {
                NSRange R = [[TV string] groupRangeAtIndex: index beginDelimiter: left endDelimiter: right];
                if (R.location != NSNotFound)
                {
                    unsigned matchIndex = isLeft? NSMaxRange(R)-1: R.location;
                    if(matchIndex != index)
                    {
                        NSLayoutManager * layoutManager = [TV layoutManager];
                        NSRange matchRange = NSMakeRange(matchIndex, 1);
                        [layoutManager textContainerForGlyphAtIndex:
                            [layoutManager glyphRangeForCharacterRange: matchRange actualCharacterRange: NULL].location
                                effectiveRange: NULL];
                        [TV setSelectedRange: matchRange affinity: NSSelectByCharacter stillSelecting: YES];
                        [TV displayIfNeeded];
                        [[TV window] flushWindow];
                        [NSThread sleepUntilDate: [NSDate dateWithTimeIntervalSinceNow: 0.125]];
                        [TV setSelectedRange: selectedRange];
                    }
                }
            }
        }
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textViewDidChangeSelection:
- (void) textViewDidChangeSelection: (NSNotification *) notification;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Change doubleClickAtIndex with a groupRangeAtIndex
"*/
{
    NSTextView * TV = [notification object];
    if ([[TV settings] boolForKey: iTMUDMatchDelimiterKey])
    {
        NSRange oldSelectedRange;
        [[[notification userInfo] objectForKey: @"NSOldSelectedCharacterRange"] getValue: &oldSelectedRange];
        [self textView: TV didChangeSelectionWithOldSelectedRange: oldSelectedRange];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  applicationWillTerminate:
- (void) applicationWillTerminate: (NSNotification *) notification;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: 
"*/
{
    [self autorelease];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"Designated initializer.
Version History: jerome.laurens@u-bourgogne.fr (12/20/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(self = [super init])
    {
        [[NSNotificationCenter defaultCenter] addObserver: self
            selector: @selector(applicationWillTerminate:)
                name: NSApplicationWillTerminateNotification
                    object: nil];
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (12/20/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [super dealloc];
    return;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  NSMenu(iTeXMac)
/*"Description forthcoming."*/
@implementation NSMenu(iTMTextWatcher)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMDeepItemWithAction:
- (id <NSMenuItem>) iTMDeepItemWithAction: (SEL) aSelector;
/*"Extends the #{itemWithTitle:} to look down the menu hierarchy beginning from the receiver.
Only items with no submenus are searched for.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    static int depth = 0;
    id result = nil;
    ++depth;
    if(depth<32)
    {
        NSEnumerator * enumerator = [[self itemArray] objectEnumerator];
        id item;
        while(item=[enumerator nextObject])
        {
            if([item hasSubmenu])
            {
                result = [[item submenu] iTMDeepItemWithAction: aSelector];
                if(result!=nil)
                    break;
            }
            else if([item action] == aSelector)
            {
                result = item;
                break;
            }
        }
    }
    --depth;
    return result;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTextWatcher
