/*
//  iTMXSpellChecker.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sun Feb 03 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import "iTMXSpellChecker.h"
#import "NSString(iTeXMac).h"

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMXSpellChecker
/*"Description forthcoming."*/
@implementation iTMXSpellChecker
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  checkSpellingOfString:startingAt:language:wrap:inSpellDocumentWithTag:wordCount:
- (NSRange) checkSpellingOfString: (NSString *) stringToCheck startingAt: (int) startingOffset language: (NSString *) language wrap: (BOOL) wrapFlag inSpellDocumentWithTag: (int) tag wordCount: (int *) wordCount;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned int length = [stringToCheck length];
    if(NSLocationInRange(startingOffset, NSMakeRange(0, length)))
    {
        // fix the spell checking bug
        NSRange R = [stringToCheck rangeOfString: @" " options: 0
                                        range: NSMakeRange(startingOffset, length - startingOffset)];
        startingOffset = (R.length>0)? R.location: length;
    }
    else
        return [super checkSpellingOfString: stringToCheck
                    startingAt: startingOffset
                        language: language
                            wrap: wrapFlag
                                inSpellDocumentWithTag: tag
                                    wordCount: wordCount];
    {
        NSRange result = NSMakeRange(0, 0);
        unsigned int wrapOffset = startingOffset;
        BOOL scanningTheEnd = YES;
        while(scanningTheEnd || (startingOffset < wrapOffset))
        {
            result = [super checkSpellingOfString: stringToCheck
                        startingAt: startingOffset
                            language: language
                                wrap: wrapFlag
                                    inSpellDocumentWithTag: tag
                                        wordCount: wordCount];
            if(result.length && (result.location>0))
            {
                BOOL escaped;
                if([stringToCheck isBackslashAtIndex: result.location - 1 escaped: &escaped] && !escaped)
                {
                    [self ignoreWord: [stringToCheck substringWithRange: result] inSpellDocumentWithTag: tag];
                    scanningTheEnd = (result.location >= startingOffset);
                    startingOffset = NSMaxRange(result);
                }
                else
                    break;
            }
            else
                break;
        }
        return result;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  checkSpellingOfString:startingAt:
- (NSRange) checkSpellingOfString: (NSString *) stringToCheck startingAt: (int) startingOffset;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
1.0.3 improved for check spelling as you type.
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSRange result = NSMakeRange(0, 0);
    while(startingOffset < [stringToCheck length])
    {
        result = [super checkSpellingOfString: stringToCheck startingAt: startingOffset];
        if(result.length)
        {
            if(result.location>0)
            {
                BOOL escaped;
                if([stringToCheck isBackslashAtIndex: result.location-1 escaped: &escaped] && !escaped)
                    startingOffset = NSMaxRange(result);
                else
                    break;
            }
            else
            {
                NSString * TeXStringToCheck = [NSString stringWithFormat: @"\\%@",
                                                    [stringToCheck substringWithRange: result]];
                NSRange r = [super checkSpellingOfString: TeXStringToCheck startingAt: 0];
                if(r.length>0)
                    break;
                else
                    startingOffset = NSMaxRange(result);
            }
        }
        else
            break;
    }
    return result;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMXSpellChecker

