/*
//  iTMXTextView.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Oct 16 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSArray.h>
#import <AppKit/NSTextStorage.h>
#import <AppKit/NSPasteboard.h>
#import "iTMXTextView.h"
#import "iTMTextStorage.h"
#import "NSString(iTeXMac).h"

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMXTextView
/*"The only purpose of this posed as NSTextView class is to provide a different changeFont and changeColor behaviour.
The message is send to the receiver and forwarded to the next responder. iTeXMac does not use the text view straight in the way the class was originally designed."*/
@implementation iTMXTextView
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  changeFont:
- (void) changeFont: (id) sender;
/*"Awfull text view.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing at first glance.
"*/
{
    [super changeFont: sender];
    [[self nextResponder] tryToPerform: _cmd with: sender];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  changeColor:
- (void) changeColor: (id) sender;
/*"Awfull text view.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing at first glance.
"*/
{
    [super changeColor: sender];
    [[self nextResponder] tryToPerform: _cmd with: sender];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  paste:
- (void) paste: (id) sender;
/*"Awfull text view. Bug catcher: cmd-A+cmd-C+cmd-V+cmd-V = forever spinning wheel
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing at first glance.
"*/
{
    if([[self textStorage] isKindOfClass: [iTMTextStorage class]])
    {
        NSString * type = [[NSPasteboard generalPasteboard] availableTypeFromArray:
                                                [NSArray arrayWithObject: NSStringPboardType]];
        if([type length])
            [self insertText: [[NSPasteboard generalPasteboard] stringForType: NSStringPboardType]];
        else
            NSLog(@"NOTHING TO INSERT");
    }
    else
        [super paste: sender];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  readSelectionFromPasteboard:type:
- (BOOL) readSelectionFromPasteboard: (NSPasteboard *) pboard type: (NSString *) type;
/*"Awfull text view. Bug catcher: use service from??? = forever spinning wheel
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing at first glance.
"*/
{
    if([[self textStorage] isKindOfClass: [iTMTextStorage class]])
    {
        if([type isEqualToString: NSStringPboardType])
        {
            [self insertText: [pboard stringForType: NSStringPboardType]];
            return YES;
        }
        else
            NSLog(@"NOTHING TO INSERT");
    }
    else
        return [super readSelectionFromPasteboard: pboard type: type];
    return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  insertText:
- (void) insertText: (id) argument;
/*"Awfull text view. Bug catcher: use service from??? = forever spinning wheel
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing at first glance.
"*/
{
    if([argument isKindOfClass: [NSString class]] && ([argument length] == 1))
    {
        switch([argument characterAtIndex: 0])
        {
            case '{'://}
            {
                BOOL isEscaped;
                NSString * S = [self string];
                int index = [self selectedRange].location;
                if(!index || ![S isBackslashAtIndex: index escaped: &isEscaped] || !isEscaped)
                {
                    [super insertText: argument];
                    {
                        NSRange R = [self selectedRange];
                        [super insertText: //{
                            @"}"];
                        [self setSelectedRange: R];
                        return;
                    }
                }
                break;
            }
            case '['://]
            {
                BOOL isEscaped;
                NSString * S = [self string];
                int index = [self selectedRange].location;
                if(!index || ![S isBackslashAtIndex: index escaped: &isEscaped] || !isEscaped)
                {
                    [super insertText: argument];
                    {
                        NSRange R = [self selectedRange];
                        [super insertText: //[
                            @"]"];
                        [self setSelectedRange: R];
                        return;
                    }
                }
                break;
            }
            case '('://)
            {
                BOOL isEscaped;
                NSString * S = [self string];
                int index = [self selectedRange].location;
                if(!index || ![S isBackslashAtIndex: index escaped: &isEscaped] || !isEscaped)
                {
                    [super insertText: argument];
                    {
                        NSRange R = [self selectedRange];
                        [super insertText: //(
                            @")"];
                        [self setSelectedRange: R];
                        return;
                    }
                }
                break;
            }
//            case '^': beware of dead letters
            case '_':
            {
                BOOL isEscaped;
                NSString * S = [self string];
                int index = [self selectedRange].location;
                if(!index || ![S isBackslashAtIndex: index escaped: &isEscaped] || !isEscaped)
                {
                    [super insertText: argument];
                    [self insertText: @"{"//}
                        ];
                    return;
                }
                break;
            }
            case '\n':
            case '\r':
            {
                NSRange R = [self selectedRange];
                [super insertText: argument];
                if(([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask) > 0)
                {
                    int begin = 0, contentsEnd = 0, index = 0;
                    NSString * S = [self string];
                    [S getLineStart: &begin end: nil contentsEnd: &contentsEnd forRange: NSMakeRange(R.location, 0)];
                    index = begin;
                    while(index <= contentsEnd)
                    {
                        unsigned u = [S characterAtIndex: index];
                        if(u==' ' || u=='\t')
                            ++index; 
                        else
                            break;
                    }
                    if(index>begin)
                        [self insertText: [S substringWithRange: NSMakeRange(begin, index - begin)]];
                }
                return;
            }
        }
    }
    [super insertText: argument];
}

@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMXTextView

