/*
//  iTMTeXSyntaxParser.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Fri Dec 07 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSNotification.h>
#import <Foundation/NSAttributedString.h>
#import <Foundation/NSException.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSCharacterSet.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSScanner.h>
#import <AppKit/NSApplication.h>
#import <AppKit/NSAttributedString.h>
#import "NSUserDefaults(iTeXMac).h"
#import "NSString(iTeXMac).h"
#import "iTMLiteScanner.h"
#import "iTMTeXSyntaxParser.h"
#import "iTMSyntaxParser.h"

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMSyntaxParser

NSString * const iTMUDTeXStandardFontKey = @"iTMUDTeXStandardFont";
NSString * const iTMUDTeXStandardColorKey = @"iTMUDTeXStandardColor";

NSString * const iTMUDTeXRegularFontKey = @"iTMUDTeXRegularFont";
NSString * const iTMUDTeXCommandFontKey = @"iTMUDTeXCommandFont";
NSString * const iTMUDTeXCommentFontKey = @"iTMUDTeXCommentFont";
NSString * const iTMUDTeXMarkFontKey = @"iTMUDTeXMarkFont";
NSString * const iTMUDTeXDollarFontKey = @"iTMUDTeXDollarFont";
NSString * const iTMUDTeXDelimiterFontKey = @"iTMUDTeXDelimiterFont";

NSString * const iTMUDTeXRegularColorKey = @"iTMUDTeXRegularColor";
NSString * const iTMUDTeXCommandColorKey = @"iTMUDTeXCommandColor";
NSString * const iTMUDTeXCommentColorKey = @"iTMUDTeXCommentColor";
NSString * const iTMUDTeXMarkColorKey = @"iTMUDTeXMarkColor";
NSString * const iTMUDTeXDollarColorKey = @"iTMUDTeXDollarColor";
NSString * const iTMUDTeXDelimiterColorKey = @"iTMUDTeXDelimiterColor";

NSString * const iTMUDLigatureAttributeKey = @"iTMUDLigaturesMode";
NSString * const iTMUDKernAttributeKey = @"iTMUDKerningOffset";
NSString * const iTMUDRegularTextKerningOnlyKey = @"iTMUDRegularTextKerningOnly";

NSString * const iTMTeXInputAttributeName = @"I";
#define diTMIAN iTMTeXInputAttributeName


//NSString * const iTMRegularInput = @"0";
NSString * const iTMBeginCommandInput = @"-1";
NSString * const iTMCommandInput = @"1";
NSString * const iTMShortCommandInput = @"-2";
NSString * const iTMBeginCommentInput = @"-3";
NSString * const iTMBeginSuperscriptInput = @"-4";
NSString * const iTMBeginSubscriptInput = @"-5";
NSString * const iTMCommentInput = @"2";
NSString * const iTMMarkInput = @"3";
NSString * const iTMDollarInput = @"4";
NSString * const iTMDelimiterInput = @"5";

#define diTMRegularInput nil
#define kiTMRegularInput 0
#define kiTMBeginCommandInput -1
#define kiTMCommandInput 1
#define kiTMShortCommandInput -2
#define kiTMBeginCommentInput -3
#define kiTMCommentInput 2
#define kiTMMarkInput 3
#define kiTMDollarInput 4
#define kiTMDelimiterInput 5

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTeXSyntaxParser
/*"This object is the delegate of an iTMTextStorage. It manages the visual attributes corresponding to the TeX contents of attributed strings. Beware, the implementation is minimal for iTeXMac use and needs more work to be fully consistent."*/
@implementation iTMTeXSyntaxParser
static id _iTMTeXSyntaxParser = nil;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialize
+ (void) initialize;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
        [[NSFont fontWithName: @"Times" size: 14.0] nameSizeDictionary], iTMUDTeXRegularFontKey,
        [[NSFont fontWithName: @"Monaco" size: 12.0] nameSizeDictionary], iTMUDTeXCommandFontKey,
        [[NSFont fontWithName: @"Helvetica-Oblique" size: 12.0] nameSizeDictionary], iTMUDTeXCommentFontKey,
        [[NSFont fontWithName: @"Helvetica-Oblique" size: 12.0] nameSizeDictionary], iTMUDTeXMarkFontKey,
        [[NSFont fontWithName: @"Monaco" size: 9.0] nameSizeDictionary], iTMUDTeXDollarFontKey,
        [[NSFont fontWithName: @"Monaco" size: 12.0] nameSizeDictionary], iTMUDTeXDelimiterFontKey,
        [[NSColor blackColor] RGBADictionary], iTMUDTeXRegularColorKey,
        [[NSColor colorWithCalibratedRed: 0.0 green: 0.0 blue: 0.6 alpha: 1.0] RGBADictionary], iTMUDTeXCommandColorKey,
        [[NSColor colorWithCalibratedRed: 0.0 green: 0.4 blue: 0.0 alpha: 1.0] RGBADictionary], iTMUDTeXCommentColorKey,
        [[NSColor colorWithCalibratedRed: 0.5 green: 0.0 blue: 0.0 alpha: 1.0] RGBADictionary], iTMUDTeXMarkColorKey,
        [[NSColor colorWithCalibratedRed: 0.4 green: 0.1 blue: 0.4 alpha: 1.0] RGBADictionary], iTMUDTeXDollarColorKey,
        [[NSColor colorWithCalibratedRed: 0.4 green: 0.1 blue: 0.4 alpha: 1.0] RGBADictionary], iTMUDTeXDelimiterColorKey,
        [NSNumber numberWithInt: 0], iTMUDLigatureAttributeKey,
        [NSNumber numberWithFloat: 0.0], iTMUDKernAttributeKey,
        [NSNumber numberWithBool: YES], iTMUDRegularTextKerningOnlyKey,
            nil]];
    // compatibility with previous versions: some keys have changed
    {
        id object;
        #define COMP(oldKey, newKey)\
        if(object = [[NSUserDefaults standardUserDefaults] objectForKey: oldKey])\
        {\
            [[NSUserDefaults standardUserDefaults] setObject: object forKey: newKey];\
            [[NSUserDefaults standardUserDefaults] removeObjectForKey: oldKey];\
        }
        COMP(@"iTMUDRegularTeXColor", iTMUDTeXRegularColorKey);
        COMP(@"iTMUDCommandColor", iTMUDTeXCommandColorKey);
        COMP(@"iTMUDCommentColor", iTMUDTeXCommentColorKey);
        COMP(@"iTMUDMarkColor", iTMUDTeXMarkColorKey);
        COMP(@"iTMUDDollarColor", iTMUDTeXDollarColorKey);
        COMP(@"iTMUDDelimiterColor", iTMUDTeXDelimiterColorKey);
        [[NSUserDefaults standardUserDefaults] synchronize];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sharedSyntaxParser
+ (id) sharedSyntaxParser;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _iTMTeXSyntaxParser? _iTMTeXSyntaxParser: (_iTMTeXSyntaxParser = [[self alloc] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fontOrColorDidChange:
- (void) fontOrColorDidChange: (NSNotification *) aNotification;
/*"The notification object is used to retrieve font and color info. If no object is given, the NSFontColorManager class object is used.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: NYI
"*/
{
    NSDictionary * D = [aNotification object];
    if(!D)
        D = [NSUserDefaults standardUserDefaults];
    {
        #define SETATTRIBUTES(mode, fontKey, colorKey)\
        NSDictionary * mode = [[D objectForKey: iTMUDRegularTextKerningOnlyKey] boolValue]?\
            [NSDictionary dictionaryWithObjectsAndKeys:\
                [NSFont fontWithNameSizeDictionary: [D objectForKey: fontKey]], NSFontAttributeName,\
                [NSColor colorWithRGBADictionary: [D objectForKey: colorKey]], NSForegroundColorAttributeName,\
                [D objectForKey: iTMUDLigatureAttributeKey], NSLigatureAttributeName,\
                    nil]:\
            [NSDictionary dictionaryWithObjectsAndKeys:\
                [NSFont fontWithNameSizeDictionary: [D objectForKey: fontKey]], NSFontAttributeName,\
                [NSColor colorWithRGBADictionary: [D objectForKey: colorKey]], NSForegroundColorAttributeName,\
                [D objectForKey: iTMUDLigatureAttributeKey], NSLigatureAttributeName,\
                [D objectForKey: iTMUDKernAttributeKey], NSKernAttributeName,\
                    nil]
        SETATTRIBUTES(command, iTMUDTeXCommandFontKey, iTMUDTeXCommandColorKey);
        SETATTRIBUTES(comment, iTMUDTeXCommentFontKey, iTMUDTeXCommentColorKey);
        SETATTRIBUTES(mark, iTMUDTeXMarkFontKey, iTMUDTeXMarkColorKey);
        SETATTRIBUTES(dollar, iTMUDTeXDollarFontKey, iTMUDTeXDollarColorKey);
        SETATTRIBUTES(delimiter, iTMUDTeXDelimiterFontKey, iTMUDTeXDelimiterColorKey);
        NSDictionary * regular = [NSDictionary dictionaryWithObjectsAndKeys:
                [NSFont fontWithNameSizeDictionary: [D objectForKey: iTMUDTeXRegularFontKey]], NSFontAttributeName,
                [NSColor colorWithRGBADictionary: [D objectForKey: iTMUDTeXRegularColorKey]], NSForegroundColorAttributeName,
                [D objectForKey: iTMUDLigatureAttributeKey], NSLigatureAttributeName,
                [D objectForKey: iTMUDKernAttributeKey], NSKernAttributeName,
                    nil];
        [self setAttributesArray: [NSArray arrayWithObjects:
                        regular, command, comment, mark, dollar, delimiter,
                                    nil]];
    }
//    NSLog(@"iTMTeXSyntaxParser-------------------------\n%@", [self attributesArray]);
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:isRegularAtIndex:
- (BOOL) attributedString: (NSAttributedString *) attributedString isRegularAtIndex: (unsigned) aLocation;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return (aLocation < [attributedString length])?
        [[attributedString attribute: diTMIAN atIndex: aLocation effectiveRange: nil] intValue] == 0:
        NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isColoring
- (BOOL) isColoring;
/*"Subclasses will answer YES or NO depending on their "coloring" status.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return YES;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:isITMCommentAtIndex:
- (BOOL) attributedString: (NSAttributedString *) attributedString isITMCommentAtIndex: (unsigned) aLocation;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return (aLocation < [attributedString length])?
        [[attributedString attribute: diTMIAN
                            atIndex: aLocation effectiveRange: nil] intValue] == kiTMMarkInput:
        NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:attributesAtIndex:effectiveRange:
- (NSDictionary *) attributedString: (NSAttributedString *) attributedString attributesAtIndex: (unsigned) aLocation
effectiveRange: (NSRangePointer) aRangePtr;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    static int level = 0;
    int mode = 0;
//    NSLog(@"BEGIN of attributedString:attributesAtIndex:effectiveRange:");
    if(aLocation < [attributedString length])
        mode = [[attributedString attribute: diTMIAN atIndex: aLocation effectiveRange: aRangePtr] intValue];
    #if 0
    if(aRangePtr)
        NSLog(@"%d attributesAtIndex(%u):\n<%@>,\n* aRangePtr: %@", level++, aLocation, attributedString,
            NSStringFromRange(*aRangePtr));
    else
        NSLog(@"%d attributesAtIndex(%u): <%@>", level++, aLocation, attributedString);
    mode = 0;
    #else
    switch(mode)
    {
        case kiTMBeginCommandInput:
        case kiTMShortCommandInput:
            mode = kiTMCommandInput;
            break;
        case kiTMBeginCommentInput:
        {
            unsigned contentsEnd = aLocation;
            [[attributedString string] getLineStart: nil end: nil contentsEnd: &contentsEnd
                forRange: NSMakeRange(aLocation, 0)];
            if(aLocation+1<contentsEnd)
            {
                mode = [[attributedString attribute: diTMIAN atIndex: contentsEnd-1 effectiveRange: nil] intValue];
                if(mode<0)
                    mode = kiTMCommentInput;
            }
            else
                mode = kiTMCommentInput;
            break;
        }
    }
    #endif
//    NSLog(@"END of attributedString:attributesAtIndex:effectiveRange:");
    return [[self attributesArray] objectAtIndex: mode];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:attribute:atIndex:longestEffectiveRange:inRange:
- (id) attributedString: (NSAttributedString *) attributedString attribute: (NSString *) name atIndex: (unsigned) aLocation longestEffectiveRange: (NSRangePointer) aRangePtr inRange: (NSRange) rangeLimit;
/*"The others are not implemented. Use the primitive deep attribute getter. This method is used in the text attributes pref pane.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: No support for marks!!!
"*/
#warning support for marks not implemented !!!
{
//    NSLog(NSStringFromSelector(_cmd));
    if(aRangePtr)
    {
        if([name isEqualToString: iTMTeXInputAttributeName])
        {
            NSNumber * result = [attributedString attribute: name atIndex: aLocation
                longestEffectiveRange: aRangePtr inRange: rangeLimit];
            int mode = [result intValue];
            if((mode==kiTMBeginCommandInput) || (mode==kiTMCommandInput) || (mode==kiTMShortCommandInput))
            {
                unsigned int max = NSMaxRange(*aRangePtr);
                unsigned int ceiling = MIN(NSMaxRange(rangeLimit), [attributedString length]);
                unsigned int floor = MAX(0, rangeLimit.location);
                unsigned int min = aRangePtr->location;
                while(max<ceiling)
                {
                    int value = [[attributedString attribute: name atIndex: max effectiveRange: nil] intValue];
                    if((value == kiTMBeginCommandInput) || (value == kiTMCommandInput) || (mode==kiTMShortCommandInput))
                        ++max;
                    else
                        break;
                }
                while(min>floor)
                {
                    int value = [[attributedString attribute: name atIndex: --min effectiveRange: nil] intValue];
                    if((value != kiTMBeginCommandInput) && (value != kiTMCommandInput) && (mode!=kiTMShortCommandInput))
                    {
                        ++min;
                        break;
                    }
                }
                *aRangePtr = NSMakeRange(min, max - min);
                return iTMCommandInput;
            }
            else if((mode==kiTMBeginCommentInput) || (mode==kiTMCommentInput))
            {
                unsigned int max = NSMaxRange(*aRangePtr);
                unsigned int ceiling = MIN(NSMaxRange(rangeLimit), [attributedString length]);
                unsigned int floor = MAX(0, rangeLimit.location);
                unsigned int min = aRangePtr->location;
                while(max<ceiling)
                {
                    int value = [[attributedString attribute: name atIndex: max effectiveRange: nil] intValue];
                    if((value == kiTMBeginCommentInput) || (value == kiTMCommentInput))
                        ++max;
                    else
                        break;
                }
                while(min>floor)
                {
                    int value = [[attributedString attribute: name atIndex: --min effectiveRange: nil] intValue];
                    if((value != kiTMBeginCommentInput) && (value != kiTMCommentInput))
                    {
                        ++min;
                        break;
                    }
                }
                *aRangePtr = NSMakeRange(min, max - min);
                return iTMCommentInput;
            }
            else
                return result;
        }
    }
    return [attributedString attribute: name atIndex: aLocation longestEffectiveRange: aRangePtr inRange: rangeLimit];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:invalidateRangeForCharactersInOldRange:replacedWithString:
- (NSRange) attributedString: (NSAttributedString *) anAttributedString invalidateRangeForCharactersInOldRange: (NSRange) aRange replacedWithString: (NSString *) aString;
/*"The changes have already been made.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned length;// = [aString length];
    NSRange result = NSMakeRange(aRange.location, 0);
//    NSLog(@"invalidateRangeForCharactersInOldRange: %@", NSStringFromRange(aRange));
//    NS_DURING
    length = [aString length];
    if((aRange.length > 0) || (length > 1))
    {
	unsigned begin, end;
	NSRange r = NSIntersectionRange(NSMakeRange(aRange.location, length),
            NSMakeRange(0, [[anAttributedString string] length]));
	[[anAttributedString string] getLineStart: &begin end: &end contentsEnd: nil forRange: r];
	result = NSMakeRange(begin, end - begin);
    }
    else if(length > 0)
    {
	unsigned theChar = [aString characterAtIndex: 0];
	unsigned location = aRange.location;
	unsigned end = location + 1;
        NSString * string = [anAttributedString string];
	NSRange r;
	switch(theChar)
	{
            #warning DEBUG
            #if 0
	    case '\r':
	    case '\n':
		#warning MAF
		r = NSMakeRange(location, 2);
		while(NSMaxRange(r) > [string length]) r.length -= 1;
		[string getLineStart: nil end: &end contentsEnd: nil forRange: r];
//		if(location > 0) --location;
		result = NSMakeRange(location, end - location);
		break;
            #endif
	    case '%':
	    case '\\':
	    case '$':
//	    case '^':
//	    case '_':
		r = NSMakeRange(location, 1);
		[string getLineStart: nil end: &end contentsEnd: nil forRange: r];
		if(location > 0) --location;
		result = NSMakeRange(location, end - location);
		break;
	    default:
		result = NSMakeRange(location, 1);
		break;
	}
    }
//    NS_HANDLER
//    NSLog(@"BOUH CPABO");
//    NS_ENDHANDLER
//    NSLog(@"invalidateRangeForCharactersInOldRange: %@", NSStringFromRange(result));
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:fixAttributesInRange:
- (void) attributedString: (NSAttributedString *) anAttributedString fixAttributesInRange: (NSRange) aRange;
/*"Description forthcoming.
Version history: 
- < 1.1: 03/10/2002
To Do List:
"*/
{
    static unsigned NNN = 0;
    NSString * string = [anAttributedString string];
    aRange = NSIntersectionRange(aRange, NSMakeRange(0, [string length]));
//    NSLog(@"fixAttributesInRange: %u, %@", NNN++, NSStringFromRange(aRange));
    if(aRange.length>0)
    {
        unsigned scanLocation = aRange.location;
        unsigned top = NSMaxRange(aRange);
        id previousMode = diTMRegularInput;
        [anAttributedString removeAttribute: diTMIAN range: aRange];
        #if 1
        if(scanLocation > 0)
            switch([string characterAtIndex: scanLocation-1])
            {
                case '\r':
                case '\n':
                    previousMode = diTMRegularInput;
                    break;
                default:
                    previousMode = [anAttributedString attribute: diTMIAN
                                                        atIndex: scanLocation - 1 effectiveRange: nil];
                    break;
            }
        #endif
        while(scanLocation < top)
        {
            unichar theChar = [string characterAtIndex: scanLocation];
            NSRange range = NSMakeRange(scanLocation, 1);
            switch([previousMode intValue])
            {
                case kiTMRegularInput:
                case kiTMShortCommandInput:
                case kiTMDelimiterInput:
                case kiTMDollarInput:
                    switch(theChar)
                    {
                        case '%':
                            [anAttributedString addAttribute: diTMIAN value: iTMBeginCommentInput range: range];
                            previousMode = iTMBeginCommentInput;
                            break;
                        case '\\':
                            [anAttributedString addAttribute: diTMIAN value: iTMBeginCommandInput range: range];
                            previousMode = iTMBeginCommandInput;
                            break;
                        case '$':
                            [anAttributedString addAttribute: diTMIAN value: iTMDollarInput range: range];
                            break;
                        case '{':
                        case '}':
                            [anAttributedString addAttribute: diTMIAN value: iTMDelimiterInput range: range];
                            break;
                        default:
                            break;
                        }
                    break;
                case kiTMCommentInput:
                case kiTMMarkInput:
                    switch(theChar)
                    {
                        case '\r':
                        case '\n':
                            previousMode = diTMRegularInput;
                            break;
                        default:
                            [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                            break;
                    }
                    break;
                case kiTMBeginCommentInput:
                    switch(theChar)
                    {
                        case '\r':
                        case '\n':
                            previousMode = diTMRegularInput;
                            break;
                        case '!':
                            [anAttributedString addAttribute: diTMIAN value: iTMMarkInput range: range];
                            previousMode = iTMMarkInput;
                            break;
                        case ' ':
                        case '\t':
                            [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                            break;
                        default:
                            [anAttributedString addAttribute: diTMIAN value: iTMCommentInput range: range];
                            previousMode = iTMCommentInput;
                            break;
                    }
                    break;
                // previous mode is not regular, nor comment, nor mark, nor begin comment.
                // so, command, begin command, delimiter, dollar
                default:
                    switch(theChar)
                    {
                        case '\r':
                        case '\n':
                            previousMode = diTMRegularInput;
                            break;
                        case '%':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMBeginCommentInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMCommentInput range: range];
                                    break;
                                default:
                                    [anAttributedString addAttribute: diTMIAN value: iTMBeginCommentInput range: range];
                                    previousMode = iTMBeginCommentInput;
                                    break;
                            }
                            break;
                        case '\\':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                                case kiTMBeginCommentInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMCommentInput range: range];
                                     break;
                                default:
                                    [anAttributedString addAttribute: diTMIAN value: iTMBeginCommandInput range: range];
                                    previousMode = iTMBeginCommandInput;
                                    break;
                            }
                            break;
                        case '$':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                                case kiTMBeginCommentInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMCommentInput range: range];
                                    break;
                                default:
                                    [anAttributedString addAttribute: diTMIAN value: iTMDollarInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                            }
                            break;
                    #if 0
                        case '^':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                                case kiTMBeginCommentInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMCommentInput range: range];
                                    break;
                                default:
                                    [anAttributedString addAttribute: diTMIAN value: iTMBeginSuperscriptInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                            }
                            break;
                        case '_':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                                case kiTMBeginCommentInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMCommentInput range: range];
                                    break;
                                default:
                                    [anAttributedString addAttribute: diTMIAN value: iTMBeginSubscriptInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                            }
                            break;
                    #endif
                        case '{':
                        case '}':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                                case kiTMBeginCommentInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMCommentInput range: range];
                                    break;
                                default:
                                    [anAttributedString addAttribute: diTMIAN value: iTMDelimiterInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                            }
                            break;
                        default:
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    if([[NSCharacterSet letterCharacterSet] characterIsMember: theChar] || (theChar == '@'))
                                    {
                                        [anAttributedString addAttribute: diTMIAN value: iTMCommandInput range: range];
                                        previousMode = iTMCommandInput;
                                    }
                                    else
                                    {
                                        [anAttributedString addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                        previousMode = diTMRegularInput;
                                    }
                                    break;
                                default:
                                    if([[NSCharacterSet letterCharacterSet] characterIsMember: theChar] ||
                                            (theChar == '@'))
                                        [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                                    else
                                        previousMode = diTMRegularInput;
                                    break;
                            }
                            break;
                        }
                    break;
            }
            ++scanLocation;
        }
    }
//    NSLog(@"%d", ++NNN);
//    NSLog(@"attributedString:fixAttributesInRange: END");
    return;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:fixAttributesInRange:
- (void) attributedString: (NSAttributedString *) anAttributedString fixAttributesInRange: (NSRange) aRange;
/*"Description forthcoming.
Version history: 
- < 1.1: 03/10/2002
To Do List:
"*/
{
    static unsigned NNN = 0;
    NSString * string = [anAttributedString string];
    #warning DEBUG
    return;
    aRange = NSIntersectionRange(aRange, NSMakeRange(0, [string length]));
//    NSLog(@"fixAttributesInRange: %u, %@", NNN++, NSStringFromRange(aRange));
    if(aRange.length>0)
    {
        const char * stringPtr = [string cString];
        const char * scanPtr = stringPtr + aRange.location;
        const char * max = stringPtr + NSMaxRange(aRange);
        id previousMode = diTMRegularInput;
        [anAttributedString removeAttribute: diTMIAN range: aRange];
        #if 1
        if(aRange.location > 0)
            switch(*(scanPtr-1))
            {
                case '\r':
                case '\n':
                    previousMode = diTMRegularInput;
                    break;
                default:
                    previousMode = [anAttributedString attribute: diTMIAN
                                                        atIndex: aRange.location - 1 effectiveRange: nil];
                    break;
            }
        #endif
        while(scanPtr < max)
        {
            char theChar = * scanPtr;
            NSRange range = NSMakeRange(scanPtr - stringPtr, 1);
            switch([previousMode intValue])
            {
                case kiTMRegularInput:
                case kiTMShortCommandInput:
                case kiTMDelimiterInput:
                case kiTMDollarInput:
                    switch(theChar)
                    {
                        case '%':
                            [anAttributedString addAttribute: diTMIAN value: iTMBeginCommentInput range: range];
                            previousMode = iTMBeginCommentInput;
                            break;
                        case '\\':
                            [anAttributedString addAttribute: diTMIAN value: iTMBeginCommandInput range: range];
                            previousMode = iTMBeginCommandInput;
                            break;
                        case '$':
                            [anAttributedString addAttribute: diTMIAN value: iTMDollarInput range: range];
                            break;
                        case '{':
                        case '}':
                            [anAttributedString addAttribute: diTMIAN value: iTMDelimiterInput range: range];
                            break;
                        default:
                            break;
                        }
                    break;
                case kiTMCommentInput:
                case kiTMMarkInput:
                    switch(theChar)
                    {
                        case '\r':
                        case '\n':
                            previousMode = diTMRegularInput;
                            break;
                        default:
                            [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                            break;
                    }
                    break;
                case kiTMBeginCommentInput:
                    switch(theChar)
                    {
                        case '\r':
                        case '\n':
                            previousMode = diTMRegularInput;
                            break;
                        case '!':
                            [anAttributedString addAttribute: diTMIAN value: iTMMarkInput range: range];
                            previousMode = iTMMarkInput;
                            break;
                        case ' ':
                        case '\t':
                            [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                            break;
                        default:
                            [anAttributedString addAttribute: diTMIAN value: iTMCommentInput range: range];
                            previousMode = iTMCommentInput;
                            break;
                    }
                    break;
                // previous mode is not regular, nor comment, nor mark, nor begin comment.
                // so, command, begin command, delimiter, dollar
                default:
                    switch(theChar)
                    {
                        case '\r':
                        case '\n':
                            previousMode = diTMRegularInput;
                            break;
                        case '%':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMBeginCommentInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMCommentInput range: range];
                                    break;
                                default:
                                    [anAttributedString addAttribute: diTMIAN value: iTMBeginCommentInput range: range];
                                    previousMode = iTMBeginCommentInput;
                                    break;
                            }
                            break;
                        case '\\':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                                case kiTMBeginCommentInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMCommentInput range: range];
                                     break;
                                default:
                                    [anAttributedString addAttribute: diTMIAN value: iTMBeginCommandInput range: range];
                                    previousMode = iTMBeginCommandInput;
                                    break;
                            }
                            break;
                        case '$':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                                case kiTMBeginCommentInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMCommentInput range: range];
                                    break;
                                default:
                                    [anAttributedString addAttribute: diTMIAN value: iTMDollarInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                            }
                            break;
                    #if 0
                        case '^':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                                case kiTMBeginCommentInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMCommentInput range: range];
                                    break;
                                default:
                                    [anAttributedString addAttribute: diTMIAN value: iTMBeginSuperscriptInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                            }
                            break;
                        case '_':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                                case kiTMBeginCommentInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMCommentInput range: range];
                                    break;
                                default:
                                    [anAttributedString addAttribute: diTMIAN value: iTMBeginSubscriptInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                            }
                            break;
                    #endif
                        case '{':
                        case '}':
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                                case kiTMCommentInput:
                                case kiTMMarkInput:
                                    [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                                    break;
                                case kiTMBeginCommentInput:
                                    [anAttributedString addAttribute: diTMIAN value: iTMCommentInput range: range];
                                    break;
                                default:
                                    [anAttributedString addAttribute: diTMIAN value: iTMDelimiterInput range: range];
                                    previousMode = diTMRegularInput;
                                    break;
                            }
                            break;
                        default:
                            switch([previousMode intValue])
                            {
                                case kiTMBeginCommandInput:
                                    if([[NSCharacterSet letterCharacterSet] characterIsMember: theChar] || (theChar == '@'))
                                    {
                                        [anAttributedString addAttribute: diTMIAN value: iTMCommandInput range: range];
                                        previousMode = iTMCommandInput;
                                    }
                                    else
                                    {
                                        [anAttributedString addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                        previousMode = diTMRegularInput;
                                    }
                                    break;
                                default:
                                    if([[NSCharacterSet letterCharacterSet] characterIsMember: theChar] ||
                                            (theChar == '@'))
                                        [anAttributedString addAttribute: diTMIAN value: previousMode range: range];
                                    else
                                        previousMode = diTMRegularInput;
                                    break;
                            }
                            break;
                        }
                    break;
            }
            ++scanPtr;
        }
    }
//    NSLog(@"%d", ++NNN);
//    NSLog(@"END");
    return;
}

#endif
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTeXSyntaxParser

