/*
//  iTMSyntaxParser.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Wed Nov 07 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSNotification.h>
#import <Foundation/NSAttributedString.h>
#import <Foundation/NSException.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSCharacterSet.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSScanner.h>
#import <AppKit/NSApplication.h>
#import <AppKit/NSAttributedString.h>
#import "NSUserDefaults(iTeXMac).h"
#import "NSString(iTeXMac).h"
#import "iTMSyntaxParser.h"
//#import "iTMTextWatcher.h"
#import "iTMTextStorage.h"
//#import "iTMLiteScanner.h"

NSString * const iTMFontOrColorDidChangeNotification = @"iTMFontOrColorDidChange";

NSString * const iTMUDStandardFontKey = @"iTMUDStandardFont";
NSString * const iTMUDStandardColorKey = @"iTMUDStandardColor";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMSyntaxParser
/*"Semi abstract class inn the sense that it is very poor, concrete subclasses are iTMTeXSyntaxParser and iTMLogSyntaxParser.
All attributes messages refer to attributes used by the cocoa layout manager to properly display text. Deep attributes are stored in the attributed string. Logical attributes are filtered deep attributes. For example the TeX command \begin is stored with a "begin command" deep attribute for '\' (which is a control char) and a "command" deep attribute for the 5 remaining chars. The attributes for this word (all 6 chars) are the font and fore ground color in the user defaults corresponding to commands. The logical attributes for the whole sequence characters is a "command" attribute, then merging the "begin command" and "command" deep attributes."*/
@implementation iTMSyntaxParser
static id _iTMSyntaxParser;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialize
+ (void) initialize;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
        [[NSFont fontWithName: @"Monaco" size: 12.0] nameSizeDictionary], iTMUDStandardFontKey,
        [[NSColor blackColor] RGBADictionary], iTMUDStandardColorKey,
            nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sharedSyntaxParser
+ (id) sharedSyntaxParser;
/*"One shared instance per class: subclassers will override this method.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _iTMSyntaxParser? _iTMSyntaxParser: (_iTMSyntaxParser = [[self alloc] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"Designated initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(self = [super init])
    {
        [self fontOrColorDidChange: nil];
        #warning Should not always observe notifications (the text view in the defaults does not need to...)
        [[NSNotificationCenter defaultCenter]
            removeObserver: self
                name: iTMFontOrColorDidChangeNotification
                    object: nil];
        [[NSNotificationCenter defaultCenter]
            addObserver: self
                selector: @selector(defaultsFontOrColorDidChange:)
                    name: iTMFontOrColorDidChangeNotification
                        object: nil];
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [[NSNotificationCenter defaultCenter] removeObserver: nil name: nil object: self];
    [self setAttributesArray: nil];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  defaultsFontOrColorDidChange:
- (void) defaultsFontOrColorDidChange: (NSNotification *) aNotification;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Completely use of the notificationobject. 
"*/
{
    [self fontOrColorDidChange: aNotification];
    [[NSNotificationCenter defaultCenter]
        postNotificationName: iTMInvalidateLayoutNotification
            object: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fontOrColorDidChange:
- (void) fontOrColorDidChange: (NSNotification *) aNotification;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Completely use of the notificationobject. 
"*/
{
    NSDictionary * standard = [NSDictionary dictionaryWithObjectsAndKeys:
        [NSFont fontWithNameSizeDictionary: [[NSUserDefaults standardUserDefaults] objectForKey: iTMUDStandardFontKey]],
            NSFontAttributeName,
        [NSColor colorWithRGBADictionary: [[NSUserDefaults standardUserDefaults] objectForKey: iTMUDStandardColorKey]],
            NSForegroundColorAttributeName,
        nil];
    [self setAttributesArray: [NSArray arrayWithObjects:
                        standard, nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributesArray
- (id) attributesArray;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _AsA;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isColoring
- (BOOL) isColoring;
/*"Subclasses will answer YES or NO depending on their "coloring" status.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setAttributesArray
- (void) setAttributesArray: (id) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSArray class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSArray argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    [_AsA autorelease];
    _AsA = [argument retain];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:invalidRangeForCharactersInOldRange:replacedWithString:
- (NSRange) attributedString: (NSAttributedString *) anAttributedString invalidRangeForCharactersInOldRange: (NSRange) aRange replacedWithString: (NSString *) aString;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return aRange;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:fixAttributesInRange:
- (void) attributedString: (NSAttributedString *) anAttributedString fixAttributesInRange: (NSRange) aRange;
/*"Does absolutely nothing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:isITMCommentAtIndex:
- (BOOL) attributedString: (NSAttributedString *) attributedString isITMCommentAtIndex: (unsigned) aLocation;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return NO;
}
@end

@implementation iTMSyntaxParser(iTMAttributes)
/*"All the following methods MUST return a non zero length range when asked for. Forever spinning wheel in iTM-1.0.32"*/
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:attributesAtIndex:effectiveRange:
- (NSDictionary *) attributedString: (NSAttributedString *) attributedString attributesAtIndex: (unsigned) aLocation
effectiveRange: (NSRangePointer) aRangePtr;
/*"This MUST be overriden by subclassers.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(aRangePtr) * aRangePtr = NSMakeRange(0, [attributedString length]);
    return [[self attributesArray] objectAtIndex: 0];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:attributesAtIndex:longestEffectiveRange:inRange:
- (NSDictionary *) attributedString: (NSAttributedString *) attributedString attributesAtIndex: (unsigned) aLocation
longestEffectiveRange: (NSRangePointer) aRangePtr inRange: (NSRange) rangeLimit;
/*"Just calls the non longest version, cutting the range pointered to the rangle limit.
Subclassers will override this method for performance reasons.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSDictionary * result = [self attributedString: attributedString
                                            attributesAtIndex: aLocation effectiveRange: aRangePtr];
    if(aRangePtr) * aRangePtr = NSIntersectionRange(rangeLimit, * aRangePtr);
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:attribute:atIndex:effectiveRange:
- (id) attributedString: (NSAttributedString *) attributedString attribute: (NSString *) name atIndex: (unsigned) aLocation effectiveRange: (NSRangePointer) aRangePtr;
/*"Subclassers will override this method for performance reasons.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self attributedString: attributedString attributesAtIndex: aLocation effectiveRange: aRangePtr]
                objectForKey: name];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedString:attributesAtIndex:effectiveRange:
- (NSDictionary *) attributedString: (NSAttributedString *) attributedString attribute: (NSString *) name atIndex: (unsigned) aLocation longestEffectiveRange: (NSRangePointer) aRangePtr inRange: (NSRange) rangeLimit;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self attributedString: attributedString attributesAtIndex: aLocation
                longestEffectiveRange: aRangePtr inRange: rangeLimit]
                    objectForKey: name];
}
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMSyntaxParser
