/*
//  iTMTeXSyntaxParser.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Fri Dec 07 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSNotification.h>
#import <Foundation/NSAttributedString.h>
#import <Foundation/NSException.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSCharacterSet.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSScanner.h>
#import <AppKit/NSApplication.h>
#import <AppKit/NSAttributedString.h>
#import "NSUserDefaults(iTeXMac).h"
#import "NSString(iTeXMac).h"
#import "NSTextView(iTeXMac).h"
#import "iTMLiteScanner.h"
#import "iTMTeXSyntaxParser.h"
#import "iTMSyntaxParser.h"

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMSyntaxParser

NSString * const iTMUDTeXStandardFontKey = @"iTMUDTeXStandardFont";
NSString * const iTMUDTeXStandardColorKey = @"iTMUDTeXStandardColor";

NSString * const iTMUDTeXRegularFontKey = @"iTMUDTeXRegularFont";
NSString * const iTMUDTeXCommandFontKey = @"iTMUDTeXCommandFont";
NSString * const iTMUDTeXCommentFontKey = @"iTMUDTeXCommentFont";
NSString * const iTMUDTeXMarkFontKey = @"iTMUDTeXMarkFont";
NSString * const iTMUDTeXDollarFontKey = @"iTMUDTeXDollarFont";
NSString * const iTMUDTeXDelimiterFontKey = @"iTMUDTeXDelimiterFont";

NSString * const iTMUDTeXRegularColorKey = @"iTMUDTeXRegularColor";
NSString * const iTMUDTeXCommandColorKey = @"iTMUDTeXCommandColor";
NSString * const iTMUDTeXCommentColorKey = @"iTMUDTeXCommentColor";
NSString * const iTMUDTeXMarkColorKey = @"iTMUDTeXMarkColor";
NSString * const iTMUDTeXDollarColorKey = @"iTMUDTeXDollarColor";
NSString * const iTMUDTeXDelimiterColorKey = @"iTMUDTeXDelimiterColor";

NSString * const iTMUDLigatureAttributeKey = @"iTMUDLigaturesMode";
NSString * const iTMUDKernAttributeKey = @"iTMUDKerningOffset";
NSString * const iTMUDRegularTextKerningOnlyKey = @"iTMUDRegularTextKerningOnly";

NSString * const iTMTeXInputAttributeName = @"I";
#define diTMIAN iTMTeXInputAttributeName


//NSString * const iTMRegularInput = @"0";
NSString * const iTMBeginCommandInput = @"-1";
NSString * const iTMCommandInput = @"1";
NSString * const iTMShortCommandInput = @"-2";
NSString * const iTMBeginCommentInput = @"-3";
NSString * const iTMBeginSuperscriptInput = @"-4";
NSString * const iTMBeginSubscriptInput = @"-5";
NSString * const iTMCommentInput = @"2";
NSString * const iTMMarkInput = @"3";
NSString * const iTMDollarInput = @"4";
NSString * const iTMDelimiterInput = @"5";

#define diTMRegularInput nil
#define kiTMRegularInput 0
#define kiTMBeginCommandInput -1
#define kiTMCommandInput 1
#define kiTMShortCommandInput -2
#define kiTMBeginCommentInput -3
#define kiTMCommentInput 2
#define kiTMMarkInput 3
#define kiTMDollarInput 4
#define kiTMDelimiterInput 5

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTeXSyntaxParser
/*"This object is the delegate of an iTMTextStorage. It manages the visual attributes corresponding to the TeX contents of attributed strings. Beware, the implementation is minimal for iTeXMac use and needs more work to be fully consistent."*/
@implementation iTMTeXSyntaxParser
static id _iTMTeXSyntaxParser = nil;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialize
+ (void) initialize;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
        [[NSFont fontWithName: @"Times" size: 14.0] nameSizeDictionary], iTMUDTeXRegularFontKey,
        [[NSFont fontWithName: @"Monaco" size: 12.0] nameSizeDictionary], iTMUDTeXCommandFontKey,
        [[NSFont fontWithName: @"Helvetica-Oblique" size: 12.0] nameSizeDictionary], iTMUDTeXCommentFontKey,
        [[NSFont fontWithName: @"Helvetica-Oblique" size: 12.0] nameSizeDictionary], iTMUDTeXMarkFontKey,
        [[NSFont fontWithName: @"Monaco" size: 9.0] nameSizeDictionary], iTMUDTeXDollarFontKey,
        [[NSFont fontWithName: @"Monaco" size: 12.0] nameSizeDictionary], iTMUDTeXDelimiterFontKey,
        [[NSColor blackColor] RGBADictionary], iTMUDTeXRegularColorKey,
        [[NSColor colorWithCalibratedRed: 0.0 green: 0.0 blue: 0.6 alpha: 1.0] RGBADictionary], iTMUDTeXCommandColorKey,
        [[NSColor colorWithCalibratedRed: 0.0 green: 0.4 blue: 0.0 alpha: 1.0] RGBADictionary], iTMUDTeXCommentColorKey,
        [[NSColor colorWithCalibratedRed: 0.5 green: 0.0 blue: 0.0 alpha: 1.0] RGBADictionary], iTMUDTeXMarkColorKey,
        [[NSColor colorWithCalibratedRed: 0.4 green: 0.1 blue: 0.4 alpha: 1.0] RGBADictionary], iTMUDTeXDollarColorKey,
        [[NSColor colorWithCalibratedRed: 0.4 green: 0.1 blue: 0.4 alpha: 1.0] RGBADictionary], iTMUDTeXDelimiterColorKey,
        [NSNumber numberWithInt: 0], iTMUDLigatureAttributeKey,
        [NSNumber numberWithFloat: 0.0], iTMUDKernAttributeKey,
        [NSNumber numberWithBool: YES], iTMUDRegularTextKerningOnlyKey,
            nil]];
    // compatibility with previous versions: some keys have changed
    {
        id object;
        #define COMP(oldKey, newKey)\
        if(object = [[NSUserDefaults standardUserDefaults] objectForKey: oldKey])\
        {\
            [[NSUserDefaults standardUserDefaults] setObject: object forKey: newKey];\
            [[NSUserDefaults standardUserDefaults] removeObjectForKey: oldKey];\
        }
        COMP(@"iTMUDRegularTeXColor", iTMUDTeXRegularColorKey);
        COMP(@"iTMUDCommandColor", iTMUDTeXCommandColorKey);
        COMP(@"iTMUDCommentColor", iTMUDTeXCommentColorKey);
        COMP(@"iTMUDMarkColor", iTMUDTeXMarkColorKey);
        COMP(@"iTMUDDollarColor", iTMUDTeXDollarColorKey);
        COMP(@"iTMUDDelimiterColor", iTMUDTeXDelimiterColorKey);
        [[NSUserDefaults standardUserDefaults] synchronize];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  sharedSyntaxParser
+ (id) sharedSyntaxParser;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _iTMTeXSyntaxParser? _iTMTeXSyntaxParser: (_iTMTeXSyntaxParser = [[self alloc] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"Designated initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(self = [super init])
    {
        [_AADs release];
        _AADs = [[NSArray arrayWithObjects:
            [NSDictionary dictionaryWithObject: iTMBeginCommandInput forKey: iTMTeXInputAttributeName],
            [NSDictionary dictionaryWithObject: iTMCommandInput forKey: iTMTeXInputAttributeName],
            [NSDictionary dictionaryWithObject: iTMShortCommandInput forKey: iTMTeXInputAttributeName],
            [NSDictionary dictionaryWithObject: iTMBeginCommentInput forKey: iTMTeXInputAttributeName],
            [NSDictionary dictionaryWithObject: iTMBeginSuperscriptInput forKey: iTMTeXInputAttributeName],
            [NSDictionary dictionaryWithObject: iTMBeginSubscriptInput forKey: iTMTeXInputAttributeName],
            [NSDictionary dictionaryWithObject: iTMCommentInput forKey: iTMTeXInputAttributeName],
            [NSDictionary dictionaryWithObject: iTMMarkInput forKey: iTMTeXInputAttributeName],
            [NSDictionary dictionaryWithObject: iTMDollarInput forKey: iTMTeXInputAttributeName],
            [NSDictionary dictionaryWithObject: iTMDelimiterInput forKey: iTMTeXInputAttributeName],
//            [NSDictionary dictionaryWithObject:  forKey: iTMTeXInputAttributeName],
                nil] retain];
    }
    return self;
}
#define diTMRegularAttr [_AADs objectAtIndex: 0]
#define diTMBeginCommandAttr [_AADs objectAtIndex: 1]
#define diTMCommandAttr [_AADs objectAtIndex: 2]
#define diTMShortCommandAttr [_AADs objectAtIndex: 3]
#define diTMBeginCommentAttr [_AADs objectAtIndex: 4]
#define diTMCommentAttr [_AADs objectAtIndex: 5]
#define diTMMarkAttr [_AADs objectAtIndex: 6]
#define diTMDollarAttr [_AADs objectAtIndex: 7]
#define diTMDelimiterAttr [_AADs objectAtIndex: 8]

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [_AADs release];
    _AADs = nil;
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fontOrColorDidChange:
- (void) fontOrColorDidChange: (NSNotification *) aNotification;
/*"The notification object is used to retrieve font and color info. If no object is given, the NSFontColorManager class object is used.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: NYI
"*/
{
    id D = [aNotification object];
    if(!D)
        D = [NSUserDefaults standardUserDefaults];
    {
        #define SETATTRIBUTES(mode, fontKey, colorKey)\
        NSDictionary * mode = [[D objectForKey: iTMUDRegularTextKerningOnlyKey] boolValue]?\
            [NSDictionary dictionaryWithObjectsAndKeys:\
                [NSFont fontWithNameSizeDictionary: [D objectForKey: fontKey]], NSFontAttributeName,\
                [NSColor colorWithRGBADictionary: [D objectForKey: colorKey]], NSForegroundColorAttributeName,\
                [D objectForKey: iTMUDLigatureAttributeKey], NSLigatureAttributeName,\
                    nil]:\
            [NSDictionary dictionaryWithObjectsAndKeys:\
                [NSFont fontWithNameSizeDictionary: [D objectForKey: fontKey]], NSFontAttributeName,\
                [NSColor colorWithRGBADictionary: [D objectForKey: colorKey]], NSForegroundColorAttributeName,\
                [D objectForKey: iTMUDLigatureAttributeKey], NSLigatureAttributeName,\
                [D objectForKey: iTMUDKernAttributeKey], NSKernAttributeName,\
                    nil]
        SETATTRIBUTES(command, iTMUDTeXCommandFontKey, iTMUDTeXCommandColorKey);
        SETATTRIBUTES(comment, iTMUDTeXCommentFontKey, iTMUDTeXCommentColorKey);
        SETATTRIBUTES(mark, iTMUDTeXMarkFontKey, iTMUDTeXMarkColorKey);
        SETATTRIBUTES(dollar, iTMUDTeXDollarFontKey, iTMUDTeXDollarColorKey);
        SETATTRIBUTES(delimiter, iTMUDTeXDelimiterFontKey, iTMUDTeXDelimiterColorKey);
        NSDictionary * regular = [NSDictionary dictionaryWithObjectsAndKeys:
                [NSFont fontWithNameSizeDictionary: [D objectForKey: iTMUDTeXRegularFontKey]], NSFontAttributeName,
                [NSColor colorWithRGBADictionary: [D objectForKey: iTMUDTeXRegularColorKey]], NSForegroundColorAttributeName,
                [D objectForKey: iTMUDLigatureAttributeKey], NSLigatureAttributeName,
                [D objectForKey: iTMUDKernAttributeKey], NSKernAttributeName,
                    nil];
        [self setAttributesArray: [NSArray arrayWithObjects:
                        regular, command, comment, mark, dollar, delimiter,
                                    nil]];
    }
//    NSLog(@"iTMTeXSyntaxParser-------------------------\n%@", [self attributesArray]);
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isRegularAtIndex:
- (BOOL) isRegularAtIndex: (unsigned) aLocation;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return (aLocation < [[self textStorage] length])?
        [[[[self textStorage] mutableAttributedString]
            attribute: diTMIAN atIndex: aLocation effectiveRange: nil] intValue] == 0:
        NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isColoring
- (BOOL) isColoring;
/*"Subclasses will answer YES or NO depending on their "coloring" status.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return YES;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isITMCommentAtIndex:
- (BOOL) isITMCommentAtIndex: (unsigned) aLocation;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return (aLocation < [[self textStorage] length])?
        [[[[self textStorage] mutableAttributedString] attribute: diTMIAN
                            atIndex: aLocation effectiveRange: nil] intValue] == kiTMMarkInput:
        NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributesAtIndex:effectiveRange:
- (NSDictionary *) attributesAtIndex: (unsigned) aLocation effectiveRange: (NSRangePointer) aRangePtr;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id attr = [[[self textStorage] mutableAttributedString] attributesAtIndex: aLocation effectiveRange: aRangePtr];
    int attrIdx = [[attr objectForKey: @"I"] intValue];
    int index;
    if(attrIdx == kiTMRegularInput)
        index = kiTMRegularInput;
    else if(attrIdx == kiTMCommandInput)
        index = kiTMCommandInput;
    else if(attrIdx == kiTMBeginCommandInput)
        index = kiTMCommandInput;
    else if(attrIdx == kiTMShortCommandInput)
        index = kiTMCommandInput;
    else if(attrIdx == kiTMCommentInput)
        index = kiTMCommentInput;
    else if(attrIdx == kiTMMarkInput)
        index = kiTMMarkInput;
    else if(attrIdx == kiTMBeginCommentInput)
        index = kiTMCommentInput;
    else if(attrIdx == kiTMDollarInput)
        index = kiTMDollarInput;
    else// if(attr == diTMDelimiterInput)
        index = kiTMDelimiterInput;
    #if 0
    if(attr == diTMRegularAttr)
        index = kiTMRegularInput;
    else if(attr == diTMCommandAttr)
        index = kiTMCommandInput;
    else if(attr == diTMBeginCommandAttr)
        index = kiTMCommandInput;
    else if(attr == diTMShortCommandAttr)
        index = kiTMCommandInput;
    else if(attr == diTMCommentAttr)
        index = kiTMCommentInput;
    else if(attr == diTMMarkAttr)
        index = kiTMMarkInput;
    else if(attr == diTMBeginCommentAttr)
        index = kiTMCommentInput;
    else if(attr == diTMDollarAttr)
        index = kiTMDollarInput;
    else// if(attr == diTMDelimiterAttr)
        index = kiTMDelimiterInput;
    #endif
    return [[self attributesArray] objectAtIndex: index];
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attribute:atIndex:longestEffectiveRange:inRange:
- (id) attribute: (NSString *) name atIndex: (unsigned) aLocation longestEffectiveRange: (NSRangePointer) aRangePtr inRange: (NSRange) rangeLimit;
/*"The others are not implemented. Use the primitive deep attribute getter. This method is used in the text attributes pref pane.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: No support for marks!!!
"*/
{
//    NSLog(NSStringFromSelector(_cmd));
    NSRange R;
    if(!aRangePtr)
        aRangePtr = &R;
    if([name isEqualToString: diTMIAN])
    {
        id result = [attributedString attribute: name atIndex: aLocation
            longestEffectiveRange: aRangePtr inRange: rangeLimit];
        int mode = [result intValue];
        if((mode== kiTMBeginCommandInput) || (mode== kiTMCommandInput) || (mode== kiTMShortCommandInput))
        {
            unsigned int max = NSMaxRange(*aRangePtr);
            unsigned int ceiling = MIN(NSMaxRange(rangeLimit), [attributedString length]);
            unsigned int floor = MAX(0, rangeLimit.location);
            unsigned int min = aRangePtr->location;
            while(max<ceiling)
            {
                int value = [[attributedString attribute: name atIndex: max effectiveRange: nil] intValue];
                if((value == kiTMBeginCommandInput) || (value == kiTMCommandInput) || (mode== kiTMShortCommandInput))
                    ++max;
                else
                    break;
            }
            while(min>floor)
            {
                int value = [[attributedString attribute: name atIndex: --min effectiveRange: nil] intValue];
                if((value != kiTMBeginCommandInput) && (value != kiTMCommandInput) && (mode!=kiTMShortCommandInput))
                {
                    ++min;
                    break;
                }
            }
            *aRangePtr = NSMakeRange(min, max - min);
            return iTMCommandInput;
        }
        else if(mode== kiTMBeginCommentInput)
        {
            unsigned int contentsEnd;
            [[attributedString string] getLineStart: nil end: nil contentsEnd: &contentsEnd
                forRange: NSMakeRange(aLocation, 0)];
            if(contentsEnd>aLocation+1)
                result = [attributedString attribute: name atIndex: contentsEnd-1 effectiveRange: nil];
            else
                result = iTMCommentInput;
            if(aRangePtr)
                *aRangePtr = NSIntersectionRange(NSMakeRange(aLocation, contentsEnd - aLocation), rangeLimit);
            return result;
        }
        else
            return result;
    }
    else
        return [attributedString attribute: name atIndex: aLocation longestEffectiveRange: aRangePtr inRange: rangeLimit];
}
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fixAttributesInRange:
- (void) fixAttributesInRange: (NSRange) aRange;
/*"Description forthcoming.
Version history: 
- < 1.1: 03/10/2002
To Do List:
"*/
{
    static unsigned NNN = 0;//NSTextStorage
    NSString * string = [[self textStorage] string];
    NSMutableAttributedString * MAS = [[self textStorage] mutableAttributedString];
    aRange = NSIntersectionRange(aRange, NSMakeRange(0, [string length]));
    NSLog(@"fixAttributesInRange: %u, %@", NNN++, NSStringFromRange(aRange));
    return;
#if 0
    if(aRange.length>1000)
    {
        NSRange R = [[[[[self textStorage] layoutManagers]objectAtIndex: 0] firstTextView] visibleRange];
        aRange = NSIntersectionRange(aRange, R);
        //NSLayoutManager, NSTextView
    }
    else
#endif
        aRange = NSIntersectionRange(aRange, NSMakeRange(0, [string length]));
    return;
    if(aRange.length>0)
    {
        unsigned scanLocation = aRange.location;
        unsigned top = NSMaxRange(aRange);
        id previousAttr = (scanLocation > 0)?
            [MAS attributesAtIndex: scanLocation - 1 effectiveRange: nil]:
            diTMRegularAttr;
        int previousIdx = [[previousAttr objectForKey: @"I"] intValue];
         while(scanLocation < top)
        {
            unichar theChar = [string characterAtIndex: scanLocation];
            NSRange R = NSMakeRange(scanLocation, 1);

            if(((theChar>64) && (theChar < 91)) || ((theChar>96) && (theChar < 123)))// ASCII letters
            {
                if(previousIdx == kiTMRegularInput)
                    ;
                else if(previousIdx == kiTMShortCommandInput)
                    previousAttr = diTMRegularAttr;
                else if(previousIdx == kiTMDollarInput)
                    previousAttr = diTMRegularAttr;
                else if(previousIdx == kiTMDelimiterInput)
                    previousAttr = diTMRegularAttr;
                else if(previousIdx == kiTMBeginCommandInput)
                    previousAttr = diTMCommandAttr;
                else if(previousIdx == kiTMBeginCommentInput)
                    previousAttr = diTMCommandAttr;

                [MAS setAttributes: previousAttr range: R];
//                NSLog(@"%@ == %@", previousAttr, [MAS attributesAtIndex: R.location effectiveRange: nil]);
            }
            else
            {
                switch(theChar)
                {
                    case ' ':
                    if(previousIdx == kiTMRegularInput)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousIdx == kiTMCommandInput)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousIdx == kiTMDelimiterInput)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousIdx == kiTMDollarInput)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousIdx == kiTMBeginCommandInput)
                    {
                        previousAttr = diTMCommandAttr;
                    }
                    else if(previousIdx == kiTMShortCommandInput)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousIdx == kiTMCommentInput)
                    {
                    }
                    else if(previousIdx == kiTMMarkInput)
                    {
                    }
                    else if(previousIdx == kiTMBeginCommentInput)
                    {
                        previousAttr = diTMCommentAttr;
                    }
                    [MAS setAttributes: previousAttr range: R];
                    break;
                    
                    case '\\':
                    if(previousIdx == kiTMRegularInput)
                    {
                        previousAttr = diTMBeginCommandAttr;
                    }
                    else if(previousIdx == kiTMCommandInput)
                    {
                        previousAttr = diTMBeginCommandAttr;
                    }
                    else if(previousIdx == kiTMBeginCommandInput)
                    {
                        previousAttr = diTMShortCommandAttr;
                    }
                    else if(previousIdx == kiTMShortCommandInput)
                    {
                        previousAttr = diTMBeginCommandAttr;
                    }
                    else if(previousIdx == kiTMDollarInput)
                    {
                        previousAttr = diTMBeginCommandAttr;
                    }
                    else if(previousIdx == kiTMCommentInput)
                    {
                    }
                    else if(previousIdx == kiTMMarkInput)
                    {
                    }
                    else if(previousIdx == kiTMBeginCommentInput)
                    {
                        previousAttr = diTMCommentAttr;
                    }
                    else// if(previousIdx == kiTMDelimiterInput)
                    {
                        previousAttr = diTMBeginCommandAttr;
                    }
                    [MAS setAttributes: previousAttr range: R];
                    break;
                    
                    case '\r':
                    case '\n':
                    previousAttr = diTMRegularAttr;
                    [MAS setAttributes: previousAttr range: R];
                    break;
                    
                    case '%':
                    if(previousIdx == kiTMRegularInput)
                    {
                        previousAttr = diTMBeginCommentAttr;
                    }
                    else if(previousIdx == kiTMCommandInput)
                    {
                        previousAttr = diTMCommentAttr;
                    }
                    else if(previousIdx == kiTMBeginCommandInput)
                    {
                        previousAttr = diTMShortCommandAttr;
                    }
                    else if(previousIdx == kiTMShortCommandInput)
                    {
                        previousAttr = diTMBeginCommentAttr;
                    }
                    else if(previousIdx == kiTMDollarInput)
                    {
                        previousAttr = diTMBeginCommentAttr;
                    }
                    else if(previousIdx == kiTMDelimiterInput)
                    {
                        previousAttr = diTMBeginCommentAttr;
                    }
                    else if(previousIdx == kiTMBeginCommentInput)
                    {
                        previousAttr = diTMCommentAttr;
                    }
                    #if 0
                    else if(previousIdx == kiTMCommentInput)
                    {
                    }
                    else if(previousIdx == kiTMMarkInput)
                    {
                    }
                    #endif
                    [MAS setAttributes: previousAttr range: R];
                    break;
                    
                    case '!':
                    if(previousIdx == kiTMRegularInput)
                    {
                    }
                    else if(previousIdx == kiTMCommandInput)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousIdx == kiTMBeginCommandInput)
                    {
                        previousAttr = diTMShortCommandAttr;
                    }
                    else if(previousIdx == kiTMShortCommandInput)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousIdx == kiTMDollarInput)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousIdx == kiTMDelimiterInput)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousIdx == kiTMCommentInput)
                    {
                    }
                    else if(previousIdx == kiTMMarkInput)
                    {
                    }
                    else if(previousIdx == kiTMBeginCommentInput)
                    {
                        previousAttr = diTMMarkAttr;
                    }
                    [MAS setAttributes: previousAttr range: R];
                    break;
                    
                    case '{':
                    case '}':
                    if(previousIdx == kiTMRegularInput)
                    {
                        previousAttr = diTMDelimiterAttr;
                    }
                    else if(previousIdx == kiTMCommandInput)
                    {
                        previousAttr = diTMDelimiterAttr;
                    }
                    else if(previousIdx == kiTMBeginCommandInput)
                    {
                        previousAttr = diTMShortCommandAttr;
                    }
                    else if(previousIdx == kiTMShortCommandInput)
                    {
                        previousAttr = diTMDelimiterAttr;
                    }
                    else if(previousIdx == kiTMDollarInput)
                    {
                        previousAttr = diTMDelimiterAttr;
                    }
                    else if(previousIdx == kiTMDelimiterInput)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousIdx == kiTMCommentInput)
                    {
                    }
                    else if(previousIdx == kiTMMarkInput)
                    {
                    }
                    else if(previousIdx == kiTMBeginCommentInput)
                    {
                        previousAttr = diTMCommentAttr;
                    }
                    [MAS setAttributes: previousAttr range: R];
                    break;

                    default:
                    if(previousIdx == kiTMRegularInput)
                    {
                    }
                    else if(previousIdx == kiTMCommandInput)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousIdx == kiTMBeginCommandInput)
                    {
                        previousAttr = diTMShortCommandAttr;
                    }
                    else if(previousIdx == kiTMShortCommandInput)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousIdx == kiTMDollarInput)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousIdx == kiTMDelimiterInput)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousIdx == kiTMCommentInput)
                    {
                    }
                    else if(previousIdx == kiTMMarkInput)
                    {
                    }
                    else if(previousIdx == kiTMBeginCommentInput)
                    {
                        previousAttr = diTMCommentAttr;
                    }
                    [MAS setAttributes: previousAttr range: R];
                    break;
                }
            }
            #if 0
            if()
                {
                    if(previousIdx == kiTMRegularInput)
                    {
                    }
                    else if(previousIdx == kiTMBeginCommandInput)
                    {
                    }
                    else if(previousIdx == kiTMCommandInput)
                    {
                    }
                    else if(previousIdx == kiTMShortCommandInput)
                    {
                    }
                    else if(previousIdx == kiTMBeginCommentInput)
                    {
                    }
                    else if(previousIdx == kiTMCommentInput)
                    {
                    }
                    else if(previousIdx == kiTMMarkInput)
                    {
                    }
                    else if(previousIdx == kiTMDollarInput)
                    {
                    }
                    else// if(previousIdx == kiTMDelimiterInput)
                    {
                    }
                }
            #endif
            previousIdx = [[previousAttr objectForKey: @"I"] intValue];
            ++scanLocation;
        }
    }
//    NSLog(@"%d", ++NNN);
//    NSLog(@"attributedString:fixAttributesInRange: END");
    return;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  fixAttributesInRange:
- (void) fixAttributesInRange: (NSRange) aRange;
/*"Description forthcoming.
Version history: 
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    static unsigned NNN = 0;//NSTextStorage
    NSString * string = [[self textStorage] string];
    NSMutableAttributedString * MAS = [[self textStorage] mutableAttributedString];
    aRange = NSIntersectionRange(aRange, NSMakeRange(0, [string length]));
//    NSLog(@"fixAttributesInRange: %u, %@", NNN++, NSStringFromRange(aRange));
    if(aRange.length>0)
    {
        unsigned scanLocation = aRange.location;
        unsigned top = NSMaxRange(aRange);
        id previousAttr = (scanLocation > 0)?
            [MAS attributesAtIndex: scanLocation - 1 effectiveRange: nil]:
            diTMRegularAttr;
         while(scanLocation < top)
        {
            unichar theChar = [string characterAtIndex: scanLocation];
            NSRange R = NSMakeRange(scanLocation, 1);

            if(((theChar>64) && (theChar < 91)) || ((theChar>96) && (theChar < 123)))// ASCII letters
            {
                if(previousAttr == diTMRegularAttr)
                    ;
                else if(previousAttr == diTMShortCommandAttr)
                    previousAttr = diTMRegularAttr;
                else if(previousAttr == diTMDollarAttr)
                    previousAttr = diTMRegularAttr;
                else if(previousAttr == diTMDelimiterAttr)
                    previousAttr = diTMRegularAttr;
                else if(previousAttr == diTMBeginCommandAttr)
                    previousAttr = diTMCommandAttr;
                else if(previousAttr == diTMBeginCommentAttr)
                    previousAttr = diTMCommandAttr;

                [MAS setAttributes: previousAttr range: R];
//                NSLog(@"%@ == %@", previousAttr, [MAS attributesAtIndex: R.location effectiveRange: nil]);
            }
            else
            {
                switch(theChar)
                {
                    case ' ':
                    if(previousAttr == diTMRegularAttr)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousAttr == diTMCommandAttr)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousAttr == diTMDelimiterAttr)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousAttr == diTMDollarAttr)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousAttr == diTMBeginCommandAttr)
                    {
                        previousAttr = diTMCommandAttr;
                    }
                    else if(previousAttr == diTMShortCommandAttr)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousAttr == diTMCommentAttr)
                    {
                    }
                    else if(previousAttr == diTMMarkAttr)
                    {
                    }
                    else if(previousAttr == diTMBeginCommentAttr)
                    {
                        previousAttr = diTMCommentAttr;
                    }
                    [MAS setAttributes: previousAttr range: R];
                    break;
                    
                    case '\\':
                    if(previousAttr == diTMRegularAttr)
                    {
                        previousAttr = diTMBeginCommandAttr;
                    }
                    else if(previousAttr == diTMCommandAttr)
                    {
                        previousAttr = diTMBeginCommandAttr;
                    }
                    else if(previousAttr == diTMBeginCommandAttr)
                    {
                        previousAttr = diTMShortCommandAttr;
                    }
                    else if(previousAttr == diTMShortCommandAttr)
                    {
                        previousAttr = diTMBeginCommandAttr;
                    }
                    else if(previousAttr == diTMDollarAttr)
                    {
                        previousAttr = diTMBeginCommandAttr;
                    }
                    else if(previousAttr == diTMCommentAttr)
                    {
                    }
                    else if(previousAttr == diTMMarkAttr)
                    {
                    }
                    else if(previousAttr == diTMBeginCommentAttr)
                    {
                        previousAttr = diTMCommentAttr;
                    }
                    else// if(previousAttr == diTMDelimiterAttr)
                    {
                        previousAttr = diTMBeginCommandAttr;
                    }
                    [MAS setAttributes: previousAttr range: R];
                    break;
                    
                    case '\r':
                    case '\n':
                    previousAttr = diTMRegularAttr;
                    [MAS setAttributes: previousAttr range: R];
                    break;
                    
                    case '%':
                    if(previousAttr == diTMRegularAttr)
                    {
                        previousAttr = diTMBeginCommentAttr;
                    }
                    else if(previousAttr == diTMCommandAttr)
                    {
                        previousAttr = diTMCommentAttr;
                    }
                    else if(previousAttr == diTMBeginCommandAttr)
                    {
                        previousAttr = diTMShortCommandAttr;
                    }
                    else if(previousAttr == diTMShortCommandAttr)
                    {
                        previousAttr = diTMBeginCommentAttr;
                    }
                    else if(previousAttr == diTMDollarAttr)
                    {
                        previousAttr = diTMBeginCommentAttr;
                    }
                    else if(previousAttr == diTMDelimiterAttr)
                    {
                        previousAttr = diTMBeginCommentAttr;
                    }
                    else if(previousAttr == diTMBeginCommentAttr)
                    {
                        previousAttr = diTMCommentAttr;
                    }
                    #if 0
                    else if(previousAttr == diTMCommentAttr)
                    {
                    }
                    else if(previousAttr == diTMMarkAttr)
                    {
                    }
                    #endif
                    [MAS setAttributes: previousAttr range: R];
                    break;
                    
                    case '!':
                    if(previousAttr == diTMRegularAttr)
                    {
                    }
                    else if(previousAttr == diTMCommandAttr)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousAttr == diTMBeginCommandAttr)
                    {
                        previousAttr = diTMShortCommandAttr;
                    }
                    else if(previousAttr == diTMShortCommandAttr)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousAttr == diTMDollarAttr)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousAttr == diTMDelimiterAttr)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousAttr == diTMCommentAttr)
                    {
                    }
                    else if(previousAttr == diTMMarkAttr)
                    {
                    }
                    else if(previousAttr == diTMBeginCommentAttr)
                    {
                        previousAttr = diTMMarkAttr;
                    }
                    [MAS setAttributes: previousAttr range: R];
                    break;
                    
                    case '{':
                    case '}':
                    if(previousAttr == diTMRegularAttr)
                    {
                        previousAttr = diTMDelimiterAttr;
                    }
                    else if(previousAttr == diTMCommandAttr)
                    {
                        previousAttr = diTMDelimiterAttr;
                    }
                    else if(previousAttr == diTMBeginCommandAttr)
                    {
                        previousAttr = diTMShortCommandAttr;
                    }
                    else if(previousAttr == diTMShortCommandAttr)
                    {
                        previousAttr = diTMDelimiterAttr;
                    }
                    else if(previousAttr == diTMDollarAttr)
                    {
                        previousAttr = diTMDelimiterAttr;
                    }
                    else if(previousAttr == diTMDelimiterAttr)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousAttr == diTMCommentAttr)
                    {
                    }
                    else if(previousAttr == diTMMarkAttr)
                    {
                    }
                    else if(previousAttr == diTMBeginCommentAttr)
                    {
                        previousAttr = diTMCommentAttr;
                    }
                    [MAS setAttributes: previousAttr range: R];
                    break;

                    default:
                    if(previousAttr == diTMRegularAttr)
                    {
                    }
                    else if(previousAttr == diTMCommandAttr)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousAttr == diTMBeginCommandAttr)
                    {
                        previousAttr = diTMShortCommandAttr;
                    }
                    else if(previousAttr == diTMShortCommandAttr)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousAttr == diTMDollarAttr)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousAttr == diTMDelimiterAttr)
                    {
                        previousAttr = diTMRegularAttr;
                    }
                    else if(previousAttr == diTMCommentAttr)
                    {
                    }
                    else if(previousAttr == diTMMarkAttr)
                    {
                    }
                    else if(previousAttr == diTMBeginCommentAttr)
                    {
                        previousAttr = diTMCommentAttr;
                    }
                    [MAS setAttributes: previousAttr range: R];
                    break;
                }
            }
            #if 0
            if()
                {
                    if(previousAttr == diTMRegularAttr)
                    {
                    }
                    else if(previousAttr == diTMBeginCommandAttr)
                    {
                    }
                    else if(previousAttr == diTMCommandAttr)
                    {
                    }
                    else if(previousAttr == diTMShortCommandAttr)
                    {
                    }
                    else if(previousAttr == diTMBeginCommentAttr)
                    {
                    }
                    else if(previousAttr == diTMCommentAttr)
                    {
                    }
                    else if(previousAttr == diTMMarkAttr)
                    {
                    }
                    else if(previousAttr == diTMDollarAttr)
                    {
                    }
                    else// if(previousAttr == diTMDelimiterAttr)
                    {
                    }
                }
            #endif
            ++scanLocation;
        }
    }
//    NSLog(@"%d", ++NNN);
//    NSLog(@"attributedString:fixAttributesInRange: END");
    return;
}
#endif
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  invalidateRangeForCharactersInOldRange:replacedWithString:
- (NSRange) invalidRangeForCharactersInOldRange: (NSRange) aRange replacedWithString: (NSString *) aString;
/*"The changes have already been made.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned length;// = [aString length];
    NSRange result = NSMakeRange(aRange.location, 0);
//    NSLog(@"invalidateRangeForCharactersInOldRange: %@", NSStringFromRange(aRange));
//    NS_DURING
    length = [aString length];
    if((aRange.length > 0) || (length > 1))
    {
	unsigned end;
	NSRange r = NSIntersectionRange(NSMakeRange(aRange.location, length),
            NSMakeRange(0, [[AS string] length]));
	[[AS string] getLineStart: nil end: &end contentsEnd: nil forRange: r];
	result = NSMakeRange(aRange.location, end - aRange.location);
    }
    else if(length > 0)
    {
	unsigned location = aRange.location;
	unsigned end = location + 1;
        NSString * string = [AS string];
	NSRange r;
        return NSMakeRange(aRange.location, 1);
	switch([aString characterAtIndex: 0])
	{
	    case '\r':
	    case '\n':
            {
                unsigned start, nextEnd;
		[string getLineStart: &start end: &end contentsEnd: nil forRange: NSMakeRange(location, 0)];
//		if(location > 0) --location;
		[string getLineStart: nil end: &nextEnd contentsEnd: nil forRange: NSMakeRange(end, 0)];
		result = NSMakeRange(start, nextEnd - start);
		break;
            }
	    case '!':
            if((location>0) &&
    ([[AS attribute: diTMIAN atIndex: --location effectiveRange: nil] intValue] == kiTMBeginCommentInput))
            {
		[string getLineStart: nil end: &end contentsEnd: nil forRange: NSMakeRange(location, 0)];
		result = NSMakeRange(location, end - location);
		break;
            }
	    case '%':
		r = NSMakeRange(location, 1);
		[string getLineStart: nil end: &end contentsEnd: nil forRange: r];
		if(location > 0) --location;
		result = NSMakeRange(location, end - location);
		break;
	    case '\\':
		r = NSMakeRange(location, 1);
		[string getLineStart: nil end: &end contentsEnd: nil forRange: r];
		if(location > 0) --location;
		result = NSMakeRange(location, end - location);
		break;
	    case '$':
		r = NSMakeRange(location, 1);
		[string getLineStart: nil end: &end contentsEnd: nil forRange: r];
		if(location > 0) --location;
		result = NSMakeRange(location, end - location);
		break;
//	    case '^':
//	    case '_':
	    default:
		result = NSMakeRange(location, 1);
		break;
	}
    }
//    NS_HANDLER
//    NSLog(@"BOUH CPABO");
//    NS_ENDHANDLER
//    NSLog(@"invalidateRangeForCharactersInOldRange: %@", NSStringFromRange(result));
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= textStorage:replaceCharactersInRange:withString:
- (void) textStorage: (iTMTextStorage *) textStorage replaceCharactersInRange: (NSRange) aRange withString: (NSString *) aString;
/*"Relies on the shared syntax parser.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    unsigned length = [aString length];
    NSMutableAttributedString * MAS = [textStorage mutableAttributedString];
    if((aRange.length != 0) || (length != 1))
    {
        [MAS replaceCharactersInRange: aRange withString: aString];
        [textStorage edited: NSTextStorageEditedCharacters range: aRange changeInLength: length - aRange.length];
        [textStorage invalidateAttributesInRange:
            [self attributedString: [textStorage mutableAttributedString]
                invalidateRangeForCharactersInOldRange: aRange replacedWithString: aString]];
    }
    else
    {
        id previousAttr = aRange.location? [MAS attribute: diTMIAN atIndex: aRange.location-1 effectiveRange: nil]: nil;
        unichar theChar = [aString characterAtIndex: 0];
        [MAS replaceCharactersInRange: aRange withString: aString];
        [textStorage edited: NSTextStorageEditedCharacters range: aRange changeInLength: 1];
        if(((theChar>64) && (theChar < 91)) || ((theChar>96) && (theChar < 123)))
        {
            #if 1
            aRange.length = 1;
            switch([previousAttr intValue])
            {
                case kiTMRegularInput:
                case kiTMShortCommandInput:
                case kiTMDollarInput:
                case kiTMDelimiterInput:
                    return;
                break;
                
                case kiTMBeginCommandInput:
                    previousAttr = iTMCommandInput;
                break;
                
                case kiTMBeginCommentInput:
                {
                    unsigned i = aRange.location + 1;
                    NSString * S = [MAS string];
                    
                    if((i<[S length]) && ([S characterAtIndex: i] == '!'))
                    {
                        unsigned int contentsEnd;
                        [S getLineStart: nil end: nil contentsEnd: &contentsEnd forRange: aRange];
                        [MAS addAttribute: diTMIAN value: iTMCommentInput range: aRange];
//                        [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                        ++aRange.location;
                        aRange.length = contentsEnd - aRange.location;
                        [textStorage invalidateAttributesInRange: aRange];
                        return;
                    }
                    previousAttr = iTMCommentInput;
                }
                break;
            }
            #endif
            [MAS addAttribute: diTMIAN value: previousAttr range: aRange];
//            [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
            return;
        }
        [MAS addAttribute: diTMIAN value: previousAttr range: aRange];
        return;
        #if 0
        else
            switch(theChar)
            {
                case '\\':
                    switch([previousAttr intValue])
                    {
                        case kiTMRegularInput:
                        break;
                        
                        case kiTMBeginCommandInput:
                        break;
                        
                        case kiTMShortCommandInput:
                        break;
                        
                        case kiTMCommandInput:
                        break;
                        
                        case kiTMBeginCommentInput:
                        break;
                        
                        case kiTMCommentInput:
                        break;
                        
                        case kiTMMarkInput:
                        break;
                        
                        case kiTMDollarInput:
                        break;
                        
                        case kiTMDelimiterInput:
                        break;                
                    }
                break;

                case :
                    switch([previousAttr intValue])
                    {
                        case kiTMRegularInput:
                        break;
                        
                        case kiTMBeginCommandInput:
                        break;
                        
                        case kiTMShortCommandInput:
                        break;
                        
                        case kiTMCommandInput:
                        break;
                        
                        case kiTMBeginCommentInput:
                        break;
                        
                        case kiTMCommentInput:
                        break;
                        
                        case kiTMMarkInput:
                        break;
                        
                        case kiTMDollarInput:
                        break;
                        
                        case kiTMDelimiterInput:
                        break;                
                    }
                break;
            }
        aRange.length = 1;
        switch([previousAttr intValue])
        {
                switch([aString characterAtIndex: 0])
                {
                    case '%'://everything is commented out or marked
                    break;
                    
                    case '\\':// a new commmand
                    {
                        unsigned start = aRange.location + 1;
                        [AS addAttribute: diTMIAN value: iTMBeginCommandInput range: aRange];
                        if(start < length)
                        {
                            NSRange R = [[MAS string] rangeOfCharactersFromSet: [NSCharacterSet letterCharacterSet]
                                            options: NSLiteralSearch | NSAnchoredSearch
                                                range: NSMakeRange(start, length - start)];
                            if(R.length)
                            {
                                [AS addAttribute: diTMIAN value: iTMCommandInput range: R];
                                aRange.length = R.length + 1;
                                [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                                return;
                            }
                        }
                        [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                        return;
                    }
                    break;
                    
                    case '$':
                        [AS addAttribute: diTMIAN value: iTMDollarInput range: aRange];
                        [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                        return;
                    break;
                    
                    case '{':
                    case '}':
                        [AS addAttribute: diTMIAN value: iTMDelimiterInput range: aRange];
                        [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                        return;
                    break;
                    default:
                    break;
                }
                break;
            case kiTMCommentInput:
            case kiTMMarkInput:
                switch(theChar)
                {
                    case '\r':
                    case '\n':
                        [AS setAttributes: nil range: aRange];
                    break;
                    default:
                        [AS addAttribute: diTMIAN value: previousAttr range: aRange];
                    break;
                }
                [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                return;
            break;
            case kiTMBeginCommentInput:
                switch(theChar)
                {
                    case '\r':
                    case '\n':
                        [AS setAttributes: nil range: aRange];
                        [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                        return;
                    break;
                    case '!':
                    {
                        unsigned int contentsEnd;
                        [[MAS string] getLineStart: nil end: nil contentsEnd: &contentsEnd forRange: aRange];
                        aRange.length = contentsEnd - aRange.location;
                        [MAS addAttribute: diTMIAN value: iTMMarkInput range: aRange];
                        [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                        return;
                    }
                    break;
                    default:
                        [AS addAttribute: diTMIAN value: iTMCommentInput range: aRange];
                        [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                        return;
                    break;
                }
                break;
            // previous mode is not regular, nor comment, nor mark, nor begin comment.
            // so, command, begin command, delimiter, dollar
            default:
                switch(theChar)
                {
                    case '\r':
                    case '\n':
                        [AS setAttributes: nil range: aRange];
                        [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                        return;
                    break;
                    
                    case '%':
                        switch([previousAttr intValue])
                        {
                            case kiTMBeginCommandInput:
                            {
                                unsigned contentsEnd;
                                [AS addAttribute: diTMIAN value: iTMShortCommandInput range: aRange];
                                [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                                [[AS string] getLineStart: nil end: nil contentsEnd: &contentsEnd
                                        forRange: aRange];
                                ++ aRange.location;
                                if(aRange.location < contentsEnd)
                                {
                                    aRange.length = contentsEnd - aRange.location;
                                    [textStorage invalidateAttributesInRange: aRange];
                                }
                                return;
                            }
                            break;
                            
                            case kiTMBeginCommentInput:
                                [AS addAttribute: diTMIAN value: iTMCommentInput range: range];
                                [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                                return;
                            break;
                            
                            default:
                            {
                                unsigned int contentsEnd;
                                [[MAS string] getLineStart: nil end: nil contentsEnd: &contentsEnd forRange: aRange];
                                [MAS addAttribute: diTMIAN value: iTMBeginCommentInput range: aRange];
                                ++aRange.location;
                                aRange.length = contentsEnd - aRange.location;
                                if(aRange.length)
                                {
                                    NSRange R = [[MAS string] rangeOfString: @"!"
                                                            options: NSLiteralSearch | NSAnchoredSearch range: aRange];
                                    [MAS addAttribute: diTMIAN value: (R.length? iTMMarkInput: iTMCommentInput) range: aRange];
                                    --aRange.location;
                                    ++aRange.length;
                                    [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                                    return;
                                }
                            }
                            break;
                        }
                    break;
                    case '\\':
                        switch([previousAttr intValue])
                        {
                            case kiTMBeginCommandInput:
                            {
                                unsigned contentsEnd;
                                [AS addAttribute: diTMIAN value: iTMShortCommandInput range: aRange];
                                [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                                [[AS string]
                                    getLineStart: nil end: nil contentsEnd: &contentsEnd forRange: aRange];
                                ++ aRange.location;
                                if(aRange.location < contentsEnd)
                                {
                                    aRange.length = contentsEnd - aRange.location;
                                    [textStorage invalidateAttributesInRange: aRange];
                                }
                                return;
                            }
                            break;
                            
                            case kiTMCommentInput:
                            case kiTMMarkInput:
                                [AS addAttribute: diTMIAN value: previousAttr range: range];
                                [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                                return;
                            break;
                            
                            case kiTMBeginCommentInput:
                            {
                                unsigned contentsEnd;
                                [[AS string]
                                    getLineStart: nil end: nil contentsEnd: &contentsEnd forRange: aRange];
                                aRange.length = contentsEnd - aRange.location;
                                [AS addAttribute: diTMIAN value: iTMCommentInput range: aRange];
                                [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                                return;
                            }
                            break;
                            
                            default:
                            {
                                unsigned contentsEnd;
                                [AS addAttribute: diTMIAN value: iTMShortCommandInput range: aRange];
                                [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                                [[AS string]
                                    getLineStart: nil end: nil contentsEnd: &contentsEnd forRange: aRange];
                                ++ aRange.location;
                                if(aRange.location < contentsEnd)
                                {
                                    aRange.length = contentsEnd - aRange.location;
                                    [textStorage invalidateAttributesInRange: aRange];
                                }
                                return;
                            }
                            {
                                unsigned contentsEnd;
                                [[AS string]
                                    getLineStart: nil end: nil contentsEnd: &contentsEnd forRange: aRange];
                                aRange.length = contentsEnd - aRange.location;
                                [AS addAttribute: diTMIAN value: iTMCommentInput range: aRange];
                                [textStorage edited: NSTextStorageEditedAttributes range: aRange changeInLength: 0];
                                return;
                            }
                            break;
                        }
                        break;
                    case '$':
                        switch([previousAttr intValue])
                        {
                            case kiTMBeginCommandInput:
                                [AS addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                previousAttr = diTMRegularInput;
                            break;
                            case kiTMCommentInput:
                            case kiTMMarkInput:
                                [AS addAttribute: diTMIAN value: previousAttr range: range];
                            break;
                            case kiTMBeginCommentInput:
                                [AS addAttribute: diTMIAN value: iTMCommentInput range: range];
                            break;
                            default:
                                [AS addAttribute: diTMIAN value: iTMDollarInput range: range];
                                previousAttr = diTMRegularInput;
                            break;
                        }
                        break;
                #if 0
                    case '^':
                        switch([previousAttr intValue])
                        {
                            case kiTMBeginCommandInput:
                                [AS addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                previousAttr = diTMRegularInput;
                            break;
                            case kiTMCommentInput:
                            case kiTMMarkInput:
                                [AS addAttribute: diTMIAN value: previousAttr range: range];
                            break;
                            case kiTMBeginCommentInput:
                                [AS addAttribute: diTMIAN value: iTMCommentInput range: range];
                            break;
                            default:
                                [AS addAttribute: diTMIAN value: iTMBeginSuperscriptInput range: range];
                                previousAttr = diTMRegularInput;
                            break;
                        }
                        break;
                    case '_':
                        switch([previousAttr intValue])
                        {
                            case kiTMBeginCommandInput:
                                [AS addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                previousAttr = diTMRegularInput;
                                break;
                            case kiTMCommentInput:
                            case kiTMMarkInput:
                                [AS addAttribute: diTMIAN value: previousAttr range: range];
                                break;
                            case kiTMBeginCommentInput:
                                [AS addAttribute: diTMIAN value: iTMCommentInput range: range];
                            break;
                            default:
                                [AS addAttribute: diTMIAN value: iTMBeginSubscriptInput range: range];
                                previousAttr = diTMRegularInput;
                            break;
                        }
                    break;
                #endif
                    case '{':
                    case '}':
                        switch([previousAttr intValue])
                        {
                            case kiTMBeginCommandInput:
                                [AS addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                previousAttr = diTMRegularInput;
                                break;
                            case kiTMCommentInput:
                            case kiTMMarkInput:
                                [AS addAttribute: diTMIAN value: previousAttr range: range];
                                break;
                            case kiTMBeginCommentInput:
                                [AS addAttribute: diTMIAN value: iTMCommentInput range: range];
                                break;
                            default:
                                [AS addAttribute: diTMIAN value: iTMDelimiterInput range: range];
                                previousAttr = diTMRegularInput;
                                break;
                        }
                    break;
                    default:
                        switch([previousAttr intValue])
                        {
                            case kiTMBeginCommandInput:
                                if([[NSCharacterSet letterCharacterSet] characterIsMember: theChar] || (theChar == '@'))
                                {
                                    [AS addAttribute: diTMIAN value: iTMCommandInput range: range];
                                    previousAttr = iTMCommandInput;
                                }
                                else
                                {
                                    [AS addAttribute: diTMIAN value: iTMShortCommandInput range: range];
                                    previousAttr = diTMRegularInput;
                                }
                                break;
                            default:
                                if([[NSCharacterSet letterCharacterSet] characterIsMember: theChar] ||
                                        (theChar == '@'))
                                    [AS addAttribute: diTMIAN value: previousAttr range: range];
                                else
                                    previousAttr = diTMRegularInput;
                                break;
                        }
                        break;
                    }
                break;
        }
    #endif
    }
    return;
}
#endif
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTeXSyntaxParser

