/*
//  iTMTextStorage.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Oct 16 2001.
//  Copyright  2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSAttributedString.h>
#import <Foundation/NSException.h>
#import <Foundation/NSEnumerator.h>
#import <Foundation/NSCharacterSet.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSArchiver.h>
#import <AppKit/NSColor.h>
#import <AppKit/NSFont.h>
#import <AppKit/NSFontManager.h>
#import <AppKit/NSLayoutManager.h>
#import <AppKit/NSApplication.h>
#import <AppKit/NSTextView.h>
#import "NSUserDefaults(iTeXMac).h"
#import "NSString(iTeXMac).h"
#import "iTMTextStorage.h"
//#import "iTMTextStorage(iTMAnchors).h"
#import "iTMSyntaxParser.h"
#import "iTMLiteScanner.h"
#import "iTMEncodingController.h"
#import "iTMTextWatcher.h"
#import "iTMSettings.h"

NSString * const iTMInvalidateLayoutNotification = @"iTMInvalidateLayout";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTextStorage
/*"Semi abstract class that defines a text storage and fixes attributes with respect to some syntax, more precisely TeX syntax and Log files syntax. Concrete classes are iTMTeXTextStorage with iTMTeXSyntaxParser, with iTMLogSyntaxParser."*/
@implementation iTMTextStorage
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initialize
+ (void) initialize;
/*"This is the one of the three initializers available!!! More precisely, the first one.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing
"*/
{
    [super initialize];
    [iTMEncodingController initialize];//Critical: forces intialization
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= init
- (id) init;
/*"This is the one of the three initializers available!!! More precisely, the first one.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing
"*/
{
    if(self = [super init])
    {
        [self->_MAS autorelease];
        self->_MAS = [[NSMutableAttributedString alloc] initWithString: [NSString string]];
        [self setStringEncoding: 0];
        [[NSNotificationCenter defaultCenter] removeObserver: self name: iTMInvalidateLayoutNotification object: nil];
        [[NSNotificationCenter defaultCenter] addObserver: self
            selector: @selector(invalidateLayoutNotified:)
                name: iTMInvalidateLayoutNotification
                    object: nil];
//        [self initAnchorsControllers];
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initWithString:attributes:
- (id) initWithString: (NSString *) aString attributes: (NSDictionary *) attributes;
/*"This is the one of the three initializers available!!! More precisely, the second one.
When no syntax parser is given, the layout is not invalidated and the attributes are not fixed.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing
"*/
{
    return [self initWithString: aString attributes: attributes syntaxParser: nil];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initWithString:attributes:syntaxParser:
- (id) initWithString: (NSString *) aString attributes: (NSDictionary *) attributes syntaxParser: (id) SP;
/*"This is the one of the three initializers available!!! More precisely, the second one.
When no syntax parser is given, the layout is not invalidated and the attributes are not fixed.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing
"*/
{
    if(self = [super init])
    {
        if(![aString length]) aString = [NSString string];
        [self->_MAS autorelease];
        self->_MAS = [[NSMutableAttributedString alloc] initWithString: aString attributes: attributes];
        [[NSNotificationCenter defaultCenter] removeObserver: self name: iTMInvalidateLayoutNotification object: nil];
        [[NSNotificationCenter defaultCenter] addObserver: self
            selector: @selector(invalidateLayoutNotified:)
                name: iTMInvalidateLayoutNotification
                    object: nil];
        [self setStringEncoding: 0];
        [self setSyntaxParser: SP];
//        [self initAnchorsControllers];
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initWithEncodedStringData:
- (id) initWithEncodedStringData: (NSData *) docData;
/*"This is the one of the three initializers available!!! More precisely, the third one.
When no syntax parser is given, the layout is not invalidated and the attributes are not fixed.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: BEWARE of unicode data beginning: not implemented.
"*/
{
    return [self initWithEncodedStringData: docData syntaxParser: nil];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= initWithEncodedStringData:syntaxParser:
- (id) initWithEncodedStringData: (NSData *) docData syntaxParser: (id) SP;
/*"This is the one of the three initializers available!!! More precisely, the third one.
When no syntax parser is given, the layout is not invalidated and the attributes are not fixed.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:.
"*/
{
    NSString * string;
    NSStringEncoding usedEncoding;
    int prefix = 0;
    // testing for unicode encoding
    if([docData length]>1 && ((prefix = [docData deserializeIntAtIndex: 0]), (prefix == 0xFFFE)) || (prefix == 0xFEFF))
    {
        usedEncoding = NSUnicodeStringEncoding;
        string = [[NSString alloc] initWithData: docData encoding: usedEncoding];
    }
    else if([[NSUserDefaults standardUserDefaults] boolForKey: iTMIsAutoStringEncodingKey])
    {
        NSStringEncoding automaticEncoding;
        NSStringEncoding preferredEncoding = [[NSUserDefaults standardUserDefaults]
                                                        integerForKey: iTMPreferredStringEncodingKey];
        NSString * headerEncoding;
        string = [[NSString alloc] initWithData: docData encoding: preferredEncoding];
        headerEncoding = [string stringForCommentedKey: iTMEncodingHeaderKey
                                    forRange: NSMakeRange(0, 0) effectiveRange: nil inHeader: YES];
        _iTMiTMFlags.canSetStringEncoding = ([headerEncoding length] > 0? 0: 1);

        automaticEncoding = [[iTMEncodingController
                                    stringEncodingFromString: headerEncoding] unsignedIntValue];
        // accepted encodings are those for which NSString gives a non void localizedNameOfStringEncoding...
        if(automaticEncoding && [[NSString localizedNameOfStringEncoding: automaticEncoding] length] &&
            (automaticEncoding != preferredEncoding))
        {
            [string release];
            string = [[NSString alloc] initWithData: docData encoding: automaticEncoding];
            usedEncoding = automaticEncoding;
        }
        else
        {
            usedEncoding = preferredEncoding;
        }
    }
    else
    {
        usedEncoding = [[NSUserDefaults standardUserDefaults] integerForKey: iTMOpenStringEncodingKey];
        if(!usedEncoding)
            usedEncoding = [[NSUserDefaults standardUserDefaults] integerForKey: iTMPreferredStringEncodingKey];
        NSAssert(usedEncoding>0, @"Broken code: default default encoding missing.");//
        string = [[NSString alloc] initWithData: docData encoding: usedEncoding];
        _iTMiTMFlags.canSetStringEncoding = 1;
    }
    self = [self initWithString: (string? string: [NSString string]) attributes: nil syntaxParser: SP];
    [self setStringEncoding: usedEncoding];
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= encodedStringDataAllowLossyConversion
- (NSData *) encodedStringDataAllowLossyConversion: (BOOL) flag;
/*"This is the one of the three initializers available!!! More precisely, the third one.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Nothing
"*/
{
    NSStringEncoding stringEncoding = [self dataStringEncoding];
    NSLog(@"Writing with %@ string encoding.", [NSString localizedNameOfStringEncoding: stringEncoding]);
    return [[[self string] stringByUsingLineEndings: [self lineEndingsUsed]]
					dataUsingEncoding: stringEncoding allowLossyConversion: flag];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= invalidateLayoutNotified:
- (void) invalidateLayoutNotified: (NSNotification *) aNotification;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([[aNotification object]isEqual: [self syntaxParser]])
        [self invalidateLayout];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= invalidateLayout
- (void) invalidateLayout;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSEnumerator * enumerator = [[self layoutManagers] objectEnumerator];
    NSLayoutManager * LM;
    NSRange range = NSMakeRange(0, [[self mutableString] length]);
    while(LM = [enumerator nextObject])
	[LM invalidateLayoutForCharacterRange: range isSoft: NO actualCharacterRange: nil];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= dealloc
- (void) dealloc;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [[NSNotificationCenter defaultCenter] removeObserver: nil name: nil object: self];
    [_MAS autorelease];
    _MAS = nil;
    [self setSyntaxParser: nil];
    #if 0
    [self setInputAnchorsController: nil];
    [self setMarkAnchorsController: nil];
    [self setSectionAnchorsController: nil];
    [self setLabelAnchorsController: nil];
    #endif
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= syntaxParser
- (id) syntaxParser;
/*"Lazy initializer, subclasses will override this message to return their own default syntax parser.
This implementation returns the shared iTMSyntaxParser;
Version History: jerome.laurens@u-bourgogne.fr (03/08/02)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _SP? _SP: (_SP = [[iTMSyntaxParser sharedSyntaxParser] retain]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setSyntaxParser:
- (void) setSyntaxParser: (id) argument;
/*"If there is a new syntax parser, it invalidates the attributes and the layout.
Version History: jerome.laurens@u-bourgogne.fr (11/07/01)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [iTMSyntaxParser class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] iTMSyntaxParser argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else
    {
        [_SP autorelease];
        _SP = [argument retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= mutableString
- (NSMutableString *) mutableString;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [_MAS mutableString];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= string
- (NSString *) string;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [_MAS string];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= mutableAttributedString
- (NSMutableAttributedString *) mutableAttributedString;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _MAS;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= attributesAtIndex:effectiveRange:
- (NSDictionary *) attributesAtIndex: (unsigned) aLocation effectiveRange: (NSRangePointer) aRangePtr;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self ensureAttributesAreFixedInRange: NSMakeRange(aLocation, 1)];
    return [[self syntaxParser] attributedString: _MAS attributesAtIndex: aLocation effectiveRange: aRangePtr];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= attributesAtIndex:longestEffectiveRange:inRange:
- (NSDictionary *) attributesAtIndex: (unsigned) aLocation longestEffectiveRange: (NSRangePointer) aRangePtr inRange: (NSRange) rangeLimit;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self ensureAttributesAreFixedInRange: NSMakeRange(aLocation, 1)];
    return [[self syntaxParser] attributedString: _MAS attributesAtIndex: aLocation longestEffectiveRange: aRangePtr inRange: rangeLimit];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= attribute:atIndex:effectiveRange:
- (NSDictionary *) attribute: (NSString *) name atIndex: (unsigned) aLocation effectiveRange: (NSRangePointer) aRangePtr;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self ensureAttributesAreFixedInRange: NSMakeRange(aLocation, 1)];
    return [[self syntaxParser] attributedString: _MAS attribute: name atIndex: aLocation effectiveRange: aRangePtr];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= attribute:atIndex:longestEffectiveRange:inRange:
- (NSDictionary *) attribute: (NSString *) name atIndex: (unsigned) aLocation longestEffectiveRange: (NSRangePointer) aRangePtr inRange: (NSRange) rangeLimit;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self ensureAttributesAreFixedInRange: NSMakeRange(aLocation, 1)];
    return [[self syntaxParser] attributedString: _MAS attribute: name atIndex: aLocation longestEffectiveRange: aRangePtr inRange: rangeLimit];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setAttributes:range:
- (void) setAttributes: (NSDictionary *) attributes range: (NSRange) aRange;
/*"Does nothing: no one should change the attributes except the receiver itself.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(@"someone has asked..., %@", attributes);
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= replaceCharactersInRange:withString:
- (void) replaceCharactersInRange: (NSRange) aRange withString: (NSString *) aString;
/*"Relies on the shared syntax parser.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [_MAS replaceCharactersInRange: aRange withString: aString];
    [self invalidateAttributesInRange:
	[[self syntaxParser] attributedString: _MAS invalidRangeForCharactersInOldRange: aRange replacedWithString: aString]];
    [self edited: NSTextStorageEditedCharacters range: aRange changeInLength: [aString length] - aRange.length];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= fixAttributesInRange:
- (void) fixAttributesInRange: (NSRange) aRange;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self syntaxParser] attributedString: _MAS fixAttributesInRange: aRange];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= fixesAttributesLazily
- (BOOL) fixesAttributesLazily;
/*"YES.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return YES;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= doubleClickAtIndex:
- (NSRange) doubleClickAtIndex: (unsigned) index;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: implement some kind of balance range for range
"*/
{
    return [_SP attributedString: _MAS doubleClickAtIndex: index
        isSmart: [[self settings] boolForKey: iTMUDSmartSelectionKey]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= isITMCommentAtIndex:
- (BOOL) isITMCommentAtIndex: (unsigned) aLocation;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self syntaxParser] attributedString: [self mutableAttributedString] isITMCommentAtIndex: aLocation];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= groupRangeAtIndex:
- (NSRange) groupRangeAtIndex: (unsigned) index;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[self mutableAttributedString] string] groupRangeAtIndex: index];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= lineEndingsUsed
- (unsigned int) lineEndingsUsed;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!_iTMiTMFlags.lineEndingsUsed)
        _iTMiTMFlags.lineEndingsUsed = [iTMEncodingController lineEndingsUsedInDocument: self] % 8;
    return _iTMiTMFlags.lineEndingsUsed;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setLineEndingsUsed:
- (void) setLineEndingsUsed: (unsigned int) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _iTMiTMFlags.lineEndingsUsed = argument % 8;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= stringEncoding
- (NSStringEncoding) stringEncoding;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _OpenStringEncoding;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= dataStringEncoding
- (NSStringEncoding) dataStringEncoding;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSStringEncoding stringEncoding = [[NSUserDefaults standardUserDefaults] boolForKey: iTMIsAutoStringEncodingKey]?
                            [[iTMEncodingController stringEncodingFromString:
                                [[self mutableString] stringForCommentedKey: iTMEncodingHeaderKey
                        forRange: NSMakeRange(0, 0) effectiveRange: nil inHeader: YES]] unsignedIntValue]:
                                                [self stringEncoding];
    if(!stringEncoding) stringEncoding = [self stringEncoding];
    if(!stringEncoding) stringEncoding = [[NSUserDefaults standardUserDefaults]
							integerForKey:iTMPreferredStringEncodingKey];
    return stringEncoding;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setStringEncoding:
- (void) setStringEncoding: (NSStringEncoding) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
   _OpenStringEncoding = argument;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= canSetStringEncoding
- (BOOL) canSetStringEncoding;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _iTMiTMFlags.canSetStringEncoding;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= processEditing
- (void) processEditing;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: cache some information...
"*/
{
    [super processEditing];
    _iTMiTMFlags.canSetStringEncoding = [[[self mutableString]
            stringForCommentedKey: iTMEncodingHeaderKey
                    forRange: NSMakeRange(0, 0) effectiveRange: nil inHeader: YES] length]? 0: 1;
    return;
}

@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMTextStorage

