/*
//  iTMRepAlbumView.h
//  TBR Tutorial
//
//  Created by jerome.laurens@u-bourgogne.fr on Wed Jun 27 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
*/
#import <Cocoa/Cocoa.h>

extern NSString * const iTMMagnificationDidChangeNotification;

extern NSString * const iTMMagnificationKey;
extern NSString * const iTMUDRepAlbumMarginKey;

extern NSString * const iTMUDRepLastMagnificationKey;
extern NSString * const iTMUDRepCurrentMagnificationKey;
extern NSString * const iTMUDRepFixedMagnificationKey;
extern NSString * const iTMUDRepAlbumDisplayModeKey;
extern NSString * const iTMUDRepAlbumStickModeKey;

typedef enum
{
    iTMFixedMode = 0, // newly created album have allways the same magnification
    iTMLastMode = 1, // newly created album have allways the last used magnification
    iTMStickMode = 2 // newly created album adapt their magnification to the enclosing view
} iTMAlbumDisplayMode;

// following modes are considered only in iTMStickMode
typedef enum
{
    iTMStickToWidthMode = 0,
    iTMStickToHeightMode = 1,
    iTMStickToViewMode = 2
} iTMAlbumStickMode;

#if 0
typedef enum
{
    iTMPortraitMode = 0,
    iTMLeftLandscapeMode = 1,
    iTMRightLandscapeMode = -1
} iTMOrientationMode;
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMRepAlbumView  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

@interface iTMRepAlbumView : NSView
{
@private
    NSDecimalNumber * _Magnification;
    BOOL _CanStick;
    BOOL _ParametersHaveChanged;
    int _OrientationMode;
}
/*"Class mathods."*/
+ (void) initialize;
+ (NSString *) currentMagnificationKey;
+ (NSString *) lastMagnificationKey;
+ (NSString *) fixedMagnificationKey;
+ (NSString *) displayModeKey;
+ (NSString *) stickModeKey;
/*"Setters and getters."*/
- (BOOL) canStickToWindow;
- (NSDecimalNumber *) magnification;
- (void) setMagnification: (NSDecimalNumber *) aMagnification;
- (NSDecimalNumber *) innerMagnification;
- (NSDecimalNumber *) outerMagnification;
- (NSDecimalNumber *) ratioContentVersusDocumentWidth;
- (NSDecimalNumber *) ratioContentVersusDocumentHeight;
- (BOOL) parametersHaveChanged;
- (void) setParametersHaveChanged: (BOOL) aFlag;
- (int) orientation;
- (void) setOrientation: (int) argument;
- (void) setMagnificationWithDisplayMode: (int) displayMode stickMode: (int) stickMode;
/*"Main mathods."*/
- (void) fixMagnification;
- (void) enclosingScrollViewFrameDidChange: (NSNotification*) notification;
- (void) registerCurrentMagnification;
- (void) prepareToUpdate;
- (void) update;
- (id) representation;
- (void) awakeFromNib;
- (NSView *) newSubview;
- (void) updateSubview: (NSView *) subview;
/*"Overriden methods."*/
- (void) setNeedsDisplay: (BOOL) aFlag;
- (void) drawRect: (NSRect) rect;
- (void) dealloc;
@end

@interface NSView(iTMRepAlbumView)
- (NSPoint) absoluteFocus;
@end


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMRepAlbumView  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
