/*
//  iTMPDFWindowController.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sun Jan 06 2002.
//  Copyright © 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/Foundation.h>
#import <AppKit/NSTextField.h>
#import <AppKit/AppKit.h>
#import "NSResponder(iTeXMac).h"
#import "iTMPDFWindowController.h"
#import "iTMPDFAlbumView.h"
#import "iTMPDFView.h"
#import "iTMFlagsChangedView.h"
#import "iTMPDFToolbarDelegate.h"
#import "iTMSettings.h"
#import "iTMDocumentController.h"

NSString * const iTMUDPDFCascadeWindowsKey = @"iTMUDPDFCascadeWindows";

@interface NSDocument(iTMPDFWindowController)
- (NSPDFImageRep *) representation;
@end

@interface iTMPDFWindowController(iTMPrivate)
- (Class) _ToolbarDelegateClass;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMPDFWindowController
/*"Description forthcoming."*/
@implementation iTMPDFWindowController
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults:
        [NSDictionary dictionaryWithObjectsAndKeys:
            [NSNumber numberWithBool: NO], iTMUDPDFCascadeWindowsKey,
                nil]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  _ToolbarDelegateClass
- (Class) _ToolbarDelegateClass;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (11/27/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([self respondsToSelector: @selector(PDFToolbarDelegateClass)])
        return [self PDFToolbarDelegateClass];
    else
        return [iTMPDFToolbarDelegate class];        
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowDidLoad
- (void) windowDidLoad;
/*"Cocoa design.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{    
    [super windowDidLoad];
    [iTMFlagsChangedResponder installResponderForWindow: [self window]];
    [self setCurrentPage: 0];
    [[self window] makeFirstResponder: [self album]];
    [self setShouldCascadeWindows: [[self settings] boolForKey: iTMUDPDFCascadeWindowsKey]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toolbarIdentifier
- (NSString *) toolbarIdentifier;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return @"PDF";
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= firstPage
- (int) firstPage;
/*"Description forthcoming."*/
{
    return 0;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= lastPage
- (int) lastPage;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[self document] representation] pageCount]-1;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= currentPage
- (int) currentPage;
/*"The album is the chief.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self album] currentPage];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setCurrentPage:
- (void) setCurrentPage: (int) aCurrentPage;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    aCurrentPage = MIN(aCurrentPage, [self lastPage]);
    if([self currentPage] != aCurrentPage)
    {
        [[NSNotificationCenter defaultCenter] postNotificationName: @"iTMPageNumberDidChange"
            object: self
                userInfo: [NSDictionary dictionaryWithObjectsAndKeys:
                    [NSNumber numberWithInt: [self currentPage]], @"OldPage",
                    [NSNumber numberWithInt: aCurrentPage], @"NewPage",
                    [NSNumber numberWithInt: [[[self document] representation] pageCount]], @"PageCount", nil]];
        [self synchronizePageWith: [NSNumber numberWithInt: aCurrentPage]];
        [[self album] setParametersHaveChanged: YES];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= synchronizePageWith:
- (void) synchronizePageWith: (NSNumber *) aCurrentPage;
/*"Sends a #{-setCurrentPage} message to the UIManager and the album.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self album] setCurrentPage: [aCurrentPage intValue]];
    #warning next is replaced by a notification ? to be tested...
//    [[[self toolbarDelegate] navigationField] setIntValue: [aCurrentPage intValue]+1];
    // critical: due to the flags changed responder, not so cool but works well
    [[[self album] window] makeFirstResponder: [self album]];
    return;
}
@end

@implementation iTMPDFWindowController(Navigation)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= displayPageNumber:
- (void) displayPageNumber: (NSNumber *) sender;
/*"Description Forthcoming. The first responder must never be the window but at least its content view unless we want to neutralize the iTMFlagsChangedResponder.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setCurrentPage: MAX(0, [sender intValue]-1)];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= displayFirstPage:
- (void) displayFirstPage: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setCurrentPage: [self firstPage]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= displayLastPage:
- (void) displayLastPage: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setCurrentPage: [self lastPage]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= displayPreviousPage:
- (void) displayPreviousPage: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setCurrentPage: MAX([[self album] currentPage]-1, [self firstPage])];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= displayNextPage:
- (void) displayNextPage: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setCurrentPage: MIN([[self album] currentPage]+1, [self lastPage])];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= displayPreviousPreviousPage:
- (void) displayPreviousPreviousPage: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setCurrentPage: MAX([[self album] currentPage]-5, [self firstPage])];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= displayNextNextPage:
- (void) displayNextNextPage: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setCurrentPage: MIN([[self album] currentPage]+5, [self lastPage])];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= displayForwardPage:
- (void) displayForwardPage: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    int CPI = [self currentPage];
    if([[[self toolbarDelegate] forwardMenu] numberOfItems]>0)
    {
        [[[self toolbarDelegate] backMenu] insertItemWithTitle: [[NSNumber numberWithInt: CPI+1] stringValue]
                        action: @selector(displayBackPageFromItem:) keyEquivalent: @"" atIndex: 0];
        [[[[self toolbarDelegate] backMenu] itemAtIndex: 0] setTarget: self];
        [[[[self toolbarDelegate] backMenu] itemAtIndex: 0] setTag: CPI];
        [self synchronizePageWith: [NSNumber numberWithInt: [[[[self toolbarDelegate] forwardMenu] itemAtIndex: 0] tag]]];
        [[[self toolbarDelegate] forwardMenu] removeItemAtIndex: 0];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= displayBackPage:
- (void) displayBackPage: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    int CPI = [self currentPage];
    if([[[self toolbarDelegate] backMenu] numberOfItems]>0)
    {
        [[[self toolbarDelegate] forwardMenu] insertItemWithTitle: [[NSNumber numberWithInt: CPI+1] stringValue]
                        action: @selector(displayForwardPageFromItem:) keyEquivalent: @"" atIndex: 0];
        [[[[self toolbarDelegate] forwardMenu] itemAtIndex: 0] setTarget: self];
        [[[[self toolbarDelegate] forwardMenu] itemAtIndex: 0] setTag: CPI];
        [self synchronizePageWith: [NSNumber numberWithInt: [[[[self toolbarDelegate] backMenu] itemAtIndex: 0] tag]]];
        [[[self toolbarDelegate] backMenu] removeItemAtIndex: 0];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= displayBackPageFromItem:
- (void) displayBackPageFromItem: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    {
    int CPI = [self currentPage];
        [[[self toolbarDelegate] forwardMenu] insertItemWithTitle: [[NSNumber numberWithInt: CPI+1] stringValue]
                        action: @selector(displayForwardPageFromItem:) keyEquivalent: @"" atIndex: 0];
        [[[[self toolbarDelegate] forwardMenu] itemAtIndex: 0] setTarget: self];
        [[[[self toolbarDelegate] forwardMenu] itemAtIndex: 0] setTag: CPI];
    }
    {
        NSMenuItem * menuItem;
        while((menuItem = [[[self toolbarDelegate] backMenu] itemAtIndex: 0]) && ![menuItem isEqual: sender])
        {
            [menuItem retain];
            [[[self toolbarDelegate] backMenu] removeItem: menuItem];
            [[[self toolbarDelegate] forwardMenu] insertItem: menuItem atIndex: 0];
            [menuItem setAction: @selector(displayForwardPageFromItem:)];
            [menuItem release];
        }
        [[[self toolbarDelegate] backMenu] removeItem: menuItem];
    }
    [self synchronizePageWith: [NSNumber numberWithInt: [sender tag]]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= displayForwardPageFromItem:
- (void) displayForwardPageFromItem: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    {
    int CPI = [self currentPage];
        [[[self toolbarDelegate] backMenu] insertItemWithTitle: [[NSNumber numberWithInt: CPI+1] stringValue]
                        action: @selector(displayBackPageFromItem:) keyEquivalent: @"" atIndex: 0];
        [[[[self toolbarDelegate] backMenu] itemAtIndex: 0] setTarget: self];
        [[[[self toolbarDelegate] backMenu] itemAtIndex: 0] setTag: CPI];
    }
    {
        NSMenuItem * menuItem;
        while((menuItem = [[[self toolbarDelegate] forwardMenu] itemAtIndex: 0]) && ![menuItem isEqual: sender])
        {
            [menuItem retain];
            [[[self toolbarDelegate] forwardMenu] removeItem: menuItem];
            [[[self toolbarDelegate] backMenu] insertItem: menuItem atIndex: 0];
            [menuItem setAction: @selector(displayBackPageFromItem:)];
            [menuItem release];
        }
        [[[self toolbarDelegate] forwardMenu] removeItem: menuItem];
    }
    [self synchronizePageWith: [NSNumber numberWithInt: [sender tag]]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= displayPageFromField:
- (void) displayPageFromField: (id) sender;
/*"Description Forthcoming. The first responder must never be the window but at least its content view unless we want to neutralize the iTMFlagsChangedResponder.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self displayPageNumber: sender];
    [[self window] makeFirstResponder: [self album]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateUserInterfaceItem:
- (BOOL) validateUserInterfaceItem: (id) anItem;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: NOT TESTED
"*/
{
    SEL action = [anItem action];
    if(action == @selector(displayFirstPage:) ||
        action == @selector(displayPreviousPreviousPage:) ||
            action == @selector(displayPreviousPage:))
    {
       return ([[self document] representation] != nil) && ([self currentPage] >[self firstPage]);
    }
    else if(action == @selector(displayNextPage:) ||
        action == @selector(displayNextNextPage:) ||
            action == @selector(displayLastPage:))
    {
        return [[self document] representation]!=nil && [self currentPage]<[self lastPage];
    }
    else if(action == @selector(allwaysEnableSender:))
    {
        return YES;
    }
    else if(action == @selector(displayPageFromField:))
    {
        return YES;
    }
    else if(action == @selector(displayForwardPage:))
    {
        return [[[self toolbarDelegate] forwardMenu] numberOfItems]>0;
    }
    else if(action == @selector(displayBackPage:))
    {
        return [[[self toolbarDelegate] backMenu] numberOfItems]>0;
    }
    else if(action == @selector(displayPageFromField:))
    {
        return([[[[[self toolbarDelegate] navigationField] formatter] maximum] intValue] > 1);
    }
    else
        return NO;
    #warning add the stepper ?
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMenuItem:
- (BOOL) validateMenuItem: (id) aMenuItem;
/*" This concerns only the magnification menu items of the PDF Display menu and the
corresponding ones in the popUp Fit button. Only takes into account the upper limit
of the magnification."*/
{
    SEL action = [aMenuItem action];
    if(action == @selector(displayFirstPage:) ||
            action == @selector(displayPreviousPage:) ||
            action == @selector(displayPreviousPreviousPage:))
    {
        return ([[self document] representation] != nil) && ([self currentPage] >0);
    }
    else if(action == @selector(displayLastPage:) ||
            action == @selector(displayNextPage:) ||
            action == @selector(displayNextNextPage:))
        return ([[self document] representation] != nil) && ([self currentPage] < [self lastPage]);
    else if(action == @selector(displayBackPageFromItem:))
        return YES;
    else if(action == @selector(displayForwardPageFromItem:))
        return YES;
    else if(action == @selector(displayForwardPage:))
        return ([[[self toolbarDelegate] forwardMenu] numberOfItems] >0);
    else if(action == @selector(displayBackPage:))
        return ([[[self toolbarDelegate] backMenu] numberOfItems] >0);
    else if(action == @selector(toggleAutoUpdatesView:))
    {
        [aMenuItem setState: [[self settings] boolForKey: iTMUDAutoUpdatesViewKey]? NSOnState: NSOffState];
        return ![[self settings] boolForKey: iTMUDUsesPDFHelperKey];
    }
    else if(action == @selector(toggleSmartUpdatesView:))
    {
        [aMenuItem setState: [[self settings] boolForKey: iTMUDSmartUpdatesViewKey]? NSOnState: NSOffState];
        return ![[self settings] boolForKey: iTMUDUsesPDFHelperKey];
    }
    else if(action == @selector(toggleThreadedPDFRendering:))
    {
        [aMenuItem setState: [[self settings] boolForKey: iTMUDThreadedPDFRenderingKey]? NSOnState: NSOffState];
        return YES;
    }
    else if(action == @selector(toggleNewPDFPageMode:))
    {
        [aMenuItem setState: ([[self settings] integerForKey: iTMUDNewPDFPageModeKey] ==[aMenuItem tag])?
            NSOnState: NSOffState];
        return YES;
    }
    else
        return [super validateMenuItem: aMenuItem];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleAutoUpdatesView:
- (IBAction) toggleAutoUpdatesView: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self settings] setBool: ([sender state] == NSOffState) forKey: iTMUDAutoUpdatesViewKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleSmartUpdatesView:
- (IBAction) toggleSmartUpdatesView: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL wasOnState = ([sender state] != NSOffState);
    [[self settings] setBool: !wasOnState forKey: iTMUDSmartUpdatesViewKey];
    if(!wasOnState)
        [[self settings] setBool: YES forKey: iTMUDAutoUpdatesViewKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleThreadedPDFRendering:
- (IBAction) toggleThreadedPDFRendering: (id) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self settings] setBool: ([sender state] == NSOffState) forKey: iTMUDThreadedPDFRenderingKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= toggleNewPDFPageMode:
- (void) toggleNewPDFPageMode: (id <NSMenuItem>) sender;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self settings] setInteger: [sender tag] forKey: iTMUDNewPDFPageModeKey];
    return;
}
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMPDFWindowController

