/*
//  iTMRepDocument.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sun Jun 24 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
*/

#import <Foundation/Foundation.h>
//#import <AppKit/NSStepper.h>
#import <AppKit/NSPrintInfo.h>
#import <AppKit/AppKit.h>
#import "NSMenu(iTeXMac).h"
#import "iTMRepDocument.h"
#import "iTMRepWindowController.h"
#import "iTMListMenu.h"
#import "iTMMagnificationFormatter.h"
#import "iTMPrintView.h"
#import "iTMSettings.h"

NSString * const iTMRepDocumentType = @"Representation Document";

NSString * const iTMUDSSetUpPageWhenBadPaperSizeKey = @"iTMPDFSetUpPageWhenBadPaperSize";
NSString * const iTMUDPreferA4PaperKey = @"iTMUDPreferA4Paper";


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMRepDocument  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
/*"Basics of the view documents."*/
@implementation iTMRepDocument
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  makeRepresentationWC
- (void) makeRepresentationWC;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self removeWindowController: [self representationWC]];
    [self setRepresentationWC: nil];
    if([self data])
        [self addWindowController: [self representationWC]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  representationWindow
- (id) representationWindow;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[self representationWC] window];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= fileModificationDate
- (NSDate *) fileModificationDate;
/*"Description forthcoming."*/
{
    return _FMD;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setFileModificationDate:
- (void) setFileModificationDate: (NSDate *) argument;
/*"Description forthcoming."*/
{
    if(![argument isEqual: _FMD])
    {
        [_FMD release];
        _FMD = [argument retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  update
- (void) update;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: save the file, if it has disappeared from the HD.
"*/
{
//    NSLog(@"Updating file %@...\nFMD: %@<\nnewFMD: %@", [self fileName], [self fileModificationDate], [[[NSFileManager defaultManager] fileAttributesAtPath: [self fileName] traverseLink: NO] fileModificationDate]);
    if(([[[[NSFileManager defaultManager] fileAttributesAtPath: [self fileName] traverseLink: NO]
                    fileModificationDate] compare: [self fileModificationDate]] == NSOrderedDescending))
        if(![self readFromFile: [self fileName] ofType: [self fileType]])
        {
            NSRunCriticalAlertPanel(
                NSLocalizedStringFromTableInBundle(@"Updating representation.", @"View",
                                [NSBundle bundleForClass: [self class]], "Informational Panel Title"),
                NSLocalizedStringFromTableInBundle(@"Unable to update.", @"View",
                                [NSBundle bundleForClass: [self class]], "Problem"),
                nil, nil, nil);
        }
#if 0
        else
            NSLog(@"Could read from file...");
    else
        NSLog(@"Nothing to update");
#endif
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  representationWCClass
- (Class) representationWCClass;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [iTMRepWindowController class];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  representationWC
- (id) representationWC;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!_RepresentationWC)
        _RepresentationWC = [[[self representationWCClass] alloc] initWithWindowNibName: NSStringFromClass([self class])];
    return _RepresentationWC;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setRepresentationWC:
- (void) setRepresentationWC: (id) aWC;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![_RepresentationWC isEqual: aWC])
    {
        [_RepresentationWC release];
        _RepresentationWC = [aWC retain];
        [_RepresentationWC setDocument: self];
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= data
- (NSData *) data;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _Data;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setData:
- (void) setData: (NSData *) data;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![data isEqual: _Data])
    {
        [_Data release];
        _Data = [data retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= representation
- (id) representation;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _Representation;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setRepresentation:
- (void) setRepresentation: (id) aRepresentation;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![aRepresentation isEqual: _Representation])
    {
        [_Representation release];
        _Representation = [aRepresentation retain];
        [[NSNotificationCenter defaultCenter] postNotificationName: @"iTMRepresentationDidChange"
            object: self
                userInfo: nil];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= settings
- (id) settings;
/*"Lazy initializer.
Version History: jerome.laurens@u-bourgogne.fr
- 1.1.b: 04/16/2002
To Do List:
"*/
{
    return _Settings? _Settings: (_Settings = [[iTMSettings alloc] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= setSettings:
- (void) setSettings: (id) argument;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [_Settings autorelease];
    _Settings = [argument retain];
    return;
}
@end

@implementation iTMRepDocument(Cocoa)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Description forthcoming."*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:
        [NSNumber numberWithBool: NO], iTMUDPreferA4PaperKey,
        [NSNumber numberWithBool: NO], iTMUDSSetUpPageWhenBadPaperSizeKey,
            nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  init
- (id) init;
/*"No undo manager: document status is never changed, even when changing the layout parameters.
The paper name print info is set to A4 if needed.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    self = [super init];
    [self setHasUndoManager: NO];
    {
        NSPrintInfo * PI = [[NSPrintInfo sharedPrintInfo] copyWithZone: [self zone]];
        if([[self settings] boolForKey: iTMUDPreferA4PaperKey])
            [PI setPaperName: @"A4"];
        [self setPrintInfo: PI];
        [PI release];
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [self setData: nil];
    [self setRepresentation: nil];
    [self setRepresentationWC: nil];
    [self setFileModificationDate: nil];
    [super dealloc];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  printShowingPrintPanel:
- (void) printShowingPrintPanel: (BOOL) flag;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSImageRep * representation = [[NSImageRep imageRepClassForData: [self data]] imageRepWithData: [self data]];
    NSPrintInfo * PI = [self printInfo];
    if([[self settings] boolForKey: iTMUDSSetUpPageWhenBadPaperSizeKey])
    {
        NSSize paperSize = [PI paperSize];
        BOOL setupPage = NO;
        NSLog(@"paperSize: %@", NSStringFromSize(paperSize));
        if((paperSize.width>0) && (paperSize.height>0))
        {
            NSSize repSize = [[[NSImageRep imageRepClassForData: [self data]] imageRepWithData: [self data]] size];
            float dH = paperSize.height - repSize.height;
            float dW = paperSize.width - repSize.width;
            float dWH = dH*dH+dW*dW;
            float WH = paperSize.height*paperSize.height + paperSize.width*paperSize.width;
            NSLog(@"repSize: %@", NSStringFromSize(repSize));
            if(dWH>0.0001*WH)
            {
                NSLog(@"bad paper size");
                setupPage = YES;
            }
        }
        else
            setupPage = YES;
        if(setupPage)
            [self runPageLayout: self];
    }
    {
        iTMPrintView * view = [[iTMPrintView allocWithZone: [self zone]] initWithRepresentation: representation];
        NSPrintOperation * PO = [NSPrintOperation printOperationWithView: view printInfo: PI];
        NSWindow * window = [[NSWindow allocWithZone:[self zone]]
                initWithContentRect: NSMakeRect(0.0, 0.0, [view frame].size.width, [view frame].size.height)
                        styleMask: NSBorderlessWindowMask backing: NSBackingStoreNonretained defer: NO];
        [[window contentView] addSubview: view];
        [PO setShowPanels: flag];
        [PO setCanSpawnSeparateThread: YES];
        [PO runOperation];
        [view release];
        [window release];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  loadFileWrapperRepresentation:ofType:
- (BOOL) loadFileWrapperRepresentation: (NSFileWrapper *) wrapper ofType: (NSString *) docType;
/*"Beware, this is not meant to be called directly.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL result = [super loadFileWrapperRepresentation: wrapper ofType: docType];
    if(result)
    	[self setFileModificationDate: [[wrapper fileAttributes] fileModificationDate]];
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  loadDataRepresentation:ofType:
- (BOOL) loadDataRepresentation: (NSData *) data ofType: (NSString *) aType;
/*" Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    Class C = [NSImageRep imageRepClassForData: data];
    if(C)
    {
        [self setData: data];
        [self setRepresentation: [[[C alloc] initWithData: data] autorelease]];
    }
    return [self representation] != nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dataRepresentationOfType:
- (NSData *) dataRepresentationOfType: (NSString *) aType;
/*" Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [self data];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  makeWindowControllers
- (void) makeWindowControllers;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self makeRepresentationWC];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isDocumentEdited
- (BOOL) isDocumentEdited;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return NO;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMRepDocument  =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
