/*
//  iTMRepWindowController(iTMMagnification).m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Sat Jan 26 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "iTMRepWindowController(iTMMagnification).h"
#import "iTMRepAlbumView.h"
#import "iTMMagnificationFormatter.h"
#import "iTMSettings.h"

@interface NSDocument(iTMRepWindowController)
- (NSData *) data;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMRepWindowController(iTMMagnification)
/*"The window controller is the delegate of the window to catch all the messages. It is a window controller of an iTMRepDocument."*/
@implementation iTMRepWindowController(iTMMagnification)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  displayZoomIn:...
- (IBAction) displayZoomIn: (id) sender
/*"And sets the display mode to iTMLastMode. (abandon any other mose!)
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self album] setMagnification: [[self album] innerMagnification]];
    [[self settings] setInteger: iTMLastMode forKey: [[[self album] class] displayModeKey]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  displayZoomOut:...
- (IBAction) displayZoomOut: (id) sender
/*"And sets the display mode to iTMLastMode.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self album] setMagnification: [[self album] outerMagnification]];
    [[self settings] setInteger: iTMLastMode forKey: [[[self album] class] displayModeKey]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= toggleStickToWindow:
- (void) toggleStickToWindow: (id) sender;
/*"Toggles the display mode between iTMStickMode and iTMLastMode.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    switch([[self settings] integerForKey: [[[self album] class] displayModeKey]])
    {
        case iTMStickMode:
            [[self settings] setInteger: iTMLastMode forKey: [[[self album] class] displayModeKey]];
            break;
        default:
            [[self settings] setInteger: iTMStickMode forKey: [[[self album] class] displayModeKey]];
            [[self album] fixMagnification]; 
            break;
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=  displayAtFixedSize:...
- (IBAction) displayAtFixedSize: (id) sender
/*"It is the message sent by a menu item of the display menu in the View menu.
It sets the display mode to iTMFixedMode then sends a #{fixMagnification}message to the album.
As a side effect, it updates the album display if the magnification has changed.
Version History: jerome.laurens@u-bourgogne.fr
- 1.1a6 : 03/10/2002
To Do List:
"*/
{
    [[self settings] setInteger: iTMLastMode forKey: [[[self album] class] displayModeKey]];
    [[self album] setMagnificationWithDisplayMode: iTMFixedMode stickMode: 0];// unused stick mode
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleStickToWidth:...
- (IBAction) toggleStickToWidth: (id) sender
/*"Radio width|height|view. Sets the display mode to iTMStickMode, sets the stick mode to iTMStickToWidthMode,
then #{fixMagnification}of the album. The display is a side effect.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self settings]setInteger: iTMStickMode forKey: [[[self album] class] displayModeKey]];
    [[self settings]setInteger: iTMStickToWidthMode forKey: [[[self album] class] stickModeKey]];
    [[self album] fixMagnification];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=  displayFitToWidth:...
- (IBAction) displayFitToWidth: (id) sender
/*"Just sets the magnification of the album: the display will come from a side effect.
The display mode switches to iTMLastMode.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self settings]setInteger: iTMLastMode forKey: [[[self album] class] displayModeKey]];
    [[self album] setMagnificationWithDisplayMode: iTMStickMode stickMode: iTMStickToWidthMode];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleStickToHeight:...
- (IBAction) toggleStickToHeight: (id) sender
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self settings]setInteger: iTMStickMode forKey: [[[self album] class] displayModeKey]];
    [[self settings]setInteger: iTMStickToHeightMode forKey: [[[self album] class] stickModeKey]];
    [[self album] fixMagnification];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=  displayFitToHeight:...
- (IBAction) displayFitToHeight: (id) sender
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self settings]setInteger: iTMLastMode forKey: [[[self album] class] displayModeKey]];
    [[self album] setMagnificationWithDisplayMode: iTMStickMode stickMode: iTMStickToHeightMode];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleStickToView:...
- (IBAction) toggleStickToView: (id) sender
/*"See #{toggleStickToWidth}.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self settings]setInteger: iTMStickMode forKey: [[[self album] class] displayModeKey]];
    [[self settings]setInteger: iTMStickToViewMode forKey: [[[self album] class] stickModeKey]];
    [[self album] fixMagnification];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  displayFitToView:...
- (IBAction) displayFitToView: (id) sender
/*"See #{displayFitToWidth}.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self settings]setInteger: iTMLastMode forKey: [[[self album] class] displayModeKey]];
    [[self album] setMagnificationWithDisplayMode: iTMStickMode stickMode: iTMStickToViewMode];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  displayAtMagnification:
- (void) displayAtMagnification: (id) sender;
/*"Description forthcoming. From the menu items
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([sender representedObject])
    {
        [[self settings]setInteger: iTMLastMode forKey: [[[self album] class] displayModeKey]];
        [[self album] setMagnification: [sender representedObject]];
    }
    else
    {
        iTMMagnificationFormatter * MF = [[iTMMagnificationFormatter alloc] init];
        NSDecimalNumber * number;
        if([MF getObjectValue: &number forString: [sender title] errorDescription: nil])
        {
            [[self settings]setInteger: iTMLastMode forKey: [[[self album] class] displayModeKey]];
            [[self album] setMagnification: number];
        }
        else
        {
            [sender setAction: NULL];
            NSLog(@"Unexpected sender(action): %@", sender);
        }
        [MF release];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  displayAtStepperMagnification:
- (void) displayAtStepperMagnification: (id) sender;
/*"Zoom In or Zoom Out.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([sender intValue]>0)
        [self displayZoomIn: sender];
    else
        [self displayZoomOut: sender];
    [sender setIntValue: 0];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  magnificationFieldSetMagnification:
- (void) magnificationFieldSetMagnification: (id) sender;
/*"Display mode is set to iTMLastMode.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self settings]setInteger: iTMLastMode forKey: [[[self album] class] displayModeKey]];
    [[self album] setMagnification: [sender objectValue]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=  toggleDisplayOrientation:
- (IBAction) toggleDisplayOrientation: (id) sender
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[self album] setOrientation: [sender tag]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateToolbarItem:
- (BOOL) validateToolbarItem: (id) anItem;
/*"Description forthcoming.
Version History: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    SEL action = [anItem action];
    if(action == @selector(displayAtMagnification:))
        return [[self document] data]!=nil;
    else if(action == @selector(printDocument:))
        return [[self document] data]!=nil;
    else
        return NO;
    #warning add the stepper ?
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMenuItem:
- (BOOL) validateMenuItem: (id) anItem;
/*" This concerns only the magnification menu items of the PDF Display menu and the
corresponding ones in the popUp Fit button. Only takes into account the upper limit
of the magnification."*/
{
#warning JL
    SEL action = [anItem action];
    BOOL hasRepresentation = ([[self document] data] != nil);
    if(action == @selector(displayZoomIn:))
        return hasRepresentation;// &&
                    //([self magnification] < [[[self newMagnificationFormatter] maximum] floatValue]);
    else if(action == @selector(displayZoomOut:) ||
            action == @selector(displayAtMagnification:) ||
            action == @selector(displayFitToWidth:) ||
            action == @selector(displayFitToHeight:) ||
            action == @selector(displayFitToView:))
        return hasRepresentation;
    else if(action == @selector(displayAtFixedSize:))
    {
        float f = [[self settings]floatForKey: [[[self album] class]fixedMagnificationKey]]* 100;
        [anItem setTitle: [NSString stringWithFormat:
            NSLocalizedStringFromTableInBundle(@"At %.0f %%", @"View", [NSBundle bundleForClass: [self class]],
                "Format for a display at fixed magnification menu item"), f]];
        return f > 0;
    }
    else if(action == @selector(toggleStickToWindow:))
    {
        [anItem setState: ([[self settings] integerForKey: [[[self album] class] displayModeKey]] == iTMStickMode?
            NSOnState: NSOffState)];
        return YES;
    }
    else if(action == @selector(toggleStickToWidth:))
    {
        [anItem setState: (([[self settings] integerForKey: [[[self album] class] stickModeKey]] == iTMStickToWidthMode)?
                    NSOnState: NSOffState)];
        return [[self settings] integerForKey: [[[self album] class] displayModeKey]] == iTMStickMode;
    }
    else if(action == @selector(toggleStickToHeight:))
    {
        [anItem setState: (([[self settings] integerForKey: [[[self album] class] stickModeKey]] == iTMStickToHeightMode)?
                    NSOnState: NSOffState)];
        return [[self settings] integerForKey: [[[self album] class] displayModeKey]] == iTMStickMode;
    }
    else if(action == @selector(toggleStickToView:))
    {
        [anItem setState: (([[self settings] integerForKey: [[[self album] class] stickModeKey]] == iTMStickToViewMode)?
                    NSOnState: NSOffState)];
        return [[self settings] integerForKey: [[[self album] class] displayModeKey]] == iTMStickMode;
    }
    else if(action == @selector(toggleDisplayOrientation:))
    {
        [anItem setState: (([anItem tag] == [[self album]orientation])? NSOnState: NSOffState)];
        return YES;
    }
    else
        return NO;
}
@end


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMRepWindowController(iTMMagnification)

