/*
//  iTMMacroMenuItem.h
//  macros
//
//  Created by jerome.laurens@u-bourgogne.fr on Sun Jan 20 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <AppKit/AppKit.h>

extern NSString * const iTMMacroMenuItemPBType;
extern NSString * const iTMMacroMenuItemsPBType;

extern NSString * const iTMMacroMenuItemDidChangeStatusNotification;

@class iTMTextMacro;

typedef enum _iTMMacroChangeType {
    iTMMacroChangeDone = 0,
    iTMMacroChangeUndone = 1,
    iTMMacroChangeCleared = 2
} iTMMacroChangeType;

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMMacroMenuItem

@interface iTMMacroMenuItem : NSMenuItem
{
@private
    iTMTextMacro * _Macro;
    NSUndoManager * _UM;
    long _ChangeCount;
    BOOL _HasUndoManager;
    BOOL _ChangesArePending;
    BOOL _actionModified;
}
/*"Class methods"*/
/*"Setters and Getters"*/
- (id) macro;
- (void) setMacro: (id) argument;
- (NSUndoManager *) undoManager;
- (void) setUndoManager: (NSUndoManager *) undoManager;
/*"Main methods"*/;
/*"Overriden methods"*/
- (void) dealloc;
- (void) encodeWithCoder: (NSCoder *) aCoder;
- (id) initWithCoder: (NSCoder *) aDecoder;
@end

@interface iTMMacroMenuItem(iTMUndoManagement)
- (BOOL) changesArePending;
- (void) setChangesArePending: (BOOL) aFlag;
- (void) updateChangeCount: (iTMMacroChangeType) change;
- (BOOL) isMacroMenuItemEdited;
- (void) updateChangeCount: (iTMMacroChangeType) change;
- (BOOL) hasUndoManager;
- (void) setHasUndoManager: (BOOL) flag;
- (void) undoManagerDidRedoChange: (NSNotification *) aNotification;
- (void) undoManagerDidUndoChange: (NSNotification *) aNotification;
- (void) undoManagerWillCloseUndoGroup: (NSNotification *) aNotification;
@end

@interface NSMenuItem(iTMMacro)
- (BOOL) isEqualOrHasSubmenuItem: (NSMenuItem *) argument;
@end

@interface NSMenu(iTMMacro)
- (void) clearMacroItemsChangeStatus;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMMacroMenuItem
