/*
//  iTMMacroDocument(iTMAction).m
//  macros
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Jan 15 2002.
//  Copyright  2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSArray.h>
#import <Foundation/Foundation.h>
#import <AppKit/NSMenu.h>
#import <AppKit/NSWindow.h>
#import <AppKit/NSOutlineView.h>
#import <AppKit/AppKit.h>
#import "iTMMacroDocument(iTMAction).h"
#import "iTMMacroOutlineView.h"
#import "iTMMacroMenuItem.h"
#import "iTMTextMacro.h"
#import "iTMHelpManager.h"
#import "NSMenu(iTeXMac).h"

@interface iTMMacroDocument(iTMUIValidation)
//- (void) validateAddItem: (id) sender;
//- (void) validateAddMenu: (id) sender;
//- (void) validateRemoveItem: (id) sender;
- (void) validateActionPopUpAction: (id) sender;
- (void) validateActionInsertSpecial: (id) sender;
- (void) validateRecordSelectionRange: (id) sender;
- (void) validateShowSelectionRange: (id) sender;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMMacroDocument(iTMAction)
/*"Description forthcoming."*/
@implementation iTMMacroDocument(iTMAction)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  pasteItem:select:edit:
- (void) pasteItem: (id <NSMenuItem>) anItem select: (BOOL) selectFlag edit: (BOOL) editFlag extendSelection: (BOOL) extendFlag
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(anItem)
    {
        NSOutlineView * OLV = [self outlineView];
        switch([OLV numberOfSelectedRows])
        {
            case 0:
            {
                NSMenu * RM = [self rootMenu];
                [self menu: RM insertItem: anItem atIndex: [RM numberOfItems]];
                break;
            }
            case 1:
            {
                id item = [OLV itemAtRow: [OLV selectedRow]];
                if([OLV isItemExpanded: item])
                {
                    NSMenu * M = [item submenu];
                    [self menu: M insertItem: anItem atIndex: [M numberOfItems]];
                }
                else
                {
                    NSMenu * M = [item menu];
                    [self menu: M insertItem: anItem atIndex: [M indexOfItem: item]];
                }
                break;
            }
            default:
                return;
        }
        if(selectFlag || editFlag)
        {
            [self deepExpandItem: anItem];
            [self setSelectedItem: nil];// Critical
            [OLV selectRow: [OLV rowForItem: anItem] byExtendingSelection: extendFlag];
            if(!extendFlag)
                [self setSelectedItem: anItem];// Critical
        }
        if(editFlag)
        {
            [OLV editColumn: [OLV columnWithIdentifier: @"iTM-MacroName"]
                                        row: [OLV rowForItem: anItem] withEvent: nil select: YES];
        }
    }
    else
        NSLog(@"Trying to paste nil item (iTMMacroDocument)");
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  addItem:
- (void) addItem: (id) irrelevant
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMMacroMenuItem * MI = [[iTMMacroMenuItem alloc] initWithTitle: @"Macro"
                                action: @selector(insertMacro:) keyEquivalent: [NSString string]];
    [MI setHasUndoManager: YES];
    [self pasteItem: MI select: YES edit: YES extendSelection: NO];
    [MI autorelease];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  addSeparator:
- (void) addSeparator: (id) irrelevant
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self pasteItem: [NSMenuItem separatorItem] select: YES edit: NO extendSelection: NO];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  addMenu:
- (void) addMenu: (id) irrelevant
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMenuItem * MI = [[NSMenuItem alloc] initWithTitle: @"Macro"
                                action: @selector(insertMacro:) keyEquivalent: [NSString string]];
    [MI setSubmenu: [[[NSMenu alloc] initWithTitle: [NSString string]] autorelease]];
    [self pasteItem: MI select: YES edit: YES extendSelection: NO];
    [MI autorelease];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  removeMenuItem:
- (void) removeMenuItem: (id <NSMenuItem>) item
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMenu * M = [item menu];
    [self menu: M removeItemAtIndex: [M indexOfItem: item]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  cut:
- (void) cut: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self copy: self];
    [self clear: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  copy:
- (void) copy: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSPasteboard * GPB = [NSPasteboard generalPasteboard];
    NSOutlineView * OLV = [self outlineView];
    int numberOfSelectedRows = [OLV numberOfSelectedRows];
    if(numberOfSelectedRows == 1)
    {
        id item = [OLV itemAtRow: [OLV selectedRow]];
        [GPB declareTypes: [NSArray arrayWithObject: iTMMacroMenuItemPBType] owner: nil];
        [GPB setData: [NSArchiver archivedDataWithRootObject: item] forType: iTMMacroMenuItemPBType];
    }
    else if(numberOfSelectedRows > 1)
    {
        NSEnumerator * E = [OLV selectedRowEnumerator];
        NSMutableArray * MA = [[NSMutableArray alloc] initWithCapacity: numberOfSelectedRows];
        NSNumber * N;
        while(N = [E nextObject])
            [MA insertObject: [OLV itemAtRow: [N intValue]] atIndex: 0];
        [GPB declareTypes: [NSArray arrayWithObject: iTMMacroMenuItemsPBType] owner: nil];
        [GPB setData: [NSArchiver archivedDataWithRootObject: MA] forType: iTMMacroMenuItemsPBType];
        [MA autorelease];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  paste:
- (void) paste: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSPasteboard * GPB = [NSPasteboard generalPasteboard];
    if([[GPB types] indexOfObject: iTMMacroMenuItemPBType] != NSNotFound)
	[self pasteItem: [NSUnarchiver unarchiveObjectWithData:
            [GPB dataForType: iTMMacroMenuItemPBType]] select: YES edit: NO extendSelection: NO];
    else if([[GPB types] indexOfObject: iTMMacroMenuItemsPBType] != NSNotFound)
    {
        NSArray * RA = [NSUnarchiver unarchiveObjectWithData: [GPB dataForType: iTMMacroMenuItemsPBType]];
        if([RA isKindOfClass: [NSArray class]])
        {
            NSEnumerator * E = [RA objectEnumerator];
            NSMenuItem * MI;
            [[self outlineView] deselectAll: self];
            [self setSelectedItem: nil];
            while(MI = [E nextObject])
                if([MI isKindOfClass: [NSMenuItem class]])
                    [self pasteItem: MI select: YES edit: NO extendSelection: YES];
                else
                    NSLog(@"iTMMacroDocument NO PASTE ERROR: NSMenuItem expected for %@", MI);
        }
        else
            NSLog(@"iTMMacroDocument NO PASTE ERROR: NSArray expected for %@", RA);
    }
    else
        NSLog(@"NO paste");
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  clear:
- (void) clear: (id) sender
/*"Removes the selected items.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSOutlineView * OLV = [self outlineView];
    int numberOfSelectedRows = [OLV numberOfSelectedRows];
    if(numberOfSelectedRows > 0)
    {
        NSEnumerator * E = [OLV selectedRowEnumerator];
        NSMutableArray * MA = [[NSMutableArray alloc] initWithCapacity: numberOfSelectedRows];
        NSNumber * N;
        int selectedRow = 0;
        while(N = [E nextObject])
            [MA addObject: N];
        E = [[MA sortedArrayUsingSelector: @selector(compare:)] reverseObjectEnumerator];
        while(N = [E nextObject])
        {
            NSMenuItem * MI = [OLV itemAtRow: (selectedRow = [N intValue])];
            NSMenu * M = [MI menu];
            [self menu: M removeItemAtIndex: [M indexOfItem: MI]];
        }
        selectedRow = MAX(0, selectedRow-1);
        {
            id item = [OLV itemAtRow: selectedRow];
            if(item)
            {
                [OLV selectRow: selectedRow byExtendingSelection: NO];
                [self setSelectedItem: item];
            }
        }
        [MA autorelease];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  actionInsertDirective:
- (void) actionInsertDirective: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id S = [self selectedItem];
    [S setAction: @selector(insertDirective:)]; 
    [S setChangesArePending: YES];
    [[self outlineView] reloadItem: S reloadChildren: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  actionInsertMacro:
- (void) actionInsertMacro: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id S = [self selectedItem];
    [S setAction: @selector(insertMacro:)]; 
    [S setChangesArePending: YES];
    [[self outlineView] reloadItem: S reloadChildren: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  actionNone:
- (void) actionNone: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id S = [self selectedItem];
    [S setAction: NULL]; 
    [S setChangesArePending: YES];
    [[self outlineView] reloadItem: S reloadChildren: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  actionEditMacroFile:
- (void) actionEditMacroFile: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id S = [self selectedItem];
    [S setAction: @selector(editMacroFile:)]; 
    [S setChangesArePending: YES];
    [[self outlineView] reloadItem: S reloadChildren: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  actionPopUpAction:
- (void) actionPopUpAction: (id) sender
/*"fake action to catch the validation and update process, see NSControl(iTeXMac) and validateActionPopUpAction
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  actionInsertSpecial:
- (void) actionInsertSpecial: (id) sender
/*"fake action to catch the validation and update process, see NSControl(iTeXMac) and validateActionInsertSpecial
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  insertSpecialSelection:
- (IBAction) insertSpecialSelection: (id) sender
/*"Inserts a key word that will be replaced by the currently selected text.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSTextView * MTV = [self macroTextView];
    [[MTV window] makeFirstResponder: MTV];
    [MTV insertText: iTMMacroSelectionKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  insertSpecialTabAnchor:
- (IBAction) insertSpecialTabAnchor: (id) sender
/*"Insert a key word to facilitate navigation, something like from field to field.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSTextView * MTV = [self macroTextView];
    [[MTV window] makeFirstResponder: MTV];
    [MTV insertText: iTMMacroTabAnchorKey];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  insertSpecialTitle:
- (IBAction) insertSpecialTitle: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSTextView * MTV = [self macroTextView];
    [[MTV window] makeFirstResponder: MTV];
    [MTV insertText: [self nameDisplayString]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  recordSelectionRange:
- (IBAction) recordSelectionRange: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    #warning NOT FULLY IMPLEMENTED BUT FULLY FUNCTIONAL
    #if 1
    NSTextView * MTV = [self macroTextView];
    NSTextStorage * TS = [MTV textStorage];
    if([TS length])
    {
        NSRange selectedRange = [MTV selectedRange];
        int begin = NSMaxRange(selectedRange);
        int end = [TS length];
        NSRange R;
        [[self selectedItem] setChangesArePending: YES];
        while(begin < end)
        {
            if([TS attribute: NSBackgroundColorAttributeName atIndex: begin
                longestEffectiveRange: &R inRange: NSMakeRange(begin, end - begin)])
            {
                if([MTV shouldChangeTextInRange: R replacementString: [NSString string]])
                {
                    [TS deleteCharactersInRange: R];
                    [MTV didChangeText];
                    end -= R.length;
                }
                else
                    NSLog(@"change refused in recordElectionRange: 1");
            }
            else
                begin += R.length;
        }
        {
            NSRange affectedCharRange = NSMakeRange(MIN(NSMaxRange(selectedRange), [TS length]), 0);
            BOOL flag;
            flag = [MTV shouldChangeTextInRange: affectedCharRange
                            replacementString: [[self attributedSelectionDelimiter] string]];
            if(flag)
            {
                [TS insertAttributedString: [self attributedSelectionDelimiter] atIndex: affectedCharRange.location];
                [MTV didChangeText];
            }
            else
                NSLog(@"change refused in recordElectionRange: 2");
        }
        begin = 0;
        end = selectedRange.location;
        while(begin < end)
        {
            if([TS attribute: NSBackgroundColorAttributeName atIndex: begin
                longestEffectiveRange: &R inRange: NSMakeRange(begin, end - begin)])
            {
                if([MTV shouldChangeTextInRange: R replacementString: [NSString string]])
                {
                    [TS deleteCharactersInRange: R];
                    end -= R.length;
                }
                else
                    NSLog(@"change refused in recordElectionRange: 3");
            }
            else
                begin += R.length;
        }
        {
            NSRange affectedCharRange = NSMakeRange(end, 0);
            if([MTV shouldChangeTextInRange: affectedCharRange
                                replacementString: [[self attributedSelectionDelimiter] string]])
            {
                [TS insertAttributedString: [self attributedSelectionDelimiter] atIndex: affectedCharRange.location];
                [MTV didChangeText];
            }
            else
                NSLog(@"change refused in recordElectionRange: 4");
        }
        #if 0
        // now we are selecting the right range
        // critical: we take into account external changes that may affect the string length
        {
            int roof = [TS length];
            if(![TS attribute: NSBackgroundColorAttributeName atIndex: 0
                                longestEffectiveRange: &R inRange: NSMakeRange(0, roof)])
            {
                begin = NSMaxRange(R);
                if((begin < roof) && [TS attribute: NSBackgroundColorAttributeName atIndex: begin
                            longestEffectiveRange: &R inRange: NSMakeRange(begin, roof - begin)])
                    begin = NSMaxRange(R);
            }
            else
                begin = NSMaxRange(R);
            if(![TS attribute: NSBackgroundColorAttributeName atIndex: roof-1
                        longestEffectiveRange: &R inRange: NSMakeRange(0, roof)])
            {
                end = R.location;
                if((end > 0) && [TS attribute: NSBackgroundColorAttributeName atIndex: end-1
                            longestEffectiveRange: &R inRange: NSMakeRange(0, end)])
                    end = R.location;
            }
            else
                end = R.location;
            [MTV setSelectedRange: NSMakeRange(begin, end - begin)];
        }
        #endif
    }
    return;
    #endif
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  tooltipFieldEdited:
- (IBAction) tooltipFieldEdited: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id SI = [self selectedItem];
    [[SI macro] setTooltip: [sender stringValue]];
    [self updateMenuItemFromUI: SI];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  showHelp:
- (void) showHelp: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[iTMHelpManager sharedManager] showHelpFor: self key: iTMHelpMacrosEditorKey]; 
    return;
}
@end

@implementation iTMMacroDocument(iTMUIValidation)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateActionPopUpAction:
- (void) validateActionPopUpAction: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id SI = [self selectedItem];
    NSMenu * M = [sender menu];
    SEL selector = NULL;
    if(SI && ![SI isEqual: [NSMenuItem separatorItem]] && ![SI hasSubmenu])
    {
        SEL action = [SI action];
        if(action == @selector(insertMacro:))
            selector = @selector(actionInsertMacro:);
        else if(action == @selector(editMacroFile:))
            selector = @selector(actionEditMacroFile:);
        else if(action == @selector(insertDirective:))
            selector = @selector(actionInsertDirective:);
        else
            selector = @selector(actionNone:);
        [sender setEnabled: YES];
    }
    else
    {
        selector = @selector(actionNone:);
        [sender setEnabled: NO];
    }
    if([M numberOfItems])
    {
        NSMenuItem * MI = [M itemWithAction: selector];
        [sender selectItem: (MI? MI: [M itemAtIndex: 0])];
    }
    [M update];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateActionInsertSpecial:
- (void) validateActionInsertSpecial: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender setEnabled: [self menuItemUsesMacro: [self selectedItem]]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateTooltipFieldEdited:
- (void) validateTooltipFieldEdited: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id SI = [self selectedItem];
    NSString * S = [self menuItemUsesMacro: SI]? [[SI macro] tooltip]: nil;
    [sender setStringValue: (S? S: [NSString string])];
    [sender setEnabled: [self menuItemUsesMacro: SI]];
    return;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateAddItem:
- (void) validateAddItem: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return YES;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateAddMenu:
- (void) validateAddMenu: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return YES;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateAddSeparator:
- (void) validateAddSeparator: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return YES;
}
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateRecordSelectionRange:
- (void) validateRecordSelectionRange: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender setEnabled: ([[self selectedItem] isKindOfClass: [iTMMacroMenuItem class]] &&
                    [[self macroTextView] isEqual: [[[self macroTextView] window] firstResponder]])];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateShowSelectionRange:
- (void) validateShowSelectionRange: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [sender setEnabled: [[self selectedItem] isKindOfClass: [iTMMacroMenuItem class]]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMenuItem:
- (BOOL) validateMenuItem: (id) sender
/*"Validates #{cut:}, #{copy:}, #{clear:} and #{paste:}.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    SEL selector = [sender action];
    // File menu
    if ((selector == @selector(cut:)) || (selector == @selector(copy:)) || (selector == @selector(clear:)))
        return [[self outlineView] numberOfSelectedRows] != 0;
    else if(selector == @selector(paste:))
    {
        NSPasteboard * GPB = [NSPasteboard generalPasteboard];
        return (([[GPB types] indexOfObject: iTMMacroMenuItemPBType] != NSNotFound) ||
                    ([[GPB types] indexOfObject: iTMMacroMenuItemsPBType] != NSNotFound));
    }
    else
        return [super validateMenuItem: sender];
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMMacroDocument(iTMAction)
