/*
//  iTMMacroDocument(iTMOutlineViewDataSource).m
//  macros
//
//  Created by jerome.laurens@u-bourgogne.fr on Tue Jan 15 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/Foundation.h>
#import <AppKit/NSMenu.h>
#import <AppKit/NSTableColumn.h>
#import <AppKit/NSAttributedString.h>
#import <AppKit/NSFont.h>
#import <AppKit/NSColor.h>
#import <AppKit/AppKit.h>
#import "iTMMacroDocument(iTMOutlineViewDataSource).h"
#import "iTMMacroDocument(iTMAction).h"
#import "iTMRootMacrosMenu.h"
#import "iTMMacroMenuItem.h"
#import "iTMMacroOutlineView.h"

// there should be only one drag operation at a time...
static NSDragOperation _iTMMacroDragOperation;
static id _iTMMacroDragObject = nil;

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMMacroDocument(iTMOutlineViewDataSource)
/*"Description forthcoming."*/
@implementation iTMMacroDocument(iTMOutlineViewDataSource)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  outlineView:numberOfChildrenOfItem:
- (int) outlineView: (NSOutlineView *) outlineView numberOfChildrenOfItem: (id) item
/*"A menu item is expandable iff it has a submenu.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return (item == nil) ? [[self rootMenu] numberOfItems] : [[item submenu] numberOfItems];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  outlineView:child:ofItem:
- (id) outlineView: (NSOutlineView *) outlineView child: (int) index ofItem: (id) item
/*"Returns the menu tem at index index of the submenu of menu item item. Ok?
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMenu * M = (item == nil) ? [self rootMenu] : [item submenu];
    if((index>=0) && (index<[M numberOfItems]))
        return [M itemAtIndex: index];
    else
    {
        #warning BUG somewhere try: insert a menu item then undo...
        NSLog(@"index out of bounds... 0<=%d<%d,\nmenu: %@,\nitem: %@", index, [M numberOfItems], M, item);
        return nil;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  outlineView:isItemExpandable:
- (BOOL) outlineView: (NSOutlineView *) outlineView isItemExpandable: (id) item
/*"YES iff the item has a submenu.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [item hasSubmenu];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  outlineView:objectValueForTableColumn:byItem:
- (id) outlineView: (NSOutlineView *) outlineView objectValueForTableColumn: (NSTableColumn *) tableColumn byItem: (id) item
/*"Returns the item title or the undo status.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([[tableColumn identifier] isEqualToString: @"iTM-MacroName"])
        return (item == nil)? [[self rootMenu] title]: [item title];
    else if([item isKindOfClass: [iTMMacroMenuItem class]] && [item isMacroMenuItemEdited])
        return [[[NSAttributedString alloc] initWithString: [NSString stringWithUTF8String: "•"]
            attributes: [NSDictionary dictionaryWithObjectsAndKeys:
                [NSFont fontWithName: @"Monaco" size: 15.0], NSFontAttributeName, nil]] autorelease];
    else if([item hasSubmenu] && [[item submenu] areMacroMenuItemsEdited])
        return [[[NSAttributedString alloc] initWithString: [NSString stringWithUTF8String: "•"]
            attributes: [NSDictionary dictionaryWithObjectsAndKeys:
                [NSFont fontWithName: @"Monaco" size: 15.0], NSFontAttributeName,
                [NSColor selectedTextBackgroundColor],
                                            NSForegroundColorAttributeName, nil]] autorelease];
    else
        return [NSString string];
}
@end

@implementation iTMMacroDocument(iTMOutlineViewDataSourceOptional)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  outlineView:setObjectValue:forTableColumn:byItem:
- (void) outlineView: (NSOutlineView *) outlineView setObjectValue: (id) object forTableColumn: (NSTableColumn *) tableColumn byItem: (id) item
/*"Sets the new title.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([[tableColumn identifier] isEqualToString: @"iTM-MacroName"])
        [self menuItem: item setTitle: object];
    return;
}
#if 0
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  outlineView:itemForPersistentObject:
- (id) outlineView: (NSOutlineView *) outlineView itemForPersistentObject: (id) object
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  outlineView:persistentObjectForItem:
- (id) outlineView: (NSOutlineView *) outlineView persistentObjectForItem: (id) item
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return;
}
#endif
@end

@implementation iTMMacroDocument(iTMOutlineViewDataSourceDragNDrop)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  outlineView:writeItems:toPasteboard:
- (BOOL) outlineView: (NSOutlineView *) olv writeItems: (NSArray*) items toPasteboard: (NSPasteboard*) pboard;
    // This method is called after it has been determined that a drag should begin, but before the drag has been started.  To refuse the drag, return NO.  To start a drag, return YES and place the drag data onto the pasteboard (data, owner, etc...).  The drag image and other drag related information will be set up and provided by the outline view once this call returns with YES.  The items array is the list of items that will be participating in the drag.
/*"_iTMMacroDragObject stores the object currently dragged.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: _iTMMacroDragObject is not clean...
"*/
{
    [self updateMenuItemFromUI: [self selectedItem]];
    [_iTMMacroDragObject autorelease];
    _iTMMacroDragObject = nil;
    if([items count] == 1)
    {
        NSMenuItem * item = [items objectAtIndex: 0];
        if([item isKindOfClass: [NSMenuItem class]])
        {
            [pboard declareTypes: [NSArray arrayWithObject: iTMMacroMenuItemPBType] owner: nil];
            [pboard setData: [NSArchiver archivedDataWithRootObject: item] forType: iTMMacroMenuItemPBType];
            _iTMMacroDragObject = [item retain];
        }
    }
    else if([items count] > 1)
    {
        BOOL result = YES;
        NSEnumerator * E = [items reverseObjectEnumerator];
        NSMenu * item;
        while((item = [E nextObject]) && (result = [item isKindOfClass: [NSMenuItem class]]));
        if(result)
        {
            [pboard declareTypes: [NSArray arrayWithObject: iTMMacroMenuItemsPBType] owner: nil];
            [pboard setData: [NSArchiver archivedDataWithRootObject: items] forType: iTMMacroMenuItemsPBType];
            _iTMMacroDragObject = [items retain];
        }
    }
    return (_iTMMacroDragObject!=nil);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  outlineView:validateDrop:proposedItem:proposedChildIndex:
- (NSDragOperation) outlineView: (NSOutlineView*) olv validateDrop: (id <NSDraggingInfo>) info proposedItem: (id) item proposedChildIndex: (int) index;
    // This method is used by NSOutlineView to determine a valid drop target.  Based on the mouse position, the outline view will suggest a proposed drop location.  This method must return a value that indicates which dragging operation the data source will perform.  The data source may "re-target" a drop if desired by calling setDropItem:dropChildIndex: and returning something other than NSDragOperationNone.  One may choose to re-target for various reasons (eg. for better visual feedback when inserting into a sorted position).
/*"Description forthcoming. Limitation in outline views drag ands drop: cannot expand between two items...
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: stronger code, no test here
"*/
{
    _iTMMacroDragOperation = NSDragOperationNone;
    if(index == NSOutlineViewDropOnItemIndex)
    {
        if(item)
        {
            NSMenu * M = [item menu];
            NSMenu * SM = [M supermenu];
            index = [M indexOfItem: item] + 1;
            item = [SM itemAtIndex: [SM indexOfItemWithSubmenu: M]];
        }
        else
            index = 0;
        [olv setDropItem: item dropChildIndex: index];
    }
    {
        NSPasteboard * pboard = [info draggingPasteboard];
        if([[pboard types] indexOfObject: iTMMacroMenuItemPBType] != NSNotFound)
            if(([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask) > 0)
                _iTMMacroDragOperation = NSDragOperationCopy;
            else if([olv isEqual: [info draggingSource]])
            {
                if([_iTMMacroDragObject isEqualOrHasSubmenuItem: item])
                    _iTMMacroDragOperation = NSDragOperationCopy;
                else
                {
                    NSMenu * M = [_iTMMacroDragObject menu];
                    if([M isEqual: (item? [item submenu]: [self rootMenu])])
                    {
                        int I = [M indexOfItem: _iTMMacroDragObject];
                        if((index != I)&&(index != I+1))
                            _iTMMacroDragOperation = NSDragOperationMove;
                        else
                            _iTMMacroDragOperation = NSDragOperationCopy;
                    }
                    else
                        _iTMMacroDragOperation = NSDragOperationMove;
                }
            }
            else
                _iTMMacroDragOperation = NSDragOperationCopy;
        else if([[pboard types] indexOfObject: iTMMacroMenuItemsPBType] != NSNotFound)
            if(([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask) > 0)
                _iTMMacroDragOperation = NSDragOperationCopy;
            else if([olv isEqual: [info draggingSource]])
            {
                BOOL canMove = YES;
                NSEnumerator *  E = [_iTMMacroDragObject objectEnumerator];
                NSMenuItem * X;
                while((X = [E nextObject]) && (canMove = ![X isEqualOrHasSubmenuItem: item]));
                if(canMove)
                    _iTMMacroDragOperation = NSDragOperationMove;
                else
                    _iTMMacroDragOperation = NSDragOperationCopy;
            }
            else
                _iTMMacroDragOperation = NSDragOperationCopy;
    }
    return _iTMMacroDragOperation;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  outlineView:acceptDrop:item:childIndex:
- (BOOL) outlineView: (NSOutlineView*) olv acceptDrop: (id <NSDraggingInfo>) info item: (id) item childIndex: (int) index;
    // This method is called when the mouse is released over an outline view that previously decided to allow a drop via the validateDrop method.  The data source should incorporate the data from the dragging pasteboard at this time.
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: there is a bug in the same view, if we drag an item to the lower part of the list and we move L<->R the cursor a little, the drag wont occur. I tried to validate the drop at the beginning but it does not work.
"*/
{
    BOOL result = NO;
    if(_iTMMacroDragObject)// local drag
    {
        id dropItem = nil;
        NSPasteboard * pboard = [info draggingPasteboard];
        if([[pboard types] indexOfObject: iTMMacroMenuItemPBType] != NSNotFound)
        {
            if(dropItem = [NSUnarchiver unarchiveObjectWithData: [pboard dataForType: iTMMacroMenuItemPBType]])
            {
                if(_iTMMacroDragOperation == NSDragOperationMove)
                {
                    if([olv isEqual: [info draggingSource]])// dragging occurs in the same view
                    {
                        NSMenu * root = item? [item submenu]: [self rootMenu];
                        NSMenu * M = [_iTMMacroDragObject menu];
                        int I = [olv rowForItem: _iTMMacroDragObject];
                        if(![M isEqual: root] || ((index != I)&&(index != I+1)))
                        {
                            [self menu: root insertItem: dropItem atIndex: index];
                            [self removeMenuItem: _iTMMacroDragObject];
                            if(item) [olv expandItem: item];
                            [olv selectRow: [olv rowForItem: dropItem] byExtendingSelection: NO];
                            [self setSelectedItem: dropItem];// Critical
                            result = YES;
                        }
                    }
                    else// dragging occurs between different view
                    {
                        NSMenu * root = item? [item submenu]: [self rootMenu];
                        [self menu: root insertItem: dropItem atIndex: index];
                        if(item) [olv expandItem: item];
                        [olv selectRow: [olv rowForItem: dropItem] byExtendingSelection: NO];
                        [self setSelectedItem: dropItem];// Critical
                        result = YES;
                    }
                }
                else if(_iTMMacroDragOperation == NSDragOperationCopy)
                {
                    NSMenu * root = item? [item submenu]: [self rootMenu];
                    [self menu: root insertItem: dropItem atIndex: index];
                    if(item) [olv expandItem: item];
                    [olv selectRow: [olv rowForItem: dropItem] byExtendingSelection: NO];
                    [self setSelectedItem: dropItem];// Critical
                    result = YES;
                }
            }
        }
        else if(([[pboard types] indexOfObject: iTMMacroMenuItemsPBType] != NSNotFound))
            if(_iTMMacroDragOperation == NSDragOperationCopy)
            {
                NSArray * dropItems;
                if((dropItems = [NSUnarchiver unarchiveObjectWithData: [pboard dataForType: iTMMacroMenuItemsPBType]]) &&
                        [dropItems isKindOfClass: [NSArray class]])
                {
                    NSEnumerator * E = [dropItems reverseObjectEnumerator];
                    NSMenu * root = item? [item submenu]: [self rootMenu];
                    while(dropItem = [E nextObject])
                        [self menu: root insertItem: dropItem atIndex: index];
                    if(item) [olv expandItem: item];
                    E = [dropItems objectEnumerator];
                    [olv deselectAll: self];
                    [self setSelectedItem: nil];// Critical
                    while(dropItem = [E nextObject])
                        [olv selectRow: [olv rowForItem: dropItem] byExtendingSelection: YES];
                    result = YES;
                }
            }
            else if(_iTMMacroDragOperation == NSDragOperationMove)
            {
                NSArray * dropItems;
                if((dropItems = [NSUnarchiver unarchiveObjectWithData: [pboard dataForType: iTMMacroMenuItemsPBType]]) &&
                        [dropItems isKindOfClass: [NSArray class]])
                {
                    NSEnumerator * E = [dropItems reverseObjectEnumerator];
                    NSMenu * root = item? [item submenu]: [self rootMenu];
                    while(dropItem = [E nextObject])
                            [self menu: root insertItem: dropItem atIndex: index];
                    if(item) [olv expandItem: item];
                    {
                        NSEnumerator * En = [_iTMMacroDragObject objectEnumerator];
                        id X;
                        while(X = [En nextObject])
                            [self removeMenuItem: X];
                    }
                    [olv deselectAll: self];
                    [self setSelectedItem: nil];// Critical
//                    E = [dropItems objectEnumerator];
    //                while(dropItem = [E nextObject])
    //                    [olv selectRow: [olv rowForItem: dropItem] byExtendingSelection: YES];
                    result = YES;
                }
            }
        [_iTMMacroDragObject autorelease];
        _iTMMacroDragObject = nil;
    }
    else if(_iTMMacroDragOperation == NSDragOperationCopy)// drag from something else than an iTMMacroOutlineView
    {
        id dropItem = nil;
        NSPasteboard * pboard = [info draggingPasteboard];
        if([[pboard types] indexOfObject: iTMMacroMenuItemPBType] != NSNotFound)
        {
            if(dropItem = [NSUnarchiver unarchiveObjectWithData: [pboard dataForType: iTMMacroMenuItemPBType]])
            {
                NSMenu * root = item? [item submenu]: [self rootMenu];
                [self menu: root insertItem: dropItem atIndex: index];
                if(item) [olv expandItem: item];
                [olv selectRow: [olv rowForItem: dropItem] byExtendingSelection: NO];
                [self setSelectedItem: dropItem];// Critical
                result = YES;
            }
        }
        else if(([[pboard types] indexOfObject: iTMMacroMenuItemsPBType] != NSNotFound))
        {
            NSArray * dropItems;
            if((dropItems = [NSUnarchiver unarchiveObjectWithData: [pboard dataForType: iTMMacroMenuItemsPBType]]) &&
                    [dropItems isKindOfClass: [NSArray class]])
            {
                NSEnumerator * E = [dropItems reverseObjectEnumerator];
                NSMenu * root = item? [item submenu]: [self rootMenu];
                while(dropItem = [E nextObject])
                    [self menu: root insertItem: dropItem atIndex: index];
                if(item) [olv expandItem: item];
                E = [dropItems objectEnumerator];
                [olv deselectAll: self];
                [self setSelectedItem: nil];// Critical
                while(dropItem = [E nextObject])
                    [olv selectRow: [olv rowForItem: dropItem] byExtendingSelection: YES];
                result = YES;
            }
        }
    }
    return result;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMMacroDocument(iTMOutlineViewDataSource)

