/*
//  iTMMacroDocument(iTMTextDelegate).m
//  macros
//
//  Created by jerome.laurens@u-bourgogne.fr on Fri Jan 18 2002.
//  Copyright © 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSNotification.h>
#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import "iTMMacroDocument.h"
#import "iTMMacroDocument(iTMTextDelegate).h"
#import "iTMMacroMenuItem.h"
#import "iTMTextMacro.h"
#import "NSControl(iTeXMac).h"

@interface iTMMacroDocument(iTMTextDelegatePrivate)
- (NSRange) textView: (NSTextView *) aTextView proposedRangeForCharacterRange: (NSRange) aCharRange keyWord: (NSString *) aKeyWord;
- (int) textView: (NSTextView *) aTextView proposedLocationForLocation: (int) aLocation keyWord: (NSString *) aKeyWord;
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMMacroDocument(iTMTextDelegate)
/*"Description forthcoming."*/
@implementation iTMMacroDocument(iTMTextDelegate)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textDidChange:
- (void) textDidChange: (NSNotification *) notification
/*"Attributes testing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMMacroMenuItem * iMI = [self selectedItem];
    NSTextView * MTV = [notification object];
    if([iMI isKindOfClass: [iTMMacroMenuItem class]])
    {
        NSTextStorage * TS = [MTV textStorage];
        NSString * name = [notification name];
        int roof = [TS length];
        [iMI setChangesArePending: YES];
        [[NSNotificationCenter defaultCenter] removeObserver: self name: name object: MTV];
        if(roof)
        {
            NSRange R;
            int beginSelection = roof;
            int endSelection = 0;
            int index = 0;
            if(![TS attribute: NSBackgroundColorAttributeName atIndex: 0
                                longestEffectiveRange: &R inRange: NSMakeRange(0, roof)])
            {
                index = NSMaxRange(R);
                if((index<roof) && [TS attribute: NSBackgroundColorAttributeName atIndex: index
                            longestEffectiveRange: &R inRange: NSMakeRange(index, roof - index)])
                {
                    beginSelection = R.location+1;
                    //tests for too many selection delimiters.
                    if(R.length>1)
                    {
                        NSRange affectedCharRange = NSMakeRange(R.location+1, R.length-1);
                        if([MTV shouldChangeTextInRange: affectedCharRange replacementString: [NSString string]])
                        {
                            [TS deleteCharactersInRange: affectedCharRange];
                            [MTV didChangeText];
                        }
                        else
                            NSLog(@"change refused in textDidChange: 1");
                    }
                }
            }
            else
            {
                beginSelection = R.location+1;
                //tests for too many selection delimiters.
                if(R.length>1)
                {
                    NSRange affectedCharRange = NSMakeRange(R.location+1, R.length-1);
                    if([MTV shouldChangeTextInRange: affectedCharRange replacementString: [NSString string]])
                    {
                        [TS deleteCharactersInRange: affectedCharRange];
                        [MTV didChangeText];
                    }
                    else
                        NSLog(@"change refused in textDidChange: 2");
                }
            }
            // now beginSelection points to the first char after the first selection delimiter.
            roof = [TS length];
            #if 1
            if(![TS attribute: NSBackgroundColorAttributeName atIndex: roof-1
                        longestEffectiveRange: &R inRange: NSMakeRange(0, roof)])
            {
                index = R.location;
                if((R.location>0) && [TS attribute: NSBackgroundColorAttributeName atIndex: index-1
                            longestEffectiveRange: &R inRange: NSMakeRange(0, index)])
                {
                    endSelection = R.location;
                    //tests for too many selection delimiters.
                    if(R.length>1)
                    {
                        NSRange affectedCharRange = NSMakeRange(R.location+1, R.length-1);
                        if([MTV shouldChangeTextInRange: affectedCharRange replacementString: [NSString string]])
                        {
                            [TS deleteCharactersInRange: affectedCharRange];
                            [MTV didChangeText];
                        }
                        else
                            NSLog(@"change refused in textDidChange: 3");
                    }
                }
            }
            else
            {
                endSelection = R.location;
                //tests for too many selection delimiters.
                if(R.length>1)
                {
                    NSRange affectedCharRange = NSMakeRange(R.location+1, R.length-1);
                    if([MTV shouldChangeTextInRange: affectedCharRange replacementString: [NSString string]])
                    {
                        [TS deleteCharactersInRange: affectedCharRange];
                        [MTV didChangeText];
                    }
                    else
                        NSLog(@"change refused in textDidChange: 4");
                }
            }
            // now endSelection points to the first char of the last selection delimiter.
            // next we remove all extra delimiters that may come from DnD or Copy/Paste operation
            while(beginSelection < endSelection)
            {
//                NSLog(@"[begin|end]Selection : %d, %d", beginSelection, endSelection);
                if([TS attribute: NSBackgroundColorAttributeName atIndex: beginSelection
                    longestEffectiveRange: &R inRange: NSMakeRange(beginSelection, endSelection - beginSelection)])
                {
                    if([MTV shouldChangeTextInRange: R replacementString: [NSString string]])
                    {
                        [TS deleteCharactersInRange: R];
                        [MTV didChangeText];
                        endSelection -= R.length;
                    }
                    else
                        NSLog(@"change refused in textDidChange: 5");
                }
                else
                    beginSelection += R.length;
            }
            #endif
            #if 1
            {
                NSString * S = [TS string];
                NSRange searchRange, foundRange = NSMakeRange([TS length], 0);
                while(YES)
                {
                    searchRange = NSMakeRange(0, foundRange.location);
                    foundRange = [S rangeOfString: [self nameDisplayString] options: NSBackwardsSearch range: searchRange];
                    if(foundRange.length)
                        if([MTV shouldChangeTextInRange: foundRange replacementString: nil])
                        {
                            [TS addAttribute: NSForegroundColorAttributeName value: [NSColor redColor] range: foundRange];
                            [MTV didChangeText];
                            endSelection -= R.length;
                        }
                        else
                            NSLog(@"change refused in textDidChange: 6");
                    else
                        break;
                }
                //coloring iTMMacroSelectionKey
                foundRange = NSMakeRange([TS length], 0);
                while(YES)
                {
                    searchRange = NSMakeRange(0, foundRange.location);
                    foundRange = [S rangeOfString: iTMMacroSelectionKey options: NSBackwardsSearch range: searchRange];
                    if(foundRange.length)
                        if([MTV shouldChangeTextInRange: foundRange replacementString: nil])
                        {
                            [TS addAttribute: NSForegroundColorAttributeName value: [NSColor redColor] range: foundRange];
                            [MTV didChangeText];
                            endSelection -= R.length;
                        }
                        else
                            NSLog(@"change refused in textDidChange: 7");
                    else
                        break;
                }
                //coloring iTMMacroTabAnchorKey
                foundRange = NSMakeRange([TS length], 0);
                while(YES)
                {
                    searchRange = NSMakeRange(0, foundRange.location);
                    foundRange = [S rangeOfString: iTMMacroTabAnchorKey options: NSBackwardsSearch range: searchRange];
                    if(foundRange.length)
                        if([MTV shouldChangeTextInRange: foundRange replacementString: nil])
                        {
                            [TS addAttribute: NSForegroundColorAttributeName value: [NSColor redColor] range: foundRange];
                            [MTV didChangeText];
                            endSelection -= R.length;
                        }
                        else
                            NSLog(@"change refused in textDidChange: 8");
                    else
                        break;
                }
            }
            #endif
        }
        [[NSNotificationCenter defaultCenter] addObserver: self selector: _cmd name: name object: MTV];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textViewDidChangeSelection:
- (void) textViewDidChangeSelection: (NSNotification *) aNotification
/*"Attributes testing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self validateUserInterfaceItems];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textView:shouldChangeTextInRange:replacementString:
- (BOOL) textView: (NSTextView *) aTextView shouldChangeTextInRange: (NSRange) affectedCharRange replacementString: (NSString *) replacementString
/*"Attributes testing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [aTextView setTypingAttributes: [NSDictionary dictionaryWithObjectsAndKeys:
                    [NSFont fontWithName: @"Monaco" size: 12.0], NSFontAttributeName, nil]];
    if(replacementString && affectedCharRange.length)
    {
        /*"If one end of the affected character range lies inside one of the key words,
        refuse the change if it does not contain the whole key word."*/
        NSRange R;
        R = [self textView: aTextView
                proposedRangeForCharacterRange: affectedCharRange
                    keyWord: [self nameDisplayString]];
        if(R.length > affectedCharRange.length)
            return NO;
        R = [self textView: aTextView
                proposedRangeForCharacterRange: affectedCharRange
                    keyWord: iTMMacroSelectionKey];
        if(R.length > affectedCharRange.length)
            return NO;
        R = [self textView: aTextView
                proposedRangeForCharacterRange: affectedCharRange
                    keyWord: iTMMacroTabAnchorKey];
        if(R.length > affectedCharRange.length)
            return NO;
    }
    return YES;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textView:proposedRangeForCharacterRange:keyWord:
- (NSRange) textView: (NSTextView *) aTextView proposedRangeForCharacterRange: (NSRange) aCharRange keyWord: (NSString *) aKeyWord;
/*"Attributes testing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    /*"If one end of the affected character range lies inside the key word,
    refuse the change if it does not contain the whole key word."*/
    NSString * S = [aTextView string];
    NSRange result = aCharRange;
    int min, max, anchor;
    if([aKeyWord length])
    {
        min = result.location +1 >= [aKeyWord length]?
                result.location - [aKeyWord length] + 1: 0;
        max = MIN(result.location + [aKeyWord length], [S length]);
        if(min<max)
        {
            NSRange foundRange = [S rangeOfString: aKeyWord options: 0 range: NSMakeRange(min, max - min)];
            if(foundRange.length)
                result = NSUnionRange(result, foundRange);
            anchor =  NSMaxRange(result);
            min = anchor > [aKeyWord length]? anchor - [aKeyWord length]: 0;
            max = MIN(anchor + [aKeyWord length] - 1, [S length]);
            foundRange = [S rangeOfString: aKeyWord options: 0 range: NSMakeRange(min, max - min)];
            if(foundRange.length)
                result = NSUnionRange(result, foundRange);
        }
    }
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textView:proposedLocationForLocation:keyWord:
- (int) textView: (NSTextView *) aTextView proposedLocationForLocation: (int) aLocation keyWord: (NSString *) aKeyWord;
/*"Attributes testing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    /*"If one end of the affected character range lies inside the key word,
    refuse the change if it does not contain the whole key word."*/
    int result = aLocation;
    if([aKeyWord length])
    {
        NSString * S = [aTextView string];
        int min = MIN((aLocation + 1 >= [aKeyWord length]? aLocation - [aKeyWord length] + 1: 0), [S length]);
        int max = MIN(aLocation + [aKeyWord length], [S length]);
        NSRange foundRange = (min < max)? 
            [S rangeOfString: aKeyWord options: 0 range: NSMakeRange(min, max - min)]: NSMakeRange(0, 0);
        if(foundRange.length)
            return (aLocation <= (foundRange.location + foundRange.length/2))?
               foundRange.location: NSMaxRange(foundRange);
    }
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  textView:willChangeSelectionFromCharacterRange:toCharacterRange:
- (NSRange) textView: (NSTextView *) aTextView willChangeSelectionFromCharacterRange: (NSRange) oldSelectedCharRange toCharacterRange: (NSRange) newSelectedCharRange
/*"Keywords for macro name and selection must not be half selected.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * S = [aTextView string];
//    NSLog(@"proposed range %@", NSStringFromRange(newSelectedCharRange));
    if(![S length])
        return newSelectedCharRange;
    else if(newSelectedCharRange.length)
        /*"If one end of the new character range lies inside one of the key words, the range is extended to
        contain the whole key word."*/
        return [self textView: aTextView proposedRangeForCharacterRange:
                    [self textView: aTextView proposedRangeForCharacterRange:
                        [self textView: aTextView proposedRangeForCharacterRange: newSelectedCharRange
                        keyWord: [self nameDisplayString]]
                    keyWord: iTMMacroSelectionKey]
                keyWord: iTMMacroTabAnchorKey];
    else
    {
        /*"If the new insertion point lies inside one of the key words, it is shifted to the nearest end."*/
        int location = newSelectedCharRange.location;
        location = [self textView: aTextView proposedLocationForLocation: newSelectedCharRange.location
                                        keyWord: [self nameDisplayString]];
        #define NEWLOCATION(key)\
        [self textView: aTextView proposedLocationForLocation: newSelectedCharRange.location keyWord: key]
        if(location == newSelectedCharRange.location)
        {
            location = NEWLOCATION(iTMMacroSelectionKey);
            if(location == newSelectedCharRange.location)
            {
                location = NEWLOCATION(iTMMacroSelectionKey);
                if(location == newSelectedCharRange.location)
                {
                    location = NEWLOCATION(iTMMacroSelectionKey);
                    if(location == newSelectedCharRange.location)
                    {
                        location = NEWLOCATION(iTMMacroSelectionKey);
                    }
                }
            }
        }
        return NSMakeRange(location, 0);
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  undoManagerForTextView:
- (NSUndoManager *) undoManagerForTextView: (NSTextView *) MTV
/*"Attributes testing.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMMacroMenuItem * MMI = [self selectedItem];
//    NSLog(@"UNDO MANAGER ASKED FOR: %@", MMI);
    if([MMI isKindOfClass: [iTMMacroMenuItem class]] && [MMI hasUndoManager])
    {
//        #warning DEBUG
//        NSLog(@"UNDO MANAGER RETURNED: %@", [MMI undoManager]);
        return [MMI undoManager];
    }
    else
        return nil;
}
@end
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMMacroDocument(iTMTextDelegate)

