/*
//  iTMMacroDocument.m
//  macros
//
//  Created by jerome.laurens@u-bourgogne.fr on Mon Jan 14 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSDictionary.h>
#import <Foundation/NSUndoManager.h>
#import <Foundation/NSArchiver.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSAttributedString.h>
#import <AppKit/NSAttributedString.h>
#import <AppKit/NSColor.h>
#import <AppKit/NSWindow.h>
#import <AppKit/NSWindowController.h>
#import <AppKit/NSFont.h>
#import <AppKit/NSOutlineView.h>
#import <AppKit/AppKit.h>
#import "iTMMacroDocument.h"
#import "NSMenu(iTeXMac).h"
#import "iTMLiteScanner.h"
#import "iTMMacroMenuItem.h"
#import "iTMTextMacro.h"
#import "iTMMacrosController.h"
#import "iTMRootMacrosMenu.h"
#import "NSMenu(iTeXMac).h"
#import "NSControl(iTeXMac).h"

NSString * const iTMMacroSelectedText = @"iTMMacroSelectedText";
NSString * const iTMMacroDocumentType = @"iTeXMac Macros";
NSString * const iTMHelpMacrosEditorKey = @"Macros+customize";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMMacroDocument
/*"For undo purpose (critical) each menu item is represented by its own text view. Separators and items with submenus
share the defaultMacroTextView (do not forget to set its initial value for example when the window controller has loaded its nib."*/
@implementation iTMMacroDocument
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [self setOutlineView: nil];
    [self setRootMenu: nil];
    [self setMacroTextView: nil];
    [self setSelectedItem: nil];
    [self setActionPopUp: nil];
    [self setTooltipField: nil];
    NSLog(@"dealloc: %@", self);[super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowNibName
- (NSString *) windowNibName
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return @"iTMMacroDocument";
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowControllerDidLoadNib:
- (void) windowControllerDidLoadNib: (NSWindowController *) aController
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super windowControllerDidLoadNib: aController];
    [[NSNotificationCenter defaultCenter] addObserver: self
        selector: @selector(windowWillClose:)
            name: NSWindowWillCloseNotification
                object: [aController window]];
    [self setSelectedItem: nil];
    [aController setWindowFrameAutosaveName: [self windowNibName]];
    [aController setShouldCascadeWindows: YES];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dataRepresentationOfType:
- (NSData *) dataRepresentationOfType: (NSString *) type
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [NSArchiver archivedDataWithRootObject: [self rootMenu]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  saveDocumentAs:
- (void) saveDocumentAs: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super saveDocumentAs: sender];
    [[self rootMenu] clearMacroItemsChangeStatus];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  saveDocument:
- (void) saveDocument: (id) sender
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super saveDocument: sender];
    [[self rootMenu] clearMacroItemsChangeStatus];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  readFromFile:ofType:
- (BOOL) readFromFile: (NSString *) fileName ofType: (NSString *) type
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setRootMenu: [[iTMMacrosController sharedMacrosController] macrosMenuAtPath: fileName]];
    return [self rootMenu] != nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  readFromFile:ofType:
- (BOOL) writeToFile: (NSString *) fileName ofType: (NSString *) type
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self updateMenuItemFromUI: _SI];
    return [super writeToFile: fileName ofType: type];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  itemDidChangeStatus:
- (void) itemDidChangeStatus: (NSNotification *) aNotification
/*"Document undo manager compliant.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    id item = [aNotification object];
    [[self outlineView] reloadItem: item reloadChildren: NO];
    {
        NSMenu * M, * SM = [item menu];
        while((M = SM, SM = [M supermenu]) && (item = [SM itemAtIndex: [SM indexOfItemWithSubmenu: M]]))
            [[self outlineView] reloadItem: item reloadChildren: NO];
    }
    [self validateUserInterfaceItems];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  rootMenu
- (NSMenu *) rootMenu
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _MM? _MM: (_MM = [[NSMenu alloc] initWithTitle: @"Root"]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setRootMenu:
- (void) setRootMenu: (NSMenu *) argument
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(!argument || [argument isKindOfClass: [NSMenu class]])
    {
        [_MM autorelease];
        _MM = [argument retain];
    }
    else
        NSLog(@"Bad macro file format");
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  outlineView
- (id) outlineView
/*"Outlet connection.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _OLV;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setOutlineView:
- (void) setOutlineView: (id) argument
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![_OLV isEqual: argument])
        {
        [_OLV autorelease];
        _OLV = [argument retain];
        if(_OLV)
        {
            [_OLV setDelegate: self];
            [_OLV setDataSource: self];
            #if 0
            {
                NSTableColumn * TC = [_OLV tableColumnWithIdentifier: @"iTM-MacroStatus"];
                NSCell * C = [[NSCell alloc] initTextCell: [NSString string]];
                [C setAlignment: NSLeftTextAlignment];
                [C setFont: [NSFont fontWithName: @"Monaco" size: 14.0]];
                [TC setDataCell: C];
                [C release];
            }
            #endif
            [_OLV setAutoresizesOutlineColumn: NO];
            [_OLV setDataSource: self];
            [_OLV registerForDraggedTypes: [NSArray arrayWithObjects: iTMMacroMenuItemPBType, iTMMacroMenuItemsPBType, nil]];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  macroTextView
- (id) macroTextView
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _MTV;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMacroTextView:
- (void) setMacroTextView: (id) argument
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSTextView class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSTextView argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else if(![_MTV isEqual: argument])
    {
        [_MTV setDelegate: nil];
        [[_MTV textStorage] setDelegate: nil];
        [_MTV autorelease];
        _MTV = [argument retain];
        if(_MTV)
        {
            [_MTV setDelegate: self];
            [[_MTV textStorage] setDelegate: self];
            [_MTV setSmartInsertDeleteEnabled: NO];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  actionPopUp
- (id) actionPopUp
/*"Outlet connexion.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _APU;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setActionPopUp:
- (void) setActionPopUp: (id) argument
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSPopUpButton class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSPopUpButton argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else if(![_APU isEqual: argument])
    {
        [_APU autorelease];
        _APU = [argument retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  tooltipField
- (id) tooltipField
/*"Outlet connexion.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _TF;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setTooltipField:
- (void) setTooltipField: (id) argument
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(argument && ![argument isKindOfClass: [NSTextField class]])
        [NSException raise: NSInvalidArgumentException format: @"-[%@ %@] NSTextField argument expected: %@.",
            [self class], NSStringFromSelector(_cmd), argument];
    else if(![_TF isEqual: argument])
    {
        [_TF autorelease];
        _TF = [argument retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  selectedItem
- (id) selectedItem
/*"Cached outline view currently selected item.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _SI;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setSelectedItem:
- (void) setSelectedItem: (id) argument
/*"Critical: this message MUST be sent each time the selection is changing or to update the data from the UI as a side effect.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self updateMenuItemFromUI: _SI];
    if(![_SI isEqual: argument])
    {
        if(_SI)
            [[NSNotificationCenter defaultCenter] removeObserver: self name: nil object: _SI];
        [_SI autorelease];
        _SI = [argument retain];
        [[NSNotificationCenter defaultCenter] addObserver: self
            selector: @selector(itemDidChangeStatus:)
                name: iTMMacroMenuItemDidChangeStatusNotification
                    object: _SI];
    }
    [self updateUIFromMenuItem: _SI];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  menuItemUsesMacro:
- (BOOL) menuItemUsesMacro: (NSMenuItem *) menuItem;
/*"Document undo manager compliant.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    SEL action = [menuItem action];
    return ([menuItem isKindOfClass: [iTMMacroMenuItem class]] &&
        (action == @selector(insertMacro:) || (action == @selector(insertDirective:))));
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  menuItem:setTitle:
- (void) menuItem: (NSMenuItem *) menuItem setTitle: (NSString *) new
/*"Document undo manager compliant.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![menuItem isEqual: [NSMenuItem separatorItem]])
    {
        NSString * old = [menuItem title];
            new = new? new: [NSString string];
        if(![old isEqualToString: new])
        {
            NSUndoManager * UM = [self undoManager];
            [[UM prepareWithInvocationTarget: self] menuItem: menuItem setTitle: old];
            [menuItem setTitle: new];
            [self updateUIFromMenuItem: menuItem];
            if([UM isUndoing] || [UM isRedoing])
            {
                [self deepExpandItem: menuItem];
                [[self outlineView] reloadItem: menuItem reloadChildren: YES];
            }
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  menu:insertItem:atIndex:
- (void) menu: (NSMenu *) menu insertItem: (NSMenuItem *) item atIndex: (int) index
/*"Document undo manager compliant.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(menu && item && (index>=0) && (index<=[menu numberOfItems]))
    {
        NSOutlineView * OLV = [self outlineView];
        NSUndoManager * UM = [self undoManager];
        [[UM prepareWithInvocationTarget: self] menu: menu removeItemAtIndex: index];
        [menu insertItem: item atIndex: index];
        if(![UM isUndoing] && ![UM isRedoing])
            [UM setActionName: NSLocalizedStringFromTableInBundle(@"Insert Item", @"TeX",
                    [NSBundle bundleForClass: [self class]], " undo/redo menu item title for macro editor")];
        if([UM isUndoing] || [UM isRedoing])
        {
            [OLV reloadData];
            [self deepExpandItem: item];
            [OLV deselectAll: self];// BUGGY problems with selected items
//            [OLV selectRow: [OLV rowForItem: item] byExtendingSelection: NO];
//            [self updateUIFromMenuItem: item];// Critical
        }
        else
        {
            if([menu isEqual: [self rootMenu]])
                [OLV reloadData];
            else
            {
                NSMenu * M = [menu supermenu];
                [OLV reloadItem: [M itemAtIndex: [M indexOfItemWithSubmenu: menu]] reloadChildren: YES];
            }
        }
    }
    else
        NSLog(@"no insert %@, %@, %d, %d", menu, item, index, [menu numberOfItems]);
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  menu:removeItemAtIndex:
- (void) menu: (NSMenu *) menu removeItemAtIndex: (int) index
/*"Designated remover (undo compliant).
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: problems with selection
"*/
{
    if(menu && (index>=0) && (index<[menu numberOfItems]))
    {
        NSOutlineView * OLV = [self outlineView];
        NSUndoManager * UM = [self undoManager];
        id old = [menu itemAtIndex: index];
        [[UM prepareWithInvocationTarget: self] menu: menu insertItem: old atIndex: index];
        if(![UM isUndoing] && ![UM isRedoing])
            [UM setActionName: @"Remove Item"];
        [menu removeItemAtIndex: index];
        if([UM isUndoing] || [UM isRedoing])
        {
            NSOutlineView * OLV = [self outlineView];
            [OLV reloadData];
            [OLV deselectAll: self];// BUG!!!
        }
        else
        {
            if([menu isEqual: [self rootMenu]])
                [OLV reloadData];
            else
            {
                NSMenu * M = [menu supermenu];
                [OLV reloadItem: [M itemAtIndex: [M indexOfItemWithSubmenu: menu]] reloadChildren: YES];
            }
        }
    }
    else
        NSLog(@"no remove %@, %d, %d", menu, index, [menu numberOfItems]);
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  deepExpandItem:
- (void) deepExpandItem: (NSMenuItem *) item;
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: Missing scroll to visible
"*/
{
    NSOutlineView * OLV = [self outlineView];
    if([OLV rowForItem: item] == -1)
    {
        NSMutableArray * RA = [NSMutableArray array];
        id X, Y = [item menu];
        while(![Y isEqual: [self rootMenu]])
        {
            X = Y;
            Y = [X supermenu];
            [RA addObject: [Y itemAtIndex: [Y indexOfItemWithSubmenu: X]]];
        }
        {
            NSEnumerator * E = [RA reverseObjectEnumerator];
            while(item = [E nextObject])
                [OLV expandItem: item];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  windowWillClose:
- (void) windowWillClose: (NSNotification *) aNotification
/*"Critical: the data mopdel must be synchronized with the view.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setSelectedItem: nil];//usefull?
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  updateMenuItemFromUI:
- (void) updateMenuItemFromUI: (id) iMI
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([iMI isKindOfClass: [iTMMacroMenuItem class]])
    {
        #warning TO DO!!!
//        [iMI setMacro: [[self macroTextView] string]];
        NSTextStorage * TS = [[[[self macroTextView] textStorage] mutableCopy] autorelease];
        NSString * before = nil;
        NSMutableString * selected = [[[NSMutableString alloc] init] autorelease];
        NSString * after = nil;
        if([TS length])
        {
            NSMutableString * MS = [TS mutableString];
            NSRange R, foundRange;
            int location = 0;
//            NSLog(@"updateMenuItemFromUI: %@", [iMI title]);
            #define REPLACE(identifier, replaceString)\
            foundRange = NSMakeRange([TS length], 0);\
            while(YES)\
            {\
                foundRange = [MS rangeOfString: identifier\
                                    options: NSBackwardsSearch range: NSMakeRange(0, foundRange.location)];\
                if(foundRange.length)\
                    [MS replaceCharactersInRange: foundRange withString: replaceString];\
                else\
                    break;\
            }
//            NSLog(@"updateMenuItemFromUI: %@", [iMI title]);
            REPLACE([self nameDisplayString], iTMMacroNameKey);
//            NSLog(@"macro replaced:\n%@", TS);
            {
                int roof = [TS length];
                if(![TS attribute: NSBackgroundColorAttributeName atIndex: 0
                            longestEffectiveRange: &R inRange: NSMakeRange(0, roof)])
                {
                    before = [MS substringWithRange: R];
                    location = NSMaxRange(R);
                }
                if((location<roof) && ![TS attribute: NSBackgroundColorAttributeName atIndex: roof-1
                            longestEffectiveRange: &R inRange: NSMakeRange(location, roof-location)])
                {
                    after = [MS substringWithRange: R];
                    roof = R.location;
                }
                while(location<roof)
                {
                    if(![TS attribute: NSBackgroundColorAttributeName atIndex: location
                                    longestEffectiveRange: &R inRange: NSMakeRange(location, roof-location)])
                        [selected appendString: [MS substringWithRange: R]];
                    location+=R.length;
                }
             }
        }
        [[iMI macro] setMacroBefore: before];
        [[iMI macro] setMacroSelected: selected];
        [[iMI macro] setMacroAfter: after];
        [[iMI macro] setTooltip: [[self tooltipField] stringValue]];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  nameDisplayString
- (NSString *) nameDisplayString
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [NSString stringWithFormat: [NSString stringWithUTF8String: "«%@»"], [[self selectedItem] title]];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  attributedSelectionDelimiter
- (NSAttributedString *) attributedSelectionDelimiter
/*"Description forthcoming. Critical: THIS IS MEANT AS A 1 CHAR LENGTH STRING.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [[[NSAttributedString alloc] initWithString: @"|"
                attributes: [NSDictionary dictionaryWithObjectsAndKeys:
                    [[NSColor selectedTextBackgroundColor] highlightWithLevel: 0.5], NSBackgroundColorAttributeName,
                    [NSFont fontWithName: @"Monaco" size: 12.0], NSFontAttributeName, nil]] autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  updateUIFromMenuItem:
- (void) updateUIFromMenuItem: (id) iMI
/*"Description forthcoming. See #{textStorageWillProcessEditing:}.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSTextView * MTV = [self macroTextView];
    [MTV setEditable: YES];
    [MTV setString: [NSString string]];
    if([self menuItemUsesMacro: iMI])
    {
        NSMutableString * before = [[[[iMI macro] macroBefore] mutableCopy] autorelease];
        NSMutableString * selected = [[[[iMI macro] macroSelected] mutableCopy] autorelease];
        NSMutableString * after = [[[[iMI macro] macroAfter] mutableCopy] autorelease];
        NSRange searchRange, foundRange;
//        NSLog(@"updateUIFromMenuItem: %@", [iMI title]);
        #undef REPLACE
        #define REPLACE(argument, identifier, replaceString)\
        if([argument length])\
        {\
            foundRange = NSMakeRange([argument length], 0);\
            while(YES)\
            {\
                searchRange = NSMakeRange(0, foundRange.location);\
                foundRange = [argument rangeOfString: identifier options: NSBackwardsSearch range: searchRange];\
                if(foundRange.length)\
                    [argument replaceCharactersInRange: foundRange withString: replaceString];\
                else\
                    break;\
            }\
        }
        REPLACE(before, iTMMacroNameKey, [self nameDisplayString]);
        REPLACE(selected, iTMMacroNameKey, [self nameDisplayString]);
        REPLACE(after, iTMMacroNameKey, [self nameDisplayString]);
        [MTV setAllowsUndo: YES];
        [[[self macroTextView] undoManager] disableUndoRegistration];
        {
            NSMutableAttributedString * MAS = [[NSMutableAttributedString new] autorelease];
            if([before length])
                [MAS appendAttributedString:
                    [[[NSAttributedString alloc] initWithString: before] autorelease]];
            [MAS appendAttributedString: [self attributedSelectionDelimiter]];
//            NSLog(@"1-%@", MAS);
            if([selected length])
            {
                [MAS appendAttributedString:
                    [[[NSAttributedString alloc] initWithString: selected] autorelease]];
                [MAS appendAttributedString: [self attributedSelectionDelimiter]];
            }
//            NSLog(@"2-%@", MAS);
            if([after length])
            {
                [MAS appendAttributedString:
                    [[[NSAttributedString alloc] initWithString: after] autorelease]];
            }
            [MTV setSelectedRange: NSMakeRange(0, [[MTV string] length])];
            [MTV insertText: ([MAS length]? MAS: [NSString string])];
        }
        [[[self macroTextView] undoManager] enableUndoRegistration];
    }
    else
    {
        [[[self macroTextView] undoManager] disableUndoRegistration];
        [MTV setSelectedRange: NSMakeRange(0, [[MTV string] length])];
        [MTV insertText: [NSString string]];
        [[[self macroTextView] undoManager] enableUndoRegistration];
        [MTV setEditable: NO];
    }
    [self validateUserInterfaceItems];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateUserInterfaceItems
- (void) validateUserInterfaceItems
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[[self outlineView] window] setDocumentEdited: [self isDocumentEdited]];
    [[self outlineView] validateWindowContent];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isDocumentEdited
- (BOOL) isDocumentEdited
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return [super isDocumentEdited] || [[self rootMenu] areMacroMenuItemsEdited];
}
@end

#if 1
@interface iTMMacroWindow: NSWindow
- (void) undo: (id) sender;
- (void) redo: (id) sender;
- (BOOL) validateMenuItem: (NSMenuItem *) anItem;
@end

@implementation iTMMacroWindow
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  performMnemonic:
- (BOOL) performMnemonic: (NSString *) aString
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([aString length])
        switch([aString characterAtIndex: 0])
        {
            case 127:
            case 177:
            {
                SEL clear = @selector(clear:);
                id helper = [[self windowController] document];
                if(!helper)
                    helper = [self delegate];
                if([helper respondsToSelector: clear])
                {
                    [helper performSelector: clear withObject: self];
                    return YES;
                }
            }
        }
//    NSLog(@"performMnemonic: <%@>, %d", aString, ([aString length]? [aString characterAtIndex: 0]: -1));
    return [super performMnemonic: aString];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= undo:
- (void) undo: (id) sender
/*"Replaces old stuff which had absolutely no cocoa flavour.
Version history: jerome.laurens@u-bourgogne.fr (08/29/2001):
- < 1.1: 03/10/2002
To Do List: implement other actions
"*/
{
    NSTextView * MTV = [[[self windowController] document] macroTextView];
    NSUndoManager * UM = ([MTV isEqual: [self firstResponder]])?
        [MTV undoManager]: [self undoManager];
    if([UM canUndo])
        [UM undo];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= redo:
- (void) redo: (id) sender
/*"Replaces old stuff which had absolutely no cocoa flavour.
Version history: jerome.laurens@u-bourgogne.fr (08/29/2001):
- < 1.1: 03/10/2002
To Do List: implement other actions
"*/
{
    NSTextView * MTV = [[[self windowController] document] macroTextView];
    NSUndoManager * UM = ([MTV isEqual: [self firstResponder]])?
        [MTV undoManager]: [self undoManager];
    if([UM canRedo])
        [UM redo];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= validateMenuItem:
- (BOOL) validateMenuItem: (NSMenuItem *) anItem
/*"Replaces old stuff which had absolutely no cocoa flavour.
Version history: jerome.laurens@u-bourgogne.fr (08/29/2001):
- < 1.1: 03/10/2002
To Do List: implement other actions
"*/
{
    SEL selector = [anItem action];
    // File menu
    if(selector == @selector(undo:))
    {
        NSTextView * MTV = [[[self windowController] document] macroTextView];
        NSUndoManager * UM = ([MTV isEqual: [self firstResponder]])?
            [MTV undoManager]: [self undoManager];
        if(UM)
        {
            NSString * title = [UM undoMenuItemTitle];
            [anItem setTitle: ([title hasPrefix: @"&"]? [title substringWithRange: NSMakeRange(1, [title length]-1)]: title)];
//            NSLog(@"%@ = %@", UM, [[[self windowController] document] undoManager]);
            return [UM canUndo];
        }
        else
            return NO;
    }
    else if(selector == @selector(redo:))
    {
        NSTextView * MTV = [[[self windowController] document] macroTextView];
        NSUndoManager * UM = ([MTV isEqual: [self firstResponder]])?
            [MTV undoManager]: [self undoManager];
        NSString * title = UM? [UM redoMenuItemTitle]: [[self undoManager] redoMenuItemTitle];
        [anItem setTitle: ([title hasPrefix: @"&"]? [title substringWithRange: NSMakeRange(1, [title length]-1)]: title)];
        return [UM canRedo];
    }
    else
        return [super validateMenuItem: anItem];
}
@end
#endif
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMMacroDocument
