/*
//  iTMMacroMenuItem.m
//  macros
//
//  Created by jerome.laurens@u-bourgogne.fr on Sun Jan 20 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSUndoManager.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSString.h>
#import <Foundation/Foundation.h>
#import "NSString(iTeXMac).h"
#import "iTMMacroMenuItem.h"
#import "iTMTextMacro.h"

NSString * const iTMMacroMenuItemPBType = @"iTMMacroMenuItemPBType";
NSString * const iTMMacroMenuItemsPBType = @"iTMMacroMenuItemsPBType";

NSString * const iTMMacroMenuItemDidChangeStatusNotification = @"iTMMacroMenuItemDidChangeStatus";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMMacroMenuItem
/*"Description forthcoming."*/
@implementation iTMMacroMenuItem
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize
/*"The version number is 1. Subclasses might have to manage their own version number.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [self setVersion: 2];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  encodeWithCoder:
- (void) encodeWithCoder: (NSCoder *) coder
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super encodeWithCoder: coder];
    {
        NSString * S = [_Macro string];
        [coder encodeObject: ([S length]? S: nil)];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initWithCoder:
- (id) initWithCoder: (NSCoder *) decoder
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    int version = [decoder versionForClassName: NSStringFromClass([iTMMacroMenuItem class])];
    self = [super initWithCoder: decoder];
    // due to performance reasons the macros menu don't auto enable their items
    if([self action]==NULL)
        [self setEnabled: NO];
    [self setHasUndoManager: YES];
    if(version < 1)
    {
        NSString * before = [decoder decodeObject];
        if(!before || [before isKindOfClass: [NSString class]])
        {
            NSString * selected = [decoder decodeObject];
            if(!selected || [selected isKindOfClass: [NSString class]])
            {
                NSString * after = [decoder decodeObject];
                if(!after || [after isKindOfClass: [NSString class]])
                {
                    [self setMacro: [iTMTextMacro macroV1WithBefore: before selected: selected after: after]];
                    return self;
                }
            }
        }
    }
    else
    {
        id object = [decoder decodeObject];
        if(!object || [object isKindOfClass: [NSString class]])
            [self setMacro: [[[iTMTextMacro alloc]initWithString: object] autorelease]];
        else
            NSLog(@"Something wrong has occured somewhere: <%@>", object);
        return self;
    }
    NSLog(@"-[iTMMacroMenuItem initWithCoder:] corrupted archive, menu item ignored.");
    [self dealloc];
    return nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  macro
- (id) macro;
/*"Lazy initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _Macro? _Macro: (_Macro = [[iTMTextMacro alloc] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMacro:
- (void) setMacro: (iTMTextMacro *) argument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![_Macro isEqual: argument])
    {
        [_Macro release];
        _Macro = [argument retain];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [[NSNotificationCenter defaultCenter] removeObserver: nil name: nil object: self];
    [self setUndoManager: nil];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setAction:
- (void) setAction: (SEL) action
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    SEL old = [self action];
    if(old != action)
    {
        [super setAction: action];
        _actionModified = YES;
        [[NSNotificationCenter defaultCenter] postNotificationName: iTMMacroMenuItemDidChangeStatusNotification object: self];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  undoManager
- (NSUndoManager *) undoManager
/*"Description forthcoming. No lazy.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _UM;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setUndoManager:
- (void) setUndoManager: (NSUndoManager *) undoManager
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(![_UM isEqual: undoManager])
    {
        if(_UM)
            [[NSNotificationCenter defaultCenter] removeObserver: self name: nil object: _UM];
        [_UM autorelease];
        _UM = [undoManager retain];
        if(_UM)
        {
            [[NSNotificationCenter defaultCenter] addObserver: self
                selector: @selector(undoManagerDidRedoChange:)
                    name: NSUndoManagerDidRedoChangeNotification
                        object: _UM];
            [[NSNotificationCenter defaultCenter] addObserver: self
                selector: @selector(undoManagerDidUndoChange:)
                    name: NSUndoManagerDidUndoChangeNotification
                        object: _UM];
            [[NSNotificationCenter defaultCenter] addObserver: self
                selector: @selector(undoManagerWillCloseUndoGroup:)
                    name: NSUndoManagerWillCloseUndoGroupNotification
                        object: _UM];
        }
    }
    return;
}
@end

@implementation iTMMacroMenuItem(iTMUndoManagement)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  changesArePending
- (BOOL) changesArePending
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _ChangesArePending;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setChangesArePending:
- (void) setChangesArePending: (BOOL) aFlag
/*"Must be called with YES if changes are actually pending.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _ChangesArePending = aFlag;
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isMacroMenuItemEdited:
- (BOOL) isMacroMenuItemEdited
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _actionModified || (_ChangeCount != 0);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  updateChangeCount
- (void) updateChangeCount: (iTMMacroChangeType) change
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
//    NSLog(@"updateChangeCount of %@: %d", self, change);
    switch(change)
    {
        case iTMMacroChangeDone:
            ++_ChangeCount;
            break;
        case iTMMacroChangeUndone:
            --_ChangeCount;
            break;
        case iTMMacroChangeCleared:
            _ChangeCount = 0;
            _actionModified = NO;
            break;
        default:
            NSLog(@"Bad change in ITMMacroMenuItem");
            break;
    }
    [[NSNotificationCenter defaultCenter] postNotificationName: iTMMacroMenuItemDidChangeStatusNotification object: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  undoManagerDidRedoChange:
- (void) undoManagerDidRedoChange: (NSNotification *) aNotification
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self updateChangeCount: iTMMacroChangeDone];
    [self setChangesArePending: NO];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  undoManagerDidUndoChange:
- (void) undoManagerDidUndoChange: (NSNotification *) aNotification
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self updateChangeCount: iTMMacroChangeUndone];
    [self setChangesArePending: NO];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  undoManagerWillCloseUndoGroup:
- (void) undoManagerWillCloseUndoGroup: (NSNotification *) aNotification
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([self changesArePending])
    {
        NSUndoManager * UM = [self undoManager];
//        NSLog(@"undoManagerWillCloseUndoGroup: undo group level: %d", [UM groupingLevel]);
        if(([UM groupingLevel] == 1) && ![UMisUndoing] && ![UMisRedoing])
            [self updateChangeCount: iTMMacroChangeDone];
    }
    [self setChangesArePending: NO];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  undoManagerDidUndoChange:
- (BOOL) hasUndoManager
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _HasUndoManager;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  undoManagerDidUndoChange:
- (void) setHasUndoManager: (BOOL) flag
/*"Default is NO to prevent registering unnecessary operations while initing the menu item.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    _HasUndoManager = flag;
    if(_HasUndoManager && ![self undoManager])
        [self setUndoManager: [[[NSUndoManager alloc]init]autorelease]];
    return;
}
@end

@implementation NSMenuItem(iTMMacro)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  isEqualOrHasSubmenuItem:
- (BOOL) isEqualOrHasSubmenuItem: (NSMenuItem *) argument
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([self isEqual: argument])
        return YES;
    else
    {
        NSMenu * M = [argument menu];
        NSMenu * SM = [M supermenu];
        return SM? [self isEqualOrHasSubmenuItem: [SM itemAtIndex: [SM indexOfItemWithSubmenu: M]]]: NO;
    }
}
@end


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMMacroMenuItem

