/*
//  iTMMacrosController.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Fri Oct 12 2001.
//  Copyright © 2001-2002 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import <Foundation/NSString.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSException.h>
#import <AppKit/NSApplication.h>
#import <AppKit/NSFileWrapper.h>
#import <AppKit/NSMenuItem.h>
#import <AppKit/NSMenu.h>
#import <AppKit/NSWindow.h>
#import <AppKit/NSTextView.h>
#import <AppKit/AppKit.h>
#import "NSMenu(iTeXMac).h"
#import "iTMMacrosController.h"
#import "iTMRootMacrosMenu.h"
//#import "iTMListMenu.h"
#import "iTMMacroMenuItem.h"
#import "iTMTextMacro.h"
#import "iTMResourcesController(iTMTeXKit).h"
//#import "iTMProjectDocument.h"
//#import "iTMProjectDocument(Accessors).h"
#import "iTMStatusNotification.h"
#import "iTMSettings.h"
#import "iTMTextView.h"
#import "iTMLiteScanner.h"

NSString * const iTMICDocumentMacrosKey = @"Document";
NSString * const iTMICGraphicsMacrosKey = @"Graphics";
NSString * const iTMICMathMacrosKey = @"Math";
NSString * const iTMICMiscMacrosKey = @"Misc";
NSString * const iTMICTextMacrosKey = @"Text";
NSString * const iTMICTmplMacrosKey = @"Tmpl";

@interface iTMMacrosController(MacrosPrivate)
- (NSString *) docMacrosPathForDocument: (NSDocument *) argument;
- (NSString *) gfxMacrosPathForDocument: (NSDocument *) argument;
- (NSString *) mathMacrosPathForDocument: (NSDocument *) argument;
- (NSString *) miscMacrosPathForDocument: (NSDocument *) argument;
- (NSString *) textMacrosPathForDocument: (NSDocument *) argument;
- (NSString *) templatesPathForDocument: (NSDocument *) argument;
- (void) loadMacros;
- (void) loadTemplates;
@end;

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMMacrosController
/*"Description forthcoming."*/
@implementation iTMMacrosController
static id _iTMMacrosController = nil;
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= sharedMacrosController
+ (id) sharedMacrosController;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return _iTMMacrosController? _iTMMacrosController:
                        (_iTMMacrosController = [[self allocWithZone: [NSApp zone]] init]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= init
- (id) init;
/*"The first insert controller created is the shared one. It is the only one ever created.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(_iTMMacrosController)
    {
        if(![self isEqual: _iTMMacrosController])
            [super dealloc];
        return [_iTMMacrosController retain];
    }
    else
    {
        self = [super init];
        [_CMTM autorelease];
        _CMTM = nil;
        [_MMI autorelease];
        _MMI = nil;
        [_TMI autorelease];
        _TMI = nil;
        return _iTMMacrosController = self;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= dealloc
- (void) dealloc;
/*"The first insert controller created is the shared one. It is the only one ever created.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [_CMTM release];
    _CMTM = nil;
    [_MMI release];
    _MMI = nil;
    [_TMI release];
    _TMI = nil;
    [super dealloc];
    return;
}
@end

@interface iTMMacrosController(iTMPrivate)
/*"Class methods"*/
/*"Setters and Getters"*/
/*"Main methods"*/
//- (void) insertTemplate: (id <NSMenuItem>) sender;
- (void) insertMacrosAtPath: (NSString *) aPath inMenu: (NSMenu *) aMenu atIndex: (int) index;
- (void) insertTemplatesAtPath: (NSString *) aPath inMenu: (NSMenu *) aMenu atIndex: (int) index;
- (NSArray *) templateFilesDirectories;
- (NSArray *) macrosFilesForKey: (NSString *) aKey;
@end

@implementation iTMMacrosController(iTMTeXKit)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= insertTemplate:
- (void) insertTemplate: (id <NSMenuItem>) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSTextView * target = [[NSApp mainWindow] firstResponder];
    if([target isKindOfClass: [NSTextView class]])
        [target insertText: [NSString stringWithContentsOfFile: [sender representedObject]]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= editMacroFile:
- (void) editMacroFile: (id <NSMenuItem>) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMRootMacrosMenu * menu = [sender menu];
    if([menu isKindOfClass: [iTMRootMacrosMenu class]])
    {
        NSString * path = [menu macroFileName];
        if(![[NSDocumentController sharedDocumentController] openDocumentWithContentsOfFile: path display: YES])
            [[NSWorkspace sharedWorkspace] openFile: path];
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= insertMacro:
- (void) insertMacro: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([sender isKindOfClass: [iTMMacroMenuItem class]])
    {
        id target = [[NSApp mainWindow] firstResponder];
        if([target isKindOfClass: [NSTextView class]])
        {
            NSRange oldSelectedRange = [target selectedRange];
            NSString * currentSelection = [[target string] substringWithRange: oldSelectedRange];
            NSRange newSelectedRange;
            NSString * indentPrefix = [NSString string];
            {
                unsigned begin;
                [[target string] getLineStart: &begin end: nil contentsEnd: nil forRange: oldSelectedRange];
                if(begin < oldSelectedRange.location)
                {
                    NSCharacterSet * blackLetterSet = [[NSCharacterSet whitespaceCharacterSet] invertedSet];
                    unsigned length = [[target string] rangeOfCharacterFromSet: blackLetterSet
                        options: 0 range: NSMakeRange(begin, oldSelectedRange.location - begin)].location - begin;
                    if(length)
                        indentPrefix = [[target string] substringWithRange: NSMakeRange(begin, length)]; 
                }
            }
            [target insertText: [[sender macro] macroStringForName: [sender title]
                selection: currentSelection indent: indentPrefix selectedRangePointer: &newSelectedRange]];
            newSelectedRange.location += oldSelectedRange.location;
            [target setSelectedRange: newSelectedRange];
            if([[[sender macro] tooltip] length])
                [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
                    [NSString stringWithFormat: @"[%@] %@", [sender title], [[sender macro] tooltip]]];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= insertDirective:
- (void) insertDirective: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([sender isKindOfClass: [iTMMacroMenuItem class]])
    {
        id target = [[NSApp mainWindow] firstResponder];
        if([target isKindOfClass: [NSTextView class]])
        {
            NSString * string = [target string];
            unsigned ceiling = [string length];
            unsigned location = 0;
            iTMLiteScanner * S = [iTMLiteScanner scannerWithString: string];
            NSCharacterSet * CRLF = [NSCharacterSet characterSetWithCharactersInString: @"\n\r"];
            BOOL alreadyNewline = YES;
            [S setScanLocation: 0];
            while([S scanLocation] < ceiling)
            {
                location = [S scanLocation];
                [S setCharactersToBeSkipped: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
                if([S scanString: @"%" intoString: nil])
                {
                    [S setCharactersToBeSkipped: [NSCharacterSet whitespaceCharacterSet]];
                    [S scanUpToCharactersFromSet: CRLF intoString: nil];
                    alreadyNewline = [S scanCharactersFromSet: CRLF intoString: nil];
                }
                else
                    break;
            }
            if(!alreadyNewline)
                [target insertText: @"\n"];
            [target setSelectedRange: NSMakeRange(location, 0)];
            [target insertText: [[sender macro] macroStringForName: [sender title]
                selection: [NSString string] indent: [NSString string] selectedRangePointer: nil]];
        }
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= respondsToSelector:
- (BOOL) respondsToSelector: (SEL) aSelector;
/*"Just to validate the menu items.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(aSelector == @selector(insertMacrosMenuItem:))
        return YES;
    else if(aSelector == @selector(insertTemplatesMenuItem:))
        return YES;
    else
        return [super respondsToSelector: aSelector];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= loadMacros
- (void) loadMacrosAndTemplates: (id) sender;
/*"Just to validate the menu items.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self loadMacros];
    [self loadTemplates];
    [[NSNotificationCenter defaultCenter] postNotificationName: @"iTMLoadMacrosAndTemplates" object: self];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= loadMacros
- (void) loadMacros;
/*"Just to validate the menu items.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMenu * menu = [_MMI menu];
    [_CMTM autorelease];
    _CMTM = nil;
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
        NSLocalizedStringFromTableInBundle(@"Loading Macros.", @"TeX",
            [NSBundle bundleForClass: [self class]], "")];
    {
        int index = [menu indexOfItem: _MMI]+1;
        NSMenuItem * MI;
        while((index < [menu numberOfItems]) &&
                (MI = [menu itemAtIndex: index], MI != [NSMenuItem separatorItem]) &&
                    ([MI hasSubmenu] ||
                        ([MI action] == @selector(insertMacro:))))
            [menu removeItemAtIndex: index];
        [self insertMacrosAtPath: [[iTMResourcesController sharedResourcesController] macrosDirectoryPath]
            inMenu: menu atIndex: index];
    }
    [menu update];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= loadTemplates
- (void) loadTemplates;
/*"Just to validate the menu items.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMenu * menu = [_TMI menu];
    [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
        NSLocalizedStringFromTableInBundle(@"Loading Templates.", @"TeX",
            [NSBundle bundleForClass: [self class]], "")];
    {
        int index = [menu indexOfItem: _TMI]+1;
        NSMenuItem * MI;
        while((index < [menu numberOfItems]) &&
                (MI = [menu itemAtIndex: index], MI != [NSMenuItem separatorItem]) &&
                    ([MI hasSubmenu] ||
                        ([MI action] == @selector(insertTemplate:))))
            [menu removeItemAtIndex: index];
        [self insertTemplatesAtPath: [[iTMResourcesController sharedResourcesController] templatesDirectoryPath]
            inMenu: menu atIndex: index];
    }
    [menu update];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  validateMenuItem:
- (BOOL) validateMenuItem: (id) aMenuItem;
/*" This concerns only the magnification menu items of the PDF Display menu and the
corresponding ones in the popUp Fit button. Only takes into account the upper limit
of the magnification."*/
{
    SEL action = [aMenuItem action];
//    NSLog(@"validating: %@", NSStringFromSelector(action));
    if((action == @selector(insertTemplate:)) || (action == @selector(insertMacro:)))
        return ([NSApp targetForAction: @selector(shouldChangeTextInRange:replacementString:)] != nil);
    else if(action == @selector(insertMacrosMenuItem:))
    {
        [_MMI autorelease];
        _MMI = [aMenuItem retain];
        [aMenuItem setAction: NULL];//critical: infinite loop if after
//        [self loadMacros];
        return NO;
    }
    else if(action == @selector(insertTemplatesMenuItem:))
    {
        [_TMI autorelease];
        _TMI = [aMenuItem retain];
        [aMenuItem setAction: NULL];//critical: infinite loop if after
//        [self loadTemplates];
        return NO;
    }
    else if(action == @selector(editMacroFile:))
    {
        return [[aMenuItem menu] isKindOfClass: [iTMRootMacrosMenu class]];
    }
    else if((action == @selector(loadMacrosAndTemplates:)))
        return YES;
    else
        return NO;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  insertMacrosAtPath:inMenu:atIndex:
- (void) insertMacrosAtPath: (NSString *) aPath inMenu: (NSMenu *) aMenu atIndex: (int) index;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSFileWrapper * FW = [[NSFileWrapper alloc] initWithPath: aPath];
    NSEnumerator * E;
    NSString * key;
    NSString * prefix;
    NSArray * RA = [[[FW fileWrappers] allKeys] sortedArrayUsingSelector: @selector(compare:)];
    index = MAX(0, MIN([aMenu numberOfItems], index));
    E = [RA objectEnumerator];
    prefix = index? @"   ": [NSString string];
    while(key = [E nextObject])
    {
        NSFileWrapper * F = [[FW fileWrappers] objectForKey: key];
        if([F isRegularFile])
        {
            NSString * name = [F preferredFilename];
            if([@"mtexmac" isEqualToString: [[name pathExtension] lowercaseString]])
            // critical: problem with encoding
            {
                NSMenuItem * MI = [aMenu insertItemWithTitle:
                    [prefix stringByAppendingString: [name stringByDeletingPathExtension]]
                        action: NULL keyEquivalent: [NSString string] atIndex: index++];
                [MI setSubmenu: [self macrosMenuAtPath: [aPath stringByAppendingPathComponent: name]]];
            }
        }
    }
    E = [RA objectEnumerator];
    while(key = [E nextObject])
    {
        NSFileWrapper * F = [[FW fileWrappers] objectForKey: key];
        if([F isDirectory])
        {
            NSString * name = [F preferredFilename];
            if(![name hasPrefix: @"."])
            {
                NSMenu * menu = [[NSMenu allocWithZone: [NSMenu menuZone]] initWithTitle: [NSString string]];
                [self insertMacrosAtPath: [aPath stringByAppendingPathComponent: name] inMenu: menu atIndex: 0];
                if([menu numberOfItems])
                {
                    NSMenuItem * MI = [aMenu insertItemWithTitle:
                        [prefix stringByAppendingString: [name stringByDeletingPathExtension]]
                            action: NULL keyEquivalent: [NSString string] atIndex: index++];
                    [MI setSubmenu: menu];
                }
                [menu autorelease];
            }
            else
                NSLog(@"skipped: %@", name);
        }
    }
    [FW autorelease];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  insertTemplatesAtPath:inMenu:atIndex:
- (void) insertTemplatesAtPath: (NSString *) aPath inMenu: (NSMenu *) aMenu atIndex: (int) index;
/*"Description Forthcoming.
Version History: jerome.laurens@u-bourgogne.fr (09/02/2001)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSFileWrapper * FW = [[NSFileWrapper alloc] initWithPath: aPath];
    NSEnumerator * E;
    NSString * key;
    NSString * prefix;
    NSArray * RA = [[[FW fileWrappers] allKeys] sortedArrayUsingSelector: @selector(compare:)];
    index = MAX(0, MIN([aMenu numberOfItems], index));
    prefix = index? @"   ": [NSString string];
    E = [RA objectEnumerator];
    while(key = [E nextObject])
    {
        NSFileWrapper * F = [[FW fileWrappers] objectForKey: key];
        if([F isRegularFile])
        {
            NSString * name = [F preferredFilename];
            if(![name hasPrefix: @"."])
            {
                NSMenuItem * MI = [aMenu insertItemWithTitle:
                    [prefix stringByAppendingString: [name stringByDeletingPathExtension]]
                        action: @selector(insertTemplate:) keyEquivalent: [NSString string] atIndex: index++];
                [MI setTarget: self];
                [MI setRepresentedObject: [aPath stringByAppendingPathComponent: name]];
            }
        }
    }
    E = [RA objectEnumerator];
    while(key = [E nextObject])
    {
        NSFileWrapper * F = [[FW fileWrappers] objectForKey: key];
        if([F isDirectory])
        {
            NSString * name = [F preferredFilename];
            if(![name hasPrefix: @"."])
            {
                NSMenu * menu = [[NSMenu allocWithZone: [NSMenu menuZone]] initWithTitle: [NSString string]];
                [self insertTemplatesAtPath: [aPath stringByAppendingPathComponent: name] inMenu: menu atIndex: 0];
                if([menu numberOfItems])
                {
                    NSMenuItem * MI = [aMenu insertItemWithTitle:
                        [prefix stringByAppendingString: [name stringByDeletingPathExtension]]
                            action: NULL keyEquivalent: [NSString string] atIndex: index++];
                    [menu deepSetAutoenablesItems: NO];
                    [MI setSubmenu: menu];
                }
                [menu autorelease];
            }
            else
                NSLog(@"skipped: %@", name);
        }
    }
    [FW autorelease];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= macrosMenuAtPath:
- (NSMenu *) macrosMenuAtPath: (NSString *) path;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([[path pathExtension] isEqualToString: @"mTeXMac"])
    {
        [NSUnarchiver decodeClassName: @"iTMRootMacroMenu" asClassName: @"iTMRootMacrosMenu"];
        {
            iTMRootMacrosMenu * menu = [NSUnarchiver unarchiveObjectWithFile: path];
            if([menu isKindOfClass: [iTMRootMacrosMenu class]])
            {
                [menu setMacroFileName: [path stringByResolvingSymlinksAndFinderAliasesInPath]];
                [menu deepSetTarget: self];
                [menu deepSetAutoenablesItems: NO];
                return menu;
            }
            else
                NSLog(@"iTMRootMacroMenu expected, got %@", menu);
        }
    }
    NSLog(@"No menu available at path:\n%@", path);
    return nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= buttonMacrosMenuAtPath:
- (NSMenu *) buttonMacrosMenuAtPath: (NSString *) path;
/*"A layer above #{macrosMenuAtPath:} to cache the macros menus. That way, only one instance of a menu is used in a button, and we gain some performance when loading files and closing source file. The next time at least.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMenu * result = [_CMTM objectForKey: path];
    if(![result isKindOfClass: [NSMenu class]])
    {
        if(!_CMTM)
            _CMTM = [[NSMutableDictionary dictionary] retain];
        result = [self macrosMenuAtPath: path];
        if(result)
            [_CMTM setObject: result forKey: path];
    }
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= MACROMENU(, )
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= docMacrosMenuForDocument:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= gfxMacrosMenuForDocument:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= mathMacrosMenuForDocument:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= miscMacrosMenuForDocument:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= textMacrosMenuForDocument:
#define MACROMENU(selector, macrosPathSelector, default)\
- (NSMenu *) selector: (id) aDocument;\
{\
    NSString * path = [self respondsToSelector: @selector(macrosPathSelector:)]?\
                                [self macrosPathSelector: aDocument]: default;\
    if([path hasPrefix: @"."])\
        path = [[[aDocument fileName] stringByDeletingLastPathComponent] stringByAppendingPathComponent: path];\
    else if(![path hasPrefix: @"/"])\
        path = [[[iTMResourcesController sharedResourcesController] macrosDirectoryPath]\
                                                        stringByAppendingPathComponent: path];\
    if(![[path pathExtension] length])\
        path = [path stringByAppendingPathExtension: @"mTeXMac"];\
    return [self buttonMacrosMenuAtPath: path];\
}
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
#warning should be in the defaults?
MACROMENU(docMacrosMenuForDocument, docMacrosPathForDocument, @"Document/LaTeX");
MACROMENU(gfxMacrosMenuForDocument, gfxMacrosPathForDocument, @"Graphics/LaTeX");
MACROMENU(mathMacrosMenuForDocument, mathMacrosPathForDocument, @"Math/LaTeX");
MACROMENU(miscMacrosMenuForDocument, miscMacrosPathForDocument, @"Misc/LaTeX");
MACROMENU(textMacrosMenuForDocument, textMacrosPathForDocument, @"Text/LaTeX");
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= templatesMenuForDocument:
- (NSMenu *) templatesMenuForDocument: (id) aDocument;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * path = [self respondsToSelector: @selector(templatesPathForDocument:)]?
                                [self templatesPathForDocument: aDocument]: @"LaTeX";
    if([path hasPrefix: @"."])
        path = [[[aDocument fileName] stringByDeletingLastPathComponent] stringByAppendingPathComponent: path];
    else if(![path hasPrefix: @"/"])
        path = [[[iTMResourcesController sharedResourcesController] templatesDirectoryPath]
                                                        stringByAppendingPathComponent: path];
    {
        NSMenu * menu = [[[NSMenu allocWithZone: [NSMenu menuZone]] initWithTitle:
                            [[path lastPathComponent] stringByDeletingPathExtension]] autorelease];
        [self insertTemplatesAtPath: path inMenu: menu atIndex: 0];
        return menu;
    }
    return nil;
}
@end

@implementation iTMMacrosController(iTMCombo)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= comboBox:indexOfItemWithStringValue:
- (unsigned int) comboBox: (NSComboBox *) aCB indexOfItemWithStringValue: (NSString *) aString;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    SEL action = [aCB action];
    if( action == @selector(macrosDocEdited:))
        return [[self macrosFilesForKey: iTMICDocumentMacrosKey] indexOfObject: aString];
    else if( action == @selector(macrosGfxEdited:))
        return [[self macrosFilesForKey: iTMICGraphicsMacrosKey] indexOfObject: aString];
    else if( action == @selector(macrosMathEdited:))
        return [[self macrosFilesForKey: iTMICMathMacrosKey] indexOfObject: aString];
    else if( action == @selector(macrosMiscEdited:))
        return [[self macrosFilesForKey: iTMICMiscMacrosKey] indexOfObject: aString];
    else if( action == @selector(macrosTextEdited:))
        return [[self macrosFilesForKey: iTMICTextMacrosKey] indexOfObject: aString];
    else// if( action == @selector(macrosTmplEdited:))
        return [[self templateFilesDirectories] indexOfObject: aString];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= comboBox:objectValueForItemAtIndex:
- (id) comboBox: (NSComboBox *) aCB objectValueForItemAtIndex: (int) index;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    SEL action = [aCB action];
    if( action == @selector(macrosDocEdited:))
    {
        NSArray * macros = [self macrosFilesForKey: iTMICDocumentMacrosKey];
        if((index>=0) && (index<[macros count]))
            return [macros objectAtIndex: index];
        else
            NSLog(@"out of bounds: %d, %d", index, [macros count]);
    }
    else if( action == @selector(macrosGfxEdited:))
    {
        NSArray * macros = [self macrosFilesForKey: iTMICGraphicsMacrosKey];
        if((index>=0) && (index<[macros count]))
            return [macros objectAtIndex: index];
        else
            NSLog(@"out of bounds: %d, %d", index, [macros count]);
    }
    else if( action == @selector(macrosMathEdited:))
    {
        NSArray * macros = [self macrosFilesForKey: iTMICMathMacrosKey];
        if((index>=0) && (index<[macros count]))
            return [macros objectAtIndex: index];
        else
            NSLog(@"out of bounds: %d, %d", index, [macros count]);
    }
    else if( action == @selector(macrosMiscEdited:))
    {
        NSArray * macros = [self macrosFilesForKey: iTMICMiscMacrosKey];
        if((index>=0) && (index<[macros count]))
            return [macros objectAtIndex: index];
        else
            NSLog(@"out of bounds: %d, %d", index, [macros count]);
    }
    else if( action == @selector(macrosTextEdited:))
    {
        NSArray * macros = [self macrosFilesForKey: iTMICTextMacrosKey];
        if((index>=0) && (index<[macros count]))
            return [macros objectAtIndex: index];
        else
            NSLog(@"out of bounds: %d, %d", index, [macros count]);
    }
    else// if( action == @selector(macrosTmplEdited:))
    {
        NSArray * macros = [self templateFilesDirectories];
        if((index>=0) && (index<[macros count]))
            return [macros objectAtIndex: index];
        else
            NSLog(@"out of bounds: %d, %d", index, [macros count]);
    }
    return nil;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= numberOfItemsInComboBox:
- (int) numberOfItemsInComboBox: (NSComboBox *) aCB;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    SEL action = [aCB action];
    if( action == @selector(macrosDocEdited:))
        return [[self macrosFilesForKey: iTMICDocumentMacrosKey] count];
    else if( action == @selector(macrosGfxEdited:))
        return [[self macrosFilesForKey: iTMICGraphicsMacrosKey] count];
    else if( action == @selector(macrosMathEdited:))
        return [[self macrosFilesForKey: iTMICMathMacrosKey] count];
    else if( action == @selector(macrosMiscEdited:))
        return [[self macrosFilesForKey: iTMICMiscMacrosKey] count];
    else if( action == @selector(macrosTextEdited:))
        return [[self macrosFilesForKey: iTMICTextMacrosKey] count];
    else// if( action == @selector(macrosTmplEdited:))
        return [[self templateFilesDirectories] count];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= templateFilesDirectories
- (NSArray *) templateFilesDirectories;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSFileWrapper * TFW = [[[NSFileWrapper alloc] initWithPath:
            [[iTMResourcesController sharedResourcesController] templatesDirectoryPath]] autorelease];
    id result = [NSMutableArray array];
    if([TFW isDirectory])
    {
        NSDictionary * TFWs = [TFW fileWrappers];
        NSEnumerator * enumerator = [[TFWs allKeys] objectEnumerator];
        NSString * key;
        while(key = [enumerator nextObject])
        {
            NSFileWrapper * fw = [TFWs objectForKey: key];
            NSString * pfn = [fw preferredFilename];
            if([fw isDirectory] && ![pfn hasPrefix: @"."])
                [result addObject: pfn];
        }
    }
    return [result sortedArrayUsingSelector: @selector(compare:)];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= macrosFilesForKey:
- (NSArray *) macrosFilesForKey: (NSString *) aKey;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSFileWrapper * MFW = [[[[[NSFileWrapper alloc] initWithPath:
            [[iTMResourcesController sharedResourcesController] macrosDirectoryPath]] autorelease]
                                        fileWrappers] objectForKey: aKey];
    id result = [NSMutableArray array];
    if([MFW isDirectory])
    {
        NSString * PFN = [MFW preferredFilename];
        NSDictionary * MFWs = [MFW fileWrappers];
        NSEnumerator * enumerator = [[MFWs allKeys] objectEnumerator];
        NSString * key;
        while(key = [enumerator nextObject])
        {
            NSFileWrapper * fw = [MFWs objectForKey: key];
            NSString * pfn = [fw preferredFilename];
            if([fw isRegularFile] && ![pfn hasPrefix: @"."] && [[pfn pathExtension] isEqualToString: @"mTeXMac"])
                [result addObject: [[PFN stringByAppendingPathComponent: pfn] stringByDeletingPathExtension]];
        }
    }
    return [result sortedArrayUsingSelector: @selector(compare:)];
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= iTMMacrosController

