/*
//  iTMRootMacrosMenu.m
//  macros
//
//  Created by jerome.laurens@u-bourgogne.fr on Fri Jan 25 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import "iTMRootMacrosMenu.h"
#import "iTMMacroMenuItem.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMRootMacrosMenu
/*"Description forthcoming."*/
@implementation iTMRootMacrosMenu
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc
/*"Description Forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setMacroFileName: nil];
    [super dealloc];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  macroFileName
- (NSString *) macroFileName
/*"if the receiver answer should be a 0 length string, asks its super menu to answer, if it can.
Part of the implementation of #{editMacroFile}
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if([_MFN length])
        return _MFN;
    else
	return [super macroFileName];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMacroFileName:
- (void) setMacroFileName: (NSString *) aFileName
/*"Part of the implementation of #{editMacroFile}. MUST be called by the controller when the macro menu has been inited.
Only the root menu is concerned.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List: ask the file manager before about the existence of the file...
"*/
{
    if(![_MFN isEqualToString: aFileName])
    {
        [_MFN autorelease];
        _MFN = [aFileName copy];
    }
    return;
}
@end

@implementation NSMenu(iTMMacroMenuItems)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  editedMacroItems
- (NSArray *) editedMacroItems
/*"Called when the file is saved to clear the undo status.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSMutableArray * result = [NSMutableArray new];
    NSEnumerator * E = [[self itemArray] objectEnumerator];
    id item;
    while(item = [E nextObject])
        if([item hasSubmenu] && [[item submenu] areMacroMenuItemsEdited])
            [result addObject: item];
        else if([item isKindOfClass: [iTMMacroMenuItem class]] && [item isMacroMenuItemEdited])
            [result addObject: item];
    return [result autorelease];
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  clearMacroItemsChangeStatus
- (void) clearMacroItemsChangeStatus
/*"Called when the file is saved to clear the undo status.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSEnumerator * E = [[self itemArray] objectEnumerator];
    id item;
    while(item = [E nextObject])
        if([item isKindOfClass: [iTMMacroMenuItem class]])
            [item updateChangeCount: iTMMacroChangeCleared];
        else if([item hasSubmenu])
            [[item submenu] clearMacroItemsChangeStatus];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  areMacroMenuItemsEdited
- (BOOL) areMacroMenuItemsEdited
/*"Called when the file is saved to clear the undo status.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    BOOL result = NO;
    NSEnumerator * E = [[self itemArray] objectEnumerator];
    id item;
    while(!result && (item = [E nextObject]))
        if([item isKindOfClass: [iTMMacroMenuItem class]])
            result = [item isMacroMenuItemEdited];
        else if([item hasSubmenu])
            result = [[item submenu] areMacroMenuItemsEdited];
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  macroFileName
- (NSString *) macroFileName
/*"if the receiver answer should be a 0 length string, asks its super menu to answer, if it can.
Part of the implementation of #{editMacroFile}
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    iTMRootMacrosMenu * SM = [self supermenu];
    return SM? [SM macroFileName]: [NSString string];
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMRootMacrosMenu

