/*
//  iTMTextMacro.m
//  iTeXMac
//
//  Created by jerome.laurens@u-bourgogne.fr on Thu Feb 21 2002.
//  Copyright (c) 2001 Laurens'Tribune. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software Foundation; either
//  version 2 of the License, or any later version, modified by the addendum below.
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details. You should have received a copy
//  of the GNU General Public License along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  GPL addendum: Any simple modification of the present code which purpose is to remove bug,
//  improve efficiency in both code execution and code reading or writing should be addressed
//  to the actual developper team.
//
//  Version history: (format "- date:contribution(contributor)") 
//  To Do List: (format "- proposition(percentage actually done)")
*/

#import "NSString(iTeXMac).h"
#import "iTMTextMacro.h"
#import "iTMLiteScanner.h"
#import "iTMStatusNotification.h"

NSString * const iTMMacroNameKey = @"<#NAME#>";
NSString * const iTMMacroSelectionKey = @"<#SEL#>";
NSString * const iTMMacroInsertionKey = @"<#INS#>";
NSString * const iTMMacroTabAnchorKey = @"<#TAB#>";
NSString * const iTMMacroTooltipKey = @"<#TOOLTIP#>";

NSString * const iTMUDTabAnchorStringKey = @"iTMUDTabAnchorString";

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTextMacro
/*"This class deals with macro strings. These are strings with some key words to control the insertion of the macro in the current text."*/
@implementation iTMTextMacro
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initialize
+ (void) initialize;
/*"Default registration.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [super initialize];
    [[NSUserDefaults standardUserDefaults] registerDefaults:
        [NSDictionary dictionaryWithObjectsAndKeys:
            @"", iTMUDTabAnchorStringKey,
                nil]];
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  macroV1WithBefore:selected:after:
+ (id) macroV1WithBefore: (NSString *) before selected: (NSString *) selected after: (NSString *) after;
/*"Compatibility with iTeXMac < 1.0.7.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * const iTMMacroNameOld = @"iTM:MacroName";
    NSString * const iTMMacroSelectionOld = @"iTM:Selection";
    id result = [[[[self class] alloc] initWithString: nil] autorelease];
    [result setMacroBefore: [[before stringWithSubstring: iTMMacroNameOld replacedByString: iTMMacroNameKey]
                stringWithSubstring: iTMMacroSelectionOld replacedByString: iTMMacroSelectionKey]];
    [result setMacroSelected: [[selected stringWithSubstring: iTMMacroNameOld replacedByString: iTMMacroNameKey]
                stringWithSubstring: iTMMacroSelectionOld replacedByString: iTMMacroSelectionKey]];
    [result setMacroAfter: [[after stringWithSubstring: iTMMacroNameOld replacedByString: iTMMacroNameKey]
                stringWithSubstring: iTMMacroSelectionOld replacedByString: iTMMacroSelectionKey]];
    [result setTooltip: [NSString string]];
    return result;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  string
- (NSString *) string
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    return ([[self macroSelected] length]>0)?
        [NSString stringWithFormat: @"%@%@%@%@%@%@%@",
            [self macroBefore], iTMMacroInsertionKey, [self macroSelected], iTMMacroInsertionKey, [self macroAfter],
                iTMMacroTooltipKey, [self tooltip]]:
        (([[self macroAfter] length]>0)?
        [NSString stringWithFormat: @"%@%@%@%@%@",
            [self macroBefore], iTMMacroInsertionKey, [self macroAfter], iTMMacroTooltipKey, [self tooltip]]:
            [NSString stringWithFormat: @"%@%@%@",
            [self macroBefore], iTMMacroTooltipKey, [self tooltip]]);
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  initWithString:
- (id) initWithString: (NSString *) argument
/*"Designated initializer.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(self = [super init])
    {
        [self setMacroBefore: nil];
        [self setMacroSelected: nil];
        [self setMacroAfter: nil];
        [self setTooltip: nil];
        if([argument isKindOfClass: [NSString class]])
        {
            NSArray * RA = [argument componentsSeparatedByString: iTMMacroTooltipKey];
            if([RA count]>1)
                [self setTooltip: [[RA subarrayWithRange: NSMakeRange(1, [RA count] - 1)]
                            componentsJoinedByString: iTMMacroTooltipKey]];
            if([RA count])
            {
                RA = [[RA objectAtIndex: 0] componentsSeparatedByString: iTMMacroInsertionKey];
                if([RA count])
                    [self setMacroBefore: [RA objectAtIndex: 0]];
                if([RA count]>2)
                {
                    [self setMacroSelected: [RA objectAtIndex: 1]];
                    [self setMacroAfter: [[RA subarrayWithRange: NSMakeRange(2, [RA count] - 2)]
                            componentsJoinedByString: [NSString string]]];
                }
                else if([RA count]==2)
                    [self setMacroAfter: [RA objectAtIndex: 1]];
            }
        }
    }
    return self;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  macroStringForName:selection:indent:selectedRangePointer:
- (NSString *) macroStringForName: (NSString *) name selection: (NSString *) selection indent: (NSString *) indentPrefix
    selectedRangePointer: (NSRangePointer) aRangePtr
/*"Description forthcoming. The selected range is relative to the returned string.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    if(aRangePtr)
    {
        NSString * before = [self macroBefore];
        NSString * selected = [self macroSelected];
        NSString * after = [self macroAfter];
        if(name)
        {
            before = [before stringWithSubstring: iTMMacroNameKey replacedByString: name];
            selected = [selected stringWithSubstring: iTMMacroNameKey replacedByString: name];
            after = [after stringWithSubstring: iTMMacroNameKey replacedByString: name];
        }
        if([selection length])
        {
            before = [before stringWithSubstring: iTMMacroSelectionKey replacedByString: selection];
            selected = [selected stringWithSubstring: iTMMacroSelectionKey replacedByString: selection];
            after = [after stringWithSubstring: iTMMacroSelectionKey replacedByString: selection];
        }
        else
        {
            before = [before stringWithSubstring: iTMMacroSelectionKey replacedByString: iTMMacroTabAnchorKey];
            selected = [selected stringWithSubstring: iTMMacroSelectionKey replacedByString: iTMMacroTabAnchorKey];
            after = [after stringWithSubstring: iTMMacroSelectionKey replacedByString: iTMMacroTabAnchorKey];
            {
                NSRange foundRange = [before rangeOfString: iTMMacroTabAnchorKey
                                                    options: 0 range: NSMakeRange(0, [before length])];
                if(foundRange.length)
                {
                    NSString * newBefore = [before substringWithRange: NSMakeRange(0, foundRange.location)];
                    int i = NSMaxRange(foundRange);
                    after = [[[before substringWithRange: NSMakeRange(i, [before length] - i)]
                                    stringByAppendingString: selected]
                                            stringByAppendingString: after];
                    selected = iTMMacroTabAnchorKey;
                    before = newBefore;
                }
            }
        }
        {
            NSString * S = [[NSUserDefaults standardUserDefaults] stringForKey: iTMUDTabAnchorStringKey];
            before = [before stringWithSubstring: iTMMacroTabAnchorKey replacedByString: S];
            selected = [selected stringWithSubstring: iTMMacroTabAnchorKey replacedByString: S];
            after = [after stringWithSubstring: iTMMacroTabAnchorKey replacedByString: S];
        }
        aRangePtr->location = [before length];
        aRangePtr->length = [selected length];
        return [[beforestringByAppendingString: selected]stringByAppendingString: after];
    }
    else
    {
        NSString * result = [[[self macroBefore]
                                    stringByAppendingString: [self macroSelected]]
                                            stringByAppendingString: [self macroAfter]];
        if([name length])
            result = [result stringWithSubstring: iTMMacroNameKey replacedByString: name];
        if([selection length])
            result = [result stringWithSubstring: iTMMacroSelectionKey replacedByString: selection];
        else
            result = [result stringWithSubstring: iTMMacroSelectionKey replacedByString: iTMMacroTabAnchorKey];
        return result;
    }
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  macroBefore
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  macroSelected
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  macroAfter
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  tooltip
#define GETTER(getter, source) - (id) getter; {return source? source: [NSString string];}
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
GETTER(macroBefore, _Before);
GETTER(macroSelected, _Selected);
GETTER(macroAfter, _After);
GETTER(tooltip, _Tooltip);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMacroBefore:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMacroSelected:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setMacroAfter:
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  setTooltip:
#define SETTER(setter, source)\
- (void) setter: (NSString *) argument;\
{\
    if(![source isEqualToString: argument])\
    {\
        [source autorelease];\
        source = [[argument stringWithSubstring: iTMMacroInsertionKey replacedByString: @""] copy];\
    }\
    return;\
}
SETTER(setMacroBefore, _Before);
SETTER(setMacroSelected, _Selected);
SETTER(setMacroAfter, _After);
SETTER(setTooltip, _Tooltip);
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  dealloc
- (void) dealloc
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr
- < 1.1: 03/10/2002
To Do List:
"*/
{
    [self setMacroBefore: nil];
    [self setMacroSelected: nil];
    [self setMacroAfter: nil];
    [self setTooltip: nil];
    [super dealloc];
    return;
}
@end

@implementation NSTextView (iTMTextMacro)
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  scrollToNextTabAnchor:
- (void) scrollToNextTabAnchor: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (1.0.10)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * S = [self string];
    int anchor = NSMaxRange([self selectedRange]);
    NSString * anchorString = [[NSUserDefaults standardUserDefaults] stringForKey: iTMUDTabAnchorStringKey];
    NSRange foundRange = [S rangeOfString: anchorString options: 0 range: NSMakeRange(anchor, [S length] - anchor)];
    if(!foundRange.length)
        foundRange = [S rangeOfString: anchorString options: 0
                            range: NSMakeRange(0, MIN([S length], anchor + [anchorString length] - 1))];
    if(foundRange.length)
    {
        [self setSelectedRange: foundRange];
        [self scrollRangeToVisible: foundRange];
    }
    else
    {
        [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
            [NSString stringWithFormat: NSLocalizedStringFromTableInBundle(@"No %@ tab anchor found.",
                @"TeX", [NSBundle bundleForClass: [self class]],
                    @"Status displayed when navigating within tab anchors."), anchorString]];
        NSBeep();
    }
    return;
}
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  scrollToPreviousTabAnchor:
- (void) scrollToPreviousTabAnchor: (id) sender;
/*"Description forthcoming.
Version history: jerome.laurens@u-bourgogne.fr (1.0.10)
- < 1.1: 03/10/2002
To Do List:
"*/
{
    NSString * S = [self string];
    int anchor = [self selectedRange].location;
    NSString * anchorString = [[NSUserDefaults standardUserDefaults] stringForKey: iTMUDTabAnchorStringKey];
    NSRange foundRange = [S rangeOfString: anchorString options: NSBackwardsSearch
                                range: NSMakeRange(0, anchor)];
    if(!foundRange.length)
    {
        anchor = (anchor < [iTMMacroTabAnchorKey length])? 0: anchor - [anchorString length] + 1;
        foundRange = [S rangeOfString: anchorString options: NSBackwardsSearch
                            range: NSMakeRange(anchor, [S length] - anchor)];
    }
    if(foundRange.length)
    {
        [self setSelectedRange: foundRange];
        [self scrollRangeToVisible: foundRange];
    }
    else
    {
        [[iTMStatusNotificationCenter defaultCenter] postNotificationWithStatus:
            [NSString stringWithFormat: NSLocalizedStringFromTableInBundle(@"No %@ tab anchor found.",
                @"TeX", [NSBundle bundleForClass: [self class]],
                    @"Status displayed when navigating within tab anchors."), anchorString]];
        NSBeep();
    }
    return;
}
@end

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=  iTMTextMacro

